@@ -1,12 +1,11 @@
 /* acconfig.h
  * #ifdefs to be controlled by "configure"
  *
- * $Id: acconfig.h,v 1.19 2001/06/02 06:21:13 guy Exp $
+ * $Id: acconfig.h,v 1.21 2001/12/12 05:26:53 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.org>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,4 +40,8 @@
 
 #undef HAVE_LIBPCAP
 
+#undef HAVE_PCAP_VERSION
+
 #undef PLUGIN_DIR
+
+#undef RED_HAT_MODIFIED_UCD_SNMP
