@@ -1,12 +1,11 @@
 /* column-utils.h
  * Definitions for column utility structures and routines
  *
- * $Id: column-utils.h,v 1.1 2001/04/01 07:32:35 hagbard Exp $
+ * $Id: column-utils.h,v 1.5 2001/12/10 00:26:16 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +22,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
 #ifndef __COLUMN_UTILS_H__
 #define __COLUMN_UTILS_H__
 
@@ -33,33 +31,30 @@
 #define COL_MAX_INFO_LEN 4096
 
 #include "column_info.h"
-#include "frame_data.h"
+#include "packet_info.h"
 
 /* Allocate all the data structures for constructing column data, given
    the number of columns. */
-void                 col_init(column_info *, gint);
+extern void	col_init(column_info *, gint);
 
 /* Utility routines used by packet*.c */
 
-void	   col_set_writable(frame_data *fd, gboolean writable);
-gint       check_col(frame_data *, gint);
-void       col_clear(frame_data *, gint);
-void       col_set_str(frame_data *, gint, gchar *);
-#if __GNUC__ == 2
-void       col_add_fstr(frame_data *, gint, gchar *, ...)
+extern void	col_set_writable(column_info *, gboolean);
+extern gint	check_col(column_info *, gint);
+extern void	col_clear(column_info *, gint);
+extern void	col_set_str(column_info *, gint, gchar *);
+#if __GNUC__ >= 2
+extern void	col_add_fstr(column_info *, gint, gchar *, ...)
     __attribute__((format (printf, 3, 4)));
-void       col_append_fstr(frame_data *, gint, gchar *, ...)
+extern void	col_append_fstr(column_info *, gint, gchar *, ...)
     __attribute__((format (printf, 3, 4)));
 #else
-void       col_add_fstr(frame_data *, gint, gchar *, ...);
-void       col_append_fstr(frame_data *, gint, gchar *, ...);
+extern void	col_add_fstr(column_info *, gint, gchar *, ...);
+extern void	col_append_fstr(column_info *, gint, gchar *, ...);
 #endif
-void       col_add_str(frame_data *, gint, const gchar *);
-void       col_append_str(frame_data *, gint, gchar *);
-void       col_set_cls_time(frame_data *, int);
-void       fill_in_columns(frame_data *);
+extern void	col_add_str(column_info *, gint, const gchar *);
+extern void	col_append_str(column_info *, gint, gchar *);
+extern void	col_set_cls_time(frame_data *, column_info *, int);
+extern void	fill_in_columns(packet_info *);
 
 #endif /* __COLUMN_UTILS_H__ */
-
-
-
