@@ -1,7 +1,7 @@
 /* conversation.c
  * Routines for building lists of packets that are part of a "conversation"
  *
- * $Id: conversation.c,v 1.10 2001/06/10 09:50:20 guy Exp $
+ * $Id: conversation.c,v 1.17 2001/11/29 09:05:25 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -84,6 +84,17 @@ static guint32 new_index;
 
 static int conversation_init_count = 200;
 
+/*
+ * Protocol-specific data attached to a conversation_t structure - protocol
+ * index and opaque pointer.
+ */
+typedef struct _conv_proto_data {
+	int	proto;
+	void	*proto_data;
+} conv_proto_data;
+
+static GMemChunk *conv_proto_data_area = NULL;
+
 /*
  * Compute the hash value for two given address/port pairs if the match
  * is to be exact.
@@ -378,6 +389,17 @@ conversation_init(void)
 	if (conversation_chunk != NULL)
 		g_mem_chunk_destroy(conversation_chunk);
 
+	/*
+	 * Free up any space allocated for conversation protocol data
+	 * areas.
+	 *
+	 * We can free the space, as the structures it contains are
+	 * pointed to by conversation data structures that were freed
+	 * above.
+	 */
+	if (conv_proto_data_area != NULL)
+		g_mem_chunk_destroy(conv_proto_data_area);
+
 	conversation_hashtable_exact =
 	    g_hash_table_new(conversation_hash_exact,
 	      conversation_match_exact);
@@ -399,6 +421,13 @@ conversation_init(void)
 	    conversation_init_count * sizeof(conversation_t),
 	    G_ALLOC_AND_FREE);
 
+	/*
+	 * Allocate a new area for conversation protocol data items.
+	 */
+	conv_proto_data_area = g_mem_chunk_new("conv_proto_data_area",
+	    sizeof(conv_proto_data), 20 * sizeof(conv_proto_data), /* FIXME*/
+	    G_ALLOC_ONLY);
+
 	/*
 	 * Start the conversation indices over at 0.
 	 */
@@ -415,7 +444,7 @@ conversation_init(void)
  */
 conversation_t *
 conversation_new(address *addr1, address *addr2, port_type ptype,
-    guint32 port1, guint32 port2, void *data, guint options)
+    guint32 port1, guint32 port2, guint options)
 {
 	conversation_t *conversation;
 	conversation_key *new_key;
@@ -431,10 +460,10 @@ conversation_new(address *addr1, address *addr2, port_type ptype,
 
 	conversation = g_mem_chunk_alloc(conversation_chunk);
 	conversation->index = new_index;
-	conversation->data = data;
+	conversation->data_list = NULL;
 
-/* clear dissector pointer */
-	conversation->dissector.new_d = NULL;
+/* clear dissector handle */
+	conversation->dissector_handle = NULL;
 
 /* set the options and key pointer */
 	conversation->options = options;
@@ -524,8 +553,12 @@ conversation_set_addr2(conversation_t *conv, address *addr)
 	}
 }
 
+/*
+ * Search a particular hash table for a conversaton with the specified
+ * addr1, port1, addr2, and port2.
+ */
 static conversation_t *
-conversation_match(GHashTable *hashtable, address *addr1, address *addr2,
+conversation_lookup_hashtable(GHashTable *hashtable, address *addr1, address *addr2,
     port_type ptype, guint32 port1, guint32 port2)
 {
 	conversation_key key;
@@ -542,11 +575,42 @@ conversation_match(GHashTable *hashtable, address *addr1, address *addr2,
 	return g_hash_table_lookup(hashtable, &key);
 }
  
+
 /*
  * Given two address/port pairs for a packet, search for a conversation
  * containing packets between those address/port pairs.  Returns NULL if
- * not found.  If the NO_ADDR_B and/or NO_PORT_B flags are set in the
- * conversation options field, that value will not be used.
+ * not found.
+ *
+ * We try to find the most exact match that we can, and then proceed to
+ * try wildcard matches on the "addr_b" and/or "port_b" argument if a more
+ * exact match failed.
+ *
+ * Either or both of the "addr_b" and "port_b" arguments may be specified as
+ * a wildcard by setting the NO_ADDR_B or NO_PORT_B flags in the "options"
+ * argument.  We do only wildcard matches on addresses and ports specified
+ * as wildcards.
+ *
+ * I.e.:
+ *
+ *	if neither "addr_b" nor "port_b" were specified as wildcards, we
+ *	do an exact match (addr_a/port_a and addr_b/port_b) and, if that
+ *	succeeds, we return a pointer to the matched conversation;
+ *
+ *	otherwise, if "port_b" wasn't specified as a wildcard, we try to
+ *	match any address 2 with the specified port 2 (addr_a/port_a and
+ *	{any}/addr_b) and, if that succeeds, we return a pointer to the
+ *	matched conversation;
+ *
+ *	otherwise, if "addr_b" wasn't specified as a wildcard, we try to
+ *	match any port 2 with the specified address 2 (addr_a/port_a and
+ *	addr_b/{any}) and, if that succeeds, we return a pointer to the
+ *	matched conversation;
+ *
+ *	otherwise, we try to match any address 2 and any port 2
+ *	(addr_a/port_a and {any}/{any}) and, if that succeeds, we return
+ *	a pointer to the matched conversation;
+ *
+ *	otherwise, we found no matching conversation, and return NULL.
  */
 conversation_t *
 find_conversation(address *addr_a, address *addr_b, port_type ptype,
@@ -554,102 +618,299 @@ find_conversation(address *addr_a, address *addr_b, port_type ptype,
 {
 	conversation_t *conversation;
 
-	if (options & NO_ADDR_B) {
-		if (options & NO_PORT_B) {
+	/*
+	 * First try an exact match, if we have two addresses and ports.
+	 */
+	if (!(options & (NO_ADDR_B|NO_PORT_B))) {
+		/*
+		 * Neither search address B nor search port B are wildcarded,
+		 * start out with an exact match.
+		 * Exact matches check both directions.
+		 */
+		conversation =
+		    conversation_lookup_hashtable(conversation_hashtable_exact,
+		      addr_a, addr_b, ptype, port_a, port_b);
+		if (conversation != NULL)
+			return conversation;
+	}
+
+	/*
+	 * Well, that didn't find anything.  Try matches that wildcard
+	 * one of the addresses, if we have two ports.
+	 */
+	if (!(options & NO_PORT_B)) {
+		/*
+		 * Search port B isn't wildcarded.
+		 *
+		 * First try looking for a conversation with the specified
+		 * address A and port A as the first address and port, and
+		 * with any address and the specified port B as the second
+		 * address and port.
+		 * ("addr_b" doesn't take part in this lookup.)
+		 */
+		conversation =
+		    conversation_lookup_hashtable(conversation_hashtable_no_addr2,
+		        addr_a, addr_b, ptype, port_a, port_b);
+		if (conversation != NULL) {
 			/*
-			 * Wildcard the address and port - first try looking
-			 * for a conversation with the specified address 1
-			 * and port 1, then try looking for one with an
-			 * address 1 and port 1 that's the specified
-			 * address *2* and port *2* (this packet may be
-			 * going in the opposite direction from the first
-			 * packet in the conversation).
+			 * If search address B isn't wildcarded, and this
+			 * is for a connection-oriented protocol, set the
+			 * second address for this conversation to address
+			 * B, as that's the address that matched the
+			 * wildcarded second address for this conversation.
+			 *
+			 * (XXX - this assumes that, for all connection-
+			 * oriented protocols, the endpoints of a connection
+			 * have only one address each, i.e. you don't get
+			 * packets in a given direction coming from more than
+			 * one address.) 
 			 */
-			conversation =
-			    conversation_match(conversation_hashtable_no_addr2_or_port2,
-			        addr_a, addr_b, ptype, port_a, port_b);
-			if (conversation != NULL)
-				return conversation;
-			return conversation_match(conversation_hashtable_no_addr2_or_port2,
-			    addr_b, addr_a, ptype, port_b, port_a);
-		} else {
-			/*
-			 * Wildcard the address - first try looking for a
-			 * conversation with the specified address 1 and
-			 * port 1 and the specified port 2, then try looking
-			 * for one with an address 1 and port 1 that's
-			 * the specified address *2* and port *2* and
-			 * a port 2 that's the specified port *1* (this
-			 * packet may be going in the opposite direction
-			 * from the first packet in the conversation).
-			 */
-			conversation =
-			    conversation_match(conversation_hashtable_no_addr2,
-			        addr_a, addr_b, ptype, port_a, port_b);
-			if (conversation != NULL)
-				return conversation;
-			return conversation_match(conversation_hashtable_no_addr2,
-			    addr_b, addr_a, ptype, port_b, port_a);
+			if (!(options & NO_ADDR_B) && ptype != PT_UDP)
+				conversation_set_addr2(conversation, addr_b);
+			return conversation;
 		}
-	} else {
-		if (options & NO_PORT_B) {
-			/*
-			 * Wildcard the port - first try looking for a
-			 * conversation with the specified address 1 and
-			 * port 1 and the specified address 2, then try
-			 * looking for one with an address 1 and port 1
-			 * that's the specified address *2* and port *2*
-			 * and an address 2 that's the specified address
-			 * *1* (this packet may be going in the opposite
-			 * direction from the first packet in the conversation).
-			 */
+
+		/*
+		 * Well, that didn't find anything.
+		 * If search address B was specified, try looking for a
+		 * conversation with the specified address B and port B as
+		 * the first address and port, and with any address and the
+		 * specified port A as the second address and port (this
+		 * packet may be going in the opposite direction from the
+		 * first packet in the conversation).
+		 * ("addr_a" doesn't take part in this lookup.)
+		 */
+		if (!(options & NO_ADDR_B)) {
 			conversation =
-			    conversation_match(conversation_hashtable_no_port2,
-			      addr_a, addr_b, ptype, port_a, port_b);
-			if (conversation != NULL)
-				return conversation;
-			return conversation_match(conversation_hashtable_no_port2,
+			    conversation_lookup_hashtable(conversation_hashtable_no_addr2,
 			    addr_b, addr_a, ptype, port_b, port_a);
-		} else {
-			/*
-			 * Search for an exact match.  That search checks both
-			 * directions.
-			 */
-			return conversation_match(conversation_hashtable_exact,
-			    addr_a, addr_b, ptype, port_a, port_b);
+			if (conversation != NULL) {
+				/*
+				 * If this is for a connection-oriented
+				 * protocol, set the second address for
+				 * this conversation to address A, as
+				 * that's the address that matched the
+				 * wildcarded second address for this
+				 * conversation.
+				 */
+				if (ptype != PT_UDP) {
+					conversation_set_addr2(conversation,
+					    addr_a);
+				}
+				return conversation;
+			}
 		}
 	}
+
+	/*
+	 * Well, that didn't find anything.  Try matches that wildcard
+	 * one of the ports, if we have two addresses.
+	 */
+	if (!(options & NO_ADDR_B)) {
+		/*
+		 * Search address B isn't wildcarded.
+		 *
+		 * First try looking for a conversation with the specified
+		 * address A and port A as the first address and port, and
+		 * with the specified address B and any port as the second
+		 * address and port.
+		 * ("port_b" doesn't take part in this lookup.)
+		 */
+		conversation =
+		    conversation_lookup_hashtable(conversation_hashtable_no_port2,
+		      addr_a, addr_b, ptype, port_a, port_b);
+		if (conversation != NULL) {
+			/*
+			 * If search port B isn't wildcarded, and this is
+			 * for a connection-oriented protocol, set the
+			 * second port for this conversation to port B,
+			 * as that's the port that matched the wildcarded
+			 * second port for this conversation.
+			 *
+			 * (XXX - this assumes that, for all connection-
+			 * oriented protocols, the endpoints of a connection
+			 * have only one port each, i.e. you don't get
+			 * packets in a given direction coming from more than
+			 * one port.)
+			 */
+			if (!(options & NO_PORT_B) && ptype != PT_UDP)
+				conversation_set_port2(conversation, port_b);
+			return conversation;
+		}
+
+		/*
+		 * Well, that didn't find anything.
+		 * If search port B was specified, try looking for a
+		 * conversation with the specified address B and port B
+		 * as the first address and port, and with the specified
+		 * address A and any port as the second address and port
+		 * (this packet may be going in the opposite direction
+		 * from the first packet in the conversation).
+		 * ("port_a" doesn't take part in this lookup.)
+		 */
+		if (!(options & NO_PORT_B)) {
+			conversation =
+			    conversation_lookup_hashtable(conversation_hashtable_no_port2,
+			      addr_b, addr_a, ptype, port_b, port_a);
+			if (conversation != NULL) {
+				/*
+				 * If this is for a connection-oriented
+				 * protocol, set the second port for
+				 * this conversation to port A, as
+				 * that's the address that matched the
+				 * wildcarded second address for this
+				 * conversation.
+				 */
+				if (ptype != PT_UDP) {
+					conversation_set_port2(conversation,
+					    port_a);
+				}
+				return conversation;
+			}
+		}
+	}
+
+	/*
+	 * Well, that didn't find anything.  Try matches that wildcard
+	 * one address/port pair.
+	 *
+	 * First try looking for a conversation with the specified address A
+	 * and port B as the first address and port.
+	 * (Neither "addr_b" nor "port_b" take part in this lookup.)
+	 */
+	conversation =
+	    conversation_lookup_hashtable(conversation_hashtable_no_addr2_or_port2,
+	      addr_a, addr_b, ptype, port_a, port_b);
+	if (conversation != NULL) {
+		/*
+		 * If this is for a connection-oriented protocol:
+		 *
+		 *	if search address B isn't wildcarded, set the
+		 *	second address for this conversation to address
+		 *	B, as that's the address that matched the
+		 *	wildcarded second address for this conversation;
+		 *
+		 *	if search port B isn't wildcarded, set the
+		 *	second port for this conversation to port B,
+		 *	as that's the port that matched the wildcarded
+		 *	second port for this conversation.
+		 */
+		if (ptype != PT_UDP) {
+			if (!(options & NO_ADDR_B))
+				conversation_set_addr2(conversation, addr_b);
+			if (!(options & NO_PORT_B))
+				conversation_set_port2(conversation, port_b);
+		}
+		return conversation;
+	}
+
+	/*
+	 * Well, that didn't find anything.
+	 * If search address and port B were specified, try looking for a
+	 * conversation with the specified address B and port B as the
+	 * first address and port, and with any second address and port
+	 * (this packet may be going in the opposite direction from the
+	 * first packet in the conversation).
+	 * (Neither "addr_a" nor "port_a" take part in this lookup.)
+	 */
+	conversation =
+	    conversation_lookup_hashtable(conversation_hashtable_no_addr2_or_port2,
+	      addr_b, addr_a, ptype, port_b, port_a);
+	if (conversation != NULL) {
+		/*
+		 * If this is for a connection-oriented protocol, set the
+		 * second address for this conversation to address A, as
+		 * that's the address that matched the wildcarded second
+		 * address for this conversation, and set the second port
+		 * for this conversation to port A, as that's the port
+		 * that matched the wildcarded second port for this
+		 * conversation.
+		 */
+		if (ptype != PT_UDP) {
+			conversation_set_addr2(conversation, addr_a);
+			conversation_set_port2(conversation, port_a);
+		}
+		return conversation;
+	}
+
+	/*
+	 * We found no conversation.
+	 */
+	return NULL;
 }
 
-/*
- * Set the dissector for a conversation.
- */
-void
-old_conversation_set_dissector(conversation_t *conversation,
-    old_dissector_t dissector)
+static gint
+p_compare(gconstpointer a, gconstpointer b)
 {
-	conversation->is_old_dissector = TRUE;
-	conversation->dissector.old_d = dissector;
+	if (((conv_proto_data *)a)->proto > ((conv_proto_data *)b)->proto)
+		return 1;
+	else if (((conv_proto_data *)a)->proto == ((conv_proto_data *)b)->proto)
+		return 0;
+	else
+		return -1;
+}
+
+void
+conversation_add_proto_data(conversation_t *conv, int proto, void *proto_data)
+{
+	conv_proto_data *p1 = g_mem_chunk_alloc(conv_proto_data_area);
+
+	p1->proto = proto;
+	p1->proto_data = proto_data;
+
+	/* Add it to the list of items for this conversation. */
+
+	conv->data_list = g_slist_insert_sorted(conv->data_list, (gpointer *)p1,
+	    p_compare);
+}
+
+void *
+conversation_get_proto_data(conversation_t *conv, int proto)
+{
+	conv_proto_data temp, *p1;
+	GSList *item;
+
+	temp.proto = proto;
+	temp.proto_data = NULL;
+
+	item = g_slist_find_custom(conv->data_list, (gpointer *)&temp,
+	    p_compare);
+
+	if (item != NULL) {
+		p1 = (conv_proto_data *)item->data;
+		return p1->proto_data;
+	}
+
+	return NULL;
+}
+
+void
+conversation_delete_proto_data(conversation_t *conv, int proto)
+{
+	conv_proto_data temp;
+	GSList *item;
+
+	temp.proto = proto;
+	temp.proto_data = NULL;
+
+	item = g_slist_find_custom(conv->data_list, (gpointer *)&temp,
+	    p_compare);
+
+	if (item != NULL)
+		conv->data_list = g_slist_remove(conv->data_list, item);
 }
 
 void
 conversation_set_dissector(conversation_t *conversation,
-    dissector_t dissector)
+    dissector_handle_t handle)
 {
-	conversation->is_old_dissector = FALSE;
-	conversation->dissector.new_d = dissector;
+	conversation->dissector_handle = handle;
 }
 
 /*
- * Given two address/port pairs for a packet, search for a conversation
- * dissector.
- * If found, call it and return TRUE, otherwise return FALSE.
- *
- * Will search for a exact match (addr_a/port_a and addr_b/port_b), then search
- * for wild card matches: try to match any address 2 with the specified port 2
- * first (addr_a/port_a and {any}/addr_b), then try to match any port 2 with the
- * specified address 2 (addr_a/port_a and addr_b/{any}), then try to match any
- * address 2 and any port 2 (addr_a/port_a and {any}/{any}).
+ * Given two address/port pairs for a packet, search for a matching
+ * conversation and, if found and it has a conversation dissector,
+ * call that dissector and return TRUE, otherwise return FALSE.
  */
 gboolean
 try_conversation_dissector(address *addr_a, address *addr_b, port_type ptype,
@@ -657,45 +918,29 @@ try_conversation_dissector(address *addr_a, address *addr_b, port_type ptype,
     proto_tree *tree)
 {
 	conversation_t *conversation;
-	const guint8 *pd;
-	int offset;
+	guint16 saved_can_desegment;
 
-	conversation = find_conversation(addr_a, addr_b, ptype, port_a, port_b,
-	    0);
+	/* can_desegment is set to 2 by anyone which offers this api/service.
+	   then everytime a subdissector is called it is decremented by one.
+	   thus only the subdissector immediately ontop of whoever offers this
+	   service can use it.
+	*/
+	saved_can_desegment=pinfo->can_desegment;
+	pinfo->can_desegment = saved_can_desegment-(saved_can_desegment>0);
 
-	if (conversation == NULL)
-		conversation = find_conversation(addr_a, addr_b, ptype, port_a,
-		    port_b, NO_ADDR_B);
-
-	if (conversation == NULL)
-		conversation = find_conversation(addr_a, addr_b, ptype, port_a,
-		    port_b, NO_PORT_B);
-
-	if (conversation == NULL)
-		conversation = find_conversation(addr_a, addr_b, ptype, port_a,
-		    port_b, NO_PORT_B | NO_ADDR_B);
+	conversation = find_conversation(addr_a, addr_b, ptype, port_a,
+	    port_b, 0);
 	
 	if (conversation != NULL) {
-		if (conversation->is_old_dissector) {
-			if (conversation->dissector.old_d == NULL)
-				return FALSE;
-
-			/*
-			 * New dissector calling old dissector; use
-			 * "tvb_compat()" to remap.
-			 *
-			 * "is_old_dissector" won't be set unless
-			 * "dissector.old_d" is set.
-			 */
-			tvb_compat(tvb, &pd, &offset);
-			(*conversation->dissector.old_d)(pd, offset, pinfo->fd,
-			    tree);
-		} else {
-			if (conversation->dissector.new_d == NULL)
-				return FALSE;
-			(*conversation->dissector.new_d)(tvb, pinfo, tree);
+		if (conversation->dissector_handle == NULL){
+			pinfo->can_desegment=saved_can_desegment;
+			return FALSE;
 		}
+		call_dissector(conversation->dissector_handle, tvb, pinfo,
+		    tree);
+		pinfo->can_desegment=saved_can_desegment;
 		return TRUE;
 	}
+	pinfo->can_desegment=saved_can_desegment;
 	return FALSE;
 }
