@@ -1,6 +1,6 @@
 /* epan.h
  *
- * $Id: epan.c,v 1.10 2001/04/02 00:38:34 hagbard Exp $
+ * $Id: epan.c,v 1.15 2001/12/18 19:09:03 gram Exp $
  *
  * Ethereal Protocol Analyzer Library
  *
@@ -11,14 +11,13 @@
 #endif
 
 #include <glib.h>
-#include <epan.h>
+#include "epan.h"
+#include "epan_dissect.h"
 
 #include "conversation.h"
-#include "dfilter/dfilter.h"
 #include "except.h"
 #include "packet.h"
-#include "proto.h"
-#include "tvbuff.h"
+#include "column-utils.h"
 
 /*
  * XXX - this takes the plugin directory as an argument, because
@@ -74,25 +73,36 @@ epan_conversation_init(void)
 
 
 epan_dissect_t*
-epan_dissect_new(void* pseudo_header, const guint8* data, frame_data *fd, proto_tree *tree)
+epan_dissect_new(gboolean create_proto_tree, gboolean proto_tree_visible)
 {
 	epan_dissect_t	*edt;
 
 	edt = g_new(epan_dissect_t, 1);
 
-	/* start with empty data source list */
-	if ( fd->data_src)
-                g_slist_free( fd->data_src);
-        fd->data_src = 0;
-
-	/* XXX - init tree */
-	edt->tree = tree;
-
-	dissect_packet(&edt->tvb, pseudo_header, data, fd, tree);
+	if (create_proto_tree) {
+		edt->tree = proto_tree_create_root();
+        proto_tree_set_visible(edt->tree, proto_tree_visible);
+	}
+	else {
+		edt->tree = NULL;
+	}
 
 	return edt;
 }
 
+void
+epan_dissect_run(epan_dissect_t *edt, void* pseudo_header,
+        const guint8* data, frame_data *fd, column_info *cinfo)
+{
+	/* start with empty data source list */
+	if (fd->data_src) {
+        g_slist_free(fd->data_src);
+    }
+    fd->data_src = NULL;
+
+	dissect_packet(edt, pseudo_header, data, fd, cinfo);
+}
+
 
 void
 epan_dissect_free(epan_dissect_t* edt)
@@ -102,5 +112,30 @@ epan_dissect_free(epan_dissect_t* edt)
 	 * would have incremented the usage count on that tvbuff_t*) */
 	tvb_free_chain(edt->tvb);
 
+	if (edt->tree) {
+		proto_tree_free(edt->tree);
+	}
+
 	g_free(edt);
 }
+
+static void
+prime_dfilter(gpointer data, gpointer user_data)
+{
+    int hfid = GPOINTER_TO_INT(data);
+    proto_tree *tree = user_data;
+
+    proto_tree_prime_hfid(tree, hfid);
+}
+
+void
+epan_dissect_prime_dfilter(epan_dissect_t *edt, dfilter_t* dfcode)
+{
+    dfilter_foreach_interesting_field(dfcode, prime_dfilter, edt->tree);
+}
+
+void
+epan_dissect_fill_in_columns(epan_dissect_t *edt)
+{
+    fill_in_columns(&edt->pi);
+}
