@@ -1,12 +1,11 @@
 /* frame_data.h
  * Definitions for frame_data structures and routines
  *
- * $Id: frame_data.h,v 1.1 2001/04/01 04:11:50 hagbard Exp $
+ * $Id: frame_data.h,v 1.3 2001/12/10 02:16:59 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,27 +27,25 @@
 
 #include "column_info.h"
 
-
 /* XXX - some of this stuff is used only while a packet is being dissected;
    should we keep that stuff in the "packet_info" structure, instead, to
    save memory? */
 typedef struct _frame_data {
   struct _frame_data *next; /* Next element in list */
   struct _frame_data *prev; /* Previous element in list */
-  GSList *pfd;              /* Per frame proto data */
-  GSList *data_src;         /* Frame data sources */
-  guint32      num;       /* Frame number */
-  guint32      pkt_len;   /* Packet length */
-  guint32      cap_len;   /* Amount actually captured */
-  gint32       rel_secs;  /* Relative seconds (yes, it can be negative) */
-  gint32       rel_usecs; /* Relative microseconds (yes, it can be negative) */
-  guint32      abs_secs;  /* Absolute seconds */
-  guint32      abs_usecs; /* Absolute microseconds */
-  gint32       del_secs;  /* Delta seconds (yes, it can be negative) */
-  gint32       del_usecs; /* Delta microseconds (yes, it can be negative) */
-  long         file_off;  /* File offset */
-  column_info *cinfo;     /* Column formatting information */
-  int          lnk_t;     /* Per-packet encapsulation/data-link type */
+  GSList      *pfd;         /* Per frame proto data */
+  GSList      *data_src;    /* Frame data sources */
+  guint32      num;         /* Frame number */
+  guint32      pkt_len;     /* Packet length */
+  guint32      cap_len;     /* Amount actually captured */
+  gint32       rel_secs;    /* Relative seconds (yes, it can be negative) */
+  gint32       rel_usecs;   /* Relative microseconds (yes, it can be negative) */
+  guint32      abs_secs;    /* Absolute seconds */
+  guint32      abs_usecs;   /* Absolute microseconds */
+  gint32       del_secs;    /* Delta seconds (yes, it can be negative) */
+  gint32       del_usecs;   /* Delta microseconds (yes, it can be negative) */
+  long         file_off;    /* File offset */
+  int          lnk_t;       /* Per-packet encapsulation/data-link type */
   struct {
 	unsigned int passed_dfilter	: 1; /* 1 = display, 0 = no display */
   	unsigned int encoding		: 2; /* Character encoding (ASCII, EBCDIC...) */
@@ -69,6 +66,3 @@ void frame_data_init(void);
 void frame_data_cleanup(void);
 
 #endif  /* __FRAME_DATA__ */
-
-
-
