@@ -1,12 +1,11 @@
 /* ftypes.h
  * Definitions for field types
  *
- * $Id: ftypes.h,v 1.2 2001/02/27 19:23:30 gram Exp $
+ * $Id: ftypes.h,v 1.7 2001/11/02 10:09:51 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 2001 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,16 +38,19 @@ enum ftenum {
 	FT_UINT16,
 	FT_UINT24,	/* really a UINT32, but displayed as 3 hex-digits if FD_HEX*/
 	FT_UINT32,
+	FT_UINT64,
 	FT_INT8,
 	FT_INT16,
-	FT_INT24,
+	FT_INT24,	/* same as for UINT24 */
 	FT_INT32,
+	FT_INT64,
 	FT_DOUBLE,
 	FT_ABSOLUTE_TIME,
 	FT_RELATIVE_TIME,
 	FT_STRING,
 	FT_STRINGZ,	/* for use with proto_tree_add_item() */
 	FT_UINT_STRING,	/* for use with proto_tree_add_item() */
+	FT_UCS2_LE,     /* Unicode, 2 byte, Little Endian     */
 	FT_ETHER,
 	FT_BYTES,
 	FT_IPv4,
@@ -112,15 +114,12 @@ ftype_can_le(enum ftenum ftype);
 
 #include "ipv4.h"
 
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
 #ifdef HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
 
 #include "tvbuff.h"
+#include "nstime.h"
 #include "dfilter/drange.h"
 
 typedef struct {
@@ -133,8 +132,7 @@ typedef struct {
 		gchar		*string;
 		GByteArray	*bytes;
 		ipv4_addr	ipv4;
-		guint8		ipv6[16];
-		struct timeval	time;
+		nstime_t	time;
 		tvbuff_t	*tvb;
 	} value;
 } fvalue_t;
