@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c,v 1.36 2001/06/29 09:46:54 guy Exp $
+ * $Id: packet.c,v 1.55 2001/12/18 19:09:03 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -79,14 +79,17 @@
 #include "resolv.h"
 #include "tvbuff.h"
 #include "plugins.h"
+#include "epan_dissect.h"
 
 static gint proto_malformed = -1;
 static dissector_handle_t frame_handle = NULL;
+static dissector_handle_t data_handle = NULL;
 
 void
 packet_init(void)
 {
   frame_handle = find_dissector("frame");
+  data_handle = find_dissector("data");
   proto_malformed = proto_get_id_by_filter_name("malformed");
 }
 
@@ -98,15 +101,12 @@ packet_cleanup(void)
 
 /*
  * Given a tvbuff, a packet_info *, and a length from a packet header,
- * adjust the length of the tvbuff, and the "len" and "captured_len"
- * members of the "packet_info" structure, to reflect the specified
- * length.
+ * adjust the length of the tvbuff to reflect the specified length.
  */
 void
 set_actual_length(tvbuff_t *tvb, packet_info *pinfo, guint specified_len)
 {
   guint payload_len, reported_payload_len;
-  int   padding;
 
   /* Length of payload handed to us. */
   reported_payload_len = tvb_reported_length(tvb);
@@ -120,23 +120,6 @@ set_actual_length(tvbuff_t *tvb, packet_info *pinfo, guint specified_len)
        probably us) may use that to determine how much of its packet
        was padding. */
     tvb_set_reported_length(tvb, specified_len);
-
-    /* XXX - can we get rid of "pinfo->len" and "pinfo->captured_len"
-       when the last dissector is tvbuffified? */
-
-    /* Shrink the total payload by the amount of padding. */
-    padding = reported_payload_len - specified_len;
-    if (pinfo->len >= padding)
-      pinfo->len -= padding;
-
-    /* Shrink the captured payload by the amount of padding in the
-       captured payload (which may be less than the amount of padding,
-       as the padding may not have been captured). */
-    if (specified_len < payload_len) {
-      padding = payload_len - specified_len;
-      if (pinfo->captured_len >= padding)
-        pinfo->captured_len -= padding;
-    }
   }
 }
 
@@ -169,35 +152,52 @@ init_all_protocols(void)
 
 /* Creates the top-most tvbuff and calls dissect_frame() */
 void
-dissect_packet(tvbuff_t **p_tvb, union wtap_pseudo_header *pseudo_header,
-		const u_char *pd, frame_data *fd, proto_tree *tree)
+dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
+	       const u_char *pd, frame_data *fd, column_info *cinfo)
 {
-	blank_packetinfo();
+	int i;
 
-	/* Set the initial payload to the packet length, and the initial
-	   captured payload to the capture length (other protocols may
-	   reduce them if their headers say they're less). */
-	pi.len = fd->pkt_len;
-	pi.captured_len = fd->cap_len;
+	edt->pi.dl_src.type = AT_NONE;
+	edt->pi.dl_dst.type = AT_NONE;
+	edt->pi.net_src.type = AT_NONE;
+	edt->pi.net_dst.type = AT_NONE;
+	edt->pi.src.type = AT_NONE;
+	edt->pi.dst.type = AT_NONE;
+	edt->pi.ethertype  = 0;
+	edt->pi.ipproto  = 0;
+	edt->pi.ipxptype = 0;
+	edt->pi.in_error_pkt = FALSE;
+	edt->pi.ptype = PT_NONE;
+	edt->pi.srcport  = 0;
+	edt->pi.destport = 0;
+	edt->pi.current_proto = "<Missing Protocol Name>";
+	edt->pi.p2p_dir = P2P_DIR_UNKNOWN;
+	edt->pi.private_data = NULL;
 
-	pi.fd = fd;
-	pi.pseudo_header = pseudo_header;
+	edt->pi.fd = fd;
+	edt->pi.pseudo_header = pseudo_header;
 
-	col_set_writable(fd, TRUE);
+	edt->pi.cinfo = cinfo;
+	if (cinfo != NULL) {
+		for (i = 0; i < cinfo->num_cols; i++) {
+			cinfo->col_buf[i][0] = '\0';
+			cinfo->col_data[i] = cinfo->col_buf[i];
+		}
+
+		col_set_writable(cinfo, TRUE);
+	}
 
 	TRY {
-		*p_tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len, "Frame");
-	/* Add this tvbuffer into the data_src list */
-                fd->data_src = g_slist_append( fd->data_src, *p_tvb);
-
-		pi.compat_top_tvb = *p_tvb;
+		edt->tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len, "Frame");
+		/* Add this tvbuffer into the data_src list */
+                fd->data_src = g_slist_append( fd->data_src, edt->tvb);
 	}
 	CATCH(BoundsError) {
 		g_assert_not_reached();
 	}
 	CATCH(ReportedBoundsError) {
 	  if(proto_malformed != -1){
-		proto_tree_add_protocol_format(tree, proto_malformed, *p_tvb, 0, 0,
+		proto_tree_add_protocol_format(edt->tree, proto_malformed, edt->tvb, 0, 0,
 				"[Malformed Frame: Packet Length]" );
 	  }
 	  else {
@@ -207,44 +207,60 @@ dissect_packet(tvbuff_t **p_tvb, union wtap_pseudo_header *pseudo_header,
 	ENDTRY;
 
 	if(frame_handle != NULL)
-	  call_dissector(frame_handle, *p_tvb, &pi, tree);
+	  call_dissector(frame_handle, edt->tvb, &edt->pi, edt->tree);
 
 	fd->flags.visited = 1;
 }
 
 /*********************** code added for sub-dissector lookup *********************/
 
-static GHashTable *dissector_tables = NULL;
-
 /*
- * XXX - for now, we support having both "old" dissectors, with packet
- * data pointer, packet offset, frame_data pointer, and protocol tree
- * pointer arguments, and "new" dissectors, with tvbuff pointer,
- * packet_info pointer, and protocol tree pointer arguments.
- *
- * Nuke this and go back to storing a pointer to the dissector when
- * the last old-style dissector is gone.
+ * An dissector handle.
  */
-typedef struct {
-	gboolean is_old_dissector;
-	union {
-		old_dissector_t	old;
-		dissector_t	new;
-	} dissector;
-	int	proto_index;
-} dissector_entry_t;
-
-struct dtbl_entry {
-	dissector_entry_t initial;
-	dissector_entry_t current;
+struct dissector_handle {
+	const char	*name;		/* dissector name */
+	dissector_t	dissector;
+	int		proto_index;
 };
 
-static void
-dissect_null(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
-{
-}
+/*
+ * An entry in the hash table portion of a dissector table.
+ */
+struct dtbl_entry {
+	dissector_handle_t initial;
+	dissector_handle_t current;
+};
 
-/* Finds a dissector table by field name. */
+/*
+ * A dissector table.
+ *
+ * "hash_table" is a hash table, indexed by port number, supplying
+ * a "struct dtbl_entry"; it records what dissector is assigned to
+ * that port number in that table.
+ *
+ * "dissector_handles" is a list of all dissectors that *could* be
+ * used in that table; not all of them are necessarily in the table,
+ * as they may be for protocols that don't have a fixed port number.
+ *
+ * "ui_name" is the name the dissector table has in the user interface.
+ *
+ * "type" is a field type giving the width of the port number for that
+ * dissector table.
+ *
+ * "base" is the base in which to display the port number for that
+ * dissector table.
+ */
+struct dissector_table {
+	GHashTable	*hash_table;
+	GSList		*dissector_handles;
+	char		*ui_name;
+	ftenum_t	type;
+	int		base;
+};
+
+static GHashTable *dissector_tables = NULL;
+
+/* Finds a dissector table by table name. */
 static dissector_table_t
 find_dissector_table(const char *name)
 {
@@ -252,11 +268,8 @@ find_dissector_table(const char *name)
 	return g_hash_table_lookup( dissector_tables, name );
 }
 
-/* add an entry, lookup the dissector table for the specified field name,  */
-/* if a valid table found, add the subdissector */
 void
-old_dissector_add(const char *name, guint32 pattern, old_dissector_t dissector,
-    int proto)
+dissector_add(const char *name, guint32 pattern, dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
 	dtbl_entry_t *dtbl_entry;
@@ -265,37 +278,18 @@ old_dissector_add(const char *name, guint32 pattern, old_dissector_t dissector,
 	g_assert( sub_dissectors);
 
 	dtbl_entry = g_malloc(sizeof (dtbl_entry_t));
-	dtbl_entry->current.is_old_dissector = TRUE;
-	dtbl_entry->current.dissector.old = dissector;
-	dtbl_entry->current.proto_index = proto;
+	dtbl_entry->current = handle;
 	dtbl_entry->initial = dtbl_entry->current;
-	proto_set_protocol_dissector(proto, dissector);
 
 /* do the table insertion */
-    	g_hash_table_insert( sub_dissectors, GUINT_TO_POINTER( pattern),
-    	 (gpointer)dtbl_entry);
-}
+    	g_hash_table_insert( sub_dissectors->hash_table,
+    	    GUINT_TO_POINTER( pattern), (gpointer)dtbl_entry);
 
-void
-dissector_add(const char *name, guint32 pattern, dissector_t dissector,
-    int proto)
-{
-	dissector_table_t sub_dissectors = find_dissector_table( name);
-	dtbl_entry_t *dtbl_entry;
-
-/* sanity check */
-	g_assert( sub_dissectors);
-
-	dtbl_entry = g_malloc(sizeof (dtbl_entry_t));
-	dtbl_entry->current.is_old_dissector = FALSE;
-	dtbl_entry->current.dissector.new = dissector;
-	dtbl_entry->current.proto_index = proto;
-	dtbl_entry->initial = dtbl_entry->current;
-	proto_set_protocol_dissector(proto, dissector);
-
-/* do the table insertion */
-    	g_hash_table_insert( sub_dissectors, GUINT_TO_POINTER( pattern),
-    	 (gpointer)dtbl_entry);
+	/*
+	 * Now add it to the list of handles that could be used with this
+	 * table, because it *is* being used with this table.
+	 */
+	dissector_add_handle(name, handle);
 }
 
 /* delete the entry for this dissector at this pattern */
@@ -306,7 +300,7 @@ dissector_add(const char *name, guint32 pattern, dissector_t dissector,
 /*	If temporary dissectors are deleted, then the original dissector must */
 /*	be available. */
 void
-dissector_delete(const char *name, guint32 pattern, dissector_t dissector)
+dissector_delete(const char *name, guint32 pattern, dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
 	dtbl_entry_t *dtbl_entry;
@@ -317,14 +311,15 @@ dissector_delete(const char *name, guint32 pattern, dissector_t dissector)
 	/*
 	 * Find the entry.
 	 */
-	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	dtbl_entry = g_hash_table_lookup(sub_dissectors->hash_table,
 	    GUINT_TO_POINTER(pattern));
 
 	if (dtbl_entry != NULL) {
 		/*
 		 * Found - remove it.
 		 */
-		g_hash_table_remove(sub_dissectors, GUINT_TO_POINTER(pattern));
+		g_hash_table_remove(sub_dissectors->hash_table,
+		    GUINT_TO_POINTER(pattern));
 
 		/*
 		 * Now free up the entry.
@@ -334,8 +329,7 @@ dissector_delete(const char *name, guint32 pattern, dissector_t dissector)
 }
 
 void
-dissector_change(const char *name, guint32 pattern, dissector_t dissector,
-		 gboolean old, int proto)
+dissector_change(const char *name, guint32 pattern, dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
 	dtbl_entry_t *dtbl_entry;
@@ -346,36 +340,31 @@ dissector_change(const char *name, guint32 pattern, dissector_t dissector,
 	/*
 	 * See if the entry already exists. If so, reuse it.
 	 */
-	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	dtbl_entry = g_hash_table_lookup(sub_dissectors->hash_table,
 	    GUINT_TO_POINTER(pattern));
 	if (dtbl_entry != NULL) {
-	  dtbl_entry->current.is_old_dissector = old;
-	  dtbl_entry->current.dissector.new = dissector ? dissector : dissect_null;
-	  dtbl_entry->current.proto_index = proto;
+	  dtbl_entry->current = handle;
 	  return;
 	}
 
 	/*
-	 * Don't create an entry if there is no dissector - I.E. the
+	 * Don't create an entry if there is no dissector handle - I.E. the
 	 * user said not to decode something that wasn't being decoded
 	 * in the first place.
 	 */
-	if (dissector == NULL)
+	if (handle == NULL)
 	  return;
 
 	dtbl_entry = g_malloc(sizeof (dtbl_entry_t));
-	dtbl_entry->initial.is_old_dissector = FALSE;
-	dtbl_entry->initial.dissector.old = NULL;
-	dtbl_entry->initial.proto_index = -1;
-	dtbl_entry->current.is_old_dissector = old;
-	dtbl_entry->current.dissector.new = dissector;
-	dtbl_entry->current.proto_index = proto;
+	dtbl_entry->initial = NULL;
+	dtbl_entry->current = handle;
 
 /* do the table insertion */
-    	g_hash_table_insert( sub_dissectors, GUINT_TO_POINTER( pattern),
-    	 (gpointer)dtbl_entry);
+    	g_hash_table_insert( sub_dissectors->hash_table,
+    	    GUINT_TO_POINTER( pattern), (gpointer)dtbl_entry);
 }
 
+/* Reset a dissector in a sub-dissector table to its initial value. */
 void
 dissector_reset(const char *name, guint32 pattern)
 {
@@ -388,7 +377,7 @@ dissector_reset(const char *name, guint32 pattern)
 	/*
 	 * Find the entry.
 	 */
-	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	dtbl_entry = g_hash_table_lookup(sub_dissectors->hash_table,
 	    GUINT_TO_POINTER(pattern));
 
 	if (dtbl_entry == NULL)
@@ -397,38 +386,34 @@ dissector_reset(const char *name, guint32 pattern)
 	/*
 	 * Found - is there an initial value?
 	 */
-	if (dtbl_entry->initial.dissector.new != NULL) {
+	if (dtbl_entry->initial != NULL) {
 		dtbl_entry->current = dtbl_entry->initial;
 	} else {
-		g_hash_table_remove(sub_dissectors, GUINT_TO_POINTER(pattern));
+		g_hash_table_remove(sub_dissectors->hash_table,
+		    GUINT_TO_POINTER(pattern));
 		g_free(dtbl_entry);
 	}
 }
 
 /* Look for a given port in a given dissector table and, if found, call
    the dissector with the arguments supplied, and return TRUE, otherwise
-   return FALSE.
-
-   If the arguments supplied don't match the arguments to the dissector,
-   do the appropriate translation. */
+   return FALSE. */
 gboolean
 dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
     tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	dtbl_entry_t *dtbl_entry;
-	const guint8 *pd;
-	int offset;
 	const char *saved_proto;
 	guint32 saved_match_port;
+	guint16 saved_can_desegment;
 
-	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	dtbl_entry = g_hash_table_lookup(sub_dissectors->hash_table,
 	    GUINT_TO_POINTER(port));
 	if (dtbl_entry != NULL) {
 		/*
-		 * Is this protocol enabled?
+		 * Is there currently a dissector handle for this entry?
 		 */
-		if (dtbl_entry->current.proto_index != -1 &&
-		    !proto_is_protocol_enabled(dtbl_entry->current.proto_index)) {
+		if (dtbl_entry->current == NULL) {
 			/*
 			 * No - pretend this dissector didn't exist,
 			 * so that other dissectors might have a chance
@@ -436,54 +421,100 @@ dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
 			 */
 			return FALSE;
 		}
-			
+
+		/*
+		 * Yes - is its protocol enabled?
+		 */
+		if (dtbl_entry->current->proto_index != -1 &&
+		    !proto_is_protocol_enabled(dtbl_entry->current->proto_index)) {
+			/*
+			 * No - pretend this dissector didn't exist,
+			 * so that other dissectors might have a chance
+			 * to dissect this packet.
+			 */
+			return FALSE;
+		}
+
 		/*
 		 * Yes, it's enabled.
 		 */
 		saved_proto = pinfo->current_proto;
 		saved_match_port = pinfo->match_port;
+		saved_can_desegment = pinfo->can_desegment;
+
+		/*
+		 * can_desegment is set to 2 by anyone which offers the
+		 * desegmentation api/service.
+		 * Then everytime a subdissector is called it is decremented
+		 * by one.
+		 * Thus only the subdissector immediately on top of whoever
+		 * offers this serveice can use it.
+		 */
+		pinfo->can_desegment = saved_can_desegment-(saved_can_desegment>0);
 		pinfo->match_port = port;
-		if (dtbl_entry->current.is_old_dissector) {
-			/*
-			 * New dissector calling old dissector; use
-			 * "tvb_compat()" to remap.
-			 */
-			tvb_compat(tvb, &pd, &offset);
-			(*dtbl_entry->current.dissector.old)(pd, offset, pinfo->fd,
-			    tree);
-		} else {
-			if (dtbl_entry->current.proto_index != -1) {
-				pinfo->current_proto =
-				    proto_get_protocol_short_name(dtbl_entry->current.proto_index);
-			}
-			(*dtbl_entry->current.dissector.new)(tvb, pinfo, tree);
+		if (dtbl_entry->current->proto_index != -1) {
+			pinfo->current_proto =
+			    proto_get_protocol_short_name(dtbl_entry->current->proto_index);
 		}
+		(*dtbl_entry->current->dissector)(tvb, pinfo, tree);
 		pinfo->current_proto = saved_proto;
 		pinfo->match_port = saved_match_port;
+		pinfo->can_desegment = saved_can_desegment;
 		return TRUE;
-	} else
-		return FALSE;
+	} 
+	return FALSE;
 }
 
-gboolean
-dissector_get_old_flag (dtbl_entry_t *dtbl_entry)
+/* Look for a given port in a given dissector table and, if found, return
+   the dissector handle for that port. */
+dissector_handle_t
+dissector_get_port_handle(dissector_table_t sub_dissectors, guint32 port)
 {
-	g_assert(dtbl_entry);
-	return(dtbl_entry->current.is_old_dissector);
+	dtbl_entry_t *dtbl_entry;
+
+	dtbl_entry = g_hash_table_lookup(sub_dissectors->hash_table,
+	    GUINT_TO_POINTER(port));
+	if (dtbl_entry != NULL)
+		return dtbl_entry->current;
+	else
+		return NULL;
 }
 
-gint
-dissector_get_proto (dtbl_entry_t *dtbl_entry)
+dissector_handle_t
+dtbl_entry_get_handle (dtbl_entry_t *dtbl_entry)
 {
-	g_assert(dtbl_entry);
-	return(dtbl_entry->current.proto_index);
+	return dtbl_entry->current;
 }
 
-gint
-dissector_get_initial_proto (dtbl_entry_t *dtbl_entry)
+/* Add a handle to the list of handles that *could* be used with this
+   table.  That list is used by code in the UI. */
+void
+dissector_add_handle(const char *name, dissector_handle_t handle)
 {
-	g_assert(dtbl_entry);
-	return(dtbl_entry->initial.proto_index);
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+	GSList *entry;
+
+	/* sanity check */
+	g_assert(sub_dissectors != NULL);
+
+	/* Is it already in this list? */
+	entry = g_slist_find(sub_dissectors->dissector_handles, (gpointer)handle);
+	if (entry != NULL) {
+		/*
+		 * Yes - don't insert it again.
+		 */
+		return;
+	}
+
+	/* Add it to the list. */
+	sub_dissectors->dissector_handles =
+	    g_slist_append(sub_dissectors->dissector_handles, (gpointer)handle);
+}
+
+dissector_handle_t
+dtbl_entry_get_initial_handle (dtbl_entry_t *dtbl_entry)
+{
+	return dtbl_entry->initial;
 }
 
 /**************************************************/
@@ -500,12 +531,10 @@ typedef struct dissector_foreach_info {
 } dissector_foreach_info_t;
 
 /*
- * Walk all dissector tables calling a user supplied function on each
- * entry.  These three routines handle traversing the hash of hashes
- * that is the dissector tables.
+ * Called for each entry in a dissector table.
  */
 static void
-dissector_all_tables_foreach_func2 (gpointer key, gpointer value, gpointer user_data)
+dissector_table_foreach_func (gpointer key, gpointer value, gpointer user_data)
 {
 	dissector_foreach_info_t *info;
 	dtbl_entry_t *dtbl_entry;
@@ -514,29 +543,44 @@ dissector_all_tables_foreach_func2 (gpointer key, gpointer value, gpointer user_
 	g_assert(user_data);
 
 	dtbl_entry = value;
-	if (dtbl_entry->current.proto_index == -1) {
-	  return;
+	if (dtbl_entry->current == NULL ||
+	    dtbl_entry->current->proto_index == -1) {
+		/*
+		 * Either there is no dissector for this entry, or
+		 * the dissector doesn't have a protocol associated
+		 * with it.
+		 *
+		 * XXX - should the latter check be done?
+		 */
+		return;
 	}
 
 	info = user_data;
 	info->caller_func(info->table_name, key, value, info->caller_data);
 }
 
+/*
+ * Called for each entry in the table of all dissector tables.
+ */
 static void
-dissector_all_tables_foreach_func1 (gpointer key, gpointer value, gpointer user_data)
+dissector_all_tables_foreach_func (gpointer key, gpointer value, gpointer user_data)
 {
-	GHashTable   *hash_table;
+	dissector_table_t sub_dissectors;
 	dissector_foreach_info_t *info;
 
 	g_assert(value);
 	g_assert(user_data);
 
-	hash_table = value;
+	sub_dissectors = value;
 	info = user_data;
 	info->table_name = (gchar*) key;
-	g_hash_table_foreach(hash_table, info->next_func, info);
+	g_hash_table_foreach(sub_dissectors->hash_table, info->next_func, info);
 }
 
+/*
+ * Walk all dissector tables calling a user supplied function on each
+ * entry.
+ */
 void
 dissector_all_tables_foreach (DATFunc func,
 			      gpointer user_data)
@@ -545,13 +589,13 @@ dissector_all_tables_foreach (DATFunc func,
 
 	info.caller_data = user_data;
 	info.caller_func = func;
-	info.next_func = dissector_all_tables_foreach_func2;
-	g_hash_table_foreach(dissector_tables, dissector_all_tables_foreach_func1, &info);
+	info.next_func = dissector_table_foreach_func;
+	g_hash_table_foreach(dissector_tables, dissector_all_tables_foreach_func, &info);
 }
 
 /*
- * Walk one dissector table calling a user supplied function on each
- * entry.
+ * Walk one dissector table's hash table calling a user supplied function
+ * on each entry.
  */
 void
 dissector_table_foreach (char *name,
@@ -559,25 +603,35 @@ dissector_table_foreach (char *name,
 			 gpointer user_data)
 {
 	dissector_foreach_info_t info;
-	GHashTable *hash_table;
-
-	hash_table = find_dissector_table(name);
-	g_assert(hash_table);
+	dissector_table_t sub_dissectors = find_dissector_table( name);
 
 	info.table_name = name;
 	info.caller_func = func;
 	info.caller_data = user_data;
-	g_hash_table_foreach(hash_table, dissector_all_tables_foreach_func2, &info);
+	g_hash_table_foreach(sub_dissectors->hash_table, dissector_table_foreach_func, &info);
 }
 
 /*
- * Walk all dissector tables calling a user supplied function only on
- * any entry that has been changed from its original state.  These two
- * routines (plus one above) handle traversing the hash of hashes that
- * is the dissector tables.
+ * Walk one dissector table's list of handles calling a user supplied
+ * function on each entry.
+ */
+void
+dissector_table_foreach_handle(char *name, DATFunc_handle func, gpointer user_data)
+{
+	dissector_foreach_info_t info;
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+	GSList *tmp;
+
+	for (tmp = sub_dissectors->dissector_handles; tmp != NULL;
+	    tmp = g_slist_next(tmp))
+		func(name, tmp->data, user_data);
+}
+
+/*
+ * Called for each entry in a dissector table.
  */
 static void
-dissector_all_tables_foreach_changed_func2 (gpointer key, gpointer value, gpointer user_data)
+dissector_table_foreach_changed_func (gpointer key, gpointer value, gpointer user_data)
 {
 	dtbl_entry_t *dtbl_entry;
 	dissector_foreach_info_t *info;
@@ -586,14 +640,21 @@ dissector_all_tables_foreach_changed_func2 (gpointer key, gpointer value, gpoint
 	g_assert(user_data);
 
 	dtbl_entry = value;
-	if (dtbl_entry->initial.proto_index == dtbl_entry->current.proto_index) {
-	    return;
+	if (dtbl_entry->initial == dtbl_entry->current) {
+		/*
+		 * Entry hasn't changed - don't call the function.
+		 */
+		return;
 	}
 
 	info = user_data;
 	info->caller_func(info->table_name, key, value, info->caller_data);
 }
 
+/*
+ * Walk all dissector tables calling a user supplied function only on
+ * any entry that has been changed from its original state.
+ */
 void
 dissector_all_tables_foreach_changed (DATFunc func,
 				      gpointer user_data)
@@ -602,8 +663,8 @@ dissector_all_tables_foreach_changed (DATFunc func,
 
 	info.caller_data = user_data;
 	info.caller_func = func;
-	info.next_func = dissector_all_tables_foreach_changed_func2;
-	g_hash_table_foreach(dissector_tables, dissector_all_tables_foreach_func1, &info);
+	info.next_func = dissector_table_foreach_changed_func;
+	g_hash_table_foreach(dissector_tables, dissector_all_tables_foreach_func, &info);
 }
 
 /*
@@ -616,19 +677,18 @@ dissector_table_foreach_changed (char *name,
 				 gpointer user_data)
 {
 	dissector_foreach_info_t info;
-	GHashTable *hash_table;
-
-	hash_table = find_dissector_table(name);
-	g_assert(hash_table);
+	dissector_table_t sub_dissectors = find_dissector_table( name);
 
 	info.table_name = name;
 	info.caller_func = func;
 	info.caller_data = user_data;
-	g_hash_table_foreach(hash_table, dissector_all_tables_foreach_changed_func2, &info);
+	g_hash_table_foreach(sub_dissectors->hash_table,
+	    dissector_table_foreach_changed_func, &info);
 }
 
 dissector_table_t
-register_dissector_table(const char *name)
+register_dissector_table(const char *name, char *ui_name, ftenum_t type,
+    int base)
 {
 	dissector_table_t	sub_dissectors;
 
@@ -643,11 +703,41 @@ register_dissector_table(const char *name)
 
 	/* Create and register the dissector table for this name; returns */
 	/* a pointer to the dissector table. */
-	sub_dissectors = g_hash_table_new( g_direct_hash, g_direct_equal );
+	sub_dissectors = g_malloc(sizeof (struct dissector_table));
+	sub_dissectors->hash_table = g_hash_table_new( g_direct_hash,
+	    g_direct_equal );
+	sub_dissectors->dissector_handles = NULL;
+	sub_dissectors->ui_name = ui_name;
+	sub_dissectors->type = type;
+	sub_dissectors->base = base;
 	g_hash_table_insert( dissector_tables, (gpointer)name, (gpointer) sub_dissectors );
 	return sub_dissectors;
 }
 
+char *
+get_dissector_table_ui_name(const char *name)
+{
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+
+	return sub_dissectors->ui_name;
+}
+
+ftenum_t
+get_dissector_table_type(const char *name)
+{
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+
+	return sub_dissectors->type;
+}
+
+int
+get_dissector_table_base(const char *name)
+{
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+
+	return sub_dissectors->base;
+}
+
 static GHashTable *heur_dissector_lists = NULL;
 
 typedef struct {
@@ -688,10 +778,20 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 	const char *saved_proto;
 	GSList *entry;
 	heur_dtbl_entry_t *dtbl_entry;
+	guint16 saved_can_desegment;
+
+	/* can_desegment is set to 2 by anyone which offers this api/service.
+	   then everytime a subdissector is called it is decremented by one.
+	   thus only the subdissector immediately ontop of whoever offers this
+	   service can use it.
+	*/
+	saved_can_desegment=pinfo->can_desegment;
+	pinfo->can_desegment = saved_can_desegment-(saved_can_desegment>0);
 
 	status = FALSE;
 	saved_proto = pinfo->current_proto;
 	for (entry = sub_dissectors; entry != NULL; entry = g_slist_next(entry)) {
+		pinfo->can_desegment = saved_can_desegment-(saved_can_desegment>0);
 		dtbl_entry = (heur_dtbl_entry_t *)entry->data;
 		if (dtbl_entry->proto_index != -1 &&
 		    !proto_is_protocol_enabled(dtbl_entry->proto_index)) {
@@ -711,6 +811,7 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 		}
 	}
 	pinfo->current_proto = saved_proto;
+	pinfo->can_desegment=saved_can_desegment;
 	return status;
 }
 
@@ -731,107 +832,6 @@ register_heur_dissector_list(const char *name, heur_dissector_list_t *sub_dissec
 	    (gpointer) sub_dissectors);
 }
 
-static GHashTable *conv_dissector_lists = NULL;
-
-struct conv_dtbl_entry {
-	dissector_t dissector;
-	int	proto_index;
-};
-
-/* Finds a conversation dissector table by table name. */
-static conv_dissector_list_t *
-find_conv_dissector_list(const char *name)
-{
-	g_assert(conv_dissector_lists != NULL);
-	return g_hash_table_lookup(conv_dissector_lists, name);
-}
-
-void
-conv_dissector_add(const char *name, dissector_t dissector, int proto)
-{
-	conv_dissector_list_t *sub_dissectors = find_conv_dissector_list(name);
-	conv_dtbl_entry_t *dtbl_entry;
-
-	/* sanity check */
-	g_assert(sub_dissectors != NULL);
-
-	dtbl_entry = g_malloc(sizeof (conv_dtbl_entry_t));
-	dtbl_entry->dissector = dissector;
-	dtbl_entry->proto_index = proto;
-	proto_set_protocol_dissector(proto, dissector);
-
-	/* do the table insertion */
-	*sub_dissectors = g_slist_append(*sub_dissectors, (gpointer)dtbl_entry);
-}
-
-void
-register_conv_dissector_list(const char *name, conv_dissector_list_t *sub_dissectors)
-{
-	/* Create our hash-of-lists if it doesn't already exist */
-	if (conv_dissector_lists == NULL) {
-		conv_dissector_lists = g_hash_table_new(g_str_hash, g_str_equal);
-		g_assert(conv_dissector_lists != NULL);
-	}
-
-	/* Make sure the registration is unique */
-	g_assert(g_hash_table_lookup(conv_dissector_lists, name) == NULL);
-
-	*sub_dissectors = NULL;	/* initially empty */
-	g_hash_table_insert(conv_dissector_lists, (gpointer)name,
-	    (gpointer) sub_dissectors);
-}
-
-gint
-conv_dissector_get_proto (conv_dtbl_entry_t *dtbl_entry)
-{
-	g_assert(dtbl_entry);
-	return(dtbl_entry->proto_index);
-}
-
-void
-dissector_conv_foreach (char *name,
-			DATFunc func,
-			gpointer user_data)
-{
-	conv_dissector_list_t *sub_dissectors = find_conv_dissector_list(name);
-	GSList *tmp;
-
-	/* sanity check */
-	g_assert(sub_dissectors != NULL);
-
-	for (tmp = *sub_dissectors; tmp; tmp = g_slist_next(tmp)) {
-		func(name, 0, tmp->data, user_data);
-	}
-}
-
-static void
-dissector_all_conv_foreach_func1 (gpointer key, gpointer value, gpointer user_data)
-{
-	conv_dissector_list_t *sub_dissectors;
-	GSList *tmp;
-	dissector_foreach_info_t *info;
-
-	g_assert(value);
-	g_assert(user_data);
-
-	sub_dissectors = value;
-	for (tmp = *sub_dissectors; tmp; tmp = g_slist_next(tmp)) {
-	  info = user_data;
-	  info->caller_func(key, 0, tmp->data, info->caller_data);
-	}
-}
-
-void
-dissector_all_conv_foreach (DATFunc func,
-			    gpointer user_data)
-{
-	dissector_foreach_info_t info;
-
-	info.caller_data = user_data;
-	info.caller_func = func;
-	g_hash_table_foreach(conv_dissector_lists, dissector_all_conv_foreach_func1, &info);
-}
-
 /*
  * Register dissectors by name; used if one dissector always calls a
  * particular dissector, or if it bases the decision of which dissector
@@ -844,14 +844,12 @@ dissector_all_conv_foreach (DATFunc func,
  */
 static GHashTable *registered_dissectors = NULL;
 
-/*
- * An entry in the list of registered dissectors.
- */
-struct dissector_handle {
-	const char	*name;		/* dissector name */
-	dissector_t	dissector;
-	int		proto_index;
-};
+/* Get the short name of the protocol for a dissector handle. */
+char *
+dissector_handle_get_short_name(dissector_handle_t handle)
+{
+	return proto_get_protocol_short_name(handle->proto_index);
+}
 
 /* Find a registered dissector by name. */
 dissector_handle_t
@@ -861,6 +859,20 @@ find_dissector(const char *name)
 	return g_hash_table_lookup(registered_dissectors, name);
 }
 
+/* Create an anonymous handle for a dissector. */
+dissector_handle_t
+create_dissector_handle(dissector_t dissector, int proto)
+{
+	struct dissector_handle *handle;
+
+	handle = g_malloc(sizeof (struct dissector_handle));
+	handle->name = NULL;
+	handle->dissector = dissector;
+	handle->proto_index = proto;
+
+	return handle;
+}
+
 /* Register a dissector by name. */
 void
 register_dissector(const char *name, dissector_t dissector, int proto)
@@ -897,7 +909,9 @@ call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
 		/*
 		 * No - just dissect this packet as data.
 		 */
-		dissect_data(tvb, 0, pinfo, tree);
+	        g_assert(data_handle != NULL);
+		g_assert(data_handle->proto_index != -1);
+		call_dissector(data_handle,tvb, pinfo, tree);
 		return;
 	}
 
