@@ -1,7 +1,7 @@
 /* packet.h
  * Definitions for packet disassembly structures and routines
  *
- * $Id: packet.h,v 1.35 2001/06/29 09:46:54 guy Exp $
+ * $Id: packet.h,v 1.48 2001/12/18 19:09:03 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -22,7 +22,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
 #ifndef __PACKET_H__
 #define __PACKET_H__
 
@@ -36,6 +35,7 @@
 #include "frame_data.h"
 #include "packet_info.h"
 #include "column-utils.h"
+#include "epan.h"
 
 #define hi_nibble(b) (((b) & 0xf0) >> 4)
 #define lo_nibble(b) ((b) & 0x0f)
@@ -43,20 +43,11 @@
 /* Useful when you have an array whose size you can tell at compile-time */
 #define array_length(x)	(sizeof x / sizeof x[0])
 
-/* Useful when highlighting regions inside a dissect_*() function. With this
- * macro, you can highlight from an arbitrary offset to the end of the
- * packet (which may come before the end of the frame).
- * See old_dissect_data() for an example.
- */
-#define END_OF_FRAME	(pi.captured_len - offset)
-
 /* Check whether the "len" bytes of data starting at "offset" is
  * entirely inside the captured data for this packet. */
-#define	BYTES_ARE_IN_FRAME(offset, len)	((offset) + (len) <= pi.captured_len)
+#define	BYTES_ARE_IN_FRAME(offset, captured_len, len) \
+	((offset) + (len) <= (captured_len))
 
-/* Check whether there's any data at all starting at "offset". */
-#define	IS_DATA_IN_FRAME(offset)	((offset) < pi.captured_len)
-		
 /* To pass one of two strings, singular or plural */
 #define plurality(d,s,p) ((d) == 1 ? (s) : (p))
 
@@ -86,52 +77,85 @@ typedef struct true_false_string {
 	char	*false_string;
 } true_false_string;
 
-void packet_init(void);
-void packet_cleanup(void);
+extern void packet_init(void);
+extern void packet_cleanup(void);
 
-/* Hash table for matching port numbers and dissectors */
-typedef GHashTable* dissector_table_t;
+/* Handle for dissectors you call directly or register with "dissector_add()".
+   This handle is opaque outside of "packet.c". */
+struct dissector_handle;
+typedef struct dissector_handle *dissector_handle_t;
+
+/* Hash table for matching port numbers and dissectors; this is opaque
+   outside of "packet.c". */
+struct dissector_table;
+typedef struct dissector_table *dissector_table_t;
 
 /* types for sub-dissector lookup */
 typedef void (*old_dissector_t)(const u_char *, int, frame_data *, proto_tree *);
 typedef void (*dissector_t)(tvbuff_t *, packet_info *, proto_tree *);
 
 typedef void (*DATFunc) (gchar *table_name, gpointer key, gpointer value, gpointer user_data);
+typedef void (*DATFunc_handle) (gchar *table_name, gpointer value, gpointer user_data);
 
 /* Opaque structure - provides type checking but no access to components */
 typedef struct dtbl_entry dtbl_entry_t;
 
-gboolean dissector_get_old_flag (dtbl_entry_t *entry);
-gint dissector_get_proto (dtbl_entry_t * entry);
-gint dissector_get_initial_proto (dtbl_entry_t * entry);
-void dissector_table_foreach_changed (char *name, DATFunc func, gpointer user_data);
-void dissector_table_foreach (char *name, DATFunc func, gpointer user_data);
-void dissector_all_tables_foreach_changed (DATFunc func, gpointer user_data);
+extern dissector_handle_t dtbl_entry_get_handle (dtbl_entry_t *dtbl_entry);
+extern dissector_handle_t dtbl_entry_get_initial_handle (dtbl_entry_t * entry);
+extern void dissector_table_foreach_changed (char *name, DATFunc func,
+    gpointer user_data);
+extern void dissector_table_foreach (char *name, DATFunc func,
+    gpointer user_data);
+extern void dissector_all_tables_foreach_changed (DATFunc func,
+    gpointer user_data);
+extern void dissector_table_foreach_handle(char *name, DATFunc_handle func,
+    gpointer user_data);
 
 /* a protocol uses the function to register a sub-dissector table */
-dissector_table_t register_dissector_table(const char *name);
+extern dissector_table_t register_dissector_table(const char *name,
+    char *ui_name, ftenum_t type, int base);
+
+/* Get the UI name for a sub-dissector table, given its internal name */
+extern char *get_dissector_table_ui_name(const char *name);
+
+/* Get the field type to use when displaying values of the selector for a
+   sub-dissector table, given the table's internal name */
+ftenum_t get_dissector_table_type(const char *name);
+
+/* Get the base to use when displaying values of the selector for a
+   sub-dissector table, given the table's internal name */
+extern int get_dissector_table_base(const char *name);
 
 /* Add a sub-dissector to a dissector table.  Called by the protocol routine */
 /* that wants to register a sub-dissector.  */
-void old_dissector_add(const char *abbrev, guint32 pattern,
-    old_dissector_t dissector, int proto);
-void dissector_add(const char *abbrev, guint32 pattern,
-    dissector_t dissector, int proto);
+extern void dissector_add(const char *abbrev, guint32 pattern,
+    dissector_handle_t handle);
 
 /* Add a sub-dissector to a dissector table.  Called by the protocol routine */
 /* that wants to de-register a sub-dissector.  */
-void dissector_delete(const char *name, guint32 pattern, dissector_t dissector);
+extern void dissector_delete(const char *name, guint32 pattern,
+    dissector_handle_t handle);
+
+extern void dissector_change(const char *abbrev, guint32 pattern,
+    dissector_handle_t handle);
 
 /* Reset a dissector in a sub-dissector table to its initial value. */
-void dissector_change(const char *abbrev, guint32 pattern,
-    dissector_t dissector, gboolean old, int proto);
-void dissector_reset(const char *name, guint32 pattern);
+extern void dissector_reset(const char *name, guint32 pattern);
 
 /* Look for a given port in a given dissector table and, if found, call
    the dissector with the arguments supplied, and return TRUE, otherwise
    return FALSE. */
-gboolean dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
-    tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+extern gboolean dissector_try_port(dissector_table_t sub_dissectors,
+    guint32 port, tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+
+/* Look for a given port in a given dissector table and, if found, return
+   the dissector handle for that port. */
+extern dissector_handle_t dissector_get_port_handle(
+    dissector_table_t sub_dissectors, guint32 port);
+
+/* Add a handle to the list of handles that *could* be used with this
+   table.  That list is used by code in the UI. */
+extern void dissector_add_handle(const char *name, dissector_handle_t handle);
 
 /* List of "heuristic" dissectors (which get handed a packet, look at it,
    and either recognize it as being for their protocol, dissect it, and
@@ -144,67 +168,42 @@ typedef gboolean (*heur_dissector_t)(tvbuff_t *, packet_info *,
 	proto_tree *);
 
 /* A protocol uses this function to register a heuristic dissector list */
-void register_heur_dissector_list(const char *name, heur_dissector_list_t *list);
+extern void register_heur_dissector_list(const char *name,
+    heur_dissector_list_t *list);
 
 /* Add a sub-dissector to a heuristic dissector list.  Called by the
    protocol routine that wants to register a sub-dissector.  */
-void heur_dissector_add(const char *name, heur_dissector_t dissector,
+extern void heur_dissector_add(const char *name, heur_dissector_t dissector,
     int proto);
 
 /* Try all the dissectors in a given heuristic dissector list until
    we find one that recognizes the protocol, in which case we return
    TRUE, or we run out of dissectors, in which case we return FALSE. */
-gboolean dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
+extern gboolean dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
     tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
-/* List of "conversation" dissectors (they're not heuristic, but are
-   assigned to a conversation if some other dissector sees some traffic
-   saying "traffic between these hosts on these ports will be of type
-   XXX", e.g. RTSP traffic doing so).
-
-   These lists are for use by the UI, which, for a given conversation,
-   would offer a list of dissectors that could be used with it; this
-   would include dissectors on the conversation dissector list for
-   the transport-layer protocol for the conversation, as well as
-   dissectors for any port-based lists for that protocol (as a conversation
-   between two ports, both of which have dissectors associated with them,
-   might have been given to the wrong one of those dissectors). */
-typedef GSList *conv_dissector_list_t;
-
-/* A protocol uses this function to register a conversation dissector list */
-void register_conv_dissector_list(const char *name, conv_dissector_list_t *list);
-
-/* Add a sub-dissector to a conversation dissector list.  Called by the
-   protocol routine that wants to register a sub-dissector.  */
-void conv_dissector_add(const char *name, dissector_t dissector,
+/* Register a dissector. */
+extern void register_dissector(const char *name, dissector_t dissector,
     int proto);
 
-/* Opaque structure - provides type checking but no access to components */
-typedef struct conv_dtbl_entry conv_dtbl_entry_t;
-
-gint conv_dissector_get_proto (conv_dtbl_entry_t * entry);
-void dissector_conv_foreach(char *name, DATFunc func, gpointer user_data);
-void dissector_all_conv_foreach(DATFunc func, gpointer user_data);
-
-/* Handle for dissectors you call directly.
-   This handle is opaque outside of "packet.c". */
-struct dissector_handle;
-typedef struct dissector_handle *dissector_handle_t;
-
-/* Register a dissector. */
-void register_dissector(const char *name, dissector_t dissector, int proto);
+/* Get the short name of the protocol for a dissector handle. */
+extern char *dissector_handle_get_short_name(dissector_handle_t handle);
 
 /* Find a dissector by name. */
-dissector_handle_t find_dissector(const char *name);
+extern dissector_handle_t find_dissector(const char *name);
+
+/* Create an anonymous handle for a dissector. */
+extern dissector_handle_t create_dissector_handle(dissector_t dissector,
+    int proto);
 
 /* Call a dissector through a handle. */
-void call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
+extern void call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
     packet_info *pinfo, proto_tree *tree);
 
 /* Do all one-time initialization. */
-void dissect_init(void);
+extern void dissect_init(void);
 
-void dissect_cleanup(void);
+extern void dissect_cleanup(void);
 
 /*
  * Given a tvbuff, a packet_info *, and a length from a packet header,
@@ -212,30 +211,29 @@ void dissect_cleanup(void);
  * members of the "packet_info" structure, to reflect the specified
  * length.
  */
-void set_actual_length(tvbuff_t *tvb, packet_info *pinfo, guint specified_len);
+extern void set_actual_length(tvbuff_t *tvb, packet_info *pinfo,
+    guint specified_len);
 
 /* Allow protocols to register "init" routines, which are called before
    we make a pass through a capture file and dissect all its packets
    (e.g., when we read in a new capture file, or run a "filter packets"
    or "colorize packets" pass over the current capture file). */
-void register_init_routine(void (*func)(void));
+extern void register_init_routine(void (*func)(void));
 
 /* Call all the registered "init" routines. */
-void init_all_protocols(void);
+extern void init_all_protocols(void);
 
 /*
  * Dissectors should never modify the packet data.
  */
-void dissect_packet(tvbuff_t **p_tvb, union wtap_pseudo_header *pseudo_header,
-		const u_char *pd, frame_data *fd, proto_tree *tree);
-void old_dissect_data(const u_char *, int, frame_data *, proto_tree *);
-void dissect_data(tvbuff_t *tvb, int, packet_info *pinfo, proto_tree *tree);
-
+extern void dissect_packet(epan_dissect_t *edt,
+    union wtap_pseudo_header *pseudo_header, const u_char *pd,
+    frame_data *fd, column_info *cinfo);
 
 /* These functions are in packet-ethertype.c */
-void capture_ethertype(guint16 etype, int offset,
-		const u_char *pd, packet_counts *ld);
-void ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_ethertype,
+extern void capture_ethertype(guint16 etype, const u_char *pd, int offset,
+		int len, packet_counts *ld);
+extern void ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_ethertype,
 		packet_info *pinfo, proto_tree *tree, proto_tree *fh_tree,
 		int etype_id, int trailer_id);
 
