@@ -1,12 +1,11 @@
 /* plugins.c
  * plugin routines
  *
- * $Id: plugins.c,v 1.24 2001/04/25 08:41:21 guy Exp $
+ * $Id: plugins.c,v 1.44 2001/12/03 10:00:21 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1999 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +26,6 @@
 # include "config.h"
 #endif
 
-#include <epan.h>
 #include "plugins.h"
 
 #ifdef HAVE_PLUGINS
@@ -63,6 +61,8 @@
 #include "prefs.h"
 
 #ifdef PLUGINS_NEED_ADDRESS_TABLE
+#include "conversation.h"
+#include "packet-giop.h"
 #include "plugins/plugin_table.h"
 static plugin_address_table_t	patable;
 #endif
@@ -70,10 +70,7 @@ static plugin_address_table_t	patable;
 /* linked list of all plugins */
 plugin *plugin_list;
 
-#ifdef WIN32
-static gchar std_plug_dir[] = "c:/program files/ethereal/plugins/" VERSION;
-static gchar local_plug_dir[] = "c:/ethereal/plugins/" VERSION;
-#else
+#ifndef WIN32
 static gchar std_plug_dir[] = "/usr/lib/ethereal/plugins/" VERSION;
 static gchar local_plug_dir[] = "/usr/local/lib/ethereal/plugins/" VERSION;
 #endif
@@ -276,7 +273,12 @@ plugins_scan_dir(const char *dirname)
 void
 init_plugins(const char *plugin_dir)
 {
+#ifdef WIN32
+    const char *datafile_dir;
+    char *install_plugin_dir;
+#else
     struct stat std_dir_stat, local_dir_stat, plugin_dir_stat;
+#endif
 
     if (plugin_list == NULL)      /* ensure init_plugins is only run once */
     {
@@ -290,7 +292,11 @@ init_plugins(const char *plugin_dir)
 	patable.p_col_append_str		= col_append_str;
 	patable.p_col_set_str			= col_set_str;
 
-	patable.p_pi				= &pi;
+	patable.p_register_init_routine		= register_init_routine;
+	patable.p_conversation_new		= conversation_new;
+	patable.p_find_conversation		= find_conversation;
+	patable.p_match_strval			= match_strval;
+	patable.p_val_to_str			= val_to_str;
 
 	patable.p_proto_register_protocol	= proto_register_protocol;
 	patable.p_proto_register_field_array	= proto_register_field_array;
@@ -298,20 +304,21 @@ init_plugins(const char *plugin_dir)
 
 	patable.p_dissector_add			= dissector_add;
 	patable.p_dissector_delete		= dissector_delete;
+	patable.p_dissector_add_handle		= dissector_add_handle;
 
 	patable.p_heur_dissector_add		= heur_dissector_add;
 
 	patable.p_register_dissector		= register_dissector;
 	patable.p_find_dissector		= find_dissector;
+	patable.p_create_dissector_handle	= create_dissector_handle;
 	patable.p_call_dissector		= call_dissector;
 
-	patable.p_dissect_data			= dissect_data;
-
 	patable.p_proto_is_protocol_enabled	= proto_is_protocol_enabled;
 
 	patable.p_proto_item_get_len		= proto_item_get_len;
 	patable.p_proto_item_set_len		= proto_item_set_len;
 	patable.p_proto_item_set_text		= proto_item_set_text;
+	patable.p_proto_item_append_text	= proto_item_append_text;
 	patable.p_proto_item_add_subtree	= proto_item_add_subtree;
 	patable.p_proto_tree_add_item		= proto_tree_add_item;
 	patable.p_proto_tree_add_item_hidden	= proto_tree_add_item_hidden;
@@ -350,7 +357,6 @@ init_plugins(const char *plugin_dir)
 	patable.p_proto_tree_add_int_hidden	= proto_tree_add_int_hidden;
 	patable.p_proto_tree_add_int_format	= proto_tree_add_int_format;
 	patable.p_proto_tree_add_text		= proto_tree_add_text;
-	patable.p_proto_tree_add_notext		= proto_tree_add_notext;
 
 	patable.p_tvb_new_subset		= tvb_new_subset;
 
@@ -366,16 +372,10 @@ init_plugins(const char *plugin_dir)
 	patable.p_tvb_get_ntohs			= tvb_get_ntohs;
 	patable.p_tvb_get_ntoh24		= tvb_get_ntoh24;
 	patable.p_tvb_get_ntohl			= tvb_get_ntohl;
-#ifdef G_HAVE_GINT64
-	patable.p_tvb_get_ntohll		= tvb_get_ntohll;
-#endif
 
 	patable.p_tvb_get_letohs		= tvb_get_letohs;
 	patable.p_tvb_get_letoh24		= tvb_get_letoh24;
 	patable.p_tvb_get_letohl		= tvb_get_letohl;
-#ifdef G_HAVE_GINT64
-	patable.p_tvb_get_letohll		= tvb_get_letohll;
-#endif
 
 	patable.p_tvb_memcpy			= tvb_memcpy;
 	patable.p_tvb_memdup			= tvb_memdup;
@@ -404,8 +404,67 @@ init_plugins(const char *plugin_dir)
 	patable.p_prefs_register_uint_preference = prefs_register_uint_preference;
 	patable.p_prefs_register_bool_preference = prefs_register_bool_preference;
 	patable.p_prefs_register_enum_preference = prefs_register_enum_preference;
+	patable.p_prefs_register_string_preference = prefs_register_string_preference;
+
+	patable.p_register_giop_user		= register_giop_user;
+	patable.p_is_big_endian			= is_big_endian;
+	patable.p_get_CDR_string		= get_CDR_string;
+	patable.p_get_CDR_ulong			= get_CDR_ulong;
+	patable.p_get_CDR_enum			= get_CDR_enum;
+	patable.p_get_CDR_object		= get_CDR_object;
+	patable.p_get_CDR_boolean		= get_CDR_boolean;
 #endif
 
+#ifdef WIN32
+	/*
+	 * On Windows, the data file directory is the installation
+	 * directory; the plugins are stored under it.
+	 *
+	 * Assume we're running the installed version of Ethereal;
+	 * on Windows, the data file directory is the directory
+	 * in which the Ethereal binary resides.
+	 */
+	datafile_dir = get_datafile_dir();
+	install_plugin_dir = g_malloc(strlen(datafile_dir) + strlen("plugins") +
+	    strlen(VERSION) + 3);
+	sprintf(install_plugin_dir, "%s\\plugins\\%s", datafile_dir, VERSION);
+
+	/*
+	 * Make sure that pathname refers to a directory.
+	 */
+	if (test_for_directory(install_plugin_dir) != EISDIR) {
+		/*
+		 * Either it doesn't refer to a directory or it
+		 * refers to something that doesn't exist.
+		 *
+		 * Assume that means we're running, for example,
+		 * a version of Ethereal we've built in a source
+		 * directory, and fall back on the default
+		 * installation directory, so you can put the plugins
+		 * somewhere so they can be used with this version
+		 * of Ethereal.
+		 *
+		 * XXX - should we, instead, have the Windows build
+		 * procedure create a subdirectory of the "plugins"
+		 * source directory, and copy the plugin DLLs there,
+		 * so that you use the plugins from the build tree?
+		 */
+		install_plugin_dir =
+		    g_strdup("C:\\Program Files\\Ethereal\\plugins\\" VERSION);
+	}
+
+	/*
+	 * Scan that directory.
+	 */
+	plugins_scan_dir(install_plugin_dir);
+	g_free(install_plugin_dir);
+#else
+	/*
+	 * XXX - why not just scan "plugin_dir"?  That's where we
+	 * installed the plugins; if Ethereal isn't installed under
+	 * "/usr" or "/usr/local", why should we search for its plugins
+	 * there?
+	 */
 	plugins_scan_dir(std_plug_dir);
 	plugins_scan_dir(local_plug_dir);
 	if ((strcmp(std_plug_dir, plugin_dir) != 0) &&
@@ -441,14 +500,9 @@ init_plugins(const char *plugin_dir)
 		else plugins_scan_dir(plugin_dir);
 	    }
 	}
+#endif
 	if (!user_plug_dir)
-	{
-	    user_plug_dir = (gchar *)g_malloc(strlen(get_home_dir()) +
-					      strlen(PF_DIR) +
-					      strlen(PLUGINS_DIR_NAME) + 3);
-	    sprintf(user_plug_dir, "%s/%s/%s", get_home_dir(), 
-		    PF_DIR, PLUGINS_DIR_NAME);
-	}
+	    user_plug_dir = get_persconffile_path(PLUGINS_DIR_NAME, FALSE);
 	plugins_scan_dir(user_plug_dir);
     }
 }
