@@ -1,7 +1,7 @@
 /* proto.c
  * Routines for protocol tree
  *
- * $Id: proto.c,v 1.30 2001/07/13 00:55:53 guy Exp $
+ * $Id: proto.c,v 1.48 2001/12/18 19:09:04 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -44,6 +44,7 @@
 #include "plugins.h"
 #include "ipv6-utils.h"
 #include "proto.h"
+#include "int-64bit.h"
 
 #define cVALS(x) (const value_string*)(x)
 
@@ -52,6 +53,8 @@ proto_tree_free_node(GNode *node, gpointer data);
 
 static void fill_label_boolean(field_info *fi, gchar *label_str);
 static void fill_label_uint(field_info *fi, gchar *label_str);
+static void fill_label_uint64(field_info *fi, gchar *label_str);
+static void fill_label_int64(field_info *fi, gchar *label_str);
 static void fill_label_enumerated_uint(field_info *fi, gchar *label_str);
 static void fill_label_enumerated_bitfield(field_info *fi, gchar *label_str);
 static void fill_label_numeric_bitfield(field_info *fi, gchar *label_str);
@@ -70,22 +73,27 @@ static proto_item*
 proto_tree_add_node(proto_tree *tree, field_info *fi);
 
 static field_info *
-alloc_field_info(int hfindex, tvbuff_t *tvb, gint start, gint length);
+alloc_field_info(proto_tree *tree, int hfindex, tvbuff_t *tvb,
+        gint start, gint length);
 
 static proto_item *
-proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint length,
-		field_info **pfi);
+proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb,
+        gint start, gint length, field_info **pfi);
 static void
 proto_tree_set_representation(proto_item *pi, const char *format, va_list ap);
 
 static void
 proto_tree_set_protocol_tvb(field_info *fi, tvbuff_t *tvb);
 static void
+proto_tree_set_uint64(field_info *fi, const guint8 *value_ptr, gboolean little_endian);
+static void
+proto_tree_set_uint64_tvb(field_info *fi, tvbuff_t *tvb, gint start, gboolean little_endian);
+static void
 proto_tree_set_bytes(field_info *fi, const guint8* start_ptr, gint length);
 static void
 proto_tree_set_bytes_tvb(field_info *fi, tvbuff_t *tvb, gint offset, gint length);
 static void
-proto_tree_set_time(field_info *fi, struct timeval *value_ptr);
+proto_tree_set_time(field_info *fi, nstime_t *value_ptr);
 static void
 proto_tree_set_string(field_info *fi, const char* value);
 static void
@@ -126,12 +134,17 @@ typedef struct {
 	GList	*last_field;	/* pointer to end of list of fields */
 	gboolean is_enabled;	/* TRUE if protocol is enabled */
 	gboolean can_disable;	/* TRUE if protocol can be disabled */
-	gpointer dissector;
 } protocol_t;
 
 /* List of all protocols */
 static GList *protocols;
 
+#define INITIAL_NUM_PROTOCOL_HFINFO     200
+#define INITIAL_NUM_FIELD_INFO          100
+#define INITIAL_NUM_PROTO_NODE          100
+#define INITIAL_NUM_ITEM_LABEL          100
+
+
 /* Contains information about protocols and header fields. Used when
  * dissectors register their data */
 static GMemChunk *gmc_hfinfo = NULL;
@@ -140,6 +153,9 @@ static GMemChunk *gmc_hfinfo = NULL;
  * proto_tree_add_item.  */
 static GMemChunk *gmc_field_info = NULL;
 
+/* Contains the space for proto_nodes. */
+static GMemChunk *gmc_proto_node = NULL;
+
 /* String space for protocol and field items for the GUI */
 static GMemChunk *gmc_item_labels = NULL;
 
@@ -154,12 +170,6 @@ gboolean	*tree_is_expanded;
 /* Number of elements in that array. */
 int		num_tree_types;
 
-/* Is the parsing being done for a visible proto_tree or an invisible one?
- * By setting this correctly, the proto_tree creation is sped up by not
- * having to call vsnprintf and copy strings around.
- */
-gboolean proto_tree_is_visible = FALSE;
-
 /* initialize data structures and register protocols and fields */
 void
 proto_init(const char *plugin_dir,void (register_all_protocols)(void),
@@ -175,21 +185,35 @@ proto_init(const char *plugin_dir,void (register_all_protocols)(void),
 		g_mem_chunk_destroy(gmc_hfinfo);
 	if (gmc_field_info)
 		g_mem_chunk_destroy(gmc_field_info);
+	if (gmc_proto_node)
+		g_mem_chunk_destroy(gmc_proto_node);
 	if (gmc_item_labels)
 		g_mem_chunk_destroy(gmc_item_labels);
 	if (gpa_hfinfo)
-		g_ptr_array_free(gpa_hfinfo, FALSE);
+		g_ptr_array_free(gpa_hfinfo, TRUE);
 	if (tree_is_expanded != NULL)
 		g_free(tree_is_expanded);
 
 	gmc_hfinfo = g_mem_chunk_new("gmc_hfinfo",
-		sizeof(header_field_info), 50 * sizeof(header_field_info), G_ALLOC_ONLY);
+		sizeof(header_field_info),
+        INITIAL_NUM_PROTOCOL_HFINFO * sizeof(header_field_info),
+        G_ALLOC_ONLY);
+
 	gmc_field_info = g_mem_chunk_new("gmc_field_info",
-		sizeof(struct field_info), 200 * sizeof(struct field_info),
+		sizeof(field_info),
+        INITIAL_NUM_FIELD_INFO * sizeof(field_info),
 		G_ALLOC_AND_FREE);
+
+	gmc_proto_node = g_mem_chunk_new("gmc_proto_node",
+		sizeof(proto_node),
+        INITIAL_NUM_PROTO_NODE * sizeof(proto_node),
+		G_ALLOC_AND_FREE);
+
 	gmc_item_labels = g_mem_chunk_new("gmc_item_labels",
-		ITEM_LABEL_LENGTH, 20 * ITEM_LABEL_LENGTH,
+		ITEM_LABEL_LENGTH,
+        INITIAL_NUM_ITEM_LABEL* ITEM_LABEL_LENGTH,
 		G_ALLOC_AND_FREE);
+
 	gpa_hfinfo = g_ptr_array_new();
 
 	/* Allocate "tree_is_expanded", with one element for ETT_NONE,
@@ -242,10 +266,14 @@ proto_cleanup(void)
 		g_mem_chunk_destroy(gmc_hfinfo);
 	if (gmc_field_info)
 		g_mem_chunk_destroy(gmc_field_info);
+	if (gmc_proto_node)
+		g_mem_chunk_destroy(gmc_proto_node);
 	if (gmc_item_labels)
 		g_mem_chunk_destroy(gmc_item_labels);
 	if (gpa_hfinfo)
-		g_ptr_array_free(gpa_hfinfo, FALSE);
+		g_ptr_array_free(gpa_hfinfo, TRUE);
+	if (tree_is_expanded != NULL)
+		g_free(tree_is_expanded);
 
 	/* Cleanup the ftype subsystem */
 	ftypes_cleanup();
@@ -255,8 +283,11 @@ proto_cleanup(void)
 void
 proto_tree_free(proto_tree *tree)
 {
+    /* Free all the data pointed to by the tree. */
 	g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, -1,
 		proto_tree_free_node, NULL);
+
+    /* Then free the tree. */
 	g_node_destroy((GNode*)tree);
 }
 
@@ -267,33 +298,82 @@ free_field_info(void *fi)
 	g_mem_chunk_free(gmc_field_info, (field_info*)fi);
 }
 
+static void
+free_GPtrArray_value(gpointer key, gpointer value, gpointer user_data)
+{
+    GPtrArray   *ptrs = value;
+
+    g_ptr_array_free(ptrs, TRUE);
+}
+
+static void
+free_node_tree_data(tree_data_t *tree_data)
+{
+        /* Free all the GPtrArray's in the interesting_hfids hash. */
+        g_hash_table_foreach(tree_data->interesting_hfids,
+            free_GPtrArray_value, NULL);
+
+        /* And then destroy the hash. */
+        g_hash_table_destroy(tree_data->interesting_hfids);
+
+        /* And finally the tree_data_t itself. */
+        g_free(tree_data);
+}
+
+static void
+free_node_field_info(field_info* finfo)
+{
+		if (finfo->representation) {
+			g_mem_chunk_free(gmc_item_labels, finfo->representation);
+		}
+		fvalue_free(finfo->value);
+		free_field_info(finfo);
+}
+
 static gboolean
 proto_tree_free_node(GNode *node, gpointer data)
 {
-	field_info *fi = (field_info*) (node->data);
+	field_info *finfo = PITEM_FINFO(node);
 
-	if (fi != NULL) {
-		if (fi->representation) {
-			g_mem_chunk_free(gmc_item_labels, fi->representation);
-		}
-		fvalue_free(fi->value);
-		free_field_info(fi);
+    if (finfo == NULL) {
+        /* This is the root GNode. Destroy the per-tree data.
+         * There is no field_info to destroy. */
+        free_node_tree_data(PTREE_DATA(node));
+    }
+    else {
+        /* This is a child GNode. Don't free the per-tree data, but
+         * do free the field_info data. */
+        free_node_field_info(finfo);
 	}
+
+    /* Free the proto_node. */
+	g_mem_chunk_free(gmc_proto_node, GNODE_PNODE(node));
+
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}	
+}
+
+/* Is the parsing being done for a visible proto_tree or an invisible one?
+ * By setting this correctly, the proto_tree creation is sped up by not
+ * having to call vsnprintf and copy strings around.
+ */
+void
+proto_tree_set_visible(proto_tree *tree, gboolean visible)
+{
+    PTREE_DATA(tree)->visible = visible;
+}
 
 /* Finds a record in the hf_info_records array by id. */
 header_field_info*
 proto_registrar_get_nth(int hfindex)
 {
-	g_assert(hfindex >= 0 && hfindex < gpa_hfinfo->len);
+	g_assert(hfindex >= 0 && (guint) hfindex < gpa_hfinfo->len);
 	return g_ptr_array_index(gpa_hfinfo, hfindex);
 }
 
 
-/* Add a node with no text */
-proto_item *
-proto_tree_add_notext(proto_tree *tree, tvbuff_t *tvb, gint start, gint length)
+/* Add a text-only node, leaving it to our caller to fill the text in */
+static proto_item *
+proto_tree_add_text_node(proto_tree *tree, tvbuff_t *tvb, gint start, gint length)
 {
 	proto_item	*pi;
 
@@ -312,7 +392,7 @@ proto_tree_add_text(proto_tree *tree, tvbuff_t *tvb, gint start, gint length,
 	proto_item	*pi;
 	va_list		ap;
 
-	pi = proto_tree_add_notext(tree, tvb, start, length);
+	pi = proto_tree_add_text_node(tree, tvb, start, length);
 	if (pi == NULL)
 		return(NULL);
 
@@ -330,7 +410,7 @@ proto_tree_add_text_valist(proto_tree *tree, tvbuff_t *tvb, gint start,
 {
 	proto_item	*pi;
 
-	pi = proto_tree_add_notext(tree, tvb, start, length);
+	pi = proto_tree_add_text_node(tree, tvb, start, length);
 	if (pi == NULL)
 		return(NULL);
 
@@ -348,7 +428,7 @@ proto_tree_add_debug_text(proto_tree *tree, const char *format, ...)
 	proto_item	*pi;
 	va_list		ap;
 
-	pi = proto_tree_add_notext(tree, NULL, 0, 0);
+	pi = proto_tree_add_text_node(tree, NULL, 0, 0);
 	if (pi == NULL)
 		return(NULL);
 
@@ -445,11 +525,13 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 	guint32		value, n;
 	char		*string;
 	int		found_length;
+    GHashTable  *hash;
+    GPtrArray   *ptrs;
 
 	if (!tree)
 		return(NULL);
 
-	new_fi = alloc_field_info(hfindex, tvb, start, length);
+	new_fi = alloc_field_info(tree, hfindex, tvb, start, length);
 
 	if (new_fi == NULL)
 		return(NULL);
@@ -485,6 +567,12 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 			    get_uint_value(tvb, start, length, little_endian));
 			break;
 
+		case FT_INT64:
+		case FT_UINT64:
+			g_assert(length == 8);
+			proto_tree_set_uint64_tvb(new_fi, tvb, start, little_endian);
+			break;
+
 		/* XXX - make these just FT_INT? */
 		case FT_INT8:
 		case FT_INT16:
@@ -542,11 +630,11 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 		case FT_UINT_STRING:
 			/* This g_strdup'ed memory is freed in proto_tree_free_node() */
 			n = get_uint_value(tvb, start, length, little_endian);
-			proto_tree_set_string_tvb(new_fi, tvb, start + 1, n);
+			proto_tree_set_string_tvb(new_fi, tvb, start + length, n);
 
 			/* Instead of calling proto_item_set_len(), since we don't yet
 			 * have a proto_item, we set the field_info's length ourselves. */
-			new_fi->length = n + 1;
+			new_fi->length = n + length;
 			break;
 		default:
 			g_error("new_fi->hfinfo->type %d (%s) not handled\n",
@@ -562,6 +650,14 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 	 * raised by a tvbuff access method doesn't leave junk in the proto_tree. */
 	pi = proto_tree_add_node(tree, new_fi);
 
+    /* If the proto_tree wants to keep a record of this finfo
+     * for quick lookup, then record it. */
+    hash = PTREE_DATA(tree)->interesting_hfids;
+    ptrs = g_hash_table_lookup(hash, GINT_TO_POINTER(hfindex));
+    if (ptrs) {
+        g_ptr_array_add(ptrs, new_fi);
+    }
+
 	return pi;
 }
 
@@ -576,7 +672,7 @@ proto_tree_add_item_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 	if (pi == NULL)
 		return(NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -679,7 +775,7 @@ proto_tree_add_bytes_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint s
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -725,7 +821,7 @@ proto_tree_set_bytes_tvb(field_info *fi, tvbuff_t *tvb, gint offset, gint length
 /* Add a FT_*TIME to a proto_tree */
 proto_item *
 proto_tree_add_time(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint length,
-		struct timeval *value_ptr)
+		nstime_t *value_ptr)
 {
 	proto_item		*pi;
 	field_info		*new_fi;
@@ -746,7 +842,7 @@ proto_tree_add_time(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gi
 
 proto_item *
 proto_tree_add_time_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint length,
-		struct timeval *value_ptr)
+		nstime_t *value_ptr)
 {
 	proto_item		*pi;
 	field_info 		*fi;
@@ -755,7 +851,7 @@ proto_tree_add_time_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -763,7 +859,7 @@ proto_tree_add_time_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 
 proto_item *
 proto_tree_add_time_format(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint length,
-		struct timeval *value_ptr, const char *format, ...)
+		nstime_t *value_ptr, const char *format, ...)
 {
 	proto_item		*pi;
 	va_list			ap;
@@ -781,7 +877,7 @@ proto_tree_add_time_format(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 
 /* Set the FT_*TIME value */
 static void
-proto_tree_set_time(field_info *fi, struct timeval *value_ptr)
+proto_tree_set_time(field_info *fi, nstime_t *value_ptr)
 {
 	fvalue_set(fi->value, value_ptr, FALSE);
 }
@@ -818,7 +914,7 @@ proto_tree_add_ipxnet_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -881,7 +977,7 @@ proto_tree_add_ipv4_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -944,7 +1040,7 @@ proto_tree_add_ipv6_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -981,6 +1077,28 @@ proto_tree_set_ipv6_tvb(field_info *fi, tvbuff_t *tvb, gint start)
 	proto_tree_set_ipv6(fi, tvb_get_ptr(tvb, start, 16));
 }
 
+static void
+proto_tree_set_uint64(field_info *fi, const guint8 *value_ptr, gboolean little_endian)
+{
+	if(little_endian){
+		unsigned char buffer[8];
+		int i;
+
+		for(i=0;i<8;i++){
+			buffer[i]=value_ptr[7-i];
+		}
+		fvalue_set(fi->value, (gpointer)buffer, FALSE);
+	} else {
+		fvalue_set(fi->value, (gpointer)value_ptr, FALSE);
+	}
+}
+
+static void
+proto_tree_set_uint64_tvb(field_info *fi, tvbuff_t *tvb, gint start, gboolean little_endian)
+{
+	proto_tree_set_uint64(fi, tvb_get_ptr(tvb, start, 8), little_endian);
+}
+
 /* Add a FT_STRING to a proto_tree */
 proto_item *
 proto_tree_add_string(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
@@ -1013,7 +1131,7 @@ proto_tree_add_string_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -1088,7 +1206,7 @@ proto_tree_add_ether_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint s
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -1157,7 +1275,7 @@ proto_tree_add_boolean_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -1220,7 +1338,7 @@ proto_tree_add_double_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -1292,7 +1410,7 @@ proto_tree_add_uint_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -1379,7 +1497,7 @@ proto_tree_add_int_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint sta
 	if (pi == NULL)
 		return (NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->visible = FALSE;
 
 	return pi;
@@ -1430,12 +1548,17 @@ proto_tree_set_int(field_info *fi, gint32 value)
 static proto_item *
 proto_tree_add_node(proto_tree *tree, field_info *fi)
 {
-	proto_item *pi;
+	GNode *new_gnode;
+    proto_node *pnode;
 
-	pi = (proto_item*) g_node_new(fi);
-	g_node_append((GNode*)tree, (GNode*)pi);
+    pnode = g_mem_chunk_alloc(gmc_proto_node);
+    pnode->finfo = fi;
+    pnode->tree_data = PTREE_DATA(tree);
 
-	return pi;
+    new_gnode = g_node_new(pnode);
+    g_node_append((GNode*)tree, new_gnode);
+
+    return (proto_item*) new_gnode;
 }
 
 
@@ -1447,13 +1570,24 @@ proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint
 {
 	proto_item	*pi;
 	field_info	*fi;
+    GHashTable  *hash;
+    GPtrArray   *ptrs;
 
 	if (!tree)
 		return(NULL);
 
-	fi = alloc_field_info(hfindex, tvb, start, length);
+	fi = alloc_field_info(tree, hfindex, tvb, start, length);
 	pi = proto_tree_add_node(tree, fi);
 
+    /* If the proto_tree wants to keep a record of this finfo
+     * for quick lookup, then record it. */
+    hash = PTREE_DATA(tree)->interesting_hfids;
+    ptrs = g_hash_table_lookup(hash, GINT_TO_POINTER(hfindex));
+    if (ptrs) {
+        g_ptr_array_add(ptrs, fi);
+    }
+
+    /* Does the caller want to know the fi pointer? */
 	if (pfi) {
 		*pfi = fi;
 	}
@@ -1462,13 +1596,19 @@ proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint
 }
 
 static field_info *
-alloc_field_info(int hfindex, tvbuff_t *tvb, gint start, gint length)
+alloc_field_info(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint length)
 {
 	field_info	*fi;
 
 	fi = g_mem_chunk_alloc(gmc_field_info);
 
-	g_assert(hfindex >= 0 && hfindex < gpa_hfinfo->len);
+	g_assert(hfindex >= 0 && (guint) hfindex < gpa_hfinfo->len);
+
+	/*
+	 * We only allow a null tvbuff if the item has a zero length,
+	 * i.e. if there's no data backing it.
+	 */
+	g_assert(tvb != NULL || length == 0);
 	fi->hfinfo = proto_registrar_get_nth(hfindex);
 	g_assert(fi->hfinfo != NULL);
 	fi->start = start;
@@ -1477,18 +1617,17 @@ alloc_field_info(int hfindex, tvbuff_t *tvb, gint start, gint length)
 	}
 	fi->length = length;
 	fi->tree_type = ETT_NONE;
-	fi->visible = proto_tree_is_visible;
+	fi->visible = PTREE_DATA(tree)->visible;
 	fi->representation = NULL;
 
 	fi->value = fvalue_new(fi->hfinfo->type);
 
 	/* add the data source name */
-	/* This has the hack to return a default name for NullTVB. This */
-	/* hack can be removed when all dissectors use tvbuffs */
-	if ( tvb)
+	if (tvb) {
 		fi->ds_name = tvb_get_name(tvb);
-	else
-		fi->ds_name = tvb_get_name(pi.compat_top_tvb);
+	} else {
+		fi->ds_name = NULL;
+	}
 
 	return fi;
 }
@@ -1498,7 +1637,7 @@ alloc_field_info(int hfindex, tvbuff_t *tvb, gint start, gint length)
 static void
 proto_tree_set_representation(proto_item *pi, const char *format, va_list ap)
 {
-	field_info *fi = (field_info*) (((GNode*)pi)->data);
+	field_info *fi = PITEM_FINFO(pi);
 
 	if (fi->visible) {
 		fi->representation = g_mem_chunk_alloc(gmc_item_labels);
@@ -1506,12 +1645,19 @@ proto_tree_set_representation(proto_item *pi, const char *format, va_list ap)
 	}
 }
 
+/* Set text of proto_item after having already been created. */
 void
 proto_item_set_text(proto_item *pi, const char *format, ...)
 {
-	field_info *fi = (field_info*) (((GNode*)pi)->data);
+	field_info *fi = NULL;
 	va_list	ap;
 
+	if (pi==NULL) {
+		return;
+	}
+
+	fi = PITEM_FINFO(pi);
+
 	if (fi->representation)
 		g_mem_chunk_free(gmc_item_labels, fi->representation);
 
@@ -1520,6 +1666,36 @@ proto_item_set_text(proto_item *pi, const char *format, ...)
 	va_end(ap);
 }
 
+/* Append to text of proto_item after having already been created. */
+void
+proto_item_append_text(proto_item *pi, const char *format, ...)
+{
+	field_info *fi = NULL;
+	size_t curlen;
+	va_list	ap;
+
+	if (pi==NULL) {
+		return;
+	}
+
+	fi = PITEM_FINFO(pi);
+
+	if (fi->visible) {
+		va_start(ap, format);
+		/*
+		 * XXX - this will blow up if we haven't already set
+		 * "fi->representation"; that seems OK to me - you
+		 * can't append to something that doesn't exist - but
+		 * there might be cases where that's not convenient.
+		 */
+		curlen = strlen(fi->representation);
+		if (ITEM_LABEL_LENGTH > curlen)
+			vsnprintf(fi->representation + curlen,
+			    ITEM_LABEL_LENGTH - curlen, format, ap);
+		va_end(ap);
+	}
+}
+
 void
 proto_item_set_len(proto_item *pi, gint length)
 {
@@ -1527,21 +1703,45 @@ proto_item_set_len(proto_item *pi, gint length)
 	
 	if (pi == NULL)
 		return;
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	fi->length = length;
 }
 
 int
 proto_item_get_len(proto_item *pi)
 {
-	field_info *fi = (field_info*) (((GNode*)pi)->data);
+	field_info *fi = PITEM_FINFO(pi);
 	return fi->length;
 }
 
 proto_tree*
 proto_tree_create_root(void)
 {
-	return (proto_tree*) g_node_new(NULL);
+    proto_node  *pnode;
+
+    /* Initialize the proto_node */
+    pnode = g_mem_chunk_alloc(gmc_proto_node);
+    pnode->finfo = NULL;
+    pnode->tree_data = g_new(tree_data_t, 1);
+
+    /* Initialize the tree_data_t */
+    pnode->tree_data->interesting_hfids =
+        g_hash_table_new(g_direct_hash, g_direct_equal);
+
+    /* Set the default to FALSE so it's easier to
+     * find errors; if we expect to see the protocol tree
+     * but for some reason the default 'visible' is not
+     * changed, then we'll find out very quickly. */
+    pnode->tree_data->visible = FALSE;
+
+	return (proto_tree*) g_node_new(pnode);
+}
+
+void
+proto_tree_prime_hfid(proto_tree *tree, int hfid)
+{
+    g_hash_table_insert(PTREE_DATA(tree)->interesting_hfids,
+            GINT_TO_POINTER(hfid), g_ptr_array_new());
 }
 
 proto_tree*
@@ -1551,7 +1751,7 @@ proto_item_add_subtree(proto_item *pi,  gint idx) {
 	if (!pi)
 		return(NULL);
 
-	fi = (field_info*) (((GNode*)pi)->data);
+	fi = PITEM_FINFO(pi);
 	g_assert(idx >= 0 && idx < num_tree_types);
 	fi->tree_type = idx;
 	return (proto_tree*) pi;
@@ -1563,7 +1763,7 @@ proto_compare_name(gconstpointer p1_arg, gconstpointer p2_arg)
 	const protocol_t *p1 = p1_arg;
 	const protocol_t *p2 = p2_arg;
 
-	return strcmp(p1->short_name, p2->short_name);
+	return g_strcasecmp(p1->short_name, p2->short_name);
 }
 
 int
@@ -1582,7 +1782,6 @@ proto_register_protocol(char *name, char *short_name, char *filter_name)
 	protocol->fields = NULL;
 	protocol->is_enabled = TRUE; /* protocol is enabled by default */
 	protocol->can_disable = TRUE;
-	protocol->dissector = NULL;
 	protocols = g_list_insert_sorted(protocols, protocol,
 	    proto_compare_name);
 
@@ -1743,32 +1942,6 @@ proto_set_cant_disable(int proto_id)
 	protocol->can_disable = FALSE;
 }
 
-gpointer
-proto_get_protocol_dissector(int proto_id)
-{
-	protocol_t *protocol;
-
-	protocol = find_protocol_by_id(proto_id);
-	if (protocol == NULL)
-		return(NULL);
-	return protocol->dissector;
-}
-
-void
-proto_set_protocol_dissector(int proto_id, gpointer dissector)
-{
-	protocol_t *protocol;
-
-	protocol = find_protocol_by_id(proto_id);
-	if (protocol != NULL) {
-		if (protocol->dissector != NULL) {
-			/* Already set */
-			return;
-		}
-		protocol->dissector = dissector;
-	}
-}
-
 /* for use with static arrays only, since we don't allocate our own copies
 of the header_field_info struct contained withing the hf_register_info struct */
 void
@@ -1929,6 +2102,10 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 			}
 			break;
 
+		case FT_UINT64:
+			fill_label_uint64(fi, label_str);
+			break;
+
 		case FT_INT8:
 		case FT_INT16:
 		case FT_INT24:
@@ -1942,6 +2119,10 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 			}
 			break;
 
+		case FT_INT64:
+			fill_label_int64(fi, label_str);
+			break;
+
 		case FT_DOUBLE:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
 				"%s: %g", hfinfo->name, fvalue_get_floating(fi->value));
@@ -1956,7 +2137,7 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 		case FT_RELATIVE_TIME:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
 				"%s: %s seconds", hfinfo->name,
-				rel_time_to_str(fvalue_get(fi->value)));
+				rel_time_to_secs_str(fvalue_get(fi->value)));
 			break;
 
 		case FT_IPXNET:
@@ -2007,6 +2188,54 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 	}
 }
 
+static void
+fill_label_uint64(field_info *fi, gchar *label_str)
+{
+	unsigned char *bytes;
+	header_field_info *hfinfo = fi->hfinfo;
+
+	bytes=fvalue_get(fi->value);
+	switch(hfinfo->display){
+	case BASE_DEC:
+		snprintf(label_str, ITEM_LABEL_LENGTH,
+			"%s: %s", hfinfo->name,
+			u64toa(bytes));
+		break;
+	case BASE_HEX:
+		snprintf(label_str, ITEM_LABEL_LENGTH,
+			"%s: %s", hfinfo->name,
+			u64toh(bytes));
+		break;
+	default:
+		g_assert_not_reached();
+		;
+	}
+}
+
+static void
+fill_label_int64(field_info *fi, gchar *label_str)
+{
+	unsigned char *bytes;
+	header_field_info *hfinfo = fi->hfinfo;
+
+	bytes=fvalue_get(fi->value);
+	switch(hfinfo->display){
+	case BASE_DEC:
+		snprintf(label_str, ITEM_LABEL_LENGTH,
+			"%s: %s", hfinfo->name,
+			i64toa(bytes));
+		break;
+	case BASE_HEX:
+		snprintf(label_str, ITEM_LABEL_LENGTH,
+			"%s: %s", hfinfo->name,
+			u64toh(bytes));
+		break;
+	default:
+		g_assert_not_reached();
+		;
+	}
+}
+
 static void
 fill_label_boolean(field_info *fi, gchar *label_str)
 {
@@ -2462,7 +2691,6 @@ proto_registrar_get_length(int n)
 /* used when calling proto search functions */
 typedef struct {
 	header_field_info	*target;
-	int			parent;
 	const guint8		*packet_data;
 	guint			packet_len;
 	gboolean		halt_on_first_hit;
@@ -2485,44 +2713,19 @@ proto_find_protocol_multi(proto_tree* tree, GNodeTraverseFunc callback,
 	g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2, callback, (gpointer*)sinfo);
 }
 
-/* Calls a traversal function for all subtrees where:
- * 1. Subtree is immediate child of root node. That is, subtree is a "protocol"
- * 2. Subtree has finfo such that finfo->hfinfo->id == sinfo->parent
+/* Calls a traversal function for all subtrees.
  */
 static gboolean
 traverse_subtree_for_field(GNode *node, gpointer data)
 {
-	field_info		*fi = (field_info*) (node->data);
+	field_info *fi = PITEM_FINFO(node);
 	proto_tree_search_info	*sinfo = (proto_tree_search_info*) data;
 
 	if (fi) { /* !fi == the top most container node which holds nothing */
-		if (fi->hfinfo->id == sinfo->parent) {
 			g_node_traverse(node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
 					sinfo->traverse_func, sinfo);
 			if (sinfo->result.node)
 				return sinfo->halt_on_first_hit; /* halt? continue? */
-		}
-	}
-	return FALSE; /* keep traversing */
-}
-
-static gboolean
-check_for_protocol_or_field_id(GNode *node, gpointer data)
-{
-	field_info		*fi = (field_info*) (node->data);
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*) data;
-	header_field_info	*hfinfo;
-
-	if (fi) { /* !fi == the top most container node which holds nothing */
-		/* Is this field one of the fields in the specified list
-		 * of fields? */
-		for (hfinfo = sinfo->target; hfinfo != NULL;
-		    hfinfo = hfinfo->same_name_next) {
-			if (fi->hfinfo == hfinfo) {
-				sinfo->result.node = node;
-				return TRUE; /* halt traversal */
-			}
-		}
 	}
 	return FALSE; /* keep traversing */
 }
@@ -2532,118 +2735,32 @@ check_for_protocol_or_field_id(GNode *node, gpointer data)
 gboolean
 proto_check_for_protocol_or_field(proto_tree* tree, int id)
 {
-	proto_tree_search_info	sinfo;
-	header_field_info	*hfinfo;
+    GPtrArray *ptrs = proto_get_finfo_ptr_array(tree, id);
 
-	hfinfo = proto_registrar_get_nth(id);
-
-	/* Find the first entry on the list of fields with the same
-	 * name as this field. */
-	while (hfinfo->same_name_prev != NULL)
-		hfinfo = hfinfo->same_name_prev;
-
-	sinfo.target		= hfinfo;
-	sinfo.result.node	= NULL;
-	sinfo.parent		= -1;
-	sinfo.traverse_func	= check_for_protocol_or_field_id;
-	sinfo.halt_on_first_hit	= TRUE;
-
-	/* do a quicker check if target is a protocol */
-	if (proto_registrar_is_protocol(id) == TRUE) {
-		proto_find_protocol_multi(tree, check_for_protocol_or_field_id, &sinfo);
-	}
-	else {
-		/* find the field's parent protocol */
-		sinfo.parent = proto_registrar_get_parent(id);
-
-		/* Go through each protocol subtree, checking if the protocols
-		 * is the parent protocol of the field that we're looking for.
-		 * We may have protocols that occur more than once (e.g., IP in IP),
-		 * so we do indeed have to check all protocol subtrees, looking
-		 * for the parent protocol. That's why proto_find_protocol()
-		 * is not used --- it assumes a protocol occurs only once. */
-		g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2,
-						traverse_subtree_for_field, &sinfo);
-	}
-
-	if (sinfo.result.node)
-		return TRUE;
-	else
-		return FALSE;
+    if (!ptrs) {
+        return FALSE;
+    }
+    else if (g_ptr_array_len(ptrs) > 0) {
+        return TRUE;
+    }
+    else {
+        return FALSE;
+    }
 }
 
-
-
-static gboolean
-get_finfo_ptr_array(GNode *node, gpointer data)
-{
-	field_info		*fi = (field_info*) (node->data);
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*) data;
-	header_field_info	*hfinfo;
-
-	if (fi) { /* !fi == the top most container node which holds nothing */
-		/* Is this field one of the fields in the specified list
-		 * of fields? */
-		for (hfinfo = sinfo->target; hfinfo != NULL;
-		    hfinfo = hfinfo->same_name_next) {
-			if (fi->hfinfo == hfinfo) {
-				if (!sinfo->result.ptr_array) {
-					sinfo->result.ptr_array = g_ptr_array_new();
-				}
-				g_ptr_array_add(sinfo->result.ptr_array,
-						(gpointer)fi);
-				return FALSE; /* keep traversing */
-			}
-		}
-	}
-	return FALSE; /* keep traversing */
-}
-
-/* Return GPtrArray* of field_info pointers for all hfindex that appear in tree
- * (we assume that a field will only appear under its registered parent's subtree) */
+/* Return GPtrArray* of field_info pointers for all hfindex that appear in tree.
+ * This only works if the hfindex was "primed" before the dissection
+ * took place, as we just pass back the already-created GPtrArray*.
+ * The caller should *not* free the GPtrArray*; proto_tree_free_node()
+ * handles that. */
 GPtrArray*
 proto_get_finfo_ptr_array(proto_tree *tree, int id)
 {
-	proto_tree_search_info	sinfo;
-	header_field_info	*hfinfo;
-
-	hfinfo = proto_registrar_get_nth(id);
-
-	/* Find the first entry on the list of fields with the same
-	 * name as this field. */
-	while (hfinfo->same_name_prev != NULL)
-		hfinfo = hfinfo->same_name_prev;
-
-	sinfo.target		= hfinfo;
-	sinfo.result.ptr_array	= NULL;
-	sinfo.parent		= -1;
-	sinfo.traverse_func	= get_finfo_ptr_array;
-	sinfo.halt_on_first_hit	= FALSE;
-
-	/* do a quicker check if target is a protocol */
-	if (proto_registrar_is_protocol(id) == TRUE) {
-		proto_find_protocol_multi(tree, get_finfo_ptr_array, &sinfo);
-	}
-	else {
-		/* find the field's parent protocol */
-		sinfo.parent = proto_registrar_get_parent(id);
-
-		/* Go through each protocol subtree, checking if the protocols
-		 * is the parent protocol of the field that we're looking for.
-		 * We may have protocols that occur more than once (e.g., IP in IP),
-		 * so we do indeed have to check all protocol subtrees, looking
-		 * for the parent protocol. That's why proto_find_protocol()
-		 * is not used --- it assumes a protocol occurs only once. */
-		sinfo.traverse_func = get_finfo_ptr_array;
-		g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2,
-						traverse_subtree_for_field, &sinfo);
-	}
-
-	return sinfo.result.ptr_array;
+    return g_hash_table_lookup(PTREE_DATA(tree)->interesting_hfids,
+            GINT_TO_POINTER(id));
 }
 
 
-
 typedef struct {
 	guint		offset;
 	field_info	*finfo;
@@ -2653,13 +2770,14 @@ typedef struct {
 static gboolean
 check_for_offset(GNode *node, gpointer data)
 {
-	field_info		*fi = node->data;
+	field_info          *fi = PITEM_FINFO(node);
 	offset_search_t		*offsearch = data;
 
 	/* !fi == the top most container node which holds nothing */
-	if (fi && fi->visible && !strcmp( offsearch->name,fi->ds_name)) {
-		if (offsearch->offset >= fi->start &&
-				offsearch->offset < (fi->start + fi->length)) {
+	if (fi && fi->visible && fi->ds_name &&
+	    strcmp(offsearch->name, fi->ds_name) == 0) {
+		if (offsearch->offset >= (guint) fi->start &&
+				offsearch->offset < (guint) (fi->start + fi->length)) {
 
 			offsearch->finfo = fi;
 			return FALSE; /* keep traversing */
@@ -2850,10 +2968,12 @@ proto_can_match_selected(field_info *finfo)
 		case FT_UINT16:
 		case FT_UINT24:
 		case FT_UINT32:
+		case FT_UINT64:
 		case FT_INT8:
 		case FT_INT16:
 		case FT_INT24:
 		case FT_INT32:
+		case FT_INT64:
 		case FT_IPv4:
 		case FT_IPXNET:
 		case FT_IPv6:
@@ -2916,6 +3036,22 @@ proto_alloc_dfilter_string(field_info *finfo, guint8 *pd)
 			snprintf(buf, dfilter_len, format, hfinfo->abbrev, fvalue_get_integer(finfo->value));
 			break;
 
+		case FT_UINT64:
+			stringified = u64toa(fvalue_get(finfo->value));
+			dfilter_len = abbrev_len + 4 + strlen(stringified) +1;
+			buf = g_malloc0(dfilter_len);
+			snprintf(buf, dfilter_len, "%s == %s", hfinfo->abbrev,
+					stringified);
+			break;
+
+		case FT_INT64:
+			stringified = i64toa(fvalue_get(finfo->value));
+			dfilter_len = abbrev_len + 4 + strlen(stringified) +1;
+			buf = g_malloc0(dfilter_len);
+			snprintf(buf, dfilter_len, "%s == %s", hfinfo->abbrev,
+					stringified);
+			break;
+
 		case FT_IPv4:
 			dfilter_len = abbrev_len + 4 + 15 + 1;
 			buf = g_malloc0(dfilter_len);
@@ -2955,7 +3091,7 @@ proto_alloc_dfilter_string(field_info *finfo, guint8 *pd)
 
 		case FT_ABSOLUTE_TIME:
 			value_str =
-			    abs_time_to_str((struct timeval *)fvalue_get(finfo->value));
+			    abs_time_to_str((nstime_t *)fvalue_get(finfo->value));
 			dfilter_len = abbrev_len + strlen(value_str) + 7;
 			buf = g_malloc0(dfilter_len);
 			snprintf(buf, dfilter_len, "%s == \"%s\"",
@@ -2964,7 +3100,7 @@ proto_alloc_dfilter_string(field_info *finfo, guint8 *pd)
 
 		case FT_RELATIVE_TIME:
 			value_str =
-			    rel_time_to_str((struct timeval *)fvalue_get(finfo->value));
+			    rel_time_to_secs_str((nstime_t *)fvalue_get(finfo->value));
 			dfilter_len = abbrev_len + strlen(value_str) + 4;
 			buf = g_malloc0(dfilter_len);
 			snprintf(buf, dfilter_len, "%s == %s",
