@@ -1,7 +1,7 @@
 /* resolv.c
  * Routines for network object lookup
  *
- * $Id: resolv.c,v 1.11 2001/06/07 22:07:02 guy Exp $
+ * $Id: resolv.c,v 1.19 2001/12/20 19:19:41 guy Exp $
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
@@ -83,11 +83,9 @@
 
 #include "prefs.h"
 
-#define EPATH_ETHERS 		"/etc/ethers"
-#define EPATH_IPXNETS 		"/etc/ipxnets"
-#define EPATH_MANUF  		DATAFILE_DIR "/manuf"
-#define EPATH_PERSONAL_ETHERS 	".ethereal/ethers"  /* with "$HOME/" prefix */
-#define EPATH_PERSONAL_IPXNETS 	".ethereal/ipxnets" /* with "$HOME/" prefix */
+#define ENAME_ETHERS 		"ethers"
+#define ENAME_IPXNETS 		"ipxnets"
+#define ENAME_MANUF		"manuf"
 
 #define MAXMANUFLEN	9	/* max vendor name length with ending '\0' */
 #define HASHETHSIZE	1024
@@ -167,14 +165,15 @@ static int 		ipxnet_resolution_initialized = 0;
 
 /*
  *  Global variables (can be changed in GUI sections)
+ *  XXX - they could be changed in GUI code, but there's currently no
+ *  GUI code to change them.
  */
 
-gchar *g_ethers_path  = EPATH_ETHERS;
-gchar *g_pethers_path = NULL; 		/* "$HOME"/EPATH_PERSONAL_ETHERS    */
-gchar *g_ipxnets_path  = EPATH_IPXNETS;
-gchar *g_pipxnets_path = NULL;		/* "$HOME"/EPATH_PERSONAL_IPXNETS   */
-gchar *g_manuf_path   = EPATH_MANUF;	/* may only be changed before the   */
-					/* first resolving call        	    */
+gchar *g_ethers_path  = NULL;		/* global ethers file    */
+gchar *g_pethers_path = NULL; 		/* personal ethers file  */
+gchar *g_ipxnets_path  = NULL;		/* global ipxnets file   */
+gchar *g_pipxnets_path = NULL;		/* personal ipxnets file */
+					/* first resolving call  */
 
 /*
  *  Local function definitions 
@@ -290,7 +289,13 @@ static guchar *host_name_lookup(guint addr, gboolean *found)
   tp->addr = addr;
   tp->next = NULL;
 
-  if (prefs.name_resolve & PREFS_RESOLV_NETWORK) {
+  /*
+   * The Windows "gethostbyaddr()" insists on translating 0.0.0.0 to
+   * the name of the host on which it's running; to work around that
+   * botch, we don't try to translate an all-zero IP address to a host
+   * name.
+   */
+  if (addr != 0 && (prefs.name_resolve & PREFS_RESOLV_NETWORK)) {
 #ifdef AVOID_DNS_TIMEOUT
     
     /* Quick hack to avoid DNS/YP timeout */
@@ -409,7 +414,7 @@ static int fgetline(char **buf, int *size, FILE *fp)
  * The following functions implement ethernet address resolution and
  * ethers files parsing (see ethers(4)). 
  *
- * /etc/manuf has the same format as ethers(4) except that names are 
+ * The manuf file has the same format as ethers(4) except that names are 
  * truncated to MAXMANUFLEN-1 characters and that an address contains 
  * only 3 bytes (instead of 6).
  *
@@ -431,7 +436,7 @@ static int fgetline(char **buf, int *size, FILE *fp)
 static int parse_ether_line(char *line, ether_t *eth, int six_bytes)
 {
   /*
-   *  See man ethers(4) for /etc/ethers file format
+   *  See man ethers(4) for ethers file format
    *  (not available on all systems).
    *  We allow both ethernet address separators (':' and '-'),
    *  as well as Ethereal's '.' separator.
@@ -624,20 +629,32 @@ static hashmanuf_t *manuf_name_lookup(const guint8 *addr)
 static void initialize_ethers(void)
 {
   ether_t *eth;
+  char *manuf_path;
+
+  /* Compute the pathname of the ethers file. */
+  if (g_ethers_path == NULL) {
+    g_ethers_path = g_malloc(strlen(get_systemfile_dir()) +
+			     strlen(ENAME_ETHERS) + 2);
+    sprintf(g_ethers_path, "%s" G_DIR_SEPARATOR_S "%s",
+	    get_systemfile_dir(), ENAME_ETHERS);
+  }
 
   /* Set g_pethers_path here, but don't actually do anything
    * with it. It's used in get_ethbyname() and get_ethbyaddr()
    */
-  if (g_pethers_path == NULL) {
-    g_pethers_path = g_malloc(strlen(get_home_dir()) + 
-			      strlen(EPATH_PERSONAL_ETHERS) + 2);
-    sprintf(g_pethers_path, "%s/%s", 
-	    get_home_dir(), EPATH_PERSONAL_ETHERS);
-  }
+  if (g_pethers_path == NULL)
+    g_pethers_path = get_persconffile_path(ENAME_ETHERS, FALSE);
 
   /* manuf hash table initialization */
 
-  set_ethent(g_manuf_path);
+  /* Compute the pathname of the manuf file */
+  manuf_path = (gchar *) g_malloc(strlen(get_datafile_dir()) +
+    strlen(ENAME_MANUF) + 2);
+  sprintf(manuf_path, "%s" G_DIR_SEPARATOR_S "%s", get_datafile_dir(),
+    ENAME_MANUF);
+  
+  /* Read it and initialize the hash table */
+  set_ethent(manuf_path);
 
   while ((eth = get_ethent(0))) {
     add_manuf_name(eth->addr, eth->name);
@@ -645,6 +662,8 @@ static void initialize_ethers(void)
 
   end_ethent();
 
+  g_free(manuf_path);
+
 } /* initialize_ethers */
 
 static hashether_t *add_eth_name(const guint8 *addr, const guchar *name)
@@ -912,16 +931,25 @@ static ipxnet_t *get_ipxnetbyaddr(guint32 addr)
 
 static void initialize_ipxnets(void)
 {
+  /* Compute the pathname of the ipxnets file.
+   *
+   * XXX - is there a notion of an "ipxnets file" in any flavor of
+   * UNIX, or with any add-on Netware package for UNIX?  If not,
+   * should the UNIX version of the ipxnets file be in the datafile
+   * directory as well?
+   */
+  if (g_ipxnets_path == NULL) {
+    g_ipxnets_path = g_malloc(strlen(get_systemfile_dir()) +
+			      strlen(ENAME_IPXNETS) + 2);
+    sprintf(g_ipxnets_path, "%s" G_DIR_SEPARATOR_S "%s",
+	    get_systemfile_dir(), ENAME_IPXNETS);
+  }
 
   /* Set g_pipxnets_path here, but don't actually do anything
    * with it. It's used in get_ipxnetbyname() and get_ipxnetbyaddr()
    */
-  if (g_pipxnets_path == NULL) {
-    g_pipxnets_path = g_malloc(strlen(get_home_dir()) + 
-			      strlen(EPATH_PERSONAL_IPXNETS) + 2);
-    sprintf(g_pipxnets_path, "%s/%s", 
-	    get_home_dir(), EPATH_PERSONAL_IPXNETS);
-  }
+  if (g_pipxnets_path == NULL)
+    g_pipxnets_path = get_persconffile_path(ENAME_IPXNETS, FALSE);
 
 } /* initialize_ipxnets */
 
@@ -1356,7 +1384,7 @@ gboolean get_host_ipaddr(const char *host, guint32 *addrp)
 			return FALSE;
 			/* Apparently, some versions of gethostbyaddr can
 			 * return IPv6 addresses. */
-		} else if (hp->h_length <= sizeof (struct in_addr)) {
+		} else if (hp->h_length <= (int) sizeof (struct in_addr)) {
 			memcpy(&ipaddr, hp->h_addr, hp->h_length);
 		} else {
 			return FALSE;
