@@ -9,9 +9,9 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.c,v 1.18 2001/07/02 07:11:40 guy Exp $
+ * $Id: tvbuff.c,v 1.26 2001/11/20 22:46:12 guy Exp $
  *
- * Copyright (c) 2000 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -143,6 +143,7 @@ tvb_init(tvbuff_t *tvb, tvbuff_type type)
 	tvb->real_data		= NULL;
 	tvb->raw_offset		= -1;
 	tvb->used_in		= NULL;
+	tvb->ds_name		= NULL;
 
 	switch(type) {
 		case TVBUFF_REAL_DATA:
@@ -203,6 +204,8 @@ tvb_free(tvbuff_t* tvb)
 			if (tvb->free_cb) {
 				tvb->free_cb(tvb->real_data);
 			}
+			if (tvb->ds_name)
+				g_free(tvb->ds_name);
 			break;
 
 		case TVBUFF_SUBSET:
@@ -211,6 +214,12 @@ tvb_free(tvbuff_t* tvb)
 			if (tvb->tvbuffs.subset.tvb) {
 				tvb_decrement_usage_count(tvb->tvbuffs.subset.tvb, 1);
 			}
+
+			/*
+			 * TVBUFF_SUBSET tvbuffs share a "ds_name" with
+			 * the parent tvbuff, so this tvbuff's "ds_name"
+			 * shouldn't be freed.
+			 */
 			break;
 
 		case TVBUFF_COMPOSITE:
@@ -228,6 +237,8 @@ tvb_free(tvbuff_t* tvb)
 				g_free(composite->end_offsets);
 			if (tvb->real_data)
 				g_free(tvb->real_data);
+			if (tvb->ds_name)
+				g_free(tvb->ds_name);
 
 			break;
 		}
@@ -360,13 +371,13 @@ compute_offset_length(tvbuff_t *tvb, gint offset, gint length,
 	/* Compute the offset */
 	if (offset >= 0) {
 		/* Positive offset - relative to the beginning of the packet. */
-		if (offset > tvb->reported_length) {
+		if ((guint) offset > tvb->reported_length) {
 			if (exception) {
 				*exception = ReportedBoundsError;
 			}
 			return FALSE;
 		}
-		else if (offset > tvb->length) {
+		else if ((guint) offset > tvb->length) {
 			if (exception) {
 				*exception = BoundsError;
 			}
@@ -378,13 +389,13 @@ compute_offset_length(tvbuff_t *tvb, gint offset, gint length,
 	}
 	else {
 		/* Negative offset - relative to the end of the packet. */
-		if (-offset > tvb->reported_length) {
+		if ((guint) -offset > tvb->reported_length) {
 			if (exception) {
 				*exception = ReportedBoundsError;
 			}
 			return FALSE;
 		}
-		else if (-offset > tvb->length) {
+		else if ((guint) -offset > tvb->length) {
 			if (exception) {
 				*exception = BoundsError;
 			}
@@ -553,7 +564,7 @@ tvb_composite_finalize(tvbuff_t* tvb)
 	composite->end_offsets = g_new(guint, num_members);
 
 	for (slist = composite->tvbs; slist != NULL; slist = slist->next) {
-		g_assert(i < num_members);
+		g_assert((guint) i < num_members);
 		member_tvb = slist->data;
 		composite->start_offsets[i] = tvb->length;
 		tvb->length += member_tvb->length;
@@ -723,15 +734,6 @@ tvb_raw_offset(tvbuff_t *tvb)
 	return tvb->raw_offset;
 }
 
-void
-tvb_compat(tvbuff_t *tvb, const guint8 **pd, int *offset)
-{
-	g_assert(tvb->initialized);
-	*pd = first_real_data_ptr(tvb);
-	*offset = tvb_raw_offset(tvb);
-}
-
-
 static guint8*
 composite_ensure_contiguous(tvbuff_t *tvb, guint abs_offset, guint abs_length)
 {
@@ -805,7 +807,7 @@ guint8_find(const guint8* haystack, size_t haystacklen, guint8 needle)
 	const guint8	*b;
 	int		i;
 
-	for (b = haystack, i = 0; i < haystacklen; i++, b++) {
+	for (b = haystack, i = 0; (guint) i < haystacklen; i++, b++) {
 		if (*b == needle) {
 			return b;
 		}
@@ -821,7 +823,7 @@ guint8_pbrk(const guint8* haystack, size_t haystacklen, guint8 *needles)
 	int		i;
 	guint8		item, *needlep, needle;
 
-	for (b = haystack, i = 0; i < haystacklen; i++, b++) {
+	for (b = haystack, i = 0; (guint) i < haystacklen; i++, b++) {
 		item = *b;
 		needlep = needles;
 		while ((needle = *needlep) != '\0') {
@@ -982,17 +984,6 @@ tvb_get_ntohl(tvbuff_t *tvb, gint offset)
 	return pntohl(ptr);
 }
 
-#ifdef G_HAVE_GINT64
-guint64
-tvb_get_ntohll(tvbuff_t *tvb, gint offset)
-{
-	guint8* ptr;
-
-	ptr = ensure_contiguous(tvb, offset, sizeof(guint64));
-	return pntohll(ptr);
-}
-#endif
-
 guint16
 tvb_get_letohs(tvbuff_t *tvb, gint offset)
 {
@@ -1020,18 +1011,6 @@ tvb_get_letohl(tvbuff_t *tvb, gint offset)
 	return pletohl(ptr);
 }
 
-#ifdef G_HAVE_GINT64
-guint64
-tvb_get_letohll(tvbuff_t *tvb, gint offset)
-{
-	guint8* ptr;
-
-	ptr = ensure_contiguous(tvb, offset, sizeof(guint64));
-	return pletohll(ptr);
-}
-#endif
-
-
 /* Find first occurence of needle in tvbuff, starting at offset. Searches
  * at most maxlength number of bytes; if maxlength is -1, searches to
  * end of tvbuff.
@@ -1040,7 +1019,7 @@ tvb_get_letohll(tvbuff_t *tvb, gint offset)
  * in that case, -1 will be returned if the boundary is reached before
  * finding needle. */
 gint
-tvb_find_guint8(tvbuff_t *tvb, gint offset, guint maxlength, guint8 needle)
+tvb_find_guint8(tvbuff_t *tvb, gint offset, gint maxlength, guint8 needle)
 {
 	const guint8	*result;
 	guint		abs_offset, junk_length;
@@ -1055,7 +1034,7 @@ tvb_find_guint8(tvbuff_t *tvb, gint offset, guint maxlength, guint8 needle)
 		/* No maximum length specified; search to end of tvbuff. */
 		limit = tvbufflen;
 	}
-	else if (tvbufflen < maxlength) {
+	else if (tvbufflen < (guint) maxlength) {
 		/* Maximum length goes past end of tvbuff; search to end
 		   of tvbuff. */
 		limit = tvbufflen;
@@ -1103,7 +1082,7 @@ tvb_find_guint8(tvbuff_t *tvb, gint offset, guint maxlength, guint8 needle)
  * in that case, -1 will be returned if the boundary is reached before
  * finding needle. */
 gint
-tvb_pbrk_guint8(tvbuff_t *tvb, gint offset, guint maxlength, guint8 *needles)
+tvb_pbrk_guint8(tvbuff_t *tvb, gint offset, gint maxlength, guint8 *needles)
 {
 	const guint8	*result;
 	guint		abs_offset, junk_length;
@@ -1118,7 +1097,7 @@ tvb_pbrk_guint8(tvbuff_t *tvb, gint offset, guint maxlength, guint8 *needles)
 		/* No maximum length specified; search to end of tvbuff. */
 		limit = tvbufflen;
 	}
-	else if (tvbufflen < maxlength) {
+	else if (tvbufflen < (guint) maxlength) {
 		/* Maximum length goes past end of tvbuff; search to end
 		   of tvbuff. */
 		limit = tvbufflen;
@@ -1330,7 +1309,7 @@ tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
 {
 	gint	stringlen;
 	guint	abs_offset, junk_length;
-	gint	limit;
+	gint	limit, len;
 
 	check_offset_length(tvb, offset, 0, &abs_offset, &junk_length);
 
@@ -1340,7 +1319,14 @@ tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
 	}
 
 	/* Only copy to end of tvbuff, w/o throwing exception. */
-	if (tvb_length_remaining(tvb, abs_offset) < maxlength) {
+	len = tvb_length_remaining(tvb, abs_offset);
+
+    /* This should not happen because check_offset_length() would
+     * have already thrown an exception if 'offset' were out-of-bounds.
+     */
+    g_assert(len != -1);
+
+    if ((guint)len < maxlength) {
 		limit = maxlength - (tvb_length(tvb) - abs_offset);
 	}
 	else {
