@@ -1,7 +1,7 @@
 /* file.c
  * File I/O routines
  *
- * $Id: file.c,v 1.242 2001/07/05 00:34:38 guy Exp $
+ * $Id: file.c,v 1.255 2001/12/18 19:09:01 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -69,6 +69,7 @@
 #endif
 
 #include <epan.h>
+#include <filesystem.h>
 
 #include "gtk/main.h"
 #include "color.h"
@@ -91,13 +92,14 @@
 #include "reassemble.h"
 #include "globals.h"
 #include "gtk/colors.h"
+#include "epan_dissect.h"
 
 extern GtkWidget *packet_list, *byte_nb_ptr, *tree_view;
 
 static guint32 firstsec, firstusec;
 static guint32 prevsec, prevusec;
 
-static void read_packet(capture_file *cf, int offset);
+static void read_packet(capture_file *cf, long offset);
 
 static void rescan_packets(capture_file *cf, const char *action,
 	gboolean refilter, gboolean redissect);
@@ -174,6 +176,7 @@ open_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
 
   cf->cd_t      = wtap_file_type(cf->wth);
   cf->count     = 0;
+  cf->marked_count = 0;
   cf->drops_known = FALSE;
   cf->drops     = 0;
   cf->esec      = 0;
@@ -313,7 +316,7 @@ read_cap_file(capture_file *cf, int *err)
   char     *errmsg;
   char      errmsg_errno[1024+1];
   gchar     err_str[2048+1];
-  int       data_offset;
+  long      data_offset;
   progdlg_t *progbar;
   gboolean  stop_flag;
   int       file_pos;
@@ -476,7 +479,7 @@ start_tail_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
 read_status_t
 continue_tail_cap_file(capture_file *cf, int to_read, int *err)
 {
-  int data_offset = 0;
+  long data_offset = 0;
 
   gtk_clist_freeze(GTK_CLIST(packet_list));
 
@@ -517,7 +520,7 @@ continue_tail_cap_file(capture_file *cf, int to_read, int *err)
 read_status_t
 finish_tail_cap_file(capture_file *cf, int *err)
 {
-  int data_offset;
+  long data_offset;
 
   gtk_clist_freeze(GTK_CLIST(packet_list));
 
@@ -613,7 +616,7 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
 {
   apply_color_filter_args args;
   gint          i, row;
-  proto_tree   *protocol_tree = NULL;
+  gboolean	create_proto_tree = FALSE;
   epan_dissect_t *edt;
   GdkColor      fg, bg;
 
@@ -628,12 +631,6 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
     firstusec = fdata->abs_usecs;
   }
 
-  fdata->cinfo = &cf->cinfo;
-  for (i = 0; i < fdata->cinfo->num_cols; i++) {
-    fdata->cinfo->col_buf[i][0] = '\0';
-    fdata->cinfo->col_data[i] = fdata->cinfo->col_buf[i];
-  }
-
   /* If either
 
 	we have a display filter and are re-applying it;
@@ -644,10 +641,19 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
      a protocol tree against which a filter expression can be
      evaluated. */
   if ((cf->dfcode != NULL && refilter) || filter_list != NULL)
-    protocol_tree = proto_tree_create_root();
+	  create_proto_tree = TRUE;
 
   /* Dissect the frame. */
-  edt = epan_dissect_new(pseudo_header, buf, fdata, protocol_tree);
+  edt = epan_dissect_new(create_proto_tree, FALSE);
+
+  if (cf->dfcode) {
+      epan_dissect_prime_dfilter(edt, cf->dfcode);
+  }
+  if (filter_list) {
+      filter_list_prime_edt(edt);
+  }
+  epan_dissect_run(edt, pseudo_header, buf, fdata, &cf->cinfo);
+
 
   /* If we have a display filter, apply it if we're refiltering, otherwise
      leave the "passed_dfilter" flag alone.
@@ -672,12 +678,6 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
     }
   }
 
-  /* There are no more filters to apply, so we don't need any protocol
-     tree; free it if we created it. */
-  if (protocol_tree != NULL)
-    proto_tree_free(protocol_tree);
-
-  epan_dissect_free(edt);
 
   if (fdata->flags.passed_dfilter) {
     /* This frame passed the display filter, so add it to the clist. */
@@ -698,8 +698,8 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
     /* If it's greater than the current elapsed time, set the elapsed time
        to it (we check for "greater than" so as not to be confused by
        time moving backwards). */
-    if (cf->esec < fdata->rel_secs
-	|| (cf->esec == fdata->rel_secs && cf->eusec < fdata->rel_usecs)) {
+    if ((gint32)cf->esec < fdata->rel_secs
+	|| ((gint32)cf->esec == fdata->rel_secs && (gint32)cf->eusec < fdata->rel_usecs)) {
       cf->esec = fdata->rel_secs;
       cf->eusec = fdata->rel_usecs;
     }
@@ -711,7 +711,7 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
     prevsec = fdata->abs_secs;
     prevusec = fdata->abs_usecs;
 
-    fill_in_columns(fdata);
+    epan_dissect_fill_in_columns(edt);
 
     /* If we haven't yet seen the first frame, this is it.
 
@@ -732,7 +732,7 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
     /* This is the last frame we've seen so far. */
     cf->last_displayed = fdata;
 
-    row = gtk_clist_append(GTK_CLIST(packet_list), fdata->cinfo->col_data);
+    row = gtk_clist_append(GTK_CLIST(packet_list), cf->cinfo.col_data);
     gtk_clist_set_row_data(GTK_CLIST(packet_list), row, fdata);
 
     if (fdata->flags.marked) {
@@ -752,19 +752,18 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
        to the clist, and thus has no row. */
     row = -1;
   }
-  fdata->cinfo = NULL;
+  epan_dissect_free(edt);
   return row;
 }
 
 static void
-read_packet(capture_file *cf, int offset)
+read_packet(capture_file *cf, long offset)
 {
   const struct wtap_pkthdr *phdr = wtap_phdr(cf->wth);
   union wtap_pseudo_header *pseudo_header = wtap_pseudoheader(cf->wth);
   const u_char *buf = wtap_buf_ptr(cf->wth);
   frame_data   *fdata;
   int           passed;
-  proto_tree   *protocol_tree;
   frame_data   *plist_end;
   epan_dissect_t *edt;
 
@@ -784,14 +783,13 @@ read_packet(capture_file *cf, int offset)
   fdata->flags.encoding = CHAR_ASCII;
   fdata->flags.visited = 0;
   fdata->flags.marked = 0;
-  fdata->cinfo = NULL;
 
   passed = TRUE;
   if (cf->rfcode) {
-    protocol_tree = proto_tree_create_root();
-    edt = epan_dissect_new(pseudo_header, buf, fdata, protocol_tree);
+    edt = epan_dissect_new(TRUE, FALSE);
+    epan_dissect_prime_dfilter(edt, cf->rfcode);
+    epan_dissect_run(edt, pseudo_header, buf, fdata, NULL);
     passed = dfilter_apply_edt(cf->rfcode, edt);
-    proto_tree_free(protocol_tree);
     epan_dissect_free(edt);
   }   
   if (passed) {
@@ -1067,7 +1065,6 @@ print_packets(capture_file *cf, print_args_t *print_args)
   guint32     progbar_quantum;
   guint32     progbar_nextstep;
   guint32     count;
-  proto_tree *protocol_tree;
   gint       *col_widths = NULL;
   gint        data_width;
   gboolean    print_separator;
@@ -1136,10 +1133,6 @@ print_packets(capture_file *cf, print_args_t *print_args)
 
   print_separator = FALSE;
 
-  /* The protocol tree will be "visible", i.e., printed, only if we're
-     not printing a summary. */
-  proto_tree_is_visible = !print_args->print_summary;
-
   /* Update the progress bar when it gets to this value. */
   progbar_nextstep = 0;
   /* When we reach the value that triggers a progress bar update,
@@ -1181,20 +1174,19 @@ print_packets(capture_file *cf, print_args_t *print_args)
     }
 
     count++;
-
-    if (fdata->flags.passed_dfilter) {
+    /* Check to see if we are suppressing unmarked packets, if so, 
+     * suppress them and then proceed to check for visibility.
+     */
+    if (((print_args->suppress_unmarked && fdata->flags.marked ) ||
+        !(print_args->suppress_unmarked)) && fdata->flags.passed_dfilter) {
       wtap_seek_read (cf->wth, fdata->file_off, &cf->pseudo_header,
       			cf->pd, fdata->cap_len);
       if (print_args->print_summary) {
         /* Fill in the column information, but don't bother creating
            the logical protocol tree. */
-        fdata->cinfo = &cf->cinfo;
-        for (i = 0; i < fdata->cinfo->num_cols; i++) {
-          fdata->cinfo->col_buf[i][0] = '\0';
-          fdata->cinfo->col_data[i] = fdata->cinfo->col_buf[i];
-        }
-        edt = epan_dissect_new(&cf->pseudo_header, cf->pd, fdata, NULL);
-        fill_in_columns(fdata);
+        edt = epan_dissect_new(FALSE, FALSE);
+        epan_dissect_run(edt, &cf->pseudo_header, cf->pd, fdata, &cf->cinfo);
+        epan_dissect_fill_in_columns(edt);
         cp = &line_buf[0];
         line_len = 0;
         for (i = 0; i < cf->cinfo.num_cols; i++) {
@@ -1228,16 +1220,16 @@ print_packets(capture_file *cf, print_args_t *print_args)
         if (print_separator)
           print_line(cf->print_fh, print_args->format, "\n");
 
-        /* Create the logical protocol tree. */
-        protocol_tree = proto_tree_create_root();
-        edt = epan_dissect_new(&cf->pseudo_header, cf->pd, fdata, protocol_tree);
+        /* Create the logical protocol tree, complete with the display
+           representation of the items; we don't need the columns here,
+           however. */
+        edt = epan_dissect_new(TRUE, TRUE);
+        epan_dissect_run(edt, &cf->pseudo_header, cf->pd, fdata, NULL);
 
         /* Print the information in that tree. */
-        proto_tree_print(FALSE, print_args, (GNode *)protocol_tree,
+        proto_tree_print(FALSE, print_args, (GNode *)edt->tree,
 			fdata, cf->print_fh);
 
-        proto_tree_free(protocol_tree);
-
 	if (print_args->print_hex) {
 	  /* Print the full packet data as hex. */
 	  print_hex_data(cf->print_fh, print_args->format, fdata);
@@ -1264,8 +1256,6 @@ print_packets(capture_file *cf, print_args_t *print_args)
  
   cf->print_fh = NULL;
 
-  proto_tree_is_visible = FALSE;
-
   return TRUE;
 }
 
@@ -1342,8 +1332,7 @@ change_time_formats(capture_file *cf)
          the answer isn't going to change from packet to packet, so we should
          simply skip all the "change_time_formats()" work if we're not
          changing anything. */
-      fdata->cinfo = &cf->cinfo;
-      if (check_col(fdata, COL_CLS_TIME)) {
+      if (check_col(&cf->cinfo, COL_CLS_TIME)) {
         /* There are columns that show the time in the "command-line-specified"
            format; update them. */
         for (i = 0; i < cf->cinfo.num_cols; i++) {
@@ -1351,7 +1340,7 @@ change_time_formats(capture_file *cf)
             /* This is one of the columns that shows the time in
                "command-line-specified" format; update it. */
             cf->cinfo.col_buf[i][0] = '\0';
-            col_set_cls_time(fdata, i);
+            col_set_cls_time(fdata, &cf->cinfo, i);
             gtk_clist_set_text(GTK_CLIST(packet_list), row, i,
 			  cf->cinfo.col_data[i]);
 	  }
@@ -1388,7 +1377,6 @@ find_packet(capture_file *cf, dfilter_t *sfcode)
   guint32 progbar_quantum;
   guint32 progbar_nextstep;
   unsigned int count;
-  proto_tree *protocol_tree;
   gboolean frame_matched;
   int row;
   epan_dissect_t	*edt;
@@ -1453,13 +1441,13 @@ find_packet(capture_file *cf, dfilter_t *sfcode)
       /* Is this packet in the display? */
       if (fdata->flags.passed_dfilter) {
         /* Yes.  Does it match the search filter? */
-        protocol_tree = proto_tree_create_root();
         wtap_seek_read(cf->wth, fdata->file_off, &cf->pseudo_header,
         		cf->pd, fdata->cap_len);
-        edt = epan_dissect_new(&cf->pseudo_header, cf->pd, fdata, protocol_tree);
+        edt = epan_dissect_new(TRUE, FALSE);
+        epan_dissect_prime_dfilter(edt, sfcode);
+        epan_dissect_run(edt, &cf->pseudo_header, cf->pd, fdata, NULL);
         frame_matched = dfilter_apply_edt(sfcode, edt);
-        proto_tree_free(protocol_tree);
-	epan_dissect_free(edt);
+        epan_dissect_free(edt);
         if (frame_matched) {
           new_fd = fdata;
           break;	/* found it! */
@@ -1518,8 +1506,6 @@ void
 select_packet(capture_file *cf, int row)
 {
   frame_data *fdata;
-  tvbuff_t *bv_tvb;
-  int i;
 
   /* Get the frame data struct pointer for this frame */
   fdata = (frame_data *) gtk_clist_get_row_data(GTK_CLIST(packet_list), row);
@@ -1563,29 +1549,22 @@ select_packet(capture_file *cf, int row)
   			cf->pd, fdata->cap_len);
 
   /* Create the logical protocol tree. */
-  if (cf->protocol_tree)
-      proto_tree_free(cf->protocol_tree);
-  cf->protocol_tree = proto_tree_create_root();
-  proto_tree_is_visible = TRUE;
   if (cf->edt != NULL) {
     epan_dissect_free(cf->edt);
     cf->edt = NULL;
   }
-  cf->edt = epan_dissect_new(&cf->pseudo_header, cf->pd, cf->current_frame,
-		cf->protocol_tree);
-  proto_tree_is_visible = FALSE;
+  /* We don't need the columns here. */
+  cf->edt = epan_dissect_new(TRUE, TRUE);
+  epan_dissect_run(cf->edt, &cf->pseudo_header, cf->pd, cf->current_frame,
+          NULL);
 
-  /* Display the GUI protocol tree and hex dump. */
+  /* Display the GUI protocol tree and hex dump.
+     XXX - why does the protocol tree not show up if we call
+     "proto_tree_draw()" before calling "add_byte_views()"? */
   clear_tree_and_hex_views();
-
-  i = 0; 
-  while((bv_tvb = g_slist_nth_data ( cf->current_frame->data_src, i++))){
-  	add_byte_view( tvb_get_name( bv_tvb), tvb_get_ptr(bv_tvb, 0, -1), tvb_length(bv_tvb));
-  }
-
-  proto_tree_draw(cf->protocol_tree, tree_view);
-
-  set_notebook_page( byte_nb_ptr, 0);
+  add_byte_views(cf->current_frame, cf->edt->tree, tree_view,
+      byte_nb_ptr);
+  proto_tree_draw(cf->edt->tree, tree_view);
 
   /* A packet is selected. */
   set_menus_for_selected_packet(TRUE);
@@ -1595,11 +1574,7 @@ select_packet(capture_file *cf, int row)
 void
 unselect_packet(capture_file *cf)
 {
-  /* Destroy the protocol tree and epan_dissect_t for that packet. */
-  if (cf->protocol_tree != NULL) {
-    proto_tree_free(cf->protocol_tree);
-    cf->protocol_tree = NULL;
-  }
+  /* Destroy the epan_dissect_t for the unselected packet. */
   if (cf->edt != NULL) {
     epan_dissect_free(cf->edt);
     cf->edt = NULL;
@@ -1645,6 +1620,26 @@ unselect_field(void)
   set_menus_for_selected_tree_row(FALSE);
 }
 
+/*
+ * Mark a particular frame.
+ */
+void
+mark_frame(capture_file *cf, frame_data *frame)
+{
+  frame->flags.marked = TRUE;
+  cf->marked_count++;
+}
+
+/*
+ * Unmark a particular frame.
+ */
+void
+unmark_frame(capture_file *cf, frame_data *frame)
+{
+  frame->flags.marked = FALSE;
+  cf->marked_count--;
+}
+
 static void
 freeze_clist(capture_file *cf)
 {
@@ -1898,7 +1893,7 @@ file_open_error_message(int err, gboolean for_writing)
 
   case WTAP_ERR_SHORT_READ:
     errmsg = "The file \"%s\" appears to have been cut short"
-             " in the middle of a packet.";
+             " in the middle of a packet or other data.";
     break;
 
   case WTAP_ERR_SHORT_WRITE:
