@@ -1,7 +1,7 @@
 /* file.h
  * Definitions for file structures and routines
  *
- * $Id: file.h,v 1.84 2001/07/05 00:34:39 guy Exp $
+ * $Id: file.h,v 1.88 2001/12/06 04:25:07 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -53,7 +53,8 @@ typedef struct _capture_file {
   guint16      cd_t;      /* File type of capture file */
   int          lnk_t;     /* Link-layer type with which to save capture */
   guint32      vers;      /* Version.  For tcpdump minor is appended to major */
-  int          count;     /* Packet count */
+  int          count;     /* Total number of frames */
+  int          marked_count; /* Number of marked frames */
   gboolean     drops_known; /* TRUE if we know how many packets were dropped */
   guint32      drops;     /* Dropped packets */
   guint32      esec;      /* Elapsed seconds */
@@ -83,9 +84,14 @@ typedef struct _capture_file {
   frame_data  *last_displayed;  /* Last frame displayed */
   column_info  cinfo;    /* Column formatting information */
   frame_data  *current_frame;  /* Frame data for current frame */
-  proto_tree  *protocol_tree; /* Protocol tree for currently selected packet */
   epan_dissect_t *edt; /* Protocol dissection fo rcurrently selected packet */
   FILE        *print_fh;  /* File we're printing to */
+#ifdef HAVE_LIBPCAP
+  gint32       autostop_filesize; /* Maximum capture file size */
+  gint32       autostop_duration; /* Maximum capture duration */
+  gboolean     ringbuffer_on; /* Ringbuffer option */
+  guint32      ringbuffer_num_files; /* Number of ringbuffer files */
+#endif
 } capture_file;
 
 /* Return values from "read_cap_file()", "continue_tail_cap_file()",
@@ -124,6 +130,16 @@ void unselect_packet(capture_file *);
 
 void unselect_field(void);
 
+/*
+ * Mark a particular frame in a particular capture.
+ */
+void mark_frame(capture_file *, frame_data *);
+
+/*
+ * Unmark a particular frame in a particular capture.
+ */
+void unmark_frame(capture_file *, frame_data *);
+
 /* Moves or copies a file. Returns 0 on failure, 1 on success */
 int file_mv(char *from, char *to);
 
