@@ -1,12 +1,11 @@
 /* filters.c
  * Code for reading and writing the filters file.
  *
- * $Id: filters.c,v 1.9 2001/04/02 09:53:42 guy Exp $
+ * $Id: filters.c,v 1.13 2001/10/24 07:18:36 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,21 +31,12 @@
 #include <ctype.h>
 #include <errno.h>
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_DIRECT_H
-#include <direct.h>		/* to declare "mkdir()" on Windows */
-#endif
-
 #include <glib.h>
 
-#include <epan.h>
 #include <filesystem.h>
 
 #include "filters.h"
@@ -91,7 +81,7 @@ void
 read_filter_list(filter_list_type_t list, char **pref_path_return,
     int *errno_return)
 {
-  char       *ff_path, *ff_dir = PF_DIR, *ff_name;
+  char       *ff_path, *ff_name;
   FILE       *ff;
   GList      **flp;
   GList      *fl_ent;
@@ -122,10 +112,7 @@ read_filter_list(filter_list_type_t list, char **pref_path_return,
   }
 
   /* To do: generalize this */
-  ff_path = (gchar *) g_malloc(strlen(get_home_dir()) + strlen(ff_dir) +  
-    strlen(ff_name) + 4);
-  sprintf(ff_path, "%s/%s/%s", get_home_dir(), ff_dir, ff_name);
-
+  ff_path = get_persconffile_path(ff_name, FALSE);
   if ((ff = fopen(ff_path, "r")) == NULL) {
     /*
      * Did that fail because we the file didn't exist?
@@ -147,7 +134,8 @@ read_filter_list(filter_list_type_t list, char **pref_path_return,
      * the filter lists, and delete the ones that don't belong in
      * a particular list.
      */
-    sprintf(ff_path, "%s/%s/%s", get_home_dir(), ff_dir, FILTER_FILE_NAME);
+    g_free(ff_path);
+    ff_path = get_persconffile_path(FILTER_FILE_NAME, FALSE);
     if ((ff = fopen(ff_path, "r")) == NULL) {
       /*
        * Well, that didn't work, either.  Just give up.
@@ -418,13 +406,11 @@ void
 save_filter_list(filter_list_type_t list, char **pref_path_return,
     int *errno_return)
 {
-  gchar      *ff_path, *ff_path_new, *ff_dir = PF_DIR, *ff_name;
-  int         path_length;
+  gchar      *ff_path, *ff_path_new, *ff_name;
   GList      *fl;
   GList      *flp;
   filter_def *filt;
   FILE       *ff;
-  struct stat s_buf;
   guchar     *p, c;
   
   *pref_path_return = NULL;	/* assume no error */
@@ -446,25 +432,13 @@ save_filter_list(filter_list_type_t list, char **pref_path_return,
     return;
   }
 
-  path_length = strlen(get_home_dir()) + strlen(ff_dir) + strlen(ff_name)
-  		+ 4 + 4;
-  ff_path = (gchar *) g_malloc(path_length);
-  sprintf(ff_path, "%s/%s", get_home_dir(), ff_dir);
-
-  if (stat(ff_path, &s_buf) != 0)
-#ifdef WIN32
-    mkdir(ff_path);
-#else
-    mkdir(ff_path, 0755);
-#endif
-    
-  sprintf(ff_path, "%s/%s/%s", get_home_dir(), ff_dir, ff_name);
+  ff_path = get_persconffile_path(ff_name, TRUE);
 
   /* Write to "XXX.new", and rename if that succeeds.
      That means we don't trash the file if we fail to write it out
      completely. */
-  ff_path_new = (gchar *) g_malloc(path_length);
-  sprintf(ff_path_new, "%s/%s/%s.new", get_home_dir(), ff_dir, ff_name);
+  ff_path_new = (gchar *) g_malloc(strlen(ff_path) + 5);
+  sprintf(ff_path_new, "%s.new", ff_path);
 
   if ((ff = fopen(ff_path_new, "w")) == NULL) {
     *pref_path_return = ff_path;
