@@ -1,12 +1,11 @@
 /* capture_dlg.c
  * Routines for packet capture windows
  *
- * $Id: capture_dlg.c,v 1.43 2001/05/31 08:36:45 guy Exp $
+ * $Id: capture_dlg.c,v 1.52 2001/12/09 03:20:19 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +22,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -56,8 +54,9 @@
 #include "filter_prefs.h"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "util.h"
+#include "pcap-util.h"
 #include "prefs.h"
+#include "ringbuffer.h"
 
 #ifdef _WIN32
 #include "capture-wpcap.h"
@@ -75,6 +74,11 @@
 #define E_CAP_M_RESOLVE_KEY   "cap_m_resolve"
 #define E_CAP_N_RESOLVE_KEY   "cap_n_resolve"
 #define E_CAP_T_RESOLVE_KEY   "cap_t_resolve"
+#define E_CAP_FILESIZE_KEY    "cap_filesize"
+#define E_CAP_DURATION_KEY    "cap_duration"
+#define E_CAP_RING_ON_TB_KEY  "cap_ringbuffer_on_tb"
+#define E_CAP_RING_NBF_LB_KEY "cap_ringbuffer_nbf_lb"
+#define E_CAP_RING_NBF_SB_KEY "cap_ringbuffer_nbf_sb"
 
 #define E_FS_CALLER_PTR_KEY       "fs_caller_ptr"
 #define E_FILE_SEL_DIALOG_PTR_KEY "file_sel_dialog_ptr"
@@ -91,6 +95,9 @@ cap_prep_fs_cancel_cb(GtkWidget *w, gpointer data);
 static void
 cap_prep_fs_destroy_cb(GtkWidget *win, gpointer data);
 
+static void
+capture_prep_adjust_sensitivity(GtkWidget *tb, gpointer parent_w);
+
 static void
 capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w);
 
@@ -121,14 +128,19 @@ capture_prep_cb(GtkWidget *w, gpointer d)
                 *count_lb, *count_cb, *main_vb,
                 *filter_bt, *filter_te,
                 *file_bt, *file_te,
+                *filesize_lb, *filesize_cb,
+                *duration_lb, *duration_cb,
                 *caplen_hb, *table,
                 *bbox, *ok_bt, *cancel_bt, *snap_lb,
                 *snap_sb, *promisc_cb, *sync_cb, *auto_scroll_cb,
-		*m_resolv_cb, *n_resolv_cb, *t_resolv_cb;
+                *m_resolv_cb, *n_resolv_cb, *t_resolv_cb,
+                *ringbuffer_hb, *ringbuffer_on_tb, *ringbuffer_nbf_lb, *ringbuffer_nbf_sb;
   GtkAccelGroup *accel_group;
-  GtkAdjustment *adj;
-  GList         *if_list, *count_list = NULL;
-  gchar         *count_item1 = "0 (Infinite)", count_item2[16];
+  GtkAdjustment *snap_adj, *ringbuffer_nbf_adj;
+  GList         *if_list, *count_list = NULL, *filesize_list = NULL, *duration_list = NULL;
+  gchar         *count_item1 = "0 (Infinite)", count_item2[16],
+                *filesize_item1 = "0 (Infinite)", filesize_item2[16],
+                *duration_item1 = "0 (Infinite)", duration_item2[16];
   int           err;
   char          err_str[PCAP_ERRBUF_SIZE];
 
@@ -142,13 +154,25 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   /* Is WPcap loaded? */
   if (!has_wpcap) {
 	  simple_dialog(ESD_TYPE_CRIT, NULL,
-			  "Unable to load WinPcap (wpcap.dll). Capturing not enabled.\n"
-			  "See http://netgroup-serv.polito.it/winpcap/ for\n"
-			  "more details on WinPcap.");
+		  "Unable to load WinPcap (wpcap.dll); Ethereal will not be able\n"
+		  "to capture packets.\n\n"
+		  "In order to capture packets, WinPcap must be installed; see\n"
+		  "\n"
+		  "        http://netgroup-serv.polito.it/winpcap/\n"
+		  "\n"
+		  "or the mirror at\n"
+		  "\n"
+		  "        http://netgroup-mirror.ethereal.com/winpcap/\n"
+		  "\n"
+		  "or the mirror at\n"
+		  "\n"
+		  "        http://www.wiretapped.net/security/packet-capture/winpcap/default.htm\n"
+		  "\n"
+		  "for a downloadable version of WinPcap and for instructions\n"
+		  "on how to install WinPcap.");
 	  return;
   }
 #endif
-	  
 
   if_list = get_interface_list(&err, err_str);
   if (if_list == NULL && err == CANT_GET_INTERFACE_LIST) {
@@ -171,8 +195,8 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   gtk_container_add(GTK_CONTAINER(cap_open_w), main_vb);
   gtk_widget_show(main_vb);
   
-  /* Table : container of the first 4 rows */
-  table = gtk_table_new (4, 2, FALSE);
+  /* Table : container of the first 6 rows */
+  table = gtk_table_new (6, 2, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE (table), 5);
   gtk_table_set_col_spacings(GTK_TABLE (table), 5);
   gtk_container_add(GTK_CONTAINER(main_vb), table);
@@ -216,34 +240,74 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   while (count_list)
     count_list = g_list_remove_link(count_list, count_list);
 
+  /* Filesize row */
+  
+  filesize_lb = gtk_label_new("File size:");
+  gtk_table_attach_defaults(GTK_TABLE(table), filesize_lb, 0, 1, 2, 3);
+  gtk_widget_show(filesize_lb);
+  
+  filesize_list = g_list_append(filesize_list, filesize_item1);
+  if (cfile.autostop_filesize) {
+    snprintf(filesize_item2, 15, "%d", cfile.autostop_filesize);
+    filesize_list = g_list_append(filesize_list, filesize_item2);
+  }
+
+  filesize_cb = gtk_combo_new();
+  gtk_combo_set_popdown_strings(GTK_COMBO(filesize_cb), filesize_list);
+  gtk_table_attach_defaults(GTK_TABLE(table), filesize_cb, 1, 2, 2, 3);
+  gtk_widget_show(filesize_cb);
+
+  while (filesize_list)
+    filesize_list = g_list_remove_link(filesize_list, filesize_list);
+
+  /* Duration row */
+  
+  duration_lb = gtk_label_new("Duration:");
+  gtk_table_attach_defaults(GTK_TABLE(table), duration_lb, 0, 1, 3, 4);
+  gtk_widget_show(duration_lb);
+  
+  duration_list = g_list_append(duration_list, duration_item1);
+  if (cfile.autostop_duration) {
+    snprintf(duration_item2, 15, "%d", cfile.autostop_duration);
+    duration_list = g_list_append(duration_list, duration_item2);
+  }
+
+  duration_cb = gtk_combo_new();
+  gtk_combo_set_popdown_strings(GTK_COMBO(duration_cb), duration_list);
+  gtk_table_attach_defaults(GTK_TABLE(table), duration_cb, 1, 2, 3, 4);
+  gtk_widget_show(duration_cb);
+
+  while (duration_list)
+    duration_list = g_list_remove_link(duration_list, duration_list);
+
   /* Filter row */
   
   filter_bt = gtk_button_new_with_label("Filter:");
   gtk_signal_connect(GTK_OBJECT(filter_bt), "clicked",
     GTK_SIGNAL_FUNC(capture_filter_construct_cb), NULL);
-  gtk_table_attach_defaults(GTK_TABLE(table), filter_bt, 0, 1, 2, 3);
+  gtk_table_attach_defaults(GTK_TABLE(table), filter_bt, 0, 1, 4, 5);
   gtk_widget_show(filter_bt);
   
   filter_te = gtk_entry_new();
   if (cfile.cfilter) gtk_entry_set_text(GTK_ENTRY(filter_te), cfile.cfilter);
   gtk_object_set_data(GTK_OBJECT(filter_bt), E_FILT_TE_PTR_KEY, filter_te);
-  gtk_table_attach_defaults(GTK_TABLE(table), filter_te, 1, 2, 2, 3);
+  gtk_table_attach_defaults(GTK_TABLE(table), filter_te, 1, 2, 4, 5);
   gtk_widget_show(filter_te);
   
   /* File row */
   
   file_bt = gtk_button_new_with_label("File:");
-  gtk_table_attach_defaults(GTK_TABLE(table), file_bt, 0, 1, 3, 4);
+  gtk_table_attach_defaults(GTK_TABLE(table), file_bt, 0, 1, 5, 6);
   gtk_widget_show(file_bt);
   
   file_te = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), file_te, 1, 2, 3, 4);
+  gtk_table_attach_defaults(GTK_TABLE(table), file_te, 1, 2, 5, 6);
   gtk_widget_show(file_te);
 
   gtk_signal_connect(GTK_OBJECT(file_bt), "clicked",
     GTK_SIGNAL_FUNC(capture_prep_file_cb), GTK_OBJECT(file_te));
 
-  /* Misc row: Capture file checkbox and snap spinbutton */
+  /* Misc row: Snap spinbutton */
   caplen_hb = gtk_hbox_new(FALSE, 3);
   gtk_container_add(GTK_CONTAINER(main_vb), caplen_hb);
   gtk_widget_show(caplen_hb);
@@ -253,9 +317,9 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   gtk_box_pack_start(GTK_BOX(caplen_hb), snap_lb, FALSE, FALSE, 6);
   gtk_widget_show(snap_lb);
 
-  adj = (GtkAdjustment *) gtk_adjustment_new((float) cfile.snap,
+  snap_adj = (GtkAdjustment *) gtk_adjustment_new((float) cfile.snap,
     MIN_PACKET_SIZE, WTAP_MAX_PACKET_SIZE, 1.0, 10.0, 0.0);
-  snap_sb = gtk_spin_button_new (adj, 0, 0);
+  snap_sb = gtk_spin_button_new (snap_adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (snap_sb), TRUE);
   gtk_widget_set_usize (snap_sb, 80, 0);
   gtk_box_pack_start (GTK_BOX(caplen_hb), snap_sb, FALSE, FALSE, 3); 
@@ -267,9 +331,43 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   gtk_container_add(GTK_CONTAINER(main_vb), promisc_cb);
   gtk_widget_show(promisc_cb);
 
+  /* Misc row: Ring buffer toggle button and Ring buffer spinbutton */
+  ringbuffer_hb = gtk_hbox_new(FALSE, 3);
+  gtk_container_add(GTK_CONTAINER(main_vb), ringbuffer_hb);
+  gtk_widget_show(ringbuffer_hb);
+
+  ringbuffer_on_tb = dlg_check_button_new_with_label_with_mnemonic(
+    "Use _ring buffer", accel_group);
+  /* Ring buffer mode is allowed only if we're not doing an "Update list of
+     packets in real time" capture, so force it off if we're doing such
+     a capture. */
+  if (prefs.capture_real_time)
+    cfile.ringbuffer_on = FALSE;
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ringbuffer_on_tb),cfile.ringbuffer_on);
+  gtk_signal_connect(GTK_OBJECT(ringbuffer_on_tb), "toggled",
+    GTK_SIGNAL_FUNC(capture_prep_adjust_sensitivity), GTK_OBJECT(cap_open_w));
+  gtk_box_pack_start(GTK_BOX(ringbuffer_hb), ringbuffer_on_tb, FALSE, FALSE, 0);
+  gtk_widget_show(ringbuffer_on_tb);
+  
+  ringbuffer_nbf_lb = gtk_label_new("Number of files");
+  gtk_misc_set_alignment(GTK_MISC(ringbuffer_nbf_lb), 1, 0.5);
+  gtk_box_pack_start(GTK_BOX(ringbuffer_hb), ringbuffer_nbf_lb, FALSE, FALSE, 6);
+  gtk_widget_show(ringbuffer_nbf_lb);
+
+  ringbuffer_nbf_adj = (GtkAdjustment *) gtk_adjustment_new((float) cfile.ringbuffer_num_files,
+    RINGBUFFER_MIN_NUM_FILES, RINGBUFFER_MAX_NUM_FILES, 1.0, 10.0, 0.0);
+  ringbuffer_nbf_sb = gtk_spin_button_new (ringbuffer_nbf_adj, 0, 0);
+  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (ringbuffer_nbf_sb), TRUE);
+  gtk_widget_set_usize (ringbuffer_nbf_sb, 40, 0);
+  gtk_box_pack_start (GTK_BOX(ringbuffer_hb), ringbuffer_nbf_sb, TRUE, TRUE, 0); 
+  gtk_widget_show(ringbuffer_nbf_sb);
+  
+  /* Misc row: Capture file checkboxes */
   sync_cb = dlg_check_button_new_with_label_with_mnemonic(
 		"_Update list of packets in real time", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sync_cb), prefs.capture_real_time);
+  gtk_signal_connect(GTK_OBJECT(sync_cb), "toggled",
+    GTK_SIGNAL_FUNC(capture_prep_adjust_sensitivity), GTK_OBJECT(cap_open_w));
   gtk_container_add(GTK_CONTAINER(main_vb), sync_cb);
   gtk_widget_show(sync_cb);
 
@@ -334,6 +432,15 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_M_RESOLVE_KEY,  m_resolv_cb);
   gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_N_RESOLVE_KEY,  n_resolv_cb);
   gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_T_RESOLVE_KEY,  t_resolv_cb);
+  gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_FILESIZE_KEY,  filesize_cb);
+  gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_DURATION_KEY,  duration_cb);
+  gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_RING_ON_TB_KEY,  ringbuffer_on_tb);
+  gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_RING_NBF_LB_KEY,  ringbuffer_nbf_lb);
+  gtk_object_set_data(GTK_OBJECT(cap_open_w), E_CAP_RING_NBF_SB_KEY,  ringbuffer_nbf_sb);
+
+  /* Set the sensitivity of various widgets as per the settings of other
+     widgets. */
+  capture_prep_adjust_sensitivity(NULL, cap_open_w);
 
   /* Catch the "activate" signal on the frame number and file name text
      entries, so that if the user types Return there, we act as if the
@@ -452,14 +559,45 @@ cap_prep_fs_destroy_cb(GtkWidget *win, gpointer data)
   gtk_widget_destroy(GTK_WIDGET(win));
 }
 
+static int
+get_positive_int(const char *string, const char *name)
+{
+  long number;
+  char *p;
+
+  number = strtol(string, &p, 10);
+  /*
+   * XXX - we allow extra stuff after 0, so that we don't have
+   * problems with the "(Infinite)" value.
+   */
+  if (p == string || (*p != '\0' && number != 0)) {
+    simple_dialog(ESD_TYPE_CRIT, NULL,
+        "The specified %s is not a decimal number.", name);
+    return -1;
+  }
+  if (number < 0) {
+    simple_dialog(ESD_TYPE_CRIT, NULL,
+        "The specified %s is a negative number.", name);
+    return -1;
+  }
+  if (number > INT_MAX) {
+    simple_dialog(ESD_TYPE_CRIT, NULL,
+        "The specified %s is too large (greater than %d).", name, INT_MAX);
+    return -1;
+  }
+  return number;
+}
+
 static void
 capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
   GtkWidget *if_cb, *filter_te, *file_te, *count_cb, *snap_sb, *promisc_cb,
-            *sync_cb, *auto_scroll_cb, *m_resolv_cb, *n_resolv_cb, *t_resolv_cb;
+            *sync_cb, *auto_scroll_cb, *m_resolv_cb, *n_resolv_cb, *t_resolv_cb,
+            *filesize_cb, *duration_cb, *ringbuffer_on_tb, *ringbuffer_nbf_sb;
   gchar *if_text;
   gchar *if_name;
   gchar *filter_text;
   gchar *save_file;
+  int value;
 
   if_cb     = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_IFACE_KEY);
   filter_te = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_FILT_KEY);
@@ -472,6 +610,10 @@ capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
   m_resolv_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_M_RESOLVE_KEY);
   n_resolv_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_N_RESOLVE_KEY);
   t_resolv_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_T_RESOLVE_KEY);
+  filesize_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_FILESIZE_KEY);
+  duration_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_DURATION_KEY);
+  ringbuffer_on_tb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_RING_ON_TB_KEY);
+  ringbuffer_nbf_sb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_RING_NBF_SB_KEY);
 
   if_text =
     g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry)));
@@ -510,7 +652,23 @@ capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
     save_file = NULL;
   }
 
-  cfile.count = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(count_cb)->entry)));
+  value = get_positive_int(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(count_cb)->entry)),
+      "maximum packet count");
+  if (value == -1)
+    return;	/* error */
+  cfile.count = value;
+
+  value = get_positive_int(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(filesize_cb)->entry)),
+      "maximum capture file size");
+  if (value == -1)
+    return;	/* error */
+  cfile.autostop_filesize = value;
+
+  value = get_positive_int(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(duration_cb)->entry)),
+      "capture duration");
+  if (value == -1)
+    return;	/* error */
+  cfile.autostop_duration = value;
 
   cfile.snap = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(snap_sb));
   if (cfile.snap < 1)
@@ -529,6 +687,25 @@ capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
   prefs.name_resolve |= (GTK_TOGGLE_BUTTON (n_resolv_cb)->active ? PREFS_RESOLV_NETWORK : PREFS_RESOLV_NONE);
   prefs.name_resolve |= (GTK_TOGGLE_BUTTON (t_resolv_cb)->active ? PREFS_RESOLV_TRANSPORT : PREFS_RESOLV_NONE);
 
+  cfile.ringbuffer_on = GTK_TOGGLE_BUTTON (ringbuffer_on_tb)->active && !(prefs.capture_real_time);
+  if (cfile.ringbuffer_on == TRUE) {
+    if (save_file == NULL) {
+      simple_dialog(ESD_TYPE_CRIT, NULL,
+        "You must specify a save file if you want to use the ring buffer.");
+      return;
+    } else if (cfile.autostop_filesize == 0) {
+      simple_dialog(ESD_TYPE_CRIT, NULL,
+        "You must specify a maximum save file size other \nthan 0 (infinite) if you want to use the ring buffer.");
+      return;
+    }
+  }
+
+  cfile.ringbuffer_num_files = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ringbuffer_nbf_sb));
+  if (cfile.ringbuffer_num_files < RINGBUFFER_MIN_NUM_FILES)
+    cfile.ringbuffer_num_files = RINGBUFFER_MIN_NUM_FILES;
+  else if (cfile.ringbuffer_num_files > RINGBUFFER_MAX_NUM_FILES)
+    cfile.ringbuffer_num_files = RINGBUFFER_MAX_NUM_FILES;
+
   gtk_widget_destroy(GTK_WIDGET(parent_w));
 
   do_capture(save_file);
@@ -569,4 +746,58 @@ capture_prep_destroy_cb(GtkWidget *win, gpointer user_data)
   cap_open_w = NULL;
 }
 
+/*
+ * Adjust the sensitivity of various widgets as per the current setting
+ * of other widgets.
+ */
+static void
+capture_prep_adjust_sensitivity(GtkWidget *tb, gpointer parent_w)
+{
+  GtkWidget *sync_cb, *auto_scroll_cb, *ringbuffer_on_tb;
+  GtkWidget *ringbuffer_nbf_lb, *ringbuffer_nbf_sb;
+
+  sync_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_SYNC_KEY);
+  auto_scroll_cb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_AUTO_SCROLL_KEY);
+  ringbuffer_on_tb  = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_RING_ON_TB_KEY);
+  ringbuffer_nbf_lb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_RING_NBF_LB_KEY);
+  ringbuffer_nbf_sb = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w), E_CAP_RING_NBF_SB_KEY);
+
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sync_cb))) {
+    /* "Update list of packets in real time" captures enabled; we don't
+       support ring buffer mode for those captures, so turn ring buffer
+       mode off if it's on, and make its toggle button, and the spin
+       button for the number of ring buffer files (and the spin button's
+       label), insensitive. */
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ringbuffer_on_tb))) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ringbuffer_on_tb), FALSE);
+    }
+    gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_on_tb), FALSE);
+
+    /* Auto-scroll mode is meaningful only in "Update list of packets
+       in real time" captures, so make its toggle button sensitive. */
+    gtk_widget_set_sensitive(GTK_WIDGET(auto_scroll_cb), TRUE);
+  } else {
+    /* "Update list of packets in real time" captures disabled; that
+       means ring buffer mode is OK, so make its toggle button
+       sensitive. */
+    gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_on_tb), TRUE);
+
+    /* Auto-scroll mode is meaningful only in "Update list of packets
+       in real time" captures, so make its toggle button insensitive. */
+    gtk_widget_set_sensitive(GTK_WIDGET(auto_scroll_cb), FALSE);
+  }
+  
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ringbuffer_on_tb))) {
+    /* Ring buffer mode enabled.  Make the spin button for the number
+       of ring buffer files, and its label, sensitive. */
+    gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_nbf_lb), TRUE);
+    gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_nbf_sb), TRUE);
+  } else {
+    /* Ring buffer mode disabled.  Make the spin button for the number
+       of ring buffer files, and its label insensitive. */
+    gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_nbf_lb), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_nbf_sb), FALSE);
+  }
+}
+
 #endif /* HAVE_LIBPCAP */
