@@ -1,12 +1,11 @@
 /* colors.c
  * Definitions for color structures and routines
  *
- * $Id: colors.c,v 1.9 2001/04/24 00:28:21 guy Exp $
+ * $Id: colors.c,v 1.17 2001/12/18 19:09:07 gram Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +21,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
-
+/*
+ * Updated 1 Dec 10 jjm
+ */
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -39,8 +40,8 @@
 #include <sys/types.h>
 #endif
 
-#include <epan.h>
 #include <epan/filesystem.h>
+
 #include "gtk/main.h"
 #include "packet.h"
 #include "colors.h"
@@ -60,31 +61,14 @@ static GdkColormap*	our_cmap = NULL;
 GdkColor	WHITE = { 0, 65535, 65535, 65535 };
 GdkColor	BLACK = { 0, 0, 0, 0 };
 
-/* This structure is used to allow you to compile in default colors if
- * you wish.  They can be later changed by a user.
- */
-#ifdef READ_DEFAULT_COLOR_LIST
-struct _default_colors {
-	gchar* proto;
-	gchar* color; /* background only */
-} default_colors[]  = {
-	{"arp",	"green2"},
-	{"ip",	"light red"},
-	{"tcp",  "light blue"}
-};
-#endif
-
+/* Initialize the filter structures (reading from file) */
 colfilter *
 colfilter_new(void)
 {
   colfilter *filter;
   gboolean got_white, got_black;
-#ifdef READ_DEFAULT_COLOR_LIST
-  color_filter_t *colorf;
-  gint i;
-  GdkColor color;
-#endif
 
+  /* Create the filter header */
   filter = (colfilter *)g_malloc(sizeof(colfilter));
   filter->num_of_filters = 0;
 
@@ -105,37 +89,15 @@ colfilter_new(void)
       simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color black.");
   }
 
-#ifdef READ_DEFAULT_COLOR_LIST
-  /* Now process defaults */
-  for (i = 0 ; i < sizeof default_colors/sizeof (struct _default_colors); i++){
-	gdk_color_parse(default_colors[i].color, &color);
-	
-	if( !get_color(&color)){
-		/* oops */
-		simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color %s.",
-		    default_colors[i].color);
-	}
-
-	colorf = new_color_filter(filter, default_colors[i].proto,
-	    default_colors[i].proto);
-	colorf->bg_color = color;
-
-	if (!dfilter_compile(default_colors[i].proto,
-	    &colorf->c_colorfilter)) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
-		  "Cannot compile default color filter %s.\n%s",
-		  default_colors[i].proto, dfilter_error_msg);
-		/* should reject this filter */
-	}
-	filter->num_of_filters++;
-  }
-#endif
   read_filters(filter);
   return filter;
 }
 
+/* Create a new filter */
 color_filter_t *
-new_color_filter(colfilter *filters, gchar *name, gchar *filter_string)
+new_color_filter(colfilter *filters,    /* The filter list (unused) */
+                 gchar *name,           /* The name of the filter to create */
+                 gchar *filter_string)  /* The string representing the filter */
 {
 	color_filter_t *colorf;
 
@@ -151,6 +113,8 @@ new_color_filter(colfilter *filters, gchar *name, gchar *filter_string)
 }
 
 void
+
+/* delete the specified filter */
 delete_color_filter(color_filter_t *colorf)
 {
 	if (colorf->filter_name != NULL)
@@ -163,67 +127,84 @@ delete_color_filter(color_filter_t *colorf)
 	g_free(colorf);
 }
 
+
+static void
+prime_edt(gpointer data, gpointer user_data)
+{
+    color_filter_t  *colorf = data;
+    epan_dissect_t   *edt = user_data;
+
+    epan_dissect_prime_dfilter(edt, colorf->c_colorfilter);
+} 
+
+/* Prime the epan_dissect_t with all the compiler
+ * color filters in 'filter_list'. */
+void
+filter_list_prime_edt(epan_dissect_t *edt)
+{
+    g_slist_foreach(filter_list, prime_edt, edt);
+}
+
+
+/* read filters from the file */
 static gboolean
 read_filters(colfilter *filter)
 {
 	/* TODO: Lots more syntax checking on the file */
 	/* I hate these fixed length names! TODO: make more dynamic */
-	/* XXX - buffer overflow possibility here */
+	/* XXX - buffer overflow possibility here
+   * sscanf blocks max size of name and filter_exp; buf is used for reading only */
 	gchar name[256],filter_exp[256], buf[1024];
 	guint16 fg_r, fg_g, fg_b, bg_r, bg_g, bg_b;
 	GdkColor fg_color, bg_color;
 	color_filter_t *colorf;
 	int i;
+	const gchar *path;
 	FILE *f;
-	gchar *path;
-	gchar *fname = PF_DIR "/colorfilters";
 	dfilter_t *temp_dfilter;
 
 	/* decide what file to open (from dfilter code) */
 
-	/* should only be called by colors_init.
-	 */
+	/* should only be called by colors_init */
 	if(filter == NULL)
 		return FALSE;
 	/* we have a clist */
 
-	path = (gchar *) g_malloc(strlen(get_home_dir()) + strlen(fname) +  4);
-	sprintf(path, "%s/%s", get_home_dir(), fname);
-
+	path = get_persconffile_path("colorfilters", FALSE);
 	if ((f = fopen(path, "r")) == NULL) {
 	  if (errno != ENOENT) {
 	    simple_dialog(ESD_TYPE_CRIT, NULL,
 	      "Could not open filter file\n\"%s\": %s.", path,
 	      strerror(errno));
 	  }
-	  g_free(path);
 	  return FALSE;
 	}
-	g_free(path);
 
 	i = 0;
 
 	do{
 	  if(!fgets(buf,sizeof buf, f))
-		break;
+      break;
 		
 	  if(strspn( buf," \t") == (size_t)((strchr(buf,'*') - buf))){
-		/* leading # comment */
-		continue;
+		  /* leading # comment */
+		  continue;
 	  }
 
-	  /* we get the @ delimiter.  It is not in any strings */
-	  if(sscanf(buf," @%[^@]@%[^@]@[%hu,%hu,%hu][%hu,%hu,%hu]",
-		name, filter_exp, &bg_r, &bg_g, &bg_b, &fg_r, &fg_g, &fg_b) == 8){
-		/* we got a filter */
+	  /* we get the @ delimiter.  It is not in any strings
+       * Format is @name@filter expression@[background r,g,b][foreground r,g,b]
+       */
+	  if(sscanf(buf," @%256[^@]@%256[^@]@[%hu,%hu,%hu][%hu,%hu,%hu]",
+      name, filter_exp, &bg_r, &bg_g, &bg_b, &fg_r, &fg_g, &fg_b) == 8){
+      /* we got a filter */
 
 	    if(!dfilter_compile(filter_exp, &temp_dfilter)) {
-		simple_dialog(ESD_TYPE_CRIT, NULL,
-		 "Could not compile color filter %s from saved filters.\n%s",
-		 name, dfilter_error_msg);
-		continue;
+        simple_dialog(ESD_TYPE_CRIT, NULL,
+		    "Could not compile color filter %s from saved filters.\n%s",
+		    name, dfilter_error_msg);
+		    continue;
 	    }
-            colorf = new_color_filter(filter, name, filter_exp);
+      colorf = new_color_filter(filter, name, filter_exp);
 	    colorf->c_colorfilter = temp_dfilter;
 	    filter->num_of_filters++;
 	    fg_color.red = fg_r;
@@ -233,24 +214,23 @@ read_filters(colfilter *filter)
 	    bg_color.green = bg_g;
 	    bg_color.blue = bg_b;
 	    if( !get_color(&fg_color)){
-		/* oops */
-		simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate fg color specified"
-		  "in input file for %s.", name);
-
-		i++;
-		continue;
+		    /* oops */
+		    simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate fg color specified"
+		    "in input file for %s.", name);
+		    i++;
+		    continue;
 	    }
 	    if( !get_color(&bg_color)){
-		/* oops */
-		simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate bg color specified"
-		  "in input file for %s.", name);
-		i++;
-		continue;
+		    /* oops */
+		    simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate bg color specified"
+		    "in input file for %s.", name);
+		    i++;
+		    continue;
 	    }
 
-            colorf->bg_color = bg_color;
-            colorf->fg_color = fg_color;
-	    i++;
+        colorf->bg_color = bg_color;
+        colorf->fg_color = fg_color;
+	      i++;
 	  }    /* if sscanf */
 	} while( !feof(f));
 	return TRUE;
@@ -272,43 +252,51 @@ write_filter(gpointer filter_arg, gpointer file_arg)
 	    colorf->fg_color.green,
 	    colorf->fg_color.blue);
 }
-	
+
+/* save filters in filter file */
 gboolean
 write_filters(colfilter *filter)
 {
+	gchar *pf_dir_path;
+	const gchar *path;
 	FILE *f;
-	gchar *path;
-	gchar *name = PF_DIR "/colorfilters";
-	/* decide what file to open (from dfilter code) */
-	path = (gchar *) g_malloc(strlen(get_home_dir()) + strlen(name) +  4);
-	sprintf(path, "%s/%s", get_home_dir(), name);
 
+	/* Create the directory that holds personal configuration files,
+	   if necessary.  */
+	if (create_persconffile_dir(&pf_dir_path) == -1) {
+	  simple_dialog(ESD_TYPE_WARN, NULL,
+		"Can't create directory\n\"%s\"\nfor color files: %s.",
+		pf_dir_path, strerror(errno));
+	  g_free(pf_dir_path);
+	  return FALSE;
+	}
+
+	path = get_persconffile_path("colorfilters", TRUE);
 	if ((f = fopen(path, "w+")) == NULL) {
 	  simple_dialog(ESD_TYPE_CRIT, NULL,
 		"Could not open\n%s\nfor writing: %s.",
 		path, strerror(errno));
-	  g_free(path);
 	  return FALSE;
 	}
         fprintf(f,"# DO NOT EDIT THIS FILE!  It was created by Ethereal\n");
         g_slist_foreach(filter_list, write_filter, f);
 	fclose(f);
-	g_free(path);
 	return TRUE;
 }
 
+/* allocate a color from the color map */
 gboolean
 get_color (GdkColor *new_color)
 {
     GdkVisual *pv;
 
     if (!our_cmap) {
-	if ( !gdk_colormap_alloc_color (sys_cmap, new_color, FALSE, TRUE)) {
-	    pv = gdk_visual_get_best();
-	    if ( !(our_cmap = gdk_colormap_new(pv, TRUE)))
-		simple_dialog(ESD_TYPE_WARN, NULL, "Could not create new colormap");
-	} else
-	    return (TRUE);
+      if ( !gdk_colormap_alloc_color (sys_cmap, new_color, FALSE, TRUE)) {
+        pv = gdk_visual_get_best();
+	      if ( !(our_cmap = gdk_colormap_new(pv, TRUE)))
+		      simple_dialog(ESD_TYPE_WARN, NULL, "Could not create new colormap");
+      } else
+	      return (TRUE);
     }
     return ( gdk_colormap_alloc_color ( our_cmap, new_color, FALSE, TRUE) );
 }
