@@ -1,6 +1,6 @@
 /* decode_as_dlg.c
  *
- * $Id: decode_as_dlg.c,v 1.9 2001/06/25 08:10:38 guy Exp $
+ * $Id: decode_as_dlg.c,v 1.21 2001/12/18 19:09:07 gram Exp $
  *
  * Routines to modify dissector tables on the fly.
  *
@@ -20,7 +20,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,6 +45,7 @@
 #include "packet.h"
 #include "ipproto.h"
 #include "ui_util.h"
+#include "epan_dissect.h"
 
 #undef DEBUG
 
@@ -65,22 +65,6 @@ enum action_type {
     E_DECODE_NO
 };
 
-/*
- * Enum used to track which transport layer protocol menu item is
- * currently selected in the dialog.  These items are labeled "TCP",
- * "UDP", and "TCP/UDP".
- */
-enum tcpudp_type {
-    /* The "TCP" menu item is currently selected. */
-    E_DECODE_TCP,
-
-    /* The "TCP" menu item is currently selected. */
-    E_DECODE_UDP,
-
-    /* The "TCP/UDP" menu item is currently selected. */
-    E_DECODE_TCPUDP
-};
-
 /*
  * Enum used to track which transport layer port menu item is
  * currently selected in the dialog.  These items are labeled "source",
@@ -98,7 +82,6 @@ enum srcdst_type {
 #define E_DECODE_MIN_HEIGHT 100
 #define E_NOTEBOOK "notebook"
 
-#define E_MENU_TCPUDP "menu_tcp_udp"
 #define E_MENU_SRCDST "menu_src_dst"
 
 #define E_PAGE_ACTION "notebook_page_action"
@@ -108,14 +91,14 @@ enum srcdst_type {
 #define E_PAGE_VALUE  "notebook_page_value"
 
 /*
- * Clist columns for a "Select" clist
+ * Clist columns for a "Select" clist.
+ * Note that most of these columns aren't displayed; they're attached
+ * to the row of the table as additional information.
  */
 #define E_CLIST_S_PROTO_NAME 0
 #define E_CLIST_S_TABLE	     1
-#define E_CLIST_S_ISOLD	     2
 /* The following is for debugging in decode_add_to_clist */
-#define E_CLIST_S_ISCONV     3
-#define E_CLIST_S_MAX	     E_CLIST_S_ISCONV
+#define E_CLIST_S_MAX	     E_CLIST_S_TABLE
 #define E_CLIST_S_COLUMNS   (E_CLIST_S_MAX + 1)
 
 /*
@@ -156,15 +139,6 @@ static GtkWidget *decode_show_w = NULL;
  */
 static GSList *decode_dimmable = NULL;
 
-/*
- * A list of additional IP port numbers that are currently being
- * decodes as either TCP or UDP.  This is used to determine whether or
- * not to include a "transport" page in the dialog notebook.  This
- * list never includes values for the standard TCP or UDP protocol
- * numbers.
- */
-static GSList *decode_as_tcpudp = NULL;
-
 /*
  * Remember the "action" radio button that is currently selected in
  * the dialog.  This value is initialized when the dialog is created,
@@ -186,7 +160,7 @@ struct dissector_delete_item {
     /* The name of the dissector table */
     const gchar *ddi_table_name;
     /* The port number in the dissector table */
-    gint   ddi_port;
+    guint   ddi_port;
 };
 
 /*
@@ -229,7 +203,7 @@ decode_build_reset_list (gchar *table_name, gpointer key,
 
     item = g_malloc(sizeof(dissector_delete_item_t));
     item->ddi_table_name = table_name;
-    item->ddi_port = GPOINTER_TO_INT(key);
+    item->ddi_port = GPOINTER_TO_UINT(key);
     dissector_reset_list = g_slist_prepend(dissector_reset_list, item);
 }
 
@@ -262,21 +236,62 @@ decode_build_show_list (gchar *table_name, gpointer key,
 			gpointer value, gpointer user_data)
 {
     GtkCList  *clist;
+    dissector_handle_t current, initial;
     gchar     *current_proto_name, *initial_proto_name, *text[E_CLIST_D_COLUMNS];
     gchar      string1[20];
-    gint       current_proto, initial_proto, row;
+    gint       row;
 
     g_assert(user_data);
     g_assert(value);
 
     clist = (GtkCList *)user_data;
-    current_proto = dissector_get_proto(value);
-    current_proto_name = proto_get_protocol_short_name(current_proto);
-    initial_proto = dissector_get_initial_proto(value);
-    initial_proto_name = proto_get_protocol_short_name(initial_proto);
+    current = dtbl_entry_get_handle(value);
+    if (current == NULL)
+	current_proto_name = "(none)";
+    else
+	current_proto_name = dissector_handle_get_short_name(current);
+    initial = dtbl_entry_get_initial_handle(value);
+    if (initial == NULL)
+	initial_proto_name = "(none)";
+    else
+	initial_proto_name = dissector_handle_get_short_name(initial);
 
-    text[E_CLIST_D_TABLE] = table_name;
-    sprintf(string1, "%d", GPOINTER_TO_INT(key));
+    text[E_CLIST_D_TABLE] = get_dissector_table_ui_name(table_name);
+    switch (get_dissector_table_base(table_name)) {
+
+    case BASE_DEC:
+	sprintf(string1, "%u", GPOINTER_TO_UINT(key));
+	break;
+
+    case BASE_HEX:
+	switch (get_dissector_table_type(table_name)) {
+
+	case FT_UINT8:
+	    sprintf(string1, "0x%02x", GPOINTER_TO_UINT(key));
+	    break;
+
+	case FT_UINT16:
+	    sprintf(string1, "0x%04x", GPOINTER_TO_UINT(key));
+	    break;
+
+	case FT_UINT24:
+	    sprintf(string1, "0x%06x", GPOINTER_TO_UINT(key));
+	    break;
+
+	case FT_UINT32:
+	    sprintf(string1, "0x%08x", GPOINTER_TO_UINT(key));
+	    break;
+
+	default:
+	    g_assert_not_reached();
+	    break;
+	}
+	break;
+
+    case BASE_OCT:
+	sprintf(string1, "%#o", GPOINTER_TO_UINT(key));
+	break;
+    }
     text[E_CLIST_D_PORT] = string1;
     text[E_CLIST_D_INITIAL] = initial_proto_name;
     text[E_CLIST_D_CURRENT] = current_proto_name;
@@ -481,34 +496,23 @@ decode_show_cb (GtkWidget * w, gpointer data)
 static void
 decode_change_one_dissector (gchar *table_name, gint selector, GtkCList *clist)
 {
-    dissector_t  dissector;
-    gboolean     old;
-    gchar       *abbrev, *oldstring;
-    gint         row, proto_num;
+    dissector_handle_t handle;
+    gchar              *abbrev;
+    gint               row;
 
     if (!clist->selection) {
-	proto_num = -1;
 	abbrev = "(NULL)";
-	old = FALSE;
-	dissector = NULL;
+	handle = NULL;
     } else {
 	row = GPOINTER_TO_INT(clist->selection->data);
-	proto_num = GPOINTER_TO_INT(gtk_clist_get_row_data(clist, row));
+	handle = gtk_clist_get_row_data(clist, row);
 	gtk_clist_get_text(clist, row, E_CLIST_S_PROTO_NAME, &abbrev);
-	gtk_clist_get_text(clist, row, E_CLIST_S_ISOLD, &oldstring);
-	old = (strcmp(oldstring, "TRUE") == 0);
-	dissector = proto_get_protocol_dissector(proto_num);
-	if ((proto_num != -1) && (dissector == NULL)) {
-	    simple_dialog(ESD_TYPE_CRIT, NULL,
-			  "Protocol dissector structure disappeared");
-	    return;
-	}
     }
 
     if (strcmp(abbrev, "(default)") == 0) {
 	dissector_reset(table_name, selector);
     } else {
-	dissector_change(table_name, selector, dissector, old, proto_num);
+	dissector_change(table_name, selector, handle);
     }
 }
 
@@ -535,18 +539,18 @@ static void
 decode_debug (GtkCList *clist, gchar *leadin)
 {
     gchar *string, *text[E_CLIST_S_COLUMNS];
-    gint row, proto_num;
+    dissector_handle_t handle;
+    gint row;
 
     string = g_malloc(1024);
     if (clist->selection) {
 	row = GPOINTER_TO_INT(clist->selection->data);
 	gtk_clist_get_text(clist, row, E_CLIST_S_PROTO_NAME, &text[E_CLIST_S_PROTO_NAME]);
 	gtk_clist_get_text(clist, row, E_CLIST_S_TABLE, &text[E_CLIST_S_TABLE]);
-	gtk_clist_get_text(clist, row, E_CLIST_S_ISOLD, &text[E_CLIST_S_ISOLD]);
-	proto_num = GPOINTER_TO_INT(gtk_clist_get_row_data(clist, row));
-	sprintf(string, "%s clist row %d: proto %d, name %s, table %s, old %s",
-		leadin, row, proto_num, text[E_CLIST_S_PROTO_NAME],
-		text[E_CLIST_S_TABLE], text[E_CLIST_S_ISOLD]);
+	handle = gtk_clist_get_row_data(clist, row);
+	sprintf(string, "%s clist row %d: <put handle here>, name %s, table %s",
+		leadin, row, text[E_CLIST_S_PROTO_NAME],
+		text[E_CLIST_S_TABLE]);
     } else {
 	sprintf(string, "%s clist row (none), aka do not decode", leadin);
     }
@@ -570,6 +574,9 @@ static void
 decode_simple (GtkWidget *notebook_pg)
 {
     GtkCList *clist;
+#ifdef DEBUG
+    gchar *string;
+#endif
     gchar *table_name;
     gint value;
 
@@ -589,71 +596,11 @@ decode_simple (GtkWidget *notebook_pg)
 }
 
 
-/*
- * This routine is called when the user clicks the "OK" button in the
- * "Decode As..." dialog window and the network page is foremost.
- * This routine takes care of making any changes requested to the
- * dissector tables.  This routine uses the decode_simple() routine to
- * perform the heavy lifting, and then updates a list of protocol that
- * are being decoded as TCP/UDP. *
- *
- * @param notebook_pg A pointer to the "network" notebook page.
- */
-static void
-decode_network (GtkWidget *notebook_pg)
-{
-    GtkCList *clist;
-    GSList *item;
-    gint row, assigned, port_num;
-
-    /* Do the real work */
-    decode_simple(notebook_pg);
-
-    /* Now tweak a local table of protocol ids currently decoded as TCP/UDP */
-    port_num = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(notebook_pg),
-						E_PAGE_VALUE));
-
-    /* Ignore changes to the normal TCP and UDP protocol numbers */
-    if ((port_num == IP_PROTO_TCP) || (port_num == IP_PROTO_UDP))
-	return;
-
-    /* Not decoding - remove any entry for this IP protocol number */
-    if (requested_action == E_DECODE_NO) {
-	decode_as_tcpudp =
-	    g_slist_remove(decode_as_tcpudp, GINT_TO_POINTER(port_num));
-	return;
-    }
-
-    /* Not assigning TCP or UDP - remove any entry for this IP protocol number.
-       Note: if the action was E_DECODE_NO, the selection on the clist
-       was cleared, so the code to get "row" below would blow up. */
-    clist = GTK_CLIST(gtk_object_get_data(GTK_OBJECT(notebook_pg), E_PAGE_CLIST));
-    row = GPOINTER_TO_INT(clist->selection->data);
-    assigned = GPOINTER_TO_INT(gtk_clist_get_row_data(clist, row));
-    if ((assigned != IP_PROTO_TCP) && (assigned != IP_PROTO_UDP)) {
-	decode_as_tcpudp =
-	    g_slist_remove(decode_as_tcpudp, GINT_TO_POINTER(port_num));
-	return;
-    }
-
-    /* Assigning TCP or UDP - add if not already present */
-    item = g_slist_find(decode_as_tcpudp, GINT_TO_POINTER(port_num));
-    if (!item) {
-	decode_as_tcpudp =
-	    g_slist_prepend(decode_as_tcpudp, GINT_TO_POINTER(port_num));
-    }
-}
-
-
 /*
  * This routine is called when the user clicks the "OK" button in the
  * "Decode As..." dialog window and the transport page is foremost.
  * This routine takes care of making any changes requested to the TCP
- * and UDP dissector tables.
- *
- * Note: The negative tests catch multiple cases.  For example, if the
- * user didn't select UDP, then they either selected TCP or TCP/UDP.
- * Either way they *did* select TCP.
+ * or UDP dissector tables.
  *
  * @param notebook_pg A pointer to the "transport" notebook page.
  */
@@ -662,16 +609,13 @@ decode_transport (GtkObject *notebook_pg)
 {
     GtkWidget *menu, *menuitem;
     GtkCList *clist;
-    gint requested_tcpudp, requested_srcdst;
+    gchar *table_name;
+    gint requested_srcdst;
 
     clist = GTK_CLIST(gtk_object_get_data(notebook_pg, E_PAGE_CLIST));
     if (requested_action == E_DECODE_NO)
 	gtk_clist_unselect_all(clist);
 
-    menu = gtk_object_get_data(notebook_pg, E_MENU_TCPUDP);
-    menuitem = gtk_menu_get_active(GTK_MENU(menu));
-    requested_tcpudp = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
-
     menu = gtk_object_get_data(notebook_pg, E_MENU_SRCDST);
     menuitem = gtk_menu_get_active(GTK_MENU(menu));
     requested_srcdst = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
@@ -681,18 +625,11 @@ decode_transport (GtkObject *notebook_pg)
     decode_debug(clist, string);
 #endif
 
-    if (requested_tcpudp != E_DECODE_UDP) {
-	if (requested_srcdst != E_DECODE_DPORT)
-	    decode_change_one_dissector("tcp.port", pi.srcport, clist);
-	if (requested_srcdst != E_DECODE_SPORT)
-	    decode_change_one_dissector("tcp.port", pi.destport, clist);
-    }
-    if (requested_tcpudp != E_DECODE_TCP) {
-	if (requested_srcdst != E_DECODE_DPORT)
-	    decode_change_one_dissector("udp.port", pi.srcport, clist);
-	if (requested_srcdst != E_DECODE_SPORT)
-	    decode_change_one_dissector("udp.port", pi.destport, clist);
-    }
+    table_name = gtk_object_get_data(GTK_OBJECT(notebook_pg), E_PAGE_TABLE);
+    if (requested_srcdst != E_DECODE_DPORT)
+	decode_change_one_dissector(table_name, cfile.edt->pi.srcport, clist);
+    if (requested_srcdst != E_DECODE_SPORT)
+	decode_change_one_dissector(table_name, cfile.edt->pi.destport, clist);
 }
 
 /**************************************************/
@@ -876,54 +813,6 @@ decode_add_pack_menu (GtkWidget *optmenu)
     return(alignment);
 }
 
-/*
- * This routine is called to add the transport protocol selection menu
- * to the dialog box.  This is a three choice menu: TCP, UDP, and
- * TCP/UDP.  The default choice for the menu is set to the transport
- * layer protocol of the currently selected packet.
- *
- * @param page A pointer notebook page that will contain all
- * widgets created by this routine.
- *
- * @return GtkWidget * A pointer to the newly created alignment into
- * which we've packed the newly created option menu.
- */
-static GtkWidget *
-decode_add_tcpudp_menu (GtkWidget *page)
-{
-    GtkWidget *optmenu, *menu, *menuitem, *alignment;
-    gint requested_tcpudp;
-
-    optmenu = gtk_option_menu_new();
-    menu = gtk_menu_new();
-    menuitem = gtk_menu_item_new_with_label("TCP");
-    gtk_object_set_user_data(GTK_OBJECT(menuitem),
-			     GINT_TO_POINTER(E_DECODE_TCP));
-    gtk_menu_append(GTK_MENU(menu), menuitem);
-    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
-
-    menuitem = gtk_menu_item_new_with_label("UDP");
-    gtk_object_set_user_data(GTK_OBJECT(menuitem),
-			     GINT_TO_POINTER(E_DECODE_UDP));
-    gtk_menu_append(GTK_MENU(menu), menuitem);
-    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
-
-    menuitem = gtk_menu_item_new_with_label("TCP/UDP");
-    gtk_object_set_user_data(GTK_OBJECT(menuitem),
-			     GINT_TO_POINTER(E_DECODE_TCPUDP));
-    gtk_menu_append(GTK_MENU(menu), menuitem);
-    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
-
-    requested_tcpudp = (pi.ipproto == IP_PROTO_TCP) ? E_DECODE_TCP : E_DECODE_UDP;
-    gtk_menu_set_active(GTK_MENU(menu), requested_tcpudp == E_DECODE_UDP);
-    gtk_object_set_data(GTK_OBJECT(page), E_MENU_TCPUDP, menu);
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
-
-    alignment = decode_add_pack_menu(optmenu);
-
-    return(alignment);
-}
-
 
 /*
  * This routine is called to add the transport port selection menu to
@@ -945,14 +834,14 @@ decode_add_srcdst_menu (GtkWidget *page)
 
     optmenu = gtk_option_menu_new();
     menu = gtk_menu_new();
-    sprintf(tmp, "source (%u)", pi.srcport);
+    sprintf(tmp, "source (%u)", cfile.edt->pi.srcport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     gtk_object_set_user_data(GTK_OBJECT(menuitem),
 			     GINT_TO_POINTER(E_DECODE_SPORT));
     gtk_menu_append(GTK_MENU(menu), menuitem);
     gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
 
-    sprintf(tmp, "destination (%u)", pi.destport);
+    sprintf(tmp, "destination (%u)", cfile.edt->pi.destport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     gtk_object_set_user_data(GTK_OBJECT(menuitem),
 			     GINT_TO_POINTER(E_DECODE_DPORT));
@@ -978,30 +867,20 @@ decode_add_srcdst_menu (GtkWidget *page)
 /**************************************************/
 
 
-typedef struct decode_build_clist_info {
-    GtkCList *clist;
-    gboolean conv;
-} decode_build_clist_info_t;
-
 /*
  * This routine creates one entry in the list of protocol dissector
- * that can be used.  It is called by the g_hash_foreach routine once
- * for each entry in a dissector table.  It guarantees unique entries
- * by iterating over the list of entries build up to this point,
+ * that can be used.  It is called by the dissector_table_foreach_handle
+ * routine once for each entry in a dissector table's list of handles
+ * for dissectors that could be used in that table.  It guarantees unique
+ * entries by iterating over the list of entries build up to this point,
  * looking for a duplicate name.  If there is no duplicate, then this
  * entry is added to the list of possible dissectors.
  *
- * @param table_name The name of the dissector hash table currently
+ * @param table_name The name of the dissector table currently
  * being walked.
  *
- * @param key A pointer to the key for this entry in the
- * dissector hash table.  This is generally the numeric selector of
- * the protocol, i.e. the ethernet type code, IP port number, TCP port
- * number, etc.
- *
- * @param value A pointer to the value for this entry in the
- * dissector hash table.  This is an opaque pointer that can only be
- * handed back to routines in the file packet.c
+ * @param value The dissector handle for this entry.  This is an opaque
+ * pointer that can only be handed back to routines in the file packet.c
  *
  * @param user_data A data block passed into each instance of this
  * routine.  It contains information from the caller of the foreach
@@ -1009,42 +888,34 @@ typedef struct decode_build_clist_info {
  * to store any information generated by this routine.
  */
 static void
-decode_add_to_clist (gchar *table_name, gpointer key,
-		     gpointer value, gpointer user_data)
+decode_add_to_clist (gchar *table_name, gpointer value, gpointer user_data)
 {
     GtkCList  *clist;
-    gchar     *proto_name, *isold, *isconv;
+    gchar     *proto_name;
     gchar     *text[E_CLIST_S_COLUMNS];
-    gint proto, row;
-    decode_build_clist_info_t *info;
+    dissector_handle_t handle;
+    gint       row;
 
     g_assert(user_data);
     g_assert(value);
 
-    info = user_data;
-    clist = info->clist;
-    if (info->conv) {
-	proto = conv_dissector_get_proto(value);
-	isold = "FALSE";
-	isconv = "TRUE";
-    } else {
-	proto = dissector_get_proto(value);
-	isold = dissector_get_old_flag(value) ? "TRUE" : "FALSE";
-	isconv = "FALSE";
-    }
-    proto_name = proto_get_protocol_short_name(proto);
+    clist = user_data;
+    handle = value;
+    proto_name = dissector_handle_get_short_name(handle);
 
-    row = gtk_clist_find_row_from_data(clist, GINT_TO_POINTER(proto));
+    row = gtk_clist_find_row_from_data(clist, handle);
     if (row != -1) {
+	/*
+	 * We already have an entry for this handle.
+	 * XXX - will this ever happen?
+	 */
 	return;
     }
 
     text[E_CLIST_S_PROTO_NAME] = proto_name;
     text[E_CLIST_S_TABLE] = table_name;
-    text[E_CLIST_S_ISOLD] = isold;
-    text[E_CLIST_S_ISCONV] = isconv;
     row = gtk_clist_prepend(clist, text);
-    gtk_clist_set_row_data(clist, row, GINT_TO_POINTER(proto));
+    gtk_clist_set_row_data(clist, row, handle);
 }
 
 
@@ -1065,8 +936,7 @@ static void
 decode_clist_menu_start (GtkWidget *page, GtkCList **clist_p,
 			 GtkWidget **scrolled_win_p)
 {
-    gchar *titles[E_CLIST_S_COLUMNS] = {"Short Name", "Table Name",
-					"Is Old", "Is Conversation"};
+    gchar *titles[E_CLIST_S_COLUMNS] = {"Short Name", "Table Name"};
     GtkCList  *clist;
     GtkWidget *window;
     gint column;
@@ -1107,10 +977,8 @@ decode_clist_menu_finish (GtkCList *clist)
 
     text[E_CLIST_S_PROTO_NAME] = "(default)";
     text[E_CLIST_S_TABLE] = "(none)";
-    text[E_CLIST_S_ISOLD] = "(who cares)";
-    text[E_CLIST_S_ISCONV] = "(who cares)";
     row = gtk_clist_prepend(clist, text);
-    gtk_clist_set_row_data(clist, row, GINT_TO_POINTER(-1));
+    gtk_clist_set_row_data(clist, row, NULL);
 
     gtk_clist_select_row(clist, 0, -1);
     gtk_clist_sort(clist);
@@ -1126,8 +994,8 @@ decode_clist_menu_finish (GtkCList *clist)
  *
  * @param page A pointer to the notebook page currently being created.
  *
- * @param table_name The name of the dissector hash table to use to
- * build this (clist) menu.
+ * @param table_name The name of the dissector table to use to build
+ * this (clist) menu.
  *
  * @return GtkWidget * A pointer to the newly created clist within a
  * scrolled window.
@@ -1137,47 +1005,9 @@ decode_add_simple_menu (GtkWidget *page, gchar *table_name)
 {
     GtkWidget *scrolled_window;
     GtkCList  *clist;
-    decode_build_clist_info_t info;
 
     decode_clist_menu_start(page, &clist, &scrolled_window);
-    {
-	info.clist = clist;
-	info.conv = FALSE;
-	dissector_table_foreach(table_name, decode_add_to_clist, &info);
-    }
-    decode_clist_menu_finish(clist);
-    return(scrolled_window);
-}
-
-/*
- * This routine is called to add the dissector selection list to 
- * notebook page.  This scrolled list contains an entry labeled
- * "default", and an entry for each protocol that has had a dissector
- * registered.  The default choice for the list is set to the
- * "default" choice, which will return the protocol/port selections to
- * their original dissector(s).
- *
- * @param page A pointer to the notebook page currently being created.
- *
- * @return GtkWidget * A pointer to the newly created option menu.
- */
-static GtkWidget *
-decode_add_transport_menu (GtkWidget *page)
-{
-    GtkWidget *scrolled_window;
-    GtkCList  *clist;
-    decode_build_clist_info_t info;
-
-    decode_clist_menu_start(page, &clist, &scrolled_window);
-    {
-	info.clist = clist;
-	info.conv = FALSE;
-	dissector_table_foreach("tcp.port", decode_add_to_clist, &info);
-	dissector_table_foreach("udp.port", decode_add_to_clist, &info);
-
-	info.conv = TRUE;
-	dissector_conv_foreach("udp", decode_add_to_clist, &info);
-    }
+    dissector_table_foreach_handle(table_name, decode_add_to_clist, clist);
     decode_clist_menu_finish(clist);
     return(scrolled_window);
 }
@@ -1187,7 +1017,7 @@ decode_add_transport_menu (GtkWidget *page)
 /**************************************************/
 
 /*
- * This routine creates a somple notebook page in the dialog box.
+ * This routine creates a sample notebook page in the dialog box.
  * This notebook page provides a prompt specifying what is being
  * changed and its current value (e.g. "IP Protocol number (17)"), and
  * a clist specifying all the available choices.  The list of choices
@@ -1196,10 +1026,9 @@ decode_add_transport_menu (GtkWidget *page)
  *
  * @param prompt The prompt for this notebook page
  *
- * @param title A table name from which all dissector names will
- * be extracted.
+ * @param title A title for this page to use when debugging.
  *
- * @param table_name The name of the dissector hash table to use to
+ * @param table_name The name of the dissector table to use to
  * build this page.
  *
  * @param value The protocol/port value that is to be changed.
@@ -1236,35 +1065,41 @@ decode_add_simple_page (gchar *prompt, gchar *title, gchar *table_name,
 
 
 /*
- * This routine creates the TCP/UDP notebook page in the dialog box.
+ * This routine creates the TCP or UDP notebook page in the dialog box.
  * All items created by this routine are packed into a single
- * horizontal box.  First is a menu allowing the user to select the
- * TCP or UDP transport layer protocol.  Second is a menu allowing the
- * user to select whether the source port, destination port, or both
- * ports will have dissectors added for them.  Last is a
- * (conditionally enabled) popup menu listing all possible dissectors
- * that can be used to decode the packets, and the choice or returning
- * to the default dissector for these ports.
+ * horizontal box.  First is a label indicating whether the port(s) for
+ * which the user can set the dissection is a TCP port or a UDP port.
+ * Second is a menu allowing the user to select whether the source port,
+ * destination port, or both ports will have dissectors added for them.
+ * Last is a (conditionally enabled) popup menu listing all possible
+ * dissectors that can be used to decode the packets, and the choice
+ * or returning to the default dissector for these ports.
  *
  * The defaults for these items are the transport layer protocol of
  * the currently selected packet, the source port of the currently
  * selected packet, and the "default dissector".
  *
+ * @param prompt The prompt for this notebook page
+ *
+ * @param table_name The name of the dissector table to use to
+ * build this page.
+ *
  * @return GtkWidget * A pointer to the notebook page created by
  * this routine.
  */
 static GtkWidget *
-decode_add_tcpudp_page (void)
+decode_add_tcpudp_page (gchar *prompt, gchar *table_name)
 {
     GtkWidget	*page, *label, *scrolled_window, *optmenu;
 
     page = gtk_hbox_new(FALSE, 5);
     gtk_object_set_data(GTK_OBJECT(page), E_PAGE_ACTION, decode_transport);
+    gtk_object_set_data(GTK_OBJECT(page), E_PAGE_TABLE, table_name);
     gtk_object_set_data(GTK_OBJECT(page), E_PAGE_TITLE, "Transport");
 
     /* Always enabled */
-    optmenu = decode_add_tcpudp_menu(page);
-    gtk_box_pack_start(GTK_BOX(page), optmenu, TRUE, TRUE, 0);
+    label = gtk_label_new(prompt);
+    gtk_box_pack_start(GTK_BOX(page), label, TRUE, TRUE, 0);
     optmenu = decode_add_srcdst_menu(page);
     gtk_box_pack_start(GTK_BOX(page), optmenu, TRUE, TRUE, 0);
     label = gtk_label_new("port(s)");
@@ -1274,7 +1109,7 @@ decode_add_tcpudp_page (void)
     label = gtk_label_new("as");
     gtk_box_pack_start(GTK_BOX(page), label, TRUE, TRUE, 0);
     decode_dimmable = g_slist_prepend(decode_dimmable, label);
-    scrolled_window = decode_add_transport_menu(page);
+    scrolled_window = decode_add_simple_menu(page, table_name);
     gtk_box_pack_start(GTK_BOX(page), scrolled_window, TRUE, TRUE, 0);
     decode_dimmable = g_slist_prepend(decode_dimmable, scrolled_window);
 
@@ -1282,23 +1117,15 @@ decode_add_tcpudp_page (void)
 }
 
 /*
- * Indicate if a transport page should be included, based upon the iP
- * protocol number.
- *
- * @param ip_protocol The IP protocol number in question.
- *
- * @return gboolean TRUE if this protocol is being decoded as TCP or
- * UDP.
+ * This routine indicates whether we'd actually have any pages in the
+ * notebook in a "Decode As" dialog box; if there wouldn't be, we
+ * inactivate the menu item for "Decode As".
  */
-static gboolean
-decode_as_transport_ok (gint ip_protocol)
+gboolean
+decode_as_ok(void)
 {
-    if ((ip_protocol == IP_PROTO_TCP) || (ip_protocol == IP_PROTO_UDP))
-	return(TRUE);
-
-    if (g_slist_find(decode_as_tcpudp, GINT_TO_POINTER(ip_protocol)))
-	return(TRUE);
-    return(FALSE);
+    return cfile.edt->pi.ethertype || cfile.edt->pi.ipproto ||
+	cfile.edt->pi.ptype == PT_TCP || cfile.edt->pi.ptype == PT_UDP;
 }
 
 
@@ -1322,25 +1149,41 @@ decode_add_notebook (GtkWidget *format_hb)
     gtk_object_set_data(GTK_OBJECT(decode_w), E_NOTEBOOK, notebook);
 
     /* Add link level selection page */
-    if (pi.ethertype) {
-	sprintf(buffer, "Ethertype 0x%04x", pi.ethertype);
-	page = decode_add_simple_page(buffer, "Link", "ethertype", pi.ethertype);
+    if (cfile.edt->pi.ethertype) {
+	sprintf(buffer, "Ethertype 0x%04x", cfile.edt->pi.ethertype);
+	page = decode_add_simple_page(buffer, "Link", "ethertype", cfile.edt->pi.ethertype);
 	label = gtk_label_new("Link");
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
     }
 
     /* Add network selection page */
-    if (pi.ipproto) {
-	sprintf(buffer, "IP protocol %u", pi.ipproto);
-	page = decode_add_simple_page(buffer, "Network", "ip.proto", pi.ipproto);
-	gtk_object_set_data(GTK_OBJECT(page), E_PAGE_ACTION, decode_network);
+    if (cfile.edt->pi.ipproto) {
+	/*
+	 * The network-layer protocol is IP.
+	 */
+	sprintf(buffer, "IP protocol %u", cfile.edt->pi.ipproto);
+	page = decode_add_simple_page(buffer, "Network", "ip.proto", cfile.edt->pi.ipproto);
+	gtk_object_set_data(GTK_OBJECT(page), E_PAGE_ACTION, decode_simple);
 	label = gtk_label_new("Network");
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
     }
 
     /* Add transport selection page */
-    if (decode_as_transport_ok(pi.ipproto)) {
-	page = decode_add_tcpudp_page();
+    switch (cfile.edt->pi.ptype) {
+
+    case PT_TCP:
+	page = decode_add_tcpudp_page("TCP", "tcp.port");
+	break;
+
+    case PT_UDP:
+	page = decode_add_tcpudp_page("UDP", "udp.port");
+	break;
+
+    default:
+	page = NULL;
+	break;
+    }
+    if (page != NULL) {
 	label = gtk_label_new("Transport");
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
     }
@@ -1439,17 +1282,6 @@ decode_as_cb (GtkWidget * w, gpointer data)
     gtk_widget_show_all(decode_w);
 }
 
-/*
- * This routine indicates whether we'd actually have any pages in the
- * notebook in a "Decode As" dialog box; if there wouldn't be, we
- * inactivate the menu item for "Decode As".
- */
-gboolean
-decode_as_ok(void)
-{
-    return pi.ethertype || pi.ipproto || decode_as_transport_ok(pi.ipproto);
-}
-
 
 /*
  * Local Variables:
