@@ -1,7 +1,7 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c,v 1.40 2001/06/05 07:38:37 guy Exp $
+ * $Id: file_dlg.c,v 1.46 2001/12/06 03:09:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,6 +36,10 @@
 
 #include <string.h>
 
+#include <glib.h>
+
+#include <epan/filesystem.h>
+
 #include "globals.h"
 #include "gtkglobals.h"
 #include "prefs.h"
@@ -47,7 +51,6 @@
 #include "menu.h"
 #include "file_dlg.h"
 #include "dlg_utils.h"
-#include "util.h"
 #include "main.h"
 
 static void file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs);
@@ -376,6 +379,7 @@ select_file_type_cb(GtkWidget *w, gpointer data)
        use Wiretap to save the file. */
     gtk_widget_set_sensitive(filter_cb, can_save_with_wiretap(new_filetype));
     filetype = new_filetype;
+    file_set_save_marked_sensitive();
   }
 }
 
@@ -453,6 +457,16 @@ file_save_as_cmd_cb(GtkWidget *w, gpointer data)
     main_vb, FALSE, FALSE, 0);
   gtk_widget_show(main_vb);
   
+  /*
+   * XXX - should this be sensitive only if the current display filter
+   * has rejected some packets, so that not all packets are currently
+   * being displayed, and if it has accepted some packets, so that some
+   * packets are currently being displayed?
+   *
+   * I'd say "no", as that complicates the UI code, and as one could,
+   * I guess, argue that the user may want to "save all the displayed
+   * packets" even if there aren't any, i.e. save an empty file.
+   */
   filter_cb = gtk_check_button_new_with_label("Save only packets currently being displayed");
   gtk_container_add(GTK_CONTAINER(main_vb), filter_cb);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(filter_cb), FALSE);
@@ -461,12 +475,21 @@ file_save_as_cmd_cb(GtkWidget *w, gpointer data)
   gtk_widget_set_sensitive(filter_cb, can_save_with_wiretap(filetype));
   gtk_widget_show(filter_cb);
 
+  /*
+   * The argument above could, I guess, be applied to the marked packets,
+   * except that you can't easily tell whether there are any marked
+   * packets, so I could imagine users doing "Save only marked packets"
+   * when there aren't any marked packets, not knowing that they'd
+   * failed to mark them, so I'm more inclined to have the "Save only
+   * marked packets" toggle button enabled only if there are marked
+   * packets to save.
+   */
   mark_cb = gtk_check_button_new_with_label("Save only marked packets");
   gtk_container_add(GTK_CONTAINER(main_vb), mark_cb);
+  marked = FALSE;
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(mark_cb), FALSE);
   gtk_signal_connect(GTK_OBJECT(mark_cb), "toggled",
 		     GTK_SIGNAL_FUNC(toggle_marked_cb), NULL);
-  gtk_widget_set_sensitive(mark_cb, can_save_with_wiretap(filetype));
   gtk_widget_show(mark_cb);
 
   /* File type row */
@@ -485,6 +508,15 @@ file_save_as_cmd_cb(GtkWidget *w, gpointer data)
   gtk_box_pack_start(GTK_BOX(ft_hb), ft_om, FALSE, FALSE, 0);
   gtk_widget_show(ft_om);
 
+  /*
+   * Set the sensitivity of the "Save only marked packets" toggle
+   * button
+   *
+   * This has to be done after we create the file type menu option,
+   * as the routine that sets it also sets that menu.
+   */
+  file_set_save_marked_sensitive();
+
   /* Connect the cancel_button to destroy the widget */
   gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION
     (file_save_as_w)->cancel_button), "clicked", (GtkSignalFunc)
@@ -496,9 +528,40 @@ file_save_as_cmd_cb(GtkWidget *w, gpointer data)
   dlg_set_cancel(file_save_as_w, GTK_FILE_SELECTION(file_save_as_w)->cancel_button);
 
   gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_save_as_w), "");
+
   gtk_widget_show(file_save_as_w);
 }
 
+/*
+ * Set the "Save only marked packets" toggle button as appropriate for
+ * the current output file type and count of marked packets.
+ *
+ * Called when the "Save As..." dialog box is created and when either
+ * the file type or the marked count changes.
+ */
+void
+file_set_save_marked_sensitive(void)
+{
+  if (file_save_as_w == NULL) {
+    /* We don't currently have a "Save As..." dialog box up. */
+    return;
+  }
+
+  /* We can request that only the marked packets be saved only if we
+     can use Wiretap to save the file and if there *are* marked packets. */
+  if (can_save_with_wiretap(filetype) && cfile.marked_count != 0)
+    gtk_widget_set_sensitive(mark_cb, TRUE);
+  else {
+    /* Force the "Save only marked packets" toggle to "false", turn
+       off the flag it controls, and update the list of types we can
+       save the file as. */
+    marked = FALSE;
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(mark_cb), FALSE);
+    set_file_type_list(ft_om);
+    gtk_widget_set_sensitive(mark_cb, FALSE);
+  }
+}
+
 static void
 file_save_as_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   gchar	*cf_name;
@@ -548,7 +611,7 @@ file_reload_cmd_cb(GtkWidget *w, gpointer data) {
 
      Also, "close_cap_file()" will free "cfile.filename", so we must make
      a copy of it first. */
-  filename = strdup(cfile.filename);
+  filename = g_strdup(cfile.filename);
   is_tempfile = cfile.is_tempfile;
   cfile.is_tempfile = FALSE;
   if (open_cap_file(filename, is_tempfile, &cfile) == 0) {
