@@ -1,6 +1,6 @@
 /* follow_dlg.c
  *
- * $Id: follow_dlg.c,v 1.13 2001/04/10 12:07:39 gram Exp $
+ * $Id: follow_dlg.c,v 1.16 2001/12/18 19:09:07 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -70,6 +70,7 @@
 #include "resolv.h"
 #include "util.h"
 #include "ui_util.h"
+#include "epan_dissect.h"
 
 /* Show Stream */
 typedef enum {
@@ -163,7 +164,7 @@ follow_stream_cb(GtkWidget * w, gpointer data)
 	follow_info_t	*follow_info;
 
 	/* we got tcp so we can follow */
-	if (pi.ipproto != 6) {
+	if (cfile.edt->pi.ipproto != 6) {
 		simple_dialog(ESD_TYPE_CRIT, NULL,
 			      "Error following stream.  Please make\n"
 			      "sure you have a TCP packet selected.");
@@ -205,7 +206,7 @@ follow_stream_cb(GtkWidget * w, gpointer data)
 	/* Create a new filter that matches all packets in the TCP stream,
 	   and set the display filter entry accordingly */
 	reset_tcp_reassembly();
-	follow_filter = build_follow_filter(&pi);
+	follow_filter = build_follow_filter(&cfile.edt->pi);
 
 	/* Set the display filter entry accordingly */
 	filter_te = gtk_object_get_data(GTK_OBJECT(w), E_DFILTER_TE_KEY);
@@ -224,6 +225,8 @@ follow_stream_cb(GtkWidget * w, gpointer data)
 	gtk_signal_connect(GTK_OBJECT(streamwindow), "destroy",
 			   GTK_SIGNAL_FUNC(follow_destroy_cb), NULL);
 
+	gtk_signal_connect (GTK_OBJECT (streamwindow), "realize",
+			   GTK_SIGNAL_FUNC (window_icon_realize_cb), NULL);
 	if (incomplete_tcp_stream) {
 	    gtk_window_set_title(GTK_WINDOW(streamwindow),
 				 "Contents of TCP stream (incomplete)");
