@@ -1,6 +1,6 @@
 /* main.c
  *
- * $Id: main.c,v 1.203 2001/07/05 00:34:40 guy Exp $
+ * $Id: main.c,v 1.221 2001/12/21 20:33:47 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -46,6 +46,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -112,6 +113,7 @@
 
 #include <epan.h>
 #include <epan/filesystem.h>
+#include <epan_dissect.h>
 
 #include "main.h"
 #include "timestamp.h"
@@ -126,10 +128,13 @@
 #include "color_utils.h"
 #include "filter_prefs.h"
 #include "prefs_dlg.h"
+#include "file_dlg.h"
 #include "column.h"
 #include "print.h"
 #include "resolv.h"
-#include "util.h"
+#ifdef HAVE_LIBPCAP
+#include "pcap-util.h"
+#endif
 #include "statusbar.h"
 #include "simple_dialog.h"
 #include "proto_draw.h"
@@ -142,13 +147,21 @@
 #include "strutil.h"
 #include "register.h"
 #include "prefs.h"
+#include "ringbuffer.h"
+#include "ui_util.h"
+#include "image/clist_ascend.xpm"
+#include "image/clist_descend.xpm"
 
 #ifdef WIN32
 #include "capture-wpcap.h"
 #endif
 
+typedef struct column_arrows {
+  GtkWidget *table;
+  GtkWidget *ascend_pm;
+  GtkWidget *descend_pm;
+} column_arrows;
 
-packet_info  pi;
 capture_file cfile;
 GtkWidget   *top_level, *packet_list, *tree_view, *byte_nb_ptr,
             *tv_scrollw, *pkt_scrollw;
@@ -327,16 +340,31 @@ packet_list_compare(GtkCList *clist, gconstpointer  ptr1, gconstpointer  ptr2)
 static void 
 packet_list_click_column_cb(GtkCList *clist, gint column, gpointer data)
 {
+  column_arrows *col_arrows = (column_arrows *) data;
+  int i;
+  
+  gtk_clist_freeze(clist);
+  
+  for (i = 0; i < cfile.cinfo.num_cols; i++) {
+    gtk_widget_hide(col_arrows[i].ascend_pm);
+    gtk_widget_hide(col_arrows[i].descend_pm);
+  }
+  
   if (column == clist->sort_column) {
-    if (clist->sort_type == GTK_SORT_ASCENDING)
+    if (clist->sort_type == GTK_SORT_ASCENDING) {
       clist->sort_type = GTK_SORT_DESCENDING;
-    else
+      gtk_widget_show(col_arrows[column].descend_pm);
+    } else {
       clist->sort_type = GTK_SORT_ASCENDING;
+      gtk_widget_show(col_arrows[column].ascend_pm);
+    }
   }
   else {
     clist->sort_type = GTK_SORT_ASCENDING;
+    gtk_widget_show(col_arrows[column].ascend_pm);
     gtk_clist_set_sort_column(clist, column);
   }
+  gtk_clist_thaw(clist);
 
   gtk_clist_sort(clist);
 }
@@ -346,15 +374,18 @@ static void
 set_frame_mark(gboolean set, frame_data *frame, gint row) {
   GdkColor fg, bg;
 
-  if (frame == NULL || row == -1) return;
-  frame->flags.marked = set;
+  if (row == -1)
+    return;
   if (set) {
+    mark_frame(&cfile, frame);
     color_t_to_gdkcolor(&fg, &prefs.gui_marked_fg);
     color_t_to_gdkcolor(&bg, &prefs.gui_marked_bg);
   } else {
+    unmark_frame(&cfile, frame);
     fg = BLACK;
     bg = WHITE;
   }
+  file_set_save_marked_sensitive();
   gtk_clist_set_background(GTK_CLIST(packet_list), row, &bg);
   gtk_clist_set_foreground(GTK_CLIST(packet_list), row, &fg);
 }
@@ -388,7 +419,6 @@ void mark_frame_cb(GtkWidget *w, gpointer data) {
 
 static void mark_all_frames(gboolean set) {
   frame_data *fdata;
-  if (cfile.plist == NULL) return;
   for (fdata = cfile.plist; fdata != NULL; fdata = fdata->next) {
     set_frame_mark(set,
 		   fdata,
@@ -420,8 +450,6 @@ void unmark_all_frames_cb(GtkWidget *w, gpointer data) {
 static void
 packet_list_select_cb(GtkWidget *w, gint row, gint col, gpointer evt) {
 
-  blank_packetinfo();
-
 /* Remove the hex display tabbed pages */
   while( (gtk_notebook_get_nth_page( GTK_NOTEBOOK(byte_nb_ptr), 0)))
     gtk_notebook_remove_page( GTK_NOTEBOOK(byte_nb_ptr), 0);
@@ -451,7 +479,8 @@ tree_view_select_row_cb(GtkCTree *ctree, GList *node, gint column, gpointer user
 	finfo = gtk_ctree_node_get_row_data( ctree, GTK_CTREE_NODE(node) );
 	if (!finfo) return;
 
-	set_notebook_page(  byte_nb_ptr, find_notebook_page( byte_nb_ptr, finfo->ds_name));
+	if (finfo->ds_name != NULL)
+		set_notebook_page(  byte_nb_ptr, find_notebook_page( byte_nb_ptr, finfo->ds_name));
 
         byte_view = gtk_object_get_data(GTK_OBJECT(byte_nb_ptr), E_BYTE_VIEW_TEXT_INFO_KEY);
         byte_data = gtk_object_get_data(GTK_OBJECT(byte_view), E_BYTE_VIEW_DATA_PTR_KEY);
@@ -528,21 +557,21 @@ tree_view_unselect_row_cb(GtkCTree *ctree, GList *node, gint column, gpointer us
 }
 
 void collapse_all_cb(GtkWidget *widget, gpointer data) {
-  if (cfile.protocol_tree)
-    collapse_all_tree(cfile.protocol_tree, tree_view);
+  if (cfile.edt->tree)
+    collapse_all_tree(cfile.edt->tree, tree_view);
 }
 
 void expand_all_cb(GtkWidget *widget, gpointer data) {
-  if (cfile.protocol_tree)
-    expand_all_tree(cfile.protocol_tree, tree_view);
+  if (cfile.edt->tree)
+    expand_all_tree(cfile.edt->tree, tree_view);
 }
 
 void resolve_name_cb(GtkWidget *widget, gpointer data) {
-  if (cfile.protocol_tree) {
+  if (cfile.edt->tree) {
     gint tmp = prefs.name_resolve;
     prefs.name_resolve = PREFS_RESOLV_ALL;
     gtk_clist_clear ( GTK_CLIST(tree_view) );
-    proto_tree_draw(cfile.protocol_tree, tree_view);
+    proto_tree_draw(cfile.edt->tree, tree_view);
     prefs.name_resolve = tmp;
   }
 }
@@ -770,20 +799,22 @@ print_usage(void) {
   fprintf(stderr, "This is GNU " PACKAGE " " VERSION ", compiled %s\n",
 	  comp_info_str->str);
 #ifdef HAVE_LIBPCAP
-  fprintf(stderr, "%s [ -vh ] [ -klpQS ] [ -B <byte view height> ] [ -c <count> ]\n",
+  fprintf(stderr, "%s [ -vh ] [ -klpQS ] [ -a <capture autostop condition> ] ...\n",
 	  PACKAGE);
-  fprintf(stderr, "\t[ -f <capture filter> ] [ -i <interface> ] [ -m <medium font> ] \n");
-  fprintf(stderr, "\t[ -n ] [ -N <resolving> ]\n");
+  fprintf(stderr, "\t[ -b <number of ringbuffer files> ] [ -B <byte view height> ]\n");
+  fprintf(stderr, "\t[ -c <count> ] [ -f <capture filter> ] [ -i <interface> ]\n");
+  fprintf(stderr, "\t[ -m <medium font> ] [ -n ] [ -N <resolving> ]\n");
   fprintf(stderr, "\t[ -o <preference setting> ] ... [ -P <packet list height> ]\n");
   fprintf(stderr, "\t[ -r <infile> ] [ -R <read filter> ] [ -s <snaplen> ] \n");
-  fprintf(stderr, "\t[ -t <time stamp format> ] [ -T <tree view height> ] [ -w <savefile> ]\n");
+  fprintf(stderr, "\t[ -t <time stamp format> ] [ -T <tree view height> ]\n");
+  fprintf(stderr, "\t[ -w <savefile> ] [ <infile> ]\n");
 #else
   fprintf(stderr, "%s [ -vh ] [ -B <byte view height> ] [ -m <medium font> ]\n",
 	  PACKAGE);
   fprintf(stderr, "\t[ -n ] [ -N <resolving> ]\n");
   fprintf(stderr, "\t[ -o <preference setting> ... [ -P <packet list height> ]\n");
   fprintf(stderr, "\t[ -r <infile> ] [ -R <read filter> ] [ -t <time stamp format> ]\n");
-  fprintf(stderr, "\t[ -T <tree view height> ]\n");
+  fprintf(stderr, "\t[ -T <tree view height> ] [ <infile> ]\n");
 #endif
 }
 
@@ -797,7 +828,7 @@ show_version(void)
   printf("%s %s, %s\n", PACKAGE, VERSION, comp_info_str->str);
 }
 
-int
+static int
 get_positive_int(const char *string, const char *name)
 {
   long number;
@@ -822,6 +853,51 @@ get_positive_int(const char *string, const char *name)
   return number;
 }
 
+/*
+ * Given a string of the form "<autostop criterion>:<value>", as might appear
+ * as an argument to a "-a" option, parse it and set the criterion in
+ * question.  Return an indication of whether it succeeded or failed
+ * in some fashion.
+ */
+static gboolean
+set_autostop_criterion(const char *autostoparg)
+{
+  u_char *p, *colonp;
+
+  colonp = strchr(autostoparg, ':');
+  if (colonp == NULL)
+    return FALSE;
+
+  p = colonp;
+  *p++ = '\0';
+
+  /*
+   * Skip over any white space (there probably won't be any, but
+   * as we allow it in the preferences file, we might as well
+   * allow it here).
+   */
+  while (isspace(*p))
+    p++;
+  if (*p == '\0') {
+    /*
+     * Put the colon back, so if our caller uses, in an
+     * error message, the string they passed us, the message
+     * looks correct.
+     */
+    *colonp = ':';
+    return FALSE;
+  }
+  if (strcmp(autostoparg,"duration") == 0) {
+    cfile.autostop_duration = get_positive_int(p,"autostop duration");
+  } else if (strcmp(autostoparg,"filesize") == 0) {
+    cfile.autostop_filesize = get_positive_int(p,"autostop filesize");
+  } else {
+    return FALSE;
+  }
+  *colonp = ':'; /* put the colon back */
+  return TRUE;
+}
+
 /* And now our feature presentation... [ fade to music ] */
 int
 main(int argc, char *argv[])
@@ -834,20 +910,18 @@ main(int argc, char *argv[])
   int                  opt;
   extern char         *optarg;
   gboolean             arg_error = FALSE;
-
 #ifdef HAVE_LIBPCAP
-#ifdef WIN32
-  char pcap_version[] = WPCAP_STRING;
-#else
+#ifdef HAVE_PCAP_VERSION
   extern char          pcap_version[];
-#endif
-#endif
+#endif /* HAVE_PCAP_VERSION */
+#endif /* HAVE_LIBPCAP */
   
 #ifdef WIN32
   WSADATA 	       wsaData; 
 #endif
 
-  char                *gpf_path, *pf_path, *cf_path, *df_path;
+  char                *gpf_path, *cf_path, *df_path;
+  const char          *pf_path;
   int                  gpf_open_errno, pf_open_errno, cf_open_errno, df_open_errno;
   int                  err;
 #ifdef HAVE_LIBPCAP
@@ -975,6 +1049,12 @@ main(int argc, char *argv[])
   cfile.save_file_fd	= -1;
   cfile.snap		= WTAP_MAX_PACKET_SIZE;
   cfile.count		= 0;
+#ifdef HAVE_LIBPCAP
+  cfile.autostop_duration = 0;
+  cfile.autostop_filesize = 0;
+  cfile.ringbuffer_on = FALSE;
+  cfile.ringbuffer_num_files = RINGBUFFER_MIN_NUM_FILES;
+#endif
   col_init(&cfile.cinfo, prefs->num_cols);
 
   /* Assemble the compile-time options */
@@ -1000,10 +1080,14 @@ main(int argc, char *argv[])
 
 #ifdef HAVE_LIBPCAP
   g_string_append(comp_info_str, ", with libpcap ");
+#ifdef HAVE_PCAP_VERSION
   g_string_append(comp_info_str, pcap_version);
-#else
+#else /* HAVE_PCAP_VERSION */
+  g_string_append(comp_info_str, "(version unknown)");
+#endif /* HAVE_PCAP_VERSION */
+#else /* HAVE_LIBPCAP */
   g_string_append(comp_info_str, ", without libpcap");
-#endif
+#endif /* HAVE_LIBPCAP */
 
 #ifdef HAVE_LIBZ
   g_string_append(comp_info_str, ", with libz ");
@@ -1036,8 +1120,28 @@ main(int argc, char *argv[])
 #endif
 
   /* Now get our args */
-  while ((opt = getopt(argc, argv, "B:c:f:hi:klm:nN:o:pP:Qr:R:Ss:t:T:w:W:vZ:")) !=  EOF) {
+  while ((opt = getopt(argc, argv, "a:b:B:c:f:hi:klm:nN:o:pP:Qr:R:Ss:t:T:w:W:vZ:")) !=  EOF) {
     switch (opt) {
+      case 'a':        /* autostop criteria */
+#ifdef HAVE_LIBPCAP
+        if (set_autostop_criterion(optarg) == FALSE) {
+          fprintf(stderr, "ethereal: Invalid or unknown -a flag \"%s\"\n", optarg);
+          exit(1);          
+        }
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+        break;
+      case 'b':        /* Ringbuffer option */
+#ifdef HAVE_LIBPCAP
+        cfile.ringbuffer_on = TRUE;
+        cfile.ringbuffer_num_files = get_positive_int(optarg, "number of ring buffer files");
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+        break;
       case 'B':        /* Byte view pane height */
         bv_size = get_positive_int(optarg, "byte view pane height");
         break;
@@ -1109,6 +1213,7 @@ main(int argc, char *argv[])
           break;
 
         case PREFS_SET_NO_SUCH_PREF:
+        case PREFS_SET_OBSOLETE:
           fprintf(stderr, "ethereal: -o flag \"%s\" specifies unknown preference\n",
 			optarg);
           exit(1);
@@ -1227,6 +1332,59 @@ main(int argc, char *argv[])
         break;
     }
   }
+  argc -= optind;
+  argv += optind;
+  if (argc >= 1) {
+    if (cf_name != NULL) {
+      /*
+       * Input file name specified with "-r" *and* specified as a regular
+       * command-line argument.
+       */
+      arg_error = TRUE;
+    } else {
+      /*
+       * Input file name not specified with "-r", and a command-line argument
+       * was specified; treat it as the input file name.
+       *
+       * Yes, this is different from tethereal, where non-flag command-line
+       * arguments are a filter, but this works better on GUI desktops
+       * where a command can be specified to be run to open a particular
+       * file - yes, you could have "-r" as the last part of the command,
+       * but that's a bit ugly.
+       */
+      cf_name = g_strdup(argv[0]);
+    }
+    argc--;
+    argv++;
+  }
+
+  if (argc != 0) {
+    /*
+     * Extra command line arguments were specified; complain.
+     */
+    arg_error = TRUE;
+  }
+
+  if (cfile.ringbuffer_on) {
+    /* Ring buffer works only under certain conditions:
+       a) ring buffer does not work with temporary files;
+       b) prefs->capture_real_time and cfile.ringbuffer_on are mutually
+          exclusive - prefs->capture_real_time takes precedence;
+       c) it makes no sense to enable the ring buffer if the maximum
+          file size is set to "infinite". */
+    if (cfile.save_file == NULL) {
+      fprintf(stderr, "ethereal: Ring buffer requested, but capture isn't being saved to a permanent file.\n");
+      cfile.ringbuffer_on = FALSE;
+    }
+    if (prefs->capture_real_time == TRUE) {
+      fprintf(stderr, "ethereal: Ring buffer requested, but an \"Update list of packets in real time\" capture is being done.\n");
+      cfile.ringbuffer_on = FALSE;
+    }
+    if (cfile.autostop_filesize == 0) {
+      fprintf(stderr, "ethereal: Ring buffer requested, but no maximum capture file size was specified.\n");
+      cfile.ringbuffer_on = FALSE;
+    }
+  }
 
 #ifdef WIN32
   /* Load wpcap if possible */
@@ -1301,8 +1459,13 @@ main(int argc, char *argv[])
   else if (cfile.snap < MIN_PACKET_SIZE)
     cfile.snap = MIN_PACKET_SIZE;
   
-  rc_file = (gchar *) g_malloc(strlen(get_home_dir()) + strlen(RC_FILE) + 4);
-  sprintf(rc_file, "%s/%s", get_home_dir(), RC_FILE);
+  /* Check the value range of the ringbuffer_num_files parameter */
+  if (cfile.ringbuffer_num_files < RINGBUFFER_MIN_NUM_FILES)
+    cfile.ringbuffer_num_files = RINGBUFFER_MIN_NUM_FILES;
+  else if (cfile.ringbuffer_num_files > RINGBUFFER_MAX_NUM_FILES)
+    cfile.ringbuffer_num_files = RINGBUFFER_MAX_NUM_FILES;
+  
+  rc_file = get_persconffile_path(RC_FILE, FALSE);
   gtk_rc_parse(rc_file);
 
   /* Try to load the regular and boldface fixed-width fonts */
@@ -1640,11 +1803,15 @@ static void
 create_main_window (gint pl_size, gint tv_size, gint bv_size, e_prefs *prefs)
 {
   GtkWidget           *main_vbox, *menubar, *u_pane, *l_pane,
-                      *stat_hbox,
+                      *stat_hbox, *column_lb,
                       *filter_bt, *filter_cm, *filter_te,
                       *filter_reset;
   GList               *filter_list = NULL;
   GtkAccelGroup       *accel;
+  GtkStyle            *win_style;
+  GdkBitmap           *ascend_bm, *descend_bm;
+  GdkPixmap           *ascend_pm, *descend_pm;
+  column_arrows       *col_arrows;
   int			i;
   /* Display filter construct dialog has an Apply button, and "OK" not
      only sets our text widget, it activates it (i.e., it causes us to
@@ -1660,6 +1827,8 @@ create_main_window (gint pl_size, gint tv_size, gint bv_size, e_prefs *prefs)
   gtk_widget_set_name(top_level, "main window");
   gtk_signal_connect(GTK_OBJECT(top_level), "delete_event", 
     GTK_SIGNAL_FUNC(main_window_delete_event_cb), NULL);
+  gtk_signal_connect (GTK_OBJECT (top_level), "realize",
+    GTK_SIGNAL_FUNC (window_icon_realize_cb), NULL);
   gtk_window_set_title(GTK_WINDOW(top_level), "The Ethereal Network Analyzer");
   gtk_widget_set_usize(GTK_WIDGET(top_level), DEF_WIDTH, -1);
   gtk_window_set_policy(GTK_WINDOW(top_level), TRUE, TRUE, FALSE);
@@ -1695,14 +1864,17 @@ create_main_window (gint pl_size, gint tv_size, gint bv_size, e_prefs *prefs)
   gtk_widget_show(pkt_scrollw);
   gtk_paned_add1(GTK_PANED(u_pane), pkt_scrollw);
 
-  packet_list = gtk_clist_new_with_titles(cfile.cinfo.num_cols, cfile.cinfo.col_title);
+  packet_list = gtk_clist_new(cfile.cinfo.num_cols);
+  /* Column titles are filled in below */
   gtk_container_add(GTK_CONTAINER(pkt_scrollw), packet_list);
+
+  col_arrows = (column_arrows *) g_malloc(sizeof(column_arrows) * cfile.cinfo.num_cols);
   
   set_plist_sel_browse(prefs->gui_plist_sel_browse);
   set_plist_font(m_r_font);
   gtk_widget_set_name(packet_list, "packet list");
   gtk_signal_connect (GTK_OBJECT (packet_list), "click_column",
-    GTK_SIGNAL_FUNC(packet_list_click_column_cb), NULL);
+    GTK_SIGNAL_FUNC(packet_list_click_column_cb), col_arrows);
   gtk_signal_connect(GTK_OBJECT(packet_list), "select_row",
     GTK_SIGNAL_FUNC(packet_list_select_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(packet_list), "unselect_row",
@@ -1798,6 +1970,34 @@ create_main_window (gint pl_size, gint tv_size, gint bv_size, e_prefs *prefs)
   gtk_widget_show(info_bar);
 
   gtk_widget_show(top_level);
+
+  /* Fill in column titles.  This must be done after the top level window
+     is displayed. */
+  win_style = gtk_widget_get_style(top_level);
+  ascend_pm = gdk_pixmap_create_from_xpm_d(top_level->window, &ascend_bm,
+  	&win_style->bg[GTK_STATE_NORMAL], (gchar **)clist_ascend_xpm);
+  descend_pm = gdk_pixmap_create_from_xpm_d(top_level->window, &descend_bm,
+  	&win_style->bg[GTK_STATE_NORMAL], (gchar **)clist_descend_xpm);
+  for (i = 0; i < cfile.cinfo.num_cols; i++) {
+    col_arrows[i].table = gtk_table_new(2, 2, FALSE);
+    gtk_table_set_col_spacings(GTK_TABLE(col_arrows[i].table), 5);
+    column_lb = gtk_label_new(cfile.cinfo.col_title[i]);
+    gtk_table_attach(GTK_TABLE(col_arrows[i].table), column_lb, 0, 1, 0, 2,
+    	GTK_SHRINK, GTK_SHRINK, 0, 0);
+    gtk_widget_show(column_lb);
+    col_arrows[i].ascend_pm = gtk_pixmap_new(ascend_pm, ascend_bm);
+    gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].ascend_pm, 
+    	1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
+    if (i == 0) {
+      gtk_widget_show(col_arrows[i].ascend_pm);
+    }
+    col_arrows[i].descend_pm = gtk_pixmap_new(descend_pm, descend_bm);
+    gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].descend_pm,
+    	1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
+    gtk_clist_set_column_widget(GTK_CLIST(packet_list), i, col_arrows[i].table);
+    gtk_widget_show(col_arrows[i].table);
+  }
+  gtk_clist_column_titles_show(GTK_CLIST(packet_list));
 }
 
 
