@@ -3,10 +3,10 @@
  *
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet_win.c,v 1.21 2001/03/24 23:53:07 guy Exp $
+ * $Id: packet_win.c,v 1.30 2001/12/18 19:09:08 gram Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  * 
  * This program is free software; you can redistribute it and/or
@@ -28,7 +28,6 @@
  * - improve the window Title and allow user to config it
  * - Add print support ? ( could be a mess)
  * - Add button to have main window jump to this packet ?
- *
  */
 
 
@@ -58,20 +57,19 @@
 #include "column.h"
 #include "print.h"
 #include "resolv.h"
-#include "util.h"
 #include "packet_win.h"
 #include "simple_dialog.h"
 #include "proto_draw.h"
 #include "keys.h"
 #include "gtkglobals.h"
 #include "plugins.h"
+#include "epan_dissect.h"
 
 /* Data structure holding information about a packet-detail window. */
 struct PacketWinData {
 	frame_data *frame;	   /* The frame being displayed */
 	union wtap_pseudo_header pseudo_header; /* Pseudo-header for packet */
 	guint8     *pd;		   /* Data for packet */
-	proto_tree *protocol_tree; /* Protocol tree for packet */
 	GtkWidget  *main;
 	GtkWidget  *tv_scrollw;
 	GtkWidget  *tree_view;
@@ -128,15 +126,13 @@ void new_window_cb(GtkWidget *w){
 
 
 static void
-create_new_window ( char *Title, gint tv_size, gint bv_size){
-
+create_new_window(char *Title, gint tv_size, gint bv_size)
+{
   GtkWidget *main_w, *main_vbox, *pane,
                       *tree_view, *tv_scrollw,
                       *bv_scrollw,
                       *bv_nb_ptr;
   struct PacketWinData *DataPtr;
-  int i;
-  tvbuff_t* bv_tvb;
 	
   main_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
@@ -171,11 +167,9 @@ create_new_window ( char *Title, gint tv_size, gint bv_size){
   memcpy(&DataPtr->pseudo_header, &cfile.pseudo_header, sizeof DataPtr->pseudo_header);
   DataPtr->pd = g_malloc(DataPtr->frame->cap_len);
   memcpy(DataPtr->pd, cfile.pd, DataPtr->frame->cap_len);
-  DataPtr->protocol_tree = proto_tree_create_root();
-  proto_tree_is_visible = TRUE;
-  DataPtr->edt = epan_dissect_new(&DataPtr->pseudo_header, DataPtr->pd, DataPtr->frame,
-		DataPtr->protocol_tree);
-  proto_tree_is_visible = FALSE;
+  DataPtr->edt = epan_dissect_new(TRUE, TRUE);
+  epan_dissect_run(DataPtr->edt, &DataPtr->pseudo_header, DataPtr->pd,
+          DataPtr->frame, &cfile.cinfo);
   DataPtr->main = main_w;
   DataPtr->tv_scrollw = tv_scrollw;
   DataPtr->tree_view = tree_view;
@@ -194,14 +188,9 @@ create_new_window ( char *Title, gint tv_size, gint bv_size){
 			GTK_SIGNAL_FUNC(destroy_new_window), DataPtr);
 
   /* draw the protocol tree & print hex data */
-  proto_tree_draw(DataPtr->protocol_tree, tree_view);
-
-  i=0;			/* do all the hex views */
-  while((bv_tvb = g_slist_nth_data ( DataPtr->frame->data_src, i++))){
-	add_byte_tab( DataPtr->bv_nb_ptr, tvb_get_name( bv_tvb),
-		tvb_get_ptr(bv_tvb, 0, -1), tvb_length(bv_tvb));
-
-  }
+  add_byte_views(DataPtr->frame, DataPtr->edt->tree, tree_view,
+		 DataPtr->bv_nb_ptr);
+  proto_tree_draw(DataPtr->edt->tree, tree_view);
 
   DataPtr->finfo_selected = NULL;
   gtk_widget_show(main_w);
@@ -213,7 +202,6 @@ destroy_new_window(GtkObject *object, gpointer user_data)
   struct PacketWinData *DataPtr = user_data;
 
   detail_windows = g_list_remove(detail_windows, DataPtr);
-  proto_tree_free(DataPtr->protocol_tree);
   epan_dissect_free(DataPtr->edt);
   g_free(DataPtr->pd);
   g_free(DataPtr);
@@ -236,9 +224,11 @@ new_tree_view_select_row_cb(GtkCTree *ctree, GList *node, gint column,
 	finfo = gtk_ctree_node_get_row_data( ctree, GTK_CTREE_NODE(node) );
 	if (!finfo) return;
 
-        i = find_notebook_page( DataPtr->bv_nb_ptr, finfo->ds_name);
-        set_notebook_page ( DataPtr->bv_nb_ptr, i);
-        len = get_byte_view_and_data( DataPtr->bv_nb_ptr, &byte_view, &data);
+	if (finfo->ds_name != NULL) {
+		i = find_notebook_page( DataPtr->bv_nb_ptr, finfo->ds_name);
+		set_notebook_page ( DataPtr->bv_nb_ptr, i);
+	}
+	len = get_byte_view_and_data( DataPtr->bv_nb_ptr, &byte_view, &data);
 
 	if ( !byte_view)	/* exit if no hex window to write in */
 		return;
