@@ -1,7 +1,7 @@
 /* prefs_dlg.h
  * Definitions for preference handling routines
  *
- * $Id: prefs_dlg.h,v 1.2 2000/02/12 06:46:54 guy Exp $
+ * $Id: prefs_dlg.h,v 1.5 2000/08/15 20:41:59 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -26,16 +26,9 @@
 #ifndef __PREFS_DLG_H__
 #define __PREFS_DLG_H__
 
-#ifndef __PREFS_H__
 #include "prefs.h"
-#endif
-
-
-#define E_PR_PG_NONE     -1
-#define E_PR_PG_PRINTING  0
-#define E_PR_PG_FILTER    1
-#define E_PR_PG_COLUMN    2
 
 void     prefs_cb(GtkWidget *, gpointer);
+void     properties_cb(GtkWidget *, gpointer);
 
 #endif
