@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c,v 1.36 2001/07/09 22:54:15 guy Exp $
+ * $Id: proto_draw.c,v 1.42 2001/12/18 19:09:08 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -44,6 +44,8 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
+
 #include "main.h"
 #include "packet.h"
 #include "util.h"
@@ -188,8 +190,6 @@ collapse_tree(GtkCTree *ctree, GtkCTreeNode *node, gpointer user_data)
 static void
 toggle_tree(GtkCTree *ctree, GdkEventKey *event, gpointer user_data)
 {
-	GtkCTreeNode *node;
-
 	if (event->keyval != GDK_Return)
 		return;
 	gtk_ctree_toggle_expansion(ctree, GTK_CTREE_NODE(ctree->clist.selection->data));
@@ -210,6 +210,11 @@ byte_num(int offset, int start_point)
 static gint
 byte_view_select(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
+	proto_tree	*tree = gtk_object_get_data(GTK_OBJECT(widget),
+			   E_BYTE_VIEW_TREE_PTR);
+	GtkWidget	*tree_view =
+			 gtk_object_get_data(GTK_OBJECT(widget),
+			   E_BYTE_VIEW_TREE_VIEW_PTR);
 	GtkCTree	*ctree = GTK_CTREE(tree_view);
 	GtkCTreeNode	*node, *parent;
 	field_info	*finfo;
@@ -280,7 +285,7 @@ byte_view_select(GtkWidget *widget, GdkEventButton *event, gpointer data)
 	name = gtk_object_get_data(GTK_OBJECT(widget), E_BYTE_VIEW_NAME_KEY);
 
 	/* Find the finfo that corresponds to our byte. */
-	finfo = proto_find_field_from_offset(cfile.protocol_tree, byte, name);
+	finfo = proto_find_field_from_offset(tree, byte, name);
 
 	if (!finfo) {
 		return FALSE;
@@ -368,7 +373,9 @@ byte_view_realize_cb( GtkWidget *bv, gpointer data){
 }
 
 
-GtkWidget *add_byte_tab( GtkWidget *byte_nb, const char *name, const guint8 *data, int len)
+static GtkWidget *
+add_byte_tab(GtkWidget *byte_nb, const char *name, const guint8 *data, int len,
+    proto_tree *tree, GtkWidget *tree_view)
 {
   GtkWidget *byte_view, *byte_scrollw, *label;
   gchar *name_ptr;
@@ -409,6 +416,11 @@ GtkWidget *add_byte_tab( GtkWidget *byte_nb, const char *name, const guint8 *dat
 		     gtk_object_get_data(GTK_OBJECT(popup_menu_object),
 					 PM_HEXDUMP_KEY));
 
+  gtk_object_set_data(GTK_OBJECT(byte_view), E_BYTE_VIEW_TREE_PTR,
+		      tree);
+  gtk_object_set_data(GTK_OBJECT(byte_view), E_BYTE_VIEW_TREE_VIEW_PTR,
+		      tree_view);
+
   gtk_widget_show(byte_view);
 
   gtk_object_set_data(GTK_OBJECT(byte_scrollw), E_BYTE_VIEW_TEXT_INFO_KEY,
@@ -423,15 +435,30 @@ GtkWidget *add_byte_tab( GtkWidget *byte_nb, const char *name, const guint8 *dat
   return byte_view;
 }
 
+void
+add_byte_views(frame_data *frame, proto_tree *tree, GtkWidget *tree_view,
+    GtkWidget *byte_nb_ptr)
+{
+	int i;
+	tvbuff_t *bv_tvb;
 
-int add_byte_view(const char *name, const guint8 *data, int len){
+	/*
+	 * Add to the specified byte view notebook tabs for hex dumps
+	 * of all the data sources for the specified frame.
+	 */
+	for (i = 0;
+	    (bv_tvb = g_slist_nth_data(frame->data_src, i)) != NULL; i++) {
+		add_byte_tab(byte_nb_ptr, tvb_get_name(bv_tvb),
+		    tvb_get_ptr(bv_tvb, 0, -1), tvb_length(bv_tvb),
+		    tree, tree_view);
+	}
 
-	add_byte_tab( byte_nb_ptr, name, data,len);
-
-	return 0;
+	/*
+	 * Initially select the first byte view.
+	 */
+	set_notebook_page(byte_nb_ptr, 0);
 }
 
-
 void
 packet_hex_print_common(GtkText *bv, guint8 *pd, int len, int bstart, int bend, int encoding)
 {
@@ -900,19 +927,27 @@ proto_tree_draw_node(GNode *node, gpointer data)
 	struct proto_tree_draw_info	info;
 	struct proto_tree_draw_info	*parent_info = (struct proto_tree_draw_info*) data;
 
-	field_info	*fi = (field_info*) (node->data);
+	field_info	*fi = PITEM_FINFO(node);
 	gchar		label_str[ITEM_LABEL_LENGTH];
 	gchar		*label_ptr;
 	GtkCTreeNode	*parent;
 	gboolean	is_leaf, is_expanded;
-	int i;
+	int		i;
 
 	if (!fi->visible)
 		return;
-	i= find_notebook_page( byte_nb_ptr, fi->ds_name);
-	if ( i < 0)
-		return; 	/* no notebook pages ?? */
-	set_notebook_page( byte_nb_ptr, i);
+	/*
+	 * XXX - why are we doing this?  This is done when we consruct
+	 * the protocol tree display, but, as far as I can tell, it only
+	 * needs to be done when a particular field in the tree is
+	 * selected.
+	 */
+	if (fi->ds_name != NULL) {
+		i = find_notebook_page(byte_nb_ptr, fi->ds_name);
+		if (i < 0)
+			return; 	/* no notebook pages ?? */
+		set_notebook_page(byte_nb_ptr, i);
+	}
 
 	/* was a free format label produced? */
 	if (fi->representation) {
