@@ -1,10 +1,10 @@
 /* packet-afs.h
  * Definitions for packet disassembly structures and routines
  *
- * $Id: packet-afs.h,v 1.5 2001/04/17 00:46:03 guy Exp $
+ * $Id: packet-afs.h,v 1.7 2001/05/27 05:00:17 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  *
@@ -26,8 +26,6 @@
 #ifndef PACKET_AFS_H
 #define PACKET_AFS_H
 
-void dissect_afs(const u_char *, int, frame_data *, proto_tree *);
-
 #define AFS_PORT_FS	7000
 #define AFS_PORT_CB	7001
 #define AFS_PORT_PROT	7002
