@@ -1,10 +1,10 @@
 /* packet-ascend.c
  * Routines for decoding Lucent/Ascend packet traces
  *
- * $Id: packet-ascend.c,v 1.26 2001/06/18 02:17:44 guy Exp $
+ * $Id: packet-ascend.c,v 1.29 2001/12/20 05:34:50 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -61,14 +61,26 @@ dissect_ascend(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   /* load the top pane info. This should be overwritten by
      the next protocol in the stack */
-  if(check_col(pinfo->fd, COL_RES_DL_SRC))
-    col_set_str(pinfo->fd, COL_RES_DL_SRC, "N/A" );
-  if(check_col(pinfo->fd, COL_RES_DL_DST))
-    col_set_str(pinfo->fd, COL_RES_DL_DST, "N/A" );
-  if(check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "N/A" );
-  if(check_col(pinfo->fd, COL_INFO))
-    col_set_str(pinfo->fd, COL_INFO, "Lucent/Ascend packet trace" );
+  if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "N/A" );
+  if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "N/A" );
+  if(check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "N/A" );
+  if(check_col(pinfo->cinfo, COL_INFO))
+    col_set_str(pinfo->cinfo, COL_INFO, "Lucent/Ascend packet trace" );
+
+  /* If this is a transmitted or received PPP frame, set the PPP direction. */
+  switch (pseudo_header->ascend.type) {
+
+  case ASCEND_PFX_WDS_X:
+    pinfo->p2p_dir = P2P_DIR_SENT;
+    break;
+
+  case ASCEND_PFX_WDS_R:
+    pinfo->p2p_dir = P2P_DIR_RECV;
+    break;
+  }
 
   /* populate a tree in the second pane with the status of the link
      layer (ie none) */
@@ -148,10 +160,14 @@ proto_register_ascend(void)
 void
 proto_reg_handoff_ascend(void)
 {
+  dissector_handle_t ascend_handle;
+
   /*
    * Get handles for the Ethernet and PPP-in-HDLC-like-framing dissectors.
    */
   eth_handle = find_dissector("eth");
   ppp_hdlc_handle = find_dissector("ppp_hdlc");
-  dissector_add("wtap_encap", WTAP_ENCAP_ASCEND, dissect_ascend, proto_ascend);
+
+  ascend_handle = create_dissector_handle(dissect_ascend, proto_ascend);
+  dissector_add("wtap_encap", WTAP_ENCAP_ASCEND, ascend_handle);
 }
