@@ -4,12 +4,11 @@
  *
  * Heikki Vatiainen <hessu@cs.tut.fi>
  *
- * $Id: packet-auto_rp.c,v 1.14 2001/06/18 02:17:44 guy Exp $
+ * $Id: packet-auto_rp.c,v 1.16 2001/12/10 00:25:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -124,15 +123,15 @@ static void dissect_auto_rp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
         guint8 ver_type, rp_count;
 
-        if (check_col(pinfo->fd, COL_PROTOCOL))
-                col_set_str(pinfo->fd, COL_PROTOCOL, "Auto-RP");
-        if (check_col(pinfo->fd, COL_INFO))
-                col_clear(pinfo->fd, COL_INFO);
+        if (check_col(pinfo->cinfo, COL_PROTOCOL))
+                col_set_str(pinfo->cinfo, COL_PROTOCOL, "Auto-RP");
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_clear(pinfo->cinfo, COL_INFO);
         
         ver_type = tvb_get_guint8(tvb, 0);
         rp_count = tvb_get_guint8(tvb, 1);
-        if (check_col(pinfo->fd, COL_INFO))
-                col_add_fstr(pinfo->fd, COL_INFO, "%s (v%s) for %u RP%s",
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_add_fstr(pinfo->cinfo, COL_INFO, "%s (v%s) for %u RP%s",
                              val_to_str(lo_nibble(ver_type), auto_rp_type_vals, "Unknown"),
                              val_to_str(hi_nibble(ver_type), auto_rp_ver_vals, "Unknown"),
                              rp_count, plurality(rp_count, "", "s"));
@@ -243,8 +242,11 @@ void proto_register_auto_rp(void)
 void
 proto_reg_handoff_auto_rp(void)
 {
-	dissector_add("udp.port", UDP_PORT_PIM_RP_DISC, dissect_auto_rp,
+	dissector_handle_t auto_rp_handle;
+
+	auto_rp_handle = create_dissector_handle(dissect_auto_rp,
 	    proto_auto_rp);
+	dissector_add("udp.port", UDP_PORT_PIM_RP_DISC, auto_rp_handle);
 }
 
 /*
