@@ -2,7 +2,7 @@
  * Routines for BACnet (NPDU) dissection
  * Copyright 2001, Hartmut Mueller <hartmut@abmlinux.org>, FH Dortmund
  *
- * $Id: packet-bacnet.c,v 1.4 2001/06/18 02:17:44 guy Exp $
+ * $Id: packet-bacnet.c,v 1.8 2001/12/10 00:25:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -49,7 +49,8 @@
 
 #include "packet.h"
 
-static dissector_table_t bacnet_dissector_table;
+static dissector_handle_t bacapp_handle;
+static dissector_handle_t data_handle;
 
 static const char*
 bacnet_mesgtyp_name (guint8 bacnet_mesgtyp){
@@ -186,7 +187,6 @@ dissect_bacnet(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gint offset;
 	guint8 bacnet_version;
 	guint8 bacnet_control;
-	guint8 bacnet_control_net;
 	guint8 bacnet_dlen;
 	guint8 bacnet_slen;
 	guint8 bacnet_mesgtyp;
@@ -197,16 +197,15 @@ dissect_bacnet(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8 j;
 	tvbuff_t *next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL)) 
-		col_set_str(pinfo->fd, COL_PROTOCOL, "BACnet-NPDU");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "BACnet-NPDU");
 
-	if (check_col(pinfo->fd, COL_INFO)) 
-		col_set_str(pinfo->fd, COL_INFO, "Building Automation and Control Network NPDU");
+	if (check_col(pinfo->cinfo, COL_INFO)) 
+		col_set_str(pinfo->cinfo, COL_INFO, "Building Automation and Control Network NPDU");
 
 	offset = 0;
 	bacnet_version = tvb_get_guint8(tvb, offset);
 	bacnet_control = tvb_get_guint8(tvb, offset+1);
-	bacnet_control_net = tvb_get_guint8(tvb, offset+1) & BAC_CONTROL_NET;
 	bacnet_dlen = 0;
 	bacnet_slen = 0;
 	bacnet_mesgtyp = 0;
@@ -355,7 +354,7 @@ dissect_bacnet(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				tvb, offset, 2, FALSE);
 			offset += 2;
 			/* attention: doesnt work here because of if(tree) */
-			dissect_data(tvb, offset, pinfo, tree);
+			call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
 		}
 		/* Performance Index (in I-Could-Be-Router-To-Network) */
 		if (bacnet_mesgtyp == BAC_NET_ICB_R) {
@@ -423,11 +422,12 @@ dissect_bacnet(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 /* dissect BACnet APDU
  */
         next_tvb = tvb_new_subset(tvb,offset,-1,-1);
-        /* Code from Guy Harris */
-        if (!dissector_try_port(bacnet_dissector_table,
-        bacnet_control_net, next_tvb, pinfo, tree)) {
+        if (bacnet_control & BAC_CONTROL_NET) {
                 /* Unknown function - dissect the payload as data */
-                dissect_data(next_tvb, 0, pinfo, tree);
+                call_dissector(data_handle, next_tvb, pinfo, tree);
+        } else {
+        	/* APDU - call the APDU dissector */
+        	call_dissector(bacapp_handle, next_tvb, pinfo, tree);
         }
 }
 
@@ -592,14 +592,18 @@ proto_register_bacnet(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 	register_dissector("bacnet", dissect_bacnet, proto_bacnet);
-	bacnet_dissector_table = register_dissector_table("bacnet_control_net");
 }
 
 void
 proto_reg_handoff_bacnet(void)
 {
-	dissector_add("bvlc.function", 0x04, dissect_bacnet, proto_bacnet);
-	dissector_add("bvlc.function", 0x09, dissect_bacnet, proto_bacnet);
-	dissector_add("bvlc.function", 0x0a, dissect_bacnet, proto_bacnet);
-	dissector_add("bvlc.function", 0x0b, dissect_bacnet, proto_bacnet);
+	dissector_handle_t bacnet_handle;
+
+	bacnet_handle = find_dissector("bacnet");
+	dissector_add("bvlc.function", 0x04, bacnet_handle);
+	dissector_add("bvlc.function", 0x09, bacnet_handle);
+	dissector_add("bvlc.function", 0x0a, bacnet_handle);
+	dissector_add("bvlc.function", 0x0b, bacnet_handle);
+	bacapp_handle = find_dissector("bacapp");
+	data_handle = find_dissector("data");
 }
