@@ -1,8 +1,8 @@
 /* packet-bootp.c
  * Routines for BOOTP/DHCP packet disassembly
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-bootp.c,v 1.54 2001/06/18 02:17:45 guy Exp $
+ * $Id: packet-bootp.c,v 1.59 2001/12/10 00:25:26 guy Exp $
  *
  * The information used comes from:
  * RFC  951: Bootstrap Protocol
@@ -11,13 +11,13 @@
  * RFC 2132: DHCP Options and BOOTP Vendor Extensions
  * RFC 2489: Procedure for Defining New DHCP Options
  * RFC 3046: DHCP Relay Agent Information Option
+ * RFC 3118: Authentication for DHCP Messages
  * BOOTP and DHCP Parameters
- *     http://www.isi.edu/in-notes/iana/assignments/bootp-dhcp-parameters
+ *     http://www.iana.org/assignments/bootp-dhcp-parameters
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include "int-64bit.h"
 #include "packet.h"
 #include "packet-arp.h"
 
@@ -113,6 +114,16 @@ get_dhcp_type(guint8 byte)
 	return opt53_text[i];
 }
 
+/* DHCP Authentication protocols */
+#define AUTHEN_PROTO_CONFIG_TOKEN	0
+#define AUTHEN_PROTO_DELAYED_AUTHEN	1
+
+/* DHCP Authentication algorithms for delayed authentication */
+#define AUTHEN_DELAYED_ALGO_HMAC_MD5	1
+
+/* DHCP Authentication Replay Detection Methods */
+#define AUTHEN_RDM_MONOTONIC_COUNTER	0x00
+
 /* Returns the number of bytes consumed by this option. */
 static int
 bootp_option(tvbuff_t *tvb, proto_tree *bp_tree, int voff, int eoff,
@@ -128,8 +139,9 @@ bootp_option(tvbuff_t *tvb, proto_tree *bp_tree, int voff, int eoff,
 	u_long			time_secs;
 	proto_tree		*v_tree;
 	proto_item		*vti;
-	const char              *md5_ptr;
-	char                    md5_str[50];
+	guint8			protocol;
+	guint8			algorithm;
+	guint8			rdm;
 
 	static const value_string nbnt_vals[] = {
 	    {0x1,   "B-node" },
@@ -138,6 +150,19 @@ bootp_option(tvbuff_t *tvb, proto_tree *bp_tree, int voff, int eoff,
 	    {0x8,   "H-node" },
 	    {0,     NULL     } };
 
+	static const value_string authen_protocol_vals[] = {
+	    {AUTHEN_PROTO_CONFIG_TOKEN,   "configuration token" },
+	    {AUTHEN_PROTO_DELAYED_AUTHEN, "delayed authentication" },
+	    {0,                           NULL     } };
+
+	static const value_string authen_da_algo_vals[] = {
+	    {AUTHEN_DELAYED_ALGO_HMAC_MD5, "HMAC_MD5" },
+	    {0,                            NULL     } };
+
+	static const value_string authen_rdm_vals[] = {
+	    {AUTHEN_RDM_MONOTONIC_COUNTER, "Monotonically-increasing counter" },
+	    {0,                            NULL     } };
+
 	static struct opt_info opt[] = {
 		/*   0 */ { "Padding",								none },
 		/*   1 */ { "Subnet Mask",							ipv4 },
@@ -229,7 +254,7 @@ bootp_option(tvbuff_t *tvb, proto_tree *bp_tree, int voff, int eoff,
 		/*  87 */ { "Novell Directory Services Context",	opaque },
 		/*  88 */ { "IEEE 1003.1 POSIX Timezone",			opaque },
 		/*  89 */ { "Fully Qualified Domain Name",			opaque },
-		/*  90 */ { "Authentication",						opaque },
+		/*  90 */ { "Authentication",				special },
 		/*  91 */ { "Vines TCP/IP Server Option",			opaque },
 		/*  92 */ { "Server Selection Option",				opaque },
 		/*  93 */ { "Client System Architecture",			opaque },
@@ -577,44 +602,81 @@ bootp_option(tvbuff_t *tvb, proto_tree *bp_tree, int voff, int eoff,
 		}
 		break;
 
-	case 210:	/* DHCP Authentication */
+	case 90:	/* DHCP Authentication */
+	case 210:	/* Was this used for authentication at one time? */
 		vti = proto_tree_add_text(bp_tree, tvb, voff,
 			vlen + 2, "Option %d: %s", code, text);
 		v_tree = proto_item_add_subtree(vti, ett_bootp_option);
-		proto_tree_add_text(v_tree, tvb, voff+2, 1, "Protocol: %d", 
-				    tvb_get_guint8(tvb, voff+2));
-		proto_tree_add_text(v_tree, tvb, voff+3, 1, "Algorithm: %d", 
-				    tvb_get_guint8(tvb, voff+3));
-		proto_tree_add_text(v_tree, tvb, voff+4, 1, "Replay Detection Method: %d", 
-				    tvb_get_guint8(tvb, voff+4));
-		/*
-		 * XXX:
-		 *
-		 *	1) this won't compile if you have a compiler that
-		 *	   doesn't support 64-bit integral quantities;
-		 *
-		 *	2) there is no standard for the printf format to
-		 *	   be used for 64-bit integral quantities, so
-		 *	   this may not display correctly.
-		 *
-		 * We need to figure out how to handle 64-bit integral
-		 * quantities portably, with some form of fallback if
-		 * the compiler doesn't support it, and some way of
-		 * handling "%ll{d,o,x}" (most platforms) vs. "%q{d,o.x}"
-		 * (FreeBSD, perhaps some versions of other BSDs) vs.
-		 * "sorry, we're an LP64 platform, %l{d,o,x} is good enough
-		 * for you" (Digital UNIX).
-		 */
-		proto_tree_add_text(v_tree, tvb, voff+5, 8, "Replay Detection Value: %0llX", 
-				    tvb_get_ntohll(tvb, voff+5));
-		if (vlen > 11) {
-		    proto_tree_add_text(v_tree, tvb, voff+13, 4, "Secret ID: %0X", 
+
+		protocol = tvb_get_guint8(tvb, voff+2);
+		proto_tree_add_text(v_tree, tvb, voff+2, 1, "Protocol: %s (%u)", 
+				    val_to_str(protocol, authen_protocol_vals, "Unknown"),
+				    protocol);
+
+		algorithm = tvb_get_guint8(tvb, voff+3);
+		switch (protocol) {
+
+		case AUTHEN_PROTO_DELAYED_AUTHEN:
+			proto_tree_add_text(v_tree, tvb, voff+3, 1,
+				    "Algorithm: %s (%u)",
+				    val_to_str(algorithm, authen_da_algo_vals, "Unknown"),
+				    algorithm);
+			break;
+
+		default:
+			proto_tree_add_text(v_tree, tvb, voff+3, 1,
+				    "Algorithm: %u", algorithm);
+			break;
+		}
+
+		rdm = tvb_get_guint8(tvb, voff+4);
+		proto_tree_add_text(v_tree, tvb, voff+4, 1,
+				    "Replay Detection Method: %s (%u)",
+				    val_to_str(rdm, authen_rdm_vals, "Unknown"),
+				    rdm);
+
+		switch (rdm) {
+
+		case AUTHEN_RDM_MONOTONIC_COUNTER:
+			proto_tree_add_text(v_tree, tvb, voff+5, 8,
+				    "Replay Detection Value: %s", 
+				    u64toh(tvb_get_ptr(tvb, voff+5, 8)));
+			break;
+
+		default:
+			proto_tree_add_text(v_tree, tvb, voff+5, 8,
+				    "Replay Detection Value: %s", 
+				    tvb_bytes_to_str(tvb, voff+5, 8));
+			break;
+		}
+
+		switch (protocol) {
+
+		case AUTHEN_PROTO_DELAYED_AUTHEN:
+			switch (algorithm) {
+
+			case AUTHEN_DELAYED_ALGO_HMAC_MD5:
+				proto_tree_add_text(v_tree, tvb, voff+13, 4,
+					"Secret ID: 0x%08x", 
 					tvb_get_ntohl(tvb, voff+13));
-		    md5_ptr = tvb_get_ptr(tvb, voff+17, 16);
-		    for (i=0; i<16; i++)
-			sprintf(&(md5_str[i*3]), "%02X ", (guint8) md5_ptr[i]);
-		    md5_str[48] = 0;
-		    proto_tree_add_text(v_tree, tvb, voff+17, 16, "HMAC MD5 Hash: %s", md5_str);
+				proto_tree_add_text(v_tree, tvb, voff+17, 16,
+					"HMAC MD5 Hash: %s",
+					tvb_bytes_to_str(tvb, voff+17, 16));
+				break;
+
+			default:
+				proto_tree_add_text(v_tree, tvb, voff+13, vlen-11,
+					"Authentication Information: %s",
+					tvb_bytes_to_str(tvb, voff+17, vlen-11));
+				break;
+			}
+			break;
+
+		default:
+			proto_tree_add_text(v_tree, tvb, voff+13, vlen-11,
+				"Authentication Information: %s",
+				tvb_bytes_to_str(tvb, voff+17, vlen-11));
+			break;
 		}
 		break;
 
@@ -1006,33 +1068,33 @@ dissect_bootp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	const char	*dhcp_type = NULL;
 	const guint8	*vendor_class_id = NULL;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "BOOTP");
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "BOOTP");
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		/*
 		 * In case we throw an exception fetching the opcode, etc.
 		 */
-		col_clear(pinfo->fd, COL_INFO);
+		col_clear(pinfo->cinfo, COL_INFO);
 	}
 
 	op = tvb_get_guint8(tvb, 0);
 	htype = tvb_get_guint8(tvb, 1);
 	hlen = tvb_get_guint8(tvb, 2);
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		switch (op) {
 
 		case BOOTREQUEST:
-			col_add_fstr(pinfo->fd, COL_INFO, "Boot Request from %s",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Boot Request from %s",
 				arphrdaddr_to_str(tvb_get_ptr(tvb, 28, hlen),
 					hlen, htype));
 			break;
 
 		case BOOTREPLY:
-			col_set_str(pinfo->fd, COL_INFO, "Boot Reply");
+			col_set_str(pinfo->cinfo, COL_INFO, "Boot Reply");
 			break;
 
 		default:
-			col_add_fstr(pinfo->fd, COL_INFO, "Unknown BOOTP message type (%u)",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Unknown BOOTP message type (%u)",
 			    op);
 			break;
 		}
@@ -1145,10 +1207,10 @@ dissect_bootp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		 * Yes, this is a DHCP packet, and "dhcp_type" is the
 		 * packet type.
 		 */
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "DHCP");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "DHCP %-8s - Transaction ID 0x%x",
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "DHCP");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "DHCP %-8s - Transaction ID 0x%x",
 			    dhcp_type, tvb_get_ntohl(tvb, 4));
 		if (tree)
 			proto_tree_add_boolean_hidden(bp_tree, hf_bootp_dhcp,
@@ -1269,5 +1331,8 @@ proto_register_bootp(void)
 void
 proto_reg_handoff_bootp(void)
 {
-  dissector_add("udp.port", UDP_PORT_BOOTPS, dissect_bootp, proto_bootp);
+  dissector_handle_t bootp_handle;
+
+  bootp_handle = create_dissector_handle(dissect_bootp, proto_bootp);
+  dissector_add("udp.port", UDP_PORT_BOOTPS, bootp_handle);
 }
