@@ -1,14 +1,13 @@
 /* packet-bpdu.c
  * Routines for BPDU (Spanning Tree Protocol) disassembly
  *
- * $Id: packet-bpdu.c,v 1.26 2001/06/18 02:17:45 guy Exp $
+ * $Id: packet-bpdu.c,v 1.30 2001/12/10 00:25:26 guy Exp $
  *
  * Copyright 1999 Christophe Tronche <ch.tronche@computer.org>
  * 
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -77,6 +76,8 @@ static int hf_bpdu_forward_delay = -1;
 static gint ett_bpdu = -1;
 
 static dissector_handle_t gvrp_handle;
+static dissector_handle_t gmrp_handle;
+static dissector_handle_t data_handle;
 
 static void
 dissect_bpdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
@@ -120,8 +121,9 @@ dissect_bpdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	    switch (pinfo->dl_dst.data[5]) {
 
 	    case 0x20:
-		  /* Future expansion for GMRP */
-		  break;
+		   /* for GMRP */
+		  call_dissector(gmrp_handle, tvb, pinfo, tree);
+		  return;
 
 	    case 0x21:
 		  /* for GVRP */
@@ -131,13 +133,13 @@ dissect_bpdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
 	    pinfo->current_proto = "GARP";
 
-	    if (check_col(pinfo->fd, COL_PROTOCOL)) {
-		    col_set_str(pinfo->fd, COL_PROTOCOL, "GARP");
+	    if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+		    col_set_str(pinfo->cinfo, COL_PROTOCOL, "GARP");
 		    /* Generic Attribute Registration Protocol */
 	    }
 
-	    if (check_col(pinfo->fd, COL_INFO)) {
-		    col_add_fstr(pinfo->fd, COL_INFO,
+	    if (check_col(pinfo->cinfo, COL_INFO)) {
+		    col_add_fstr(pinfo->cinfo, COL_INFO,
 		        "Unknown GARP application (0x%02X)",
 			pinfo->dl_dst.data[5]);
             }
@@ -145,11 +147,11 @@ dissect_bpdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	    return;
       }
 
-      if (check_col(pinfo->fd, COL_PROTOCOL)) {
-	    col_set_str(pinfo->fd, COL_PROTOCOL, "STP"); /* Spanning Tree Protocol */
+      if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+	    col_set_str(pinfo->cinfo, COL_PROTOCOL, "STP"); /* Spanning Tree Protocol */
       }
-      if (check_col(pinfo->fd, COL_INFO)) {
-	    col_clear(pinfo->fd, COL_INFO);
+      if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_clear(pinfo->cinfo, COL_INFO);
       }
 
       bpdu_type = tvb_get_guint8(tvb, BPDU_TYPE);
@@ -171,14 +173,14 @@ dissect_bpdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	    port_identifier = 0;
       }
 
-      if (check_col(pinfo->fd, COL_INFO)) {
+      if (check_col(pinfo->cinfo, COL_INFO)) {
 	    if (bpdu_type == 0)
-		  col_add_fstr(pinfo->fd, COL_INFO, "Conf. %sRoot = %d/%s  Cost = %d  Port = 0x%04x", 
+		  col_add_fstr(pinfo->cinfo, COL_INFO, "Conf. %sRoot = %d/%s  Cost = %d  Port = 0x%04x", 
 			       flags & 0x1 ? "TC + " : "",
 			       root_identifier_bridge_priority, root_identifier_mac_str, root_path_cost,
 			       port_identifier);
 	    else if (bpdu_type == 0x80)
-		  col_add_fstr(pinfo->fd, COL_INFO, "Topology Change Notification");
+		  col_add_fstr(pinfo->cinfo, COL_INFO, "Topology Change Notification");
       }
 
       if (tree) {
@@ -211,7 +213,7 @@ dissect_bpdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 				       bpdu_type == 0x80 ? "Topology Change Notification" : "Unknown");
 
 	    if (bpdu_type != 0) {
-	      dissect_data(tvb, BPDU_TYPE + 1, pinfo, tree);
+	      call_dissector(data_handle,tvb_new_subset(tvb, BPDU_TYPE + 1,-1,tvb_reported_length_remaining(tvb,BPDU_TYPE + 1)), pinfo, tree);
 	      return;
 	    }
 
@@ -335,11 +337,20 @@ proto_register_bpdu(void)
 void
 proto_reg_handoff_bpdu(void)
 {
+  dissector_handle_t bpdu_handle;
+
   /*
    * Get handle for the GVRP dissector.
    */
   gvrp_handle = find_dissector("gvrp");
+  
+  /*
+   * Get handle for the GMRP dissector.
+   */
+  gmrp_handle = find_dissector("gmrp");
+  data_handle = find_dissector("data");
 
-  dissector_add("llc.dsap", SAP_BPDU, dissect_bpdu, proto_bpdu);
-  dissector_add("ppp.protocol", PPP_BPDU, dissect_bpdu, proto_bpdu);
+  bpdu_handle = find_dissector("bpdu");
+  dissector_add("llc.dsap", SAP_BPDU, bpdu_handle);
+  dissector_add("ppp.protocol", PPP_BPDU, bpdu_handle);
 }
