@@ -2,7 +2,7 @@
  * Routines for BACnet/IP (BVLL, BVLC) dissection
  * Copyright 2001, Hartmut Mueller <hartmut@abmlinux.org>, FH Dortmund
  *
- * $Id: packet-bvlc.c,v 1.4 2001/06/18 02:17:45 guy Exp $
+ * $Id: packet-bvlc.c,v 1.8 2001/12/10 00:25:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -67,7 +67,7 @@ static int hf_bvlc_fdt_timeout = -1;
 static int hf_bvlc_fwd_ip = -1;
 static int hf_bvlc_fwd_port = -1;
 
-
+static dissector_handle_t data_handle;
 
 static dissector_table_t bvlc_dissector_table;
 
@@ -128,11 +128,11 @@ dissect_bvlc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint16 bvlc_result;
 	tvbuff_t *next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "BVLC");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "BVLC");
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_set_str(pinfo->fd, COL_INFO, "BACnet Virtual Link Control");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_set_str(pinfo->cinfo, COL_INFO, "BACnet Virtual Link Control");
 
 	offset = 0;
 
@@ -293,7 +293,7 @@ dissect_bvlc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (!dissector_try_port(bvlc_dissector_table, 
 	bvlc_function, next_tvb, pinfo, tree)) {
 		/* Unknown function - dissect the paylod as data */
-		dissect_data(next_tvb, 0, pinfo, tree);
+		call_dissector(data_handle,next_tvb, pinfo, tree);
 	}
 }
 
@@ -389,13 +389,18 @@ proto_register_bvlc(void)
 
 	register_dissector("bvlc", dissect_bvlc, proto_bvlc);
 
-	bvlc_dissector_table = register_dissector_table("bvlc.function");
+	bvlc_dissector_table = register_dissector_table("bvlc.function",
+	    "BVLC Function", FT_UINT8, BASE_HEX);
 }
 
 void
 proto_reg_handoff_bvlc(void)
 {
-	dissector_add("udp.port", 0xBAC0, dissect_bvlc, proto_bvlc); /* added proto_bvlc */
+	dissector_handle_t bvlc_handle;
+
+	bvlc_handle = find_dissector("bvlc");
+	dissector_add("udp.port", 0xBAC0, bvlc_handle);
+	data_handle = find_dissector("data");
 }
 /* Taken from add-135a (BACnet-IP-standard paper):
  *
@@ -410,4 +415,3 @@ proto_reg_handoff_bvlc(void)
  * If you changed your BACnet port locally, use the ethereal feature
  * "Decode As".
  */
-
