@@ -1,10 +1,11 @@
 /* packet-chdlc.c
  * Routines for Cisco HDLC packet disassembly
  *
- * $Id: packet-chdlc.c,v 1.4 2001/06/18 02:17:45 guy Exp $
+ * $Id: packet-chdlc.c,v 1.9 2001/12/10 00:25:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
+ * Copyright 1998 Gerald Combs
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,6 +57,8 @@ static int hf_slarp_yoursequence = -1;
 
 static gint ett_slarp = -1;
 
+static dissector_handle_t data_handle;
+
 /*
  * Protocol types for the Cisco HDLC format.
  *
@@ -99,10 +102,14 @@ const value_string chdlc_vals[] = {
 };
 
 void
-capture_chdlc( const u_char *pd, int offset, packet_counts *ld ) {
+capture_chdlc( const u_char *pd, int offset, int len, packet_counts *ld ) {
+  if (!BYTES_ARE_IN_FRAME(offset, len, 2)) {
+    ld->other++;
+    return;
+  }
   switch (pntohs(&pd[offset + 2])) {
     case ETHERTYPE_IP:
-      capture_ip(pd, offset + 4, ld);
+      capture_ip(pd, offset + 4, len, ld);
       break;
     default:
       ld->other++;
@@ -126,9 +133,9 @@ chdlctype(guint16 chdlctype, tvbuff_t *tvb, int offset_after_chdlctype,
 
   /* do lookup with the subdissector table */
   if (!dissector_try_port(subdissector_table, chdlctype, next_tvb, pinfo, tree)) {
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-      col_add_fstr(pinfo->fd, COL_PROTOCOL, "0x%04x", chdlctype);
-    dissect_data(next_tvb, 0, pinfo, tree);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+      col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "0x%04x", chdlctype);
+    call_dissector(data_handle,next_tvb, pinfo, tree);
   }
 }
 
@@ -140,14 +147,14 @@ dissect_chdlc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint8     addr;
   guint16    proto;
 
-  if (check_col(pinfo->fd, COL_RES_DL_SRC))
-    col_set_str(pinfo->fd, COL_RES_DL_SRC, "N/A");
-  if (check_col(pinfo->fd, COL_RES_DL_DST))
-    col_set_str(pinfo->fd, COL_RES_DL_DST, "N/A");
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "CHDLC");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_RES_DL_SRC))
+    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "N/A");
+  if (check_col(pinfo->cinfo, COL_RES_DL_DST))
+    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "N/A");
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "CHDLC");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   addr = tvb_get_guint8(tvb, 0);
   proto = tvb_get_ntohs(tvb, 2);
@@ -182,7 +189,8 @@ proto_register_chdlc(void)
   proto_register_subtree_array(ett, array_length(ett));
 
 /* subdissector code */
-  subdissector_table = register_dissector_table("chdlctype");
+  subdissector_table = register_dissector_table("chdlctype",
+	"Cisco HDLC frame type", FT_UINT16, BASE_HEX);
 
   register_dissector("chdlc", dissect_chdlc, proto_chdlc);
 }
@@ -190,7 +198,11 @@ proto_register_chdlc(void)
 void
 proto_reg_handoff_chdlc(void)
 {
-  dissector_add("wtap_encap", WTAP_ENCAP_CHDLC, dissect_chdlc, proto_chdlc);
+  dissector_handle_t chdlc_handle;
+
+  data_handle = find_dissector("data");
+  chdlc_handle = find_dissector("chdlc");
+  dissector_add("wtap_encap", WTAP_ENCAP_CHDLC, chdlc_handle);
 }
 
 #define SLARP_REQUEST	0
@@ -213,10 +225,10 @@ dissect_slarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint32 mysequence;
   guint32 yoursequence;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "SLARP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "SLARP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   code = tvb_get_ntohl(tvb, 0);
 
@@ -229,8 +241,8 @@ dissect_slarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   case SLARP_REQUEST:
   case SLARP_REPLY:
-    if (check_col(pinfo->fd, COL_INFO)) {
-      col_add_fstr(pinfo->fd, COL_INFO, "%s, from %s, mask %s",
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+      col_add_fstr(pinfo->cinfo, COL_INFO, "%s, from %s, mask %s",
         match_strval(code, slarp_ptype_vals),
         get_hostname(htonl(tvb_get_ntohl(tvb, 4))),
         ip_to_str(tvb_get_ptr(tvb, 8, 4)));
@@ -246,8 +258,8 @@ dissect_slarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   case SLARP_LINECHECK:
     mysequence = tvb_get_ntohl(tvb, 4);
     yoursequence = tvb_get_ntohl(tvb, 8);
-    if (check_col(pinfo->fd, COL_INFO)) {
-      col_add_fstr(pinfo->fd, COL_INFO,
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+      col_add_fstr(pinfo->cinfo, COL_INFO,
         "%s, outgoing sequence %u, returned sequence %u",
 	match_strval(code, slarp_ptype_vals),
         mysequence, yoursequence);
@@ -262,11 +274,11 @@ dissect_slarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     break;
 
   default:
-    if (check_col(pinfo->fd, COL_INFO))
-      col_add_fstr(pinfo->fd, COL_INFO, "Unknown packet type 0x%08X", code);
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_add_fstr(pinfo->cinfo, COL_INFO, "Unknown packet type 0x%08X", code);
     if (tree) {
       proto_tree_add_uint(slarp_tree, hf_slarp_ptype, tvb, 0, 4, code);
-      dissect_data(tvb, 4, pinfo, slarp_tree);
+      call_dissector(data_handle,tvb_new_subset(tvb, 4,-1,tvb_reported_length_remaining(tvb,4)), pinfo, slarp_tree);
     }
     break;
   }
@@ -303,5 +315,8 @@ proto_register_slarp(void)
 void
 proto_reg_handoff_slarp(void)
 {
-  dissector_add("chdlctype", CISCO_SLARP, dissect_slarp, proto_slarp);
+  dissector_handle_t slarp_handle;
+
+  slarp_handle = create_dissector_handle(dissect_slarp, proto_slarp);
+  dissector_add("chdlctype", CISCO_SLARP, slarp_handle);
 }
