@@ -1,10 +1,10 @@
 /* packet-clip.c
  * Routines for clip packet disassembly
  *
- * $Id: packet-clip.c,v 1.14 2001/01/09 06:31:34 guy Exp $
+ * $Id: packet-clip.c,v 1.18 2001/12/10 00:25:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  *
  * This file created by Thierry Andry <Thierry.Andry@advalvas.be>
  * from nearly-the-same packet-raw.c created by Mike Hall <mlh@io.com>
@@ -43,9 +43,9 @@ static gint ett_clip = -1;
 static dissector_handle_t ip_handle;
 
 void
-capture_clip( const u_char *pd, packet_counts *ld ) {
+capture_clip( const u_char *pd, int len, packet_counts *ld ) {
 
-    capture_ip(pd, 0, ld);
+    capture_ip(pd, 0, len, ld);
 }
 
 static void
@@ -58,25 +58,38 @@ dissect_clip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   /* load the top pane info. This should be overwritten by
      the next protocol in the stack */
-  if(check_col(pinfo->fd, COL_RES_DL_SRC))
-    col_set_str(pinfo->fd, COL_RES_DL_SRC, "N/A" );
-  if(check_col(pinfo->fd, COL_RES_DL_DST))
-    col_set_str(pinfo->fd, COL_RES_DL_DST, "N/A" );
-  if(check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "CLIP" );
-  if(check_col(pinfo->fd, COL_INFO))
-    col_set_str(pinfo->fd, COL_INFO, "Classical IP frame" );
+  if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "N/A" );
+  if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "N/A" );
+  if(check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "CLIP" );
+  if(check_col(pinfo->cinfo, COL_INFO))
+    col_set_str(pinfo->cinfo, COL_INFO, "Classical IP frame" );
 
   /* populate a tree in the second pane with the status of the link
      layer (ie none)
 
-     XXX - the ATM on Linux code includes a patch to "tcpdump"
+     XXX - the Linux Classical IP code supports both LLC Encapsulation,
+     which puts an LLC header and possibly a SNAP header in front of
+     the network-layer header, and VC Based Multiplexing, which puts
+     no headers in front of the network-layer header.
+
+     The ATM on Linux code includes a patch to "tcpdump"
      that compares the first few bytes of the packet with the
      LLC header that Classical IP frames may have and, if there's
      a SNAP LLC header at the beginning of the packet, it gets
      the packet type from that header and uses that, otherwise
      it treats the packet as being raw IP with no link-level
-     header. */
+     header, in order to handle both of those.
+
+     This code, however, won't handle LLC Encapsulation.  We've
+     not yet seen a capture taken on a machine using LLC Encapsulation,
+     however.  If we see one, we can modify the code.
+
+     A future version of libpcap, however, will probably use DLT_LINUX_SLL
+     for both of those cases, to avoid the headache of having to
+     generate capture-filter code to handle both of those cases. */
   if(tree) {
     ti = proto_tree_add_text(tree, tvb, 0, 0, "Classical IP frame" );
     fh_tree = proto_item_add_subtree(ti, ett_clip);
@@ -98,10 +111,14 @@ proto_register_clip(void)
 void
 proto_reg_handoff_clip(void)
 {
+  dissector_handle_t clip_handle;
+
   /*
    * Get a handle for the IP dissector.
    */
   ip_handle = find_dissector("ip");
-  dissector_add("wtap_encap", WTAP_ENCAP_LINUX_ATM_CLIP, dissect_clip,
-		-1);	/* XXX */
+
+  clip_handle = create_dissector_handle(dissect_clip, -1);
+      /* XXX - no protocol, can't be disabled */
+  dissector_add("wtap_encap", WTAP_ENCAP_LINUX_ATM_CLIP, clip_handle);
 }
