@@ -1,7 +1,7 @@
 /* packet-clnp.c
  * Routines for ISO/OSI network and transport protocol packet disassembly
  *
- * $Id: packet-clnp.c,v 1.33 2001/06/18 02:17:45 guy Exp $
+ * $Id: packet-clnp.c,v 1.43 2001/12/10 00:25:26 guy Exp $
  * Laurent Deniel <deniel@worldnet.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -22,7 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -78,6 +77,8 @@ static int hf_clnp_segment_multiple_tails = -1;
 static int hf_clnp_segment_too_long_segment = -1;
 static int hf_clnp_segment_error = -1;
 
+static dissector_handle_t data_handle;
+
 /*
  * ISO 8473 OSI CLNP definition (see RFC994)
  *
@@ -532,7 +533,7 @@ static gboolean osi_decode_tp_var_part(tvbuff_t *tvb, int offset,
     case VP_TPDU_SIZE:
       c1 = tvb_get_guint8(tvb, offset) & 0x0F;
       proto_tree_add_text(tree, tvb, offset, length, 
-		"TPDU size: %u", 2 << c1);
+		"TPDU size: %u", 1 << c1);
       offset += length;
       vp_length -= length;
       break;
@@ -692,8 +693,8 @@ static int osi_decode_DR(tvbuff_t *tvb, int offset,
       break;
   }
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO,
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO,
 		"DR TPDU src-ref: 0x%04x dst-ref: 0x%04x",
 		 src_ref, dst_ref);
 
@@ -715,7 +716,7 @@ static int osi_decode_DR(tvbuff_t *tvb, int offset,
   offset += li + 1;
 
   /* User data */
-  dissect_data(tvb, offset, pinfo, tree);
+  call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
 
@@ -787,8 +788,8 @@ static int osi_decode_DT(tvbuff_t *tvb, int offset,
       break;
   }
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO, "DT TPDU (%u) dst-ref: 0x%04x %s", 
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, "DT TPDU (%u) dst-ref: 0x%04x %s", 
 		 tpdu_nr,
 		 dst_ref,
 		 (fragment)? "(fragment)" : "");
@@ -849,10 +850,10 @@ static int osi_decode_DT(tvbuff_t *tvb, int offset,
 		*subdissector_found = TRUE;
 	} else {
 	  /* Fill in other Dissectors using inactive subset here */
-	  dissect_data(next_tvb, 0, pinfo, tree);
+	  call_dissector(data_handle,next_tvb, pinfo, tree);
 	}
   } else
-	dissect_data(next_tvb, 0, pinfo, tree);
+	call_dissector(data_handle,next_tvb, pinfo, tree);
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
 
@@ -910,8 +911,8 @@ static int osi_decode_ED(tvbuff_t *tvb, int offset,
       break;
   } /* li */
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO, "ED TPDU (%u) dst-ref: 0x%04x", 
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, "ED TPDU (%u) dst-ref: 0x%04x", 
 		 tpdu_nr, dst_ref);
 
   if (tree) {
@@ -957,7 +958,7 @@ static int osi_decode_ED(tvbuff_t *tvb, int offset,
   offset += li;
 
   next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-  dissect_data(next_tvb, 0, pinfo, tree);
+  call_dissector(data_handle,next_tvb, pinfo, tree);
 
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
@@ -988,8 +989,8 @@ static int osi_decode_RJ(tvbuff_t *tvb, int offset,
       break;
   }
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO, "RJ TPDU (%u) dst-ref: 0x%04x", 
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, "RJ TPDU (%u) dst-ref: 0x%04x", 
 		 tpdu_nr, dst_ref);
 
   if (tree) {
@@ -1037,8 +1038,8 @@ static int osi_decode_CC(tvbuff_t *tvb, int offset,
   if (class_option > 4)
     return -1;
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO,
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO,
 		 "%s TPDU src-ref: 0x%04x dst-ref: 0x%04x",
 		 (tpdu == CR_TPDU) ? "CR" : "CC",
 		 src_ref,
@@ -1086,7 +1087,7 @@ static int osi_decode_CC(tvbuff_t *tvb, int offset,
   offset += li;
 
   /* User data */
-  dissect_data(tvb, offset, pinfo, tree);
+  call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
 
@@ -1106,8 +1107,8 @@ static int osi_decode_DC(tvbuff_t *tvb, int offset,
 
   src_ref = tvb_get_ntohs(tvb, offset + P_SRC_REF);
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO,
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO,
 		 "DC TPDU src-ref: 0x%04x dst-ref: 0x%04x", 
 		 src_ref,
 		 dst_ref);
@@ -1164,8 +1165,8 @@ static int osi_decode_AK(tvbuff_t *tvb, int offset,
 
     tpdu_nr = tvb_get_guint8(tvb, offset + P_TPDU_NR_234);
 
-    if (check_col(pinfo->fd, COL_INFO))
-      col_append_fstr(pinfo->fd, COL_INFO, "AK TPDU (%u) dst-ref: 0x%04x", 
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_append_fstr(pinfo->cinfo, COL_INFO, "AK TPDU (%u) dst-ref: 0x%04x", 
 		   tpdu_nr, dst_ref);
 
     if (tree) {
@@ -1208,8 +1209,8 @@ static int osi_decode_AK(tvbuff_t *tvb, int offset,
     tpdu_nr   = tvb_get_ntohl(tvb, offset + P_TPDU_NR_234);
     cdt_in_ak = tvb_get_ntohs(tvb, offset + P_CDT_IN_AK);
 
-    if (check_col(pinfo->fd, COL_INFO))
-      col_append_fstr(pinfo->fd, COL_INFO, "AK TPDU (%u) dst-ref: 0x%04x", 
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_append_fstr(pinfo->cinfo, COL_INFO, "AK TPDU (%u) dst-ref: 0x%04x", 
 		   tpdu_nr, dst_ref);
     
     if (tree) {
@@ -1302,8 +1303,8 @@ static int osi_decode_EA(tvbuff_t *tvb, int offset,
       break;
   } /* li */
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO, 
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, 
 		 "EA TPDU (%u) dst-ref: 0x%04x", tpdu_nr, dst_ref);
 
   if (tree) {
@@ -1381,8 +1382,8 @@ static int osi_decode_ER(tvbuff_t *tvb, int offset,
       break;
   }
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_fstr(pinfo->fd, COL_INFO, "ER TPDU dst-ref: 0x%04x", dst_ref);
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, "ER TPDU dst-ref: 0x%04x", dst_ref);
 
   if (tree) {
     ti = proto_tree_add_item(tree, proto_cotp, tvb, offset, li + 1, FALSE);
@@ -1411,8 +1412,8 @@ static int osi_decode_UD(tvbuff_t *tvb, int offset,
   proto_tree *cltp_tree = NULL;
   tvbuff_t   *next_tvb;
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_append_str(pinfo->fd, COL_INFO, "UD TPDU");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_str(pinfo->cinfo, COL_INFO, "UD TPDU");
 
   if (tree) {
     ti = proto_tree_add_item(tree, proto_cltp, tvb, offset, li + 1, FALSE);
@@ -1434,7 +1435,7 @@ static int osi_decode_UD(tvbuff_t *tvb, int offset,
   offset += li;
 
   next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-  dissect_data(next_tvb, 0, pinfo, tree);
+  call_dissector(data_handle,next_tvb, pinfo, tree);
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
 
@@ -1468,19 +1469,19 @@ static gboolean dissect_ositp_internal(tvbuff_t *tvb, packet_info *pinfo,
 
   /* Initialize the COL_INFO field; each of the TPDUs will have its
      information appended. */
-  if (check_col(pinfo->fd, COL_INFO))
-    col_add_str(pinfo->fd, COL_INFO, "");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_str(pinfo->cinfo, COL_INFO, "");
 
   while (tvb_offset_exists(tvb, offset)) {
     if (!first_tpdu) {
-      if (check_col(pinfo->fd, COL_INFO))
-        col_append_str(pinfo->fd, COL_INFO, ", ");
+      if (check_col(pinfo->cinfo, COL_INFO))
+        col_append_str(pinfo->cinfo, COL_INFO, ", ");
     }
     if ((li = tvb_get_guint8(tvb, offset + P_LI)) == 0) {
-      if (check_col(pinfo->fd, COL_INFO))
-        col_append_str(pinfo->fd, COL_INFO, "Length indicator is zero");
+      if (check_col(pinfo->cinfo, COL_INFO))
+        col_append_str(pinfo->cinfo, COL_INFO, "Length indicator is zero");
       if (!first_tpdu)
-        dissect_data(tvb, offset, pinfo, tree);
+        call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
       return found_ositp;
     }
 
@@ -1526,23 +1527,23 @@ static gboolean dissect_ositp_internal(tvbuff_t *tvb, packet_info *pinfo,
         is_cltp = TRUE;
         break;
       default      :
-        if (first_tpdu && check_col(pinfo->fd, COL_INFO))
-          col_append_fstr(pinfo->fd, COL_INFO, "Unknown TPDU type (0x%x)", tpdu);
+        if (first_tpdu && check_col(pinfo->cinfo, COL_INFO))
+          col_append_fstr(pinfo->cinfo, COL_INFO, "Unknown TPDU type (0x%x)", tpdu);
         new_offset = -1;	/* bad PDU type */
         break;
     }
 
     if (new_offset == -1) { /* incorrect TPDU */
       if (!first_tpdu)
-        dissect_data(tvb, offset, pinfo, tree);
+        call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
       break;
     }
 
     if (first_tpdu) {
       /* Well, we found at least one valid COTP or CLTP PDU, so I guess this
          is either COTP or CLTP. */
-      if (!subdissector_found && check_col(pinfo->fd, COL_PROTOCOL))
-        col_set_str(pinfo->fd, COL_PROTOCOL, is_cltp ? "CLTP" : "COTP");
+      if (!subdissector_found && check_col(pinfo->cinfo, COL_PROTOCOL))
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, is_cltp ? "CLTP" : "COTP");
       found_ositp = TRUE;
     }
 
@@ -1555,7 +1556,7 @@ static gboolean dissect_ositp_internal(tvbuff_t *tvb, packet_info *pinfo,
 static void dissect_ositp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) 
 {
   if (!dissect_ositp_internal(tvb, pinfo, tree, FALSE))
-    dissect_data(tvb, 0, pinfo, tree);
+    call_dissector(data_handle,tvb, pinfo, tree);
 }
 
 
@@ -1592,21 +1593,20 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   address     save_net_dst;
   address     save_src;
   address     save_dst;
+  gboolean    save_in_error_pkt;
   fragment_data *fd_head;
-  tvbuff_t   *next_tvb;
-  packet_info save_pi;
-  gboolean must_restore_pi = FALSE;
+  tvbuff_t   *volatile next_tvb;
   gboolean update_col_info = TRUE;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "CLNP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "CLNP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   cnf_proto_id = tvb_get_guint8(tvb, P_CLNP_PROTO_ID);
   if (cnf_proto_id == NLPID_NULL) {
-    if (check_col(pinfo->fd, COL_INFO))
-      col_set_str(pinfo->fd, COL_INFO, "Inactive subset");
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_set_str(pinfo->cinfo, COL_INFO, "Inactive subset");
     if (tree) {
       ti = proto_tree_add_item(tree, proto_clnp, tvb, P_CLNP_PROTO_ID, 1, FALSE);
       clnp_tree = proto_item_add_subtree(ti, ett_clnp);
@@ -1622,7 +1622,7 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   /* return if version not known */
   cnf_vers = tvb_get_guint8(tvb, P_CLNP_VERS);
   if (cnf_vers != ISO8473_V1) {
-    dissect_data(tvb, 0, pinfo, tree);
+    call_dissector(data_handle,tvb, pinfo, tree);
     return;
   }
 
@@ -1687,8 +1687,8 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      we set it otherwise. */
 
   if (!tvb_bytes_exist(tvb, 0, cnf_hdr_len)) {
-    if (check_col(pinfo->fd, COL_INFO))
-      col_add_fstr(pinfo->fd, COL_INFO, "%s NPDU %s", pdu_type_string, flag_string);
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_add_fstr(pinfo->cinfo, COL_INFO, "%s NPDU %s", pdu_type_string, flag_string);
   }
 
   segment_length = tvb_get_ntohs(tvb, P_CLNP_SEGLEN);
@@ -1811,14 +1811,6 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   /* Length of CLNP datagram plus headers above it. */
   len = segment_length;
 
-  /* Set the payload and captured-payload lengths to the minima of (the
-     datagram length plus the length of the headers above it) and the
-     frame lengths. */
-  if (pinfo->len > len)
-    pinfo->len = len;
-  if (pinfo->captured_len > len)
-    pinfo->captured_len = len;
-
   offset = cnf_hdr_len;
 
   /* For now, dissect the payload of segments other than the initial
@@ -1911,8 +1903,8 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       }
       if (fd_head->flags & (FD_OVERLAPCONFLICT
                         |FD_MULTIPLETAILS|FD_TOOLONGFRAGMENT) ) {
-        if (check_col(pinfo->fd, COL_INFO)) {
-          col_set_str(pinfo->fd, COL_INFO, "[Illegal segments]");
+        if (check_col(pinfo->cinfo, COL_INFO)) {
+          col_set_str(pinfo->cinfo, COL_INFO, "[Illegal segments]");
           update_col_info = FALSE;
         }
       }
@@ -1931,14 +1923,6 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
       /* It's not fragmented. */
       pinfo->fragmented = FALSE;
-
-      /* Save the current value of "pi", and adjust certain fields to
-         reflect the new tvbuff. */
-      save_pi = pi;
-      pi.compat_top_tvb = next_tvb;
-      pi.len = tvb_reported_length(next_tvb);
-      pi.captured_len = tvb_length(next_tvb);
-      must_restore_pi = TRUE;
     } else {
       /* We don't have the complete reassembled payload. */
       next_tvb = NULL;
@@ -1971,13 +1955,13 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   if (next_tvb == NULL) {
     /* Just show this as a segment. */
-    if (check_col(pinfo->fd, COL_INFO))
-      col_add_fstr(pinfo->fd, COL_INFO, "Fragmented %s NPDU %s(off=%u)",
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_add_fstr(pinfo->cinfo, COL_INFO, "Fragmented %s NPDU %s(off=%u)",
 		pdu_type_string, flag_string, segment_offset);
 
     /* As we haven't reassembled anything, we haven't changed "pi", so
        we don't have to restore it. */
-    dissect_data(tvb, offset, pinfo, tree);
+    call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
     return;
   }
 
@@ -2000,8 +1984,8 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       /* The payload is the header and "none, some, or all of the data
          part of the discarded PDU", i.e. it's like an ICMP error;
 	 dissect it as a CLNP PDU. */
-      if (check_col(pinfo->fd, COL_INFO))
-        col_add_fstr(pinfo->fd, COL_INFO, "%s NPDU %s", pdu_type_string, flag_string);
+      if (check_col(pinfo->cinfo, COL_INFO))
+        col_add_fstr(pinfo->cinfo, COL_INFO, "%s NPDU %s", pdu_type_string, flag_string);
       if (tree) {
         next_length = tvb_length_remaining(tvb, offset);
         if (next_length != 0) {
@@ -2009,7 +1993,7 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
              Make the columns non-writable, so the packet isn't shown
              in the summary based on what the discarded PDU's contents
              are. */
-          col_set_writable(pinfo->fd, FALSE);
+          col_set_writable(pinfo->cinfo, FALSE);
 
           /* Also, save the current values of the addresses, and restore
              them when we're finished dissecting the contained packet, so
@@ -2022,11 +2006,33 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
           save_src = pinfo->src;
           save_dst = pinfo->dst;
 
-          /* Dissect the contained packet. */
+          /* Save the current value of the "we're inside an error packet"
+             flag, and set that flag; subdissectors may treat packets
+             that are the payload of error packets differently from
+             "real" packets. */
+          save_in_error_pkt = pinfo->in_error_pkt;
+          pinfo->in_error_pkt = TRUE;
+
+          /* Dissect the contained packet.
+             Catch ReportedBoundsError, and do nothing if we see it,
+             because it's not an error if the contained packet is short;
+             there's no guarantee that all of it was included.
+
+             XXX - should catch BoundsError, and re-throw it after cleaning
+             up. */
           ti = proto_tree_add_text(clnp_tree, tvb, offset, next_length,
             "Discarded PDU");
           discpdu_tree = proto_item_add_subtree(ti, ett_clnp_disc_pdu);
-          dissect_clnp(next_tvb, pinfo, discpdu_tree);
+          TRY {
+            dissect_clnp(next_tvb, pinfo, discpdu_tree);
+          }
+          CATCH(ReportedBoundsError) {
+            ; /* do nothing */
+          }
+          ENDTRY;
+
+          /* Restore the "we're inside an error packet" flag. */
+          pinfo->in_error_pkt = save_in_error_pkt;
 
           /* Restore the addresses. */
           pinfo->dl_src = save_dl_src;
@@ -2045,9 +2051,9 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       break;
     }
   }
-  if (check_col(pinfo->fd, COL_INFO))
-    col_add_fstr(pinfo->fd, COL_INFO, "%s NPDU %s", pdu_type_string, flag_string);
-  dissect_data(next_tvb, 0, pinfo, tree);
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO, "%s NPDU %s", pdu_type_string, flag_string);
+  call_dissector(data_handle,next_tvb, pinfo, tree);
 
 } /* dissect_clnp */
 
@@ -2191,8 +2197,12 @@ void proto_register_cltp(void)
 void
 proto_reg_handoff_clnp(void)
 {
-	dissector_add("osinl", NLPID_ISO8473_CLNP, dissect_clnp,
-	    proto_clnp);
-	dissector_add("osinl", NLPID_NULL, dissect_clnp,
-	    proto_clnp);	/* Inactive subset */
+	dissector_handle_t clnp_handle;
+
+        data_handle = find_dissector("data");
+
+	clnp_handle = create_dissector_handle(dissect_clnp, proto_clnp);
+	dissector_add("osinl", NLPID_ISO8473_CLNP, clnp_handle);
+	dissector_add("osinl", NLPID_NULL, clnp_handle); /* Inactive subset */
+	dissector_add("x.25.spi", NLPID_ISO8473_CLNP, clnp_handle);
 }
