@@ -4,12 +4,11 @@
  *
  * Copyright 2000, Heikki Vatiainen <hessu@cs.tut.fi>
  *
- * $Id: packet-cops.c,v 1.13 2001/06/18 02:17:45 guy Exp $
+ * $Id: packet-cops.c,v 1.15 2001/12/10 00:25:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -266,14 +265,14 @@ static void dissect_cops(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
         guint8 op_code;
 
-        if (check_col(pinfo->fd, COL_PROTOCOL)) 
-                col_set_str(pinfo->fd, COL_PROTOCOL, "COPS");
-        if (check_col(pinfo->fd, COL_INFO)) 
-                col_clear(pinfo->fd, COL_INFO);
+        if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+                col_set_str(pinfo->cinfo, COL_PROTOCOL, "COPS");
+        if (check_col(pinfo->cinfo, COL_INFO)) 
+                col_clear(pinfo->cinfo, COL_INFO);
     
         op_code = tvb_get_guint8(tvb, 1);
-        if (check_col(pinfo->fd, COL_INFO))
-                col_add_fstr(pinfo->fd, COL_INFO, "COPS %s",
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_add_fstr(pinfo->cinfo, COL_INFO, "COPS %s",
                              val_to_str(op_code, cops_op_code_vals, "Unknown Op Code"));
 
         if (tree) {
@@ -827,5 +826,8 @@ void proto_register_cops(void)
 void
 proto_reg_handoff_cops(void)
 {
-        dissector_add("tcp.port", TCP_PORT_COPS, dissect_cops, proto_cops);
+	dissector_handle_t cops_handle;
+
+	cops_handle = create_dissector_handle(dissect_cops, proto_cops);
+        dissector_add("tcp.port", TCP_PORT_COPS, cops_handle);
 }
