@@ -1,7 +1,7 @@
 /* packet-dns.c
  * Routines for DNS packet disassembly
  *
- * $Id: packet-dns.c,v 1.70 2001/07/02 07:29:03 guy Exp $
+ * $Id: packet-dns.c,v 1.77 2001/12/10 00:25:27 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,6 +39,7 @@
 #include "ipproto.h"
 #include "resolv.h"
 #include "packet-dns.h"
+#include "prefs.h"
 
 static int proto_dns = -1;
 static int hf_dns_length = -1;
@@ -59,6 +60,9 @@ static gint ett_dns_ans = -1;
 static gint ett_dns_flags = -1;
 static gint ett_t_key_flags = -1;
 
+/* desegmentation of DNS over TCP */
+static gboolean dns_desegment = TRUE;
+
 /* DNS structs and definitions */
 
 /* Ports used for DNS. */
@@ -615,7 +619,7 @@ rfc1867_angle(tvbuff_t *tvb, int offset, const char *nsew)
 
 static int
 dissect_dns_query(tvbuff_t *tvb, int offset, int dns_data_offset,
-  frame_data *fd, proto_tree *dns_tree)
+  column_info *cinfo, proto_tree *dns_tree)
 {
   int len;
   char name[MAXDNAME];
@@ -644,8 +648,8 @@ dissect_dns_query(tvbuff_t *tvb, int offset, int dns_data_offset,
   class_name = dns_class_name(class);
   long_type_name = dns_long_type_name(type);
 
-  if (fd != NULL)
-    col_append_fstr(fd, COL_INFO, " %s %s", type_name, name);
+  if (cinfo != NULL)
+    col_append_fstr(cinfo, COL_INFO, " %s %s", type_name, name);
   if (dns_tree != NULL) {
     tq = proto_tree_add_text(dns_tree, tvb, offset, len, "%s: type %s, class %s", 
 		   name, type_name, class_name);
@@ -752,7 +756,7 @@ static const value_string cert_vals[] = {
 
 static int
 dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
-  frame_data *fd, proto_tree *dns_tree)
+  column_info *cinfo, proto_tree *dns_tree)
 {
   int len;
   char name[MAXDNAME];
@@ -790,11 +794,13 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   data_offset += 2;
   cur_offset += 2;
 
-  if (fd != NULL)
-    col_append_fstr(fd, COL_INFO, " %s", type_name);
+  if (cinfo != NULL)
+    col_append_fstr(cinfo, COL_INFO, " %s", type_name);
   if (dns_tree != NULL) {
-    trr = proto_tree_add_notext(dns_tree, tvb, offset,
-				(data_offset - data_start) + data_len);
+    trr = proto_tree_add_text(dns_tree, tvb, offset,
+				(data_offset - data_start) + data_len,
+				"%s: type %s, class %s",
+				name, type_name, class_name);
     if (type != T_OPT) {
       rr_tree = add_rr_to_tree(trr, ett_dns_rr, tvb, offset, name, name_len,
 		     long_type_name, class_name, ttl, data_len);
@@ -812,12 +818,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint32 addr_int;
 
       addr = tvb_get_ptr(tvb, cur_offset, 4);
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", ip_to_str(addr));
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", ip_to_str(addr));
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, addr %s",
-		     name, type_name, class_name,
-		     ip_to_str(addr));
+	proto_item_append_text(trr, ", addr %s", ip_to_str(addr));
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Addr: %s",
 		     ip_to_str(addr));
       }
@@ -834,11 +838,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int ns_name_len;
       
       ns_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, ns_name, sizeof(ns_name));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", ns_name);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", ns_name);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, ns %s",
-		       name, type_name, class_name, ns_name);
+	proto_item_append_text(trr, ", ns %s", ns_name);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, ns_name_len, "Name server: %s",
 			ns_name);
       }
@@ -851,11 +854,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int cname_len;
       
       cname_len = get_dns_name(tvb, cur_offset, dns_data_offset, cname, sizeof(cname));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", cname);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", cname);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, cname %s",
-		     name, type_name, class_name, cname);
+	proto_item_append_text(trr, ", cname %s", cname);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, cname_len, "Primary name: %s",
 			cname);
       }
@@ -875,11 +877,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint32 minimum;
 
       mname_len = get_dns_name(tvb, cur_offset, dns_data_offset, mname, sizeof(mname));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", mname);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", mname);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, mname %s",
-		     name, type_name, class_name, mname);
+	proto_item_append_text(trr, ", mname %s", mname);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, mname_len, "Primary name server: %s",
 		       mname);
 	cur_offset += mname_len;
@@ -922,11 +923,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int pname_len;
       
       pname_len = get_dns_name(tvb, cur_offset, dns_data_offset, pname, sizeof(pname));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", pname);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", pname);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, ptr %s",
-		     name, type_name, class_name, pname);
+	proto_item_append_text(trr, ", ptr %s", pname);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, pname_len, "Domain name: %s",
 			pname);
       }
@@ -947,12 +947,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       char portnumstring[10+1];
       
       wks_addr = tvb_get_ptr(tvb, cur_offset, 4);
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", ip_to_str(wks_addr));
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", ip_to_str(wks_addr));
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, addr %s",
-		     name, type_name, class_name,
-		     ip_to_str(wks_addr));
+	proto_item_append_text(trr, ", addr %s", ip_to_str(wks_addr));
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Addr: %s",
 		     ip_to_str(wks_addr));
 	cur_offset += 4;
@@ -1019,13 +1017,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       os_offset = cpu_offset + 1 + cpu_len;
       os_len = tvb_get_guint8(tvb, os_offset);
       os = tvb_get_ptr(tvb, os_offset + 1, os_len);
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %.*s %.*s", cpu_len, cpu,
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %.*s %.*s", cpu_len, cpu,
 	    os_len, os);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		     "%s: type %s, class %s, CPU %.*s, OS %.*s",
-		     name, type_name, class_name,
+	proto_item_append_text(trr, ", CPU %.*s, OS %.*s",
 		     cpu_len, cpu, os_len, os);
 	proto_tree_add_text(rr_tree, tvb, cpu_offset, 1 + cpu_len, "CPU: %.*s",
 			cpu_len, cpu);
@@ -1044,12 +1040,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       
       preference = tvb_get_ntohs(tvb, cur_offset);
       mx_name_len = get_dns_name(tvb, cur_offset + 2, dns_data_offset, mx_name, sizeof(mx_name));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %u %s", preference, mx_name);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %u %s", preference, mx_name);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		       "%s: type %s, class %s, preference %u, mx %s",
-		       name, type_name, class_name, preference, mx_name);
+	proto_item_append_text(trr, ", preference %u, mx %s",
+		       preference, mx_name);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Preference: %u", preference);
 	proto_tree_add_text(rr_tree, tvb, cur_offset + 2, mx_name_len, "Mail exchange: %s",
 			mx_name);
@@ -1064,9 +1059,6 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int txt_len;
 
       if (dns_tree != NULL) {
-        proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
-
 	txt_offset = cur_offset;
 	while (rr_len != 0) {
 	  txt_len = tvb_get_guint8(tvb, txt_offset);
@@ -1083,14 +1075,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
     {
       int rr_len = data_len;
       guint16 type_covered;
-      struct timeval unixtime;
+      nstime_t nstime;
       char signer_name[MAXDNAME];
       int signer_name_len;
 
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
-
 	type_covered = tvb_get_ntohs(tvb, cur_offset);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Type covered: %s (%s)",
 		dns_type_name(type_covered),
@@ -1114,17 +1103,17 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	cur_offset += 4;
 	rr_len -= 4;
 
-	unixtime.tv_sec = tvb_get_ntohl(tvb, cur_offset);
-	unixtime.tv_usec = 0;
+	nstime.secs = tvb_get_ntohl(tvb, cur_offset);
+	nstime.nsecs = 0;
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Signature expiration: %s",
-		abs_time_to_str(&unixtime));
+		abs_time_to_str(&nstime));
 	cur_offset += 4;
 	rr_len -= 4;
 
-	unixtime.tv_sec = tvb_get_ntohl(tvb, cur_offset);
-	unixtime.tv_usec = 0;
+	nstime.secs = tvb_get_ntohl(tvb, cur_offset);
+	nstime.nsecs = 0;
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Time signed: %s",
-		abs_time_to_str(&unixtime));
+		abs_time_to_str(&nstime));
 	cur_offset += 4;
 	rr_len -= 4;
 
@@ -1152,9 +1141,6 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       proto_tree *flags_tree;
 
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
-
         flags = tvb_get_ntohs(tvb, cur_offset);
 	tf = proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Flags: 0x%04X", flags);
 	flags_tree = proto_item_add_subtree(tf, ett_t_key_flags);
@@ -1220,13 +1206,12 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       const guint8 *addr6;
 
       addr6 = tvb_get_ptr(tvb, cur_offset, 16);
-      if (fd != NULL) {
-	col_append_fstr(fd, COL_INFO, " %s",
+      if (cinfo != NULL) {
+	col_append_fstr(cinfo, COL_INFO, " %s",
 			ip6_to_str((struct e_in6_addr *)addr6));
       }
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, addr %s",
-		     name, type_name, class_name,
+	proto_item_append_text(trr, ", addr %s",
 		     ip6_to_str((struct e_in6_addr *)addr6));
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 16, "Addr: %s",
 		     ip6_to_str((struct e_in6_addr *)addr6));
@@ -1267,8 +1252,8 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
         pname_len = 0;
       }
 
-      if (fd != NULL) {
-        col_append_fstr(fd, COL_INFO, " %d %s %s", 
+      if (cinfo != NULL) {
+        col_append_fstr(cinfo, COL_INFO, " %d %s %s", 
                         pre_len, 
                         ip6_to_str((struct e_in6_addr *)&suffix), 
                         pname);
@@ -1287,10 +1272,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
           proto_tree_add_text(rr_tree, tvb, a6_offset, pname_len, 
                               "Prefix name: %s", pname);
         }
-        proto_item_set_text(trr, "%s: type %s, class %s, addr %d %s %s",
-                            name, 
-                            type_name, 
-                            class_name, 
+        proto_item_append_text(trr, ", addr %d %s %s",
                             pre_len, 
                             ip6_to_str((struct e_in6_addr *)&suffix), 
                             pname);
@@ -1305,11 +1287,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       
       dname_len = get_dns_name(tvb, cur_offset, dns_data_offset, 
 			       dname, sizeof(dname));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", dname);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", dname);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, dname %s",
-			    name, type_name, class_name, dname);
+	proto_item_append_text(trr, ", dname %s", dname);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 
 			    dname_len, "Target name: %s", dname);
       }
@@ -1321,9 +1302,6 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint8 version;
 
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
-
 	version = tvb_get_guint8(tvb, cur_offset);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 1, "Version: %u", version);
 	if (version == 0) {
@@ -1371,11 +1349,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
       next_domain_name_len = get_dns_name(tvb, cur_offset, dns_data_offset,
 			next_domain_name, sizeof(next_domain_name));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", next_domain_name);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", next_domain_name);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, next domain name %s",
-		     name, type_name, class_name, next_domain_name);
+	proto_item_append_text(trr, ", next domain name %s",
+		     next_domain_name);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, next_domain_name_len,
 			"Next domain name: %s", next_domain_name);
 	cur_offset += next_domain_name_len;
@@ -1409,12 +1387,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       
       preference = tvb_get_ntohs(tvb, cur_offset);
       kx_name_len = get_dns_name(tvb, cur_offset + 2, dns_data_offset, kx_name, sizeof(kx_name));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %u %s", preference, kx_name);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %u %s", preference, kx_name);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		       "%s: type %s, class %s, preference %u, kx %s",
-		       name, type_name, class_name, preference, kx_name);
+	proto_item_append_text(trr, ", preference %u, kx %s",
+		       preference, kx_name);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Preference: %u", preference);
 	proto_tree_add_text(rr_tree, tvb, cur_offset + 2, kx_name_len, "Key exchange: %s",
 			kx_name);
@@ -1454,10 +1431,8 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
     break;
 
   case T_OPT:
-    if (dns_tree != NULL) {
-      proto_item_set_text(trr, "%s: type %s", name, type_name);
+    if (dns_tree != NULL)
       proto_tree_add_text(rr_tree, tvb, cur_offset, data_len, "Data");
-    }
     break;
 
   case T_TKEY:
@@ -1466,35 +1441,33 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int tkey_algname_len;
       guint16 tkey_mode, tkey_error, tkey_keylen, tkey_otherlen;
       int rr_len = data_len;
-      struct timeval unixtime;
+      nstime_t nstime;
       static const value_string tkey_modes[] = {
 		  { TKEYMODE_SERVERASSIGNED,   "Server assigned"   },
 		  { TKEYMODE_DIFFIEHELLMAN,    "Diffie Hellman"    },
-		  { TKEYMODE_GSSAPI,           "GSSAPI "           },
+		  { TKEYMODE_GSSAPI,           "GSSAPI"            },
 		  { TKEYMODE_RESOLVERASSIGNED, "Resolver assigned" },
 		  { TKEYMODE_DELETE,           "Delete"            },
 		  { 0,                         NULL                } };
 
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
 	tkey_algname_len = get_dns_name(tvb, cur_offset, dns_data_offset, tkey_algname, sizeof(tkey_algname));
 	proto_tree_add_text(rr_tree, tvb, cur_offset, tkey_algname_len,
 		"Algorithm name: %s", tkey_algname);
 	cur_offset += tkey_algname_len;
 	rr_len -= tkey_algname_len;
 
-	unixtime.tv_sec = tvb_get_ntohl(tvb, cur_offset);
-	unixtime.tv_usec = 0;
+	nstime.secs = tvb_get_ntohl(tvb, cur_offset);
+	nstime.nsecs = 0;
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Signature inception: %s",
-		abs_time_to_str(&unixtime));
+		abs_time_to_str(&nstime));
 	cur_offset += 4;
 	rr_len -= 4;
 
-	unixtime.tv_sec = tvb_get_ntohl(tvb, cur_offset);
-	unixtime.tv_usec = 0;
+	nstime.secs = tvb_get_ntohl(tvb, cur_offset);
+	nstime.nsecs = 0;
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Signature expiration: %s",
-		abs_time_to_str(&unixtime));
+		abs_time_to_str(&nstime));
 	cur_offset += 4;
 	rr_len -= 4;
 
@@ -1539,12 +1512,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint32 tsig_timelo;
       char tsig_algname[MAXDNAME];
       int tsig_algname_len;
-      struct timeval unixtime;
+      nstime_t nstime;
       int rr_len = data_len;
 
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
 	tsig_algname_len = get_dns_name(tvb, cur_offset, dns_data_offset, tsig_algname, sizeof(tsig_algname));
 	proto_tree_add_text(rr_tree, tvb, cur_offset, tsig_algname_len,
 		"Algorithm name: %s", tsig_algname);
@@ -1553,10 +1524,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
 	tsig_timehi = tvb_get_ntohs(tvb, cur_offset);
 	tsig_timelo = tvb_get_ntohl(tvb, cur_offset + 2);
-	unixtime.tv_sec = tsig_timelo;
-	unixtime.tv_usec = 0;
+	nstime.secs = tsig_timelo;
+	nstime.nsecs = 0;
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 6, "Time signed: %s%s",
-		abs_time_to_str(&unixtime), tsig_timehi == 0 ? "" : "(high bits set)");
+		abs_time_to_str(&nstime), tsig_timehi == 0 ? "" : "(high bits set)");
 	cur_offset += 6;
 	rr_len -= 6;
 
@@ -1607,8 +1578,6 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint32 nservers;
 
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s", name, type_name,
-				class_name);
 	local_flag = tvb_get_ntohl(tvb, cur_offset);
 	if (dns_tree != NULL) {
 	  proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Local flag: %s",
@@ -1688,11 +1657,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       rr_len -= 4;
 
       dname_len = get_dns_name(tvb, cur_offset, dns_data_offset, dname, sizeof(dname));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %s", dname);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %s", dname);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr, "%s: type %s, class %s, name result domain %s",
-		     name, type_name, class_name, dname);
+	proto_item_append_text(trr, ", name result domain %s", dname);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, dname_len, "Name result domain: %s",
 			dname);
       }
@@ -1712,12 +1680,12 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       port = tvb_get_ntohs(tvb, cur_offset+4);
 
       target_len = get_dns_name(tvb, cur_offset + 6, dns_data_offset, target, sizeof(target));
-      if (fd != NULL)
-	col_append_fstr(fd, COL_INFO, " %u %u %u %s", priority, weight, port, target);
+      if (cinfo != NULL)
+	col_append_fstr(cinfo, COL_INFO, " %u %u %u %s", priority, weight, port, target);
       if (dns_tree != NULL) {
-	proto_item_set_text(trr,
-		       "%s: type %s, class %s, priority %u, weight %u, port %u, target %s",
-		       name, type_name, class_name, priority, weight, port, target);
+	proto_item_append_text(trr,
+		       ", priority %u, weight %u, port %u, target %s",
+		       priority, weight, port, target);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Priority: %u", priority);
 	proto_tree_add_text(rr_tree, tvb, cur_offset + 2, 2, "Weight: %u", weight);
 	proto_tree_add_text(rr_tree, tvb, cur_offset + 4, 2, "Port: %u", port);
@@ -1730,11 +1698,8 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
     /* TODO: parse more record types */
 
   default:
-    if (dns_tree != NULL) {
-      proto_item_set_text(trr,
-		"%s: type %s, class %s", name, type_name, class_name);
+    if (dns_tree != NULL)
       proto_tree_add_text(rr_tree, tvb, cur_offset, data_len, "Data");
-    }
     break;
   }
   
@@ -1745,7 +1710,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
 static int
 dissect_query_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
-    int count, frame_data *fd, proto_tree *dns_tree, int isupdate)
+    int count, column_info *cinfo, proto_tree *dns_tree, int isupdate)
 {
   int start_off, add_off;
   proto_tree *qatree = NULL;
@@ -1758,7 +1723,7 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
     qatree = proto_item_add_subtree(ti, ett_dns_qry);
   }
   while (count-- > 0) {
-    add_off = dissect_dns_query(tvb, cur_off, dns_data_offset, fd, qatree);
+    add_off = dissect_dns_query(tvb, cur_off, dns_data_offset, cinfo, qatree);
     if (add_off <= 0) {
       /* We ran past the end of the captured data in the packet. */
       break;
@@ -1773,7 +1738,7 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
 
 static int
 dissect_answer_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
-    int count, frame_data *fd, proto_tree *dns_tree, char *name)
+    int count, column_info *cinfo, proto_tree *dns_tree, char *name)
 {
   int start_off, add_off;
   proto_tree *qatree = NULL;
@@ -1785,7 +1750,7 @@ dissect_answer_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
     qatree = proto_item_add_subtree(ti, ett_dns_ans);
   }
   while (count-- > 0) {
-    add_off = dissect_dns_answer(tvb, cur_off, dns_data_offset, fd, qatree);
+    add_off = dissect_dns_answer(tvb, cur_off, dns_data_offset, cinfo, qatree);
     if (add_off <= 0) {
       /* We ran past the end of the captured data in the packet. */
       break;
@@ -1798,12 +1763,12 @@ dissect_answer_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
   return cur_off - start_off;
 }
 
-static void
-dissect_dns_common(tvbuff_t *tvb, int offset, int cap_len, packet_info *pinfo,
+static int
+dissect_dns_common(tvbuff_t *tvb, int offset, int msg_len, packet_info *pinfo,
 	proto_tree *tree, gboolean is_tcp)
 {
   int dns_data_offset;
-  frame_data *fd;
+  column_info *cinfo;
   proto_tree *dns_tree = NULL, *field_tree;
   proto_item *ti, *tf;
   guint16    id, flags, quest, ans, auth, add;
@@ -1820,16 +1785,16 @@ dissect_dns_common(tvbuff_t *tvb, int offset, int cap_len, packet_info *pinfo,
 
   dns_data_offset = offset;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "DNS");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "DNS");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   /* To do: check for errs, etc. */
   id    = tvb_get_ntohs(tvb, offset + DNS_ID);
   flags = tvb_get_ntohs(tvb, offset + DNS_FLAGS);
 
-  if (check_col(pinfo->fd, COL_INFO)) {
+  if (check_col(pinfo->cinfo, COL_INFO)) {
     strcpy(buf, val_to_str(flags & F_OPCODE, opcode_vals, "Unknown operation (%x)"));
     if (flags & F_RESPONSE) {
       strcat(buf, " response");
@@ -1839,14 +1804,14 @@ dissect_dns_common(tvbuff_t *tvb, int offset, int cap_len, packet_info *pinfo,
             "Unknown error (%x)"));
       }
     }
-    col_add_str(pinfo->fd, COL_INFO, buf);
-    fd = pinfo->fd;
+    col_add_str(pinfo->cinfo, COL_INFO, buf);
+    cinfo = pinfo->cinfo;
   } else {
-    /* Set "fd" to NULL; we pass a NULL "fd" to the query and answer
+    /* Set "cinfo" to NULL; we pass a NULL "cinfo" to the query and answer
        dissectors, as a way of saying that they shouldn't add stuff
-       to the COL_INFO column (a call to "check_col(fd, COL_INFO)"
+       to the COL_INFO column (a call to "check_col(cinfo, COL_INFO)"
        is more expensive than a check that a pointer isn't NULL). */
-    fd = NULL;
+    cinfo = NULL;
   }
   if ((flags & F_OPCODE) == OPCODE_UPDATE)
     isupdate = 1;
@@ -1854,14 +1819,14 @@ dissect_dns_common(tvbuff_t *tvb, int offset, int cap_len, packet_info *pinfo,
     isupdate = 0;
 
   if (tree) {
-    ti = proto_tree_add_protocol_format(tree, proto_dns, tvb, offset, cap_len,
+    ti = proto_tree_add_protocol_format(tree, proto_dns, tvb, offset, msg_len,
       "Domain Name System (%s)", (flags & F_RESPONSE) ? "response" : "query");
     
     dns_tree = proto_item_add_subtree(ti, ett_dns);
 
     if (is_tcp) {
       /* Put the length indication into the tree. */
-      proto_tree_add_uint(dns_tree, hf_dns_length, tvb, offset - 2, 2, cap_len);
+      proto_tree_add_uint(dns_tree, hf_dns_length, tvb, offset - 2, 2, msg_len);
     }
 
     if (flags & F_RESPONSE)
@@ -1960,32 +1925,34 @@ dissect_dns_common(tvbuff_t *tvb, int offset, int cap_len, packet_info *pinfo,
     /* If this is a response, don't add information about the queries
        to the summary, just add information about the answers. */
     cur_off += dissect_query_records(tvb, cur_off, dns_data_offset, quest,
-					(!(flags & F_RESPONSE) ? fd : NULL),
-					dns_tree, isupdate);
+				     (!(flags & F_RESPONSE) ? cinfo : NULL),
+				     dns_tree, isupdate);
   }
     
   if (ans > 0) {
     /* If this is a request, don't add information about the answers
        to the summary, just add information about the queries. */
-    char *s = (isupdate ?  "Prerequisites" : "Answers");
     cur_off += dissect_answer_records(tvb, cur_off, dns_data_offset, ans,
-					((flags & F_RESPONSE) ? fd : NULL),
-					dns_tree, s);
+				      ((flags & F_RESPONSE) ? cinfo : NULL),
+				      dns_tree,
+				      (isupdate ?  "Prerequisites" : "Answers"));
   }
     
-  if (tree) {
-    /* Don't add information about the authoritative name servers, or the
-       additional records, to the summary. */
-    if (auth > 0) {
-      char *s = (isupdate ?  "Updates" : "Authoritative nameservers");
-      cur_off += dissect_answer_records(tvb, cur_off, dns_data_offset, auth,
-          NULL, dns_tree, s);
-    }
-
-    if (add > 0)
-      cur_off += dissect_answer_records(tvb, cur_off, dns_data_offset, add,
-          NULL, dns_tree, "Additional records");
+  /* Don't add information about the authoritative name servers, or the
+     additional records, to the summary. */
+  if (auth > 0) {
+    cur_off += dissect_answer_records(tvb, cur_off, dns_data_offset, auth,
+				      NULL, dns_tree,
+				      (isupdate ?  "Updates" :
+						   "Authoritative nameservers"));
   }
+
+  if (add > 0) {
+    cur_off += dissect_answer_records(tvb, cur_off, dns_data_offset, add,
+				      NULL, dns_tree, "Additional records");
+  }
+
+  return cur_off;
 }
 
 static void
@@ -2001,9 +1968,37 @@ dissect_dns_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint16 plen;
 
 	for (;;) {
+		/*
+		 * XXX - should handle a length field split across segment
+		 * boundaries.
+		 */
 		if (!tvb_bytes_exist(tvb, offset, 2))
 			break;
 		plen = tvb_get_ntohs(tvb, offset);
+
+		/*
+		 * Desegmentation check.
+		 */
+		if (dns_desegment) {
+			if (pinfo->can_desegment
+			    && plen > tvb_length_remaining(tvb, offset+2)) {
+				/*
+				 * This frame doesn't have all of the data
+				 * for this message, but we can do reassembly
+				 * on it.
+				 *
+				 * Tell the TCP dissector where the data for
+				 * this message starts in the data it handed
+				 * us, and how many more bytes we need, and
+				 * return.
+				 */
+				pinfo->desegment_offset = offset;
+				pinfo->desegment_len =
+				    plen - tvb_length_remaining(tvb, offset+2);
+				return;
+			}
+		}
+
 		offset += 2;
 
 		/*
@@ -2015,7 +2010,8 @@ dissect_dns_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		/*
 		 * Yes - dissect it.
 		 */
-		dissect_dns_common(tvb, offset, plen, pinfo, tree, TRUE);
+		offset = dissect_dns_common(tvb, offset, plen, pinfo, tree,
+		    TRUE);
 	}
 }
 
@@ -2069,15 +2065,27 @@ proto_register_dns(void)
     &ett_dns_flags,
     &ett_t_key_flags,
   };
+  module_t *dns_module;
 
   proto_dns = proto_register_protocol("Domain Name Service", "DNS", "dns");
   proto_register_field_array(proto_dns, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
+
+  dns_module = prefs_register_protocol(proto_dns, NULL);
+  prefs_register_bool_preference(dns_module, "desegment_dns_messages",
+    "Desegment all DNS messages spanning multiple TCP segments",
+    "Whether the DNS dissector should desegment all messages spanning multiple TCP segments",
+    &dns_desegment);
 }
 
 void
 proto_reg_handoff_dns(void)
 {
-  dissector_add("udp.port", UDP_PORT_DNS, dissect_dns_udp, proto_dns);
-  dissector_add("tcp.port", TCP_PORT_DNS, dissect_dns_tcp, proto_dns);
+  dissector_handle_t dns_udp_handle;
+  dissector_handle_t dns_tcp_handle;
+
+  dns_udp_handle = create_dissector_handle(dissect_dns_udp, proto_dns);
+  dns_tcp_handle = create_dissector_handle(dissect_dns_tcp, proto_dns);
+  dissector_add("udp.port", UDP_PORT_DNS, dns_udp_handle);
+  dissector_add("tcp.port", TCP_PORT_DNS, dns_tcp_handle);
 }
