@@ -2,12 +2,11 @@
  * Routines for EIGRP dissection
  * Copyright 2000, Paul Ionescu <paul@acorp.ro>
  *
- * $Id: packet-eigrp.c,v 1.17 2001/06/18 02:17:46 guy Exp $
+ * $Id: packet-eigrp.c,v 1.19 2001/12/10 00:25:27 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -149,17 +148,17 @@ dissect_eigrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
   guint16 tlv,size, offset = EIGRP_HEADER_LENGTH;
   guint32 ack;
       
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "EIGRP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "EIGRP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   opcode_tmp=opcode=tvb_get_guint8(tvb,1);
   ack = tvb_get_ntohl(tvb,12);
   if (opcode==EIGRP_HELLO) { if (ack == 0) opcode_tmp=EIGRP_HI; else opcode_tmp=EIGRP_ACK; }
   
-  if (check_col(pinfo->fd, COL_INFO))
-    col_add_str(pinfo->fd, COL_INFO,
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_str(pinfo->cinfo, COL_INFO,
 	val_to_str(opcode_tmp , eigrp_opcode_vals, "Unknown (0x%04x)"));
 
 
@@ -476,8 +475,11 @@ proto_register_eigrp(void)
 void
 proto_reg_handoff_eigrp(void)
 {
+    dissector_handle_t eigrp_handle;
+
     ipxsap_handle = find_dissector("ipxsap");
-    dissector_add("ip.proto", IP_PROTO_EIGRP, dissect_eigrp, proto_eigrp);
-    dissector_add("ddp.type", DDP_EIGRP, dissect_eigrp, proto_eigrp);
-    dissector_add("ipx.socket", IPX_SOCKET_EIGRP, dissect_eigrp, proto_eigrp);
+    eigrp_handle = create_dissector_handle(dissect_eigrp, proto_eigrp);
+    dissector_add("ip.proto", IP_PROTO_EIGRP, eigrp_handle);
+    dissector_add("ddp.type", DDP_EIGRP, eigrp_handle);
+    dissector_add("ipx.socket", IPX_SOCKET_EIGRP, eigrp_handle);
 }
