@@ -1,8 +1,8 @@
 /* packet-esis.c
  * Routines for ISO/OSI End System to Intermediate System  
- * Routeing Exchange Protocol ISO 9542.
+ * Routing Exchange Protocol ISO 9542.
  *
- * $Id: packet-esis.c,v 1.16 2001/06/18 02:17:46 guy Exp $
+ * $Id: packet-esis.c,v 1.19 2001/12/10 00:25:27 guy Exp $
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
  * Ethereal - Network traffic analyzer
@@ -22,9 +22,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
-*/
+ */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -293,10 +291,10 @@ dissect_esis(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
    u_int       tmp_uint     = 0;
    char       *cksum_status;
 
-   if (check_col(pinfo->fd, COL_PROTOCOL))
-     col_set_str(pinfo->fd, COL_PROTOCOL, "ESIS");
-   if (check_col(pinfo->fd, COL_INFO))
-     col_clear(pinfo->fd, COL_INFO);
+   if (check_col(pinfo->cinfo, COL_PROTOCOL))
+     col_set_str(pinfo->cinfo, COL_PROTOCOL, "ESIS");
+   if (check_col(pinfo->cinfo, COL_INFO))
+     col_clear(pinfo->cinfo, COL_INFO);
 
    tvb_memcpy(tvb, (guint8 *)&ehdr, 0, sizeof ehdr);
    
@@ -370,8 +368,8 @@ dissect_esis(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
     * here.  First, dump the name into info column, and THEN
     * dispatch the sub-type.
     */
-   if (check_col(pinfo->fd, COL_INFO)) {
-     col_add_str(pinfo->fd, COL_INFO, 
+   if (check_col(pinfo->cinfo, COL_INFO)) {
+     col_add_str(pinfo->cinfo, COL_INFO, 
                  val_to_str( ehdr.esis_type&OSI_PDU_TYPE_MASK, esis_vals,
                              "Unknown (0x%x)" ) );
    } 
@@ -450,5 +448,8 @@ proto_register_esis(void) {
 void
 proto_reg_handoff_esis(void)
 {
-  dissector_add("osinl", NLPID_ISO9542_ESIS, dissect_esis, proto_esis);
+  dissector_handle_t esis_handle;
+
+  esis_handle = create_dissector_handle(dissect_esis, proto_esis);
+  dissector_add("osinl", NLPID_ISO9542_ESIS, esis_handle);
 }
