@@ -1,7 +1,7 @@
 /* packet-eth.c
  * Routines for ethernet packet disassembly
  *
- * $Id: packet-eth.c,v 1.66 2001/06/29 09:42:45 guy Exp $
+ * $Id: packet-eth.c,v 1.71 2001/12/10 00:25:27 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -71,12 +71,12 @@ static dissector_handle_t isl_handle;
 #define ETHERNET_SNAP	3
 
 void
-capture_eth(const u_char *pd, int offset, packet_counts *ld)
+capture_eth(const u_char *pd, int offset, int len, packet_counts *ld)
 {
   guint16    etype, length;
   int     ethhdr_type;	/* the type of ethernet frame */
 
-  if (!BYTES_ARE_IN_FRAME(offset, ETH_HEADER_SIZE)) {
+  if (!BYTES_ARE_IN_FRAME(offset, len, ETH_HEADER_SIZE)) {
     ld->other++;
     return;
   }
@@ -106,7 +106,7 @@ capture_eth(const u_char *pd, int offset, packet_counts *ld)
        01-00-0C-00-00 for ISL frames. */
     if (pd[offset] == 0x01 && pd[offset+1] == 0x00 && pd[offset+2] == 0x0C
 	&& pd[offset+3] == 0x00 && pd[offset+4] == 0x00) {
-      capture_isl(pd, offset, ld);
+      capture_isl(pd, offset, len, ld);
       return;
     }
 
@@ -116,10 +116,8 @@ capture_eth(const u_char *pd, int offset, packet_counts *ld)
        and set the payload and captured-payload lengths to the minima
        of the total length and the frame lengths. */
     length += offset + ETH_HEADER_SIZE;
-    if (pi.len > length)
-      pi.len = length;
-    if (pi.captured_len > length)
-      pi.captured_len = length;
+    if (len > length)
+      len = length;
   } else {
     ethhdr_type = ETHERNET_II;
   }
@@ -127,13 +125,13 @@ capture_eth(const u_char *pd, int offset, packet_counts *ld)
 
   switch (ethhdr_type) {
     case ETHERNET_802_3:
-      capture_ipx(pd, offset, ld);
+      capture_ipx(pd, offset, len, ld);
       break;
     case ETHERNET_802_2:
-      capture_llc(pd, offset, ld);
+      capture_llc(pd, offset, len, ld);
       break;
     case ETHERNET_II:
-      capture_ethertype(etype, offset, pd, ld);
+      capture_ethertype(etype, pd, offset, len, ld);
       break;
   }
 }
@@ -141,23 +139,15 @@ capture_eth(const u_char *pd, int offset, packet_counts *ld)
 static void
 dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-  int        		orig_captured_len;
   proto_item		*ti;
   const guint8		*dst, *src;
-  const guint8		*pd;
 
   guint16		etype;
   volatile gboolean	is_802_2;
-  int			eth_offset;
-  volatile guint16  	length;
   proto_tree		*volatile fh_tree = NULL;
 
-  tvb_compat(tvb, &pd, (int*)&eth_offset);
-
-  orig_captured_len = pinfo->captured_len;
-
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "Ethernet");
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "Ethernet");
 
   src = tvb_get_ptr(tvb, 6, 6);
   dst = tvb_get_ptr(tvb, 0, 6);
@@ -170,8 +160,6 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	/* either ethernet802.3 or ethernet802.2 */
   if (etype <= IEEE_802_3_MAX_LEN) {
-    length = etype;
-
     /* Oh, yuck.  Cisco ISL frames require special interpretation of the
        destination address field; fortunately, they can be recognized by
        checking the first 5 octets of the destination address, which are
@@ -203,8 +191,8 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
     ENDTRY;
 
-    if (check_col(pinfo->fd, COL_INFO)) {
-      col_add_fstr(pinfo->fd, COL_INFO, "IEEE 802.3 Ethernet %s",
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+      col_add_fstr(pinfo->cinfo, COL_INFO, "IEEE 802.3 Ethernet %s",
 		(is_802_2 ? "" : "Raw "));
     }
     if (tree) {
@@ -221,25 +209,11 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       proto_tree_add_ether_hidden(fh_tree, hf_eth_addr, tvb, 6, 6, src);
     }
 
-    /* Convert the LLC length from the 802.3 header to a total
-       frame length, by adding in the size of any data that preceded
-       the Ethernet header, and adding in the Ethernet header size,
-       and set the payload and captured-payload lengths to the minima
-       of the total length and the frame lengths.
-
-       XXX - when all dissectors are tvbuffified we shouldn't have to
-       do this any more. */
-    length += eth_offset + ETH_HEADER_SIZE;
-    if (pinfo->len > length)
-      pinfo->len = length;
-    if (pinfo->captured_len > length)
-      pinfo->captured_len = length;
-
     dissect_802_3(etype, is_802_2, tvb, ETH_HEADER_SIZE, pinfo, tree, fh_tree,
 		  hf_eth_len, hf_eth_trailer);
   } else {
-    if (check_col(pinfo->fd, COL_INFO))
-      col_set_str(pinfo->fd, COL_INFO, "Ethernet II");
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_set_str(pinfo->cinfo, COL_INFO, "Ethernet II");
     if (tree) {
       ti = proto_tree_add_protocol_format(tree, proto_eth, tvb, 0, ETH_HEADER_SIZE,
 		"Ethernet II");
@@ -303,17 +277,16 @@ proto_register_eth(void)
 void
 proto_reg_handoff_eth(void)
 {
+	dissector_handle_t eth_handle;
+
 	/*
 	 * Get a handle for the ISL dissector.
 	 */
 	isl_handle = find_dissector("isl");
 
-	dissector_add("wtap_encap", WTAP_ENCAP_ETHERNET, dissect_eth,
-	    proto_eth);
-	dissector_add("ethertype", ETHERTYPE_ETHBRIDGE, dissect_eth,
-	    proto_eth);
-	dissector_add("chdlctype", ETHERTYPE_ETHBRIDGE, dissect_eth,
-	    proto_eth);
-	dissector_add("gre.proto", ETHERTYPE_ETHBRIDGE, dissect_eth,
-	    proto_eth);
+	eth_handle = find_dissector("eth");
+	dissector_add("wtap_encap", WTAP_ENCAP_ETHERNET, eth_handle);
+	dissector_add("ethertype", ETHERTYPE_ETHBRIDGE, eth_handle);
+	dissector_add("chdlctype", ETHERTYPE_ETHBRIDGE, eth_handle);
+	dissector_add("gre.proto", ETHERTYPE_ETHBRIDGE, eth_handle);
 }
