@@ -1,14 +1,13 @@
 /* ethertype.c
  * Routines for calling the right protocol for the ethertype.
  *
- * $Id: packet-ethertype.c,v 1.18 2001/06/16 00:54:33 guy Exp $
+ * $Id: packet-ethertype.c,v 1.24 2001/12/10 00:25:27 guy Exp $
  *
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +43,8 @@
 
 static dissector_table_t ethertype_dissector_table;
 
+static dissector_handle_t data_handle;
+
 const value_string etype_vals[] = {
     {ETHERTYPE_IP,		"IP"				},
     {ETHERTYPE_IPv6,		"IPv6"				},
@@ -61,6 +62,7 @@ const value_string etype_vals[] = {
     {ETHERTYPE_PPPOED,		"PPPoE Discovery"		}, 
     {ETHERTYPE_PPPOES,		"PPPoE Session"			}, 
     {ETHERTYPE_VLAN,		"802.1Q Virtual LAN"		},
+    {ETHERTYPE_EAPOL,		"802.1X Authentication"         },
     {ETHERTYPE_MPLS,		"MPLS label switched packet"	},
     {ETHERTYPE_MPLS_MULTI,	"MPLS multicast label switched packet" },
     {ETHERTYPE_3C_NBP_DGRAM,	"3Com NBP Datagram"		},
@@ -99,21 +101,21 @@ static void add_trailer(proto_tree *fh_tree, int trailer_id, tvbuff_t *tvb,
     tvbuff_t *next_tvb, int offset_after_etype, guint length_before);
 
 void
-capture_ethertype(guint16 etype, int offset,
-		const u_char *pd, packet_counts *ld)
+capture_ethertype(guint16 etype, const u_char *pd, int offset, int len,
+		  packet_counts *ld)
 {
   switch (etype) {
     case ETHERTYPE_IP:
-      capture_ip(pd, offset, ld);
+      capture_ip(pd, offset, len, ld);
       break;
     case ETHERTYPE_IPX:
-      capture_ipx(pd, offset, ld);
+      capture_ipx(pd, offset, len, ld);
       break;
     case ETHERTYPE_VLAN:
-      capture_vlan(pd, offset, ld);
+      capture_vlan(pd, offset, len, ld);
       break;
     case ETHERTYPE_VINES:
-      capture_vines(pd, offset, ld);
+      capture_vines(pd, offset, len, ld);
       break;
     default:
       ld->other++;
@@ -177,28 +179,28 @@ ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_etype,
 	if (!dissector_found) {
 		/* No sub-dissector found.
 		   Label rest of packet as "Data" */
-		dissect_data(next_tvb, 0, pinfo, tree);
+		call_dissector(data_handle,next_tvb, pinfo, tree);
 
 		/* Label protocol */
 		switch (etype) {
 
 		case ETHERTYPE_LOOP:
-			if (check_col(pinfo->fd, COL_PROTOCOL)) {
-				col_add_fstr(pinfo->fd, COL_PROTOCOL, "LOOP");
+			if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+				col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "LOOP");
 			}
 			break;
 
 		default:
-			if (check_col(pinfo->fd, COL_PROTOCOL)) {
-				col_add_fstr(pinfo->fd, COL_PROTOCOL, "0x%04x",
+			if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+				col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "0x%04x",
 				    etype);
 			}
 			break;
 		}
-		if (check_col(pinfo->fd, COL_INFO)) {
+		if (check_col(pinfo->cinfo, COL_INFO)) {
 			description = match_strval(etype, etype_vals);
 			if (description) {
-				col_add_fstr(pinfo->fd, COL_INFO, "%s",
+				col_add_fstr(pinfo->cinfo, COL_INFO, "%s",
 				    description);
 			}
 		}
@@ -261,5 +263,11 @@ void
 proto_register_ethertype(void)
 {
 	/* subdissector code */
-	ethertype_dissector_table = register_dissector_table("ethertype");
+	ethertype_dissector_table = register_dissector_table("ethertype",
+	    "Ethertype", FT_UINT16, BASE_HEX);
+}
+
+void
+proto_reg_handoff_ethertype(void){
+  data_handle = find_dissector("data");
 }
