@@ -2,12 +2,11 @@
  *
  * Top-most dissector. Decides dissector based on Wiretap Encapsulation Type.
  *
- * $Id: packet-frame.c,v 1.8 2001/06/18 02:17:46 guy Exp $
+ * $Id: packet-frame.c,v 1.16 2001/12/24 17:06:53 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 2000 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,6 +32,7 @@
 #include "timestamp.h"
 #include "tvbuff.h"
 #include "packet-frame.h"
+#include "prefs.h"
 
 static int proto_frame = -1;
 static int hf_frame_arrival_time = -1;
@@ -42,11 +42,17 @@ static int hf_frame_number = -1;
 static int hf_frame_packet_len = -1;
 static int hf_frame_capture_len = -1;
 static int hf_frame_p2p_dir = -1;
+static int hf_frame_file_off = -1;
 static int proto_short = -1;
 int proto_malformed = -1;
 
 static gint ett_frame = -1;
 
+static dissector_handle_t data_handle;
+
+/* Preferences */
+static gboolean show_file_off = FALSE;
+
 static const value_string p2p_dirs[] = {
 	{ P2P_DIR_SENT,	"Sent" },
 	{ P2P_DIR_RECV, "Received" },
@@ -60,16 +66,22 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	proto_tree	*fh_tree;
 	proto_item	*ti;
-	struct timeval	tv;
+	nstime_t	ts;
 	int		cap_len, pkt_len;
 
 	pinfo->current_proto = "Frame";
 
 	if (pinfo->fd->lnk_t == WTAP_ENCAP_LAPD ||
+			pinfo->fd->lnk_t == WTAP_ENCAP_CHDLC ||
 			pinfo->fd->lnk_t == WTAP_ENCAP_PPP_WITH_PHDR) {
 
 		pinfo->p2p_dir = pinfo->pseudo_header->p2p.sent ? P2P_DIR_SENT : P2P_DIR_RECV;
 	}
+	else if (pinfo->fd->lnk_t == WTAP_ENCAP_LAPB ||
+			pinfo->fd->lnk_t == WTAP_ENCAP_FRELAY) {
+
+		pinfo->p2p_dir = (pinfo->pseudo_header->x25.flags & 0x80) ? P2P_DIR_RECV : P2P_DIR_SENT;
+	}
 
 	/* Put in frame header information. */
 	if (tree) {
@@ -82,23 +94,23 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	  fh_tree = proto_item_add_subtree(ti, ett_frame);
 
-	  tv.tv_sec = pinfo->fd->abs_secs;
-	  tv.tv_usec = pinfo->fd->abs_usecs;
+	  ts.secs = pinfo->fd->abs_secs;
+	  ts.nsecs = pinfo->fd->abs_usecs*1000;
 
 	  proto_tree_add_time(fh_tree, hf_frame_arrival_time, tvb,
-		0, 0, &tv);
+		0, 0, &ts);
 
-	  tv.tv_sec = pinfo->fd->del_secs;
-	  tv.tv_usec = pinfo->fd->del_usecs;
+	  ts.secs = pinfo->fd->del_secs;
+	  ts.nsecs = pinfo->fd->del_usecs*1000;
 
 	  proto_tree_add_time(fh_tree, hf_frame_time_delta, tvb,
-		0, 0, &tv);
+		0, 0, &ts);
 
-	  tv.tv_sec = pinfo->fd->rel_secs;
-	  tv.tv_usec = pinfo->fd->rel_usecs;
+	  ts.secs = pinfo->fd->rel_secs;
+	  ts.nsecs = pinfo->fd->rel_usecs*1000;
 
 	  proto_tree_add_time(fh_tree, hf_frame_time_relative, tvb,
-		0, 0, &tv);
+		0, 0, &ts);
 
 	  proto_tree_add_uint(fh_tree, hf_frame_number, tvb,
 		0, 0, pinfo->fd->num);
@@ -116,6 +128,13 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		  proto_tree_add_uint(fh_tree, hf_frame_p2p_dir, tvb,
 				  0, 0, pinfo->p2p_dir);
 	  }
+
+	  if (show_file_off) {
+		  proto_tree_add_int_format(fh_tree, hf_frame_file_off, tvb,
+				  0, 0, pinfo->fd->file_off,
+				  "File Offset: %ld (0x%lx)",
+				  pinfo->fd->file_off, pinfo->fd->file_off);
+	  }
 	}
 
 
@@ -123,22 +142,23 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		if (!dissector_try_port(wtap_encap_dissector_table, pinfo->fd->lnk_t,
 					tvb, pinfo, tree)) {
 
-			if (check_col(pinfo->fd, COL_PROTOCOL))
-				col_set_str(pinfo->fd, COL_PROTOCOL, "UNKNOWN");
-			if (check_col(pinfo->fd, COL_INFO))
-				col_add_fstr(pinfo->fd, COL_INFO, "WTAP_ENCAP = 0x%x", pinfo->fd->lnk_t);
-			dissect_data(tvb, 0, pinfo, tree);
+			if (check_col(pinfo->cinfo, COL_PROTOCOL))
+				col_set_str(pinfo->cinfo, COL_PROTOCOL, "UNKNOWN");
+			if (check_col(pinfo->cinfo, COL_INFO))
+				col_add_fstr(pinfo->cinfo, COL_INFO, "WTAP_ENCAP = %u",
+				    pinfo->fd->lnk_t);
+			call_dissector(data_handle,tvb, pinfo, tree);
 		}
 	}
 	CATCH(BoundsError) {
-		if (check_col(pinfo->fd, COL_INFO))
-			col_append_str(pinfo->fd, COL_INFO, "[Short Frame]");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_append_str(pinfo->cinfo, COL_INFO, "[Short Frame]");
 		proto_tree_add_protocol_format(tree, proto_short, tvb, 0, 0,
 				"[Short Frame: %s]", pinfo->current_proto );
 	}
 	CATCH(ReportedBoundsError) {
-		if (check_col(pinfo->fd, COL_INFO))
-			col_append_str(pinfo->fd, COL_INFO, "[Malformed Frame]");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_append_str(pinfo->cinfo, COL_INFO, "[Malformed Frame]");
 		proto_tree_add_protocol_format(tree, proto_malformed, tvb, 0, 0,
 				"[Malformed Frame: %s]", pinfo->current_proto );
 	}
@@ -178,12 +198,19 @@ proto_register_frame(void)
 		{ &hf_frame_p2p_dir,
 		{ "Point-to-Point Direction",	"frame.p2p_dir", FT_UINT8, BASE_DEC, VALS(p2p_dirs), 0x0,
 			"", HFILL }},
+
+		{ &hf_frame_file_off,
+		{ "File Offset",	"frame.file_off", FT_INT32, BASE_DEC, NULL, 0x0,
+			"", HFILL }},
+
 	};
 	static gint *ett[] = {
 		&ett_frame,
 	};
+	module_t *frame_module; 
 
-	wtap_encap_dissector_table = register_dissector_table("wtap_encap");
+	wtap_encap_dissector_table = register_dissector_table("wtap_encap",
+	    "Wiretap encapsulation type", FT_UINT32, BASE_DEC);
 
 	proto_frame = proto_register_protocol("Frame", "Frame", "frame");
 	proto_register_field_array(proto_frame, hf, array_length(hf));
@@ -202,4 +229,15 @@ proto_register_frame(void)
 	   they're error indications; disabling them makes no sense. */
 	proto_set_cant_disable(proto_short);
 	proto_set_cant_disable(proto_malformed);
+
+	/* Our preferences */
+	frame_module = prefs_register_protocol(proto_frame, NULL);
+	prefs_register_bool_preference(frame_module, "show_file_off",
+	    "Show File Offset", "Show File Offset", &show_file_off);
+}
+
+void
+proto_reg_handoff_frame(void)
+{
+	data_handle = find_dissector("data");
 }
