@@ -2,7 +2,7 @@
  * Routines for gnutella dissection
  * Copyright 2001, B. Johannessen <bob@havoq.com>
  *
- * $Id: packet-gnutella.c,v 1.2 2001/06/18 02:17:46 guy Exp $
+ * $Id: packet-gnutella.c,v 1.10 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -94,11 +94,7 @@ static int hf_gnutella_push_port = -1;
 
 static gint ett_gnutella = -1;
 
-static void dissect_gnutella_pong(tvbuff_t *tvb, int offset, proto_tree *tree, int size) {
-
-	int port, ip0, ip1, ip2, ip3;
-	int files, kbytes;
-	unsigned long ip;
+static void dissect_gnutella_pong(tvbuff_t *tvb, guint offset, proto_tree *tree, guint size) {
 
 	if(offset + size > tvb_length(tvb)) {
 		proto_tree_add_item(tree,
@@ -110,54 +106,37 @@ static void dissect_gnutella_pong(tvbuff_t *tvb, int offset, proto_tree *tree, i
 		return;
 	}
 
-	port = tvb_get_letohs(tvb, offset + GNUTELLA_PONG_PORT_OFFSET);
-
-	ip  = tvb_get_letohl(tvb, offset + GNUTELLA_PONG_IP_OFFSET);
-
-	ip0 = tvb_get_guint8(tvb, offset + GNUTELLA_PONG_IP_OFFSET);
-	ip1 = tvb_get_guint8(tvb, offset + GNUTELLA_PONG_IP_OFFSET + 1);
-	ip2 = tvb_get_guint8(tvb, offset + GNUTELLA_PONG_IP_OFFSET + 2);
-	ip3 = tvb_get_guint8(tvb, offset + GNUTELLA_PONG_IP_OFFSET + 3);
-
-	files = tvb_get_letohl(tvb, offset + GNUTELLA_PONG_FILES_OFFSET);
-
-	kbytes = tvb_get_letohl(tvb, offset + GNUTELLA_PONG_KBYTES_OFFSET);
-
 	proto_tree_add_item(tree,
 		hf_gnutella_pong_port,
 		tvb,
 		offset + GNUTELLA_PONG_PORT_OFFSET,
 		GNUTELLA_PORT_LENGTH,
-		port);
+		TRUE);
 
-	proto_tree_add_ipv4_format(tree,
+	proto_tree_add_item(tree,
 		hf_gnutella_pong_ip,
 		tvb,
 		offset + GNUTELLA_PONG_IP_OFFSET,
 		GNUTELLA_IP_LENGTH,
-		ip,
-		"IP: %i.%i.%i.%i",
-		ip0, ip1, ip2, ip3);
+        FALSE);
 
 	proto_tree_add_item(tree,
 		hf_gnutella_pong_files,
 		tvb,
 		offset + GNUTELLA_PONG_FILES_OFFSET,
 		GNUTELLA_LONG_LENGTH,
-		files);
+		TRUE);
 
 	proto_tree_add_item(tree,
 		hf_gnutella_pong_kbytes,
 		tvb,
 		offset + GNUTELLA_PONG_KBYTES_OFFSET,
 		GNUTELLA_LONG_LENGTH,
-		kbytes);
+		TRUE);
 
 }
 
-static void dissect_gnutella_query(tvbuff_t *tvb, int offset, proto_tree *tree, int size) {
-
-	int min_speed;
+static void dissect_gnutella_query(tvbuff_t *tvb, guint offset, proto_tree *tree, guint size) {
 
 	if(offset + size > tvb_length(tvb)) {
 		proto_tree_add_item(tree,
@@ -169,35 +148,40 @@ static void dissect_gnutella_query(tvbuff_t *tvb, int offset, proto_tree *tree,
 		return;
 	}
 
-	min_speed = tvb_get_letohs(tvb, offset + GNUTELLA_QUERY_SPEED_OFFSET);
-
 	proto_tree_add_item(tree,
 		hf_gnutella_query_min_speed,
 		tvb,
 		offset + GNUTELLA_QUERY_SPEED_OFFSET,
 		GNUTELLA_SHORT_LENGTH,
-		min_speed);
+        TRUE);
 
-	proto_tree_add_item(tree,
-		hf_gnutella_query_search,
-		tvb,
-		offset + GNUTELLA_QUERY_SEARCH_OFFSET,
-		size - GNUTELLA_SHORT_LENGTH,
-		FALSE);
+    if (size > GNUTELLA_SHORT_LENGTH) {
+        proto_tree_add_item(tree,
+            hf_gnutella_query_search,
+            tvb,
+            offset + GNUTELLA_QUERY_SEARCH_OFFSET,
+            size - GNUTELLA_SHORT_LENGTH,
+            FALSE);
+    }
+    else {
+        proto_tree_add_text(tree,
+            tvb,
+            offset + GNUTELLA_QUERY_SEARCH_OFFSET,
+            0,
+            "Missing data for Query Search.");
+    }
 }
 
-static void dissect_gnutella_queryhit(tvbuff_t *tvb, int offset, proto_tree *tree, int size) {
+static void dissect_gnutella_queryhit(tvbuff_t *tvb, guint offset, proto_tree *tree, guint size) {
 
 	proto_tree *qhi, *hit_tree;
-	int hit_count, port, speed;
-	int ip0, ip1, ip2, ip3, i;
-	int hit_offset, idx, hit_size;
+	int hit_count, i;
+	int hit_offset;
 	int name_length, extra_length;
 	int idx_at_offset, size_at_offset;
 	int servent_id_at_offset;
 	int name_at_offset, extra_at_offset;
 	int cur_char, remaining, used;
-	unsigned long ip;
 
 	if(offset + size > tvb_length(tvb)) {
 		proto_tree_add_item(tree,
@@ -211,18 +195,7 @@ static void dissect_gnutella_queryhit(tvbuff_t *tvb, int offset, proto_tree *tre
 
 	hit_count = tvb_get_guint8(tvb, offset + GNUTELLA_QUERYHIT_COUNT_OFFSET);
 
-	port = tvb_get_ntohs(tvb, offset + GNUTELLA_QUERYHIT_PORT_OFFSET);
-
-	ip  = tvb_get_letohl(tvb, offset + GNUTELLA_QUERYHIT_IP_OFFSET);
-
-	ip0 = tvb_get_guint8(tvb, offset + GNUTELLA_QUERYHIT_IP_OFFSET);
-	ip1 = tvb_get_guint8(tvb, offset + GNUTELLA_QUERYHIT_IP_OFFSET + 1);
-	ip2 = tvb_get_guint8(tvb, offset + GNUTELLA_QUERYHIT_IP_OFFSET + 2);
-	ip3 = tvb_get_guint8(tvb, offset + GNUTELLA_QUERYHIT_IP_OFFSET + 3);
-
-	speed = tvb_get_letohl(tvb, offset + GNUTELLA_QUERYHIT_SPEED_OFFSET);
-
-	proto_tree_add_item(tree,
+	proto_tree_add_uint(tree,
 		hf_gnutella_queryhit_count,
 		tvb,
 		offset + GNUTELLA_QUERYHIT_COUNT_OFFSET,
@@ -234,30 +207,25 @@ static void dissect_gnutella_queryhit(tvbuff_t *tvb, int offset, proto_tree *tre
 		tvb,
 		offset + GNUTELLA_QUERYHIT_PORT_OFFSET,
 		GNUTELLA_PORT_LENGTH,
-		port);
+		TRUE);
 
-	proto_tree_add_ipv4_format(tree,
+	proto_tree_add_item(tree,
 		hf_gnutella_queryhit_ip,
 		tvb,
 		offset + GNUTELLA_QUERYHIT_IP_OFFSET,
 		GNUTELLA_IP_LENGTH,
-		ip,
-		"IP: %i.%i.%i.%i",
-		ip0, ip1, ip2, ip3);
+		FALSE);
 
 	proto_tree_add_item(tree,
 		hf_gnutella_queryhit_speed,
 		tvb,
 		offset + GNUTELLA_QUERYHIT_SPEED_OFFSET,
 		GNUTELLA_LONG_LENGTH,
-		speed);
+		TRUE);
 
 	hit_offset = offset + GNUTELLA_QUERYHIT_FIRST_HIT_OFFSET;
 
 	for(i = 0; i < hit_count; i++) {
-		idx  = tvb_get_letohl(tvb, hit_offset + GNUTELLA_QUERYHIT_HIT_INDEX_OFFSET);
-		hit_size = tvb_get_letohl(tvb, hit_offset + GNUTELLA_QUERYHIT_HIT_SIZE_OFFSET);
-
 		idx_at_offset  = hit_offset;
 		size_at_offset = hit_offset + GNUTELLA_QUERYHIT_HIT_SIZE_OFFSET;
 
@@ -308,14 +276,14 @@ static void dissect_gnutella_queryhit(tvbuff_t *tvb, int offset, proto_tree *tre
 			tvb,
 			idx_at_offset,
 			GNUTELLA_LONG_LENGTH,
-			idx);
+			TRUE);
 
 		proto_tree_add_item(hit_tree,
 			hf_gnutella_queryhit_hit_size,
 			tvb,
 			size_at_offset,
 			GNUTELLA_LONG_LENGTH,
-			hit_size);
+			TRUE);
 
 		proto_tree_add_item(hit_tree,
 			hf_gnutella_queryhit_hit_name,
@@ -360,10 +328,7 @@ static void dissect_gnutella_queryhit(tvbuff_t *tvb, int offset, proto_tree *tre
 
 }
 
-static void dissect_gnutella_push(tvbuff_t *tvb, int offset, proto_tree *tree, int size) {
-
-	int idx, ip0, ip1, ip2, ip3, port;
-	unsigned long ip;
+static void dissect_gnutella_push(tvbuff_t *tvb, guint offset, proto_tree *tree, guint size) {
 
 	if(offset + size > tvb_length(tvb)) {
 		proto_tree_add_item(tree,
@@ -375,14 +340,6 @@ static void dissect_gnutella_push(tvbuff_t *tvb, int offset, proto_tree *tree, i
 		return;
 	}
 
-	idx  = tvb_get_letohl(tvb, offset + GNUTELLA_PUSH_INDEX_OFFSET);
-	ip   = tvb_get_letohl(tvb, offset + GNUTELLA_PUSH_IP_OFFSET);
-	ip0  = tvb_get_guint8(tvb, offset + GNUTELLA_PUSH_IP_OFFSET);
-	ip1  = tvb_get_guint8(tvb, offset + GNUTELLA_PUSH_IP_OFFSET + 1);
-	ip2  = tvb_get_guint8(tvb, offset + GNUTELLA_PUSH_IP_OFFSET + 2);
-	ip3  = tvb_get_guint8(tvb, offset + GNUTELLA_PUSH_IP_OFFSET + 3);
-	port = tvb_get_letohs(tvb, offset + GNUTELLA_PUSH_PORT_OFFSET);
-
 	proto_tree_add_item(tree,
 		hf_gnutella_push_servent_id,
 		tvb,
@@ -395,23 +352,21 @@ static void dissect_gnutella_push(tvbuff_t *tvb, int offset, proto_tree *tree, i
 		tvb,
 		offset + GNUTELLA_PUSH_INDEX_OFFSET,
 		GNUTELLA_LONG_LENGTH,
-		idx);
+		TRUE);
 
-	proto_tree_add_ipv4_format(tree,
+	proto_tree_add_item(tree,
 		hf_gnutella_push_ip,
 		tvb,
 		offset + GNUTELLA_PUSH_IP_OFFSET,
 		GNUTELLA_IP_LENGTH,
-		ip,
-		"IP: %i.%i.%i.%i",
-		ip0, ip1, ip2, ip3);
+		FALSE);
 
 	proto_tree_add_item(tree,
 		hf_gnutella_push_port,
 		tvb,
 		offset + GNUTELLA_PUSH_PORT_OFFSET,
 		GNUTELLA_PORT_LENGTH,
-		port);
+		TRUE);
 
 }
 
@@ -421,46 +376,31 @@ static void dissect_gnutella(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
 	proto_tree *gnutella_tree, *gnutella_header_tree, *gnutella_pong_tree;
 	proto_tree *gnutella_queryhit_tree, *gnutella_push_tree;
 	proto_tree *gnutella_query_tree;
-	int snap_len, payload_descriptor, ttl, hops, offset;
+	int snap_len, payload_descriptor, offset;
 	unsigned int size;
 	char *payload_descriptor_text;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL)) 
-		col_set_str(pinfo->fd, COL_PROTOCOL, "Gnutella");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "Gnutella");
     
-	if (check_col(pinfo->fd, COL_INFO)) 
-		col_set_str(pinfo->fd, COL_INFO, "Gnutella");
+	if (check_col(pinfo->cinfo, COL_INFO)) 
+		col_set_str(pinfo->cinfo, COL_INFO, "Gnutella");
 
 	snap_len = tvb_length(tvb);
 
 	if(snap_len < GNUTELLA_HEADER_LENGTH) {
-		if (check_col(pinfo->fd, COL_INFO)) 
-			col_append_fstr(pinfo->fd, COL_INFO,
+		if (check_col(pinfo->cinfo, COL_INFO)) 
+			col_append_fstr(pinfo->cinfo, COL_INFO,
 					", %i bytes [INCOMPLETE]", snap_len);
 		return;
 	}
 	else {
-		if (check_col(pinfo->fd, COL_INFO)) 
-			col_append_fstr(pinfo->fd, COL_INFO,
+		if (check_col(pinfo->cinfo, COL_INFO)) 
+			col_append_fstr(pinfo->cinfo, COL_INFO,
 					", %i bytes", snap_len);
 	}
 
 	if (tree) {
-		offset = 0;
-
-		size = tvb_get_letohl(
-			tvb,
-			offset + GNUTELLA_HEADER_SIZE_OFFSET);
-		if(size > GNUTELLA_MAX_SNAP_SIZE) {
-			proto_tree_add_item(tree,
-				hf_gnutella_stream,
-				tvb,
-				offset,
-				snap_len,
-				FALSE);
-			return;
-		}
-
 		ti = proto_tree_add_item(tree,
 			proto_gnutella,
 			tvb,
@@ -469,17 +409,26 @@ static void dissect_gnutella(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
 			FALSE);
 		gnutella_tree = proto_item_add_subtree(ti, ett_gnutella);
 
+		offset = 0;
+
+		size = tvb_get_letohl(
+			tvb,
+			offset + GNUTELLA_HEADER_SIZE_OFFSET);
+		if(size > GNUTELLA_MAX_SNAP_SIZE) {
+			proto_tree_add_item(gnutella_tree,
+				hf_gnutella_stream,
+				tvb,
+				offset,
+				snap_len,
+				FALSE);
+			return;
+		}
+
 		while(snap_len - offset >= GNUTELLA_HEADER_LENGTH) {
 			payload_descriptor = tvb_get_guint8(
 				tvb,
 				offset +
 				GNUTELLA_HEADER_PAYLOAD_OFFSET);
-			ttl = tvb_get_guint8(
-				tvb,
-				offset + GNUTELLA_HEADER_TTL_OFFSET);
-			hops = tvb_get_guint8(
-				tvb,
-				offset + GNUTELLA_HEADER_HOPS_OFFSET);
 			size = tvb_get_letohl(
 				tvb,
 				offset + GNUTELLA_HEADER_SIZE_OFFSET);
@@ -535,92 +484,95 @@ static void dissect_gnutella(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
 				tvb,
 				offset + GNUTELLA_HEADER_TTL_OFFSET,
 				GNUTELLA_BYTE_LENGTH,
-				ttl);
+				FALSE);
 
 			proto_tree_add_item(gnutella_header_tree,
 				hf_gnutella_header_hops,
 				tvb,
 				offset + GNUTELLA_HEADER_HOPS_OFFSET,
 				GNUTELLA_BYTE_LENGTH,
-				hops);
+				FALSE);
 
-			proto_tree_add_item(gnutella_header_tree,
+			proto_tree_add_uint(gnutella_header_tree,
 				hf_gnutella_header_size,
 				tvb,
 				offset + GNUTELLA_HEADER_SIZE_OFFSET,
 				GNUTELLA_LONG_LENGTH,
 				size);
 
-			switch(payload_descriptor) {
-				case GNUTELLA_PONG:
-					pi = proto_tree_add_item(
-						gnutella_header_tree,
-						hf_gnutella_pong_payload,
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						size,
-						FALSE);
-					gnutella_pong_tree = proto_item_add_subtree(
-						pi,
-						ett_gnutella);
-					dissect_gnutella_pong(
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						gnutella_pong_tree,
-						size);
-					break;
-				case GNUTELLA_PUSH:
-					pi = proto_tree_add_item(
-						gnutella_header_tree,
-						hf_gnutella_push_payload,
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						size,
-						FALSE);
-					gnutella_push_tree = proto_item_add_subtree(
-						pi,
-						ett_gnutella);
-					dissect_gnutella_push(
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						gnutella_push_tree,
-						size);
-					break;
-				case GNUTELLA_QUERY:
-					pi = proto_tree_add_item(
-						gnutella_header_tree,
-						hf_gnutella_query_payload,
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						size,
-						FALSE);
-					gnutella_query_tree = proto_item_add_subtree(
-						pi,
-						ett_gnutella);
-					dissect_gnutella_query(
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						gnutella_query_tree,
-						size);
-					break;
-				case GNUTELLA_QUERYHIT:
-					pi = proto_tree_add_item(
-						gnutella_header_tree,
-						hf_gnutella_queryhit_payload,
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						size,
-						FALSE);
-					gnutella_queryhit_tree = proto_item_add_subtree(
-						pi,
-						ett_gnutella);
-					dissect_gnutella_queryhit(
-						tvb,
-						offset + GNUTELLA_HEADER_LENGTH,
-						gnutella_queryhit_tree,
-						size);
-					break;
-			}
+            if (size > 0) {
+
+                switch(payload_descriptor) {
+                    case GNUTELLA_PONG:
+                        pi = proto_tree_add_item(
+                            gnutella_header_tree,
+                            hf_gnutella_pong_payload,
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            size,
+                            FALSE);
+                        gnutella_pong_tree = proto_item_add_subtree(
+                            pi,
+                            ett_gnutella);
+                        dissect_gnutella_pong(
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            gnutella_pong_tree,
+                            size);
+                        break;
+                    case GNUTELLA_PUSH:
+                        pi = proto_tree_add_item(
+                            gnutella_header_tree,
+                            hf_gnutella_push_payload,
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            size,
+                            FALSE);
+                        gnutella_push_tree = proto_item_add_subtree(
+                            pi,
+                            ett_gnutella);
+                        dissect_gnutella_push(
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            gnutella_push_tree,
+                            size);
+                        break;
+                    case GNUTELLA_QUERY:
+                        pi = proto_tree_add_item(
+                            gnutella_header_tree,
+                            hf_gnutella_query_payload,
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            size,
+                            FALSE);
+                        gnutella_query_tree = proto_item_add_subtree(
+                            pi,
+                            ett_gnutella);
+                        dissect_gnutella_query(
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            gnutella_query_tree,
+                            size);
+                        break;
+                    case GNUTELLA_QUERYHIT:
+                        pi = proto_tree_add_item(
+                            gnutella_header_tree,
+                            hf_gnutella_queryhit_payload,
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            size,
+                            FALSE);
+                        gnutella_queryhit_tree = proto_item_add_subtree(
+                            pi,
+                            ett_gnutella);
+                        dissect_gnutella_queryhit(
+                            tvb,
+                            offset + GNUTELLA_HEADER_LENGTH,
+                            gnutella_queryhit_tree,
+                            size);
+                        break;
+                }
+            }
 
 			offset = offset + GNUTELLA_HEADER_LENGTH + size;
 		}
@@ -813,9 +765,9 @@ void proto_register_gnutella(void) {
 }
 
 void proto_reg_handoff_gnutella(void) {
-	dissector_add("tcp.port",
-			GNUTELLA_TCP_PORT,
-			dissect_gnutella,
-			proto_gnutella);
-}
+	dissector_handle_t gnutella_handle;
 
+	gnutella_handle = create_dissector_handle(dissect_gnutella,
+			proto_gnutella);
+	dissector_add("tcp.port", GNUTELLA_TCP_PORT, gnutella_handle);
+}
