@@ -2,12 +2,11 @@
  * Routines for the Generic Routing Encapsulation (GRE) protocol
  * Brad Robel-Forrest <brad.robel-forrest@watchguard.com>
  *
- * $Id: packet-gre.c,v 1.44 2001/06/18 02:17:46 guy Exp $
+ * $Id: packet-gre.c,v 1.49 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +42,14 @@
 #include "greproto.h"
 #include "ipproto.h"
 
+/*
+ * See RFC 1701 "Generic Routing Encapsulation (GRE)", RFC 1702
+ * "Generic Routing Encapsulation over IPv4 networks", RFC 2637
+ * "Point-to-Point Tunneling Protocol (PPTP)", RFC 2784 "Generic
+ * Routing Encapsulation (GRE)", and RFC 2890 "Key and Sequence
+ * Number Extensions to GRE".
+ */
+
 static int proto_gre = -1;
 static int hf_gre_proto = -1;
 
@@ -51,6 +58,7 @@ static gint ett_gre_flags = -1;
 static gint ett_gre_wccp2_redirect_header = -1;
 
 static dissector_table_t gre_dissector_table;
+static dissector_handle_t data_handle;
 
 /* bit positions for flags in header */
 #define GH_B_C		0x8000
@@ -95,11 +103,11 @@ dissect_gre(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   flags_and_ver = tvb_get_ntohs(tvb, offset);
   type = tvb_get_ntohs(tvb, offset + sizeof(flags_and_ver));
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "GRE");
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "GRE");
 
-  if (check_col(pinfo->fd, COL_INFO)) {
-    col_add_fstr(pinfo->fd, COL_INFO, "Encapsulated %s",
+  if (check_col(pinfo->cinfo, COL_INFO)) {
+    col_add_fstr(pinfo->cinfo, COL_INFO, "Encapsulated %s",
 		 val_to_str(type, typevals, "0x%04X (unknown)"));
   }
 
@@ -253,9 +261,21 @@ dissect_gre(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       offset += 4;
     }
   }
+
+  /* If the S bit is not set, this packet might not have a payload, so
+     check whether there's any data left, first.
+
+     XXX - the S bit isn't in RFC 2784, which deprecates that bit
+     and some other bits in RFC 1701 and says that they should be
+     zero for RFC 2784-compliant GRE; as such, the absence of the
+     S bit doesn't necessarily mean there's no payload.  */
+  if (!(flags_and_ver & GH_B_S)) {
+    if (tvb_reported_length_remaining(tvb, offset) <= 0)
+      return;	/* no payload */
+  }
   next_tvb = tvb_new_subset(tvb, offset, -1, -1);
   if (!dissector_try_port(gre_dissector_table, type, next_tvb, pinfo, tree))
-    dissect_data(next_tvb, 0, pinfo, gre_tree);
+    call_dissector(data_handle,next_tvb, pinfo, gre_tree);
 }
 
 static void
@@ -355,11 +375,16 @@ proto_register_gre(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 	/* subdissector code */
-	gre_dissector_table = register_dissector_table("gre.proto");
+	gre_dissector_table = register_dissector_table("gre.proto",
+	    "GRE protocol type", FT_UINT16, BASE_HEX);
 }
 
 void
 proto_reg_handoff_gre(void)
 {
-	dissector_add("ip.proto", IP_PROTO_GRE, dissect_gre, proto_gre);
+	dissector_handle_t gre_handle;
+
+	gre_handle = create_dissector_handle(dissect_gre, proto_gre);
+	dissector_add("ip.proto", IP_PROTO_GRE, gre_handle);
+	data_handle = find_dissector("data");
 }
