@@ -3,12 +3,11 @@
  *
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-http.c,v 1.38 2001/06/18 02:17:46 guy Exp $
+ * $Id: packet-http.c,v 1.44 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,8 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,6 +55,9 @@ static int hf_http_request = -1;
 
 static gint ett_http = -1;
 
+static dissector_handle_t data_handle;
+static dissector_handle_t http_handle;
+
 #define TCP_PORT_HTTP			80
 #define TCP_PORT_PROXY_HTTP		3128
 #define TCP_PORT_PROXY_ADMIN_HTTP	3132
@@ -80,6 +80,7 @@ typedef enum {
 static int is_http_request_or_reply(const u_char *data, int linelen, http_type_t *type);
 
 static dissector_table_t subdissector_table;
+static heur_dissector_list_t heur_subdissector_list;
 
 static void
 dissect_http(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
@@ -110,9 +111,9 @@ dissect_http(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		break;
 	}
 	
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, proto_tag);
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, proto_tag);
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		/*
 		 * Put the first line from the buffer into the summary
 		 * if it's an HTTP request or reply (but leave out the
@@ -127,10 +128,10 @@ dissect_http(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		line = tvb_get_ptr(tvb, offset, linelen);
 		http_type = HTTP_OTHERS;	/* type not known yet */
 		if (is_http_request_or_reply(line, linelen, &http_type))
-			col_add_str(pinfo->fd, COL_INFO,
+			col_add_str(pinfo->cinfo, COL_INFO,
 			    format_text(line, linelen));
 		else
-			col_set_str(pinfo->fd, COL_INFO, "Continuation");
+			col_set_str(pinfo->cinfo, COL_INFO, "Continuation");
 	}
 
 	if (tree) {
@@ -271,8 +272,16 @@ dissect_http(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			 */
 			if (ti != NULL)
 				proto_item_set_len(ti, offset);
+		} else if(dissector_try_heuristic(heur_subdissector_list,
+						  next_tvb,pinfo,tree)){
+			/*
+			 * Yes.  Fix up the top-level item so that it
+			 * doesn't include the stuff for that protocol.
+			 */
+			if (ti != NULL)
+				proto_item_set_len(ti, offset);
 		} else
-			dissect_data(tvb, offset, pinfo, http_tree);
+			call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, http_tree);
 	}
 }
 
@@ -393,6 +402,9 @@ proto_register_http(void)
 	proto_register_field_array(proto_http, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
+	register_dissector("http", dissect_http, proto_http);
+	http_handle = find_dissector("http");
+
 	/*
 	 * Dissectors shouldn't register themselves in this table;
 	 * instead, they should call "http_dissector_add()", and
@@ -402,41 +414,50 @@ proto_register_http(void)
 	 * This only works for protocols such as IPP that run over
 	 * HTTP on a specific non-HTTP port.
 	 */
-	subdissector_table = register_dissector_table("http.port");
+	subdissector_table = register_dissector_table("http.port",
+	    "TCP port for protocols using HTTP", FT_UINT16, BASE_DEC);
+
+	/* 
+	 * Heuristic dissectors SHOULD register themselves in 
+	 * this table using the standard heur_dissector_add() 
+	 * function.
+	 */
+
+	register_heur_dissector_list("http",&heur_subdissector_list);
+	
 }
 
 /*
  * Called by dissectors for protocols that run atop HTTP/TCP.
  */
 void
-http_dissector_add(guint32 port, dissector_t dissector, int proto)
+http_dissector_add(guint32 port, dissector_handle_t handle)
 {
 	/*
 	 * Register ourselves as the handler for that port number
 	 * over TCP.
 	 */
-	dissector_add("tcp.port", port, dissect_http, proto_http);
+	dissector_add("tcp.port", port, http_handle);
 
 	/*
 	 * And register them in *our* table for that port.
 	 */
-	dissector_add("http.port", port, dissector, proto);
+	dissector_add("http.port", port, handle);
 }
 
 void
 proto_reg_handoff_http(void)
 {
-	dissector_add("tcp.port", TCP_PORT_HTTP, dissect_http, proto_http);
-	dissector_add("tcp.port", TCP_ALT_PORT_HTTP, dissect_http, proto_http);
-	dissector_add("tcp.port", TCP_PORT_PROXY_HTTP, dissect_http,
-	    proto_http);
-	dissector_add("tcp.port", TCP_PORT_PROXY_ADMIN_HTTP, dissect_http,
-	    proto_http);
+        data_handle = find_dissector("data");
+	dissector_add("tcp.port", TCP_PORT_HTTP, http_handle);
+	dissector_add("tcp.port", TCP_ALT_PORT_HTTP, http_handle);
+	dissector_add("tcp.port", TCP_PORT_PROXY_HTTP, http_handle);
+	dissector_add("tcp.port", TCP_PORT_PROXY_ADMIN_HTTP, http_handle);
 
 	/*
 	 * XXX - is there anything to dissect in the body of an SSDP
 	 * request or reply?  I.e., should there be an SSDP dissector?
 	 */
-	dissector_add("tcp.port", TCP_PORT_SSDP, dissect_http, proto_http);
-	dissector_add("udp.port", UDP_PORT_SSDP, dissect_http, proto_http);
+	dissector_add("tcp.port", TCP_PORT_SSDP, http_handle);
+	dissector_add("udp.port", UDP_PORT_SSDP, http_handle);
 }
