@@ -1,7 +1,7 @@
 /* packet-icmpv6.c
  * Routines for ICMPv6 packet disassembly
  *
- * $Id: packet-icmpv6.c,v 1.48 2001/07/02 07:11:39 guy Exp $
+ * $Id: packet-icmpv6.c,v 1.56 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -9,6 +9,8 @@
  *
  * MobileIPv6 support added by Tomislav Borosa <tomislav.borosa@siemens.hr>
  *
+ * HMIPv6 support added by Martti Kuparinen <martti.kuparinen@iki.fi>
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -80,6 +82,7 @@ static gint ett_nodeinfo_nodebitmap = -1;
 static gint ett_nodeinfo_nodedns = -1;
 
 static dissector_handle_t ipv6_handle;
+static dissector_handle_t data_handle;
 
 static const value_string names_nodeinfo_qtype[] = {
     { NI_QTYPE_NOOP,		"NOOP" },
@@ -99,7 +102,7 @@ static const value_string names_rrenum_matchcode[] = {
 
 static const value_string names_router_pref[] = {
         { ND_RA_FLAG_RTPREF_HIGH,	"High" }, 
-        { ND_RA_FLAG_RTPREF_MEDIUM,	"Meidum" }, 
+        { ND_RA_FLAG_RTPREF_MEDIUM,	"Medium" }, 
         { ND_RA_FLAG_RTPREF_LOW,	"Low" }, 
         { ND_RA_FLAG_RTPREF_RSV,	"Reserved" }, 
 };
@@ -114,6 +117,7 @@ dissect_contained_icmpv6(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
     address save_net_dst;
     address save_src;
     address save_dst;
+    gboolean save_in_error_pkt;
 
     next_tvb = tvb_new_subset(tvb, offset, -1, -1);
 
@@ -124,7 +128,7 @@ dissect_contained_icmpv6(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 	   Set the columns non-writable, so that the packet list
 	   shows this as an ICMPv6 packet, not as the type of packet
 	   for which the ICMPv6 packet was generated. */
-	col_set_writable(pinfo->fd, FALSE);
+	col_set_writable(pinfo->cinfo, FALSE);
 
 	/* Also, save the current values of the addresses, and restore
 	   them when we're finished dissecting the contained packet, so
@@ -137,8 +141,30 @@ dissect_contained_icmpv6(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 	save_src = pinfo->src;
 	save_dst = pinfo->dst;
 
-	/* Dissect the contained packet. */
-	call_dissector(ipv6_handle, next_tvb, pinfo, tree);
+	/* Save the current value of the "we're inside an error packet"
+	   flag, and set that flag; subdissectors may treat packets
+	   that are the payload of error packets differently from
+	   "real" packets. */
+	save_in_error_pkt = pinfo->in_error_pkt;
+	pinfo->in_error_pkt = TRUE;
+
+	/* Dissect the contained packet.
+	   Catch ReportedBoundsError, and do nothing if we see it,
+	   because it's not an error if the contained packet is short;
+	   there's no guarantee that all of it was included.
+
+	   XXX - should catch BoundsError, and re-throw it after cleaning
+	   up. */
+	TRY {
+	    call_dissector(ipv6_handle, next_tvb, pinfo, tree);
+	}
+	CATCH(ReportedBoundsError) {
+	    ; /* do nothing */
+	}
+	ENDTRY;
+
+	/* Restore the "we're inside an error packet" flag. */
+	pinfo->in_error_pkt = save_in_error_pkt;
 
 	/* Restore the addresses. */
 	pinfo->dl_src = save_dl_src;
@@ -148,14 +174,14 @@ dissect_contained_icmpv6(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 	pinfo->src = save_src;
 	pinfo->dst = save_dst;
     } else
-	dissect_data(next_tvb, 0, pinfo, tree);
+	call_dissector(data_handle,next_tvb, pinfo, tree);
 }
 
 static void
 dissect_icmpv6opt(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
     proto_tree *icmp6opt_tree, *field_tree;
-	proto_item *ti, *tf;
+    proto_item *ti, *tf;
     struct nd_opt_hdr nd_opt_hdr, *opt;
     int len;
     char *typename;
@@ -205,6 +231,9 @@ again:
     case ND_OPT_HOMEAGENT_INFO:
 	typename = "Home Agent Information";
 	break;
+    case ND_OPT_MAP:
+	typename = "HMIPv6 MAP option";
+	break;
     default:
 	typename = "Unknown";
 	break;
@@ -312,6 +341,59 @@ again:
 	    pntohs(&pi->nd_opt_ha_info_ha_life));
 	break;
       }
+    case ND_OPT_MAP:
+      {
+	struct nd_opt_map_info mapbuf, *map;
+	int flagoff;
+
+	map = &mapbuf;
+	tvb_memcpy(tvb, (guint8 *)map, offset, sizeof *map);
+	proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset + offsetof(struct nd_opt_map_info, nd_opt_map_distance),
+	    1, "Distance: %d", map->nd_opt_map_distance);
+	proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset + offsetof(struct nd_opt_map_info, nd_opt_map_preference),
+	    1, "Preference: %d", map->nd_opt_map_preference);
+	proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset + offsetof(struct nd_opt_map_info, nd_opt_map_prefixlen),
+	    1, "Prefix Length: %d", map->nd_opt_map_prefixlen);
+	flagoff = offset + offsetof(struct nd_opt_map_info,
+	    nd_opt_map_flags);
+	tf = proto_tree_add_text(icmp6opt_tree, tvb, flagoff, 1,
+	    "Flags: 0x%02x",
+	    tvb_get_guint8(tvb, offset + offsetof(struct nd_opt_map_info,
+	    nd_opt_map_flags)));
+	field_tree = proto_item_add_subtree(tf, ett_icmpv6flag);
+	proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+	    decode_boolean_bitfield(map->nd_opt_map_flags,
+		ND_OPT_MAP_FLAG_R, 8, "R", ""));
+	proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+	    decode_boolean_bitfield(map->nd_opt_map_flags,
+		ND_OPT_MAP_FLAG_M, 8, "M", ""));
+	proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+	    decode_boolean_bitfield(map->nd_opt_map_flags,
+		ND_OPT_MAP_FLAG_I, 8, "I", ""));
+	proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+	    decode_boolean_bitfield(map->nd_opt_map_flags,
+		ND_OPT_MAP_FLAG_T, 8, "T", ""));
+	proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+	    decode_boolean_bitfield(map->nd_opt_map_flags,
+		ND_OPT_MAP_FLAG_P, 8, "P", ""));
+	proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset + offsetof(struct nd_opt_map_info, nd_opt_map_lifetime),
+	    4, "Lifetime: %d", pntohs(&map->nd_opt_map_lifetime));
+
+	proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset + offsetof(struct nd_opt_map_info, nd_opt_map_address), 16,
+#ifdef INET6
+	    "Address of MAP: %s (%s)",
+	    get_hostname6(&map->nd_opt_map_address),
+#else
+	    "Address of MAP: %s",
+#endif
+	    ip6_to_str(&map->nd_opt_map_address));
+	break;
+      }
     case ND_OPT_ROUTE_INFO:
       {
 	struct nd_opt_route_info ribuf, *ri;
@@ -429,7 +511,7 @@ bitrange0(v, s, buf, buflen)
 			l = snprintf(p, ep - p, ",%d-%d", s + off,
 			    s + off + i - 1);
 		}
-		if (l > ep - p) {
+		if (l == -1 || l > ep - p) {
 			buf[0] = '\0';
 			return NULL;
 		}
@@ -715,7 +797,7 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 nodata:;
 
     /* the rest of data */
-    dissect_data(tvb_new_subset(tvb, offset + off, -1, -1), 0, pinfo, tree);
+    call_dissector(data_handle,tvb_new_subset(tvb, offset + off, -1, -1), pinfo, tree);
 }
 
 static void
@@ -763,7 +845,7 @@ dissect_rrenum(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
     proto_tree_add_text(tree, tvb,
 	offset + offsetof(struct icmp6_router_renum, rr_maxdelay), 2,
 	"Max delay: 0x%04x", pntohs(&rr->rr_maxdelay));
-    dissect_data(tvb_new_subset(tvb, offset + sizeof(*rr), -1, -1), 0, pinfo, tree);	/*XXX*/
+    call_dissector(data_handle,tvb_new_subset(tvb, offset + sizeof(*rr), -1, -1), pinfo, tree);	/*XXX*/
 
     if (rr->rr_code == ICMP6_ROUTER_RENUMBERING_COMMAND) {
 	off = offset + sizeof(*rr);
@@ -898,10 +980,10 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     int offset;
     tvbuff_t *next_tvb;
 
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-	col_set_str(pinfo->fd, COL_PROTOCOL, "ICMPv6");
-    if (check_col(pinfo->fd, COL_INFO))
-	col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "ICMPv6");
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_clear(pinfo->cinfo, COL_INFO);
 
     offset = 0;
     tvb_memcpy(tvb, (guint8 *)&icmp6_hdr, offset, sizeof icmp6_hdr);
@@ -1057,7 +1139,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	break;
     }
 
-    if (check_col(pinfo->fd, COL_INFO)) {
+    if (check_col(pinfo->cinfo, COL_INFO)) {
 	char typebuf[256], codebuf[256];
 
 	if (coltypename && strcmp(coltypename, "Unknown") == 0) {
@@ -1071,9 +1153,9 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    colcodename = codebuf;
 	}
 	if (colcodename) {
-	    col_add_fstr(pinfo->fd, COL_INFO, "%s (%s)", coltypename, colcodename);
+	    col_add_fstr(pinfo->cinfo, COL_INFO, "%s (%s)", coltypename, colcodename);
 	} else {
-	    col_add_fstr(pinfo->fd, COL_INFO, "%s", coltypename);
+	    col_add_fstr(pinfo->cinfo, COL_INFO, "%s", coltypename);
 	}
     }
 
@@ -1170,7 +1252,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		offset + offsetof(struct icmp6_hdr, icmp6_seq), 2,
 		"Sequence: 0x%04x", (guint16)ntohs(dp->icmp6_seq));
 	    next_tvb = tvb_new_subset(tvb, offset + sizeof(*dp), -1, -1);
-	    dissect_data(next_tvb, 0, pinfo, icmp6_tree);
+	    call_dissector(data_handle,next_tvb, pinfo, icmp6_tree);
 	    break;
 	case ICMP6_MEMBERSHIP_QUERY:
 	case ICMP6_MEMBERSHIP_REPORT:
@@ -1327,7 +1409,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    break;
 	default:
 	    next_tvb = tvb_new_subset(tvb, offset + sizeof(*dp), -1, -1);
-	    dissect_data(next_tvb, 0, pinfo, tree);
+	    call_dissector(data_handle,next_tvb, pinfo, tree);
 	    break;
 	}
     }
@@ -1372,10 +1454,14 @@ proto_register_icmpv6(void)
 void
 proto_reg_handoff_icmpv6(void)
 {
-  dissector_add("ip.proto", IP_PROTO_ICMPV6, dissect_icmpv6, proto_icmpv6);
+  dissector_handle_t icmpv6_handle;
+
+  icmpv6_handle = create_dissector_handle(dissect_icmpv6, proto_icmpv6);
+  dissector_add("ip.proto", IP_PROTO_ICMPV6, icmpv6_handle);
 
   /*
    * Get a handle for the IPv6 dissector.
    */
   ipv6_handle = find_dissector("ipv6");
+  data_handle = find_dissector("data");
 }
