@@ -1,13 +1,15 @@
 /* packet-icp.c
  * Routines for ICP (internet cache protocol) packet disassembly
  * RFC 2186 && RFC 2187
- *
- * $Id: packet-icp.c,v 1.18 2001/06/18 02:17:46 guy Exp $
- *
- * Ethereal - Network traffic analyzer
  * By Peter Torvals
  * Copyright 1999 Peter Torvals
-
+ *
+ * $Id: packet-icp.c,v 1.20 2001/12/10 00:25:28 guy Exp $
+ *
+ * Ethereal - Network traffic analyzer
+ * By Gerald Combs <gerald@ethereal.com>
+ * Copyright 1998
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -158,18 +160,18 @@ static void dissect_icp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint32 options;
   guint32 option_data;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-        col_set_str(pinfo->fd, COL_PROTOCOL, "ICP");
-  if (check_col(pinfo->fd, COL_INFO))
-        col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "ICP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+        col_clear(pinfo->cinfo, COL_INFO);
 
   opcode=tvb_get_guint8(tvb, 0);
   message_length=tvb_get_ntohs(tvb, 2);
   request_number=tvb_get_ntohl(tvb, 4);
 
-  if (check_col(pinfo->fd, COL_INFO))
+  if (check_col(pinfo->cinfo, COL_INFO))
   {
-        col_add_fstr(pinfo->fd,COL_INFO,"Opcode: %s (%u), Req Nr: %u",
+        col_add_fstr(pinfo->cinfo,COL_INFO,"Opcode: %s (%u), Req Nr: %u",
 		val_to_str(opcode, opcode_vals, "Unknown"), opcode,
 		request_number);
   }
@@ -253,5 +255,8 @@ proto_register_icp(void)
 void
 proto_reg_handoff_icp(void)
 {
-	dissector_add("udp.port", UDP_PORT_ICP, dissect_icp, proto_icp);
+	dissector_handle_t icp_handle;
+
+	icp_handle = create_dissector_handle(dissect_icp, proto_icp);
+	dissector_add("udp.port", UDP_PORT_ICP, icp_handle);
 }
