@@ -3,7 +3,7 @@
  * Copyright 2000, Axis Communications AB 
  * Inquiries/bugreports should be sent to Johan.Jorgensen@axis.com
  *
- * $Id: packet-ieee80211.c,v 1.36 2001/06/22 08:12:11 guy Exp $
+ * $Id: packet-ieee80211.c,v 1.46 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -65,6 +65,7 @@
 #include "proto.h"
 #include "packet.h"
 #include "resolv.h"
+#include "packet-ipx.h"
 #include "packet-llc.h"
 #include "packet-ieee80211.h"
 #include "etypes.h"
@@ -72,7 +73,7 @@
 /* ************************************************************************* */
 /*                          Miscellaneous Constants                          */
 /* ************************************************************************* */
-#define SHORT_STR 128
+#define SHORT_STR 256
 
 /* ************************************************************************* */
 /*  Define some very useful macros that are used to analyze frame types etc. */
@@ -88,10 +89,6 @@
 #define COOK_FLAGS(x)           (((x) & 0xFF00) >> 8)
 #define COOK_DS_STATUS(x)       ((x) & 0x3)
 #define COOK_WEP_KEY(x)       (((x) & 0xC0) >> 6)
-#define COL_SHOW_INFO(fd,info) if (check_col(fd,COL_INFO)) \
-				col_add_str(fd,COL_INFO,info);
-#define COL_SHOW_INFO_CONST(fd,info) if (check_col(fd,COL_INFO)) \
-				col_set_str(fd,COL_INFO,info);
 
 #define FLAG_TO_DS		0x01
 #define FLAG_FROM_DS		0x02
@@ -267,6 +264,7 @@ static int hf_addr_ra = -1;	/* Receiver address subfield */
 static int hf_addr_ta = -1;	/* Transmitter address subfield */
 static int hf_addr_bssid = -1;	/* address is bssid */
 
+static int hf_addr = -1;	/* Source or destination address subfield */
 
 
 /* ************************************************************************* */
@@ -338,6 +336,8 @@ static gint ett_tagged_parameters = -1;
 static gint ett_wep_parameters = -1;
 
 static dissector_handle_t llc_handle;
+static dissector_handle_t ipx_handle;
+static dissector_handle_t data_handle;
 
 /* ************************************************************************* */
 /*            Return the length of the current header (in bytes)             */
@@ -378,10 +378,15 @@ find_header_length (guint16 fcf)
 /*          This is the capture function used to update packet counts        */
 /* ************************************************************************* */
 void
-capture_ieee80211 (const u_char * pd, int offset, packet_counts * ld)
+capture_ieee80211 (const u_char * pd, int offset, int len, packet_counts * ld)
 {
   guint16 fcf, hdr_length;
 
+  if (!BYTES_ARE_IN_FRAME(offset, len, 2)) {
+    ld->other++;
+    return;
+  }
+
   fcf = pletohs (&pd[0]);
 
   if (IS_WEP(COOK_FLAGS(fcf)))
@@ -398,7 +403,27 @@ capture_ieee80211 (const u_char * pd, int offset, packet_counts * ld)
     case DATA_CF_POLL:
     case DATA_CF_ACK_POLL:
       hdr_length = find_header_length (fcf);
-      capture_llc (pd, offset + hdr_length, ld);
+      /* I guess some bridges take Netware Ethernet_802_3 frames,
+         which are 802.3 frames (with a length field rather than
+         a type field, but with no 802.2 header in the payload),
+         and just stick the payload into an 802.11 frame.  I've seen
+         captures that show frames of that sort.
+
+         This means we have to do the same check for Netware 802.3 -
+         or, if you will, "Netware 802.11" - that we do in the
+         Ethernet dissector, i.e. checking for 0xffff as the first
+         four bytes of the payload and, if we find it, treating it
+         as an IPX frame. */
+      if (!BYTES_ARE_IN_FRAME(offset+hdr_length, len, 2)) {
+        ld->other++;
+        return;
+      }
+      if (pd[offset+hdr_length] == 0xff && pd[offset+hdr_length+1] == 0xff) {
+        capture_ipx (pd, offset + hdr_length, len, ld);
+      }
+      else {
+        capture_llc (pd, offset + hdr_length, len, ld);
+      }
       break;
 
     default:
@@ -578,7 +603,7 @@ add_tagged_field (proto_tree * tree, tvbuff_t * tvb, int offset)
   const guint8 *tag_data_ptr;
   guint32 tag_no, tag_len;
   unsigned int i;
-  int n;
+  int n, ret;
   char out_buff[SHORT_STR];
 
 
@@ -648,13 +673,20 @@ add_tagged_field (proto_tree * tree, tvbuff_t * tvb, int offset)
       strcpy (out_buff, "Supported rates: ");
       n = strlen (out_buff);
 
-      for (i = 0; i < tag_len; i++)
+      for (i = 0; i < tag_len && n < SHORT_STR; i++)
 	{
-	    n += snprintf (out_buff + n, SHORT_STR - n, "%2.1f%s ",
+	    ret = snprintf (out_buff + n, SHORT_STR - n, "%2.1f%s ",
 			   (tag_data_ptr[i] & 0x7F) * 0.5,
 			   (tag_data_ptr[i] & 0x80) ? "(B)" : "");
+	    if (ret == -1) {
+	      /* Some versions of snprintf return -1 if they'd truncate
+	         the output. */
+	      break;
+	    }
+	    n += ret;
 	}
-      snprintf (out_buff + n, SHORT_STR - n, "[Mbit/sec]");
+      if (n < SHORT_STR)
+	snprintf (out_buff + n, SHORT_STR - n, "[Mbit/sec]");
 
       proto_tree_add_string (tree, tag_interpretation, tvb, offset + 2,
 			     tag_len, out_buff);
@@ -780,7 +812,7 @@ dissect_ieee80211_mgt (guint16 fcf, tvbuff_t * tvb, packet_info * pinfo,
   guint32 next_len;
   int tagged_parameter_tree_len;
 
-  CHECK_DISPLAY_AS_DATA(proto_wlan_mgt, tvb, pinfo, tree);
+  CHECK_DISPLAY_AS_X(data_handle,proto_wlan_mgt, tvb, pinfo, tree);
 
   if (tree)
     {
@@ -980,22 +1012,22 @@ dissect_ieee80211_mgt (guint16 fcf, tvbuff_t * tvb, packet_info * pinfo,
 static void
 set_src_addr_cols(packet_info *pinfo, const guint8 *addr, char *type)
 {
-  if (check_col(pinfo->fd, COL_RES_DL_SRC))
-    col_add_fstr(pinfo->fd, COL_RES_DL_SRC, "%s (%s)",
+  if (check_col(pinfo->cinfo, COL_RES_DL_SRC))
+    col_add_fstr(pinfo->cinfo, COL_RES_DL_SRC, "%s (%s)",
 		    get_ether_name(addr), type);
-  if (check_col(pinfo->fd, COL_UNRES_DL_SRC))
-    col_add_fstr(pinfo->fd, COL_UNRES_DL_SRC, "%s (%s)",
+  if (check_col(pinfo->cinfo, COL_UNRES_DL_SRC))
+    col_add_fstr(pinfo->cinfo, COL_UNRES_DL_SRC, "%s (%s)",
 		     ether_to_str(addr), type);
 }
 
 static void
 set_dst_addr_cols(packet_info *pinfo, const guint8 *addr, char *type)
 {
-  if (check_col(pinfo->fd, COL_RES_DL_DST))
-    col_add_fstr(pinfo->fd, COL_RES_DL_DST, "%s (%s)",
+  if (check_col(pinfo->cinfo, COL_RES_DL_DST))
+    col_add_fstr(pinfo->cinfo, COL_RES_DL_DST, "%s (%s)",
 		     get_ether_name(addr), type);
-  if (check_col(pinfo->fd, COL_UNRES_DL_DST))
-    col_add_fstr(pinfo->fd, COL_UNRES_DL_DST, "%s (%s)",
+  if (check_col(pinfo->cinfo, COL_UNRES_DL_DST))
+    col_add_fstr(pinfo->cinfo, COL_UNRES_DL_DST, "%s (%s)",
 		     ether_to_str(addr), type);
 }
 
@@ -1016,19 +1048,21 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
   guint16 hdr_len;
   tvbuff_t *next_tvb;
   guint32 addr_type;
+  volatile gboolean is_802_2;
 
-  if (check_col (pinfo->fd, COL_PROTOCOL))
-    col_set_str (pinfo->fd, COL_PROTOCOL, "IEEE 802.11");
-  if (check_col (pinfo->fd, COL_INFO))
-    col_clear (pinfo->fd, COL_INFO);
+  if (check_col (pinfo->cinfo, COL_PROTOCOL))
+    col_set_str (pinfo->cinfo, COL_PROTOCOL, "IEEE 802.11");
+  if (check_col (pinfo->cinfo, COL_INFO))
+    col_clear (pinfo->cinfo, COL_INFO);
 
   fcf = tvb_get_letohs (tvb, 0);
   hdr_len = find_header_length (fcf);
   frame_type_subtype = COMPOSE_FRAME_TYPE(fcf);
 
-  COL_SHOW_INFO_CONST (pinfo->fd,
-      val_to_str(frame_type_subtype, frame_type_subtype_vals,
-          "Unrecognized (Reserved frame)"));
+  if (check_col (pinfo->cinfo, COL_INFO))
+      col_set_str (pinfo->cinfo, COL_INFO,
+          val_to_str(frame_type_subtype, frame_type_subtype_vals,
+              "Unrecognized (Reserved frame)"));
 
   /* Add the FC to the current tree */
   if (tree)
@@ -1128,6 +1162,10 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 
 	  proto_tree_add_ether (hdr_tree, hf_addr_sa, tvb, 10, 6, src);
 
+	  /* add items for wlan.addr filter */
+	  proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 4, 6, dst);
+	  proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 10, 6, src);
+
 	  proto_tree_add_ether (hdr_tree, hf_addr_bssid, tvb, 16, 6,
 				tvb_get_ptr (tvb, 16, 6));
 
@@ -1283,6 +1321,10 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 	      proto_tree_add_uint (hdr_tree, hf_seq_number, tvb, 22, 2,
 				   COOK_SEQUENCE_NUMBER (tvb_get_letohs
 							 (tvb, 22)));
+
+	      /* add items for wlan.addr filter */
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 4, 6, dst);
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 10, 6, src);
 	      break;
 	      
 	      
@@ -1297,6 +1339,10 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 	      proto_tree_add_uint (hdr_tree, hf_seq_number, tvb, 22, 2,
 				   COOK_SEQUENCE_NUMBER (tvb_get_letohs
 							 (tvb, 22)));
+
+	      /* add items for wlan.addr filter */
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 4, 6, dst);
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 16, 6, src);
 	      break;
    
 
@@ -1305,12 +1351,17 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 				    tvb_get_ptr (tvb, 4, 6));
 	      proto_tree_add_ether (hdr_tree, hf_addr_sa, tvb, 10, 6, src);
 	      proto_tree_add_ether (hdr_tree, hf_addr_da, tvb, 16, 6, dst);
+
 	      proto_tree_add_uint (hdr_tree, hf_frag_number, tvb, 22, 2,
 				   COOK_FRAGMENT_NUMBER (tvb_get_letohs
 							 (tvb, 22)));
 	      proto_tree_add_uint (hdr_tree, hf_seq_number, tvb, 22, 2,
 				   COOK_SEQUENCE_NUMBER (tvb_get_letohs
 							 (tvb, 22)));
+
+	      /* add items for wlan.addr filter */
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 10, 6, src);
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 16, 6, dst);
 	      break;
 	      
 
@@ -1327,6 +1378,10 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 				   COOK_SEQUENCE_NUMBER (tvb_get_letohs
 							 (tvb, 22)));
 	      proto_tree_add_ether (hdr_tree, hf_addr_sa, tvb, 24, 6, src);
+
+	      /* add items for wlan.addr filter */
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 16, 6, dst);
+	      proto_tree_add_ether_hidden(hdr_tree, hf_addr, tvb, 24, 6, src);
 	      break;
 	    }
 
@@ -1393,7 +1448,7 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 	    cap_len -= 4 - (pkt_len - cap_len);
 	  pkt_len -= 4;
 	  if (cap_len > 0 && pkt_len > 0)
-	    dissect_data (tvb, hdr_len + 4, pinfo, tree);
+	    call_dissector(data_handle,tvb_new_subset(tvb, hdr_len + 4, -1,tvb_reported_length_remaining(tvb,hdr_len + 4)),pinfo, tree);
 	}
 	return;
     }
@@ -1411,7 +1466,32 @@ dissect_ieee80211 (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 
 
     case DATA_FRAME:
-      call_dissector (llc_handle, next_tvb, pinfo, tree);
+      /* I guess some bridges take Netware Ethernet_802_3 frames,
+         which are 802.3 frames (with a length field rather than
+         a type field, but with no 802.2 header in the payload),
+         and just stick the payload into an 802.11 frame.  I've seen
+         captures that show frames of that sort.
+
+         This means we have to do the same check for Netware 802.3 -
+         or, if you will, "Netware 802.11" - that we do in the
+         Ethernet dissector, i.e. checking for 0xffff as the first
+         four bytes of the payload and, if we find it, treating it
+         as an IPX frame. */
+      is_802_2 = TRUE;
+      TRY {
+        if (tvb_get_ntohs(next_tvb, 0) == 0xffff)
+          is_802_2 = FALSE;
+      }
+      CATCH2(BoundsError, ReportedBoundsError) {
+	    ; /* do nothing */
+
+      }
+      ENDTRY;
+
+      if (is_802_2)
+        call_dissector(llc_handle, next_tvb, pinfo, tree);
+      else
+        call_dissector(ipx_handle, next_tvb, pinfo, tree);
       break;
     }
 }
@@ -1644,11 +1724,15 @@ proto_register_wlan (void)
 
     {&hf_addr_da,
      {"Destination address", "wlan.da", FT_ETHER, BASE_NONE, NULL, 0,
-      "Destination Hardware address", HFILL }},
+      "Destination Hardware Address", HFILL }},
 
     {&hf_addr_sa,
      {"Source address", "wlan.sa", FT_ETHER, BASE_NONE, NULL, 0,
-      "Source Hardware address", HFILL }},
+      "Source Hardware Address", HFILL }},
+
+    { &hf_addr,
+      {"Source or Destination address", "wlan.addr", FT_ETHER, BASE_NONE, NULL, 0,
+       "Source or Destination Hardware Address", HFILL }},
 
     {&hf_addr_ra,
      {"Receiver address", "wlan.ra", FT_ETHER, BASE_NONE, NULL, 0,
@@ -1806,16 +1890,22 @@ proto_register_wlan (void)
 					"802.11 MGT", "wlan_mgt");
   proto_register_field_array (proto_wlan_mgt, ff, array_length (ff));
   proto_register_subtree_array (tree_array, array_length (tree_array));
+
+  register_dissector("wlan", dissect_ieee80211, proto_wlan);
 }
 
 void
 proto_reg_handoff_wlan(void)
 {
+  dissector_handle_t ieee80211_handle;
+
   /*
-   * Get a handle for the LLC dissector.
+   * Get handles for the LLC and IPX dissectors.
    */
   llc_handle = find_dissector("llc");
+  ipx_handle = find_dissector("ipx");
+  data_handle = find_dissector("data");
 
-  dissector_add("wtap_encap", WTAP_ENCAP_IEEE_802_11, dissect_ieee80211,
-		proto_wlan);
+  ieee80211_handle = find_dissector("wlan");
+  dissector_add("wtap_encap", WTAP_ENCAP_IEEE_802_11, ieee80211_handle);
 }
