@@ -2,10 +2,10 @@
  * Routines for imap packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-imap.c,v 1.15 2001/06/18 02:17:47 guy Exp $
+ * $Id: packet-imap.c,v 1.17 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
@@ -64,8 +64,8 @@ dissect_imap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int		tokenlen;
 	const u_char	*next_token;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IMAP");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IMAP");
 
 	/*
 	 * Find the end of the first line.
@@ -82,12 +82,12 @@ dissect_imap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	else
 		is_request = FALSE;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		/*
 		 * Put the first line from the buffer into the summary
 		 * (but leave out the line terminator).
 		 */
-		col_add_fstr(pinfo->fd, COL_INFO, "%s: %s",
+		col_add_fstr(pinfo->cinfo, COL_INFO, "%s: %s",
 		    is_request ? "Request" : "Response",
 		    format_text(line, linelen));
 	}
@@ -182,5 +182,8 @@ proto_register_imap(void)
 void
 proto_reg_handoff_imap(void)
 {
-  dissector_add("tcp.port", TCP_PORT_IMAP, dissect_imap, proto_imap);
+  dissector_handle_t imap_handle;
+
+  imap_handle = create_dissector_handle(dissect_imap, proto_imap);
+  dissector_add("tcp.port", TCP_PORT_IMAP, imap_handle);
 }
