@@ -1,7 +1,7 @@
 /* packet-ip.c
  * Routines for IP and miscellaneous IP protocol packet disassembly
  *
- * $Id: packet-ip.c,v 1.139 2001/06/29 09:46:52 guy Exp $
+ * $Id: packet-ip.c,v 1.152 2001/12/10 00:25:28 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -116,13 +116,37 @@ static gint ett_ip_fragment  = -1;
 /* Used by IPv6 as well, so not static */
 dissector_table_t ip_dissector_table;
 
+static dissector_handle_t ip_handle;
+static dissector_handle_t data_handle;
+
 static int proto_icmp = -1;
 static int hf_icmp_type = -1;
 static int hf_icmp_code = -1;
 static int hf_icmp_checksum = -1;
 static int hf_icmp_checksum_bad = -1;
 
+/* Mobile ip */
+static int hf_icmp_mip_type = -1;
+static int hf_icmp_mip_length = -1;
+static int hf_icmp_mip_prefix_length = -1;
+static int hf_icmp_mip_seq = -1;
+static int hf_icmp_mip_life = -1;
+static int hf_icmp_mip_flags = -1;
+static int hf_icmp_mip_r = -1;
+static int hf_icmp_mip_b = -1;
+static int hf_icmp_mip_h = -1;
+static int hf_icmp_mip_f = -1;
+static int hf_icmp_mip_m = -1;
+static int hf_icmp_mip_g = -1;
+static int hf_icmp_mip_v = -1;
+static int hf_icmp_mip_res = -1;
+static int hf_icmp_mip_reserved = -1;
+static int hf_icmp_mip_coa = -1;
+static int hf_icmp_mip_challenge = -1;
+
 static gint ett_icmp = -1;
+static gint ett_icmp_mip = -1;
+static gint ett_icmp_mip_flags = -1;
 
 /* ICMP definitions */
 
@@ -310,8 +334,8 @@ ip_defragment_init(void)
 }
 
 void
-capture_ip(const u_char *pd, int offset, packet_counts *ld) {
-  if (!BYTES_ARE_IN_FRAME(offset, IPH_MIN_LEN)) {
+capture_ip(const u_char *pd, int offset, int len, packet_counts *ld) {
+  if (!BYTES_ARE_IN_FRAME(offset, len, IPH_MIN_LEN)) {
     ld->other++;
     return;
   }
@@ -344,7 +368,7 @@ capture_ip(const u_char *pd, int offset, packet_counts *ld) {
 
 static void
 dissect_ipopt_security(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
-			guint optlen, frame_data *fd, proto_tree *opt_tree)
+			guint optlen, packet_info *pinfo, proto_tree *opt_tree)
 {
   proto_tree *field_tree = NULL;
   proto_item *tf;
@@ -396,7 +420,7 @@ dissect_ipopt_security(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
 
 static void
 dissect_ipopt_route(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
-			guint optlen, frame_data *fd, proto_tree *opt_tree)
+			guint optlen, packet_info *pinfo, proto_tree *opt_tree)
 {
   proto_tree *field_tree = NULL;
   proto_item *tf;
@@ -441,7 +465,7 @@ dissect_ipopt_route(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
 
 static void
 dissect_ipopt_sid(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
-			guint optlen, frame_data *fd, proto_tree *opt_tree)
+			guint optlen, packet_info *pinfo, proto_tree *opt_tree)
 {
   proto_tree_add_text(opt_tree, tvb, offset,      optlen,
     "%s: %u", optp->name, tvb_get_ntohs(tvb, offset + 2));
@@ -450,7 +474,7 @@ dissect_ipopt_sid(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
 
 static void
 dissect_ipopt_timestamp(const ip_tcp_opt *optp, tvbuff_t *tvb,
-    int offset, guint optlen, frame_data *fd, proto_tree *opt_tree)
+    int offset, guint optlen, packet_info *pinfo, proto_tree *opt_tree)
 {
   proto_tree *field_tree = NULL;
   proto_item *tf;
@@ -521,7 +545,7 @@ dissect_ipopt_timestamp(const ip_tcp_opt *optp, tvbuff_t *tvb,
 
 static void
 dissect_ipopt_ra(const ip_tcp_opt *optp, tvbuff_t *tvb, int offset,
-		guint optlen, frame_data *fd, proto_tree *opt_tree)
+		guint optlen, packet_info *pinfo, proto_tree *opt_tree)
 {
   /* Router-Alert, as defined by RFC2113 */
   int opt = tvb_get_ntohs(tvb, offset + 2);
@@ -616,7 +640,7 @@ static const ip_tcp_opt ipopts[] = {
 void
 dissect_ip_tcp_options(tvbuff_t *tvb, int offset, guint length,
 			const ip_tcp_opt *opttab, int nopts, int eol,
-			frame_data *fd, proto_tree *opt_tree)
+			packet_info *pinfo, proto_tree *opt_tree)
 {
   u_char            opt;
   const ip_tcp_opt *optp;
@@ -625,7 +649,7 @@ dissect_ip_tcp_options(tvbuff_t *tvb, int offset, guint length,
   char             *name;
   char              name_str[7+1+1+2+2+1+1];	/* "Unknown (0x%02x)" */
   void            (*dissect)(const struct ip_tcp_opt *, tvbuff_t *,
-				int, guint, frame_data *, proto_tree *);
+				int, guint, packet_info *, proto_tree *);
   guint             len;
 
   while (length > 0) {
@@ -697,7 +721,7 @@ dissect_ip_tcp_options(tvbuff_t *tvb, int offset, guint length,
         } else {
           if (dissect != NULL) {
             /* Option has a dissector. */
-            (*dissect)(optp, tvb, offset,          len, fd, opt_tree);
+            (*dissect)(optp, tvb, offset,          len, pinfo, opt_tree);
           } else {
             /* Option has no data, hence no dissector. */
             proto_tree_add_text(opt_tree, tvb, offset,  len, "%s", name);
@@ -793,20 +817,17 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   proto_item *ti, *tf;
   int        offset = 0;
   guint      hlen, optlen, len;
-  int        padding;
   guint16    flags;
   guint8     nxt;
   guint16    ipsum;
   fragment_data *ipfd_head;
   tvbuff_t   *next_tvb;
-  packet_info save_pi;
-  gboolean must_restore_pi = FALSE;
   gboolean update_col_info = TRUE;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "IP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "IP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   /* Avoids alignment problems on many architectures. */
   tvb_memcpy(tvb, (guint8 *)&iph, offset, sizeof(e_ip));
@@ -815,7 +836,12 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   iph.ip_off = ntohs(iph.ip_off);
   iph.ip_sum = ntohs(iph.ip_sum);
 
-  /* Length of IP datagram. */
+  /* Length of IP datagram.
+     XXX - what if this is greater than the reported length of the
+     tvbuff?  This could happen, for example, in an IP datagram
+     inside an ICMP datagram; we need to somehow let the
+     dissector we call know that, as it might want to avoid
+     doing its checksumming. */
   len = iph.ip_len;
 
   /* Adjust the length of this tvbuff to include only the IP datagram. */
@@ -836,8 +862,8 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   }
 
   if (hlen < IPH_MIN_LEN) {
-    if (check_col(pinfo->fd, COL_INFO))
-      col_add_fstr(pinfo->fd, COL_INFO, "Bogus IP header length (%u, must be at least %u)",
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_add_fstr(pinfo->cinfo, COL_INFO, "Bogus IP header length (%u, must be at least %u)",
        hlen, IPH_MIN_LEN);
     if (tree) {
       proto_tree_add_uint_format(ip_tree, hf_ip_hdr_len, tvb, offset, 1, hlen,
@@ -920,7 +946,7 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         "Options: (%u bytes)", optlen);
       field_tree = proto_item_add_subtree(tf, ett_ip_options);
       dissect_ip_tcp_options(tvb, offset + 20, optlen,
-         ipopts, N_IP_OPTS, IPOPT_END, pinfo->fd, field_tree);
+         ipopts, N_IP_OPTS, IPOPT_END, pinfo, field_tree);
     }
   }
 
@@ -1023,8 +1049,8 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       }
       if (ipfd_head->flags & (FD_OVERLAPCONFLICT
                         |FD_MULTIPLETAILS|FD_TOOLONGFRAGMENT) ) {
-        if (check_col(pinfo->fd, COL_INFO)) {
-          col_set_str(pinfo->fd, COL_INFO, "[Illegal fragments]");
+        if (check_col(pinfo->cinfo, COL_INFO)) {
+          col_set_str(pinfo->cinfo, COL_INFO, "[Illegal fragments]");
           update_col_info = FALSE;
         }
       }
@@ -1043,14 +1069,6 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
       /* It's not fragmented. */
       pinfo->fragmented = FALSE;
-
-      /* Save the current value of "pi", and adjust certain fields to
-         reflect the new tvbuff. */
-      save_pi = pi;
-      pi.compat_top_tvb = next_tvb;
-      pi.len = tvb_reported_length(next_tvb);
-      pi.captured_len = tvb_length(next_tvb);
-      must_restore_pi = TRUE;
     } else {
       /* We don't have the complete reassembled payload. */
       next_tvb = NULL;
@@ -1083,13 +1101,10 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   if (next_tvb == NULL) {
     /* Just show this as a fragment. */
-    if (check_col(pinfo->fd, COL_INFO))
-      col_add_fstr(pinfo->fd, COL_INFO, "Fragmented IP protocol (proto=%s 0x%02x, off=%u)",
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_add_fstr(pinfo->cinfo, COL_INFO, "Fragmented IP protocol (proto=%s 0x%02x, off=%u)",
 	ipprotostr(iph.ip_p), iph.ip_p, (iph.ip_off & IP_OFFSET) * 8);
-    dissect_data(tvb, offset, pinfo, tree);
-
-    /* As we haven't reassembled anything, we haven't changed "pi", so
-       we don't have to restore it. */
+    call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
     return;
   }
 
@@ -1103,16 +1118,162 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (!dissector_try_port(ip_dissector_table, nxt, next_tvb, pinfo, tree)) {
     /* Unknown protocol */
     if (update_col_info) {
-      if (check_col(pinfo->fd, COL_INFO))
-        col_add_fstr(pinfo->fd, COL_INFO, "%s (0x%02x)", ipprotostr(iph.ip_p), iph.ip_p);
+      if (check_col(pinfo->cinfo, COL_INFO))
+        col_add_fstr(pinfo->cinfo, COL_INFO, "%s (0x%02x)", ipprotostr(iph.ip_p), iph.ip_p);
     }
-    dissect_data(next_tvb, 0, pinfo, tree);
+    call_dissector(data_handle,next_tvb, pinfo, tree);
   }
-
-  if (must_restore_pi)
-    pi = save_pi;
 }
 
+#define ICMP_MIP_EXTENSION_PAD	0
+#define ICMP_MIP_MOB_AGENT_ADV	16
+#define ICMP_MIP_PREFIX_LENGTHS	19
+#define ICMP_MIP_CHALLENGE	24
+
+static value_string mip_extensions[] = {
+  { ICMP_MIP_EXTENSION_PAD, "One byte padding extension"},  /* RFC 2002 */
+  { ICMP_MIP_MOB_AGENT_ADV, "Mobility Agent Advertisement Extension"},
+							    /* RFC 2002 */
+  { ICMP_MIP_PREFIX_LENGTHS, "Prefix Lengths Extension"},   /* RFC 2002 */
+  { ICMP_MIP_CHALLENGE, "Challenge Extension"},             /* RFC 3012 */
+  { 0, NULL}
+};
+
+/*
+ * Dissect the mobile ip advertisement extensions.
+ */
+static void
+dissect_mip_extensions(tvbuff_t *tvb, size_t offset, packet_info *pinfo,
+					   proto_tree *tree)
+{
+  guint8       type;
+  guint8       length;
+  guint8       flags;
+  proto_item   *ti;
+  proto_tree   *mip_tree=NULL;
+  proto_tree   *flags_tree=NULL;
+  gint         numCOAs;
+  gint         i;
+
+  /* Not much to do if we're not parsing everything */
+  if (!tree) return;
+  
+  while ((tvb_length(tvb) - offset) > 0) {
+
+	type = tvb_get_guint8(tvb, offset + 0);
+	if (type)
+	  length = tvb_get_guint8(tvb, offset + 1);
+	else
+	  length=0;
+
+	ti = proto_tree_add_text(tree, tvb, offset,
+							 type?(length + 2):1,
+							 "Ext: %s",
+							 val_to_str(type, mip_extensions,
+										"Unknown ext %d"));
+	mip_tree = proto_item_add_subtree(ti, ett_icmp_mip);
+  
+
+	switch (type) {
+	case ICMP_MIP_EXTENSION_PAD:
+	  /* One byte padding extension */
+	  /* Add our fields */
+	  /* type */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_type, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  break;
+	case ICMP_MIP_MOB_AGENT_ADV:
+	  /* Mobility Agent Advertisement Extension (RFC 2002)*/
+	  /* Add our fields */
+	  /* type */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_type, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  /* length */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_length, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  /* sequence number */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_seq, tvb, offset, 
+						  2, FALSE);
+	  offset+=2;
+	  /* Registration Lifetime */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_life, tvb, offset, 
+						  2, FALSE);
+	  offset+=2;
+	  /* flags */
+	  flags = tvb_get_guint8(tvb, offset);
+	  ti = proto_tree_add_item(mip_tree, hf_icmp_mip_flags, tvb, offset,
+							   1, FALSE);
+	  flags_tree = proto_item_add_subtree(ti, ett_icmp_mip_flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_r, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_b, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_h, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_f, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_m, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_g, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_v, tvb, offset, 1, flags);
+	  proto_tree_add_boolean(flags_tree, hf_icmp_mip_res, tvb, offset, 1, flags);
+	  offset++;
+	  
+	  /* Reserved */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_reserved, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  
+	  /* COAs */
+	  numCOAs = (length - 6) / 4;
+	  for (i=0; i<numCOAs; i++) {
+		proto_tree_add_item(mip_tree, hf_icmp_mip_coa, tvb, offset, 
+							4, FALSE);
+		offset+=4;
+	  }
+	  break;
+	case ICMP_MIP_PREFIX_LENGTHS:
+	  /* Prefix-Lengths Extension  (RFC 2002)*/
+	  /* Add our fields */
+	  /* type */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_type, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  /* length */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_length, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+
+	  /* prefix lengths */
+	  for(i=0; i<length; i++) {
+		proto_tree_add_item(mip_tree, hf_icmp_mip_prefix_length, tvb, offset,
+							1, FALSE);
+		offset++;
+	  }
+	  break;
+	case ICMP_MIP_CHALLENGE:
+	  /* Challenge Extension  (RFC 3012)*/
+	  /* type */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_type, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  /* length */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_length, tvb, offset, 
+						  1, FALSE);
+	  offset++;
+	  /* challenge */
+	  proto_tree_add_item(mip_tree, hf_icmp_mip_challenge, tvb, offset, 
+						  length, FALSE);
+	  offset+=length;
+	  
+	  break;
+	default:
+	  g_warning("Unknown type(%d)!  I hope the length is right (%d)",
+				type, length);
+	  offset += length;
+	  break;
+	} /* switch type */
+  } /* end while */
+  
+} /* dissect_mip_extensions */
 
 static const gchar *unreach_str[] = {"Network unreachable",
                                      "Host unreachable",
@@ -1149,6 +1310,12 @@ static const gchar *par_str[] = {"IP header bad", "Required option missing"};
 
 #define	N_PARAMPROB	(sizeof par_str / sizeof par_str[0])
 
+/*
+ * RFC 792 for basic ICMP.
+ * RFC 1191 for ICMP_FRAG_NEEDED (with MTU of next hop).
+ * RFC 1256 for router discovery messages.
+ * RFC 2002 and 3012 for Mobile IP stuff.
+ */
 static void
 dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
@@ -1162,11 +1329,19 @@ dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint8     num_addrs = 0;
   guint8     addr_entry_size = 0;
   int        i;
+  address    save_dl_src;
+  address    save_dl_dst;
+  address    save_net_src;
+  address    save_net_dst;
+  address    save_src;
+  address    save_dst;
+  gboolean   save_in_error_pkt;
+  tvbuff_t   *next_tvb;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "ICMP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "ICMP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   /* To do: check for runts, errs, etc. */
   icmp_type = tvb_get_guint8(tvb, 0);
@@ -1200,7 +1375,14 @@ dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       strcpy(type_str, "Echo (ping) request");
       break;
     case ICMP_RTRADVERT:
-      strcpy(type_str, "Router advertisement");
+      switch (icmp_code) {
+      case 16: /* Mobile-Ip */
+        strcpy(type_str, "Mobile IP Advertisement");
+        break;
+      default:
+        strcpy(type_str, "Router advertisement");
+        break;
+      } /* switch icmp_code */
       break;
     case ICMP_RTRSOLICIT:
       strcpy(type_str, "Router solicitation");
@@ -1241,10 +1423,11 @@ dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       break;
     default:
       strcpy(type_str, "Unknown ICMP (obsolete or malformed?)");
+      break;
   }
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_add_str(pinfo->fd, COL_INFO, type_str);
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_str(pinfo->cinfo, COL_INFO, type_str);
 
   if (tree) {
     length = tvb_length(tvb);
@@ -1298,14 +1481,14 @@ dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  tvb_get_guint8(tvb, 6), tvb_get_guint8(tvb, 7));
 	break;
 
-       case ICMP_UNREACH:
-         switch (icmp_code) {
-           case ICMP_FRAG_NEEDED:
-                 proto_tree_add_text(icmp_tree, tvb, 6, 2, "MTU of next hop: %u",
-                   tvb_get_ntohs(tvb, 6));
-                 break;
-           }
-         break;
+      case ICMP_UNREACH:
+        switch (icmp_code) {
+          case ICMP_FRAG_NEEDED:
+            proto_tree_add_text(icmp_tree, tvb, 6, 2, "MTU of next hop: %u",
+                  tvb_get_ntohs(tvb, 6));
+            break;
+	}
+        break;
 
       case ICMP_RTRADVERT:
         num_addrs = tvb_get_guint8(tvb, 4);
@@ -1339,14 +1522,60 @@ dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* Decode the IP header and first 64 bits of data from the
 	   original datagram.
 
-	   XXX - for now, just display it as data; not all dissection
-	   routines can handle a short packet without exploding. */
-	dissect_data(tvb, 8, pinfo, icmp_tree);
+	   Set the columns non-writable, so that the packet list
+	   shows this as an ICMP packet, not as the type of packet
+	   for which the ICMP packet was generated. */
+	col_set_writable(pinfo->cinfo, FALSE);
+
+	/* Also, save the current values of the addresses, and restore
+	   them when we're finished dissecting the contained packet, so
+	   that the address columns in the summary don't reflect the
+	   contained packet, but reflect this packet instead. */
+	save_dl_src = pinfo->dl_src;
+	save_dl_dst = pinfo->dl_dst;
+	save_net_src = pinfo->net_src;
+	save_net_dst = pinfo->net_dst;
+	save_src = pinfo->src;
+	save_dst = pinfo->dst;
+
+	/* Save the current value of the "we're inside an error packet"
+	   flag, and set that flag; subdissectors may treat packets
+	   that are the payload of error packets differently from
+	   "real" packets. */
+	save_in_error_pkt = pinfo->in_error_pkt;
+	pinfo->in_error_pkt = TRUE;
+
+	/* Dissect the contained packet.
+	   Catch ReportedBoundsError, and do nothing if we see it,
+	   because it's not an error if the contained packet is short;
+	   there's no guarantee that all of it was included.
+
+	   XXX - should catch BoundsError, and re-throw it after cleaning
+	   up. */
+	next_tvb = tvb_new_subset(tvb, 8, -1, -1);
+	TRY {
+	  call_dissector(ip_handle, next_tvb, pinfo, icmp_tree);
+	}
+	CATCH(ReportedBoundsError) {
+	  ; /* do nothing */
+	}
+	ENDTRY;
+
+	/* Restore the "we're inside an error packet" flag. */
+	pinfo->in_error_pkt = save_in_error_pkt;
+
+	/* Restore the addresses. */
+	pinfo->dl_src = save_dl_src;
+	pinfo->dl_dst = save_dl_dst;
+	pinfo->net_src = save_net_src;
+	pinfo->net_dst = save_net_dst;
+	pinfo->src = save_src;
+	pinfo->dst = save_dst;
 	break;
 
       case ICMP_ECHOREPLY:
       case ICMP_ECHO:
-	dissect_data(tvb, 8, pinfo, icmp_tree);
+	call_dissector(data_handle,tvb_new_subset(tvb, 8,-1,tvb_reported_length_remaining(tvb,8)), pinfo, icmp_tree);
 	break;
 
       case ICMP_RTRADVERT:
@@ -1356,10 +1585,14 @@ dissect_icmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	      "Router address: %s",
 	      ip_to_str(tvb_get_ptr(tvb, 8 + (i*8), 4)));
 	    proto_tree_add_text(icmp_tree, tvb, 12 + (i*8), 4,
-	      "Preference level: %u", tvb_get_ntohl(tvb, 12 + (i*8)));
+	      "Preference level: %d", tvb_get_ntohl(tvb, 12 + (i*8)));
+	  }
+	  if (icmp_code == 16) {
+		/* Mobile-Ip */
+		dissect_mip_extensions(tvb,8 + i*8, pinfo, icmp_tree);
 	  }
 	} else
-	  dissect_data(tvb, 8, pinfo, icmp_tree);
+	  call_dissector(data_handle,tvb_new_subset(tvb, 8,-1,tvb_reported_length_remaining(tvb,8)), pinfo, icmp_tree);
 	break;
 
       case ICMP_TSTAMP:
@@ -1541,7 +1774,8 @@ proto_register_ip(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 	/* subdissector code */
-	ip_dissector_table = register_dissector_table("ip.proto");
+	ip_dissector_table = register_dissector_table("ip.proto",
+	    "IP protocol", FT_UINT8, BASE_DEC);
 
 	/* Register configuration options */
 	ip_module = prefs_register_protocol(proto_ip, NULL);
@@ -1565,16 +1799,21 @@ proto_register_ip(void)
 void
 proto_reg_handoff_ip(void)
 {
-	dissector_add("ethertype", ETHERTYPE_IP, dissect_ip, proto_ip);
-	dissector_add("ppp.protocol", PPP_IP, dissect_ip, proto_ip);
-	dissector_add("ppp.protocol", ETHERTYPE_IP, dissect_ip, proto_ip);
-	dissector_add("gre.proto", ETHERTYPE_IP, dissect_ip, proto_ip);
-	dissector_add("gre.proto", GRE_WCCP, dissect_ip, proto_ip);
-	dissector_add("llc.dsap", SAP_IP, dissect_ip, proto_ip);
-	dissector_add("ip.proto", IP_PROTO_IPIP, dissect_ip, proto_ip);
-	dissector_add("null.type", BSD_AF_INET, dissect_ip, proto_ip);
-	dissector_add("chdlctype", ETHERTYPE_IP, dissect_ip, proto_ip);
-	dissector_add("fr.ietf", NLPID_IP, dissect_ip, proto_ip);
+	dissector_handle_t ip_handle;
+
+        data_handle = find_dissector("data");
+        ip_handle = find_dissector("ip");
+	dissector_add("ethertype", ETHERTYPE_IP, ip_handle);
+	dissector_add("ppp.protocol", PPP_IP, ip_handle);
+	dissector_add("ppp.protocol", ETHERTYPE_IP, ip_handle);
+	dissector_add("gre.proto", ETHERTYPE_IP, ip_handle);
+	dissector_add("gre.proto", GRE_WCCP, ip_handle);
+	dissector_add("llc.dsap", SAP_IP, ip_handle);
+	dissector_add("ip.proto", IP_PROTO_IPIP, ip_handle);
+	dissector_add("null.type", BSD_AF_INET, ip_handle);
+	dissector_add("chdlctype", ETHERTYPE_IP, ip_handle);
+	dissector_add("fr.ietf", NLPID_IP, ip_handle);
+	dissector_add("x.25.spi", NLPID_IP, ip_handle);
 }
 
 void
@@ -1597,9 +1836,79 @@ proto_register_icmp(void)
     { &hf_icmp_checksum_bad,
       { "Bad Checksum",	"icmp.checksum_bad",	FT_BOOLEAN, BASE_NONE,	NULL, 0x0,
 	"", HFILL }},
+
+    { &hf_icmp_mip_type,
+      { "Extension Type", "icmp.mip.type",	FT_UINT8, BASE_DEC,
+	VALS(mip_extensions), 0x0,"", HFILL}},
+
+    { &hf_icmp_mip_length,
+      { "Length", "icmp.mip.length",		FT_UINT8, BASE_DEC, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_prefix_length,
+      { "Prefix Length", "icmp.mip.prefixlength",  FT_UINT8, BASE_DEC, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_seq,
+      { "Sequence Number", "icmp.mip.seq",	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_life,
+      { "Registration Lifetime", "icmp.mip.life",  FT_UINT16, BASE_DEC, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_flags,
+      { "Flags", "icmp.mip.flags",            FT_UINT8, BASE_HEX, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_r,
+      { "Registration Required", "icmp.mip.r", FT_BOOLEAN, 8, NULL, 128,
+	"Registration with this FA is required", HFILL }},
+
+    { &hf_icmp_mip_b,
+      { "Busy", "icmp.mip.b", FT_BOOLEAN, 8, NULL, 64,
+	"This FA will not accept requests at this time", HFILL }},
+
+    { &hf_icmp_mip_h,
+      { "Home Agent", "icmp.mip.h", FT_BOOLEAN, 8, NULL, 32,
+	"Home Agent Services Offered", HFILL }},
+
+    { &hf_icmp_mip_f,
+      { "Foreign Agent", "icmp.mip.f", FT_BOOLEAN, 8, NULL, 16,
+	"Foreign Agent Services Offered", HFILL }},
+
+    { &hf_icmp_mip_m,
+      { "Minimal Encapsulation", "icmp.mip.m", FT_BOOLEAN, 8, NULL, 8,
+	"Minimal encapsulation tunneled datagram support", HFILL }},
+
+    { &hf_icmp_mip_g,
+      { "GRE", "icmp.mip.g", FT_BOOLEAN, 8, NULL, 4,
+	"GRE encapsulated tunneled datagram support", HFILL }},
+
+    { &hf_icmp_mip_v,
+      { "VJ Comp", "icmp.mip.v", FT_BOOLEAN, 8, NULL, 2,
+	"Van Jacobson Header Compression Support", HFILL }},
+
+    { &hf_icmp_mip_res,
+      { "Reserved", "icmp.mip.res", FT_BOOLEAN, 8, NULL, 1,
+	"Reserved", HFILL }},
+
+    { &hf_icmp_mip_reserved,
+      { "Reserved", "icmp.mip.reserved",     FT_UINT8, BASE_HEX, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_coa,
+      { "Care-Of-Address", "icmp.mip.coa",    FT_IPv4, BASE_NONE, NULL, 0x0,
+	"", HFILL}},
+
+    { &hf_icmp_mip_challenge,
+      { "Challenge", "icmp.mip.challenge",    FT_BYTES, BASE_NONE, NULL, 0x0,
+	"", HFILL}},
   };
   static gint *ett[] = {
     &ett_icmp,
+	&ett_icmp_mip,
+	&ett_icmp_mip_flags
   };
   
   proto_icmp = proto_register_protocol("Internet Control Message Protocol", 
@@ -1611,5 +1920,13 @@ proto_register_icmp(void)
 void
 proto_reg_handoff_icmp(void)
 {
-  dissector_add("ip.proto", IP_PROTO_ICMP, dissect_icmp, proto_icmp);
+  dissector_handle_t icmp_handle;
+
+  /*
+   * Get handle for the IP dissector.
+   */
+  ip_handle = find_dissector("ip");
+
+  icmp_handle = create_dissector_handle(dissect_icmp, proto_icmp);
+  dissector_add("ip.proto", IP_PROTO_ICMP, icmp_handle);
 }
