@@ -1,12 +1,11 @@
 /* packet-ip.h
  * Definitions for IP packet disassembly structures and routines
  *
- * $Id: packet-ip.h,v 1.19 2001/04/23 17:51:33 guy Exp $
+ * $Id: packet-ip.h,v 1.21 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +26,7 @@
 #ifndef __PACKET_IP_H__
 #define __PACKET_IP_H__
 
-void capture_ip(const u_char *, int, packet_counts *);
+void capture_ip(const u_char *, int, int, packet_counts *);
 
 typedef enum {
   NO_LENGTH,		/* option has no data, hence no length */
@@ -43,13 +42,13 @@ typedef struct ip_tcp_opt {
   opt_len_type len_type; /* type of option length field */
   int	optlen;		/* value length should be (minimum if VARIABLE) */
   void	(*dissect)(const struct ip_tcp_opt *, tvbuff_t *, int, guint,
-  		   frame_data *, proto_tree *);
+  		   packet_info *, proto_tree *);
 			/* routine to dissect option */
 } ip_tcp_opt;
 
 /* Routine to dissect IP or TCP options. */
 void       dissect_ip_tcp_options(tvbuff_t *, int, guint,
-    const ip_tcp_opt *, int, int, frame_data *, proto_tree *);
+    const ip_tcp_opt *, int, int, packet_info *, proto_tree *);
 
 /* Dissector table for "ip.proto"; used by IPv6 as well as IPv4 */
 extern dissector_table_t ip_dissector_table;
