@@ -3,12 +3,11 @@
  *
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-ipp.c,v 1.22 2001/01/11 06:30:54 guy Exp $
+ * $Id: packet-ipp.c,v 1.27 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,8 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +46,8 @@ static gint ett_ipp = -1;
 static gint ett_ipp_as = -1;
 static gint ett_ipp_attr = -1;
 
+static dissector_handle_t data_handle;
+
 #define	PRINT_JOB		0x0002
 #define	PRINT_URI		0x0003
 #define	VALIDATE_JOB		0x0004
@@ -169,17 +168,13 @@ dissect_ipp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint16 status_code;
 	gchar *status_fmt;
 
-	CHECK_DISPLAY_AS_DATA(proto_ipp, tvb, pinfo, tree);
-
-	pinfo->current_proto = "IPP";
-
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IPP");
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPP");
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		if (is_request)
-			col_set_str(pinfo->fd, COL_INFO, "IPP request");
+			col_set_str(pinfo->cinfo, COL_INFO, "IPP request");
 		else
-			col_set_str(pinfo->fd, COL_INFO, "IPP response");
+			col_set_str(pinfo->cinfo, COL_INFO, "IPP response");
 	}
 
 	if (tree) {
@@ -236,7 +231,7 @@ dissect_ipp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		offset = parse_attributes(tvb, offset, ipp_tree);
 
 		if (tvb_offset_exists(tvb, offset))
-			dissect_data(tvb, offset, pinfo, ipp_tree);
+			call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, ipp_tree);
 	}
 }
 
@@ -565,8 +560,12 @@ proto_register_ipp(void)
 void
 proto_reg_handoff_ipp(void)
 {
+	dissector_handle_t ipp_handle;
+
 	/*
 	 * Register ourselves as running atop HTTP and using port 631.
 	 */
-	http_dissector_add(631, dissect_ipp, proto_ipp);
+	ipp_handle = create_dissector_handle(dissect_ipp, proto_ipp);
+	http_dissector_add(631, ipp_handle);
+        data_handle = find_dissector("data");
 }
