@@ -1,7 +1,7 @@
 /* packet-ipv6.c
  * Routines for IPv6 packet disassembly
  *
- * $Id: packet-ipv6.c,v 1.61 2001/06/29 09:46:52 guy Exp $
+ * $Id: packet-ipv6.c,v 1.71 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -47,8 +47,6 @@
 #include "packet-ip.h"
 #include "packet-ipsec.h"
 #include "packet-ipv6.h"
-#include "packet-tcp.h"
-#include "packet-udp.h"
 #include "resolv.h"
 #include "prefs.h"
 #include "reassemble.h"
@@ -56,6 +54,7 @@
 #include "etypes.h"
 #include "ppptypes.h"
 #include "aftypes.h"
+#include "nlpid.h"
 
 /*
  * NOTE: ipv6.nxt is not very useful as we will have chained header.
@@ -107,6 +106,8 @@ static gint ett_ipv6 = -1;
 static gint ett_ipv6_fragments = -1;
 static gint ett_ipv6_fragment  = -1;
 
+static dissector_handle_t data_handle;
+
 /* Reassemble fragmented datagrams */
 static gboolean ipv6_reassemble = FALSE;
 
@@ -128,7 +129,7 @@ ipv6_reassemble_init(void)
 static int
 dissect_routing6(tvbuff_t *tvb, int offset, proto_tree *tree) {
     struct ip6_rthdr rt;
-    int len;
+    guint len;
     proto_tree *rthdr_tree;
 	proto_item *ti;
     char buf[sizeof(struct ip6_rthdr0) + sizeof(struct e_in6_addr) * 23];
@@ -197,8 +198,8 @@ dissect_frag6(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
     frag.ip6f_offlg = ntohs(frag.ip6f_offlg);
     *offlg = frag.ip6f_offlg;
     *ident = frag.ip6f_ident;
-    if (check_col(pinfo->fd, COL_INFO)) {
-	col_add_fstr(pinfo->fd, COL_INFO,
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+	col_add_fstr(pinfo->cinfo, COL_INFO,
 	    "IPv6 fragment (nxt=%s (0x%02x) off=%u id=0x%x)",
 	    ipprotostr(frag.ip6f_nxt), frag.ip6f_nxt,
 	    frag.ip6f_offlg & IP6F_OFF_MASK, frag.ip6f_ident);
@@ -655,16 +656,14 @@ dissect_ipv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   int offset;
   fragment_data *ipfd_head;
   tvbuff_t   *next_tvb;
-  packet_info save_pi;
-  gboolean must_restore_pi = FALSE;
   gboolean update_col_info = TRUE;
 
   struct ip6_hdr ipv6;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "IPv6");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPv6");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   offset = 0;
   tvb_memcpy(tvb, (guint8 *)&ipv6, offset, sizeof(ipv6));
@@ -890,8 +889,8 @@ again:
       }
       if (ipfd_head->flags & (FD_OVERLAPCONFLICT
                         |FD_MULTIPLETAILS|FD_TOOLONGFRAGMENT) ) {
-        if (check_col(pinfo->fd, COL_INFO)) {
-          col_set_str(pinfo->fd, COL_INFO, "[Illegal fragments]");
+        if (check_col(pinfo->cinfo, COL_INFO)) {
+          col_set_str(pinfo->cinfo, COL_INFO, "[Illegal fragments]");
           update_col_info = FALSE;
         }
       }
@@ -910,14 +909,6 @@ again:
 
       /* It's not fragmented. */
       pinfo->fragmented = FALSE;
-
-      /* Save the current value of "pi", and adjust certain fields to
-         reflect the new tvbuff. */
-      save_pi = pi;
-      pi.compat_top_tvb = next_tvb;
-      pi.len = tvb_reported_length(next_tvb);
-      pi.captured_len = tvb_length(next_tvb);
-      must_restore_pi = TRUE;
     } else {
       /* We don't have the complete reassembled payload. */
       next_tvb = NULL;
@@ -951,7 +942,7 @@ again:
   if (next_tvb == NULL) {
     /* Just show this as a fragment. */
     /* COL_INFO was filled in by "dissect_frag6()" */
-    dissect_data(tvb, offset, pinfo, tree);
+    call_dissector(data_handle,tvb_new_subset(tvb, offset, -1,tvb_reported_length_remaining(tvb,offset)),pinfo, tree);
 
     /* As we haven't reassembled anything, we haven't changed "pi", so
        we don't have to restore it. */
@@ -961,9 +952,9 @@ again:
   /* do lookup with the subdissector table */
   if (!dissector_try_port(ip_dissector_table, nxt, next_tvb, pinfo, tree)) {
     /* Unknown protocol */
-    if (check_col(pinfo->fd, COL_INFO))
-      col_add_fstr(pinfo->fd, COL_INFO, "%s (0x%02x)", ipprotostr(nxt),nxt);
-    dissect_data(next_tvb, 0, pinfo, tree);
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_add_fstr(pinfo->cinfo, COL_INFO, "%s (0x%02x)", ipprotostr(nxt),nxt);
+    call_dissector(data_handle,next_tvb, pinfo, tree);
   }
 }
 
@@ -971,11 +962,11 @@ static void
 dissect_ipv6_none(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
     if (hf_ipv6_mipv6_length != -1) {
-	if (check_col(pinfo->fd, COL_INFO))
-	    col_add_fstr(pinfo->fd, COL_INFO, "MobileIPv6 Destination Option");
+	if (check_col(pinfo->cinfo, COL_INFO))
+	    col_add_fstr(pinfo->cinfo, COL_INFO, "Mobile IPv6 Destination Option");
     } else {
-	if (check_col(pinfo->fd, COL_INFO))
-	    col_add_fstr(pinfo->fd, COL_INFO, "IPv6 no next header");
+	if (check_col(pinfo->cinfo, COL_INFO))
+	    col_add_fstr(pinfo->cinfo, COL_INFO, "IPv6 no next header");
     }
     /* XXX - dissect the payload as padding? */
 }
@@ -1162,10 +1153,20 @@ proto_register_ipv6(void)
 void
 proto_reg_handoff_ipv6(void)
 {
-  dissector_add("ethertype", ETHERTYPE_IPv6, dissect_ipv6, proto_ipv6);
-  dissector_add("ppp.protocol", PPP_IPV6, dissect_ipv6, proto_ipv6);
-  dissector_add("ip.proto", IP_PROTO_IPV6, dissect_ipv6, proto_ipv6);
-  dissector_add("null.type", BSD_AF_INET6_BSD, dissect_ipv6, proto_ipv6);
-  dissector_add("null.type", BSD_AF_INET6_FREEBSD, dissect_ipv6, proto_ipv6);
-  dissector_add("ip.proto", IP_PROTO_NONE, dissect_ipv6_none, proto_ipv6);
+  dissector_handle_t ipv6_handle, ipv6_none_handle;
+
+  data_handle = find_dissector("data");
+  ipv6_handle = find_dissector("ipv6");
+  dissector_add("ethertype", ETHERTYPE_IPv6, ipv6_handle);
+  dissector_add("ppp.protocol", PPP_IPV6, ipv6_handle);
+  dissector_add("ppp.protocol", ETHERTYPE_IPv6, ipv6_handle);
+  dissector_add("gre.proto", ETHERTYPE_IPv6, ipv6_handle);
+  dissector_add("ip.proto", IP_PROTO_IPV6, ipv6_handle);
+  ipv6_none_handle = create_dissector_handle(dissect_ipv6_none, proto_ipv6);
+  dissector_add("ip.proto", IP_PROTO_NONE, ipv6_none_handle);
+  dissector_add("null.type", BSD_AF_INET6_BSD, ipv6_handle);
+  dissector_add("null.type", BSD_AF_INET6_FREEBSD, ipv6_handle);
+  dissector_add("chdlctype", ETHERTYPE_IPv6, ipv6_handle);
+  dissector_add("fr.ietf", NLPID_IP6, ipv6_handle);
+  dissector_add("x.25.spi", NLPID_IP6, ipv6_handle);
 }
