@@ -1,8 +1,8 @@
 /* packet-ipx.c
  * Routines for NetWare's IPX
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-ipx.c,v 1.88 2001/07/03 04:56:45 guy Exp $
+ * $Id: packet-ipx.c,v 1.98 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -99,6 +99,8 @@ static int proto_ipxmsg = -1;
 static int hf_msg_conn = -1;
 static int hf_msg_sigchar = -1;
 
+static dissector_handle_t data_handle;
+
 static void
 dissect_spx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
@@ -143,21 +145,19 @@ static const value_string ipx_socket_vals[] = {
 	{ IPX_SOCKET_ADSM,			"ADSM" },
 	{ IPX_SOCKET_EIGRP,			"Cisco EIGRP for IPX" },
 	{ IPX_SOCKET_WIDE_AREA_ROUTER,		"Wide Area Router" },
+	{ 0xE885,				"NT Server-RPC/GW" },
+	{ 0x400C,				"HP LaserJet/QuickSilver" },
+	{ 0x907B,				"SMS Testing and Development" },
+	{ 0x8F83,				"Powerchute UPS Monitoring" },
+	{ 0x4006,				"Netware Directory Server" },
+	{ 0x8104,				"Netware 386" },
 	{ 0x0000,				NULL }
 };
 
 static const char*
 socket_text(guint16 socket)
 {
-	const char	*p;
-
-	p = match_strval(socket, ipx_socket_vals);
-	if (p) {
-		return p;
-	}
-	else {
-		return "Unknown";
-	}
+	return val_to_str(socket, ipx_socket_vals, "Unknown");
 }
 
 static const value_string ipx_packet_type_vals[] = {
@@ -192,7 +192,7 @@ static const value_string ipxmsg_sigchar_vals[] = {
 };
 
 void
-capture_ipx(const u_char *pd, int offset, packet_counts *ld)
+capture_ipx(const u_char *pd, int offset, int len, packet_counts *ld)
 {
 	ld->ipx++;
 }
@@ -212,10 +212,10 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	guint16		ipx_dsocket, ipx_ssocket;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IPX");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPX");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	/* Calculate here for use in pinfo and in tree */
 	ipx_dsocket	= tvb_get_ntohs(tvb, 16);
@@ -234,8 +234,8 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	SET_ADDRESS(&pinfo->net_dst,	AT_IPX, 10, dst_net_node);
 	SET_ADDRESS(&pinfo->dst,	AT_IPX, 10, dst_net_node);
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_add_fstr(pinfo->fd, COL_INFO, "%s (0x%04X)",
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_fstr(pinfo->cinfo, COL_INFO, "%s (0x%04x)",
 				socket_text(ipx_dsocket), ipx_dsocket);
 
 	if (tree) {
@@ -254,15 +254,13 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		/* Destination */
 		proto_tree_add_item(ipx_tree, hf_ipx_dnet, tvb, 6, 4, FALSE);
 		proto_tree_add_item(ipx_tree, hf_ipx_dnode, tvb, 10, 6, FALSE);
-		proto_tree_add_uint_format(ipx_tree, hf_ipx_dsocket, tvb, 16, 2,
-			ipx_dsocket, "Destination Socket: %s (0x%04X)",
-			socket_text(ipx_dsocket), ipx_dsocket);
+		proto_tree_add_uint(ipx_tree, hf_ipx_dsocket, tvb, 16, 2,
+			ipx_dsocket);
 
 		/* Source */
 		proto_tree_add_item(ipx_tree, hf_ipx_snet, tvb, 18, 4, FALSE);
 		proto_tree_add_item(ipx_tree, hf_ipx_snode, tvb, 22, 6, FALSE);
-		proto_tree_add_uint_format(ipx_tree, hf_ipx_ssocket, tvb, 28, 2,
-			ipx_ssocket, "Source Socket: %s (0x%04X)", socket_text(ipx_ssocket),
+		proto_tree_add_uint(ipx_tree, hf_ipx_ssocket, tvb, 28, 2,
 			ipx_ssocket);
 	}
 
@@ -284,7 +282,7 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (dissector_try_port(ipx_socket_dissector_table, ipx_ssocket,
 	    next_tvb, pinfo, tree))
 		return;
-	dissect_data(next_tvb, 0, pinfo, tree);
+	call_dissector(data_handle,next_tvb, pinfo, tree);
 }
 
 
@@ -339,10 +337,10 @@ dissect_spx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8		conn_ctrl;
 	guint8		datastream_type;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "SPX");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_set_str(pinfo->fd, COL_INFO, "SPX");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SPX");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_set_str(pinfo->cinfo, COL_INFO, "SPX");
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_spx, tvb, 0, SPX_HEADER_LEN, FALSE);
@@ -367,7 +365,7 @@ dissect_spx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		proto_tree_add_item(spx_tree, hf_spx_all_nr, tvb, 10, 2, FALSE);
 
 		next_tvb = tvb_new_subset(tvb, SPX_HEADER_LEN, -1, -1);
-		dissect_data(next_tvb, 0, pinfo, tree);
+		call_dissector(data_handle,next_tvb, pinfo, tree);
 	}
 }
 
@@ -381,16 +379,16 @@ dissect_ipxmsg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	proto_item	*ti;
 	guint8		conn_number, sig_char;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IPX MSG");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPX MSG");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	conn_number = tvb_get_guint8(tvb, 0);
 	sig_char = tvb_get_guint8(tvb, 1);
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO, 
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO, 
 			"%s, Connection %d", 
 			val_to_str(sig_char, ipxmsg_sigchar_vals, "Unknown Signature Char"), conn_number);
 	}
@@ -420,16 +418,16 @@ dissect_ipxrip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	char		*rip_type[3] = { "Request", "Response", "Unknown" };
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IPX RIP");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPX RIP");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	operation = tvb_get_ntohs(tvb, 0) - 1;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		/* rip_types 0 and 1 are valid, anything else becomes 2 or "Unknown" */
-		col_add_str(pinfo->fd, COL_INFO, rip_type[MIN(operation, 2)]);
+		col_add_str(pinfo->cinfo, COL_INFO, rip_type[MIN(operation, 2)]);
 	}
 
 	if (tree) {
@@ -490,59 +488,238 @@ server_type(guint16 type)
 {
 	const char *p;
 
-	/* some of these are from ncpfs, others are from the book */
+	/*
+	 * Some of these are from ncpfs, others are from the book,
+	 * others are from the page at
+	 *
+	 *	http://www.iana.org/assignments/novell-sap-numbers
+	 *
+	 * and some from the page at
+	 *
+	 *	http://www.rware.demon.co.uk/ipxsap.htm
+	 *
+	 * (see also the page at
+	 *
+	 *	http://developer.novell.com/research/appnotes/1998/february/03/06.htm
+	 *
+	 * which has a huge list - but many of the entries list only the
+	 * organization owning the SAP type, not what the type is for).
+	 */
 	static const value_string server_vals[] = {
+		{ 0x0000,	"Unknown" },
 		{ 0x0001,	"User" },
 		{ 0x0002,	"User Group" },
-		{ 0x0003,	"Print Queue" },
-		{ 0x0004,	"File server" },
-		{ 0x0005,	"Job server" },
-		{ 0x0007,	"Print server" },
-		{ 0x0008,	"Archive server" },
-		{ 0x0009,	"Archive server" },
-		{ 0x000a,	"Job queue" },
+		{ 0x0003,	"Print Queue or Print Group" },
+		{ 0x0004,	"File Server (SLIST source)" },
+		{ 0x0005,	"Job Server" },
+		{ 0x0006,	"Gateway" },
+		{ 0x0007,	"Print Server or Silent Print Server" },
+		{ 0x0008,	"Archive Queue" },
+		{ 0x0009,	"Archive Server" },
+		{ 0x000a,	"Job Queue" },
 		{ 0x000b,	"Administration" },
-		{ 0x0021,	"NAS SNA gateway" },
-		{ 0x0024,	"Remote bridge" },
-		{ 0x0026,	"Bridge server" },
-		{ 0x0027,	"TCP/IP gateway" },
-		{ 0x002d,	"Time Synchronization VAP" },
-		{ 0x002e,	"Archive Server Dynamic SAP" },
-		{ 0x0047,	"Advertising print server" },
-		{ 0x004b,	"Btrieve VAP 5.0" },
-		{ 0x004c,	"SQL VAP" },
-		{ 0x0050,	"Btrieve VAP" },
-		{ 0x0053,	"Print Queue VAP" },
-		{ 0x007a,	"TES NetWare for VMS" },
-		{ 0x0098,	"NetWare access server" },
-		{ 0x009a,	"Named Pipes server" },
-		{ 0x009e,	"Portable NetWare Unix" },
-		{ 0x0107,	"NetWare 386" },
-		{ 0x0111,	"Test server" },
-		{ 0x0133,	"NetWare Name Service" },
-		{ 0x0166,	"NetWare management" },
+		{ 0x000F,	"Novell TI-RPC" },
+		{ 0x0017,	"Diagnostics" },
+		{ 0x0020,	"NetBIOS" },
+		{ 0x0021,	"NAS SNA Gateway" },
+		{ 0x0023,	"NACS Async Gateway or Asynchronous Gateway" },
+		{ 0x0024,	"Remote Bridge or Routing Service" },
+		{ 0x0026,	"Bridge Server or Asynchronous Bridge Server" },
+		{ 0x0027,	"TCP/IP Gateway Server" },
+		{ 0x0028,	"Point to Point (Eicon) X.25 Bridge Server" },
+		{ 0x0029,	"Eicon 3270 Gateway" },
+		{ 0x002a,	"CHI Corp" },
+		{ 0x002c,	"PC Chalkboard" },
+		{ 0x002d,	"Time Synchronization Server or Asynchronous Timer" },
+		{ 0x002e,	"ARCserve 5.0 / Palindrome Backup Director 4.x (PDB4)" },
+		{ 0x0045,	"DI3270 Gateway" },
+		{ 0x0047,	"Advertising Print Server" },
+		{ 0x004a,	"NetBlazer Modems" },
+		{ 0x004b,	"Btrieve VAP/NLM 5.0" },
+		{ 0x004c,	"Netware SQL VAP/NLM Server" },
+		{ 0x004d,	"Xtree Network Version/Netware XTree" },
+		{ 0x0050,	"Btrieve VAP 4.11" },
+		{ 0x0052,	"QuickLink (Cubix)" },
+		{ 0x0053,	"Print Queue User" },
+		{ 0x0058,	"Multipoint X.25 Eicon Router" },
+		{ 0x0060,	"STLB/NLM" },
+		{ 0x0064,	"ARCserve" },
+		{ 0x0066,	"ARCserve 3.0" },
+		{ 0x0072,	"WAN Copy Utility" },
+		{ 0x007a,	"TES-Netware for VMS" },
+		{ 0x0092,	"WATCOM Debugger or Emerald Tape Backup Server" },
+		{ 0x0095,	"DDA OBGYN" },
+		{ 0x0098,	"Netware Access Server (Asynchronous gateway)" },
+		{ 0x009a,	"Netware for VMS II or Named Pipe Server" },
+		{ 0x009b,	"Netware Access Server" },
+		{ 0x009e,	"Portable Netware Server or SunLink NVT" },
+		{ 0x00a1,	"Powerchute APC UPS NLM" },
+		{ 0x00aa,	"LAWserve" },
+		{ 0x00ac,	"Compaq IDA Status Monitor" },
+		{ 0x0100,	"PIPE STAIL" },
+		{ 0x0102,	"LAN Protect Bindery" },
+		{ 0x0103,	"Oracle DataBase Server" },
+		{ 0x0107,	"Netware 386 or RSPX Remote Console" },
+		{ 0x010f,	"Novell SNA Gateway" },
+		{ 0x0111,	"Test Server" },
+		{ 0x0112,	"Print Server (HP)" },
+		{ 0x0114,	"CSA MUX (f/Communications Executive)" },
+		{ 0x0115,	"CSA LCA (f/Communications Executive)" },
+		{ 0x0116,	"CSA CM (f/Communications Executive)" },
+		{ 0x0117,	"CSA SMA (f/Communications Executive)" },
+		{ 0x0118,	"CSA DBA (f/Communications Executive)" },
+		{ 0x0119,	"CSA NMA (f/Communications Executive)" },
+		{ 0x011a,	"CSA SSA (f/Communications Executive)" },
+		{ 0x011b,	"CSA STATUS (f/Communications Executive)" },
+		{ 0x011e,	"CSA APPC (f/Communications Executive)" },
+		{ 0x0126,	"SNA TEST SSA Profile" },
+		{ 0x012a,	"CSA TRACE (f/Communications Executive)" },
+		{ 0x012b,	"Netware for SAA" },
+		{ 0x012e,	"IKARUS virus scan utility" },
+		{ 0x0130,	"Communications Executive" },
+		{ 0x0133,	"NNS Domain Server or Netware Naming Services Domain" },
+		{ 0x0135,	"Netware Naming Services Profile" },
+		{ 0x0137,	"Netware 386 Print Queue or NNS Print Queue" },
+		{ 0x0141,	"LAN Spool Server (Vap, Intel)" },
+		{ 0x0152,	"IRMALAN Gateway" },
+		{ 0x0154,	"Named Pipe Server" },
+		{ 0x0166,	"NetWare Management" },
+		{ 0x0168,	"Intel PICKIT Comm Server or Intel CAS Talk Server" },
+		{ 0x0173,	"Compaq" },
+		{ 0x0174,	"Compaq SNMP Agent" },
+		{ 0x0175,	"Compaq" },
+		{ 0x0180,	"XTree Server or XTree Tools" },
+		{ 0x018A,	"NASI services broadcast server (Novell)" },
+		{ 0x01b0,	"GARP Gateway (net research)" },
+		{ 0x01b1,	"Binfview (Lan Support Group)" },
+		{ 0x01bf,	"Intel LanDesk Manager" },
+		{ 0x01ca,	"AXTEC" },
+		{ 0x01cb,	"Shiva NetModem/E" },
+		{ 0x01cc,	"Shiva LanRover/E" },
+		{ 0x01cd,	"Shiva LanRover/T" },
+		{ 0x01ce,	"Shiva Universal" },
+		{ 0x01d8,	"Castelle FAXPress Server" },
+		{ 0x01da,	"Castelle LANPress Print Server" },
+		{ 0x01dc,	"Castelle FAX/Xerox 7033 Fax Server/Excel Lan Fax" },
+		{ 0x01f0,	"LEGATO" },
+		{ 0x01f5,	"LEGATO" },
+		{ 0x0233,	"NMS Agent or Netware Management Agent" },
+		{ 0x0237,	"NMS IPX Discovery or LANtern Read/Write Channel" },
+		{ 0x0238,	"NMS IP Discovery or LANtern Trap/Alarm Channel" },
+		{ 0x023a,	"LANtern" },
+		{ 0x023c,	"MAVERICK" },
 		{ 0x023f,	"SMS Testing and Development" },
-		{ 0x026a,	"NetWare management" },
-		{ 0x026b,	"Time synchronization" },
-		{ 0x027b,	"NetWare Management Agent" },
-		{ 0x0278,	"NetWare Directory server" },
-		{ 0x030c,	"HP LaserJet / Quick Silver" },
-		{ 0x0355,	"Arcada Software" },
+		{ 0x024e,	"Netware Connect" },
+		{ 0x024f,	"NASI server broadcast (Cisco)" },
+		{ 0x026a,	"Network Management (NMS) Service Console" },
+		{ 0x026b,	"Time Synchronization Server (Netware 4.x)" },
+		{ 0x0278,	"Directory Server (Netware 4.x)" },
+		{ 0x027b,	"Netware Management Agent" },
+		{ 0x0280,	"Novell File and Printer Sharing Service for PC" },
+		{ 0x0304,	"Novell SAA Gateway" },
+		{ 0x0308,	"COM or VERMED 1" },
+		{ 0x030a,	"Galacticomm's Worldgroup Server" },
+		{ 0x030c,	"Intel Netport 2 or HP JetDirect or HP Quicksilver" },
+		{ 0x0320,	"Attachmate Gateway" },
+		{ 0x0327,	"Microsoft Diagnostics" },
+		{ 0x0328,	"WATCOM SQL server" },
+		{ 0x0335,	"MultiTech Systems Multisynch Comm Server" },
+		{ 0x0343,	"Xylogics Remote Access Server or LAN Modem" },
+		{ 0x0355,	"Arcada Backup Exec" },
+		{ 0x0358,	"MSLCD1" },
 		{ 0x0361,	"NETINELO" },
 		{ 0x037e,	"Powerchute UPS Monitoring" },
-		{ 0x03e1,	"UnixWare Application Server" },
-		{ 0x044c,	"Archive" },
+		{ 0x037f,	"ViruSafe Notify" },
+		{ 0x0386,	"HP Bridge" },
+		{ 0x0387,	"HP Hub" },
+		{ 0x0394,	"NetWare SAA Gateway" },
+		{ 0x039b,	"Lotus Notes" },
+		{ 0x03b7,	"Certus Anti Virus NLM" },
+		{ 0x03c4,	"ARCserve 4.0 (Cheyenne)" },
+		{ 0x03c7,	"LANspool 3.5 (Intel)" },
+		{ 0x03d7,	"Lexmark printer server (type 4033-011)" },
+		{ 0x03d8,	"Lexmark XLE printer server (type 4033-301)" },
+		{ 0x03dd,	"Banyan ENS for Netware Client NLM" },
+		{ 0x03de,	"Gupta Sequel Base Server or NetWare SQL" },
+		{ 0x03e1,	"Univel Unixware" },
+		{ 0x03e4,	"Univel Unixware" },
+		{ 0x03fc,	"Intel Netport" },
+		{ 0x03fd,	"Intel Print Server Queue" },
+		{ 0x040A,	"ipnServer" },
+		{ 0x040D,	"LVERRMAN" },
+		{ 0x040E,	"LVLIC" },
+		{ 0x0414,	"NET Silicon (DPI)/Kyocera" },
+		{ 0x0429,	"Site Lock Virus (Brightworks)" },
+		{ 0x0432,	"UFHELP R" },
+		{ 0x0433,	"Synoptics 281x Advanced SNMP Agent" },
+		{ 0x0444,	"Microsoft NT SNA Server" },
+		{ 0x0448,	"Oracle" },
+		{ 0x044c,	"ARCserve 5.01" },
+		{ 0x0457,	"Canon GP55 Running on a Canon GP55 network printer" },
+		{ 0x045a,	"QMS Printers" },
+		{ 0x045b,	"Dell SCSI Array (DSA) Monitor" },
+		{ 0x0491,	"NetBlazer Modems" },
+		{ 0x04ac,	"On-Time Scheduler NLM" },
+		{ 0x04b0,	"CD-Net (Meridian)" },
+		{ 0x0513,	"Emulex NQA" },
+		{ 0x0520,	"Site Lock Checks" },
+		{ 0x0529,	"Site Lock Checks (Brightworks)" },
+		{ 0x052d,	"Citrix OS/2 App Server" },
+		{ 0x0535,	"Tektronix" },
+		{ 0x0536,	"Milan" },
 		{ 0x055d,	"Attachmate SNA gateway" },
+		{ 0x056b,	"IBM 8235 modem server" },
+		{ 0x056c,	"Shiva LanRover/E PLUS" },
+		{ 0x056d,	"Shiva LanRover/T PLUS" },
+		{ 0x0580,	"McAfee's NetShield anti-virus" },
+		{ 0x05B8,	"NLM to workstation communication (Revelation Software)" },
+		{ 0x05BA,	"Compatible Systems Routers" },
+		{ 0x05BE,	"Cheyenne Hierarchical Storage Manager" },
+		{ 0x0606,	"JCWatermark Imaging" },
+		{ 0x060c,	"AXIS Network Printer" },
 		{ 0x0610,	"Adaptec SCSI Management" },
-		{ 0x0640,	"NT Server-RPC/GW for NW/Win95 User Level Sec" },
-		{ 0x064e,	"NT Server-IIS" },
+		{ 0x0621,	"IBM AntiVirus NLM" },
+		{ 0x0640,	"Microsoft Gateway Services for NetWare" },
+/*		{ 0x0640,	"NT Server-RPC/GW for NW/Win95 User Level Sec" }, */
+		{ 0x064e,	"Microsoft Internet Information Server" },
+		{ 0x067b,	"Microsoft Win95/98 File and Print Sharing for NetWare" },
+		{ 0x067c,	"Microsoft Win95/98 File and Print Sharing for NetWare" },
+		{ 0x076C,	"Xerox" },
+		{ 0x079b,	"Shiva LanRover/E 115" },
+		{ 0x079c,	"Shiva LanRover/T 115" },
+		{ 0x07B4,	"Cubix WorldDesk" },
+		{ 0x07c2,	"Quarterdeck IWare Connect V2.x NLM" },
+		{ 0x07c1,	"Quarterdeck IWare Connect V3.x NLM" },
 		{ 0x0810,	"ELAN License Server Demo" },
-		{ 0x8002,	"Intel NetPort Print Server" },
-
-/* For any unidentified ones, I found a really big list of them at: */
-/*    http://www.inpnet.org/cnpweb/saplist.txt */
-/*    http://www.isi.edu/in-notes/iana/assignments/novell-sap-numbers */
-
+		{ 0x0824,	"Shiva LanRover Access Switch/E" },
+		{ 0x086a,	"ISSC collector NLMs" },
+		{ 0x087f,	"ISSC DAS agent for AIX" },
+		{ 0x0880,	"Intel Netport PRO" },
+		{ 0x0881,	"Intel Netport PRO" },
+		{ 0x0b29,	"Site Lock" },
+		{ 0x0c29,	"Site Lock Applications" },
+		{ 0x0c2c,	"Licensing Server" },
+		{ 0x2101,	"Performance Technology Instant Internet" },
+		{ 0x2380,	"LAI Site Lock" },
+		{ 0x238c,	"Meeting Maker" },
+		{ 0x4808,	"Site Lock Server or Site Lock Metering VAP/NLM" },
+		{ 0x5555,	"Site Lock User" },
+		{ 0x6312,	"Tapeware" },
+		{ 0x6f00,	"Rabbit Gateway (3270)" },
+		{ 0x7703,	"MODEM" },
+		{ 0x8002,	"NetPort Printers (Intel) or LANport" },
+		{ 0x8008,	"WordPerfect Network Version" },
+		{ 0x85BE,	"Cisco Enhanced Interior Routing Protocol (EIGRP)" },
+		{ 0x8888,	"WordPerfect Network Version or Quick Network Management" },
+		{ 0x9000,	"McAfee's NetShield anti-virus" },
+		{ 0x9604,	"CSA-NT_MON" },
+		{ 0xb6a8,	"Ocean Isle Reachout Remote Control" },
+		{ 0xf11f,	"Site Lock Metering VAP/NLM" },
+		{ 0xf1ff,	"Site Lock" },
+		{ 0xf503,	"Microsoft SQL Server" },
+		{ 0xf905,	"IBM Time and Place/2 application" },
+		{ 0xfbfb,	"TopCall III fax server" },
+		{ 0xffff,	"Any Service or Wildcard" },
 		{ 0x0000,	NULL }
 	};
 
@@ -567,20 +744,20 @@ dissect_ipxsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	char		*sap_type[4] = { "General Query", "General Response",
 		"Nearest Query", "Nearest Response" };
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IPX SAP");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPX SAP");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	query.query_type = tvb_get_ntohs(tvb, 0);
 	query.server_type = tvb_get_ntohs(tvb, 2);
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		if (query.query_type >= 1 && query.query_type <= 4) {
-			col_add_str(pinfo->fd, COL_INFO, sap_type[query.query_type - 1]);
+			col_add_str(pinfo->cinfo, COL_INFO, sap_type[query.query_type - 1]);
 		}
 		else {
-			col_set_str(pinfo->fd, COL_INFO, "Unknown Packet Type");
+			col_set_str(pinfo->cinfo, COL_INFO, "Unknown Packet Type");
 		}
 	}
 
@@ -627,7 +804,7 @@ dissect_ipxsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 						ipxnet_to_string((guint8*)tvb_get_ptr(tvb, cursor+50, 4)));
 				proto_tree_add_text(s_tree, tvb, cursor+54, 6, "Node: %s",
 						ether_to_str((guint8*)tvb_get_ptr(tvb, cursor+54, 6)));
-				proto_tree_add_text(s_tree, tvb, cursor+60, 2, "Socket: %s (0x%04X)",
+				proto_tree_add_text(s_tree, tvb, cursor+60, 2, "Socket: %s (0x%04x)",
 						socket_text(server.server_port), server.server_port);
 				proto_tree_add_text(s_tree, tvb, cursor+62, 2,
 						"Intermediate Networks: %d",
@@ -671,7 +848,8 @@ proto_register_ipx(void)
 			"", HFILL }},
 
 		{ &hf_ipx_dsocket,
-		{ "Destination Socket",	"ipx.dst.socket", FT_UINT16, BASE_HEX, NULL, 0x0,
+		{ "Destination Socket",	"ipx.dst.socket", FT_UINT16, BASE_HEX,
+			VALS(ipx_socket_vals), 0x0,
 			"", HFILL }},
 
 		{ &hf_ipx_snet,
@@ -683,7 +861,8 @@ proto_register_ipx(void)
 			"", HFILL }},
 
 		{ &hf_ipx_ssocket,
-		{ "Source Socket",	"ipx.src.socket", FT_UINT16, BASE_HEX, NULL, 0x0,
+		{ "Source Socket",	"ipx.src.socket", FT_UINT16, BASE_HEX,
+			VALS(ipx_socket_vals), 0x0,
 			"", HFILL }},
 	};
 
@@ -795,26 +974,34 @@ proto_register_ipx(void)
 
 	proto_register_subtree_array(ett, array_length(ett));
 
-	ipx_type_dissector_table = register_dissector_table("ipx.packet_type");
-	ipx_socket_dissector_table = register_dissector_table("ipx.socket");
+	ipx_type_dissector_table = register_dissector_table("ipx.packet_type",
+	    "IPX packet type", FT_UINT8, BASE_HEX);
+	ipx_socket_dissector_table = register_dissector_table("ipx.socket",
+	    "IPX socket", FT_UINT16, BASE_HEX);
 }
 
 void
 proto_reg_handoff_ipx(void)
 {
-	dissector_add("udp.port", UDP_PORT_IPX, dissect_ipx, proto_ipx);
-	dissector_add("ethertype", ETHERTYPE_IPX, dissect_ipx, proto_ipx);
-	dissector_add("chdlctype", ETHERTYPE_IPX, dissect_ipx, proto_ipx);
-	dissector_add("ppp.protocol", PPP_IPX, dissect_ipx, proto_ipx);
-	dissector_add("llc.dsap", SAP_NETWARE, dissect_ipx, proto_ipx);
-	dissector_add("null.type", BSD_AF_IPX, dissect_ipx, proto_ipx);
-	dissector_add("gre.proto", ETHERTYPE_IPX, dissect_ipx, proto_ipx);
-	dissector_add("ipx.packet_type", IPX_PACKET_TYPE_SPX, dissect_spx,
-	    proto_spx);
-	dissector_add("ipx.socket", IPX_SOCKET_SAP, dissect_ipxsap,
-	    proto_sap);
-	dissector_add("ipx.socket", IPX_SOCKET_IPXRIP, dissect_ipxrip,
-	    proto_ipxrip);
-	dissector_add("ipx.socket", IPX_SOCKET_IPX_MESSAGE, dissect_ipxmsg,
-	    proto_ipxmsg);
+	dissector_handle_t ipx_handle, spx_handle;
+	dissector_handle_t ipxsap_handle, ipxrip_handle;
+	dissector_handle_t ipxmsg_handle;
+
+	ipx_handle = find_dissector("ipx");
+	dissector_add("udp.port", UDP_PORT_IPX, ipx_handle);
+	dissector_add("ethertype", ETHERTYPE_IPX, ipx_handle);
+	dissector_add("chdlctype", ETHERTYPE_IPX, ipx_handle);
+	dissector_add("ppp.protocol", PPP_IPX, ipx_handle);
+	dissector_add("llc.dsap", SAP_NETWARE, ipx_handle);
+	dissector_add("null.type", BSD_AF_IPX, ipx_handle);
+	dissector_add("gre.proto", ETHERTYPE_IPX, ipx_handle);
+	spx_handle = create_dissector_handle(dissect_spx, proto_spx);
+	dissector_add("ipx.packet_type", IPX_PACKET_TYPE_SPX, spx_handle);
+	ipxsap_handle = find_dissector("ipxsap");
+	dissector_add("ipx.socket", IPX_SOCKET_SAP, ipxsap_handle);
+	ipxrip_handle = create_dissector_handle(dissect_ipxrip, proto_ipxrip);
+	dissector_add("ipx.socket", IPX_SOCKET_IPXRIP, ipxrip_handle);
+	ipxmsg_handle = create_dissector_handle(dissect_ipxmsg, proto_ipxmsg);
+	dissector_add("ipx.socket", IPX_SOCKET_IPX_MESSAGE, ipxmsg_handle);
+	data_handle = find_dissector("data");
 }
