@@ -1,10 +1,10 @@
 /* packet-irc.c
  * Routines for IRC packet dissection
  *
- * $Id: packet-irc.c,v 1.14 2001/06/18 02:17:47 guy Exp $
+ * $Id: packet-irc.c,v 1.16 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
@@ -81,12 +81,12 @@ dissect_irc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gint		next_offset;
 	int		linelen;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "IRC");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IRC");
 
-	if (check_col(pinfo->fd, COL_INFO))
+	if (check_col(pinfo->cinfo, COL_INFO))
 	{
-		col_set_str(pinfo->fd, COL_INFO,
+		col_set_str(pinfo->cinfo, COL_INFO,
 		    (pinfo->match_port == pinfo->destport) ? "Request" : "Response");
 	}
 
@@ -164,6 +164,8 @@ proto_register_irc(void)
 void
 proto_reg_handoff_irc(void)
 {
-	dissector_add("tcp.port", TCP_PORT_IRC, dissect_irc, proto_irc);
-}
+	dissector_handle_t irc_handle;
 
+	irc_handle = create_dissector_handle(dissect_irc, proto_irc);
+	dissector_add("tcp.port", TCP_PORT_IRC, irc_handle);
+}
