@@ -2,7 +2,7 @@
  * Routines for ISO/OSI network and transport protocol packet disassembly, core
  * bits.
  *
- * $Id: packet-isis.c,v 1.25 2001/07/02 02:09:26 guy Exp $
+ * $Id: packet-isis.c,v 1.28 2001/12/20 07:33:21 guy Exp $
  * Stuart Stanley <stuarts@mxmail.net>
  *
  * Ethereal - Network traffic analyzer
@@ -22,8 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -129,15 +127,15 @@ dissect_isis(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8 isis_type;
 	guint8 isis_system_id_len;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "ISIS");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "ISIS");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	isis_version = tvb_get_guint8(tvb, 2);
 	if (isis_version != ISIS_REQUIRED_VERSION){
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO,
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 				"Unknown ISIS version (%u vs %u)",
 				isis_version, ISIS_REQUIRED_VERSION );
 		}
@@ -181,8 +179,8 @@ dissect_isis(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	isis_type_reserved = tvb_get_guint8(tvb, 4);
 	isis_type = isis_type_reserved & ISIS_TYPE_MASK;
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_str(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_str(pinfo->cinfo, COL_INFO,
 			val_to_str ( isis_type, isis_vals, "Unknown (0x%x)" ) );
 	}
 	if (tree) {
@@ -316,10 +314,22 @@ proto_register_isis(void) {
     proto_isis = proto_register_protocol(PROTO_STRING_ISIS, "ISIS", "isis");
     proto_register_field_array(proto_isis, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
+
+    /*
+     * Call registration routines for other source files in the ISIS
+     * dissector.
+     */
+    isis_register_hello(proto_isis);
+    isis_register_lsp(proto_isis);
+    isis_register_csnp(proto_isis);
+    isis_register_psnp(proto_isis);
 }
 
 void
 proto_reg_handoff_isis(void)
 {
-    dissector_add("osinl", NLPID_ISO10589_ISIS, dissect_isis, proto_isis);
+    dissector_handle_t isis_handle;
+
+    isis_handle = create_dissector_handle(dissect_isis, proto_isis);
+    dissector_add("osinl", NLPID_ISO10589_ISIS, isis_handle);
 }
