@@ -1,12 +1,11 @@
 /* packet-isl.c
  * Routines for Cisco ISL Ethernet header disassembly
  *
- * $Id: packet-isl.c,v 1.25 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-isl.c,v 1.29 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -84,13 +83,14 @@ static gint ett_isl = -1;
 
 static dissector_handle_t eth_handle;
 static dissector_handle_t tr_handle;
+static dissector_handle_t data_handle;
 
 void
-capture_isl(const u_char *pd, int offset, packet_counts *ld)
+capture_isl(const u_char *pd, int offset, int len, packet_counts *ld)
 {
   guint8 type;
 
-  if (!BYTES_ARE_IN_FRAME(offset, ISL_HEADER_SIZE)) {
+  if (!BYTES_ARE_IN_FRAME(offset, len, ISL_HEADER_SIZE)) {
     ld->other++;
     return;
   }
@@ -101,12 +101,12 @@ capture_isl(const u_char *pd, int offset, packet_counts *ld)
 
   case TYPE_ETHER:
     offset += 14+12;	/* skip the header */
-    capture_eth(pd, offset, ld);
+    capture_eth(pd, offset, len, ld);
     break;
 
   case TYPE_TR:
     offset += 14+17;	/* skip the header */
-    capture_tr(pd, offset, ld);
+    capture_tr(pd, offset, len, ld);
     break;
 
   default:
@@ -151,13 +151,11 @@ dissect_isl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   gint crc_offset;
   gint captured_length;
   tvbuff_t *next_tvb;
-  const guint8 *compat_pd;
-  int compat_offset;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "ISL");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "ISL");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   type = (tvb_get_guint8(tvb, 5) >> 4)&0x0F;
 
@@ -196,8 +194,8 @@ dissect_isl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        field (which is, admittedly, an OUI). */
     proto_tree_add_item(fh_tree, hf_isl_hsa, tvb, 17, 3, FALSE);
   }
-  if (check_col(pinfo->fd, COL_INFO))
-    col_add_fstr(pinfo->fd, COL_INFO, "VLAN ID: 0x%04X",
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO, "VLAN ID: 0x%04X",
 		 tvb_get_ntohs(tvb, 20) >> 1);
   if (tree) {
     proto_tree_add_item(fh_tree, hf_isl_vlan_id, tvb, 20, 2, FALSE);
@@ -242,15 +240,6 @@ dissect_isl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
         next_tvb = tvb_new_subset(tvb, ISL_HEADER_SIZE, captured_length, length);
 
-        /* Set "pinfo"'s payload and captured-payload lengths to the values
-           we calculated.
-
-           XXX - when all dissectors are tvbuffified we shouldn't have to
-           do this any more. */
-        tvb_compat(next_tvb, &compat_pd, &compat_offset);
-        pinfo->len = compat_offset + length;
-        pinfo->captured_len = compat_offset + captured_length;
-
         call_dissector(eth_handle, next_tvb, pinfo, tree);
       }
     }
@@ -271,7 +260,7 @@ dissect_isl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   default:
     next_tvb = tvb_new_subset(tvb, ISL_HEADER_SIZE, -1, -1);
-    dissect_data(next_tvb, 0, pinfo, tree);
+    call_dissector(data_handle,next_tvb, pinfo, tree);
     break;
   }
 }
@@ -356,4 +345,5 @@ proto_reg_handoff_isl(void)
    */
   eth_handle = find_dissector("eth");
   tr_handle = find_dissector("tr");
+  data_handle = find_dissector("data");
 }
