@@ -8,7 +8,7 @@
  *
  * Copyright 2000, Michael Txen <Michael.Tuexen@icn.siemens.de>
  *
- * $Id: packet-iua.c,v 1.7 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-iua.c,v 1.10 2001/12/10 00:25:29 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -423,9 +423,9 @@ dissect_iua_common_header(tvbuff_t *common_header_tvb, packet_info *pinfo, proto
   message_type   = tvb_get_guint8(common_header_tvb, MESSAGE_TYPE_OFFSET);
   message_length = tvb_get_ntohl (common_header_tvb, MESSAGE_LENGTH_OFFSET);
 
-  if (check_col(pinfo->fd, COL_INFO)) {
-    col_append_str(pinfo->fd, COL_INFO, val_to_str(message_class * 256 + message_type, iua_message_class_type_acro_values, "UNKNOWN"));
-    col_append_str(pinfo->fd, COL_INFO, " ");
+  if (check_col(pinfo->cinfo, COL_INFO)) {
+    col_append_str(pinfo->cinfo, COL_INFO, val_to_str(message_class * 256 + message_type, iua_message_class_type_acro_values, "UNKNOWN"));
+    col_append_str(pinfo->cinfo, COL_INFO, " ");
   };
 
   if (iua_tree) {
@@ -844,8 +844,8 @@ dissect_iua(tvbuff_t *message_tvb, packet_info *pinfo, proto_tree *tree)
   proto_tree *iua_tree;
 
   /* make entry in the Protocol column on summary display */
-  if (check_col(pinfo->fd, COL_PROTOCOL)) 
-    col_add_str(pinfo->fd, COL_PROTOCOL, "IUA");
+  if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+    col_add_str(pinfo->cinfo, COL_PROTOCOL, "IUA");
   
   /* In the interest of speed, if "tree" is NULL, don't do any work not
      necessary to generate protocol tree items. */
@@ -913,27 +913,27 @@ proto_register_iua(void)
 	"", HFILL }
     }, 
     { &hf_iua_spare_bit,
-      { "Spare bit", "hf.iua.spare_bit",
+      { "Spare bit", "iua.spare_bit",
 	FT_BOOLEAN, 8, NULL, SPARE_BIT_MASK,          
 	"", HFILL }
     }, 
     { &hf_iua_sapi,
-      { "SAPI", "hf.iua.sapi",
+      { "SAPI", "iua.sapi",
 	FT_UINT8, BASE_HEX, NULL, SAPI_MASK,          
 	"", HFILL }
     }, 
     { &hf_iua_zero_bit,
-      { "Zero bit", "hf.iua.zero_bit",
+      { "Zero bit", "iua.zero_bit",
 	FT_BOOLEAN, 8, NULL, ZERO_BIT_MASK,          
 	"", HFILL }
     }, 
     { &hf_iua_one_bit,
-      { "One bit", "hf.iua.one_bit",
+      { "One bit", "iua.one_bit",
 	FT_BOOLEAN, 8, NULL, ONE_BIT_MASK,          
 	"", HFILL }
     }, 
     { &hf_iua_tei,
-      { "TEI", "hf.iua.tei",
+      { "TEI", "iua.tei",
 	FT_UINT8, BASE_HEX, NULL, TEI_MASK,          
 	"", HFILL }
     },
@@ -1008,6 +1008,9 @@ proto_register_iua(void)
 void
 proto_reg_handoff_iua(void)
 {
-  dissector_add("sctp.port", SCTP_PORT_IUA, dissect_iua, proto_iua);
-  dissector_add("sctp.ppi", IUA_PAYLOAD_PROTO_ID, dissect_iua, proto_iua);
+  dissector_handle_t iua_handle;
+
+  iua_handle = create_dissector_handle(dissect_iua, proto_iua);
+  dissector_add("sctp.port", SCTP_PORT_IUA, iua_handle);
+  dissector_add("sctp.ppi", IUA_PAYLOAD_PROTO_ID, iua_handle);
 }
