@@ -7,7 +7,7 @@
  * Laurent Cazalet <laurent.cazalet@mailclub.net>
  * Thomas Parvais <thomas.parvais@advalvas.be>
  *
- * $Id: packet-l2tp.c,v 1.24 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-l2tp.c,v 1.29 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -47,6 +47,7 @@ static int hf_l2tp_avp_hidden = -1;
 static int hf_l2tp_avp_length = -1;
 static int hf_l2tp_avp_vendor_id = -1;
 static int hf_l2tp_avp_type = -1;
+static int hf_l2tp_tie_breaker = -1;
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -155,6 +156,13 @@ static const value_string l2tp_type_vals[] = {
 	{ 0, NULL },
 };
 
+static const value_string cause_code_direction_vals[] = {
+	{ 0, "global error" },
+	{ 1, "at peer" },
+	{ 2, "at local" },
+	{ 0, NULL },
+};
+
 static const true_false_string l2tp_length_bit_truth =
 	{ "Length field is present", "Length field is not present" };
 
@@ -216,6 +224,7 @@ static const value_string authen_type_vals[] = {
 #define  PRIVATE_GROUP_ID 37
 #define  RX_CONNECT_SPEED 38
 #define  SEQUENCING_REQUIRED 39
+#define  PPP_DISCONNECT_CAUSE_CODE 46	/* RFC 3145 */
 
 #define NUM_AVP_TYPES  40
 static const value_string avp_type_vals[] = {
@@ -258,9 +267,43 @@ static const value_string avp_type_vals[] = {
   { PRIVATE_GROUP_ID,          "Private group ID" },
   { RX_CONNECT_SPEED,          "RxConnect Speed" },
   { SEQUENCING_REQUIRED,       "Sequencing Required" },
+  { PPP_DISCONNECT_CAUSE_CODE, "PPP Disconnect Cause Code" },
   { 0,                         NULL }
 };
 
+/*
+ * These are SMI Network Management Private Enterprise Codes for
+ * organizations; see
+ *
+ *      http://www.isi.edu/in-notes/iana/assignments/enterprise-numbers
+ *
+ * for a list.
+ */
+#define VENDOR_IETF 0
+#define VENDOR_ACC 5
+#define VENDOR_CISCO 9
+#define VENDOR_SHIVA 166
+#define VENDOR_LIVINGSTON 307
+#define VENDOR_3COM 429
+#define VENDOR_ASCEND 529
+#define VENDOR_BAY 1584
+#define VENDOR_JUNIPER 2636
+#define VENDOR_COSINE 3085
+#define VENDOR_UNISPHERE 4874
+
+static const value_string avp_vendor_id_vals[] = 
+{{VENDOR_IETF,"IETF"},
+{VENDOR_ACC,"ACC"},
+{VENDOR_CISCO,"Cisco"},
+{VENDOR_SHIVA,"Shiva"},
+{VENDOR_LIVINGSTON,"Livingston"},
+{VENDOR_3COM,"3Com"},
+{VENDOR_ASCEND,"Ascend"},
+{VENDOR_BAY,"Bay Networks"},
+{VENDOR_JUNIPER,"Juniper Networks"},
+{VENDOR_COSINE,"CoSine Communications"},
+{VENDOR_UNISPHERE,"Unisphere Networks"},
+{0,NULL}};
 
 static gchar textbuffer[200];
 
@@ -280,6 +323,7 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint16 cid;			/* Call ID */
   guint16 offset_size;		/* Offset size */
   guint16 ver_len_hidden;
+  guint16 avp_vendor_id;
   guint16 avp_type;
   guint16 msg_type;
   guint16 avp_len;
@@ -290,16 +334,16 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint16	control;
   tvbuff_t	*next_tvb;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))	/* build output for closed L2tp frame displayed  */
-        col_set_str(pinfo->fd, COL_PROTOCOL, "L2TP"); 
-  if (check_col(pinfo->fd, COL_INFO))
-        col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))	/* build output for closed L2tp frame displayed  */
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "L2TP"); 
+  if (check_col(pinfo->cinfo, COL_INFO))
+        col_clear(pinfo->cinfo, COL_INFO);
 
   control = tvb_get_ntohs(tvb, 0);
 
   if (L2TP_VERSION(control) != 2) {
-	  if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO, "L2TP Version %u", L2TP_VERSION(control) );
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO, "L2TP Version %u", L2TP_VERSION(control) );
 	  }
 	  return;
   }
@@ -317,7 +361,7 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   index += 2;
   cid = tvb_get_ntohs(tvb, index);
 
-  if (check_col(pinfo->fd, COL_INFO)) {
+  if (check_col(pinfo->cinfo, COL_INFO)) {
         if (CONTROL_BIT(control)) {
             /* CONTROL MESSAGE */
             tmp_index = index;
@@ -362,7 +406,7 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                sprintf(textbuffer,"%s            (tunnel id=%d, session id=%d)",
                        data_msg, tid ,cid);
         }
-        col_add_fstr(pinfo->fd,COL_INFO,textbuffer);
+        col_add_fstr(pinfo->cinfo,COL_INFO,textbuffer);
   }
 
   if (LENGTH_BIT(control)) {
@@ -446,11 +490,19 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	while (index < length ) {    /* Process AVP's */
    		ver_len_hidden	= tvb_get_ntohs(tvb, index);
    		avp_len		= AVP_LENGTH(ver_len_hidden);
+		avp_vendor_id	= tvb_get_ntohs(tvb, index + 2);
    		avp_type	= tvb_get_ntohs(tvb, index + 4);
 
-		tf =  proto_tree_add_text(l2tp_tree, tvb, index, avp_len,
-		    "%s AVP",
-		    val_to_str(avp_type, avp_type_vals, "Unknown (%u)"));
+		if (avp_vendor_id == VENDOR_IETF) {
+			tf =  proto_tree_add_text(l2tp_tree, tvb, index, 
+			avp_len, "%s AVP",
+		        val_to_str(avp_type, avp_type_vals, "Unknown (%u)"));
+		} else {	 /* Vendor-Specific AVP */
+			tf =  proto_tree_add_text(l2tp_tree, tvb, index, 
+			avp_len, "Vendor %s AVP",
+		        val_to_str(avp_vendor_id, avp_vendor_id_vals, "Unknown (%u)"));
+		}
+
                 l2tp_avp_tree = proto_item_add_subtree(tf,  ett_l2tp_avp);
 
                 proto_tree_add_boolean_format(l2tp_avp_tree,hf_l2tp_avp_mandatory, tvb, index, 1,
@@ -479,6 +531,21 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		index += 2;
 		avp_len -= 2;
 
+		if (avp_vendor_id != VENDOR_IETF) {
+			proto_tree_add_text(l2tp_avp_tree, tvb, index, 2,
+					    "Type: %u", avp_type);
+			index += 2;
+			avp_len -= 2;
+
+			/* For the time being, we don't decode any Vendor-
+			   specific AVP. */
+			proto_tree_add_text(l2tp_avp_tree, tvb, index, 
+					    avp_len, "Vendor-Specific AVP");
+
+			index += avp_len;
+			continue;
+		}
+
 		proto_tree_add_uint(l2tp_avp_tree, hf_l2tp_avp_type,
 		    tvb, index, 2, avp_type);
 		index += 2;
@@ -551,17 +618,7 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			break;
 
 		case TIE_BREAKER:
-			/*
-			 * XXX - 64-bit values aren't supported on
-			 * platforms/compilers where "guint64" isn't
-			 * supported, and, even on those platforms,
-			 * there's no *printf standard for the format
-			 * to use when printing those values, so we print it
-			 * as a string of hex digits.
-			 */
-			proto_tree_add_text(l2tp_avp_tree, tvb, index, 8,
-			  "Tie Breaker: %s",
-			  tvb_bytes_to_str(tvb, index, 8));
+			proto_tree_add_item(l2tp_avp_tree, hf_l2tp_tie_breaker, tvb, index, 8, FALSE);
 			break;
 
 		case FIRMWARE_REVISION:
@@ -865,6 +922,40 @@ dissect_l2tp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			  tvb_get_ntohl(tvb, index));
 			break;
 
+		case PPP_DISCONNECT_CAUSE_CODE:
+			if (avp_len < 2)
+				break;
+			proto_tree_add_text(l2tp_avp_tree, tvb, index, 2,
+			  "Disconnect Code: %u",
+			  tvb_get_ntohs(tvb, index));
+			index += 2;
+			avp_len -= 2;
+
+			if (avp_len < 2)
+				break;
+			proto_tree_add_text(l2tp_avp_tree, tvb, index, 2,
+			  "Control Protocol Number: %u",
+			  tvb_get_ntohs(tvb, index));
+			index += 2;
+			avp_len -= 2;
+
+			if (avp_len < 1)
+				break;
+			proto_tree_add_text(l2tp_avp_tree, tvb, index, 1,
+			  "Direction: %s",
+			  val_to_str(tvb_get_guint8(tvb, index), 
+				     cause_code_direction_vals, 
+				     "Reserved (%u)"));
+			index += 1;
+			avp_len -= 1;
+
+			if (avp_len == 0)
+				break;
+			proto_tree_add_text(l2tp_avp_tree, tvb, index, avp_len,
+			  "Message: %.*s", avp_len,
+			  tvb_get_ptr(tvb, index, avp_len));
+			break;
+
 		default:
 			proto_tree_add_text(l2tp_avp_tree, tvb, index, avp_len,
 			  "Unknown AVP");
@@ -947,12 +1038,17 @@ proto_register_l2tp(void)
 			"AVP Length", HFILL }},
 
 		{ &hf_l2tp_avp_vendor_id,
-		{ "Vendor ID", "lt2p.avp.vendor_id", FT_UINT16, BASE_DEC, NULL, 0,
+		{ "Vendor ID", "lt2p.avp.vendor_id", FT_UINT16, BASE_DEC, VALS(avp_vendor_id_vals), 0,
 			"AVP Vendor ID", HFILL }},
 
 		{ &hf_l2tp_avp_type,
 		{ "Type", "lt2p.avp.type", FT_UINT16, BASE_DEC, VALS(avp_type_vals), 0,
 			"AVP Type", HFILL }},
+
+		{ &hf_l2tp_tie_breaker,
+		{ "Tie Breaker", "lt2p.tie_breaker", FT_UINT64, BASE_HEX, NULL, 0,
+			"Tie Breaker", HFILL }},
+
 	};
 
 	static gint *ett[] = {
@@ -970,8 +1066,10 @@ proto_register_l2tp(void)
 void
 proto_reg_handoff_l2tp(void)
 {
-	dissector_add("udp.port", UDP_PORT_L2TP, dissect_l2tp,
-	    proto_l2tp);
+	dissector_handle_t l2tp_handle;
+
+	l2tp_handle = create_dissector_handle(dissect_l2tp, proto_l2tp);
+	dissector_add("udp.port", UDP_PORT_L2TP, l2tp_handle);
 
 	/*
 	 * Get a handle for the PPP-in-HDLC-like-framing dissector.
