@@ -1,13 +1,12 @@
 /* packet-lapd.c
  * Routines for LAPD frame disassembly
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-lapd.c,v 1.23 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-lapd.c,v 1.27 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -59,6 +58,7 @@ static gint ett_lapd_address = -1;
 static gint ett_lapd_control = -1;
 
 static dissector_handle_t q931_handle;
+static dissector_handle_t data_handle;
 
 /*
  * Bits in the address field.
@@ -94,10 +94,10 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gboolean	is_response;
 	tvbuff_t	*next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "LAPD");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "LAPD");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	address = tvb_get_ntohs(tvb, 0);
 	cr = address & LAPD_CR;
@@ -106,17 +106,17 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	if (pinfo->pseudo_header->p2p.sent) {
 		is_response = cr ? TRUE : FALSE;
-		if(check_col(pinfo->fd, COL_RES_DL_DST))
-			col_set_str(pinfo->fd, COL_RES_DL_DST, "Network");
-		if(check_col(pinfo->fd, COL_RES_DL_SRC))
-			col_set_str(pinfo->fd, COL_RES_DL_SRC, "User");
+		if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+			col_set_str(pinfo->cinfo, COL_RES_DL_DST, "Network");
+		if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+			col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "User");
 	}
 	else {
 		is_response = cr ? FALSE : TRUE;
-		if(check_col(pinfo->fd, COL_RES_DL_DST))
-		    col_set_str(pinfo->fd, COL_RES_DL_DST, "User");
-		if(check_col(pinfo->fd, COL_RES_DL_SRC))
-		    col_set_str(pinfo->fd, COL_RES_DL_SRC, "Network");
+		if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+		    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "User");
+		if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+		    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "Network");
 	}
 
 	if (tree) {
@@ -150,11 +150,11 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			break;
 
 		default:
-			dissect_data(next_tvb, 0, pinfo, tree);
+			call_dissector(data_handle,next_tvb, pinfo, tree);
 			break;
 		}
 	} else
-		dissect_data(next_tvb, 0, pinfo, tree);
+		call_dissector(data_handle,next_tvb, pinfo, tree);
 }
 
 void
@@ -204,10 +204,14 @@ proto_register_lapd(void)
 void
 proto_reg_handoff_lapd(void)
 {
+	dissector_handle_t lapd_handle;
+
 	/*
 	 * Get handle for the Q.931 dissector.
 	 */
 	q931_handle = find_dissector("q931");
+	data_handle = find_dissector("data");
 
-	dissector_add("wtap_encap", WTAP_ENCAP_LAPD, dissect_lapd, proto_lapd);
+	lapd_handle = create_dissector_handle(dissect_lapd, proto_lapd);
+	dissector_add("wtap_encap", WTAP_ENCAP_LAPD, lapd_handle);
 }
