@@ -1,13 +1,14 @@
 /* packet-ldp.c
- * Routines for ldp packet disassembly
+ * Routines for LDP (RFC 3036) packet disassembly
  *
- * $Id: packet-ldp.c,v 1.17 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-ldp.c,v 1.23 2001/12/10 00:25:30 guy Exp $
  * 
  * Copyright (c) November 2000 by Richard Sharpe <rsharpe@ns.aus.com>
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1999 Gerald Combs
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -44,6 +45,7 @@
 #include "packet.h"
 #include "resolv.h"
 #include "prefs.h"
+#include "afn.h"
 
 #define TCP_PORT_LDP 646
 #define UDP_PORT_LDP 646
@@ -190,20 +192,6 @@ static const value_string fec_types[] = {
   {0, NULL}
 };
 
-static const value_string fec_af_types[] = {
-  {0, "Reserved"},
-  {1, "IP (IPv4)"},
-  {2, "IP6 (IPv6)"},
-  {3, "NSAP"},
-  {4, "HDLC (8-bit multidrop)"},
-  {5, "BBN 1822"},
-  {6, "802"},
-  {10, "X.121 (X.25, Frame Relay)"},
-  {11, "IPX"},
-  {12, "Appletalk"},
-  {0, NULL}
-};
-
 static const true_false_string hello_requested_vals = {
   "Source requests periodic hellos",
   "Source does not request periodic hellos"
@@ -217,8 +205,8 @@ void dissect_tlv_common_hello_parms(tvbuff_t *tvb, guint offset, proto_tree *tre
 
   if (tree) {
 
-    ti = proto_tree_add_bytes(tree, hf_ldp_tlv_value, tvb, offset, rem,
-			      tvb_get_ptr(tvb, offset, rem));
+    ti = proto_tree_add_item(tree, hf_ldp_tlv_value, tvb, offset, rem,
+			     FALSE);
 
     val_tree = proto_item_add_subtree(ti, ett_ldp_tlv_val);
 
@@ -387,8 +375,8 @@ int dissect_tlv(tvbuff_t *tvb, guint offset, proto_tree *tree, int rem)
       break;
 
     default:
-      proto_tree_add_bytes(tlv_tree, hf_ldp_tlv_value, tvb, offset + 4, 
-			   length, tvb_get_ptr(tvb, offset + 4, length));
+      proto_tree_add_item(tlv_tree, hf_ldp_tlv_value, tvb, offset + 4, 
+			   length, FALSE);
 
       break;
     }
@@ -559,55 +547,73 @@ dissect_ldp_label_abort_request(tvbuff_t *tvb, guint offset, packet_info *pinfo,
 
 }
 
-static void
-dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+static int
+dissect_ldp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
-  proto_tree     *ldp_tree = NULL, 
-                 *ti = NULL,
-                 *hdr_tree = NULL, *ldpid_tree = NULL;
-  int	         offset = 0, msg_cnt = 0;
+  proto_tree     *ldp_tree = NULL, *hdr_tree = NULL, *ldpid_tree = NULL;
+  proto_item     *ldp_item = NULL, *hdr_item = NULL, *ldpid_item = NULL;
+  int	         msg_cnt = 0;
   guint16        ldp_message = 0;
+  guint          pdu_len;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_add_str(pinfo->fd, COL_PROTOCOL, "LDP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   if (tree) {  /* Build the tree info ..., this is wrong! FIXME */
 
-    ti = proto_tree_add_item(tree, proto_ldp, tvb, offset,
-			     tvb_length_remaining(tvb, offset), FALSE);
-    ldp_tree = proto_item_add_subtree(ti, ett_ldp);
-
-    ti = proto_tree_add_text(ldp_tree, tvb, 0, 10, "Header");
-
-    hdr_tree = proto_item_add_subtree(ti, ett_ldp_header);
-
-    proto_tree_add_item(hdr_tree, hf_ldp_version, tvb, offset, 2, FALSE);
-
-    offset += 2;
-
-    proto_tree_add_item(hdr_tree, hf_ldp_pdu_len, tvb, offset, 2, FALSE);
-
-    offset += 2;
-
-    ti = proto_tree_add_text(hdr_tree, tvb, offset, 6, "LDP Identifier");
-
-    ldpid_tree = proto_item_add_subtree(ti, ett_ldp_ldpid);
-
-    proto_tree_add_item(ldpid_tree, hf_ldp_lsr, tvb, offset, 4, FALSE);
-
-    offset += 4;
-
-    proto_tree_add_item(ldpid_tree, hf_ldp_ls_id, tvb, offset, 2, FALSE);
-
-    offset += 2;
+    ldp_item = proto_tree_add_item(tree, proto_ldp, tvb, offset,
+				   tvb_length_remaining(tvb, offset), FALSE);
+    ldp_tree = proto_item_add_subtree(ldp_item, ett_ldp);
 
   }
 
-  offset = 10;
+  /* Dissect LDP Header */
 
-  while (tvb_length_remaining(tvb, offset) > 0) { /* Dissect a message */
+  hdr_item = proto_tree_add_text(ldp_tree, tvb, offset, 10, "Header");
+
+  hdr_tree = proto_item_add_subtree(hdr_item, ett_ldp_header);
+
+  proto_tree_add_item(hdr_tree, hf_ldp_version, tvb, offset, 2, FALSE);
+
+  offset += 2;
+
+  proto_tree_add_item(hdr_tree, hf_ldp_pdu_len, tvb, offset, 2, FALSE);
+  pdu_len = tvb_get_ntohs(tvb, offset);
+  proto_item_set_len(ldp_item, pdu_len + 2);
+
+  /*
+   * XXX - do TCP reassembly, to handle LDP PDUs that cross TCP segment
+   * boundaries.
+   */
+
+  offset += 2;
+
+  if (pdu_len < 6) {
+    /*
+     * PDU is too short to hold the LDP identifier.
+     */
+    proto_tree_add_text(hdr_tree, tvb, offset, pdu_len,
+        "PDU too short (%u bytes, should be at least 6) for LDP Identifier",
+        pdu_len);
+    offset += pdu_len;
+    return offset;
+  }
+
+  ldpid_item = proto_tree_add_text(hdr_tree, tvb, offset, 6, "LDP Identifier");
+
+  ldpid_tree = proto_item_add_subtree(ldpid_item, ett_ldp_ldpid);
+
+  proto_tree_add_item(ldpid_tree, hf_ldp_lsr, tvb, offset, 4, FALSE);
+
+  offset += 4;
+  pdu_len -= 4;
+
+  proto_tree_add_item(ldpid_tree, hf_ldp_ls_id, tvb, offset, 2, FALSE);
+
+  offset += 2;
+  pdu_len -= 2;
+
+  while (pdu_len > 0) { /* Dissect LDP TLV */
 
     guint msg_len;
 
@@ -615,15 +621,15 @@ dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
     msg_len = tvb_get_ntohs(tvb, offset + 2);
 
-    if (check_col(pinfo->fd, COL_INFO)) {  /* Check the type ... */
+    if (check_col(pinfo->cinfo, COL_INFO)) {  /* Check the type ... */
 
       if (msg_cnt > 0) 
-	col_append_fstr(pinfo->fd, COL_INFO, ", %s",
+	col_append_fstr(pinfo->cinfo, COL_INFO, ", %s",
 			val_to_str(ldp_message, ldp_message_types, "Unknown Message (0x%04X)"));
       else
-	col_add_fstr(pinfo->fd, COL_INFO, "%s", 
+	col_add_fstr(pinfo->cinfo, COL_INFO, "%s", 
 		     val_to_str(ldp_message, ldp_message_types, "Unknown Message (0x%04X)"));
-
+	
     }
 
     msg_cnt++;
@@ -638,13 +644,25 @@ dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			       val_to_str(ldp_message, ldp_message_types, "Unknown Message (0x%04X)"));
 
       msg_tree = proto_item_add_subtree(ti, ett_ldp_message);
-
+	
       proto_tree_add_item(msg_tree, hf_ldp_msg_type, tvb, offset, 2, FALSE);
 
       proto_tree_add_item(msg_tree, hf_ldp_msg_len, tvb, offset + 2, 2, FALSE);
 
+      if (msg_len < 4) {
+	proto_tree_add_text(msg_tree, tvb, offset + 4, msg_len,
+	    "Message too short (%u bytes, should be at least 4) for Message ID",
+	    msg_len);
+	goto next;
+      }
+
       proto_tree_add_item(msg_tree, hf_ldp_msg_id, tvb, offset + 4, 4, FALSE);
 
+      if (msg_len == 4) {
+      	/* Nothing past the message ID */
+      	goto next;
+      }
+
       switch (ldp_message) {
 
       case LDP_NOTIFICATION:
@@ -668,7 +686,7 @@ dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       case LDP_KEEPALIVE:
 
 	dissect_ldp_keepalive(tvb, offset + 8, pinfo, msg_tree, msg_len - 4);
-
+	  
 	break;
 
       case LDP_ADDRESS:
@@ -716,6 +734,8 @@ dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       default:
 
 	/* Some sort of unknown message, treat as undissected data */
+	proto_tree_add_text(msg_tree, tvb, offset + 8, msg_len - 4,
+			    "Message data"); 
 
 	break;
 
@@ -723,9 +743,33 @@ dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     
     }
 
+next:
     offset += msg_len + 4;
+    pdu_len -= msg_len + 4;
 
   }
+  return offset;
+}
+
+static void
+dissect_ldp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_add_str(pinfo->cinfo, COL_PROTOCOL, "LDP");
+
+  dissect_ldp_pdu(tvb, 0, pinfo, tree);
+}
+
+static void
+dissect_ldp_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+  int	         offset = 0;
+
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_add_str(pinfo->cinfo, COL_PROTOCOL, "LDP");
+
+  while (tvb_reported_length_remaining(tvb, offset) > 0) /* Dissect LDP PDUs */
+    offset = dissect_ldp_pdu(tvb, offset, pinfo, tree);
 }
 
 /* Register all the bits needed with the filtering engine */
@@ -790,7 +834,7 @@ proto_register_ldp(void)
       { "FEC Element Type", "ldp.msg.tlv.fec.type", FT_UINT8, BASE_DEC, VALS(fec_types), 0x0, "Forwarding Equivalence Class Element Types", HFILL }},
 
     { &hf_ldp_tlv_fec_af,
-      { "FEC Element Address Type", "ldp.msg.tlv.fec.af", FT_UINT16, BASE_DEC, VALS(fec_af_types), 0x0, "Forwarding Equivalence Class Element Address Family", HFILL }},
+      { "FEC Element Address Type", "ldp.msg.tlv.fec.af", FT_UINT16, BASE_DEC, VALS(afn_vals), 0x0, "Forwarding Equivalence Class Element Address Family", HFILL }},
 
     { &hf_ldp_tlv_fec_len,
       { "FEC Element Length", "ldp.msg.tlv.fec.len", FT_UINT8, BASE_DEC, NULL, 0x0, "Forwarding Equivalence Class Element Length", HFILL }},
@@ -840,16 +884,20 @@ void
 proto_reg_handoff_ldp(void)
 {
   static int ldp_prefs_initialized = FALSE;
+  static dissector_handle_t ldp_tcp_handle, ldp_handle;
 
-  if (ldp_prefs_initialized) {
+  if (!ldp_prefs_initialized) {
 
-    dissector_delete("tcp.port", tcp_port, dissect_ldp);
-    dissector_delete("udp.port", udp_port, dissect_ldp);
+    ldp_tcp_handle = create_dissector_handle(dissect_ldp_tcp, proto_ldp);
+    ldp_handle = create_dissector_handle(dissect_ldp, proto_ldp);
+
+    ldp_prefs_initialized = TRUE;
 
   }
   else {
 
-    ldp_prefs_initialized = TRUE;
+    dissector_delete("tcp.port", tcp_port, ldp_tcp_handle);
+    dissector_delete("udp.port", udp_port, ldp_handle);
 
   }
 
@@ -858,7 +906,7 @@ proto_reg_handoff_ldp(void)
   tcp_port = global_ldp_tcp_port;
   udp_port = global_ldp_udp_port;
 
-  dissector_add("tcp.port", global_ldp_tcp_port, dissect_ldp, proto_ldp);
-  dissector_add("udp.port", global_ldp_udp_port, dissect_ldp, proto_ldp);
+  dissector_add("tcp.port", global_ldp_tcp_port, ldp_tcp_handle);
+  dissector_add("udp.port", global_ldp_udp_port, ldp_handle);
 
 }
