@@ -1,13 +1,12 @@
 /* packet-llc.c
  * Routines for IEEE 802.2 LLC layer
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-llc.c,v 1.87 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-llc.c,v 1.93 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,8 +65,9 @@ static dissector_handle_t bpdu_handle;
 static dissector_handle_t eth_handle;
 static dissector_handle_t fddi_handle;
 static dissector_handle_t tr_handle;
+static dissector_handle_t data_handle;
 
-typedef void (capture_func_t)(const u_char *, int, packet_counts *);
+typedef void (capture_func_t)(const u_char *, int, int, packet_counts *);
 
 /* The SAP info is split into two tables, one value_string table and one
  * table of sap_info. This is so that the value_string can be used in the
@@ -191,7 +191,7 @@ sap_capture_func(u_char sap) {
 }
 
 void
-capture_llc(const u_char *pd, int offset, packet_counts *ld) {
+capture_llc(const u_char *pd, int offset, int len, packet_counts *ld) {
 
 	int		is_snap;
 	guint16		control;
@@ -200,7 +200,7 @@ capture_llc(const u_char *pd, int offset, packet_counts *ld) {
 	guint16		etype;
 	capture_func_t	*capture;
 
-	if (!BYTES_ARE_IN_FRAME(offset, 2)) {
+	if (!BYTES_ARE_IN_FRAME(offset, len, 2)) {
 		ld->other++;
 		return;
 	}
@@ -218,7 +218,7 @@ capture_llc(const u_char *pd, int offset, packet_counts *ld) {
 	llc_header_len += XDLC_CONTROL_LEN(control, TRUE);
 	if (is_snap)
 		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of protocol ID */
-	if (!BYTES_ARE_IN_FRAME(offset, llc_header_len)) {
+	if (!BYTES_ARE_IN_FRAME(offset, len, llc_header_len)) {
 		ld->other++;
 		return;
 	}
@@ -239,12 +239,12 @@ capture_llc(const u_char *pd, int offset, packet_counts *ld) {
 				   AppleTalk data packets - but used
 				   OUI_ENCAP_ETHER and an Ethernet
 				   packet type for AARP packets. */
-				capture_ethertype(etype, offset+8, pd,
+				capture_ethertype(etype, pd, offset+8, len,
 				    ld);
 				break;
 			case OUI_CISCO:
-				capture_ethertype(etype,
-						offset + 8, pd, ld);
+				capture_ethertype(etype, pd, offset + 8, len,
+				    ld);
 				break;
 			default:
 				ld->other++;
@@ -260,7 +260,7 @@ capture_llc(const u_char *pd, int offset, packet_counts *ld) {
 			offset += llc_header_len;
 
 			if (capture) {
-				capture(pd, offset, ld);
+				capture(pd, offset, len, ld);
 			}
 			else {
 				ld->other++;
@@ -280,11 +280,11 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8		dsap, ssap;
 	tvbuff_t	*next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL)) {
-		col_set_str(pinfo->fd, COL_PROTOCOL, "LLC");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "LLC");
 	}
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_clear(pinfo->cinfo, COL_INFO);
 	}
 
 	dsap = tvb_get_guint8(tvb, 0);
@@ -331,8 +331,8 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		    hf_llc_oui, hf_llc_type, hf_llc_pid, 2);
 	}
 	else {
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_append_fstr(pinfo->fd, COL_INFO, 
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_fstr(pinfo->cinfo, COL_INFO, 
 			    "; DSAP %s %s, SSAP %s %s",
 			    val_to_str(dsap & SAP_MASK, sap_vals, "%02x"),
 			    dsap & DSAP_GI_BIT ?
@@ -349,10 +349,10 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			/* do lookup with the subdissector table */
 			if (!dissector_try_port(subdissector_table, dsap,
 			    next_tvb, pinfo, tree)) {
-				dissect_data(next_tvb, 0, pinfo, tree);
+				call_dissector(data_handle,next_tvb, pinfo, tree);
 			}
 		} else {
-			dissect_data(next_tvb, 0, pinfo, tree);
+			call_dissector(data_handle,next_tvb, pinfo, tree);
 		}
 	}
 }
@@ -372,8 +372,8 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 	oui =	tvb_get_ntoh24(tvb, offset);
 	etype = tvb_get_ntohs(tvb, offset+3);
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_append_fstr(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO,
 		    "; SNAP, OUI 0x%06X (%s), PID 0x%04X",
 		    oui, val_to_str(oui, oui_vals, "Unknown"), etype);
 	}
@@ -398,7 +398,7 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 			    pinfo, tree, snap_tree, hf_type, -1);
 		} else {
 			next_tvb = tvb_new_subset(tvb, offset+5, -1, -1);
-			dissect_data(next_tvb, 0, pinfo, tree);
+			call_dissector(data_handle,next_tvb, pinfo, tree);
 		}
 		break;
 
@@ -454,7 +454,7 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 
 		default:
 			next_tvb = tvb_new_subset(tvb, offset+5, -1, -1);
-			dissect_data(next_tvb, 0, pinfo, tree);
+			call_dissector(data_handle,next_tvb, pinfo, tree);
 			break;
 		}
 		break;
@@ -475,9 +475,9 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 			/* for future reference, 0x0102 is Cisco DRIP */
 			if (!dissector_try_port(cisco_subdissector_table,
 			    etype, next_tvb, pinfo, tree))
-				dissect_data(next_tvb, 0, pinfo, tree);
+				call_dissector(data_handle,next_tvb, pinfo, tree);
 		} else
-			dissect_data(next_tvb, 0, pinfo, tree);
+			call_dissector(data_handle,next_tvb, pinfo, tree);
 		break;
 
 	case OUI_CABLE_BPDU:    /* DOCSIS cable modem spanning tree BPDU */
@@ -495,7 +495,7 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 			    etype);
 		}
 		next_tvb = tvb_new_subset(tvb, offset+5, -1, -1);
-		dissect_data(next_tvb, 0, pinfo, tree);
+		call_dissector(data_handle,next_tvb, pinfo, tree);
 		break;
 	}
 }
@@ -550,8 +550,10 @@ proto_register_llc(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 /* subdissector code */
-	subdissector_table = register_dissector_table("llc.dsap");
-	cisco_subdissector_table = register_dissector_table("llc.cisco_pid");
+	subdissector_table = register_dissector_table("llc.dsap",
+	  "LLC SAP", FT_UINT8, BASE_HEX);
+	cisco_subdissector_table = register_dissector_table("llc.cisco_pid",
+	  "Cisco OUI PID", FT_UINT16, BASE_HEX);
 
 	register_dissector("llc", dissect_llc, proto_llc);
 }
@@ -559,6 +561,8 @@ proto_register_llc(void)
 void
 proto_reg_handoff_llc(void)
 {
+	dissector_handle_t llc_handle;
+
 	/*
 	 * Get handles for the BPDU, Ethernet, FDDI, and Token Ring
 	 * dissectors.
@@ -567,7 +571,8 @@ proto_reg_handoff_llc(void)
 	eth_handle = find_dissector("eth");
 	fddi_handle = find_dissector("fddi");
 	tr_handle = find_dissector("tr");
+	data_handle = find_dissector("data");
 
-	dissector_add("wtap_encap", WTAP_ENCAP_ATM_RFC1483, dissect_llc,
-	    proto_llc);
+	llc_handle = find_dissector("llc");
+	dissector_add("wtap_encap", WTAP_ENCAP_ATM_RFC1483, llc_handle);
 }
