@@ -2,12 +2,11 @@
  * Routines for Frame Relay Local Management Interface (LMI) disassembly
  * Copyright 2001, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-lmi.c,v 1.5 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-lmi.c,v 1.8 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -149,8 +148,8 @@ dissect_lmi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int		offset = 2, len;
 	guint8		ele_id;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "LMI");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "LMI");
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_lmi, tvb, 0, 3, FALSE);
@@ -160,7 +159,7 @@ dissect_lmi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		proto_tree_add_uint(lmi_tree, hf_lmi_msg_type,  tvb, 1, 1,  tvb_get_guint8( tvb, 1));
 
 	/* Display the LMI elements */
-		while( offset < tvb_length( tvb)){
+		while (tvb_reported_length_remaining(tvb, offset) > 0) {
 			ele_id = tvb_get_guint8( tvb, offset);
 			len =  tvb_get_guint8( tvb, offset + 1);
 
@@ -247,5 +246,8 @@ proto_register_lmi(void)
 void
 proto_reg_handoff_lmi(void)
 {
-	dissector_add("fr.ietf", NLPID_LMI, dissect_lmi, proto_lmi);
+	dissector_handle_t lmi_handle;
+
+	lmi_handle = create_dissector_handle(dissect_lmi, proto_lmi);
+	dissector_add("fr.ietf", NLPID_LMI, lmi_handle);
 }
