@@ -1,13 +1,12 @@
 /* packet-lpd.c
  * Routines for LPR and LPRng packet disassembly
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-lpd.c,v 1.28 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-lpd.c,v 1.32 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,6 +48,8 @@ enum lpr_type { request, response, unknown };
 
 static gint find_printer_string(tvbuff_t *tvb, int offset);
 
+static dissector_handle_t data_handle;
+
 static void
 dissect_lpd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
@@ -79,10 +80,10 @@ dissect_lpd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		"Bad job format, do not retry"
 	};
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "LPD");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "LPD");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	/* rfc1179 states that all responses are 1 byte long */
 	code = tvb_get_guint8(tvb, 0);
@@ -96,15 +97,15 @@ dissect_lpd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		lpr_packet_type = unknown;
 	}
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		if (lpr_packet_type == request) {
-			col_add_str(pinfo->fd, COL_INFO, lpd_client_code[code]);
+			col_add_str(pinfo->cinfo, COL_INFO, lpd_client_code[code]);
 		}
 		else if (lpr_packet_type == response) {
-			col_set_str(pinfo->fd, COL_INFO, "LPD response");
+			col_set_str(pinfo->cinfo, COL_INFO, "LPD response");
 		}
 		else {
-			col_set_str(pinfo->fd, COL_INFO, "LPD continuation");
+			col_set_str(pinfo->cinfo, COL_INFO, "LPD continuation");
 		}
 	}
 
@@ -132,7 +133,7 @@ dissect_lpd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					 tvb_format_text(tvb, 1, printer_len));
 			}
 			else {
-				dissect_data(tvb, 0, pinfo, tree);
+				call_dissector(data_handle,tvb, pinfo, tree);
 			}
 		}
 		else if (lpr_packet_type == response) {
@@ -141,11 +142,11 @@ dissect_lpd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					"Response: %s", lpd_server_code[code]);
 			}
 			else {
-				dissect_data(tvb, 0, pinfo, tree);
+				call_dissector(data_handle,tvb, pinfo, tree);
 			}
 		}
 		else {
-			dissect_data(tvb, 0, pinfo, tree);
+			call_dissector(data_handle,tvb, pinfo, tree);
 		}
 	}
 }
@@ -192,5 +193,9 @@ proto_register_lpd(void)
 void
 proto_reg_handoff_lpd(void)
 {
-  dissector_add("tcp.port", TCP_PORT_PRINTER, &dissect_lpd, proto_lpd);
+  dissector_handle_t lpd_handle;
+
+  lpd_handle = create_dissector_handle(dissect_lpd, proto_lpd);
+  dissector_add("tcp.port", TCP_PORT_PRINTER, lpd_handle);
+  data_handle = find_dissector("data");
 }
