@@ -1,10 +1,10 @@
 /* packet-mapi.c
  * Routines for MSX mapi packet dissection
  *
- * $Id: packet-mapi.c,v 1.16 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-mapi.c,v 1.18 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
@@ -55,12 +55,12 @@ dissect_mapi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	proto_tree      *mapi_tree, *ti;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "MAPI");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "MAPI");
 
-	if (check_col(pinfo->fd, COL_INFO))
+	if (check_col(pinfo->cinfo, COL_INFO))
 	{
-		col_add_fstr(pinfo->fd, COL_INFO, "%s", 
+		col_add_fstr(pinfo->cinfo, COL_INFO, "%s", 
 			(pinfo->match_port == pinfo->destport) ? "Request" : "Response");	  
 	}
 
@@ -131,5 +131,8 @@ proto_register_mapi(void)
 void
 proto_reg_handoff_mapi(void)
 {
-	dissector_add("tcp.port", TCP_PORT_MAPI, dissect_mapi, proto_mapi);
+	dissector_handle_t mapi_handle;
+
+	mapi_handle = create_dissector_handle(dissect_mapi, proto_mapi);
+	dissector_add("tcp.port", TCP_PORT_MAPI, mapi_handle);
 }
