@@ -10,7 +10,7 @@
  *
  * for information on Modbus/TCP.
  *
- * $Id: packet-mbtcp.c,v 1.4 2001/06/18 02:17:48 guy Exp $
+ * $Id: packet-mbtcp.c,v 1.7 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -230,11 +230,11 @@ dissect_mbtcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8		exception_code = 0, exception_returned = 0;
 	
 /* Make entries in Protocol column on summary display */
-	if (check_col(pinfo->fd, COL_PROTOCOL)) 
-		col_set_str(pinfo->fd, COL_PROTOCOL, "Modbus/TCP");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "Modbus/TCP");
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 /* Make entries in Info column on summary display (updated after building proto tree) */
 	tvb_memcpy(tvb, (guint8 *)&mh, offset, sizeof(mbtcp_hdr));
@@ -246,7 +246,7 @@ dissect_mbtcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		exception_returned = 1;
 	}
 	func_string = function_string(mh.mdbs_hdr.function_code);
-	if (check_col(pinfo->fd, COL_INFO))
+	if (check_col(pinfo->cinfo, COL_INFO))
 	{
 		packet_type = classify_packet(pinfo);
 		switch ( packet_type ) {
@@ -262,7 +262,7 @@ dissect_mbtcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		}
 		if ( exception_returned )
 			strcpy(err_str, "Exception returned ");
-		col_add_fstr(pinfo->fd, COL_INFO, 
+		col_add_fstr(pinfo->cinfo, COL_INFO, 
 				"%8s [%2u pkt(s)]: trans: %5u; unit: %3u, func: %3u: %s. %s", 
 				pkt_type_str, 1, mh.transaction_id, (unsigned char) mh.mdbs_hdr.unit_id, 
 				(unsigned char) mh.mdbs_hdr.function_code, func_string, err_str);
@@ -323,11 +323,11 @@ dissect_mbtcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		}
 		offset = offset + sizeof(mbtcp_hdr) + (mh.len - sizeof(modbus_hdr));
 		packet_num++;
-	} while ( offset < tvb_length(tvb) );
+	} while ( tvb_reported_length_remaining(tvb, offset) > 0 );
 
 	
 /* Update entries in Info column on summary display */
-	if (check_col(pinfo->fd, COL_INFO))
+	if (check_col(pinfo->cinfo, COL_INFO))
 	{
 		switch ( packet_type ) {
 			case query_packet : 			strcpy(pkt_type_str, "query");  
@@ -342,7 +342,7 @@ dissect_mbtcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		}
 		if ( exception_returned )
 			strcpy(err_str, "Exception returned ");
-		col_add_fstr(pinfo->fd, COL_INFO, 
+		col_add_fstr(pinfo->cinfo, COL_INFO, 
 				"%8s [%2u pkt(s)]: trans: %5u; unit: %3u, func: %3u: %s. %s", 
 				pkt_type_str, packet_num, mh.transaction_id, (unsigned char) mh.mdbs_hdr.unit_id, 
 				(unsigned char) mh.mdbs_hdr.function_code, func_string, err_str); 
@@ -416,5 +416,8 @@ proto_register_modbus(void)
 void
 proto_reg_handoff_mbtcp(void)
 {
-	dissector_add("tcp.port", TCP_PORT_MBTCP, dissect_mbtcp, proto_mbtcp);
+	dissector_handle_t mbtcp_handle;
+
+	mbtcp_handle = create_dissector_handle(dissect_mbtcp, proto_mbtcp);
+	dissector_add("tcp.port", TCP_PORT_MBTCP, mbtcp_handle);
 }
