@@ -1,10 +1,10 @@
 /* packet-mount.c
  * Routines for mount dissection
  *
- * $Id: packet-mount.c,v 1.23 2001/04/07 09:25:23 guy Exp $
+ * $Id: packet-mount.c,v 1.27 2001/07/12 08:52:59 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-smb.c
@@ -33,6 +33,7 @@
 #include <sys/types.h>
 #endif
 
+#include <string.h>
 
 #include "packet-rpc.h"
 #include "packet-mount.h"
@@ -94,7 +95,7 @@ dissect_fhstatus(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 	gint32 status;
 
 	status=tvb_get_ntohl(tvb,offset);
-	offset = dissect_rpc_uint32_tvb(tvb,pinfo,tree,hf_mount3_status,offset);
+	offset = dissect_rpc_uint32(tvb,pinfo,tree,hf_mount3_status,offset);
 
 	switch (status) {
 		case 0:
@@ -114,7 +115,7 @@ dissect_mount_dirpath_call(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_
 {
 	if ( tree )
 	{
-		offset = dissect_rpc_string_tvb(tvb,pinfo,tree,hf_mount_path,offset,NULL);
+		offset = dissect_rpc_string(tvb,pinfo,tree,hf_mount_path,offset,NULL);
 	}
 	
 	return offset;
@@ -150,9 +151,9 @@ dissect_mountlist(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tre
 			lock_tree = proto_item_add_subtree(lock_item, ett_mount_mountlist);
 	}
 
-	offset = dissect_rpc_string_tvb(tvb, pinfo, lock_tree, 
+	offset = dissect_rpc_string(tvb, pinfo, lock_tree, 
 			hf_mount_mountlist_hostname, offset, &hostname);
-	offset = dissect_rpc_string_tvb(tvb, pinfo, lock_tree,
+	offset = dissect_rpc_string(tvb, pinfo, lock_tree,
 			hf_mount_mountlist_directory, offset, &directory);
 
 	if (lock_item) {
@@ -173,7 +174,7 @@ dissect_mountlist(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tre
 static int
 dissect_mount_dump_reply(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
-	offset = dissect_rpc_list_tvb(tvb, pinfo, tree, offset, dissect_mountlist);
+	offset = dissect_rpc_list(tvb, pinfo, tree, offset, dissect_mountlist);
 
 	return offset;
 }
@@ -185,21 +186,21 @@ dissect_mount_dump_reply(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 static int
 dissect_group(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
-	int len,strlen;
+	int len,str_len;
 	len=tvb_get_ntohl(tvb,offset);
-	strlen=tvb_get_nstringz(tvb,offset+4,
+	str_len=tvb_get_nstringz(tvb,offset+4,
 		MAX_GROUP_NAME_LIST-5-group_names_len,
 		group_name_list+group_names_len);
-	if((group_names_len>=(MAX_GROUP_NAME_LIST-5))||(strlen<0)){
+	if((group_names_len>=(MAX_GROUP_NAME_LIST-5))||(str_len<0)){
 		strcpy(group_name_list+(MAX_GROUP_NAME_LIST-5),"...");
 		group_names_len=MAX_GROUP_NAME_LIST-1;
 	} else {
-		group_names_len+=strlen;
+		group_names_len+=str_len;
 		group_name_list[group_names_len++]=' ';
 	}
 	group_name_list[group_names_len]=0;
 
-	offset = dissect_rpc_string_tvb(tvb, pinfo, tree, 
+	offset = dissect_rpc_string(tvb, pinfo, tree, 
 			hf_mount_groups_group, offset, NULL);
 
 	return offset;
@@ -228,7 +229,7 @@ dissect_exportlist(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tr
 			exportlist_tree = proto_item_add_subtree(exportlist_item, ett_mount_exportlist);
 	}
 
-	offset = dissect_rpc_string_tvb(tvb, pinfo, exportlist_tree,
+	offset = dissect_rpc_string(tvb, pinfo, exportlist_tree,
 			hf_mount_exportlist_directory, offset, &directory);
 	groups_offset = offset;
 
@@ -239,7 +240,7 @@ dissect_exportlist(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tr
 			groups_tree = proto_item_add_subtree(groups_item, ett_mount_groups);
 	}
 
-	offset = dissect_rpc_list_tvb(tvb, pinfo, groups_tree, offset, dissect_group);
+	offset = dissect_rpc_list(tvb, pinfo, groups_tree, offset, dissect_group);
 	if (groups_item) {
 		/* mark empty lists */
 		if (offset - groups_offset == 4) {
@@ -267,7 +268,7 @@ dissect_exportlist(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tr
 static int
 dissect_mount_export_reply(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
-	offset = dissect_rpc_list_tvb(tvb, pinfo, tree, offset, dissect_exportlist);
+	offset = dissect_rpc_list(tvb, pinfo, tree, offset, dissect_exportlist);
 
 	return offset;
 }
@@ -353,7 +354,7 @@ dissect_mount_pathconf_reply(tvbuff_t *tvb, int offset, packet_info *pinfo, prot
 	pc_mask = tvb_get_ntohl(tvb, offset+OFFS_MASK) & 0xffff;
 	if (!(pc_mask & (PC_ERROR_LINK_MAX|PC_ERROR_ALL))) {
 		if (tree) {
-			dissect_rpc_uint32_tvb(tvb,pinfo,tree,hf_mount_pathconf_link_max,offset);
+			dissect_rpc_uint32(tvb,pinfo,tree,hf_mount_pathconf_link_max,offset);
 		}
 	}
 	offset += 4;
@@ -472,7 +473,7 @@ dissect_mountstat3(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offs
 
 	mountstat3 = tvb_get_ntohl(tvb, offset);
 
-	offset = dissect_rpc_uint32_tvb(tvb,pinfo,tree,hfindex,offset);
+	offset = dissect_rpc_uint32(tvb,pinfo,tree,hfindex,offset);
 	*status = mountstat3;
 	return offset;
 }
@@ -583,109 +584,109 @@ proto_register_mount(void)
 	static hf_register_info hf[] = {
 		{ &hf_mount_path, {
 			"Path", "mount.path", FT_STRING, BASE_DEC,
-			NULL, 0, "Path" }},
+			NULL, 0, "Path", HFILL }},
 		{ &hf_mount3_status, {
 			"Status", "mount.status", FT_UINT32, BASE_DEC,
-			VALS(mount3_mountstat3), 0, "Status" }},
+			VALS(mount3_mountstat3), 0, "Status", HFILL }},
 		{ &hf_mount_mountlist_hostname, {
 			"Hostname", "mount.dump.hostname", FT_STRING, BASE_DEC,
-			NULL, 0, "Hostname" }},
+			NULL, 0, "Hostname", HFILL }},
 		{ &hf_mount_mountlist_directory, {
 			"Directory", "mount.dump.directory", FT_STRING, BASE_DEC,
-			NULL, 0, "Directory" }},
+			NULL, 0, "Directory", HFILL }},
 		{ &hf_mount_mountlist, {
 			"Mount List Entry", "mount.dump.entry", FT_NONE, 0,
-			NULL, 0, "Mount List Entry" }},
+			NULL, 0, "Mount List Entry", HFILL }},
 		{ &hf_mount_groups_group, {
 			"Group", "mount.export.group", FT_STRING, BASE_DEC,
-			NULL, 0, "Group" }},
+			NULL, 0, "Group", HFILL }},
 		{ &hf_mount_groups, {
 			"Groups", "mount.export.groups", FT_NONE, 0,
-			NULL, 0, "Groups" }},
+			NULL, 0, "Groups", HFILL }},
 		{ &hf_mount_exportlist_directory, {
 			"Directory", "mount.export.directory", FT_STRING, BASE_DEC,
-			NULL, 0, "Directory" }},
+			NULL, 0, "Directory", HFILL }},
 		{ &hf_mount_exportlist, {
 			"Export List Entry", "mount.export.entry", FT_NONE, 0,
-			NULL, 0, "Export List Entry" }},
+			NULL, 0, "Export List Entry", HFILL }},
 		{ &hf_mount_pathconf_link_max, {
 			"Maximum number of links to a file", "mount.pathconf.link_max",
 			FT_UINT32, BASE_DEC,
-			NULL, 0, "Maximum number of links allowed to a file" }},
+			NULL, 0, "Maximum number of links allowed to a file", HFILL }},
 		{ &hf_mount_pathconf_max_canon, {
 			"Maximum terminal input line length", "mount.pathconf.max_canon",
 			FT_UINT16, BASE_DEC,
-			NULL, 0, "Max tty input line length" }},
+			NULL, 0, "Max tty input line length", HFILL }},
 		{ &hf_mount_pathconf_max_input, {
 			"Terminal input buffer size", "mount.pathconf.max_input",
 			FT_UINT16, BASE_DEC,
-			NULL, 0, "Terminal input buffer size" }},
+			NULL, 0, "Terminal input buffer size", HFILL }},
 		{ &hf_mount_pathconf_name_max, {
 			"Maximum file name length", "mount.pathconf.name_max",
 			FT_UINT16, BASE_DEC,
-			NULL, 0, "Maximum file name length" }},
+			NULL, 0, "Maximum file name length", HFILL }},
 		{ &hf_mount_pathconf_path_max, {
 			"Maximum path name length", "mount.pathconf.path_max",
 			FT_UINT16, BASE_DEC,
-			NULL, 0, "Maximum path name length" }},
+			NULL, 0, "Maximum path name length", HFILL }},
 		{ &hf_mount_pathconf_pipe_buf, {
 			"Pipe buffer size", "mount.pathconf.pipe_buf",
 			FT_UINT16, BASE_DEC,
-			NULL, 0, "Maximum amount of data that can be written atomically to a pipe" }},
+			NULL, 0, "Maximum amount of data that can be written atomically to a pipe", HFILL }},
 		{ &hf_mount_pathconf_vdisable, {
 			"VDISABLE character", "mount.pathconf.vdisable_char",
 			FT_UINT8, BASE_HEX,
-			NULL, 0, "Character value to disable a terminal special character" }},
+			NULL, 0, "Character value to disable a terminal special character", HFILL }},
 		{ &hf_mount_pathconf_mask, {
 			"Reply error/status bits", "mount.pathconf.mask",
 			FT_UINT16, BASE_HEX,
-			NULL, 0, "Bit mask with error and status bits" }},
+			NULL, 0, "Bit mask with error and status bits", HFILL }},
 		{ &hf_mount_pathconf_error_all, {
 			"ERROR_ALL",	"mount.pathconf.mask.error_all",
 			FT_BOOLEAN, 16, TFS(&tos_error_all),
-			PC_ERROR_ALL, "" }},
+			PC_ERROR_ALL, "", HFILL }},
 		{ &hf_mount_pathconf_error_link_max, {
 			"ERROR_LINK_MAX", "mount.pathconf.mask.error_link_max",
 			FT_BOOLEAN, 16, TFS(&tos_error_link_max),
-			PC_ERROR_LINK_MAX, "" }},
+			PC_ERROR_LINK_MAX, "", HFILL }},
 		{ &hf_mount_pathconf_error_max_canon, {
 			"ERROR_MAX_CANON", "mount.pathconf.mask.error_max_canon",
 			FT_BOOLEAN, 16, TFS(&tos_error_max_canon),
-			PC_ERROR_MAX_CANON, "" }},
+			PC_ERROR_MAX_CANON, "", HFILL }},
 		{ &hf_mount_pathconf_error_max_input, {
 			"ERROR_MAX_INPUT", "mount.pathconf.mask.error_max_input",
 			FT_BOOLEAN, 16, TFS(&tos_error_max_input),
-			PC_ERROR_MAX_INPUT, "" }},
+			PC_ERROR_MAX_INPUT, "", HFILL }},
 		{ &hf_mount_pathconf_error_name_max, {
 			"ERROR_NAME_MAX", "mount.pathconf.mask.error_name_max",
 			FT_BOOLEAN, 16, TFS(&tos_error_name_max),
-			PC_ERROR_NAME_MAX, "" }},
+			PC_ERROR_NAME_MAX, "", HFILL }},
 		{ &hf_mount_pathconf_error_path_max, {
 			"ERROR_PATH_MAX", "mount.pathconf.mask.error_path_max",
 			FT_BOOLEAN, 16, TFS(&tos_error_path_max),
-			PC_ERROR_PATH_MAX, "" }},
+			PC_ERROR_PATH_MAX, "", HFILL }},
 		{ &hf_mount_pathconf_error_pipe_buf, {
 			"ERROR_PIPE_BUF", "mount.pathconf.mask.error_pipe_buf",
 			FT_BOOLEAN, 16, TFS(&tos_error_pipe_buf),
-			PC_ERROR_PIPE_BUF, "" }},
+			PC_ERROR_PIPE_BUF, "", HFILL }},
 		{ &hf_mount_pathconf_chown_restricted, {
 			"CHOWN_RESTRICTED", "mount.pathconf.mask.chown_restricted",
 			FT_BOOLEAN, 16, TFS(&tos_chown_restricted),
-			PC_CHOWN_RESTRICTED, "" }},
+			PC_CHOWN_RESTRICTED, "", HFILL }},
 		{ &hf_mount_pathconf_no_trunc, {
 			"NO_TRUNC", "mount.pathconf.mask.no_trunc",
 			FT_BOOLEAN, 16, TFS(&tos_no_trunc),
-			PC_NO_TRUNC, "" }},
+			PC_NO_TRUNC, "", HFILL }},
 		{ &hf_mount_pathconf_error_vdisable, {
 			"ERROR_VDISABLE", "mount.pathconf.mask.error_vdisable",
 			FT_BOOLEAN, 16, TFS(&tos_error_vdisable),
-			PC_ERROR_VDISABLE, "" }},
+			PC_ERROR_VDISABLE, "", HFILL }},
 		{ &hf_mount_flavors, {
 			"Flavors", "mount.flavors", FT_UINT32, BASE_DEC,
-			NULL, 0, "Flavors" }},
+			NULL, 0, "Flavors", HFILL }},
 		{ &hf_mount_flavor, {
 			"Flavor", "mount.flavor", FT_UINT32, BASE_DEC,
-			VALS(rpc_auth_flavor), 0, "Flavor" }},
+			VALS(rpc_auth_flavor), 0, "Flavor", HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_mount,
