@@ -3,12 +3,11 @@
  * 
  * (c) Copyright Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: packet-mpls.c,v 1.21 2001/06/18 02:17:49 guy Exp $
+ * $Id: packet-mpls.c,v 1.24 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -108,7 +107,8 @@ static hf_register_info mplsf_info[] = {
       "", HFILL }},
 };
 
-static dissector_handle_t ip_handle;
+static dissector_handle_t ipv4_handle;
+static dissector_handle_t ipv6_handle;
 
 /*
  * Given a 4-byte MPLS label starting at offset "offset", in tvbuff "tvb",
@@ -138,17 +138,18 @@ dissect_mpls(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     guint8 exp;
     guint8 bos;
     guint8 ttl;
+    guint8 ipvers;
 
     proto_tree  *mpls_tree;
     proto_item  *ti;
     tvbuff_t *next_tvb;
 
-    if (check_col(pinfo->fd, COL_PROTOCOL)) {
-	col_set_str(pinfo->fd,COL_PROTOCOL, "MPLS");
+    if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+	col_set_str(pinfo->cinfo,COL_PROTOCOL, "MPLS");
     }
     
-    if (check_col(pinfo->fd,COL_INFO)) {
-	col_add_fstr(pinfo->fd,COL_INFO,"MPLS Label Switched Packet");
+    if (check_col(pinfo->cinfo,COL_INFO)) {
+	col_add_fstr(pinfo->cinfo,COL_INFO,"MPLS Label Switched Packet");
     }
 
     /* Start Decoding Here. */
@@ -180,7 +181,13 @@ dissect_mpls(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (bos) break;
     }
     next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-    call_dissector(ip_handle, next_tvb, pinfo, tree);
+
+    ipvers = (tvb_get_guint8(tvb, offset) >> 4) & 0x0F;
+    if (ipvers == 6) {
+	call_dissector(ipv6_handle, next_tvb, pinfo, tree);
+    } else {
+        call_dissector(ipv4_handle, next_tvb, pinfo, tree);
+    }
 }
 
 void
@@ -199,11 +206,15 @@ proto_register_mpls(void)
 void
 proto_reg_handoff_mpls(void)
 {
-	/*
-	 * Get a handle for the IP dissector.
-	 */
-	ip_handle = find_dissector("ip");
+	dissector_handle_t mpls_handle;
 
-	dissector_add("ethertype", ETHERTYPE_MPLS, dissect_mpls, proto_mpls);
-	dissector_add("ppp.protocol", PPP_MPLS_UNI, dissect_mpls, proto_mpls);
+	/*
+	 * Get a handle for the IPv4 and IPv6 dissectors.
+	 */
+	ipv4_handle = find_dissector("ip");
+	ipv6_handle = find_dissector("ipv6");
+
+	mpls_handle = create_dissector_handle(dissect_mpls, proto_mpls);
+	dissector_add("ethertype", ETHERTYPE_MPLS, mpls_handle);
+	dissector_add("ppp.protocol", PPP_MPLS_UNI, mpls_handle);
 }
