@@ -4,7 +4,7 @@
  *
  * Copyright 2001, Heikki Vatiainen <hessu@cs.tut.fi>
  *
- * $Id: packet-msdp.c,v 1.2 2001/07/11 17:56:31 guy Exp $
+ * $Id: packet-msdp.c,v 1.4 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -188,11 +188,11 @@ dissect_msdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         proto_tree *msdp_tree;
         int offset;
 
-        if (check_col(pinfo->fd, COL_PROTOCOL))
-                col_set_str(pinfo->fd, COL_PROTOCOL, "MSDP");
+        if (check_col(pinfo->cinfo, COL_PROTOCOL))
+                col_set_str(pinfo->cinfo, COL_PROTOCOL, "MSDP");
 
-        if (check_col(pinfo->fd, COL_INFO))
-                col_set_str(pinfo->fd, COL_INFO, val_to_str(tvb_get_guint8(tvb, 0),
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_set_str(pinfo->cinfo, COL_INFO, val_to_str(tvb_get_guint8(tvb, 0),
                                                             msdp_types,
                                                             "<Unknown MSDP message type>"));
 
@@ -300,7 +300,7 @@ static void dissect_msdp_sa(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
                  * reflect the MSDP packet rather than the
                  * encapsulated packet.
                  */
-                col_set_writable(pinfo->fd, FALSE);
+                col_set_writable(pinfo->cinfo, FALSE);
                 call_dissector(ip_handle, next_tvb, pinfo, enc_tree);
         }
         *offset += tvb_length_remaining(tvb, *offset);
@@ -544,10 +544,12 @@ proto_register_msdp(void)
 }
 
 void
-proto_reg_handoff_PROTOABBREV(void)
+proto_reg_handoff_msdp(void)
 {
-        dissector_add("tcp.port", 639, dissect_msdp,
-            proto_msdp);
+	dissector_handle_t msdp_handle;
+
+	msdp_handle = create_dissector_handle(dissect_msdp, proto_msdp);
+        dissector_add("tcp.port", 639, msdp_handle);
 
         ip_handle = find_dissector("ip");
 
