@@ -2,7 +2,7 @@
  * Routines for Microsoft Proxy packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-msproxy.c,v 1.21 2001/06/18 02:17:49 guy Exp $
+ * $Id: packet-msproxy.c,v 1.25 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -100,6 +100,8 @@ static int hf_msproxy_server_int_port = -1;
 static int hf_msproxy_server_ext_addr = -1;
 static int hf_msproxy_server_ext_port = -1;
 
+static dissector_handle_t msproxy_sub_handle;
+
 
 #define UDP_PORT_MSPROXY 1745
 
@@ -207,7 +209,7 @@ static void msproxy_sub_dissector( tvbuff_t *tvb, packet_info *pinfo,
 		proto_tree *tree) {
 
 /* Conversation dissector called from TCP or UDP dissector. Decode and	*/
-/* display the socks header, the pass the rest of the data to the tcp	*/
+/* display the msproxy header, the pass the rest of the data to the tcp	*/
 /* or udp port decode routine to  handle the payload.			*/
 
 	guint32 *ptr;
@@ -221,13 +223,14 @@ static void msproxy_sub_dissector( tvbuff_t *tvb, packet_info *pinfo,
 
 	g_assert( conversation);	/* should always find a conversation */
 
-	redirect_info = (redirect_entry_t*)conversation->data;
+	redirect_info = conversation_get_proto_data(conversation,
+		proto_msproxy);
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "MS Proxy");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "MS Proxy");
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_set_str(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_set_str(pinfo->cinfo, COL_INFO,
 			(( redirect_info->proto == PT_TCP) ? "TCP stream" :
 			 "UDP packets"));
 
@@ -280,30 +283,38 @@ static void add_msproxy_conversation( packet_info *pinfo,
 /*	and pinfo->dst will not be correct and this routine will have	*/
 /*	to change.							*/
 
+	conversation_t *conversation;
 	redirect_entry_t *new_conv_info;
 	
-	conversation_t *conversation = find_conversation( &pinfo->src,
+	if (pinfo->fd->flags.visited) {
+		/*
+		 * We've already processed this frame once, so we
+		 * should already have done this.
+		 */
+		return;
+	}
+
+	conversation = find_conversation( &pinfo->src,
 		&pinfo->dst, hash_info->proto, hash_info->server_int_port,
 		hash_info->clnt_port, 0);
 
-	if ( conversation)
-		return;
+	if ( !conversation) {
+		conversation = conversation_new( &pinfo->src, &pinfo->dst,
+			hash_info->proto, hash_info->server_int_port,
+			hash_info->clnt_port, 0);
+	}
+	conversation_set_dissector(conversation, msproxy_sub_handle);
 
 	new_conv_info = g_mem_chunk_alloc(redirect_vals);
-	conversation = conversation_new( &pinfo->src, &pinfo->dst,
-		hash_info->proto, hash_info->server_int_port,
-		hash_info->clnt_port, new_conv_info, 0);
-
-	g_assert( new_conv_info);
-	g_assert( conversation);
 
 	new_conv_info->remote_addr = hash_info->dst_addr;
 	new_conv_info->clnt_port = hash_info->clnt_port;
 	new_conv_info->remote_port = hash_info->dst_port;
 	new_conv_info->server_int_port = hash_info->server_int_port;
 	new_conv_info->proto = hash_info->proto;
-	
-	conversation_set_dissector(conversation, msproxy_sub_dissector);
+
+	conversation_add_proto_data(conversation, proto_msproxy,
+		new_conv_info);
 }
 
 
@@ -1101,34 +1112,34 @@ static void dissect_msproxy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	hash_entry_t *hash_info;
 	conversation_t *conversation;
 	
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "MSproxy");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "MSproxy");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	conversation = find_conversation( &pinfo->src, &pinfo->dst,
 		pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
 
-	if ( conversation)			/* conversation found */
-		hash_info = conversation->data;
-
-			/* new conversation create local data structure */
-	else {				
+	if ( !conversation) {
+		conversation = conversation_new( &pinfo->src, &pinfo->dst,
+			pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+	}
+	hash_info = conversation_get_proto_data(conversation, proto_msproxy);
+	if ( !hash_info) {
     		hash_info = g_mem_chunk_alloc(vals);
-
-		conversation_new( &pinfo->src, &pinfo->dst, pinfo->ptype,
-			pinfo->srcport, pinfo->destport, hash_info, 0);
+		conversation_add_proto_data(conversation, proto_msproxy,
+			hash_info);
 	}
 
-	if (check_col(pinfo->fd, COL_INFO)){
+	if (check_col(pinfo->cinfo, COL_INFO)){
 	
 		cmd = tvb_get_ntohs( tvb, 36);
 		
 		if ( pinfo->srcport == UDP_PORT_MSPROXY)
-			col_add_fstr( pinfo->fd, COL_INFO, "Server message: %s",
+			col_add_fstr( pinfo->cinfo, COL_INFO, "Server message: %s",
 				get_msproxy_cmd_name( cmd, FROM_SERVER));
 		else
-			col_add_fstr(pinfo->fd, COL_INFO, "Client message: %s",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Client message: %s",
 				get_msproxy_cmd_name( cmd, FROM_CLIENT));
 		
 	}
@@ -1279,6 +1290,9 @@ proto_register_msproxy( void){
 	proto_register_subtree_array(ett, array_length(ett));  
 
 	register_init_routine( &msproxy_reinit);	/* register re-init routine */
+
+	msproxy_sub_handle = create_dissector_handle(msproxy_sub_dissector,
+	    proto_msproxy);
 }
 
 
@@ -1286,7 +1300,10 @@ void
 proto_reg_handoff_msproxy(void) {
 
 	/* dissector install routine */ 
- 
-	dissector_add("udp.port", UDP_PORT_MSPROXY, dissect_msproxy,
+
+	dissector_handle_t msproxy_handle;
+
+	msproxy_handle = create_dissector_handle(dissect_msproxy,
 	    proto_msproxy);
+	dissector_add("udp.port", UDP_PORT_MSPROXY, msproxy_handle);
 }
