@@ -1,13 +1,12 @@
 /* packet-nbipx.c
  * Routines for NetBIOS over IPX packet disassembly
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-nbipx.c,v 1.38 2001/04/23 18:10:28 guy Exp $
+ * $Id: packet-nbipx.c,v 1.44 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +36,6 @@
 #include "packet.h"
 #include "packet-ipx.h"
 #include "packet-netbios.h"
-#include "packet-smb.h"
 
 static int proto_nbipx = -1;
 
@@ -220,17 +218,15 @@ dissect_nbipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8		name_type_flag;
 	proto_tree	*name_type_flag_tree;
 	proto_item	*tf;
-	tvbuff_t	*next_tvb;
-	const guint8	*next_pd;
-	int		next_offset;
 	char		name[(NETBIOS_NAME_LEN - 1)*4 + 1];
 	int		name_type;
 	gboolean	has_payload;
+	tvbuff_t	*next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NBIPX");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NBIPX");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	if (pinfo->ipxptype == IPX_PACKET_TYPE_WANBCAST) {
 		/*
@@ -292,8 +288,8 @@ dissect_nbipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	case NBIPX_DEREGISTER_NAME:
 		name_type_flag = tvb_get_guint8(tvb, offset);
 		name_type = get_netbios_name(tvb, offset+2, name);
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO, "%s %s<%02x>",
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO, "%s %s<%02x>",
 				val_to_str(packet_type, nbipx_data_stream_type_vals, "Unknown"),
 				name, name_type);
 		}
@@ -341,8 +337,8 @@ dissect_nbipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	case NBIPX_SESSION_DATA:
 	case NBIPX_SESSION_END:
 	case NBIPX_SESSION_END_ACK:
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO, "%s",
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO, "%s",
 				val_to_str(packet_type, nbipx_data_stream_type_vals, "Unknown"));
 		}
 		dissect_conn_control(tvb, offset, nbipx_tree);
@@ -414,8 +410,8 @@ dissect_nbipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		break;
 
 	case NBIPX_DIRECTED_DATAGRAM:
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO, "%s",
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO, "%s",
 				val_to_str(packet_type, nbipx_data_stream_type_vals, "Unknown"));
 		}
 		dissect_conn_control(tvb, offset, nbipx_tree);
@@ -441,8 +437,8 @@ dissect_nbipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		break;
 
 	default:
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO, "%s",
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO, "%s",
 				val_to_str(packet_type, nbipx_data_stream_type_vals, "Unknown"));
 		}
 
@@ -471,9 +467,7 @@ dissect_nbipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	if (has_payload && tvb_offset_exists(tvb, offset)) {
 		next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-		tvb_compat(next_tvb, &next_pd, &next_offset);
-		dissect_smb(next_pd, next_offset, pinfo->fd, tree,
-		    tvb_length(next_tvb));
+		dissect_netbios_payload(next_tvb, pinfo, tree);
 	}
 }
 
@@ -542,8 +536,10 @@ proto_register_nbipx(void)
 void
 proto_reg_handoff_nbipx(void)
 {
-	dissector_add("ipx.socket", IPX_SOCKET_NETBIOS, dissect_nbipx,
-	    proto_nbipx);
+	dissector_handle_t nbipx_handle;
+
+	nbipx_handle = create_dissector_handle(dissect_nbipx, proto_nbipx);
+	dissector_add("ipx.socket", IPX_SOCKET_NETBIOS, nbipx_handle);
 }
 
 /*
@@ -681,13 +677,11 @@ dissect_nmpi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	char		node_name[(NETBIOS_NAME_LEN - 1)*4 + 1];
 	int		node_name_type = 0;
 	tvbuff_t	*next_tvb;
-	const guint8	*next_pd;
-	int		next_offset;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NMPI");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NMPI");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_nmpi, tvb, offset, 68,
@@ -706,51 +700,51 @@ dissect_nmpi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	name_type = get_netbios_name(tvb, offset+4, name);
 	node_name_type = get_netbios_name(tvb, offset+20, node_name);
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		switch (opcode) {
 
 		case INAME_CLAIM:
-			col_add_fstr(pinfo->fd, COL_INFO, "Claim name %s<%02x>",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Claim name %s<%02x>",
 					name, name_type);
 			break;
 
 		case INAME_DELETE:
-			col_add_fstr(pinfo->fd, COL_INFO, "Delete name %s<%02x>",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Delete name %s<%02x>",
 					name, name_type);
 			break;
 
 		case INAME_QUERY:
-			col_add_fstr(pinfo->fd, COL_INFO, "Query name %s<%02x>",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Query name %s<%02x>",
 					name, name_type);
 			break;
 
 		case INAME_FOUND:
-			col_add_fstr(pinfo->fd, COL_INFO, "Name %s<%02x> found",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Name %s<%02x> found",
 					name, name_type);
 			break;
 
 		case IMSG_HANGUP:
-			col_add_fstr(pinfo->fd, COL_INFO,
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 			    "Messenger hangup on %s<%02x>", name, name_type);
 			break;
 
 		case IMSLOT_SEND:
-			col_add_fstr(pinfo->fd, COL_INFO,
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 			    "Mailslot write to %s<%02x>", name, name_type);
 			break;
 
 		case IMSLOT_FIND:
-			col_add_fstr(pinfo->fd, COL_INFO,
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 			    "Find mailslot name %s<%02x>", name, name_type);
 			break;
 
 		case IMSLOT_NAME:
-			col_add_fstr(pinfo->fd, COL_INFO,
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 			    "Mailslot name %s<%02x> found", name, name_type);
 			break;
 
 		default:
-			col_add_fstr(pinfo->fd, COL_INFO,
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 			    "Unknown NMPI op 0x%02x: name %s<%02x>",
 			    opcode, name, name_type);
 			break;
@@ -777,9 +771,7 @@ dissect_nmpi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	if (opcode == IMSLOT_SEND && tvb_offset_exists(tvb, offset)) {
 		next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-		tvb_compat(next_tvb, &next_pd, &next_offset);
-		dissect_smb(next_pd, next_offset, pinfo->fd, tree,
-		    tvb_length(next_tvb));
+		dissect_netbios_payload(next_tvb, pinfo, tree);
 	}
 }
 
@@ -804,8 +796,11 @@ proto_register_nmpi(void)
 void
 proto_reg_handoff_nmpi(void)
 {
+	dissector_handle_t nmpi_handle;
+
+	nmpi_handle = create_dissector_handle(dissect_nmpi, proto_nmpi);
 	dissector_add("ipx.socket", IPX_SOCKET_NWLINK_SMB_NAMEQUERY,
-	    dissect_nmpi, proto_nmpi);
+	    nmpi_handle);
 	dissector_add("ipx.socket", IPX_SOCKET_NWLINK_SMB_MAILSLOT,
-	    dissect_nmpi, proto_nmpi);
+	    nmpi_handle);
 }
