@@ -1,10 +1,9 @@
 /* packet-nbns.c
  * Routines for NetBIOS-over-TCP packet disassembly (the name dates back
  * to when it had only NBNS)
- * Gilbert Ramirez <gram@xiexie.org>
- * Much stuff added by Guy Harris <guy@alum.mit.edu>
+ * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-nbns.c,v 1.53 2001/07/02 07:11:39 guy Exp $
+ * $Id: packet-nbns.c,v 1.67 2001/12/10 00:25:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,7 +39,8 @@
 #include "packet.h"
 #include "packet-dns.h"
 #include "packet-netbios.h"
-#include "packet-smb.h"
+#include "packet-tcp.h"
+#include "prefs.h"
 
 static int proto_nbns = -1;
 static int hf_nbns_response = -1;
@@ -78,6 +78,9 @@ static int hf_nbss_flags = -1;
 static gint ett_nbss = -1;
 static gint ett_nbss_flags = -1;
 
+/* desegmentation of NBSS over TCP */
+static gboolean nbss_desegment = TRUE;
+
 /* See RFC 1001 and 1002 for information on the first three, and see
 
 	http://www.cifs.com/specs/draft-leach-cifs-v1-spec-01.txt
@@ -328,7 +331,7 @@ add_name_and_type(proto_tree *tree, tvbuff_t *tvb, int offset, int len,
 
 static int
 dissect_nbns_query(tvbuff_t *tvb, int offset, int nbns_data_offset,
-    frame_data *fd, proto_tree *nbns_tree)
+    column_info *cinfo, proto_tree *nbns_tree)
 {
 	int len;
 	char name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME];
@@ -352,8 +355,8 @@ dissect_nbns_query(tvbuff_t *tvb, int offset, int nbns_data_offset,
 	type_name = nbns_type_name(type);
 	class_name = dns_class_name(class);
 
-	if (fd != NULL)
-		col_append_fstr(fd, COL_INFO, " %s %s", type_name, name);
+	if (cinfo != NULL)
+		col_append_fstr(cinfo, COL_INFO, " %s %s", type_name, name);
 	if (nbns_tree != NULL) {
 		tq = proto_tree_add_text(nbns_tree, tvb, offset, len,
 		    "%s: type %s, class %s",  name, type_name, class_name);
@@ -547,7 +550,7 @@ nbns_add_name_flags(proto_tree *rr_tree, tvbuff_t *tvb, int offset,
 
 static int
 dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
-    frame_data *fd, proto_tree *nbns_tree, int opcode)
+    column_info *cinfo, proto_tree *nbns_tree, int opcode)
 {
 	int len;
 	char name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME + 64];
@@ -591,9 +594,9 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
 
 	switch (type) {
 	case T_NB: 		/* "NB" record */
-		if (fd != NULL) {
+		if (cinfo != NULL) {
 			if (opcode != OPCODE_WACK) {
-				col_append_fstr(fd, COL_INFO, " %s %s",
+				col_append_fstr(cinfo, COL_INFO, " %s %s",
 				    type_name,
 				    ip_to_str(tvb_get_ptr(tvb, data_offset+2, 4)));
 			}
@@ -651,8 +654,8 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
 		break;
 
 	case T_NBSTAT: 	/* "NBSTAT" record */
-		if (fd != NULL)
-			col_append_fstr(fd, COL_INFO, " %s", type_name);
+		if (cinfo != NULL)
+			col_append_fstr(cinfo, COL_INFO, " %s", type_name);
 		if (nbns_tree == NULL)
 			break;
 		trr = proto_tree_add_text(nbns_tree, tvb, offset,
@@ -895,8 +898,8 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
 		break;
 
 	default:
-		if (fd != NULL)
-			col_append_fstr(fd, COL_INFO, " %s", type_name);
+		if (cinfo != NULL)
+			col_append_fstr(cinfo, COL_INFO, " %s", type_name);
 		if (nbns_tree == NULL)
 			break;
 		trr = proto_tree_add_text(nbns_tree, tvb, offset,
@@ -915,7 +918,7 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
 
 static int
 dissect_query_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
-    int count, frame_data *fd, proto_tree *nbns_tree)
+    int count, column_info *cinfo, proto_tree *nbns_tree)
 {
 	int start_off, add_off;
 	proto_tree *qatree = NULL;
@@ -928,7 +931,7 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
 	}
 	while (count-- > 0) {
 		add_off = dissect_nbns_query(tvb, cur_off, nbns_data_offset,
-		    fd, qatree);
+		    cinfo, qatree);
 		if (add_off <= 0) {
 			/* We ran past the end of the captured data in the
 			   packet. */
@@ -946,7 +949,8 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
 
 static int
 dissect_answer_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
-    int count, frame_data *fd, proto_tree *nbns_tree, int opcode, char *name)
+    int count, column_info *cinfo, proto_tree *nbns_tree, int opcode,
+    char *name)
 {
 	int start_off, add_off;
 	proto_tree *qatree = NULL;
@@ -959,7 +963,7 @@ dissect_answer_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
 	}
 	while (count-- > 0) {
 		add_off = dissect_nbns_answer(tvb, cur_off, nbns_data_offset,
-					fd, qatree, opcode);
+					cinfo, qatree, opcode);
 		if (add_off <= 0) {
 			/* We ran past the end of the captured data in the
 			   packet. */
@@ -977,7 +981,7 @@ dissect_nbns(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	int			offset = 0;
 	int			nbns_data_offset;
-	frame_data		*fd;
+	column_info		*cinfo;
 	proto_tree		*nbns_tree = NULL;
 	proto_item		*ti;
 	guint16			id, flags, quest, ans, auth, add;
@@ -985,28 +989,28 @@ dissect_nbns(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	nbns_data_offset = offset;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NBNS");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NBNS");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	/* To do: check for runts, errs, etc. */
 	id    = tvb_get_ntohs(tvb, offset + NBNS_ID);
 	flags = tvb_get_ntohs(tvb, offset + NBNS_FLAGS);
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO, "%s%s",
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO, "%s%s",
 		    val_to_str(flags & F_OPCODE, opcode_vals,
 		      "Unknown operation (%x)"),
 		    (flags & F_RESPONSE) ? " response" : "");
-		fd = pinfo->fd;
+		cinfo = pinfo->cinfo;
 	} else {
-		/* Set "fd" to NULL; we pass a NULL "fd" to the query and
-		   answer dissectors, as a way of saying that they shouldn't
-		   add stuff to the COL_INFO column (a call to
-		   "check_col(fd, COL_INFO)" is more expensive than a check
-		   that a pointer isn't NULL). */
-		fd = NULL;
+		/* Set "cinfo" to NULL; we pass a NULL "cinfo" to the query
+		   and answer dissectors, as a way of saying that they
+		   shouldn't add stuff to the COL_INFO column (a call to
+		   "check_col(cinfo, COL_INFO)" is more expensive than
+		   a check that a pointer isn't NULL). */
+		cinfo = NULL;
 	}
 
 	if (tree) {
@@ -1057,7 +1061,7 @@ dissect_nbns(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		   answers. */
 		cur_off += dissect_query_records(tvb, cur_off,
 		    nbns_data_offset, quest,
-		    (!(flags & F_RESPONSE) ? fd : NULL), nbns_tree);
+		    (!(flags & F_RESPONSE) ? cinfo : NULL), nbns_tree);
 	}
 
 	if (ans > 0) {
@@ -1066,7 +1070,7 @@ dissect_nbns(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		   queries. */
 		cur_off += dissect_answer_records(tvb, cur_off,
 			nbns_data_offset, ans,
-			((flags & F_RESPONSE) ? fd : NULL), nbns_tree,
+			((flags & F_RESPONSE) ? cinfo : NULL), nbns_tree,
 			flags & F_OPCODE, "Answers");
 	}
 
@@ -1109,52 +1113,67 @@ struct nbdgm_header {
 	guint8		error_code;
 };
 
+/*
+ * NBDS message types.
+ */
+#define NBDS_DIRECT_UNIQUE	0x10
+#define NBDS_DIRECT_GROUP	0x11
+#define NBDS_BROADCAST		0x12
+#define NBDS_ERROR		0x13
+#define NBDS_QUERY_REQUEST	0x14
+#define NBDS_POS_QUERY_RESPONSE	0x15
+#define NBDS_NEG_QUERY_RESPONSE	0x16
+
+static const value_string nbds_msgtype_vals[] = {
+	{ NBDS_DIRECT_UNIQUE,      "Direct_unique datagram" },
+	{ NBDS_DIRECT_GROUP,       "Direct_group datagram" },
+	{ NBDS_BROADCAST,          "Broadcast datagram" },
+	{ NBDS_ERROR,              "Datagram error" },
+	{ NBDS_QUERY_REQUEST,      "Datagram query request" },
+	{ NBDS_POS_QUERY_RESPONSE, "Datagram positive query response" },
+	{ NBDS_NEG_QUERY_RESPONSE, "Datagram negative query response" },
+	{ 0,                       NULL }
+};
+
+static const true_false_string yesno = {  
+	"Yes",
+	"No"
+};
+
+static const value_string node_type_vals[] = {
+	{ 0, "B node" },
+	{ 1, "P node" },
+	{ 2, "M node" },
+	{ 3, "NBDD" },
+	{ 0, NULL }
+};
+
 static void
 dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	int			offset = 0;
 	proto_tree		*nbdgm_tree = NULL;
-	proto_item		*ti;
+	proto_item		*ti = NULL;
 	struct nbdgm_header	header;
 	int			flags;
 	int			message_index;
-	int			max_data = tvb_length_remaining(tvb, offset);
+	tvbuff_t		*next_tvb;
 
-	char *message[] = {
-		"Unknown",
-		"Direct_unique datagram",
-		"Direct_group datagram",
-		"Broadcast datagram",
-		"Datagram error",
-		"Datagram query request",
-		"Datagram positive query response",
-		"Datagram negative query response"
-	};
-
-	char *node[] = {
-		"B node",
-		"P node",
-		"M node",
-		"NBDD"
-	};
-
-	static value_string error_codes[] = {
+	static const value_string error_codes[] = {
 		{ 0x82, "Destination name not present" },
 		{ 0x83, "Invalid source name format" },
 		{ 0x84, "Invalid destination name format" },
 		{ 0x00,	NULL }
 	};
 
-	char *yesno[] = { "No", "Yes" };
-
 	char name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME];
 	int name_type;
 	int len;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NBDS");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NBDS");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	header.msg_type = tvb_get_guint8(tvb, offset);
 	
@@ -1167,13 +1186,18 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	tvb_memcpy(tvb, (guint8 *)&header.src_ip, offset+4, 4);
 	header.src_port = tvb_get_ntohs(tvb, offset+8);
 
-	if (header.msg_type == 0x10 ||
-			header.msg_type == 0x11 || header.msg_type == 0x12) {
+	switch (header.msg_type) {
+
+	case NBDS_DIRECT_UNIQUE:
+	case NBDS_DIRECT_GROUP:
+	case NBDS_BROADCAST:
 		header.dgm_length = tvb_get_ntohs(tvb, offset+10);
 		header.pkt_offset = tvb_get_ntohs(tvb, offset+12);
-	}
-	else if (header.msg_type == 0x13) {
+		break;
+
+	case NBDS_ERROR:
 		header.error_code = tvb_get_ntohs(tvb, offset+10);
+		break;
 	}
 
 	message_index = header.msg_type - 0x0f;
@@ -1181,34 +1205,29 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		message_index = 0;
 	}
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_set_str(pinfo->fd, COL_INFO, message[message_index]);
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_str(pinfo->cinfo, COL_INFO,
+		    val_to_str(header.msg_type, nbds_msgtype_vals,
+		      "Unknown message type (0x%02X)"));
 	}
 
 	if (tree) {
-		ti = proto_tree_add_item(tree, proto_nbdgm, tvb, offset, header.dgm_length, FALSE);
+		ti = proto_tree_add_item(tree, proto_nbdgm, tvb, offset,
+		    tvb_length_remaining(tvb, offset), FALSE);
 		nbdgm_tree = proto_item_add_subtree(ti, ett_nbdgm);
 
-		proto_tree_add_uint_format(nbdgm_tree, hf_nbdgm_type, tvb,
-					   offset, 1, 
-					   header.msg_type,   
-					   "Message Type: %s",
-					   message[message_index]);
-		proto_tree_add_boolean_format(nbdgm_tree, hf_nbdgm_fragment, tvb,
-					   offset+1, 1, 
-					   header.flags.more,
-					   "More fragments follow: %s",
-					   yesno[header.flags.more]);
-		proto_tree_add_boolean_format(nbdgm_tree, hf_nbdgm_first, tvb,
-					   offset+1, 1, 
-					   header.flags.first,
-					   "This is first fragment: %s",
-					   yesno[header.flags.first]);
-		proto_tree_add_uint_format(nbdgm_tree, hf_nbdgm_node_type, tvb,
-					   offset+1, 1, 
-					   header.flags.node_type,
-					   "Node Type: %s",
-					   node[header.flags.node_type]);
+		proto_tree_add_uint(nbdgm_tree, hf_nbdgm_type, tvb,
+				     offset, 1, 
+				     header.msg_type);
+		proto_tree_add_boolean(nbdgm_tree, hf_nbdgm_fragment, tvb,
+				       offset+1, 1, 
+				       header.flags.more);
+		proto_tree_add_boolean(nbdgm_tree, hf_nbdgm_first, tvb,
+				       offset+1, 1, 
+				       header.flags.first);
+		proto_tree_add_uint(nbdgm_tree, hf_nbdgm_node_type, tvb,
+				     offset+1, 1, 
+				     header.flags.node_type);
 
 		proto_tree_add_uint(nbdgm_tree, hf_nbdgm_datagram_id, tvb,
 				    offset+2, 2, header.dgm_id);
@@ -1220,11 +1239,12 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
 
 	offset += 10;
-	max_data -= 10;
 
-	if (header.msg_type == 0x10 ||
-			header.msg_type == 0x11 || header.msg_type == 0x12) {
+	switch (header.msg_type) {
 
+	case NBDS_DIRECT_UNIQUE:
+	case NBDS_DIRECT_GROUP:
+	case NBDS_BROADCAST:
 		if (tree) {
 			proto_tree_add_text(nbdgm_tree, tvb, offset, 2,
 					"Datagram length: %d bytes", header.dgm_length);
@@ -1233,7 +1253,6 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		}
 
 		offset += 4;
-		max_data -= 4;
 
 		/* Source name */
 		len = get_nbns_name(tvb, offset, offset, name, &name_type);
@@ -1243,7 +1262,6 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			    "Source name", name, name_type);
 		}
 		offset += len;
-		max_data -= len;
 
 		/* Destination name */
 		len = get_nbns_name(tvb, offset, offset, name, &name_type);
@@ -1253,31 +1271,31 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			    "Destination name", name, name_type);
 		}
 		offset += len;
-		max_data -= len;
 
-		  /*
-		   * Here we can pass the packet off to the next protocol.
-		   */
-		  {
-			tvbuff_t	*next_tvb;
-			const guint8	*next_pd;
-			int		next_offset;
+		/*
+		 * Here we can pass the packet off to the next protocol.
+		 * Set the length of our top-level tree item to include
+		 * only our stuff.
+		 *
+		 * XXX - take the datagram length into account?
+		 */
+		proto_item_set_len(ti, offset);
+		next_tvb = tvb_new_subset(tvb, offset, -1, -1);
+		dissect_netbios_payload(next_tvb, pinfo, tree);
+		break;
 
-			next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-			tvb_compat(next_tvb, &next_pd, &next_offset);
-
-			dissect_smb(next_pd, next_offset, pinfo->fd, tree,
-			    max_data);
-		  }
-	}
-	else if (header.msg_type == 0x13) {
+	case NBDS_ERROR:
 		if (tree) {
 			proto_tree_add_text(nbdgm_tree, tvb, offset, 1, "Error code: %s",
 				val_to_str(header.error_code, error_codes, "Unknown (0x%x)"));
 		}
-	}
-	else if (header.msg_type == 0x14 ||
-			header.msg_type == 0x15 || header.msg_type == 0x16) {
+		offset += 1;
+		proto_item_set_len(ti, offset);
+		break;
+
+	case NBDS_QUERY_REQUEST:
+	case NBDS_POS_QUERY_RESPONSE:
+	case NBDS_NEG_QUERY_RESPONSE:
 		/* Destination name */
 		len = get_nbns_name(tvb, offset, offset, name, &name_type);
 
@@ -1285,6 +1303,9 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			add_name_and_type(nbdgm_tree, tvb, offset, len,
 			    "Destination name", name, name_type);
 		}
+		offset += len;
+		proto_item_set_len(ti, offset);
+		break;
 	}
 }
 
@@ -1341,15 +1362,17 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
     proto_tree *tree, int max_data, int is_cifs)
 {
 	proto_tree	*nbss_tree = NULL;
-	proto_item	*ti;
+	proto_item	*ti = NULL;
 	proto_tree	*field_tree;
 	proto_item	*tf;
 	guint8		msg_type;
 	guint8		flags;
-	guint16		length;
+	int		length;
 	int		len;
 	char		name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME];
 	int		name_type;
+	gint		reported_len;
+	tvbuff_t	*next_tvb;
 
 	msg_type = tvb_get_guint8(tvb, offset);
 
@@ -1363,6 +1386,25 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			length += 65536;
 	}
 
+	/*Desegmentation */
+	if (nbss_desegment) {
+		if (pinfo->can_desegment
+		    && length > tvb_length_remaining(tvb, offset+4)) {
+			/*
+			 * This frame doesn't have all of the data for
+			 * this message, but we can do reassembly on it.
+			 *
+			 * Tell the TCP dissector where the data for this
+			 * message starts in the data it handed us, and
+			 * how many more bytes we need, and return.
+			 */
+			pinfo->desegment_offset = offset;
+			pinfo->desegment_len =
+			    length - tvb_length_remaining(tvb, offset+4);
+			return max_data;
+		}
+	}
+
 	if (tree) {
 	  ti = proto_tree_add_item(tree, proto_nbss, tvb, offset, length + 4, FALSE);
 	  nbss_tree = proto_item_add_subtree(ti, ett_nbss);
@@ -1441,19 +1483,19 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 	case SESSION_MESSAGE:
 	  /*
-	   * Here we can pass the packet off to the next protocol.
+	   * Here we can pass the message off to the next protocol.
+	   * Set the length of our top-level tree item to include
+	   * only our stuff.
 	   */
-	  {
-		tvbuff_t	*next_tvb;
-		const guint8	*next_pd;
-		int		next_offset;
-
-		next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-		tvb_compat(next_tvb, &next_pd, &next_offset);
-
-		dissect_smb(next_pd, next_offset, pinfo->fd, tree,
-		    max_data - 4);
-	  }
+	  proto_item_set_len(ti, offset);
+	  len = tvb_length_remaining(tvb, offset);
+	  reported_len = tvb_reported_length_remaining(tvb, offset);
+	  if (len > length)
+	    len = length;
+	  if (reported_len > length)
+	    reported_len = length;
+	  next_tvb = tvb_new_subset(tvb, offset, len, reported_len);
+	  dissect_netbios_payload(next_tvb, pinfo, tree);
 	  break;
 
 	}
@@ -1463,25 +1505,26 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 static void
 dissect_nbss(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
+	struct tcpinfo	*tcpinfo = pinfo->private_data;
 	int		offset = 0;
-	guint8		msg_type;
 	int		max_data;
+	guint8		msg_type;
+	guint8		flags;
+	guint32		length;
 	int		len;
 	gboolean	is_cifs;
+	proto_tree	*nbss_tree;
+	proto_item	*ti;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NBSS");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NBSS");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
+
+	max_data = tvb_length(tvb);
 
 	msg_type = tvb_get_guint8(tvb, offset);
 
-	/*
-	 * XXX - we should set this based on both the length remaining
-	 * and "length"....
-	 */
-	max_data = tvb_length_remaining(tvb, offset);
-
 	if (pinfo->match_port == TCP_PORT_CIFS) {
 		/*
 		 * Windows 2000 CIFS clients can dispense completely
@@ -1496,40 +1539,134 @@ dissect_nbss(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		is_cifs = FALSE;
 	}
 
-	/* Hmmm, it may be a continuation message ... */
+	/*
+	 * This might be a continuation of an earlier message.
+	 * (Yes, that might be true even if we're doing TCP reassembly,
+	 * as the first TCP segment in the capture might start in the
+	 * middle of an NBNS message.)
+	 */
 
-#define RJSHACK 1
-#ifdef RJSHACK
-	if (((msg_type != SESSION_REQUEST) && 
-	     (msg_type != POSITIVE_SESSION_RESPONSE) &&
-	     (msg_type != NEGATIVE_SESSION_RESPONSE) &&
-	     (msg_type != RETARGET_SESSION_RESPONSE) &&
-	     (msg_type != SESSION_KEEP_ALIVE) &&
-	     (msg_type != SESSION_MESSAGE)) ||
-	    ((msg_type == SESSION_MESSAGE) &&
-	     (max_data < 8 || tvb_memeql(tvb, offset + 4, "\377SMB", 4) != 0))) {
- 
-	  /*
-	   * Either the first byte isn't one of the known message types,
-	   * or it's a session message but we either don't have enough
-	   * data in the frame for the NBSS/CIFS header plus an SMB header,
-	   * or we do but the message data doesn't begin with 0xFF S M B.
-	   * Assume it's a continuation message.
-	   */
-	  if (check_col(pinfo->fd, COL_INFO)) {
-	    col_add_fstr(pinfo->fd, COL_INFO, "NBSS Continuation Message");
-	  }
+	/*
+	 * If this isn't reassembled data, check to see whether it
+	 * looks like a continuation of a message.
+	 * (If it is reassembled data, it shouldn't be a continuation,
+	 * as reassembly should've gathered the continuations together
+	 * into a message.)
+	 */
+	if (!tcpinfo->is_reassembled) {
+		if (max_data < 4) {
+			/*
+			 * Not enough data for an NBSS header; assume
+			 * it's a continuation of a message.
+			 *
+			 * XXX - if there's not enough data, we should
+			 * attempt to reassemble the data, if the first byte
+			 * is a valid message type.
+			 */
+			goto continuation;
+		}
 
-	  if (tree)
-	    proto_tree_add_text(tree, tvb, offset, max_data, "Continuation data");
+		/*
+		 * We have enough data for an NBSS header.
+		 * Get the flags and length of the message,
+		 * and see if they're sane.
+		 */
+		if (is_cifs) {
+			flags = 0;
+			length = tvb_get_ntoh24(tvb, offset + 1);
+		} else {
+			flags = tvb_get_guint8(tvb, offset + 1);
+			length = tvb_get_ntohs(tvb, offset + 2);
+			if (flags & NBSS_FLAGS_E)
+				length += 65536;
+		}
+		if ((flags & (~NBSS_FLAGS_E)) != 0) {
+			/*
+			 * A bogus flag was set; assume it's a continuation.
+			 */
+			goto continuation;
+		}
 
-	  return;
+		switch (msg_type) {
+
+		case SESSION_MESSAGE:
+			/*
+			 * This is variable-length.
+			 * All we know is that it shouldn't be zero.
+			 * (XXX - can we get zero-length messages?
+			 * Not with SMB, but perhaps other NetBIOS-based
+			 * protocols have them.)
+			 */
+			if (length == 0)
+				goto continuation;
+			break;
+
+		case SESSION_REQUEST:
+			/*
+			 * This is variable-length.
+			 * The names are DNS-encoded 32-byte values;
+			 * we need at least 2 bytes (one for each name;
+			 * actually, we should have more for the first
+			 * name, as there's no name preceding it so
+			 * there should be no compression), and we
+			 * shouldn't have more than 128 bytes (actually,
+			 * we shouldn't have that many).
+			 *
+			 * XXX - actually, MacOS X 10.1 (yes, that's
+			 * redundant, but that's what Apple calls it,
+			 * not MacOS X.1) puts names longer than 16
+			 * characters into session request messages,
+			 * so we can have more than 32 bytes of
+			 * name value, so we can have more than 128
+			 * bytes of data.
+			 */
+			if (length < 2 || length > 256)
+				goto continuation;
+			break;
+
+		case POSITIVE_SESSION_RESPONSE:
+			/*
+			 * This has no data, so the length must be zero.
+			 */
+			if (length != 0)
+				goto continuation;
+			break;
+
+		case NEGATIVE_SESSION_RESPONSE:
+			/*
+			 * This has 1 byte of data.
+			 */
+			if (length != 1)
+				goto continuation;
+			break;
+
+		case RETARGET_SESSION_RESPONSE:
+			/*
+			 * This has 6 bytes of data.
+			 */
+			if (length != 6)
+				goto continuation;
+			break;
+
+		case SESSION_KEEP_ALIVE:
+			/*
+			 * This has no data, so the length must be zero.
+			 */
+			if (length != 0)
+				goto continuation;
+			break;
+
+		default:
+			/*
+			 * Unknown message type; assume it's a continuation.
+			 */
+			goto continuation;
+		}
 	}
-#endif
-
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO,
-		    val_to_str(msg_type, message_types, "Unknown (%x)"));
+ 
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO,
+		    val_to_str(msg_type, message_types, "Unknown (%02x)"));
 	}
 
 	while (max_data > 0) {
@@ -1538,6 +1675,23 @@ dissect_nbss(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		offset += len;
 		max_data -= len;
 	}
+
+	return;
+
+continuation:
+	/*
+	 * It looks like a continuation.
+	 */
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_fstr(pinfo->cinfo, COL_INFO, "NBSS Continuation Message");
+
+	if (tree) {
+		ti = proto_tree_add_item(tree, proto_nbss, tvb, 0,
+		    max_data, FALSE);
+		nbss_tree = proto_item_add_subtree(ti, ett_nbss);
+		proto_tree_add_text(nbss_tree, tvb, 0, max_data,
+		    "Continuation data");
+	}
 }
 
 void
@@ -1578,19 +1732,19 @@ proto_register_nbt(void)
   static hf_register_info hf_nbdgm[] = {
     { &hf_nbdgm_type,
       { "Message Type",		"nbdgm.type",  
-	FT_UINT8, BASE_DEC, NULL, 0x0,
+	FT_UINT8, BASE_DEC, VALS(nbds_msgtype_vals), 0x0,
 	"NBDGM message type", HFILL }},
     { &hf_nbdgm_fragment,
-      { "Fragmented",		"nbdgm.next",  
-	FT_BOOLEAN, BASE_NONE, NULL, 0x0,
+      { "More fragments follow",	"nbdgm.next",  
+	FT_BOOLEAN, BASE_NONE, TFS(&yesno), 0x0,
 	"TRUE if more fragments follow", HFILL }},
     { &hf_nbdgm_first,
-      { "First fragment",	"nbdgm.first",  
-	FT_BOOLEAN, BASE_NONE, NULL, 0x0,
+      { "This is first fragment",	"nbdgm.first",  
+	FT_BOOLEAN, BASE_NONE, TFS(&yesno), 0x0,
 	"TRUE if first fragment", HFILL }},
     { &hf_nbdgm_node_type,
       { "Node Type",		"nbdgm.node_type",  
-	FT_UINT8, BASE_DEC, NULL, 0x0,
+	FT_UINT8, BASE_DEC, VALS(node_type_vals), 0x0,
 	"Node type", HFILL }},
     { &hf_nbdgm_datagram_id,
       { "Datagram ID",		"nbdgm.dgram_id",  
@@ -1629,6 +1783,7 @@ proto_register_nbt(void)
     &ett_nbss,
     &ett_nbss_flags,
   };
+  module_t *nbss_module;
 
   proto_nbns = proto_register_protocol("NetBIOS Name Service", "NBNS", "nbns");
   proto_register_field_array(proto_nbns, hf_nbns, array_length(hf_nbns));
@@ -1642,13 +1797,24 @@ proto_register_nbt(void)
   proto_register_field_array(proto_nbss, hf_nbss, array_length(hf_nbss));
 
   proto_register_subtree_array(ett, array_length(ett));
+
+  nbss_module = prefs_register_protocol(proto_nbss, NULL);
+  prefs_register_bool_preference(nbss_module, "desegment_nbss_commands",
+    "Desegment all NBSS packets spanning multiple TCP segments",
+    "Whether NBSS dissector should desegment all packets spanning multiple TCP segments",
+    &nbss_desegment);
 }
 
 void
 proto_reg_handoff_nbt(void)
 {
-  dissector_add("udp.port", UDP_PORT_NBNS, dissect_nbns, proto_nbns);
-  dissector_add("udp.port", UDP_PORT_NBDGM, dissect_nbdgm, proto_nbdgm);
-  dissector_add("tcp.port", TCP_PORT_NBSS, dissect_nbss, proto_nbss);
-  dissector_add("tcp.port", TCP_PORT_CIFS, dissect_nbss, proto_nbss);
+  dissector_handle_t nbns_handle, nbdgm_handle, nbss_handle;
+
+  nbns_handle = create_dissector_handle(dissect_nbns, proto_nbns);
+  dissector_add("udp.port", UDP_PORT_NBNS, nbns_handle);
+  nbdgm_handle = create_dissector_handle(dissect_nbdgm, proto_nbdgm);
+  dissector_add("udp.port", UDP_PORT_NBDGM, nbdgm_handle);
+  nbss_handle = create_dissector_handle(dissect_nbss, proto_nbss);
+  dissector_add("tcp.port", TCP_PORT_NBSS, nbss_handle);
+  dissector_add("tcp.port", TCP_PORT_CIFS, nbss_handle);
 }
