@@ -5,12 +5,11 @@
  * 
  * derived from the packet-nbns.c
  *
- * $Id: packet-netbios.c,v 1.34 2001/06/18 02:17:49 guy Exp $
+ * $Id: packet-netbios.c,v 1.43 2001/12/10 00:25:31 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,8 +26,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -44,7 +41,6 @@
 #include "packet.h"
 #include "llcsaps.h"
 #include "packet-netbios.h"
-#include "packet-smb.h"
 
 /* Netbios command numbers */
 #define NB_ADD_GROUP		0x00
@@ -111,6 +107,8 @@ static gint ett_netb_name = -1;
 static gint ett_netb_flags = -1;
 static gint ett_netb_status = -1;
 
+static dissector_handle_t data_handle;
+
 /* The strings for the station type, used by get_netbios_name function;
    many of them came from the file "NetBIOS.txt" in the Zip archive at
 
@@ -222,7 +220,7 @@ static const true_false_string netb_version_str = {
 };
 
 
-void capture_netbios(const u_char *pd, int offset, packet_counts *ld)
+void capture_netbios(const u_char *pd, int offset, int len, packet_counts *ld)
 {
 	ld->netbios++;
 }
@@ -520,7 +518,7 @@ static void dissect_netb_unknown( tvbuff_t *tvb, int offset, proto_tree *tree)
 
 {/* Handle any unknow commands, do nothing */
 
-/*	dissect_data( tvb, offset + NB_COMMAND + 1, pinfo, tree); */
+/*	call_dissector(data_handle, tvb_new_subset(tvb, offset + NB_COMMAND + 1,-1,tvb_reported_length_remaining(tvb,offset_NB_COMMAND + 1)), pinfo, tree); */
 }
 
 
@@ -957,6 +955,19 @@ void (*dissect_netb[])(tvbuff_t *, int, proto_tree *) = {
   dissect_netb_unknown,	
 };
 
+static heur_dissector_list_t netbios_heur_subdissector_list;
+
+void
+dissect_netbios_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	/*
+	 * Try the heuristic dissectors for NetBIOS; if none of them
+	 * accept the packet, dissect it as data.
+	 */
+	if (!dissector_try_heuristic(netbios_heur_subdissector_list,
+				    tvb, pinfo, tree))
+		call_dissector(data_handle,tvb, pinfo, tree);
+}
 
 static void
 dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
@@ -967,12 +978,13 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint16			hdr_len, command;
 	char 			name[(NETBIOS_NAME_LEN - 1)*4 + 1];
 	int			name_type;
+	tvbuff_t		*next_tvb;
 
 	int offset = 0;
 
 					/* load the display labels 	*/
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NetBIOS");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NetBIOS");
 
 
 /* Find NetBIOS marker EFFF, this is done because I have seen an extra LLC */
@@ -982,8 +994,8 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		++offset;
 		if ( 0xefff != tvb_get_letohs(tvb, 3)){
 
-			if (check_col( pinfo->fd, COL_INFO)) 	/* print bad packet */
-				col_set_str( pinfo->fd, COL_INFO, "Bad packet, no 0xEFFF marker");
+			if (check_col( pinfo->cinfo, COL_INFO)) 	/* print bad packet */
+				col_set_str( pinfo->cinfo, COL_INFO, "Bad packet, no 0xEFFF marker");
 
 			return;		/* this is an unknow packet, no marker */
 		}
@@ -995,11 +1007,11 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					/* limit command so no table overflows */
 	command = MIN( command, sizeof( dissect_netb)/ sizeof(void *));
 
-        if (check_col( pinfo->fd, COL_INFO)) {              /* print command name */
+        if (check_col( pinfo->cinfo, COL_INFO)) {              /* print command name */
                 switch ( command ) {
                 case NB_NAME_QUERY:
                         name_type = get_netbios_name( tvb, offset + 12, name);
-                        col_add_fstr( pinfo->fd, COL_INFO, "%s for %s<%02x>",
+                        col_add_fstr( pinfo->cinfo, COL_INFO, "%s for %s<%02x>",
                             CommandName[ command], name, name_type);
                         break;
 
@@ -1007,18 +1019,18 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 case NB_ADD_NAME:
                 case NB_ADD_GROUP:
                         name_type = get_netbios_name( tvb, offset + 28, name);
-                        col_add_fstr( pinfo->fd, COL_INFO, "%s - %s<%02x>",
+                        col_add_fstr( pinfo->cinfo, COL_INFO, "%s - %s<%02x>",
                             CommandName[ command], name, name_type);
                         break;
 
 		default:
-			col_add_fstr( pinfo->fd, COL_INFO, "%s", CommandName[ command]);
+			col_add_fstr( pinfo->cinfo, COL_INFO, "%s", CommandName[ command]);
 			break;
 		}
 	}
 
 	if ( tree) {
-		ti = proto_tree_add_item(tree, proto_netbios, tvb, 0, tvb_length(tvb), FALSE);
+		ti = proto_tree_add_item(tree, proto_netbios, tvb, 0, hdr_len, FALSE);
 		netb_tree = proto_item_add_subtree(ti, ett_netb);
 
 		proto_tree_add_uint_format(netb_tree, hf_netb_hdr_len, tvb, offset, 2, hdr_len,
@@ -1037,29 +1049,10 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			(dissect_netb[ command])( tvb, offset, netb_tree);		
 	}
 
-							/* Test for SMB data */
-	if (  tvb_length(tvb)> ( hdr_len + 4)){		/* if enough data */
+	offset += hdr_len;			/* move past header */
 
-		offset += hdr_len;			/* move past header */
-
-		if (( tvb_get_guint8( tvb, offset) == 0xff) &&	/* if SMB marker */
-		    ( tvb_get_guint8( tvb, offset + 1) == 'S') &&
-		    ( tvb_get_guint8( tvb, offset + 2) == 'M') &&
-		    ( tvb_get_guint8( tvb, offset + 3) == 'B')) {
-
-			tvbuff_t	*next_tvb;
-			const guint8	*next_pd;
-			int		next_offset;
-
-			next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-			tvb_compat(next_tvb, &next_pd, &next_offset);
-
-							/* decode SMB */
-			dissect_smb( next_pd, next_offset, pinfo->fd, tree, 
-				tvb_length_remaining(tvb, offset) - next_offset);
-
-		}
-	}
+	next_tvb = tvb_new_subset(tvb, offset, -1, -1);
+	dissect_netbios_payload(next_tvb, pinfo, tree);
 }
 
 
@@ -1141,10 +1134,17 @@ void proto_register_netbios(void)
 	proto_netbios = proto_register_protocol("NetBIOS", "NetBIOS", "netbios");
 	proto_register_subtree_array(ett, array_length(ett));                 
 	proto_register_field_array(proto_netbios, hf_netb, array_length(hf_netb));
+
+	register_heur_dissector_list("netbios", &netbios_heur_subdissector_list);
 }
 
 void
 proto_reg_handoff_netbios(void)
 {
-	dissector_add("llc.dsap", SAP_NETBIOS, dissect_netbios, proto_netbios);
+	dissector_handle_t netbios_handle;
+
+	netbios_handle = create_dissector_handle(dissect_netbios,
+	    proto_netbios);
+	dissector_add("llc.dsap", SAP_NETBIOS, netbios_handle);
+	data_handle = find_dissector("data");
 }
