@@ -5,12 +5,11 @@
  * 
  * derived from the packet-nbns.c
  *
- * $Id: packet-netbios.h,v 1.9 2001/01/15 04:39:28 guy Exp $
+ * $Id: packet-netbios.h,v 1.12 2001/11/20 21:59:13 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,7 +32,7 @@
 /* Length of NetBIOS names */
 #define NETBIOS_NAME_LEN	16
 
-void capture_netbios(const u_char *, int, packet_counts *);
+void capture_netbios(const u_char *, int, int, packet_counts *);
 
 extern int process_netbios_name(const u_char *name_ptr, char *name_ret);
 extern int get_netbios_name(tvbuff_t *tvb, int offset,
@@ -41,5 +40,7 @@ extern int get_netbios_name(tvbuff_t *tvb, int offset,
 extern char *netbios_name_type_descr(int name_type);
 extern void netbios_add_name( char* label, tvbuff_t *tvb, int offset,
     proto_tree *tree);
+extern void dissect_netbios_payload(tvbuff_t *tvb, packet_info *pinfo,
+    proto_tree *tree);
 
 #endif
