@@ -2,10 +2,10 @@
  * Routines for NTP packet dissection
  * Copyright 1999, Nathan Neulinger <nneul@umr.edu>
  *
- * $Id: packet-ntp.c,v 1.28 2001/06/18 02:17:50 guy Exp $
+ * $Id: packet-ntp.c,v 1.32 2001/12/10 00:25:31 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
@@ -236,10 +236,14 @@ ntp_fmt_ts(const guint8 *reftime, char* buff)
 	} else {
 		temptime = tempstmp - (guint32) NTP_BASETIME;
 		bd = gmtime(&temptime);
-		fractime = bd->tm_sec + tempfrac / 4294967296.0;
-		snprintf(buff, NTP_TS_SIZE, "%04d-%02d-%02d %02d:%02d:%07.4f UTC",
-			 bd->tm_year + 1900, bd->tm_mon + 1, bd->tm_mday,
-			 bd->tm_hour, bd->tm_min, fractime);
+		if (bd != NULL) {
+			fractime = bd->tm_sec + tempfrac / 4294967296.0;
+			snprintf(buff, NTP_TS_SIZE,
+				 "%04d-%02d-%02d %02d:%02d:%07.4f UTC",
+				 bd->tm_year + 1900, bd->tm_mon + 1, bd->tm_mday,
+				 bd->tm_hour, bd->tm_min, fractime);
+		} else
+			strncpy(buff, "Not representable", NTP_TS_SIZE);
 	}
 	return buff;
 }
@@ -268,11 +272,11 @@ dissect_ntp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gchar		buff[NTP_TS_SIZE];
 	int		i;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "NTP");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NTP");
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_set_str(pinfo->fd, COL_INFO, "NTP");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_set_str(pinfo->cinfo, COL_INFO, "NTP");
 
 	if (tree) {
 		/* Adding NTP item and subtree */
@@ -498,6 +502,9 @@ proto_register_ntp(void)
 void
 proto_reg_handoff_ntp(void)
 {
-	dissector_add("udp.port", UDP_PORT_NTP, dissect_ntp, proto_ntp);
-	dissector_add("tcp.port", TCP_PORT_NTP, dissect_ntp, proto_ntp);
+	dissector_handle_t ntp_handle;
+
+	ntp_handle = create_dissector_handle(dissect_ntp, proto_ntp);
+	dissector_add("udp.port", UDP_PORT_NTP, ntp_handle);
+	dissector_add("tcp.port", TCP_PORT_NTP, ntp_handle);
 }
