@@ -1,11 +1,10 @@
 /* packet-null.h
  *
- * $Id: packet-null.h,v 1.4 2000/11/29 05:16:15 gram Exp $
+ * $Id: packet-null.h,v 1.5 2001/11/20 21:59:13 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +24,6 @@
 #ifndef __PACKET_NULL_H__
 #define __PACKET_NULL_H__
 
-void capture_null(const u_char *, packet_counts *);
+void capture_null(const u_char *, int, packet_counts *);
 
 #endif
