@@ -2,7 +2,7 @@
  * Routines for ISO/OSI network and transport protocol packet disassembly
  * Main entrance point and common functions
  *
- * $Id: packet-osi.c,v 1.45 2001/06/05 09:06:19 guy Exp $
+ * $Id: packet-osi.c,v 1.52 2001/12/10 00:25:31 guy Exp $
  * Laurent Deniel <deniel@worldnet.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -23,7 +23,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +54,7 @@ calc_checksum( tvbuff_t *tvb, int offset, u_int len, u_int checksum) {
   const guint8 *p;
   guint32 c0, c1;
   u_int   seglen;
-  int     i;
+  u_int   i;
 
   if ( 0 == checksum )
     return( NO_CKSUM );
@@ -103,9 +102,17 @@ calc_checksum( tvbuff_t *tvb, int offset, u_int len, u_int checksum) {
 
 /* main entry point */
 
+/*
+ * These assume the NLPID is a secondary protocol identifier, not an
+ * initial protocol identifier.
+ *
+ * This is an issue only if, in any packet where an NLPID appears, it's
+ * an initial protocol identifier *AND* it can have the value 1, which
+ * means T.70 for an IPI and X.29 for an SPI.
+ */
 const value_string nlpid_vals[] = {
 	{ NLPID_NULL,            "NULL" },
-	{ NLPID_T_70,            "T.70" },
+	{ NLPID_SPI_X_29,        "X.29" },
 	{ NLPID_X_633,           "X.633" },
 	{ NLPID_Q_931,           "Q.931" },
 	{ NLPID_Q_2931,          "Q.2931" },
@@ -126,6 +133,7 @@ const value_string nlpid_vals[] = {
 };
 
 dissector_table_t osinl_subdissector_table;
+static dissector_handle_t data_handle;
 
 static void dissect_osi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) 
 {
@@ -144,25 +152,25 @@ static void dissect_osi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* ESIS (X.25) is not currently decoded */
 
     case NLPID_ISO9542X25_ESIS:
-      if (check_col(pinfo->fd, COL_PROTOCOL)) {
-	col_set_str(pinfo->fd, COL_PROTOCOL, "ESIS (X.25)");
+      if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "ESIS (X.25)");
       }
-      dissect_data(tvb, 0, pinfo, tree);
+      call_dissector(data_handle,tvb, pinfo, tree);
       break;
     case NLPID_ISO10747_IDRP:
-      if (check_col(pinfo->fd, COL_PROTOCOL)) {
-        col_set_str(pinfo->fd, COL_PROTOCOL, "IDRP");
+      if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "IDRP");
       }
-      dissect_data(tvb, 0, pinfo, tree);
+      call_dissector(data_handle,tvb, pinfo, tree);
       break;
     default:
-      if (check_col(pinfo->fd, COL_PROTOCOL)) {
-	col_set_str(pinfo->fd, COL_PROTOCOL, "ISO");
+      if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "ISO");
       }
-      if (check_col(pinfo->fd, COL_INFO)) {
-	col_add_fstr(pinfo->fd, COL_INFO, "Unknown ISO protocol (%02x)", nlpid);
+      if (check_col(pinfo->cinfo, COL_INFO)) {
+	col_add_fstr(pinfo->cinfo, COL_INFO, "Unknown ISO protocol (%02x)", nlpid);
       }
-      dissect_data(tvb, 0, pinfo, tree);
+      call_dissector(data_handle,tvb, pinfo, tree);
       break;
   }
 } /* dissect_osi */
@@ -173,13 +181,19 @@ proto_register_osi(void)
 	/* There's no "OSI" protocol *per se*, but we do register a
 	   dissector table so various protocols running at the
 	   network layer can register themselves. */
-	osinl_subdissector_table = register_dissector_table("osinl");
+	osinl_subdissector_table = register_dissector_table("osinl",
+	    "OSI NLPID", FT_UINT8, BASE_HEX);
 }
 
 void
 proto_reg_handoff_osi(void)
 {
-	dissector_add("llc.dsap", SAP_OSINL, dissect_osi, -1);
-        dissector_add("ppp.protocol", PPP_OSI, dissect_osi, -1);
-	dissector_add("null.type", BSD_AF_ISO, dissect_osi, -1);
+	dissector_handle_t osi_handle;
+
+	osi_handle = create_dissector_handle(dissect_osi, -1);
+	dissector_add("llc.dsap", SAP_OSINL, osi_handle);
+        dissector_add("ppp.protocol", PPP_OSI, osi_handle);
+	dissector_add("null.type", BSD_AF_ISO, osi_handle);
+	dissector_add("gre.proto", SAP_OSINL, osi_handle);
+	data_handle = find_dissector("data");
 }
