@@ -2,7 +2,7 @@
  * Routines for PIM disassembly
  * (c) Copyright Jun-ichiro itojun Hagino <itojun@itojun.org>
  *
- * $Id: packet-pim.c,v 1.31 2001/07/02 09:42:40 guy Exp $
+ * $Id: packet-pim.c,v 1.36 2001/12/10 00:25:32 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,6 +45,7 @@
 
 #include "packet.h"
 #include "ipproto.h"
+#include "afn.h"
 #include "packet-ipv6.h"
 #include "in_cksum.h"
 
@@ -66,26 +67,6 @@ static gint ett_pim = -1;
 static dissector_handle_t ip_handle;
 static dissector_handle_t ipv6_handle;
 
-/*
- * Address family values.
- */
-#define PIM_AF_RESERVED		0
-#define PIM_AF_IP		1	/* IPv4 */
-#define PIM_AF_IPV6		2	/* IPv6 */
-#define PIM_AF_NSAP		3	/* NSAP */
-#define PIM_AF_HDLC		4	/* HDLC (8-bit multidrop) */
-#define PIM_AF_BBN_1822		5	/* BBN 1822 */
-#define PIM_AF_802		6	/* 802 (D/I/X Ethernet, 802.x, FDDI) */
-#define PIM_AF_E_163		7	/* E.163 */
-#define PIM_AF_E_164		8	/* E.164 (SMDS, Frame Relay, ATM) */
-#define PIM_AF_F_69		9	/* F.69 (Telex) */
-#define PIM_AF_X_121		10	/* X.121 (X.25, Frame Relay) */
-#define PIM_AF_IPX		11	/* IPX */
-#define PIM_AF_ATALK		12	/* Appletalk */
-#define PIM_AF_DECNET_IV	13	/* DECnet Phase IV */
-#define PIM_AF_VINES		14	/* Banyan Vines */
-#define PIM_AF_E_164_NSAP	15	/* E.164 with NSAP format subaddress */
-
 /*
  * For PIM v1, see the PDF slides at
  *
@@ -148,10 +129,10 @@ dissect_pimv1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	return offset+tvb_length_remaining(tvb, offset);
     }
 
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-        col_set_str(pinfo->fd, COL_PROTOCOL, "PIMv1");
-    if (check_col(pinfo->fd, COL_INFO))
-	col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "PIMv1");
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_clear(pinfo->cinfo, COL_INFO);
 
     if (tree) {
 	ti = proto_tree_add_item(tree, proto_pim, tvb, offset,
@@ -165,8 +146,8 @@ dissect_pimv1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     offset += 1;
 
     pim_type = tvb_get_guint8(tvb, offset);
-    if (check_col(pinfo->fd, COL_INFO))
-	col_add_str(pinfo->fd, COL_INFO,
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_add_str(pinfo->cinfo, COL_INFO,
 	    val_to_str(pim_type, type1vals, "Unknown (%u)"));
 
     if (tree) {
@@ -204,7 +185,7 @@ dissect_pimv1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	 * this register will overwrite the PIM info in the columns.
 	 */
 	pim_length = 8;
-	col_set_writable(pinfo->fd, FALSE);
+	col_set_writable(pinfo->cinfo, FALSE);
     } else {
 	/*
 	 * Other message - checksum the entire packet.
@@ -522,7 +503,7 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
     int len = 0;
 
     af = tvb_get_guint8(tvb, offset);
-    if (af != PIM_AF_IP && af != PIM_AF_IPV6) {
+    if (af != AFNUM_INET && af != AFNUM_INET6) {
 	/*
 	 * We don't handle the other formats, and addresses don't include
 	 * a length field, so we can't even show them as raw bytes.
@@ -544,13 +525,13 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
     switch (at) {
     case pimv2_unicast:
 	switch (af) {
-	case PIM_AF_IP:
+	case AFNUM_INET:
 	    len = 4;
 	    (void)snprintf(buf, sizeof(buf), "%s",
 	        ip_to_str(tvb_get_ptr(tvb, offset + 2, len)));
 	    break;
 
-	case PIM_AF_IPV6:
+	case AFNUM_INET6:
 	    len = 16;
 	    (void)snprintf(buf, sizeof(buf), "%s",
 		ip6_to_str((struct e_in6_addr *)tvb_get_ptr(tvb, offset + 2, len)));
@@ -563,13 +544,13 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
     case pimv2_group:
 	mask_len = tvb_get_guint8(tvb, offset + 3);
 	switch (af) {
-	case PIM_AF_IP:
+	case AFNUM_INET:
 	    len = 4;
 	    (void)snprintf(buf, sizeof(buf), "%s/%u",
 		ip_to_str(tvb_get_ptr(tvb, offset + 4, len)), mask_len);
 	    break;
 
-	case PIM_AF_IPV6:
+	case AFNUM_INET6:
 	    len = 16;
 	    (void)snprintf(buf, sizeof(buf), "%s/%u",
 		ip6_to_str((struct e_in6_addr *)tvb_get_ptr(tvb, offset + 4, len)), mask_len);
@@ -583,13 +564,13 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
 	flags = tvb_get_guint8(tvb, offset + 2);
 	mask_len = tvb_get_guint8(tvb, offset + 3);
 	switch (af) {
-	case PIM_AF_IP:
+	case AFNUM_INET:
 	    len = 4;
 	    (void)snprintf(buf, sizeof(buf), "%s/%u",
 		ip_to_str(tvb_get_ptr(tvb, offset + 4, len)), mask_len);
 	    break;
 
-	case PIM_AF_IPV6:
+	case AFNUM_INET6:
 	    len = 16;
 	    (void)snprintf(buf, sizeof(buf), "%s/%u",
 		ip6_to_str((struct e_in6_addr *)tvb_get_ptr(tvb, offset + 4, len)), mask_len);
@@ -625,23 +606,29 @@ static const value_string type2vals[] = {
     { 0, NULL },
 };
 
+/*
+ * For PIM v2, see RFC 2362, and draft-ietf-pim-sm-v2-new-03 (when PIM
+ * is run over IPv6, the rules for computing the PIM checksum from the
+ * draft in question, not from RFC 2362, should be used).
+ */
 static void 
 dissect_pim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
     int offset = 0;
     guint8 pim_typever;
     guint length, pim_length;
     guint16 pim_cksum, computed_cksum;
-    vec_t cksum_vec[1];
+    vec_t cksum_vec[4];
+    guint32 phdr[2];
     char *typestr;
     proto_tree *pim_tree = NULL;
     proto_item *ti; 
     proto_tree *pimopt_tree = NULL;
     proto_item *tiopt; 
 
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-        col_set_str(pinfo->fd, COL_PROTOCOL, "PIM");
-    if (check_col(pinfo->fd, COL_INFO))
-	col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "PIM");
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_clear(pinfo->cinfo, COL_INFO);
 
     pim_typever = tvb_get_guint8(tvb, 0);
 
@@ -655,12 +642,12 @@ dissect_pim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	break;
     }
 
-    if (check_col(pinfo->fd, COL_PROTOCOL)) {
-        col_add_fstr(pinfo->fd, COL_PROTOCOL, "PIMv%d",
+    if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+        col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "PIMv%d",
 	    PIM_VER(pim_typever));
     }
-    if (check_col(pinfo->fd, COL_INFO))
-	col_add_str(pinfo->fd, COL_INFO, typestr); 
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_add_str(pinfo->cinfo, COL_INFO, typestr); 
 
     if (tree) {
 	ti = proto_tree_add_item(tree, proto_pim, tvb, offset,
@@ -688,7 +675,7 @@ dissect_pim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 		 * this register will overwrite the PIM info in the columns.
 		 */
 		pim_length = 8;
-		col_set_writable(pinfo->fd, FALSE);
+		col_set_writable(pinfo->cinfo, FALSE);
 	    } else {
 		/*
 		 * Other message - checksum the entire packet.
@@ -707,9 +694,34 @@ dissect_pim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	     * The packet isn't part of a fragmented datagram and isn't
 	     * truncated, so we can checksum it.
 	     */
-	    cksum_vec[0].ptr = tvb_get_ptr(tvb, 0, pim_length);
-	    cksum_vec[0].len = pim_length;
-	    computed_cksum = in_cksum(&cksum_vec[0], 1);
+
+	    switch (pinfo->src.type) {
+	    case AT_IPv4:
+		cksum_vec[0].ptr = tvb_get_ptr(tvb, 0, pim_length);
+		cksum_vec[0].len = pim_length;
+		computed_cksum = in_cksum(&cksum_vec[0], 1);
+		break;
+	    case AT_IPv6:
+		/* Set up the fields of the pseudo-header. */
+		cksum_vec[0].ptr = pinfo->src.data;
+		cksum_vec[0].len = pinfo->src.len;
+		cksum_vec[1].ptr = pinfo->dst.data;
+		cksum_vec[1].len = pinfo->dst.len;
+		cksum_vec[2].ptr = (const guint8 *)&phdr;
+		phdr[0] = htonl(pim_length);
+		phdr[1] = htonl(IP_PROTO_PIM);
+		cksum_vec[2].len = 8;
+		cksum_vec[3].ptr = tvb_get_ptr(tvb, 0, pim_length);
+		cksum_vec[3].len = pim_length;
+		computed_cksum = in_cksum(&cksum_vec[0], 4);
+		break;
+	    default:
+	    	/* PIM is available for IPv4 and IPv6 right now */
+	    	computed_cksum = 0;	/* squelch GCC complaints */
+		g_assert_not_reached();
+		break;
+	    }
+
 	    if (computed_cksum == 0) {
 		proto_tree_add_uint_format(pim_tree, hf_pim_cksum, tvb,
 			offset + 2, 2, pim_cksum,
@@ -1118,7 +1130,10 @@ proto_register_pim(void)
 void
 proto_reg_handoff_pim(void)
 {
-    dissector_add("ip.proto", IP_PROTO_PIM, dissect_pim, proto_pim);
+    dissector_handle_t pim_handle;
+
+    pim_handle = create_dissector_handle(dissect_pim, proto_pim);
+    dissector_add("ip.proto", IP_PROTO_PIM, pim_handle);
 
     /*
      * Get handles for the IPv4 and IPv6 dissectors.
