@@ -1,11 +1,11 @@
 /* packet-pim.h
+ * Declarations of routines for IGMP/PIMv1 packet disassembly
  *
- * $Id: packet-pim.h,v 1.1 2000/02/15 21:02:50 gram Exp $
+ * $Id: packet-pim.h,v 1.3 2001/07/02 09:23:02 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,5 +22,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_PIM_H__
+#define __PACKET_PIM_H__
 
-void dissect_pim(const u_char *, int, frame_data *, proto_tree *);
+int dissect_pimv1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int offset);
+
+#endif
