@@ -2,10 +2,10 @@
  * Routines for pop packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-pop.c,v 1.25 2001/06/18 02:17:50 guy Exp $
+ * $Id: packet-pop.c,v 1.28 2001/12/10 00:25:32 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
@@ -50,6 +50,8 @@ static int hf_pop_request = -1;
 
 static gint ett_pop = -1;
 
+static dissector_handle_t data_handle;
+
 #define TCP_PORT_POP			110
 
 static gboolean response_is_continuation(const u_char *data);
@@ -68,8 +70,8 @@ dissect_pop(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int		tokenlen;
 	const u_char	*next_token;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "POP");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "POP");
 
 	/*
 	 * Find the end of the first line.
@@ -89,7 +91,7 @@ dissect_pop(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		is_continuation = response_is_continuation(line);
 	}
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		/*
 		 * Put the first line from the buffer into the summary
 		 * if it's a POP request or reply (but leave out the
@@ -97,9 +99,9 @@ dissect_pop(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		 * Otherwise, just call it a continuation.
 		 */
 		if (is_continuation)
-			col_set_str(pinfo->fd, COL_INFO, "Continuation");
+			col_set_str(pinfo->cinfo, COL_INFO, "Continuation");
 		else
-			col_add_fstr(pinfo->fd, COL_INFO, "%s: %s",
+			col_add_fstr(pinfo->cinfo, COL_INFO, "%s: %s",
 			    is_request ? "Request" : "Response",
 			    format_text(line, linelen));
 	}
@@ -113,7 +115,7 @@ dissect_pop(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			/*
 			 * Put the whole packet into the tree as data.
 			 */
-			dissect_data(tvb, 0, pinfo, pop_tree);
+			call_dissector(data_handle,tvb, pinfo, pop_tree);
 			return;
 		}
 
@@ -222,5 +224,9 @@ proto_register_pop(void)
 void
 proto_reg_handoff_pop(void)
 {
-  dissector_add("tcp.port", TCP_PORT_POP, dissect_pop, proto_pop);
+  dissector_handle_t pop_handle;
+
+  pop_handle = create_dissector_handle(dissect_pop, proto_pop);
+  dissector_add("tcp.port", TCP_PORT_POP, pop_handle);
+  data_handle = find_dissector("data");
 }
