@@ -1,11 +1,10 @@
 /* packet-ppp.h
  *
- * $Id: packet-ppp.h,v 1.7 2001/03/30 06:15:47 guy Exp $
+ * $Id: packet-ppp.h,v 1.9 2001/12/19 21:14:49 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +24,9 @@
 #ifndef __PACKET_PPP_H__
 #define __PACKET_PPP_H__
 
-void capture_ppp_hdlc(const u_char *, int, packet_counts *);
+/* PPP options */
+extern gboolean ppp_vj_decomp;/* FALSE = No VJ header decompression,
+                                 TRUE  = Decompress VJ */
+void capture_ppp_hdlc(const u_char *, int, int, packet_counts *);
 
 #endif
