@@ -2,12 +2,11 @@
  * Routines for the Point-to-Point Tunnelling Protocol (PPTP) (RFC 2637)
  * Brad Robel-Forrest <brad.robel-forrest@watchguard.com>
  *
- * $Id: packet-pptp.c,v 1.19 2001/06/18 02:17:50 guy Exp $
+ * $Id: packet-pptp.c,v 1.22 2001/12/10 00:25:32 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,6 +45,8 @@ static int hf_pptp_message_type = -1;
 
 static gint ett_pptp = -1;
 
+static dissector_handle_t data_handle;
+
 #define TCP_PORT_PPTP			1723
 
 #define MAGIC_COOKIE		0x1A2B3C4D
@@ -242,16 +243,16 @@ dissect_pptp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint16		len;
   guint16		cntrl_type;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "PPTP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "PPTP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
   
   len	     = tvb_get_ntohs(tvb, offset);
   cntrl_type = tvb_get_ntohs(tvb, offset + 8);
 
-  if (check_col(pinfo->fd, COL_INFO))
-    col_add_fstr(pinfo->fd, COL_INFO, "%s", cntrltype2str(cntrl_type));
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO, "%s", cntrltype2str(cntrl_type));
 
   if (tree) {
     guint32		cookie;
@@ -289,7 +290,7 @@ dissect_pptp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     if (cntrl_type < NUM_CNTRL_TYPES)
       ( *(strfuncs[cntrl_type].func))(tvb, offset, pinfo, pptp_tree);
     else
-      dissect_data(tvb, offset, pinfo, pptp_tree);
+      call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, pptp_tree);
   }
 }
 
@@ -297,7 +298,7 @@ static void
 dissect_unknown(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		proto_tree *tree)
 {
-  dissect_data(tvb, offset, pinfo, tree);
+  call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
 }
 
 static void
@@ -828,5 +829,9 @@ proto_register_pptp(void)
 void
 proto_reg_handoff_pptp(void)
 {
-  dissector_add("tcp.port", TCP_PORT_PPTP, dissect_pptp, proto_pptp);
+  dissector_handle_t pptp_handle;
+
+  pptp_handle = create_dissector_handle(dissect_pptp, proto_pptp);
+  dissector_add("tcp.port", TCP_PORT_PPTP, pptp_handle);
+  data_handle = find_dissector("data");
 }
