@@ -2,7 +2,7 @@
  * Routines for Q.931 frame disassembly
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-q931.c,v 1.31 2001/07/03 04:56:45 guy Exp $
+ * $Id: packet-q931.c,v 1.33 2001/12/10 00:25:32 guy Exp $
  *
  * Modified by Andreas Sikkema for possible use with H.323
  *
@@ -2099,7 +2099,6 @@ q931_dissector(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     gboolean started_heuristic)
 {
 	int		offset = 0;
-	guint		reported_length;
 	proto_tree	*q931_tree = NULL;
 	proto_item	*ti;
 	proto_tree	*ie_tree;
@@ -2223,8 +2222,8 @@ q931_dissector(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		return FALSE;
 	}
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "Q.931");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "Q.931");
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_q931, tvb, offset,
@@ -2246,8 +2245,8 @@ q931_dissector(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		offset += call_ref_len;
 	}
 	message_type = tvb_get_guint8(tvb, offset);
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_str(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_str(pinfo->cinfo, COL_INFO,
 		    val_to_str(message_type, q931_message_type_vals,
 		      "Unknown message type (0x%02X)"));
 	}
@@ -2260,8 +2259,7 @@ q931_dissector(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	 */
 	codeset = 0;	/* start out in codeset 0 */
 	non_locking_shift = TRUE;
-	reported_length = tvb_reported_length(tvb);
-	while (offset < reported_length) {
+	while (tvb_reported_length_remaining(tvb, offset) > 0) {
 		info_element = tvb_get_guint8(tvb, offset);
 
 		/*
