@@ -4,7 +4,7 @@
  * Uwe Girlich <uwe@planetquake.com>
  *	http://www.idsoftware.com/q1source/q1source.zip
  *
- * $Id: packet-quake.c,v 1.17 2001/07/03 04:56:45 guy Exp $
+ * $Id: packet-quake.c,v 1.23 2001/12/10 00:25:32 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -80,6 +80,8 @@ static gint ett_quake_control = -1;
 static gint ett_quake_control_colors = -1;
 static gint ett_quake_flags = -1;
 
+static dissector_handle_t quake_handle;
+static dissector_handle_t data_handle;
 
 /* I took these names directly out of the Q1 source. */
 #define NETFLAG_LENGTH_MASK 0x0000ffff
@@ -241,9 +243,9 @@ dissect_quake_CCREP_ACCEPT
 
 	port = tvb_get_letohl(tvb, 0);
 	c = conversation_new( &pinfo->src, &pinfo->dst, PT_UDP, port,
-	    pinfo->destport, NULL, 0);
+	    pinfo->destport, 0);
 	if (c) {
-		conversation_set_dissector(c, dissect_quake);
+		conversation_set_dissector(c, quake_handle);
 	}
 	if (tree) {
 		proto_tree_add_uint(tree, hf_quake_CCREP_ACCEPT_port,
@@ -443,8 +445,8 @@ dissect_quake_control(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	command = tvb_get_guint8(tvb, 0);
 	direction = (command & 0x80) ? CCREP : CCREQ;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO, "%s %s",
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO, "%s %s",
 			val_to_str(command,names_control_command, "%u"),
 			val_to_str(direction,names_control_direction,"%u"));
 	}
@@ -501,7 +503,7 @@ dissect_quake_control(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			(next_tvb, pinfo, control_tree);
 		break;
 		default:
-			dissect_data(next_tvb, 0, pinfo, control_tree);
+			call_dissector(data_handle,next_tvb, pinfo, control_tree);
 		break;
 	}
 }
@@ -518,19 +520,10 @@ dissect_quake(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint		rest_length;
 	tvbuff_t	*next_tvb;
 
-	/*
-	 * XXX - this is a conversation dissector, and the code to
-	 * call a conversation dissector doesn't check for disabled
-	 * protocols or set "pinfo->current_proto".
-	 */
-	CHECK_DISPLAY_AS_DATA(proto_quake, tvb, pinfo, tree);
-
-	pinfo->current_proto = "QUAKE";
-
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "QUAKE");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "QUAKE");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	length = tvb_get_ntohl(tvb, 0);
 	flags = length & (~NETFLAG_LENGTH_MASK);
@@ -585,8 +578,8 @@ dissect_quake(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
 
 	sequence = tvb_get_ntohl(tvb, 4);
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO, "seq 0x%x", sequence);
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO, "seq 0x%x", sequence);
 	}
 	if (quake_tree) {
 		proto_tree_add_uint(quake_tree, hf_quake_header_sequence,
@@ -595,7 +588,7 @@ dissect_quake(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	rest_length = tvb_reported_length(tvb) - 8;
 	next_tvb = tvb_new_subset(tvb, 8, rest_length , rest_length);
-	dissect_data(next_tvb, 0, pinfo, quake_tree);
+	call_dissector(data_handle,next_tvb, pinfo, quake_tree);
 }
 
 
@@ -606,7 +599,7 @@ proto_reg_handoff_quake(void)
 	static int ServerPort=0;
  
 	if (Initialized) {
-		dissector_delete("udp.port", ServerPort, dissect_quake);
+		dissector_delete("udp.port", ServerPort, quake_handle);
 	} else {
 		Initialized=TRUE;
 	}
@@ -614,8 +607,8 @@ proto_reg_handoff_quake(void)
 	/* set port for future deletes */
 	ServerPort=gbl_quakeServerPort;
  
-	dissector_add("udp.port", gbl_quakeServerPort,
-                        dissect_quake, proto_quake);
+	dissector_add("udp.port", gbl_quakeServerPort, quake_handle);
+	data_handle = find_dissector("data");
 }
 
 
@@ -741,6 +734,8 @@ proto_register_quake(void)
 	proto_register_field_array(proto_quake, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
+	quake_handle = create_dissector_handle(dissect_quake, proto_quake);
+
 	/* Register a configuration option for port */
 	quake_module = prefs_register_protocol(proto_quake,
 		proto_reg_handoff_quake);
@@ -750,4 +745,3 @@ proto_register_quake(void)
 					10, &gbl_quakeServerPort);
 }
 
-
