@@ -2,7 +2,7 @@
  * Routines for RADIUS packet disassembly
  * Copyright 1999 Johan Feyaerts
  *
- * $Id: packet-radius.c,v 1.32 2001/06/18 09:31:15 guy Exp $
+ * $Id: packet-radius.c,v 1.40 2001/12/10 00:25:33 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,6 +40,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <glib.h>
+#include <time.h>
 #include "packet.h"
 #include "resolv.h"
 
@@ -133,6 +134,9 @@ typedef struct _value_value_pair {
 #define RD_TP_ACCT_TERMINATE_CAUSE 49
 #define RD_TP_ACCT_MULTI_SESSION_ID 50
 #define RD_TP_ACCT_LINK_COUNT 51
+#define RD_TP_ACCT_INPUT_GIGAWORDS 52
+#define RD_TP_ACCT_OUTPUT_GIGAWORDS 53
+#define RD_TP_EVENT_TIMESTAMP 55
 #define RD_TP_CHAP_CHALLENGE 60
 #define RD_TP_NAS_PORT_TYPE 61
 #define RD_TP_PORT_LIMIT 62
@@ -141,8 +145,37 @@ typedef struct _value_value_pair {
 #define RD_TP_TUNNEL_MEDIUM_TYPE 65
 #define RD_TP_TUNNEL_CLIENT_ENDPOINT 66
 #define RD_TP_TUNNEL_SERVER_ENDPOINT 67
+#define RD_TP_TUNNEL_CONNECTION 68
 #define RD_TP_TUNNEL_PASSWORD 69
+#define RD_TP_CONNECT_INFO 77
+#define RD_TP_MESSAGE_AUTHENTICATOR 80
+#define RD_TP_TUNNEL_PRIVATE_GROUP_ID 81
 #define RD_TP_TUNNEL_ASSIGNMENT_ID 82
+#define RD_TP_TUNNEL_TUNNEL_PREFERENCE 83
+#define RD_TP_TUNNEL_PACKETS_LOST 86
+#define RD_TP_NAS_PORT_ID 87
+#define RD_TP_TUNNEL_CLIENT_AUTH_ID 90
+#define RD_TP_TUNNEL_SERVER_AUTH_ID 91
+#define RD_TP_ASCEND_MODEM_PORTNO 120
+#define RD_TP_ASCEND_MODEM_SLOTNO 121
+#define RD_TP_ASCEND_MULTILINK_ID 187
+#define RD_TP_ASCEND_NUM_IN_MULTILINK 188
+#define RD_TP_ASCEND_FIRST_DEST 189
+#define RD_TP_ASCEND_PRE_INPUT_OCTETS 190
+#define RD_TP_ASCEND_PRE_OUTPUT_OCTETS 191
+#define RD_TP_ASCEND_PRE_INPUT_PACKETS 192
+#define RD_TP_ASCEND_PRE_OUTPUT_PACKETS 193
+#define RD_TP_ASCEND_MAXIMUM_TIME 194
+#define RD_TP_ASCEND_DISCONNECT_CAUSE 195
+#define RD_TP_ASCEND_CONNECT_PROGRESS 196
+#define RD_TP_ASCEND_DATA_RATE 197
+#define RD_TP_ASCEND_PRESESSION_TIME 198
+#define RD_TP_ASCEND_ASSIGN_IP_POOL 218
+#define RD_TP_ASCEND_XMIT_RATE 255
+
+
+
+
 
 #define AUTHENTICATOR_LENGTH 16
 #define RD_HDR_LENGTH 4
@@ -168,6 +201,8 @@ typedef struct _value_value_pair {
 #define RADIUS_TUNNEL_MEDIUM_TYPE 18
 #define RADIUS_STRING_TAGGED 19
 #define RADIUS_VENDOR_SPECIFIC 20
+#define RADIUS_TIMESTAMP 21
+#define RADIUS_INTEGER4_TAGGED 22
 
 static value_string radius_vals[] = {
  {RADIUS_ACCESS_REQUEST, "Access Request"},
@@ -202,15 +237,48 @@ static value_string radius_service_type_vals[]=
  *
  * for a list.
  */
+#define VENDOR_ACC 5
+#define VENDOR_CISCO 9
+#define VENDOR_SHIVA 166
+#define VENDOR_LIVINGSTON 307
+#define VENDOR_3COM 429
+#define VENDOR_ASCEND 529
+#define VENDOR_BAY 1584
+#define VENDOR_JUNIPER 2636
+#define VENDOR_COSINE 3085
+#define VENDOR_UNISPHERE 4874
+
 static value_string radius_vendor_specific_vendors[]=
-{{5, "ACC"},
-{9,"Cisco"},
-{166,"Shiva"},
-{307,"Livingston"},
-{429,"3Com"},
-{529,"Ascend"},
-{1584,"Bay Networks"},
-{2636,"Juniper Networks"},
+{{VENDOR_ACC,"ACC"},
+{VENDOR_CISCO,"Cisco"},
+{VENDOR_SHIVA,"Shiva"},
+{VENDOR_LIVINGSTON,"Livingston"},
+{VENDOR_3COM,"3Com"},
+{VENDOR_ASCEND,"Ascend"},
+{VENDOR_BAY,"Bay Networks"},
+{VENDOR_JUNIPER,"Juniper Networks"},
+{VENDOR_COSINE,"CoSine Communications"},
+{VENDOR_UNISPHERE,"Unisphere Networks"},
+{0,NULL}};
+
+#define VENDOR_COSINE_VSA_CONNECION_PROFILE_NAME 1
+#define VENDOR_COSINE_VSA_ENTERPRISE_ID 2
+#define VENDOR_COSINE_VSA_ADDRESS_POOL_NAME 3
+#define VENDOR_COSINE_VSA_DS_BYTE 4
+#define VENDOR_COSINE_VSA_VPI_VCI 5
+#define VENDOR_COSINE_VSA_DLCI 6
+#define VENDOR_COSINE_VSA_LNS_IP_ADDRESS 7
+#define VENDOR_COSINE_VSA_CLI_USER_PERMISSION_ID 8
+
+static value_string radius_vendor_cosine_types[]=
+{{VENDOR_COSINE_VSA_CONNECION_PROFILE_NAME,"Connection Profile Name"},
+{VENDOR_COSINE_VSA_ENTERPRISE_ID,"Enterprise ID"},
+{VENDOR_COSINE_VSA_ADDRESS_POOL_NAME,"Address Pool Name"},
+{VENDOR_COSINE_VSA_DS_BYTE,"DS Byte"},
+{VENDOR_COSINE_VSA_VPI_VCI,"VPI/VCI"},
+{VENDOR_COSINE_VSA_DLCI,"DLCI"},
+{VENDOR_COSINE_VSA_LNS_IP_ADDRESS,"LNS IP Address"},
+{VENDOR_COSINE_VSA_CLI_USER_PERMISSION_ID,"CLI User Permission ID"},
 {0,NULL}};
 
 static value_string radius_framed_protocol_vals[]=
@@ -220,6 +288,8 @@ static value_string radius_framed_protocol_vals[]=
 {4, "Gandalf proprietary Singlelink/Multilink Protocol"},
 {5, "Xylogics proprietary IPX/SLIP"},
 {6, "X.75 Synchronous"},
+{256, "Ascend MPP"},
+{262, "Ascend MP"},
 {0,NULL}};
 
 static value_string radius_framed_routing_vals[]=
@@ -258,11 +328,19 @@ static value_string radius_accounting_status_type_vals[]=
 {3, "Interim-Update"},
 {7,"Accounting-On"},
 {8,"Accounting-Off"},
+{9, "Tunnel-Start"}, /* Tunnel accounting */
+{10, "Tunnel-Stop"}, /* Tunnel accounting */
+{11, "Tunnel-Reject"}, /* Tunnel accounting */
+{12, "Tunnel-Link-Start"}, /* Tunnel accounting */
+{13, "Tunnel-Link-Stop"}, /* Tunnel accounting */
+{14, "Tunnel-Link-Reject"}, /* Tunnel accounting */
 {0,NULL}};
 
 static value_string radius_accounting_authentication_vals[]=
 {{1, "Radius"},
 {2, "Local"},
+{3,"Remote"},
+/* RFC 2866 says 3 is Remote. Is 7 a mistake? */
 {7,"Remote"},
 {0,NULL}};
 
@@ -336,6 +414,11 @@ static value_string radius_nas_port_type_vals[]=
 {12, "ADSL-CAP"},
 {13, "ADSL-DMT"},
 {14,"IDSL - ISDN"},
+{15,"Ethernet"},
+{16,"xDSL"},
+{17,"Cable"},
+{18,"Wireless Other"},
+{19,"Wireless IEEE 802.11"},
 {0,NULL}};
 
 static value_value_pair radius_printinfo[] = {
@@ -390,6 +473,9 @@ static value_value_pair radius_printinfo[] = {
 { RD_TP_ACCT_TERMINATE_CAUSE, RADIUS_ACCT_TERMINATE_CAUSE},
 { RD_TP_ACCT_MULTI_SESSION_ID, RADIUS_STRING},
 { RD_TP_ACCT_LINK_COUNT, RADIUS_INTEGER4},
+{ RD_TP_ACCT_INPUT_GIGAWORDS, RADIUS_INTEGER4},
+{ RD_TP_ACCT_OUTPUT_GIGAWORDS, RADIUS_INTEGER4},
+{ RD_TP_EVENT_TIMESTAMP, RADIUS_TIMESTAMP},
 { RD_TP_CHAP_CHALLENGE, RADIUS_BINSTRING},
 { RD_TP_NAS_PORT_TYPE, RADIUS_NAS_PORT_TYPE},
 { RD_TP_PORT_LIMIT, RADIUS_INTEGER4},
@@ -398,8 +484,33 @@ static value_value_pair radius_printinfo[] = {
 { RD_TP_TUNNEL_MEDIUM_TYPE, RADIUS_TUNNEL_MEDIUM_TYPE},
 { RD_TP_TUNNEL_CLIENT_ENDPOINT, RADIUS_STRING_TAGGED},
 { RD_TP_TUNNEL_SERVER_ENDPOINT, RADIUS_STRING_TAGGED},
+{ RD_TP_TUNNEL_CONNECTION, RADIUS_BINSTRING},
 { RD_TP_TUNNEL_PASSWORD, RADIUS_STRING_TAGGED},
+{ RD_TP_CONNECT_INFO, RADIUS_STRING_TAGGED},
+{ RD_TP_MESSAGE_AUTHENTICATOR, RADIUS_BINSTRING},
+{ RD_TP_TUNNEL_PRIVATE_GROUP_ID, RADIUS_STRING_TAGGED},
 { RD_TP_TUNNEL_ASSIGNMENT_ID, RADIUS_STRING_TAGGED},
+{ RD_TP_TUNNEL_TUNNEL_PREFERENCE, RADIUS_INTEGER4_TAGGED},
+{ RD_TP_TUNNEL_PACKETS_LOST, RADIUS_INTEGER4},
+{ RD_TP_NAS_PORT_ID, RADIUS_STRING},
+{ RD_TP_TUNNEL_CLIENT_AUTH_ID, RADIUS_STRING_TAGGED},
+{ RD_TP_TUNNEL_SERVER_AUTH_ID, RADIUS_STRING_TAGGED},
+{ RD_TP_ASCEND_MODEM_PORTNO, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_MODEM_SLOTNO, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_MULTILINK_ID, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_NUM_IN_MULTILINK, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_FIRST_DEST, RADIUS_IP_ADDRESS},
+{ RD_TP_ASCEND_PRE_INPUT_OCTETS, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_PRE_OUTPUT_OCTETS, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_PRE_INPUT_PACKETS, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_PRE_OUTPUT_PACKETS, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_MAXIMUM_TIME, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_DISCONNECT_CAUSE, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_CONNECT_PROGRESS, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_DATA_RATE, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_PRESESSION_TIME, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_ASSIGN_IP_POOL, RADIUS_INTEGER4},
+{ RD_TP_ASCEND_XMIT_RATE, RADIUS_INTEGER4},
 {0,0},
 };
 
@@ -455,6 +566,9 @@ static value_string radius_attrib_type_vals[] = {
 { RD_TP_ACCT_TERMINATE_CAUSE, "Acct Terminate Cause"},
 { RD_TP_ACCT_MULTI_SESSION_ID, "Acct Multi Session Id"},
 { RD_TP_ACCT_LINK_COUNT, "Acct Link Count"},
+{ RD_TP_ACCT_INPUT_GIGAWORDS, "Acct Input Gigawords"},
+{ RD_TP_ACCT_OUTPUT_GIGAWORDS, "Acct Output Gigawords"},
+{ RD_TP_EVENT_TIMESTAMP, "Event Timestamp"},
 { RD_TP_CHAP_CHALLENGE, "Chap Challenge"},
 { RD_TP_NAS_PORT_TYPE, "NAS Port Type"},
 { RD_TP_PORT_LIMIT, "Port Limit"},
@@ -463,8 +577,33 @@ static value_string radius_attrib_type_vals[] = {
 { RD_TP_TUNNEL_MEDIUM_TYPE, "Tunnel Medium Type"},
 { RD_TP_TUNNEL_CLIENT_ENDPOINT, "Tunnel Client Endpoint"},
 { RD_TP_TUNNEL_SERVER_ENDPOINT, "Tunnel Server Endpoint"},
+{ RD_TP_TUNNEL_CONNECTION, "Tunnel Connection"},
 { RD_TP_TUNNEL_PASSWORD, "Tunnel Password"},
+{ RD_TP_CONNECT_INFO, "Connect-Info"},
+{ RD_TP_MESSAGE_AUTHENTICATOR, "Message Authenticator"},
+{ RD_TP_TUNNEL_PRIVATE_GROUP_ID, "Tunnel Private Group ID"},
 { RD_TP_TUNNEL_ASSIGNMENT_ID, "Tunnel Assignment ID"},
+{ RD_TP_TUNNEL_TUNNEL_PREFERENCE, "Tunnel Preference"},
+{ RD_TP_TUNNEL_PACKETS_LOST, "Tunnel Packets Lost"},
+{ RD_TP_NAS_PORT_ID, "NAS Port ID"},
+{ RD_TP_TUNNEL_CLIENT_AUTH_ID, "Tunnel Client Auth ID"},
+{ RD_TP_TUNNEL_SERVER_AUTH_ID, "Tunnel Server Auth ID"},
+{ RD_TP_ASCEND_MODEM_PORTNO, "Ascend Modem Port No"},
+{ RD_TP_ASCEND_MODEM_SLOTNO, "Ascend Modem Slot No"},
+{ RD_TP_ASCEND_MULTILINK_ID, "Ascend Multilink ID"},
+{ RD_TP_ASCEND_NUM_IN_MULTILINK, "Ascend Num In Multilink"},
+{ RD_TP_ASCEND_FIRST_DEST, "Ascend First Dest"},
+{ RD_TP_ASCEND_PRE_INPUT_OCTETS, "Ascend Pre Input Octets"},
+{ RD_TP_ASCEND_PRE_OUTPUT_OCTETS, "Ascend Pre Output Octets"},
+{ RD_TP_ASCEND_PRE_INPUT_PACKETS, "Ascend Pre Input Packets"},
+{ RD_TP_ASCEND_PRE_OUTPUT_PACKETS, "Ascend Pre Output Packets"},
+{ RD_TP_ASCEND_MAXIMUM_TIME, "Ascend Maximum Time"},
+{ RD_TP_ASCEND_DISCONNECT_CAUSE, "Ascend Disconnect Cause"},
+{ RD_TP_ASCEND_CONNECT_PROGRESS, "Ascend Connect Progress"},
+{ RD_TP_ASCEND_DATA_RATE, "Ascend Data Rate"},
+{ RD_TP_ASCEND_PRESESSION_TIME, "Ascend PreSession Time"},
+{ RD_TP_ASCEND_ASSIGN_IP_POOL, "Ascend Assign IP Pool"},
+{ RD_TP_ASCEND_XMIT_RATE, "Ascend Xmit Rate"},
 {0,NULL},
 };
 
@@ -523,6 +662,9 @@ gchar *rd_value_to_str(e_avphdr *avph, tvbuff_t *tvb, int offset)
   guint32 intval;
   const guint8 *pd;
   guint8 tag;
+  guint8 vtype;
+  char *rtimestamp;
+  extern char *tzname[2];
 
 /* prints the values of the attribute value pairs into a text buffer */
   print_type=match_numval(avph->avp_type,radius_printinfo);
@@ -625,10 +767,67 @@ gchar *rd_value_to_str(e_avphdr *avph, tvbuff_t *tvb, int offset)
                 break;
 	case ( RADIUS_VENDOR_SPECIFIC ):
 		valstrarr=radius_vendor_specific_vendors;
-		sprintf(textbuffer,"Vendor:%s, Value:",
-				rd_match_strval(tvb_get_ntohl(tvb,offset+2),valstrarr));
+		sprintf(textbuffer,"Vendor:%s,",
+			rd_match_strval(tvb_get_ntohl(tvb,offset+2),valstrarr));
 		cont=&textbuffer[strlen(textbuffer)];
-		rdconvertbufftostr(cont,tvb,offset+6,avph->avp_length-6);
+		switch (tvb_get_ntohl(tvb,offset+2)) {
+		case ( VENDOR_COSINE ):
+			vtype = tvb_get_guint8(tvb,offset+6);
+			switch (vtype) {
+			case ( VENDOR_COSINE_VSA_CONNECION_PROFILE_NAME ):
+			case ( VENDOR_COSINE_VSA_ENTERPRISE_ID ):
+			case ( VENDOR_COSINE_VSA_ADDRESS_POOL_NAME ):
+			case ( VENDOR_COSINE_VSA_CLI_USER_PERMISSION_ID ):
+				sprintf(cont," Type:%s, Value:",
+					rd_match_strval(vtype, radius_vendor_cosine_types));
+				cont=&textbuffer[strlen(textbuffer)];
+				rdconvertbufftostr(cont,tvb,offset+8,avph->avp_length-8);
+				break;
+			case ( VENDOR_COSINE_VSA_VPI_VCI ):
+				sprintf(cont," Type:%s, Value:%u/%u",
+					rd_match_strval(vtype, radius_vendor_cosine_types),
+					tvb_get_ntohs(tvb,offset+8),
+					tvb_get_ntohs(tvb,offset+10));
+				break;
+			case ( VENDOR_COSINE_VSA_DS_BYTE ):
+			case ( VENDOR_COSINE_VSA_DLCI ):
+				sprintf(cont," Type:%s, Value:%u",
+					rd_match_strval(vtype, radius_vendor_cosine_types),
+					tvb_get_ntohl(tvb,offset+8));
+				break;
+			case ( VENDOR_COSINE_VSA_LNS_IP_ADDRESS ):
+				sprintf(cont," Type:%s, Value:",
+					rd_match_strval(vtype, radius_vendor_cosine_types));
+				cont=&textbuffer[strlen(textbuffer)];
+				ip_to_str_buf(tvb_get_ptr(tvb,offset+8,4),cont);
+				break;
+			default:
+				sprintf(cont," Unknown Value Type");
+				break;
+			}
+			break;
+		default: 
+			sprintf(cont, " Value:");
+			rdconvertbufftostr(cont,tvb,offset+6,avph->avp_length-6);
+			break;
+		}
+		break;
+        case( RADIUS_TIMESTAMP ):
+		intval=tvb_get_ntohl(tvb,offset+2);
+		rtimestamp=ctime((time_t*)&intval);
+		rtimestamp[strlen(rtimestamp)-1]=0;
+		sprintf(cont,"%d (%s %s)", tvb_get_ntohl(tvb,offset+2), rtimestamp, *tzname);
+		break;
+        case( RADIUS_INTEGER4_TAGGED ):
+		intval = tvb_get_ntohl(tvb,offset+2);
+		/* Tagged ? */
+		if (intval >> 24) {
+			sprintf(textbuffer, "Tag:%u, Value:%u",
+				intval >> 24,
+				intval & 0xffffff);
+			break;
+		}
+		sprintf(cont,"%u", intval);
 		break;
         case( RADIUS_UNKNOWN ):
         default:
@@ -659,17 +858,24 @@ void dissect_attribute_value_pairs(tvbuff_t *tvb, int offset, proto_tree *tree,
   {
 
      tvb_memcpy(tvb,(guint8 *)&avph,offset,sizeof(e_avphdr));
-     avplength=avplength-avph.avp_length;
      avptpstrval=match_strval(avph.avp_type, radius_attrib_type_vals);
      if (avptpstrval == NULL) avptpstrval="Unknown Type";
+     if (avph.avp_length < 2) {
+	/*
+	 * This AVP is bogus - the length includes the type and length
+	 * fields, so it must be >= 2.
+	 */
+	proto_tree_add_text(tree, tvb, offset, avph.avp_length,
+	  "t:%s(%u) l:%u (length not >= 2)",
+	   avptpstrval,avph.avp_type,avph.avp_length);
+	break;
+     }
      valstr=rd_value_to_str(&avph, tvb, offset);
      proto_tree_add_text(tree, tvb,offset,avph.avp_length,
         "t:%s(%u) l:%u, %s",
         avptpstrval,avph.avp_type,avph.avp_length,valstr);
      offset=offset+avph.avp_length;
-     if (avph.avp_length == 0) {
-	break;
-     }
+     avplength=avplength-avph.avp_length;
   }
 }
 
@@ -685,10 +891,10 @@ static void dissect_radius(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   gchar *codestrval;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-        col_set_str(pinfo->fd, COL_PROTOCOL, "RADIUS");
-  if (check_col(pinfo->fd, COL_INFO))
-        col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "RADIUS");
+  if (check_col(pinfo->cinfo, COL_INFO))
+        col_clear(pinfo->cinfo, COL_INFO);
 
   tvb_memcpy(tvb,(guint8 *)&rh,0,sizeof(e_radiushdr));
 
@@ -700,9 +906,9 @@ static void dissect_radius(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   {
 	codestrval="Unknown Packet";
   }
-  if (check_col(pinfo->fd, COL_INFO))
+  if (check_col(pinfo->cinfo, COL_INFO))
   {
-        col_add_fstr(pinfo->fd,COL_INFO,"%s(%d) (id=%d, l=%d)",
+        col_add_fstr(pinfo->cinfo,COL_INFO,"%s(%d) (id=%d, l=%d)",
 		codestrval, rhcode, rhident, rhlength);
   }
 
@@ -774,12 +980,11 @@ proto_register_radius(void)
 void
 proto_reg_handoff_radius(void)
 {
-	dissector_add("udp.port", UDP_PORT_RADIUS, dissect_radius,
-	    proto_radius);
-	dissector_add("udp.port", UDP_PORT_RADIUS_NEW, dissect_radius,
-	    proto_radius);
-	dissector_add("udp.port", UDP_PORT_RADACCT, dissect_radius,
-	    proto_radius);
-	dissector_add("udp.port", UDP_PORT_RADACCT_NEW, dissect_radius,
-	    proto_radius);
+	dissector_handle_t radius_handle;
+
+	radius_handle = create_dissector_handle(dissect_radius, proto_radius);
+	dissector_add("udp.port", UDP_PORT_RADIUS, radius_handle);
+	dissector_add("udp.port", UDP_PORT_RADIUS_NEW, radius_handle);
+	dissector_add("udp.port", UDP_PORT_RADACCT, radius_handle);
+	dissector_add("udp.port", UDP_PORT_RADACCT_NEW, radius_handle);
 }
