@@ -1,10 +1,10 @@
 /* packet-raw.c
  * Routines for raw packet disassembly
  *
- * $Id: packet-raw.c,v 1.26 2001/03/30 06:15:47 guy Exp $
+ * $Id: packet-raw.c,v 1.29 2001/12/10 00:25:33 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  *
  * This file created and by Mike Hall <mlh@io.com>
  * Copyright 1998
@@ -47,7 +47,7 @@ static dissector_handle_t ip_handle;
 static dissector_handle_t ppp_hdlc_handle;
 
 void
-capture_raw(const u_char *pd, packet_counts *ld)
+capture_raw(const u_char *pd, int len, packet_counts *ld)
 {
   /* So far, the only time we get raw connection types are with Linux and
    * Irix PPP connections.  We can't tell what type of data is coming down
@@ -57,25 +57,25 @@ capture_raw(const u_char *pd, packet_counts *ld)
   /* Currently, the Linux 2.1.xxx PPP driver passes back some of the header
    * sometimes.  This check should be removed when 2.2 is out.
    */
-  if (BYTES_ARE_IN_FRAME(0,2) && pd[0] == 0xff && pd[1] == 0x03) {
-    capture_ppp_hdlc(pd, 0, ld);
+  if (BYTES_ARE_IN_FRAME(0,len,2) && pd[0] == 0xff && pd[1] == 0x03) {
+    capture_ppp_hdlc(pd, 0, len, ld);
   }
   /* The Linux ISDN driver sends a fake MAC address before the PPP header
    * on its ippp interfaces... */
-  else if (BYTES_ARE_IN_FRAME(0,8) && pd[6] == 0xff && pd[7] == 0x03) {
-    capture_ppp_hdlc(pd, 6, ld);
+  else if (BYTES_ARE_IN_FRAME(0,len,8) && pd[6] == 0xff && pd[7] == 0x03) {
+    capture_ppp_hdlc(pd, 6, len, ld);
   }
   /* ...except when it just puts out one byte before the PPP header... */
-  else if (BYTES_ARE_IN_FRAME(0,3) && pd[1] == 0xff && pd[2] == 0x03) {
-    capture_ppp_hdlc(pd, 1, ld);
+  else if (BYTES_ARE_IN_FRAME(0,len,3) && pd[1] == 0xff && pd[2] == 0x03) {
+    capture_ppp_hdlc(pd, 1, len, ld);
   }
   /* ...and if the connection is currently down, it sends 10 bytes of zeroes
    * instead of a fake MAC address and PPP header. */
-  else if (BYTES_ARE_IN_FRAME(0,10) && memcmp(pd, zeroes, 10) == 0) {
-    capture_ip(pd, 10, ld);
+  else if (BYTES_ARE_IN_FRAME(0,len,10) && memcmp(pd, zeroes, 10) == 0) {
+    capture_ip(pd, 10, len, ld);
   }
   else {
-    capture_ip(pd, 0, ld);
+    capture_ip(pd, 0, len, ld);
   }
 }
 
@@ -88,14 +88,14 @@ dissect_raw(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   /* load the top pane info. This should be overwritten by
      the next protocol in the stack */
-  if(check_col(pinfo->fd, COL_RES_DL_SRC))
-    col_set_str(pinfo->fd, COL_RES_DL_SRC, "N/A" );
-  if(check_col(pinfo->fd, COL_RES_DL_DST))
-    col_set_str(pinfo->fd, COL_RES_DL_DST, "N/A" );
-  if(check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "N/A" );
-  if(check_col(pinfo->fd, COL_INFO))
-    col_set_str(pinfo->fd, COL_INFO, "Raw packet data" );
+  if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "N/A" );
+  if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "N/A" );
+  if(check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "N/A" );
+  if(check_col(pinfo->cinfo, COL_INFO))
+    col_set_str(pinfo->cinfo, COL_INFO, "Raw packet data" );
 
   /* populate a tree in the second pane with the status of the link
      layer (ie none) */
@@ -158,10 +158,13 @@ proto_register_raw(void)
 void
 proto_reg_handoff_raw(void)
 {
+  dissector_handle_t raw_handle;
+
   /*
    * Get handles for the IP and PPP-in-HDLC-like-framing dissectors.
    */
   ip_handle = find_dissector("ip");
   ppp_hdlc_handle = find_dissector("ppp_hdlc");
-  dissector_add("wtap_encap", WTAP_ENCAP_RAW_IP, dissect_raw, -1);
+  raw_handle = create_dissector_handle(dissect_raw, -1);
+  dissector_add("wtap_encap", WTAP_ENCAP_RAW_IP, raw_handle);
 }
