@@ -2,7 +2,7 @@
  * Routines for rpc dissection
  * Copyright 1999, Uwe Girlich <Uwe.Girlich@philosys.de>
  * 
- * $Id: packet-rpc.c,v 1.64 2001/07/03 02:05:47 guy Exp $
+ * $Id: packet-rpc.c,v 1.77 2001/12/10 00:25:33 guy Exp $
  * 
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,6 +40,7 @@
 #include "packet.h"
 #include "conversation.h"
 #include "packet-rpc.h"
+#include "prefs.h"
 
 /*
  * See:
@@ -60,6 +61,9 @@
 
 #define RPC_RM_FRAGLEN  0x7fffffffL
 
+/* desegmentation of RPC over TCP */
+static gboolean rpc_desegment = TRUE;
+
 static struct true_false_string yesno = { "Yes", "No" };
 
 
@@ -92,7 +96,7 @@ static const value_string rpc_authgss_proc[] = {
 	{ 0, NULL }
 };
 
-static const value_string rpc_authgss_svc[] = {
+value_string rpc_authgss_svc[] = {
 	{ RPCSEC_GSS_SVC_NONE, "rpcsec_gss_svc_none" },
 	{ RPCSEC_GSS_SVC_INTEGRITY, "rpcsec_gss_svc_integrity" },
 	{ RPCSEC_GSS_SVC_PRIVACY, "rpcsec_gss_svc_privacy" },
@@ -189,6 +193,10 @@ static gint ett_rpc_gids = -1;
 static gint ett_rpc_gss_data = -1;
 static gint ett_rpc_array = -1;
 
+static dissector_handle_t rpc_tcp_handle;
+static dissector_handle_t rpc_handle;
+static dissector_handle_t data_handle;
+
 /* Hash table with info on RPC program numbers */
 static GHashTable *rpc_progs;
 
@@ -217,6 +225,8 @@ typedef struct _rpc_prog_info_value {
 	char* progname;
 } rpc_prog_info_value;
 
+static void dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+static void dissect_rpc_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
 /***********************************/
 /* Hash array with procedure names */
@@ -371,13 +381,20 @@ typedef struct _rpc_call_info_key {
 
 static GMemChunk *rpc_call_info_key_chunk;
 
+typedef enum {
+	FLAVOR_UNKNOWN,		/* authentication flavor unknown */
+	FLAVOR_NOT_GSSAPI,	/* flavor isn't GSSAPI */
+	FLAVOR_GSSAPI_NO_INFO,	/* flavor is GSSAPI, procedure & service unknown */
+	FLAVOR_GSSAPI		/* flavor is GSSAPI, procedure & service known */
+} flavor_t;
+
 typedef struct _rpc_call_info_value {
 	guint32	req_num;	/* frame number of first request seen */
 	guint32	rep_num;	/* frame number of first reply seen */
 	guint32	prog;
 	guint32	vers;
 	guint32	proc;
-	guint32 flavor;
+	flavor_t flavor;
 	guint32 gss_proc;
 	guint32 gss_svc;
 	rpc_proc_info_value*	proc_info;
@@ -443,19 +460,12 @@ int
 dissect_rpc_uint64(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 int hfindex, int offset)
 {
-	guint32 value_low;
-	guint32 value_high;
+	header_field_info	*hfinfo;
 
-	value_high = tvb_get_ntohl(tvb, offset + 0);
-	value_low  = tvb_get_ntohl(tvb, offset + 4);
-
-	if (tree) {
-		if (value_high)
-			proto_tree_add_text(tree, tvb, offset, 8,
-				"%s: 0x%x%08x", proto_registrar_get_name(hfindex), value_high, value_low);
-		else
-			proto_tree_add_uint(tree, hfindex, tvb, offset, 8, value_low);
-	}
+	hfinfo = proto_registrar_get_nth(hfindex);
+	g_assert(hfinfo->type == FT_UINT64);
+	if (tree)
+		proto_tree_add_item(tree, hfindex, tvb, offset, 8, FALSE);
 
 	return offset + 8;
 }
@@ -1068,12 +1078,14 @@ dissect_rpc_authgss_integ_data(tvbuff_t *tvb, packet_info *pinfo,
 		proto_tree_add_uint(gtree, hf_rpc_authgss_seq,
 				    tvb, offset+4, 4, seq);
 	}
+	offset += 8;
+
 	if (dissect_function != NULL) {
 		/* offset = */
 		call_dissect_function(tvb, pinfo, gtree, offset,
 				      dissect_function, progname);
 	}
-	offset += 8 + length;
+	offset += length - 4;
 	offset = dissect_rpc_data(tvb, pinfo, tree, hf_rpc_authgss_checksum,
 			offset);
 	return offset;
@@ -1116,13 +1128,35 @@ dissect_rpc_indir_call(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
 		/* Keep track of the address and port whence the call came,
 		   and the port to which the call is being sent, so that
-		   we can match up calls wityh replies.  (We don't worry
+		   we can match up calls with replies.
+
+		   If the transport is connection-oriented (we check, for
+		   now, only for "pinfo->ptype" of PT_TCP), we take
+		   into account the address from which the call was sent
+		   and the address to which the call was sent, because
+		   the addresses of the two endpoints should be the same
+		   for all calls and replies.
+
+		   If the transport is connectionless, we don't worry
 		   about the address to which the call was sent and from
 		   which the reply was sent, because there's no
 		   guarantee that the reply will come from the address
-		   to which the call was sent.) */
-		conversation = find_conversation(&pinfo->src, &null_address,
-		    pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+		   to which the call was sent. */
+		if (pinfo->ptype == PT_TCP) {
+			conversation = find_conversation(&pinfo->src,
+			    &pinfo->dst, pinfo->ptype, pinfo->srcport,
+			    pinfo->destport, 0);
+		} else {
+			/*
+			 * XXX - can we just use NO_ADDR_B?  Unfortunately,
+			 * you currently still have to pass a non-null
+			 * pointer for the second address argument even
+			 * if you do that.
+			 */
+			conversation = find_conversation(&pinfo->src,
+			    &null_address, pinfo->ptype, pinfo->srcport,
+			    pinfo->destport, 0);
+		}
 		if (conversation == NULL) {
 			/* It's not part of any conversation - create a new
 			   one.
@@ -1130,11 +1164,22 @@ dissect_rpc_indir_call(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 			   XXX - this should never happen, as we should've
 			   created a conversation for it in the RPC
 			   dissector. */
-			conversation = conversation_new(&pinfo->src,
-			    &null_address, pinfo->ptype, pinfo->srcport,
-			    pinfo->destport, NULL, 0);
+			if (pinfo->ptype == PT_TCP) {
+				conversation = conversation_new(&pinfo->src,
+				    &pinfo->dst, pinfo->ptype, pinfo->srcport,
+				    pinfo->destport, 0);
+			} else {
+				conversation = conversation_new(&pinfo->src,
+				    &null_address, pinfo->ptype, pinfo->srcport,
+				    pinfo->destport, 0);
+			}
 		}
 
+		/* Make the dissector for this conversation the non-heuristic
+		   RPC dissector. */
+		conversation_set_dissector(conversation,
+		    (pinfo->ptype == PT_TCP) ? rpc_tcp_handle : rpc_handle);
+
 		/* Prepare the key data.
 
 		   Dissectors for RPC procedure calls and replies shouldn't
@@ -1159,7 +1204,12 @@ dissect_rpc_indir_call(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 			rpc_call->prog = prog;
 			rpc_call->vers = vers;
 			rpc_call->proc = proc;
-			rpc_call->flavor = 0;
+
+			/*
+			 * XXX - what about RPCSEC_GSS?
+			 * Do we have to worry about it?
+			 */
+			rpc_call->flavor = FLAVOR_NOT_GSSAPI;
 			rpc_call->gss_proc = 0;
 			rpc_call->gss_svc = 0;
 			rpc_call->proc_info = value;
@@ -1212,12 +1262,33 @@ dissect_rpc_indir_reply(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	   calls.  A reply must match a call that we've seen, and the
 	   reply must be sent to the same port and address that the
 	   call came from, and must come from the port to which the
-	   call was sent.  (We don't worry about the address to which
-	   the call was sent and from which the reply was sent, because
-	   there's no guarantee that the reply will come from the address
-	   to which the call was sent.) */
-	conversation = find_conversation(&null_address, &pinfo->dst,
-	    pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+	   call was sent.
+
+	   If the transport is connection-oriented (we check, for
+	   now, only for "pinfo->ptype" of PT_TCP), we take
+	   into account the address from which the call was sent
+	   and the address to which the call was sent, because
+	   the addresses of the two endpoints should be the same
+	   for all calls and replies.
+
+	   If the transport is connectionless, we don't worry
+	   about the address to which the call was sent and from
+	   which the reply was sent, because there's no
+	   guarantee that the reply will come from the address
+	   to which the call was sent. */
+	if (pinfo->ptype == PT_TCP) {
+		conversation = find_conversation(&pinfo->src, &pinfo->dst,
+		    pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+	} else {
+		/*
+		 * XXX - can we just use NO_ADDR_B?  Unfortunately,
+		 * you currently still have to pass a non-null
+		 * pointer for the second address argument even
+		 * if you do that.
+		 */
+		conversation = find_conversation(&null_address, &pinfo->dst,
+		    pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+	}
 	if (conversation == NULL) {
 		/* We haven't seen an RPC call for that conversation,
 		   so we can't check for a reply to that call.
@@ -1292,10 +1363,33 @@ dissect_rpc_indir_reply(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	return offset;
 }
 
+/*
+ * Just mark this as a continuation of an earlier packet.
+ */
+static void
+dissect_rpc_continuation(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	proto_item *rpc_item;
+	proto_tree *rpc_tree;
+
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RPC");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_set_str(pinfo->cinfo, COL_INFO, "Continuation");
+
+	if (tree) {
+		rpc_item = proto_tree_add_item(tree, proto_rpc, tvb, 0,
+				tvb_length(tvb), FALSE);
+		rpc_tree = proto_item_add_subtree(rpc_item, ett_rpc);
+		proto_tree_add_text(rpc_tree, tvb, 0, tvb_length(tvb),
+		    "Continuation data");
+	}
+}
+
 static gboolean
-dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_rpc_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
+    proto_tree *tree, gboolean use_rm, guint32 rpc_rm)
 {
-	int offset = 0;
 	guint32	msg_type;
 	rpc_call_info_key rpc_call_key;
 	rpc_call_info_value *rpc_call = NULL;
@@ -1307,7 +1401,7 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	unsigned int prog = 0;
 	unsigned int vers = 0;
 	unsigned int proc = 0;
-	unsigned int flavor = 0;
+	flavor_t flavor = FLAVOR_UNKNOWN;
 	unsigned int gss_proc = 0;
 	unsigned int gss_svc = 0;
 	int	proto = 0;
@@ -1334,9 +1428,6 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	proto_tree *ptree = NULL;
 	int offset_old = offset;
 
-	int use_rm = 0;
-	guint32 rpc_rm = 0;
-
 	rpc_call_info_key	*new_rpc_call_key;
 	rpc_proc_info_key	key;
 	rpc_proc_info_value	*value = NULL;
@@ -1345,17 +1436,6 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	dissect_function_t *dissect_function = NULL;
 
-	/* TCP uses record marking */
-	use_rm = (pinfo->ptype == PT_TCP);
-
-	/* the first 4 bytes are special in "record marking mode" */
-	if (use_rm) {
-		if (!tvb_bytes_exist(tvb, offset, 4))
-			return FALSE;
-		rpc_rm = tvb_get_ntohl(tvb, offset);
-		offset += 4;
-	}
-
 	/*
 	 * Check to see whether this looks like an RPC call or reply.
 	 */
@@ -1395,13 +1475,35 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		/* Check for RPC reply.  A reply must match a call that
 		   we've seen, and the reply must be sent to the same
 		   port and address that the call came from, and must
-		   come from the port to which the call was sent.  (We
-		   don't worry about the address to which the call was
-		   sent and from which the reply was sent, because there's
-		   no guarantee that the reply will come from the address
-		   to which the call was sent.) */
-		conversation = find_conversation(&null_address, &pinfo->dst,
-		    pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+		   come from the port to which the call was sent.
+
+		   If the transport is connection-oriented (we check, for
+		   now, only for "pinfo->ptype" of PT_TCP), we take
+		   into account the address from which the call was sent
+		   and the address to which the call was sent, because
+		   the addresses of the two endpoints should be the same
+		   for all calls and replies.
+
+		   If the transport is connectionless, we don't worry
+		   about the address to which the call was sent and from
+		   which the reply was sent, because there's no
+		   guarantee that the reply will come from the address
+		   to which the call was sent. */
+		if (pinfo->ptype == PT_TCP) {
+			conversation = find_conversation(&pinfo->src,
+			    &pinfo->dst, pinfo->ptype, pinfo->srcport,
+			    pinfo->destport, 0);
+		} else {
+			/*
+			 * XXX - can we just use NO_ADDR_B?  Unfortunately,
+			 * you currently still have to pass a non-null
+			 * pointer for the second address argument even
+			 * if you do that.
+			 */
+			conversation = find_conversation(&null_address,
+			    &pinfo->dst, pinfo->ptype, pinfo->srcport,
+			    pinfo->destport, 0);
+		}
 		if (conversation == NULL) {
 			/* We haven't seen an RPC call for that conversation,
 			   so we can't check for a reply to that call. */
@@ -1426,10 +1528,10 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		return FALSE;
 	}
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "RPC");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RPC");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	if (tree) {
 		rpc_item = proto_tree_add_item(tree, proto_rpc, tvb, 0,
@@ -1483,10 +1585,10 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				"Program: %s (%u)", progname, prog);
 		}
 		
-		if (check_col(pinfo->fd, COL_PROTOCOL)) {
+		if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
 			/* Set the protocol name to the underlying
 			   program name. */
-			col_set_str(pinfo->fd, COL_PROTOCOL, progname);
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, progname);
 		}
 
 		vers = tvb_get_ntohl(tvb, offset+8);
@@ -1497,14 +1599,6 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 		proc = tvb_get_ntohl(tvb, offset+12);
 
-		/* Check for RPCSEC_GSS */
-		if (proc == 0) {
-			flavor = tvb_get_ntohl(tvb, offset+16);
-			if (flavor == RPCSEC_GSS) {
-				gss_proc = tvb_get_ntohl(tvb, offset+28);
-				gss_svc = tvb_get_ntohl(tvb, offset+34);
-			}
-		}
 		key.prog = prog;
 		key.vers = vers;
 		key.proc = proc;
@@ -1529,31 +1623,98 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				"Procedure: %s (%u)", procname, proc);
 		}
 
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO,"V%u %s %s XID 0x%x",
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO,"V%u %s %s XID 0x%x",
 				vers,
 				procname,
 				msg_type_name,
 				xid);
 		}
 
+		/* Check for RPCSEC_GSS */
+		if (tvb_bytes_exist(tvb, offset+16, 4)) {
+			switch (tvb_get_ntohl(tvb, offset+16)) {
+
+			case RPCSEC_GSS:
+				/*
+				 * It's GSS-API authentication...
+				 */
+				if (tvb_bytes_exist(tvb, offset+28, 8)) {
+					/*
+					 * ...and we have the procedure
+					 * and service information for it.
+					 */
+					flavor = FLAVOR_GSSAPI;
+					gss_proc = tvb_get_ntohl(tvb, offset+28);
+					gss_svc = tvb_get_ntohl(tvb, offset+36);
+				} else {
+					/*
+					 * ...but the procedure and service
+					 * information isn't available.
+					 */
+					flavor = FLAVOR_GSSAPI_NO_INFO;
+				}
+				break;
+
+			default:
+				/*
+				 * It's not GSS-API authentication.
+				 */
+				flavor = FLAVOR_NOT_GSSAPI;
+				break;
+			}
+		}
+
 		/* Keep track of the address and port whence the call came,
 		   and the port to which the call is being sent, so that
-		   we can match up calls wityh replies.  (We don't worry
+		   we can match up calls with replies.
+
+		   If the transport is connection-oriented (we check, for
+		   now, only for "pinfo->ptype" of PT_TCP), we take
+		   into account the address from which the call was sent
+		   and the address to which the call was sent, because
+		   the addresses of the two endpoints should be the same
+		   for all calls and replies.
+
+		   If the transport is connectionless, we don't worry
 		   about the address to which the call was sent and from
 		   which the reply was sent, because there's no
 		   guarantee that the reply will come from the address
-		   to which the call was sent.) */
-		conversation = find_conversation(&pinfo->src, &null_address,
-		    pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+		   to which the call was sent. */
+		if (pinfo->ptype == PT_TCP) {
+			conversation = find_conversation(&pinfo->src,
+			    &pinfo->dst, pinfo->ptype, pinfo->srcport,
+			    pinfo->destport, 0);
+		} else {
+			/*
+			 * XXX - can we just use NO_ADDR_B?  Unfortunately,
+			 * you currently still have to pass a non-null
+			 * pointer for the second address argument even
+			 * if you do that.
+			 */
+			conversation = find_conversation(&pinfo->src,
+			    &null_address, pinfo->ptype, pinfo->srcport,
+			    pinfo->destport, 0);
+		}
 		if (conversation == NULL) {
 			/* It's not part of any conversation - create a new
 			   one. */
-			conversation = conversation_new(&pinfo->src,
-			    &null_address, pinfo->ptype, pinfo->srcport,
-			    pinfo->destport, NULL, 0);
+			if (pinfo->ptype == PT_TCP) {
+				conversation = conversation_new(&pinfo->src,
+				    &pinfo->dst, pinfo->ptype, pinfo->srcport,
+				    pinfo->destport, 0);
+			} else {
+				conversation = conversation_new(&pinfo->src,
+				    &null_address, pinfo->ptype, pinfo->srcport,
+				    pinfo->destport, 0);
+			}
 		}
 
+		/* Make the dissector for this conversation the non-heuristic
+		   RPC dissector. */
+		conversation_set_dissector(conversation,
+		    (pinfo->ptype == PT_TCP) ? rpc_tcp_handle : rpc_handle);
+
 		/* prepare the key data */
 		rpc_call_key.xid = xid;
 		rpc_call_key.conversation = conversation;
@@ -1567,8 +1728,8 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			if (pinfo->fd->num != rpc_call->req_num) {
 				/* No, so it's a duplicate request.
 				   Mark it as such. */
-				if (check_col(pinfo->fd, COL_INFO)) {
-					col_append_fstr(pinfo->fd, COL_INFO,
+				if (check_col(pinfo->cinfo, COL_INFO)) {
+					col_append_fstr(pinfo->cinfo, COL_INFO,
 						" dup XID 0x%x", xid);
 					if (rpc_tree) {
 						proto_tree_add_uint_hidden(rpc_tree,
@@ -1655,15 +1816,15 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			ett = rpc_prog->ett;
 			progname = rpc_prog->progname;
 
-			if (check_col(pinfo->fd, COL_PROTOCOL)) {
+			if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
 				/* Set the protocol name to the underlying
 				   program name. */
-				col_set_str(pinfo->fd, COL_PROTOCOL, progname);
+				col_set_str(pinfo->cinfo, COL_PROTOCOL, progname);
 			}
 		}
 
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO,"V%u %s %s XID 0x%x",
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO,"V%u %s %s XID 0x%x",
 				vers,
 				procname,
 				msg_type_name,
@@ -1692,8 +1853,8 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			if (rpc_call->rep_num != pinfo->fd->num) {
 				/* No, so it's a duplicate reply.
 				   Mark it as such. */
-				if (check_col(pinfo->fd, COL_INFO)) {
-					col_append_fstr(pinfo->fd, COL_INFO,
+				if (check_col(pinfo->cinfo, COL_INFO)) {
+					col_append_fstr(pinfo->cinfo, COL_INFO,
 						" dup XID 0x%x", xid);
 					if (rpc_tree) {
 						proto_tree_add_uint_hidden(rpc_tree,
@@ -1810,8 +1971,45 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (!proto_is_protocol_enabled(proto))
 		dissect_function = NULL;
 
-	/* RPCSEC_GSS processing. */
-	if (flavor == RPCSEC_GSS) {
+	/*
+	 * Handle RPCSEC_GSS specially.
+	 */
+	switch (flavor) {
+
+	case FLAVOR_UNKNOWN:
+		/*
+		 * We don't know the authentication flavor, so we can't
+		 * dissect the payload.
+		 */
+		proto_tree_add_text(ptree, tvb, offset, tvb_length_remaining(tvb, offset),
+		    "Unknown authentication flavor - cannot dissect");
+		return TRUE;
+
+	case FLAVOR_NOT_GSSAPI:
+		/*
+		 * It's not GSS-API authentication.  Just dissect the
+		 * payload.
+		 */
+		offset = call_dissect_function(tvb, pinfo, ptree, offset,
+				dissect_function, progname);
+		break;
+
+	case FLAVOR_GSSAPI_NO_INFO:
+		/*
+		 * It's GSS-API authentication, but we don't have the
+		 * procedure and service information, so we can't dissect
+		 * the payload.
+		 */
+		proto_tree_add_text(ptree, tvb, offset, tvb_length_remaining(tvb, offset),
+		    "GSS-API authentication, but procedure and service unknown - cannot dissect");
+		return TRUE;
+
+	case FLAVOR_GSSAPI:
+		/*
+		 * It's GSS-API authentication, and we have the procedure
+		 * and service information; process the GSS-API stuff,
+		 * and process the payload if there is any.
+		 */
 		switch (gss_proc) {
 
 		case RPCSEC_GSS_INIT:
@@ -1849,18 +2047,153 @@ dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			break;
 		}
 	}
-	else {
-		offset=call_dissect_function(tvb, pinfo, ptree, offset,
-				dissect_function, progname);
-	}
 
 	/* dissect any remaining bytes (incomplete dissection) as pure data in
 	   the ptree */
-	dissect_data(tvb, offset, pinfo, ptree);
+	call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, ptree);
 
 	return TRUE;
 }
 
+static gboolean
+dissect_rpc_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	return dissect_rpc_message(tvb, 0, pinfo, tree, FALSE, 0);
+}
+
+static void
+dissect_rpc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	if (!dissect_rpc_message(tvb, 0, pinfo, tree, FALSE, 0))
+		dissect_rpc_continuation(tvb, pinfo, tree);
+}
+
+/*
+ * Can return:
+ *
+ *	NEED_MORE_DATA, if we don't have enough data to dissect anything;
+ *
+ *	IS_RPC, if we dissected at least one message in its entirety
+ *	as RPC;
+ *
+ *	IS_NOT_RPC, if we found no RPC message.
+ */
+typedef enum {
+	NEED_MORE_DATA,
+	IS_RPC,
+	IS_NOT_RPC
+} rpc_tcp_return_t;
+
+static rpc_tcp_return_t
+dissect_rpc_tcp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
+    gboolean is_heur)
+{
+	int offset = 0;
+	guint32 rpc_rm;
+	gboolean saw_rpc = FALSE;
+	gint32 len, seglen;
+	gint tvb_len, tvb_reported_len;
+	tvbuff_t *msg_tvb;
+
+	while (tvb_reported_length_remaining(tvb, offset) != 0) {
+		/*
+		 * XXX - we need to handle records that don't have the "last
+		 * fragment" bit set, and reassemble fragments.
+		 */
+
+		/* the first 4 bytes are special in "record marking mode" */
+		if (!tvb_bytes_exist(tvb, offset, 4)) {
+			/*
+			 * XXX - we should somehow arrange to handle
+			 * a record mark split across TCP segments.
+			 */
+			return saw_rpc ? IS_RPC : IS_NOT_RPC;
+		}
+		rpc_rm = tvb_get_ntohl(tvb, offset);
+
+		len = rpc_rm&RPC_RM_FRAGLEN;
+
+		/*
+		 * XXX - reject fragments bigger than 2 megabytes.
+		 * This is arbitrary, but should at least prevent
+		 * some crashes from either packets with really
+		 * large RPC-over-TCP fragments or from stuff that's
+		 * not really RPC.
+		 */
+		if (len > 2*1024*1024)
+			return saw_rpc ? IS_RPC : IS_NOT_RPC;
+		if (rpc_desegment) {
+			seglen = tvb_length_remaining(tvb, offset + 4);
+
+			if (len > seglen && pinfo->can_desegment) {
+				/*
+				 * This frame doesn't have all of the
+				 * data for this message, but we can do
+				 * reassembly on it.
+				 *
+				 * If this is a heuristic dissector, just
+				 * return IS_NOT_RPC - we don't want to try
+				 * to get more data, as that's too likely
+				 * to cause us to misidentify this as
+				 * RPC.
+				 *
+				 * If this isn't a heuristic dissector,
+				 * we've already identified this conversation
+				 * as containing RPC data, as we saw RPC
+				 * data in previous frames.  Try to get
+				 * more data.
+				 */
+				if (is_heur)
+					return IS_NOT_RPC;
+				else {
+					pinfo->desegment_offset = offset;
+					pinfo->desegment_len = len - seglen;
+					return NEED_MORE_DATA;
+				}
+			}
+		}
+		len += 4;	/* include record mark */
+		tvb_len = tvb_length_remaining(tvb, offset);
+		tvb_reported_len = tvb_reported_length_remaining(tvb, offset);
+		if (tvb_len > len)
+			tvb_len = len;
+		if (tvb_reported_len > len)
+			tvb_reported_len = len;
+		msg_tvb = tvb_new_subset(tvb, offset, tvb_len,
+		    tvb_reported_len);
+		if (!dissect_rpc_message(msg_tvb, 4, pinfo, tree,
+		    TRUE, rpc_rm))
+			break;
+		offset += len;
+		saw_rpc = TRUE;
+	}
+	return saw_rpc ? IS_RPC : IS_NOT_RPC;
+}
+
+static gboolean
+dissect_rpc_tcp_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	switch (dissect_rpc_tcp_common(tvb, pinfo, tree, TRUE)) {
+
+	case IS_RPC:
+		return TRUE;
+
+	case IS_NOT_RPC:
+		return FALSE;
+
+	default:
+		/* "Can't happen" */
+		g_assert_not_reached();
+		return FALSE;
+	}
+}
+
+static void
+dissect_rpc_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	if (dissect_rpc_tcp_common(tvb, pinfo, tree, FALSE) == IS_NOT_RPC)
+		dissect_rpc_continuation(tvb, pinfo, tree);
+}
 
 /* Discard any state we've saved. */
 static void
@@ -1887,7 +2220,6 @@ rpc_init_protocol(void)
 	    G_ALLOC_ONLY);
 }
 
-
 /* will be called once from register.c at startup time */
 void
 proto_register_rpc(void)
@@ -2041,14 +2373,23 @@ proto_register_rpc(void)
 		&ett_rpc_cred,
 		&ett_rpc_verf,
 		&ett_rpc_gids,
+		&ett_rpc_gss_data,
 		&ett_rpc_array,
 	};
+	module_t *rpc_module;
 
 	proto_rpc = proto_register_protocol("Remote Procedure Call",
 	    "RPC", "rpc");
 	proto_register_field_array(proto_rpc, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 	register_init_routine(&rpc_init_protocol);
+	rpc_tcp_handle = create_dissector_handle(dissect_rpc_tcp, proto_rpc);
+	rpc_handle = create_dissector_handle(dissect_rpc, proto_rpc);
+	rpc_module = prefs_register_protocol(proto_rpc, NULL);
+	prefs_register_bool_preference(rpc_module, "desegment_rpc_over_tcp",
+		"Desegment all RPC over TCP commands",
+		"Whether the RPC dissector should desegment all RPC over TCP commands",
+		&rpc_desegment);
 
 	/*
 	 * Init the hash tables.  Dissectors for RPC protocols must
@@ -2065,10 +2406,10 @@ proto_register_rpc(void)
 	rpc_procs = g_hash_table_new(rpc_proc_hash, rpc_proc_equal);
 }
 
-
 void
 proto_reg_handoff_rpc(void)
 {
-	heur_dissector_add("tcp", dissect_rpc, proto_rpc);
-	heur_dissector_add("udp", dissect_rpc, proto_rpc);
+	heur_dissector_add("tcp", dissect_rpc_tcp_heur, proto_rpc);
+	heur_dissector_add("udp", dissect_rpc_heur, proto_rpc);
+	data_handle = find_dissector("data");
 }
