@@ -4,12 +4,11 @@
  * Robert Tsai <rtsai@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-rsh.c,v 1.11 2001/06/18 02:17:51 guy Exp $
+ * $Id: packet-rsh.c,v 1.14 2001/12/10 00:25:33 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +23,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,9 +53,9 @@ dissect_rsh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gint		next_offset;
 	int		linelen;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "RSH");
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RSH");
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 		/* Put the first line from the buffer into the summary. */
 		tvb_find_line_end(tvb, offset, -1, &next_offset);
 		linelen = next_offset - offset;	/* include the line terminator */
@@ -68,9 +65,9 @@ dissect_rsh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		 * the captured data in the packet, so we don't throw
 		 * an exception in the "tvb_get_ptr()" call.
 		 */
-		if (linelen > tvb_length(tvb))
+		if (linelen > (int) tvb_length(tvb))
 			linelen = tvb_length(tvb);
-		col_add_str(pinfo->fd, COL_INFO,
+		col_add_str(pinfo->cinfo, COL_INFO,
 		    tvb_format_text(tvb, offset, linelen));
 	}
 	if (tree) {
@@ -132,5 +129,8 @@ proto_register_rsh(void)
 void
 proto_reg_handoff_rsh(void)
 {
-	dissector_add("tcp.port", TCP_PORT_RSH, dissect_rsh, proto_rsh);
+	dissector_handle_t rsh_handle;
+
+	rsh_handle = create_dissector_handle(dissect_rsh, proto_rsh);
+	dissector_add("tcp.port", TCP_PORT_RSH, rsh_handle);
 }
