@@ -3,13 +3,12 @@
  *
  * (c) Copyright Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: packet-rsvp.c,v 1.46 2001/06/26 20:50:30 guy Exp $
+ * $Id: packet-rsvp.c,v 1.49 2001/12/10 00:25:33 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  * 
- * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -666,15 +665,15 @@ dissect_rsvp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     int offset2;
     char *objtype;
 
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-        col_set_str(pinfo->fd, COL_PROTOCOL, "RSVP");
-    if (check_col(pinfo->fd, COL_INFO))
-        col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "RSVP");
+    if (check_col(pinfo->cinfo, COL_INFO))
+        col_clear(pinfo->cinfo, COL_INFO);
 
     ver_flags = tvb_get_guint8(tvb, offset+0);
     message_type = tvb_get_guint8(tvb, offset+1);
-    if (check_col(pinfo->fd, COL_INFO)) {
-        col_add_str(pinfo->fd, COL_INFO,
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+        col_add_str(pinfo->cinfo, COL_INFO,
             val_to_str(message_type, message_type_vals, "Unknown (%u)")); 
     }
 
@@ -700,7 +699,7 @@ dissect_rsvp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				   offset+1, 1, 1);
 	}
 	cksum = tvb_get_ntohs(tvb, offset+2);
-	if (!pinfo->fragmented && tvb_length(tvb) >= msg_length) {
+	if (!pinfo->fragmented && (int) tvb_length(tvb) >= msg_length) {
 	    /* The packet isn't part of a fragmented datagram and isn't
 	       truncated, so we can checksum it. */
 	    cksum_vec[0].ptr = tvb_get_ptr(tvb, 0, msg_length);
@@ -2069,5 +2068,8 @@ proto_register_rsvp(void)
 void
 proto_reg_handoff_rsvp(void)
 {
-	dissector_add("ip.proto", IP_PROTO_RSVP, dissect_rsvp, proto_rsvp);
+	dissector_handle_t rsvp_handle;
+
+	rsvp_handle = create_dissector_handle(dissect_rsvp, proto_rsvp);
+	dissector_add("ip.proto", IP_PROTO_RSVP, rsvp_handle);
 }
