@@ -1,6 +1,6 @@
 /* packet-rtcp.c
  *
- * $Id: packet-rtcp.c,v 1.18 2001/07/03 04:56:45 guy Exp $
+ * $Id: packet-rtcp.c,v 1.27 2001/12/10 00:25:33 guy Exp $
  *
  * Routines for RTCP dissection
  * RTCP = Real-time Transport Control Protocol
@@ -178,15 +178,24 @@ static gint ett_sdes_item      = -1;
 static address fake_addr;
 static int heur_init = FALSE;
 
-static char rtcp_proto[] = "RTCP";
-
 static gboolean dissect_rtcp_heur( tvbuff_t *tvb, packet_info *pinfo,
     proto_tree *tree );
+static void dissect_rtcp( tvbuff_t *tvb, packet_info *pinfo,
+     proto_tree *tree );
 
-void rtcp_add_address( const unsigned char* ip_addr, int prt )
+void rtcp_add_address( packet_info *pinfo, const unsigned char* ip_addr,
+    int prt )
 {
 	address src_addr;
-	conversation_t* pconv = ( conversation_t* ) NULL;
+	conversation_t* pconv;
+
+	/*
+	 * If this isn't the first time this packet has been processed,
+	 * we've already done this work, so we don't need to do it
+	 * again.
+	 */
+	if (pinfo->fd->flags.visited)
+		return;
 
 	src_addr.type = AT_IPv4;
 	src_addr.len = 4;
@@ -209,10 +218,12 @@ void rtcp_add_address( const unsigned char* ip_addr, int prt )
 
 	/*
 	 * If not, add
+	 * XXX - use wildcard address and port B?
 	 */
 	if ( ! pconv ) {
-		conversation_new( &src_addr, &fake_addr, PT_UDP, (guint32) prt,
-		    (guint32) 0, (void*) rtcp_proto, 0 );
+		pconv = conversation_new( &src_addr, &fake_addr, PT_UDP,
+		    (guint32) prt, (guint32) 0, 0 );
+		conversation_add_proto_data(pconv, proto_rtcp, NULL);
 	}
 
 }
@@ -240,9 +251,6 @@ dissect_rtcp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
 	conversation_t* pconv;
 
-	if (!proto_is_protocol_enabled(proto_rtcp))
-		return FALSE;	/* RTCP has been disabled */
-
 	/* This is a heuristic dissector, which means we get all the UDP
 	 * traffic not sent to a known dissector and not claimed by
 	 * a heuristic dissector called before us!
@@ -262,15 +270,11 @@ dissect_rtcp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 
 
 	/*
-	 * An RTCP conversation always contains data
+	 * An RTCP conversation always has a data item for RTCP.
+	 * (Its existence is sufficient to indicate that this is an RTCP
+	 * conversation.)
 	 */
-	if ( pconv->data == NULL )
-		return FALSE;
-
-	/*
-	 * An RTCP conversation data always contains "RTCP"
-	 */
-	if ( strcmp( pconv->data, rtcp_proto ) != 0 )
+	if (conversation_get_proto_data(pconv, proto_rtcp) == NULL)
 		return FALSE;
 
 	/*
@@ -369,7 +373,7 @@ dissect_rtcp_app( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
 
 static int
 dissect_rtcp_bye( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
-    int count )
+    unsigned int count )
 {
 	unsigned int chunk          = 1;
 	unsigned int reason_length  = 0;
@@ -401,7 +405,7 @@ dissect_rtcp_bye( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
 
 static int
 dissect_rtcp_sdes( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
-    int count )
+    unsigned int count )
 {
 	unsigned int chunk          = 1;
 	proto_item *sdes_item;
@@ -442,8 +446,8 @@ dissect_rtcp_sdes( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
 		 * Not every message is ended with "null" bytes, so check for
 		 * end of frame instead.
 		 */
-		while ( ( tvb_get_guint8( tvb, offset ) != RTCP_SDES_END )
-		    && ( tvb_bytes_exist( tvb, offset, 2) ) ) {
+		while ( ( tvb_reported_length_remaining( tvb, offset ) > 0 )
+		    && ( tvb_get_guint8( tvb, offset ) != RTCP_SDES_END ) ) {
 			/* ID, 8 bits */
 			sdes_type = tvb_get_guint8( tvb, offset );
 			proto_tree_add_item( sdes_item_tree, hf_rtcp_ssrc_type, tvb, offset, 1, FALSE );
@@ -506,7 +510,7 @@ dissect_rtcp_sdes( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
 
 static int
 dissect_rtcp_rr( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
-    int count )
+    unsigned int count )
 {
 	unsigned int counter = 1;
 	proto_tree *ssrc_tree = (proto_tree*) NULL;
@@ -579,7 +583,7 @@ dissect_rtcp_rr( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
 
 static int
 dissect_rtcp_sr( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
-    int count )
+    unsigned int count )
 {
 #if 0
 	gchar buff[ NTP_TS_SIZE ];
@@ -613,13 +617,13 @@ dissect_rtcp_sr( tvbuff_t *tvb, int offset, frame_data *fd, proto_tree *tree,
 	offset += 4;
 
 	/* The rest of the packet is equal to the RR packet */
-	if ( count > 0 )
+	if ( count != 0 )
 		offset = dissect_rtcp_rr( tvb, offset, fd, tree, count );
 
 	return offset;
 }
 
-void
+static void
 dissect_rtcp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
 	proto_item *ti           = NULL;
@@ -631,41 +635,37 @@ dissect_rtcp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	unsigned int offset      = 0;
 	guint16 packet_length    = 0;
 
-	CHECK_DISPLAY_AS_DATA(proto_rtcp, tvb, pinfo, tree);
-
-	pinfo->current_proto = "RTCP";
-
-	if ( check_col( pinfo->fd, COL_PROTOCOL ) )   {
-		col_set_str( pinfo->fd, COL_PROTOCOL, "RTCP" );
+	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )   {
+		col_set_str( pinfo->cinfo, COL_PROTOCOL, "RTCP" );
 	}
 	
-	if ( check_col( pinfo->fd, COL_INFO) ) {
+	if ( check_col( pinfo->cinfo, COL_INFO) ) {
 		/* The second octet contains the packet type */
 		/* switch ( pd[ offset + 1 ] ) { */
 		switch ( tvb_get_guint8( tvb, 1 ) ) {
 			case RTCP_SR:
-				col_set_str( pinfo->fd, COL_INFO, "Sender Report");
+				col_set_str( pinfo->cinfo, COL_INFO, "Sender Report");
 				break;
 			case RTCP_RR:
-				col_set_str( pinfo->fd, COL_INFO, "Receiver Report");
+				col_set_str( pinfo->cinfo, COL_INFO, "Receiver Report");
 				break;
 			case RTCP_SDES:
-				col_set_str( pinfo->fd, COL_INFO, "Source Description");
+				col_set_str( pinfo->cinfo, COL_INFO, "Source Description");
 				break;
 			case RTCP_BYE:
-				col_set_str( pinfo->fd, COL_INFO, "Goodbye");
+				col_set_str( pinfo->cinfo, COL_INFO, "Goodbye");
 				break;
 			case RTCP_APP:
-				col_set_str( pinfo->fd, COL_INFO, "Application defined");
+				col_set_str( pinfo->cinfo, COL_INFO, "Application defined");
 				break;
 			case RTCP_FIR:
-				col_set_str( pinfo->fd, COL_INFO, "Full Intra-frame Request (H.261)");
+				col_set_str( pinfo->cinfo, COL_INFO, "Full Intra-frame Request (H.261)");
 				break;
 			case RTCP_NACK:
-				col_set_str( pinfo->fd, COL_INFO, "Negative Acknowledgement (H.261)");
+				col_set_str( pinfo->cinfo, COL_INFO, "Negative Acknowledgement (H.261)");
 				break;
 			default:
-				col_set_str( pinfo->fd, COL_INFO, "Unknown packet type");
+				col_set_str( pinfo->cinfo, COL_INFO, "Unknown packet type");
 				break;
 		}
 	}
@@ -1213,6 +1213,8 @@ proto_register_rtcp(void)
 	proto_register_field_array(proto_rtcp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
+	register_dissector("rtcp", dissect_rtcp, proto_rtcp);
+
 #if 0
 	register_init_routine( &rtcp_init );
 #endif
@@ -1221,9 +1223,12 @@ proto_register_rtcp(void)
 void
 proto_reg_handoff_rtcp(void)
 {
+	dissector_handle_t rtcp_handle;
+
 	/*
-	 * Register this dissector as one that can be assigned to a
-	 * UDP conversation.
+	 * Register this dissector as one that can be selected by a
+	 * UDP port number.
 	 */
-	conv_dissector_add("udp", dissect_rtcp, proto_rtcp);
+	rtcp_handle = find_dissector("rtcp");
+	dissector_add_handle("udp.port", rtcp_handle);
 }
