@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <andreas.sikkema@philips.com>
  *
- * $Id: packet-rtp.c,v 1.22 2001/07/03 04:56:45 guy Exp $
+ * $Id: packet-rtp.c,v 1.31 2001/12/10 00:25:33 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -100,6 +100,12 @@ static gint ett_hdr_ext   = -1;
 
 static dissector_handle_t h261_handle;
 static dissector_handle_t mpeg1_handle;
+static dissector_handle_t data_handle;
+
+static gboolean dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo,
+    proto_tree *tree );
+static void dissect_rtp( tvbuff_t *tvb, packet_info *pinfo,
+    proto_tree *tree );
 
 /*
  * Fields in the first octet of the RTP header.
@@ -194,12 +200,19 @@ static const value_string rtp_payload_type_vals[] =
 static address fake_addr;
 static int heur_init = FALSE;
 
-static const char rtp_proto[] = "RTP";
-
-void rtp_add_address( const unsigned char* ip_addr, int prt )
+void rtp_add_address( packet_info *pinfo, const unsigned char* ip_addr,
+    int prt )
 {
 	address src_addr;
-	conversation_t* pconv = ( conversation_t* ) NULL;
+	conversation_t* pconv;
+
+	/*
+	 * If this isn't the first time this packet has been processed,
+	 * we've already done this work, so we don't need to do it
+	 * again.
+	 */
+	if (pinfo->fd->flags.visited)
+		return;
 
 	src_addr.type = AT_IPv4;
 	src_addr.len = 4;
@@ -222,10 +235,12 @@ void rtp_add_address( const unsigned char* ip_addr, int prt )
 
 	/*
 	 * If not, add
+	 * XXX - use wildcard address and port B?
 	 */
 	if ( ! pconv ) {
-		conversation_new( &src_addr, &fake_addr, PT_UDP, (guint32) prt,
-		    (guint32) 0, ( void * ) rtp_proto, 0 );
+		pconv = conversation_new( &src_addr, &fake_addr, PT_UDP,
+		    (guint32) prt, (guint32) 0, 0 );
+		conversation_add_proto_data(pconv, proto_rtp, NULL);
 	}
 
 }
@@ -248,14 +263,11 @@ static void rtp_init( void )
 }
 #endif
 
-gboolean
+static gboolean
 dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
 	conversation_t* pconv;
 
-	if (!proto_is_protocol_enabled(proto_rtp))
-		return FALSE;	/* RTP has been disabled */
-
 	/* This is a heuristic dissector, which means we get all the TCP
 	 * traffic not sent to a known dissector and not claimed by
 	 * a heuristic dissector called before us!
@@ -274,15 +286,11 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	}
 
 	/*
-	 * An RTP conversation always contains data
+	 * An RTP conversation always has a data item for RTP.
+	 * (Its existence is sufficient to indicate that this is an RTP
+	 * conversation.)
 	 */
-	if ( pconv->data == NULL )
-		return FALSE;
-
-	/*
-	 * An RTP conversation data always contains "RTP"
-	 */
-	if ( strcmp( pconv->data, rtp_proto ) != 0 )
+	if (conversation_get_proto_data(pconv, proto_rtp) == NULL)
 		return FALSE;
 
 	dissect_rtp( tvb, pinfo, tree );
@@ -311,12 +319,12 @@ dissect_rtp_data( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 			break;	  
 
 		default:
-			proto_tree_add_bytes( rtp_tree, hf_rtp_data, tvb, offset, data_len, tvb_get_ptr( tvb, offset, data_len ) );
+			proto_tree_add_item( rtp_tree, hf_rtp_data, tvb, offset, data_len, FALSE );
 			break;
 	}
 }
 
-void
+static void
 dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
 	proto_item *ti            = NULL;
@@ -339,10 +347,6 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	guint32     sync_src;
 	guint32     csrc_item;
 
-	CHECK_DISPLAY_AS_DATA(proto_rtp, tvb, pinfo, tree);
-
-	pinfo->current_proto = "RTP";
-
 	/* Get the fields in the first octet */
 	octet = tvb_get_guint8( tvb, offset );
 	version = RTP_VERSION( octet );
@@ -351,12 +355,12 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		/*
 		 * Unknown or unsupported version.
 		 */
-		if ( check_col( pinfo->fd, COL_PROTOCOL ) )   {
-			col_set_str( pinfo->fd, COL_PROTOCOL, "RTP" );
+		if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )   {
+			col_set_str( pinfo->cinfo, COL_PROTOCOL, "RTP" );
 		}
 	
-		if ( check_col( pinfo->fd, COL_INFO) ) {
-			col_add_fstr( pinfo->fd, COL_INFO,
+		if ( check_col( pinfo->cinfo, COL_INFO) ) {
+			col_add_fstr( pinfo->cinfo, COL_INFO,
 			    "Unknown RTP version %u", version);
 		}
 
@@ -384,12 +388,12 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	timestamp = tvb_get_ntohl( tvb, offset + 4 );
 	sync_src = tvb_get_ntohl( tvb, offset + 8 );
 
-	if ( check_col( pinfo->fd, COL_PROTOCOL ) )   {
-		col_set_str( pinfo->fd, COL_PROTOCOL, "RTP" );
+	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )   {
+		col_set_str( pinfo->cinfo, COL_PROTOCOL, "RTP" );
 	}
 	
-	if ( check_col( pinfo->fd, COL_INFO) ) {
-		col_add_fstr( pinfo->fd, COL_INFO,
+	if ( check_col( pinfo->cinfo, COL_INFO) ) {
+		col_add_fstr( pinfo->cinfo, COL_INFO,
 		    "Payload type=%s, SSRC=%u, Seq=%u, Time=%u%s",
 		    val_to_str( payload_type, rtp_payload_type_vals,
 		        "Unknown (%u)" ),
@@ -489,7 +493,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 				 */
 				proto_tree_add_text(rtp_tree, tvb, 0, 0,
 				    "Frame has padding, but not all the frame data was captured");
-				dissect_data(tvb, offset, pinfo, rtp_tree);
+				call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, rtp_tree);
 				return;
 			}
 
@@ -761,6 +765,8 @@ proto_register_rtp(void)
 	proto_register_field_array(proto_rtp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
+	register_dissector("rtp", dissect_rtp, proto_rtp);
+
 #if 0
 	register_init_routine( &rtp_init );
 #endif
@@ -769,15 +775,19 @@ proto_register_rtp(void)
 void
 proto_reg_handoff_rtp(void)
 {
+	dissector_handle_t rtp_handle;
+
 	/*
 	 * Get handles for the H.261 and MPEG-1 dissectors.
 	 */
 	h261_handle = find_dissector("h261");
 	mpeg1_handle = find_dissector("mpeg1");
+	data_handle = find_dissector("data");
 
 	/*
-	 * Register this dissector as one that can be assigned to a
-	 * UDP conversation.
+	 * Register this dissector as one that can be selected by a
+	 * UDP port number.
 	 */
-	conv_dissector_add("udp", dissect_rtp, proto_rtp);
+	rtp_handle = find_dissector("rtp");
+	dissector_add_handle("udp.port", rtp_handle);
 }
