@@ -4,7 +4,7 @@
  * Based on routines from tcpdump patches by
  *   Ken Hornstein <kenh@cmf.nrl.navy.mil>
  *
- * $Id: packet-rx.c,v 1.23 2001/07/03 00:46:52 guy Exp $
+ * $Id: packet-rx.c,v 1.29 2001/12/10 00:25:34 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -156,26 +156,24 @@ dissect_rx_response_encrypted(tvbuff_t *tvb, packet_info *pinfo, proto_tree *par
 
 	/* epoch : 4 bytes */
 	{
-		struct timeval tv;
-		tv.tv_sec = tvb_get_ntohl(tvb, offset);
-		tv.tv_usec = 0;
+		nstime_t ts;
+		ts.secs = tvb_get_ntohl(tvb, offset);
+		ts.nsecs = 0;
 		
 		proto_tree_add_time(tree, hf_rx_epoch, tvb,
-			offset, 4, &tv);
+			offset, 4, &ts);
 		offset += 4;
 	}
 
 	/* cid : 4 bytes */
-	proto_tree_add_uint(tree, hf_rx_cid, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_cid, tvb, offset, 4, FALSE);
 	offset += 4;
 
 	/*FIXME dont know how to handle this checksum, skipping it */
 	offset += 4;
 
 	/* sequrityindex : 1 byte */
-	proto_tree_add_uint(tree, hf_rx_securityindex, tvb,
-		offset, 1, tvb_get_guint8(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_securityindex, tvb, offset, 1, FALSE);
 	offset += 4;
 
 	for (i=0; i<RX_MAXCALLS; i++) {
@@ -187,13 +185,11 @@ dissect_rx_response_encrypted(tvbuff_t *tvb, packet_info *pinfo, proto_tree *par
 	}
 
 	/* inc nonce : 4 bytes */
-	proto_tree_add_uint(tree, hf_rx_inc_nonce, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_inc_nonce, tvb, offset, 4, FALSE);
 	offset += 4;
 
 	/* level : 4 bytes */
-	proto_tree_add_uint(tree, hf_rx_level, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_level, tvb, offset, 4, FALSE);
 	offset += 4;
 	
 	proto_item_set_len(item, offset-old_offset);	
@@ -209,8 +205,8 @@ dissect_rx_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	guint32 version, tl;
 	int old_offset=offset;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO,
 			"RESPONSE  "
 			"Seq: %lu  "
 			"Call: %lu  "
@@ -239,8 +235,7 @@ dissect_rx_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 		offset = dissect_rx_response_encrypted(tvb, pinfo, tree, offset);
 
 		/* kvno */
-		proto_tree_add_uint(tree, hf_rx_kvno, tvb,
-			offset, 4, tvb_get_ntohl(tvb, offset));
+		proto_tree_add_item(tree, hf_rx_kvno, tvb, offset, 4, FALSE);
 		offset += 4;
 		
 		/* ticket_len */
@@ -265,8 +260,8 @@ dissect_rx_challenge(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	guint32 version;
 	int old_offset=offset;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO,
 			"CHALLENGE  "
 			"Seq: %lu  "
 			"Call: %lu  "
@@ -288,12 +283,10 @@ dissect_rx_challenge(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	offset += 4;
 
 	if (version==2) {
-		proto_tree_add_uint(tree, hf_rx_nonce, tvb,
-			offset, 4, tvb_get_ntohl(tvb, offset));
+		proto_tree_add_item(tree, hf_rx_nonce, tvb, offset, 4, FALSE);
 		offset += 4;
 
-		proto_tree_add_uint(tree, hf_rx_min_level, tvb,
-			offset, 4, tvb_get_ntohl(tvb, offset));
+		proto_tree_add_item(tree, hf_rx_min_level, tvb, offset, 4, FALSE);
 		offset += 4;
 	}
 
@@ -309,8 +302,8 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 	guint8 num;
 	int old_offset = offset;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO,
 			"ACK  "
 			"Seq: %lu  "
 			"Call: %lu  "
@@ -328,44 +321,37 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 
 
 	/* bufferspace: 2 bytes*/
-	proto_tree_add_uint(tree, hf_rx_bufferspace, tvb,
-		offset, 2, tvb_get_ntohs(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_bufferspace, tvb, offset, 2, FALSE);
 	offset += 2;
 	
 	/* maxskew: 2 bytes*/
-	proto_tree_add_uint(tree, hf_rx_maxskew, tvb,
-		offset, 2, tvb_get_ntohs(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_maxskew, tvb, offset, 2, FALSE);
 	offset += 2;
 	
 	/* first packet: 4 bytes*/
-	proto_tree_add_uint(tree, hf_rx_first_packet, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_first_packet, tvb, offset, 4, FALSE);
 	offset += 4;
 	
 	/* prev packet: 4 bytes*/
-	proto_tree_add_uint(tree, hf_rx_prev_packet, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_prev_packet, tvb, offset, 4, FALSE);
 	offset += 4;
 
 	/* serial : 4 bytes */	
-	proto_tree_add_uint(tree, hf_rx_serial, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset) );
+	proto_tree_add_item(tree, hf_rx_serial, tvb, offset, 4, FALSE);
 	offset += 4;
 
 	/* reason : 1 byte */
-	proto_tree_add_uint(tree, hf_rx_reason, tvb,
-		offset, 1, tvb_get_guint8(tvb, offset) );
+	proto_tree_add_item(tree, hf_rx_reason, tvb, offset, 1, FALSE);
 	offset += 1;
 
 	/* nACKs */
 	num = tvb_get_guint8(tvb, offset);
-	proto_tree_add_uint(tree, hf_rx_numacks, tvb,
-		offset, 1, tvb_get_guint8(tvb, offset) );
+	proto_tree_add_uint(tree, hf_rx_numacks, tvb, offset, 1, num);
 	offset += 1;
 
 	while(num--){
-		proto_tree_add_uint(tree, hf_rx_ack_type, tvb,
-			offset, 1, tvb_get_guint8(tvb, offset) );
+		proto_tree_add_item(tree, hf_rx_ack_type, tvb, offset, 1,
+			FALSE);
 		offset += 1;
 	}
 
@@ -380,24 +366,24 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 	if (tvb_length_remaining(tvb, offset)>3) {
 		offset += 3;	/* guess. some implementations adds 3 bytes */
 
-		if (tvb_length_remaining(tvb, offset) >= 4){
-			proto_tree_add_uint(tree, hf_rx_ifmtu, tvb, offset, 4,
-				tvb_get_ntohl(tvb, offset));
+		if (tvb_reported_length_remaining(tvb, offset) >= 4){
+			proto_tree_add_item(tree, hf_rx_ifmtu, tvb, offset, 4,
+				FALSE);
 			offset += 4;
 		}
-		if (tvb_length_remaining(tvb, offset) >= 4){
-			proto_tree_add_uint(tree, hf_rx_maxmtu, tvb, offset, 4,
-				tvb_get_ntohl(tvb, offset));
+		if (tvb_reported_length_remaining(tvb, offset) >= 4){
+			proto_tree_add_item(tree, hf_rx_maxmtu, tvb, offset, 4,
+				FALSE);
 			offset += 4;
 		}
-		if (tvb_length_remaining(tvb, offset) >= 4){
-			proto_tree_add_uint(tree, hf_rx_rwind, tvb, offset, 4,
-				tvb_get_ntohl(tvb, offset));
+		if (tvb_reported_length_remaining(tvb, offset) >= 4){
+			proto_tree_add_item(tree, hf_rx_rwind, tvb, offset, 4,
+				FALSE);
 			offset += 4;
 		}
-		if (tvb_length_remaining(tvb, offset) >= 4){
-			proto_tree_add_uint(tree, hf_rx_maxpackets, tvb, offset, 4,
-				tvb_get_ntohl(tvb, offset));
+		if (tvb_reported_length_remaining(tvb, offset) >= 4){
+			proto_tree_add_item(tree, hf_rx_maxpackets, tvb, offset, 4,
+				FALSE);
 			offset += 4;
 		}
 	}
@@ -408,14 +394,14 @@ dissect_rx_acks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int
 
 
 static int
-dissect_rx_flags(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, int offset)
+dissect_rx_flags(tvbuff_t *tvb, struct rxinfo *rxinfo, proto_tree *parent_tree, int offset)
 {
 	proto_tree *tree;
 	proto_item *item;
 	guint8 flags;
 
 	flags = tvb_get_guint8(tvb, offset);
-	pinfo->ps.rx.flags = flags;
+	rxinfo->flags = flags;
 
 	item = proto_tree_add_uint(parent_tree, hf_rx_flags, tvb,
 		offset, 1, flags);
@@ -442,14 +428,15 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	proto_tree *tree;
 	proto_item *item;
 	int offset = 0;
+	struct rxinfo rxinfo;
 	guint8 type;
 	guint32 seq, callnumber;
 	guint16 serviceid;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "RX");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RX");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	item = proto_tree_add_protocol_format(parent_tree, proto_rx, tvb, 
 		offset,	28, "RX Protocol");
@@ -457,18 +444,17 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 
 	/* epoch : 4 bytes */
 	{
-		struct timeval tv;
-		tv.tv_sec = tvb_get_ntohl(tvb, offset);
-		tv.tv_usec = 0;
+		nstime_t ts;;
+		ts.secs = tvb_get_ntohl(tvb, offset);
+		ts.nsecs = 0;
 		
 		proto_tree_add_time(tree, hf_rx_epoch, tvb,
-			offset, 4, &tv);
+			offset, 4, &ts);
 		offset += 4;
 	}
 
 	/* cid : 4 bytes */
-	proto_tree_add_uint(tree, hf_rx_cid, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_cid, tvb, offset, 4, FALSE);
 	offset += 4;
 
 	/* callnumber : 4 bytes */
@@ -476,18 +462,17 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	proto_tree_add_uint(tree, hf_rx_callnumber, tvb,
 		offset, 4, callnumber);
 	offset += 4;
-	pinfo->ps.rx.callnumber = callnumber;
+	rxinfo.callnumber = callnumber;
 
 	/* seq : 4 bytes */
 	seq = tvb_get_ntohl(tvb, offset);
 	proto_tree_add_uint(tree, hf_rx_seq, tvb,
 		offset, 4, seq);
 	offset += 4;
-	pinfo->ps.rx.seq = seq;
+	rxinfo.seq = seq;
 
 	/* serial : 4 bytes */
-	proto_tree_add_uint(tree, hf_rx_serial, tvb,
-		offset, 4, tvb_get_ntohl(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_serial, tvb, offset, 4, FALSE);
 	offset += 4;
 
 	/* type : 1 byte */
@@ -495,24 +480,27 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	proto_tree_add_uint(tree, hf_rx_type, tvb,
 		offset, 1, type);
 	offset += 1;
-	pinfo->ps.rx.type = type;
+	rxinfo.type = type;
 
 	/* flags : 1 byte */
-	offset = dissect_rx_flags(tvb, pinfo, tree, offset);
+	offset = dissect_rx_flags(tvb, &rxinfo, tree, offset);
 
 	/* userstatus : 1 byte */
-	proto_tree_add_uint(tree, hf_rx_userstatus, tvb,
-		offset, 1, tvb_get_guint8(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_userstatus, tvb, offset, 1, FALSE);
 	offset += 1;
 
 	/* sequrityindex : 1 byte */
-	proto_tree_add_uint(tree, hf_rx_securityindex, tvb,
-		offset, 1, tvb_get_guint8(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_securityindex, tvb, offset, 1, FALSE);
 	offset += 1;
 
+	/*
+	 * How clever: even though the AFS header files indicate that the
+	 * serviceId is first, it's really encoded _after_ the spare field.
+	 * I wasted a day figuring that out!
+	 */
+
 	/* spare */
-	proto_tree_add_uint(tree, hf_rx_spare, tvb,
-		offset, 2, tvb_get_ntohs(tvb, offset));
+	proto_tree_add_item(tree, hf_rx_spare, tvb, offset, 2, FALSE);
 	offset += 2;
 
 	/* service id : 2 bytes */
@@ -520,7 +508,7 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	proto_tree_add_uint(tree, hf_rx_serviceid, tvb,
 		offset, 2, serviceid);
 	offset += 2;
-	pinfo->ps.rx.serviceid = serviceid;
+	rxinfo.serviceid = serviceid;
 
 	switch (type) {
 	case RX_PACKET_TYPE_ACK:
@@ -532,8 +520,8 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		break;
 	case RX_PACKET_TYPE_ACKALL:
 		/* does not contain any payload */
-		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO,
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_add_fstr(pinfo->cinfo, COL_INFO,
 				"ACKALL  "
 				"Seq: %lu  "
 				"Call: %lu  "
@@ -554,6 +542,7 @@ dissect_rx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		break;
 	case RX_PACKET_TYPE_DATA: {
 		tvbuff_t *next_tvb;
+		pinfo->private_data = &rxinfo;
 		next_tvb = tvb_new_subset(tvb, offset, -1, -1);
 		call_dissector(afs_handle, next_tvb, pinfo, parent_tree);
 		};
@@ -740,6 +729,8 @@ proto_register_rx(void)
 void
 proto_reg_handoff_rx(void)
 {
+	dissector_handle_t rx_handle;
+
 	int port;
 
 	/*
@@ -749,8 +740,8 @@ proto_reg_handoff_rx(void)
 
 	/* Ports in the range UDP_PORT_RX_LOW to UDP_PORT_RX_HIGH
 	   are all used for various AFS services. */
+	rx_handle = create_dissector_handle(dissect_rx, proto_rx);
 	for (port = UDP_PORT_RX_LOW; port <= UDP_PORT_RX_HIGH; port++)
-		dissector_add("udp.port", port, dissect_rx, proto_rx);
-	dissector_add("udp.port", UDP_PORT_RX_AFS_BACKUPS, dissect_rx,
-	    proto_rx);
+		dissector_add("udp.port", port, rx_handle);
+	dissector_add("udp.port", UDP_PORT_RX_AFS_BACKUPS, rx_handle);
 }
