@@ -4,13 +4,21 @@
  * 
  * TODO: Pay attention to Content-Type: It might not always be SDP.
  *       Add hf_* fields for filtering support.
+ *       Add sip msg body dissection based on Content-Type for:
+ *                SDP, MIME, and other types
+ *       Align SIP methods with recent Internet Drafts or RFC
+ *               (SIP INFO, rfc2976 - done)
+ *               (SIP SUBSCRIBE-NOTIFY - done)
+ *               (SIP REFER - done)
+ *               check for other
  *
  * Copyright 2000, Heikki Vatiainen <hessu@cs.tut.fi>
+ * Copyright 2001, Jean-Francois Mule <jfm@clarent.com>
  *
- * $Id: packet-sip.c,v 1.15 2001/06/18 02:17:52 guy Exp $
+ * $Id: packet-sip.c,v 1.20 2001/12/10 00:25:34 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-cops.c
@@ -59,13 +67,18 @@ static const char *sip_methods[] = {
         "OPTIONS",
         "BYE",
         "CANCEL",
-        "REGISTER"
+        "REGISTER",
+        "INFO",
+        "REFER",
+        "SUBSCRIBE",
+        "NOTIFY"
 };
 
-static int sip_is_request(tvbuff_t *tvb, guint32 offset);
+static gboolean sip_is_request(tvbuff_t *tvb, guint32 offset);
 static gint sip_get_msg_offset(tvbuff_t *tvb, guint32 offset);
  
 static dissector_handle_t sdp_handle;
+static dissector_handle_t data_handle;
 
 #define SIP2_HDR "SIP/2.0 "
 #define SIP2_HDR_LEN (strlen (SIP2_HDR))
@@ -93,12 +106,12 @@ static void dissect_sip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (tvb_strneql(tvb, 0, SIP2_HDR, SIP2_HDR_LEN) != 0 && ! is_request)
 		goto bad;
 	  
-        if (check_col(pinfo->fd, COL_PROTOCOL)) 
-                col_set_str(pinfo->fd, COL_PROTOCOL, "SIP");
+        if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+                col_set_str(pinfo->cinfo, COL_PROTOCOL, "SIP");
     
 
-        if (check_col(pinfo->fd, COL_INFO))
-                col_add_fstr(pinfo->fd, COL_INFO, "%s: %s",
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_add_fstr(pinfo->cinfo, COL_INFO, "%s: %s",
                              is_request ? "Request" : "Status",
                              is_request ? 
                              tvb_format_text(tvb, 0, eol - SIP2_HDR_LEN) :
@@ -123,7 +136,7 @@ static void dissect_sip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 hdr_tree = proto_item_add_subtree(th, ett_sip_hdr);
 
                 /* - 2 since we have a CRLF separating the message-body */
-                while (msg_offset - 2 > offset) {
+                while (msg_offset - 2 > (int) offset) {
                         eol = tvb_find_line_end(tvb, offset, -1, &next_offset);
                         proto_tree_add_text(hdr_tree, tvb, offset, next_offset - offset, "%s",
                                             tvb_format_text(tvb, offset, eol));
@@ -141,7 +154,7 @@ static void dissect_sip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   bad:
         next_tvb = tvb_new_subset(tvb, offset, -1, -1);
-        dissect_data(next_tvb, 0, pinfo, tree);
+        call_dissector(data_handle,next_tvb, pinfo, tree);
 
         return;
 }
@@ -164,16 +177,16 @@ static gint sip_get_msg_offset(tvbuff_t *tvb, guint32 offset)
         return -1;
 }
                 
-static int sip_is_request(tvbuff_t *tvb, guint32 offset)
+static gboolean sip_is_request(tvbuff_t *tvb, guint32 offset)
 {
-        int i;
+        u_int i;
 
         for (i = 1; i < array_length(sip_methods); i++) {
                 if (tvb_strneql(tvb, offset, sip_methods[i], strlen(sip_methods[i])) == 0)
-                        return i;
+                        return TRUE;
         }
 
-        return 0;
+        return FALSE;
 }
 
 /* Register the protocol with Ethereal */
@@ -208,11 +221,15 @@ void proto_register_sip(void)
 void
 proto_reg_handoff_sip(void)
 {
-        dissector_add("tcp.port", TCP_PORT_SIP, dissect_sip, proto_sip);
-        dissector_add("udp.port", UDP_PORT_SIP, dissect_sip, proto_sip);
+	dissector_handle_t sip_handle;
+
+	sip_handle = create_dissector_handle(dissect_sip, proto_sip);
+        dissector_add("tcp.port", TCP_PORT_SIP, sip_handle);
+        dissector_add("udp.port", UDP_PORT_SIP, sip_handle);
 
 	/*
 	 * Get a handle for the SDP dissector.
 	 */
 	sdp_handle = find_dissector("sdp");
+	data_handle = find_dissector("data");
 }
