@@ -2,10 +2,10 @@
  * Declarations of routines for SMB named pipe packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-smb-pipe.h,v 1.1 2001/03/18 03:23:30 guy Exp $
+ * $Id: packet-smb-pipe.h,v 1.9 2001/11/19 11:41:51 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  * 
  * This program is free software; you can redistribute it and/or
@@ -23,15 +23,12 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-gboolean
-dissect_pipe_lanman(const u_char *pd, int offset, frame_data *fd,
-	proto_tree *parent, proto_tree *tree, struct smb_info si,
-	int max_data, int SMB_offset, int errcode, int dirn,
-	const u_char *command, int DataOffset, int DataCount,
-	int ParameterOffset, int ParameterCount);
+#ifndef _PACKET_SMB_PIPE_H_
+#define _PACKET_SMB_PIPE_H_
 
-gboolean
-dissect_pipe_smb(const u_char *pd, int offset, frame_data *fd,
-    proto_tree *parent, proto_tree *tree, struct smb_info si, int max_data,
-    int SMB_offset, int errcode, int dirn, const u_char *command,
-    int DataOffset, int DataCount, int ParameterOffset, int ParameterCount);
+extern gboolean
+dissect_pipe_smb(tvbuff_t *sp_tvb, tvbuff_t *s_tvb, tvbuff_t *pd_tvb,
+		 tvbuff_t *p_tvb, tvbuff_t *d_tvb, const char *pipe,
+		 packet_info *pinfo, proto_tree *tree);
+
+#endif
