@@ -1,8 +1,8 @@
 /* packet-sna.c
  * Routines for SNA
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-sna.c,v 1.31 2001/07/03 04:56:46 guy Exp $
+ * $Id: packet-sna.c,v 1.36 2001/12/10 00:25:36 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -120,6 +120,8 @@ static gint ett_sna_rh_0 = -1;
 static gint ett_sna_rh_1 = -1;
 static gint ett_sna_rh_2 = -1;
 
+static dissector_handle_t data_handle;
+
 /* Format Identifier */
 static const value_string sna_th_fid_vals[] = {
 	{ 0x0,	"SNA device <--> Non-SNA Device" },
@@ -326,10 +328,10 @@ dissect_sna(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int		sna_header_len = 0, th_header_len = 0;
 	int		offset;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "SNA");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SNA");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	/* SNA data should be printed in EBCDIC, not ASCII */
 	pinfo->fd->flags.encoding = CHAR_EBCDIC;
@@ -338,8 +340,8 @@ dissect_sna(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	th_fid = hi_nibble(tvb_get_guint8(tvb, 0));
 
 	/* Summary information */
-	if (check_col(pinfo->fd, COL_INFO))
-		col_add_str(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_str(pinfo->cinfo, COL_INFO,
 				val_to_str(th_fid, sna_th_fid_vals, "Unknown FID: %01x"));
 
 	if (tree) {
@@ -378,7 +380,7 @@ dissect_sna(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			th_header_len = dissect_fidf(tvb, th_tree);
 			break;
 		default:
-			dissect_data(tvb, 1, pinfo, tree);
+			call_dissector(data_handle,tvb_new_subset(tvb, 1,-1,tvb_reported_length_remaining(tvb,1)), pinfo, tree);
 	}
 
 	sna_header_len += th_header_len;
@@ -402,7 +404,7 @@ dissect_sna(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
 
 	if (tvb_offset_exists(tvb, offset+1)) {
-		dissect_data(tvb, offset, pinfo, tree);
+		call_dissector(data_handle,tvb_new_subset(tvb, offset, -1, tvb_reported_length_remaining(tvb,offset)),pinfo, tree);
 	}
 }
 
@@ -1191,11 +1193,15 @@ proto_register_sna(void)
 	    "SNA", "sna");
 	proto_register_field_array(proto_sna, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+	register_dissector("sna", dissect_sna, proto_sna);
 }
 
 void
 proto_reg_handoff_sna(void)
 {
-	dissector_add("llc.dsap", SAP_SNA_PATHCTRL, dissect_sna,
-	    proto_sna);
+	dissector_handle_t sna_handle;
+
+	sna_handle = find_dissector("sna");
+	dissector_add("llc.dsap", SAP_SNA_PATHCTRL, sna_handle);
+	data_handle = find_dissector("data");
 }
