@@ -8,7 +8,7 @@
  *
  * See RFCs 1905, 1906, 1909, and 1910 for SNMPv2u.
  *
- * $Id: packet-snmp.c,v 1.69 2001/06/18 02:17:52 guy Exp $
+ * $Id: packet-snmp.c,v 1.77 2001/12/12 05:26:53 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -121,6 +121,12 @@
 #  define VALTYPE_OBJECTID	ASN_OBJECT_ID
 #  define VALTYPE_BITSTR	ASN_BIT_STR
 #  define VALTYPE_COUNTER64	ASN_COUNTER64
+
+#  ifdef RED_HAT_MODIFIED_UCD_SNMP
+#    include <ucd-snmp/parse.h>
+#  endif
+
+
 # elif defined(HAVE_SNMP_SNMP_H)
    /*
     * CMU SNMP.
@@ -223,6 +229,9 @@ static int hf_snmpv3_flags_auth = -1;
 static int hf_snmpv3_flags_crypt = -1;
 static int hf_snmpv3_flags_report = -1;
 
+static dissector_handle_t snmp_handle;
+static dissector_handle_t data_handle;
+
 #define TH_AUTH   0x01
 #define TH_CRYPT  0x02
 #define TH_REPORT 0x04
@@ -551,14 +560,14 @@ dissect_snmp_parse_error(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		break;
 	}
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO,
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO,
 		    "ERROR: Couldn't parse %s: %s", field_name, errstr);
 	}
 	if (tree != NULL) {
 		proto_tree_add_text(tree, tvb, offset, 0,
 		    "ERROR: Couldn't parse %s: %s", field_name, errstr);
-		dissect_data(tvb, offset, pinfo, tree);
+		call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
 	}
 }
 
@@ -566,12 +575,12 @@ static void
 dissect_snmp_error(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		   proto_tree *tree, const char *message)
 {
-	if (check_col(pinfo->fd, COL_INFO))
-		col_add_str(pinfo->fd, COL_INFO, message);
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_str(pinfo->cinfo, COL_INFO, message);
 
 	if (tree != NULL) {
 		proto_tree_add_text(tree, tvb, offset, 0, "%s", message);
-		dissect_data(tvb, offset, pinfo, tree);
+		call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
 	}
 }
 
@@ -580,7 +589,8 @@ format_oid(subid_t *oid, guint oid_length)
 {
 	char *result;
 	int result_len;
-	int len, i;
+	int len;
+	unsigned int i;
 	char *buf;
 
 	result_len = oid_length * 22;
@@ -691,7 +701,12 @@ format_var(struct variable_list *variable, subid_t *variable_oid,
 	}
 	variable->val_len = vb_length;
 
+# ifdef RED_HAT_MODIFIED_UCD_SNMP
+	sprint_value(binit(NULL, buf, sizeof(buf)), variable_oid,
+	    variable_oid_length, variable);
+# else
 	sprint_value(buf, variable_oid, variable_oid_length, variable);
+# endif
 	return buf;
 }
 #endif
@@ -726,7 +741,7 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 	long value;
 #endif
 #endif	/* HAVE_SPRINT_VALUE */
-	int i;
+	unsigned int i;
 	gchar *buf;
 	int len;
 
@@ -996,8 +1011,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 	pdu_type_string = val_to_str(pdu_type, pdu_types,
 	    "Unknown PDU type %#x");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_add_str(pinfo->fd, COL_INFO, pdu_type_string);
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 	length = asn1.offset - start;
 	if (tree) {
 		proto_tree_add_text(tree, tvb, offset, length,
@@ -1236,8 +1251,13 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			    variable_oid_length);
 			
 #if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+# ifdef RED_HAT_MODIFIED_UCD_SNMP
+			sprint_objid(binit(NULL, vb_oid_string, sizeof(vb_oid_string)), 
+			    variable_oid, variable_oid_length);
+# else
 			sprint_objid(vb_oid_string, variable_oid,
 			    variable_oid_length);
+# endif
 			proto_tree_add_text(tree, tvb, offset, sequence_length,
 			    "Object identifier %d: %s (%s)", vb_index,
 			    oid_string, vb_oid_string);
@@ -1472,8 +1492,8 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	int ret;
 	guint cls, con, tag;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_add_str(pinfo->fd, COL_PROTOCOL, proto_name);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_add_str(pinfo->cinfo, COL_PROTOCOL, proto_name);
 
 	if (tree) {
 		item = proto_tree_add_item(tree, proto, tvb, offset,
@@ -1762,8 +1782,8 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			proto_tree_add_text(snmp_tree, tvb, offset, length,
 			    "Encrypted PDU (%d bytes)", length);
 			g_free(cryptpdu);
-			if (check_col(pinfo->fd, COL_INFO))
-				col_set_str(pinfo->fd, COL_INFO, "Encrypted PDU");
+			if (check_col(pinfo->cinfo, COL_INFO))
+				col_set_str(pinfo->cinfo, COL_INFO, "Encrypted PDU");
 			return;
 		}
 		ret = asn1_sequence_decode(&asn1, &global_length, &length);
@@ -1859,8 +1879,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	int ret;
 	guint cls, con;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "SMUX");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SMUX");
 
 	if (tree) {
 		item = proto_tree_add_item(tree, proto, tvb, offset,
@@ -1887,8 +1907,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (cls == ASN1_APL && con == ASN1_CON && pdu_type == SMUX_MSG_OPEN) {
 		pdu_type_string = val_to_str(pdu_type, smux_types,
 		    "Unknown PDU type %#x");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_str(pinfo->fd, COL_INFO, pdu_type_string);
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
@@ -1956,8 +1976,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (cls == ASN1_APL && con == ASN1_PRI && pdu_type == SMUX_MSG_CLOSE) {
 		pdu_type_string = val_to_str(pdu_type, smux_types,
 		    "Unknown PDU type %#x");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_str(pinfo->fd, COL_INFO, pdu_type_string);
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
@@ -1982,8 +2002,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (cls == ASN1_APL && con == ASN1_CON && pdu_type == SMUX_MSG_RREQ) {
 		pdu_type_string = val_to_str(pdu_type, smux_types,
 		    "Unknown PDU type %#x");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_str(pinfo->fd, COL_INFO, pdu_type_string);
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
@@ -2035,8 +2055,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (cls == ASN1_APL && con == ASN1_PRI && pdu_type == SMUX_MSG_RRSP) {
 		pdu_type_string = val_to_str(pdu_type, smux_types,
 		    "Unknown PDU type %#x");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_str(pinfo->fd, COL_INFO, pdu_type_string);
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
@@ -2061,8 +2081,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (cls == ASN1_APL && con == ASN1_PRI && pdu_type == SMUX_MSG_SOUT) {
 		pdu_type_string = val_to_str(pdu_type, smux_types,
 		    "Unknown PDU type %#x");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_str(pinfo->fd, COL_INFO, pdu_type_string);
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
@@ -2097,15 +2117,6 @@ dissect_snmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	conversation_t  *conversation;
 
-	/*
-	 * XXX - this is a conversation dissector, and the code to
-	 * call a conversation dissector doesn't check for disabled
-	 * protocols or set "pinfo->current_proto".
-	 */
-	CHECK_DISPLAY_AS_DATA(proto_snmp, tvb, pinfo, tree);
-
-	pinfo->current_proto = "SNMP";
-
 	/*
 	 * The first SNMP packet goes to the SNMP port; the second one
 	 * may come from some *other* port, but goes back to the same
@@ -2129,9 +2140,8 @@ dissect_snmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					   pinfo->srcport, 0, NO_PORT_B);
 	  if (conversation == NULL) {
 	    conversation = conversation_new(&pinfo->src, &pinfo->dst, PT_UDP,
-					    pinfo->srcport, 0, NULL,
-					    NO_PORT2);
-	    conversation_set_dissector(conversation, dissect_snmp);
+					    pinfo->srcport, 0, NO_PORT2);
+	    conversation_set_dissector(conversation, snmp_handle);
 	  }
 	}
 
@@ -2296,17 +2306,20 @@ proto_register_snmp(void)
 	    "SMUX", "smux");
         proto_register_field_array(proto_snmp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+	snmp_handle = create_dissector_handle(dissect_snmp, proto_snmp);
 }
 
 void
 proto_reg_handoff_snmp(void)
 {
-	dissector_add("udp.port", UDP_PORT_SNMP, dissect_snmp, proto_snmp);
-	dissector_add("udp.port", UDP_PORT_SNMP_TRAP, dissect_snmp, proto_snmp);
-	dissector_add("tcp.port", TCP_PORT_SMUX, dissect_smux, proto_smux);
-	dissector_add("ethertype", ETHERTYPE_SNMP, dissect_snmp, proto_snmp);
-	dissector_add("ipx.socket", IPX_SOCKET_SNMP_AGENT, dissect_snmp,
-	    proto_snmp);
-	dissector_add("ipx.socket", IPX_SOCKET_SNMP_SINK, dissect_snmp,
-	    proto_snmp);
+	dissector_handle_t smux_handle;
+
+	dissector_add("udp.port", UDP_PORT_SNMP, snmp_handle);
+	dissector_add("udp.port", UDP_PORT_SNMP_TRAP, snmp_handle);
+	smux_handle = create_dissector_handle(dissect_smux, proto_smux);
+	dissector_add("tcp.port", TCP_PORT_SMUX, smux_handle);
+	dissector_add("ethertype", ETHERTYPE_SNMP, snmp_handle);
+	dissector_add("ipx.socket", IPX_SOCKET_SNMP_AGENT, snmp_handle);
+	dissector_add("ipx.socket", IPX_SOCKET_SNMP_SINK, snmp_handle);
+	data_handle = find_dissector("data");
 }
