@@ -2,7 +2,7 @@
  * Routines for ssl dissection
  * Copyright (c) 2000-2001, Scott Renfro <scott@renfro.org>
  *
- * $Id: packet-ssl.c,v 1.1 2001/07/11 18:19:15 guy Exp $
+ * $Id: packet-ssl.c,v 1.11 2001/12/10 00:25:36 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -96,19 +96,25 @@ static int hf_ssl_handshake_client_version   = -1;
 static int hf_ssl_handshake_server_version   = -1;
 static int hf_ssl_handshake_random_time      = -1;
 static int hf_ssl_handshake_random_bytes     = -1;
+static int hf_ssl_handshake_cipher_suites_len = -1;
 static int hf_ssl_handshake_cipher_suites    = -1;
 static int hf_ssl_handshake_cipher_suite     = -1;
 static int hf_ssl_handshake_session_id       = -1;
+static int hf_ssl_handshake_comp_methods_len = -1;
 static int hf_ssl_handshake_comp_methods     = -1;
 static int hf_ssl_handshake_comp_method      = -1;
+static int hf_ssl_handshake_certificates_len = -1;
+static int hf_ssl_handshake_certificates     = -1;
 static int hf_ssl_handshake_certificate      = -1;
 static int hf_ssl_handshake_certificate_len  = -1;
+static int hf_ssl_handshake_cert_types_count = -1;
 static int hf_ssl_handshake_cert_types       = -1;
 static int hf_ssl_handshake_cert_type        = -1;
 static int hf_ssl_handshake_finished         = -1;
 static int hf_ssl_handshake_md5_hash         = -1;
 static int hf_ssl_handshake_sha_hash         = -1;
 static int hf_ssl_handshake_session_id_len   = -1;
+static int hf_ssl_handshake_dnames_len       = -1;
 static int hf_ssl_handshake_dnames           = -1;
 static int hf_ssl_handshake_dname_len        = -1;
 static int hf_ssl_handshake_dname            = -1;
@@ -244,6 +250,12 @@ static const value_string ssl_20_cipher_suites[] = {
     { 0x000064, "TLS_RSA_EXPORT1024_WITH_RC4_56_SHA" },
     { 0x000065, "TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA" },
     { 0x000066, "TLS_DHE_DSS_WITH_RC4_128_SHA" },
+    /* these from http://www.mozilla.org/projects/
+         security/pki/nss/ssl/fips-ssl-ciphersuites.html */
+    { 0x00fefe, "SSL_RSA_FIPS_WITH_DES_CBC_SHA"},
+    { 0x00feff, "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA" },
+    { 0x00ffe0, "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA" },
+    { 0x00ffe1, "SSL_RSA_FIPS_WITH_DES_CBC_SHA"},
     /* note that ciphersuites of {0x00????} are TLS cipher suites in
      * a sslv2 client hello message; the ???? above is the two-byte
      * tls cipher suite id
@@ -393,6 +405,12 @@ static const value_string ssl_31_ciphersuite[] = {
     { 0x0064, "TLS_RSA_EXPORT1024_WITH_RC4_56_SHA" },
     { 0x0065, "TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA" },
     { 0x0066, "TLS_DHE_DSS_WITH_RC4_128_SHA" },
+    /* these from http://www.mozilla.org/projects/
+         security/pki/nss/ssl/fips-ssl-ciphersuites.html */
+    { 0xfefe, "SSL_RSA_FIPS_WITH_DES_CBC_SHA"},
+    { 0xfeff, "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA" },
+    { 0xffe0, "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA" },
+    { 0xffe1, "SSL_RSA_FIPS_WITH_DES_CBC_SHA"},
     /* note that ciphersuites 0xff00 - 0xffff are private */
     { 0x00, NULL }
 };
@@ -508,6 +526,7 @@ dissect_ssl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 
     conversation_t *conversation;
+    void *conv_data;
     guint conv_version     = SSL_VER_UNKNOWN;
     proto_item *ti         = NULL;
     proto_tree *ssl_tree   = NULL;
@@ -532,25 +551,25 @@ dissect_ssl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     {
         /* create a new conversation */
         conversation = conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype,
-                                        pinfo->srcport, pinfo->destport,
-                                        (void*)SSL_VER_UNKNOWN, 0);
+                                        pinfo->srcport, pinfo->destport, 0);
     }
-    if (conversation)
+    conv_data = conversation_get_proto_data(conversation, proto_ssl);
+    if (conv_data != NULL)
     {
-        conv_version = (guint)conversation->data;
+        conv_version = (guint)conv_data;
     }
 
     /* Initialize the protocol column; we'll set it later when we
      * figure out what flavor of SSL it is (assuming we don't
      * throw an exception before we get the chance to do so). */
-    if (check_col(pinfo->fd, COL_PROTOCOL))
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
     {
-        col_set_str(pinfo->fd, COL_PROTOCOL, "SSL");
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "SSL");
     }
 
     /* clear the the info column */
-    if (check_col(pinfo->fd, COL_INFO))
-        col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_INFO))
+        col_clear(pinfo->cinfo, COL_INFO);
 
     /* TCP packets and SSL records are orthogonal.
      * A tcp packet may contain multiple ssl records and an ssl
@@ -581,9 +600,9 @@ dissect_ssl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
          * add a delimiter on info column
          */
         if (!first_record_in_frame
-            && check_col(pinfo->fd, COL_INFO))
+            && check_col(pinfo->cinfo, COL_INFO))
         {
-            col_append_str(pinfo->fd, COL_INFO, ", ");
+            col_append_str(pinfo->cinfo, COL_INFO, ", ");
         }
 
         /* first try to dispatch off the cached version
@@ -637,22 +656,29 @@ dissect_ssl(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                  * continuation data
                  */
                 offset = tvb_length(tvb);
-                if (check_col(pinfo->fd, COL_INFO))
-                    col_append_str(pinfo->fd, COL_INFO,
+                if (check_col(pinfo->cinfo, COL_INFO))
+                    col_append_str(pinfo->cinfo, COL_INFO,
                                    "Continuation Data");
 
                 /* Set the protocol column */
-                if (check_col(pinfo->fd, COL_PROTOCOL))
+                if (check_col(pinfo->cinfo, COL_PROTOCOL))
                 {
-                    col_set_str(pinfo->fd, COL_PROTOCOL,
+                    col_set_str(pinfo->cinfo, COL_PROTOCOL,
                          ssl_version_short_names[conv_version]);
                 }
             }
             break;
         }
 
+        /* If we haven't already set the version information for
+         * this conversation, do so. */
+        if (conv_data == NULL)
+        {
+            conv_data = (void *)conv_version;
+            conversation_add_proto_data(conversation, proto_ssl, conv_data);
+        }
+
         /* set up for next record in frame, if any */
-        conversation->data = (void*)conv_version;
         first_record_in_frame = FALSE;
     }
 
@@ -707,13 +733,13 @@ dissect_ssl3_record(tvbuff_t *tvb, packet_info *pinfo,
      */
     if (!ssl_is_valid_content_type(content_type))
     {
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, "Continuation Data");
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, "Continuation Data");
 
         /* Set the protocol column */
-        if (check_col(pinfo->fd, COL_PROTOCOL))
+        if (check_col(pinfo->cinfo, COL_PROTOCOL))
         {
-            col_set_str(pinfo->fd, COL_PROTOCOL,
+            col_set_str(pinfo->cinfo, COL_PROTOCOL,
                         ssl_version_short_names[*conv_version]);
         }
         return offset + 5 + record_length;
@@ -776,21 +802,21 @@ dissect_ssl3_record(tvbuff_t *tvb, packet_info *pinfo,
             ssl_set_conv_version(pinfo, *conv_version);
         }
     }
-    if (check_col(pinfo->fd, COL_PROTOCOL))
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
     {
         if (version == 0x0300)
         {
-            col_set_str(pinfo->fd, COL_PROTOCOL,
+            col_set_str(pinfo->cinfo, COL_PROTOCOL,
                         ssl_version_short_names[SSL_VER_SSLv3]);
         }
         else if (version == 0x0301)
         {
-            col_set_str(pinfo->fd, COL_PROTOCOL,
+            col_set_str(pinfo->cinfo, COL_PROTOCOL,
                         ssl_version_short_names[SSL_VER_TLS]);
         }
         else
         {
-            col_set_str(pinfo->fd, COL_PROTOCOL,
+            col_set_str(pinfo->cinfo, COL_PROTOCOL,
                         ssl_version_short_names[*conv_version]);
         }
     }
@@ -800,8 +826,8 @@ dissect_ssl3_record(tvbuff_t *tvb, packet_info *pinfo,
      */
     switch (content_type) {
     case SSL_ID_CHG_CIPHER_SPEC:
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, "Change Cipher Spec");
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, "Change Cipher Spec");
         dissect_ssl3_change_cipher_spec(tvb, pinfo, ssl_record_tree,
                                         offset, conv_version);
         break;
@@ -814,8 +840,8 @@ dissect_ssl3_record(tvbuff_t *tvb, packet_info *pinfo,
                                record_length, conv_version);
         break;
     case SSL_ID_APP_DATA:
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, "Application Data");
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, "Application Data");
         if (ssl_record_tree)
         {
             proto_item_set_text(ssl_record_tree,
@@ -828,8 +854,8 @@ dissect_ssl3_record(tvbuff_t *tvb, packet_info *pinfo,
 
     default:
         /* shouldn't get here since we check above for valid types */
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, "Bad SSLv3 Content Type");
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, "Bad SSLv3 Content Type");
         break;
     }
     offset += record_length; /* skip to end of record */
@@ -896,15 +922,15 @@ dissect_ssl3_alert(tvbuff_t *tvb, packet_info *pinfo,
     /* now set the text in the record layer line */
     if (level && desc)
     {
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_fstr(pinfo->fd, COL_INFO,
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_fstr(pinfo->cinfo, COL_INFO,
                             "Alert (Level: %s, Description: %s)",
                             level, desc);
     }
     else
     {
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, "Encrypted Alert");
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, "Encrypted Alert");
     }
 
     if (tree)
@@ -991,15 +1017,15 @@ dissect_ssl3_handshake(tvbuff_t *tvb, packet_info *pinfo,
         /* on second and later iterations, add comma to info col */
         if (!first_iteration)
         {
-            if (check_col(pinfo->fd, COL_INFO))
-                col_append_fstr(pinfo->fd, COL_INFO, ", ");
+            if (check_col(pinfo->cinfo, COL_INFO))
+                col_append_fstr(pinfo->cinfo, COL_INFO, ", ");
         }
 
         /*
          * Update our info string
          */
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_fstr(pinfo->fd, COL_INFO, "%s", (msg_type_str != NULL)
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_fstr(pinfo->cinfo, COL_INFO, "%s", (msg_type_str != NULL)
                             ? msg_type_str : "Encrypted Handshake Message");
 
         if (tree)
@@ -1103,14 +1129,14 @@ dissect_ssl3_hnd_hello_common(tvbuff_t *tvb, proto_tree *tree,
 {
     /* show the client's random challenge */
     guint32 initial_offset = offset;
-    struct timeval gmt_unix_time;
+    nstime_t gmt_unix_time;
     guint8  session_id_length = 0;
 
     if (tree)
     {
         /* show the time */
-        gmt_unix_time.tv_sec = tvb_get_ntohl(tvb, offset);
-        gmt_unix_time.tv_usec = 0;
+        gmt_unix_time.secs = tvb_get_ntohl(tvb, offset);
+        gmt_unix_time.nsecs = 0;
         proto_tree_add_time(tree, hf_ssl_handshake_random_time,
                                      tvb, offset, 4, &gmt_unix_time);
         offset += 4;
@@ -1126,8 +1152,9 @@ dissect_ssl3_hnd_hello_common(tvbuff_t *tvb, proto_tree *tree,
                             tvb, offset++, 1, 0);
         if (session_id_length > 0)
         {
-            proto_tree_add_string_format(tree, hf_ssl_handshake_session_id,
-                                         tvb, offset, session_id_length, NULL,
+            proto_tree_add_bytes_format(tree, hf_ssl_handshake_session_id,
+                                         tvb, offset, session_id_length,
+                                         tvb_get_ptr(tvb, offset, session_id_length),
                                          "Session ID (%u byte%s)",
                                          session_id_length,
                                          plurality(session_id_length, "", "s"));
@@ -1168,18 +1195,18 @@ dissect_ssl3_hnd_cli_hello(tvbuff_t *tvb, packet_info *pinfo,
 
         /* tell the user how many cipher suites there are */
         cipher_suite_length = tvb_get_ntohs(tvb, offset);
+        proto_tree_add_uint(tree, hf_ssl_handshake_cipher_suites_len,
+                            tvb, offset, 2, cipher_suite_length);
         offset += 2;            /* skip opaque length */
 
         if (cipher_suite_length > 0)
         {
-            ti = proto_tree_add_string_format(tree,
-                                              hf_ssl_handshake_cipher_suites,
-                                              tvb, offset-2, 2,
-                                              NULL,
-                                              "Cipher Suites (%u suite%s)",
-                                              cipher_suite_length / 2,
-                                              plurality(cipher_suite_length/2,
-                                                "", "s"));
+            ti = proto_tree_add_none_format(tree,
+                                            hf_ssl_handshake_cipher_suites,
+                                            tvb, offset, cipher_suite_length,
+                                            "Cipher Suites (%u suite%s)",
+                                            cipher_suite_length / 2,
+                                            plurality(cipher_suite_length/2, "", "s"));
 
             /* make this a subtree */
             cs_tree = proto_item_add_subtree(ti, ett_ssl_cipher_suites);
@@ -1198,17 +1225,20 @@ dissect_ssl3_hnd_cli_hello(tvbuff_t *tvb, packet_info *pinfo,
         }
 
         /* tell the user how man compression methods there are */
-        compression_methods_length = tvb_get_guint8(tvb, offset++);
+        compression_methods_length = tvb_get_guint8(tvb, offset);
+        proto_tree_add_uint(tree, hf_ssl_handshake_comp_methods_len,
+                            tvb, offset, 1, compression_methods_length);
+        offset++;
 
         if (compression_methods_length > 0)
         {
-            ti = proto_tree_add_string_format(tree,
-                                             hf_ssl_handshake_comp_methods,
-                                             tvb, offset-1, 1, NULL,
-                                             "Compression Methods (%u method%s)",
-                                             compression_methods_length,
-                                             plurality(compression_methods_length,
-                                               "", "s"));
+            ti = proto_tree_add_none_format(tree,
+                                            hf_ssl_handshake_comp_methods,
+                                            tvb, offset, compression_methods_length,
+                                            "Compression Methods (%u method%s)",
+                                            compression_methods_length,
+                                            plurality(compression_methods_length,
+                                              "", "s"));
 
             /* make this a subtree */
             cs_tree = proto_item_add_subtree(ti, ett_ssl_comp_methods);
@@ -1282,19 +1312,19 @@ dissect_ssl3_hnd_cert(tvbuff_t *tvb, packet_info *pinfo,
     if (tree)
     {
         certificate_list_length = tvb_get_ntoh24(tvb, offset);
+        proto_tree_add_uint(tree, hf_ssl_handshake_certificates_len,
+                            tvb, offset, 3, certificate_list_length);
         offset += 3;            /* 24-bit length value */
 
         if (certificate_list_length > 0)
         {
-            ti = proto_tree_add_string_format(tree,
-                                              hf_ssl_handshake_certificate,
-                                              tvb, offset-3,
-                                              3 + certificate_list_length,
-                                              NULL,
-                                              "Certificates (%u byte%s)",
-                                              certificate_list_length,
-                                              plurality(certificate_list_length,
-                                                "", "s"));
+            ti = proto_tree_add_none_format(tree,
+                                            hf_ssl_handshake_certificates,
+                                            tvb, offset, certificate_list_length,
+                                            "Certificates (%u byte%s)",
+                                            certificate_list_length,
+                                            plurality(certificate_list_length,
+                                              "", "s"));
 
             /* make it a subtree */
             subtree = proto_item_add_subtree(ti, ett_ssl_certs);
@@ -1314,12 +1344,13 @@ dissect_ssl3_hnd_cert(tvbuff_t *tvb, packet_info *pinfo,
                                     tvb, offset, 3, FALSE);
                 offset += 3;
 
-                proto_tree_add_string_format(subtree,
-                                             hf_ssl_handshake_certificate,
-                                             tvb, offset, cert_length, NULL,
-                                             "Certificate (%u byte%s)",
-                                             cert_length,
-                                             plurality(cert_length, "", "s"));
+                proto_tree_add_bytes_format(subtree,
+                                            hf_ssl_handshake_certificate,
+                                            tvb, offset, cert_length,
+                                            tvb_get_ptr(tvb, offset, cert_length),
+                                            "Certificate (%u byte%s)",
+                                            cert_length,
+                                            plurality(cert_length, "", "s"));
                 offset += cert_length;
             }
         }
@@ -1353,16 +1384,18 @@ dissect_ssl3_hnd_cert_req(tvbuff_t *tvb, packet_info *pinfo,
     if (tree)
     {
         cert_types_count = tvb_get_guint8(tvb, offset);
+        proto_tree_add_uint(tree, hf_ssl_handshake_cert_types_count,
+                            tvb, offset, 1, cert_types_count);
         offset++;
 
         if (cert_types_count > 0)
         {
-            ti = proto_tree_add_string_format(tree,
-                                              hf_ssl_handshake_cert_types,
-                                              tvb, offset - 1, 1, NULL,
-                                              "Certificate types (%u type%s)",
-                                              cert_types_count,
-                                              plurality(cert_types_count, "", "s"));
+            ti = proto_tree_add_none_format(tree,
+                                            hf_ssl_handshake_cert_types,
+                                            tvb, offset, cert_types_count,
+                                            "Certificate types (%u type%s)",
+                                            cert_types_count,
+                                            plurality(cert_types_count, "", "s"));
             subtree = proto_item_add_subtree(ti, ett_ssl_cert_types);
             if (!subtree)
             {
@@ -1379,18 +1412,18 @@ dissect_ssl3_hnd_cert_req(tvbuff_t *tvb, packet_info *pinfo,
         }
 
         dnames_length = tvb_get_ntohs(tvb, offset);
+        proto_tree_add_uint(tree, hf_ssl_handshake_dnames_len,
+                            tvb, offset, 2, dnames_length);
         offset += 2;
 
         if (dnames_length > 0)
         {
-            ti = proto_tree_add_string_format(tree,
-                                              hf_ssl_handshake_dnames,
-                                              tvb, offset - 2,
-                                              dnames_length +2,
-                                              NULL,
-                                              "Distinguished Names (%d byte%s)",
-                                              dnames_length,
-                                              plurality(dnames_length, "", "s"));
+            ti = proto_tree_add_none_format(tree,
+                                            hf_ssl_handshake_dnames,
+                                            tvb, offset, dnames_length,
+                                            "Distinguished Names (%d byte%s)",
+                                            dnames_length,
+                                            plurality(dnames_length, "", "s"));
             subtree = proto_item_add_subtree(ti, ett_ssl_dnames);
             if (!subtree)
             {
@@ -1407,12 +1440,13 @@ dissect_ssl3_hnd_cert_req(tvbuff_t *tvb, packet_info *pinfo,
                                     tvb, offset, 2, FALSE);
                 offset += 2;
 
-                proto_tree_add_string_format(subtree,
-                                             hf_ssl_handshake_dname,
-                                             tvb, offset, name_length, NULL,
-                                             "Distinguished Name (%u byte%s)",
-                                             name_length,
-                                             plurality(name_length, "", "s"));
+                proto_tree_add_bytes_format(subtree,
+                                            hf_ssl_handshake_dname,
+                                            tvb, offset, name_length,
+                                            tvb_get_ptr(tvb, offset, name_length),
+                                            "Distinguished Name (%u byte%s)",
+                                            name_length,
+                                            plurality(name_length, "", "s"));
                 offset += name_length;
             }
         }
@@ -1488,9 +1522,9 @@ dissect_ssl2_record(tvbuff_t *tvb, packet_info *pinfo, proto_tree
      * conversation, then set a version for just this frame
      * (e.g., on a client hello)
      */
-    if (check_col(pinfo->fd, COL_PROTOCOL))
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
     {
-        col_set_str(pinfo->fd, COL_PROTOCOL, "SSLv2");
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "SSLv2");
     }
 
     /* pull first byte; if high bit is set, then record
@@ -1548,14 +1582,14 @@ dissect_ssl2_record(tvbuff_t *tvb, packet_info *pinfo, proto_tree
             proto_item_set_text(ssl_record_tree, "SSLv2 Record Layer: %s",
                                 "Encrypted Data");
         }
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, "Encrypted Data");
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, "Encrypted Data");
         return initial_offset + record_length_length + record_length;
     }
     else
     {
-        if (check_col(pinfo->fd, COL_INFO))
-            col_append_str(pinfo->fd, COL_INFO, msg_type_str);
+        if (check_col(pinfo->cinfo, COL_INFO))
+            col_append_str(pinfo->cinfo, COL_INFO, msg_type_str);
 
         if (ssl_record_tree)
         {
@@ -1692,10 +1726,10 @@ dissect_ssl2_hnd_client_hello(tvbuff_t *tvb, packet_info *pinfo,
         offset += 2;
 
         /* tell the user how many cipher specs they've won */
-        ti = proto_tree_add_string_format(tree, hf_ssl_handshake_cipher_suites,
-                                          tvb, offset, cipher_spec_length,
-                                          NULL, "Cipher Specs (%u specs)",
-                                          cipher_spec_length/3);
+        ti = proto_tree_add_none_format(tree, hf_ssl_handshake_cipher_suites,
+                                        tvb, offset, cipher_spec_length,
+                                        "Cipher Specs (%u specs)",
+                                        cipher_spec_length/3);
 
         /* make this a subtree and expand the actual specs below */
         cs_tree = proto_item_add_subtree(ti, ett_ssl_cipher_suites);
@@ -1716,10 +1750,11 @@ dissect_ssl2_hnd_client_hello(tvbuff_t *tvb, packet_info *pinfo,
         /* if there's a session id, show it */
         if (session_id_length > 0)
         {
-            proto_tree_add_string_format(tree,
+            proto_tree_add_bytes_format(tree,
                                          hf_ssl_handshake_session_id,
                                          tvb, offset, session_id_length,
-                                         NULL, "Session ID (%u byte%s)",
+                                         tvb_get_ptr(tvb, offset, session_id_length),
+                                         "Session ID (%u byte%s)",
                                          session_id_length,
                                          plurality(session_id_length, "", "s"));
 
@@ -1886,22 +1921,24 @@ dissect_ssl2_hnd_server_hello(tvbuff_t *tvb, packet_info *pinfo,
     /* now the variable length fields */
     if (certificate_length > 0)
     {
-        proto_tree_add_string_format(tree, hf_ssl_handshake_certificate,
-                                     tvb, offset, certificate_length,
-                                     NULL, "Certificate (%u byte%s)",
-                                     certificate_length,
-                                     plurality(certificate_length, "", "s"));
+        proto_tree_add_bytes_format(tree, hf_ssl_handshake_certificate,
+                                    tvb, offset, certificate_length,
+                                    tvb_get_ptr(tvb, offset, certificate_length),
+                                    "Certificate (%u byte%s)",
+                                    certificate_length,
+                                    plurality(certificate_length, "", "s"));
         offset += certificate_length;
     }
 
     if (cipher_spec_length > 0)
     {
         /* provide a collapsing node for the cipher specs */
-        ti = proto_tree_add_string_format(tree,
-                                          hf_ssl_handshake_cipher_suites,
-                                          tvb, offset, cipher_spec_length,
-                                          NULL, "Cipher Specs (%u specs)",
-                                          cipher_spec_length/3);
+        ti = proto_tree_add_none_format(tree, 
+                                        hf_ssl_handshake_cipher_suites,
+                                        tvb, offset, cipher_spec_length,
+                                        "Cipher Specs (%u spec%s)",
+                                        cipher_spec_length/3,
+                                        plurality(cipher_spec_length/3, "", "s"));
         subtree = proto_item_add_subtree(ti, ett_ssl_cipher_suites);
         if (!subtree)
         {
@@ -1940,19 +1977,31 @@ static void
 ssl_set_conv_version(packet_info *pinfo, guint version)
 {
     conversation_t *conversation;
+
+    if (pinfo->fd->flags.visited)
+    {
+        /* We've already processed this frame; no need to do any more
+         * work on it.
+         */
+        return;
+    }
+
     conversation = find_conversation(&pinfo->src, &pinfo->dst, pinfo->ptype,
                                      pinfo->srcport, pinfo->destport, 0);
 
-    if (conversation)
-    {
-        conversation->data = (void*)version;
-    }
-    else
+    if (conversation == NULL)
     {
         /* create a new conversation */
-        conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype,
-                         pinfo->srcport, pinfo->destport, (void*)version, 0);
+        conversation = conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype,
+                                        pinfo->srcport, pinfo->destport, 0);
     }
+
+    if (conversation_get_proto_data(conversation, proto_ssl) != NULL)
+    {
+        /* get rid of the current data */
+        conversation_delete_proto_data(conversation, proto_ssl);
+    }
+    conversation_add_proto_data(conversation, proto_ssl, (void *)version);
 }
 
 static int
@@ -2205,7 +2254,7 @@ proto_register_ssl(void)
         { & hf_ssl2_record,
           { "SSLv2 Record Header", "ssl.record",
             FT_NONE, BASE_DEC, NULL, 0x0,
-            "Length of SSLv2 record data", HFILL }
+            "SSLv2 record data", HFILL }
         },
         { &hf_ssl2_record_is_escape,
           { "Is Escape", "ssl.record.is_escape",
@@ -2272,9 +2321,14 @@ proto_register_ssl(void)
             FT_NONE, BASE_NONE, NULL, 0x0,
             "Random challenge used to authenticate server", HFILL }
         },
+        { &hf_ssl_handshake_cipher_suites_len,
+          { "Cipher Suites Length", "ssl.handshake.cipher_suites_length",
+            FT_UINT16, BASE_DEC, NULL, 0x0,
+            "Length of cipher suites field", HFILL }
+        },
         { &hf_ssl_handshake_cipher_suites,
-          { "Cipher Suites", "ssl.handshake.cipherspecs",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+          { "Cipher Suites", "ssl.handshake.ciphersuites",
+            FT_NONE, BASE_NONE, NULL, 0x0,
             "List of cipher suites supported by client", HFILL }
         },
         { &hf_ssl_handshake_cipher_suite,
@@ -2289,12 +2343,17 @@ proto_register_ssl(void)
         },
         { &hf_ssl_handshake_session_id,
           { "Session ID", "ssl.handshake.session_id",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+            FT_BYTES, BASE_NONE, NULL, 0x0,
             "Identifies the SSL session, allowing later resumption", HFILL }
         },
+        { &hf_ssl_handshake_comp_methods_len,
+          { "Compression Methods Length", "ssl.handshake.comp_methods_length",
+            FT_UINT8, BASE_DEC, NULL, 0x0,
+            "Length of compression methods field", HFILL }
+        },
         { &hf_ssl_handshake_comp_methods,
           { "Compression Methods", "ssl.handshake.comp_methods",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+            FT_NONE, BASE_NONE, NULL, 0x0,
             "List of compression methods supported by client", HFILL }
         },
         { &hf_ssl_handshake_comp_method,
@@ -2302,9 +2361,19 @@ proto_register_ssl(void)
             FT_UINT8, BASE_DEC, VALS(ssl_31_compression_method), 0x0,
             "Compression Method", HFILL }
         },
+        { &hf_ssl_handshake_certificates_len,
+          { "Certificates Length", "ssl.handshake.certificates_length",
+            FT_UINT24, BASE_DEC, NULL, 0x0,
+            "Length of certificates field", HFILL }
+        },
+        { &hf_ssl_handshake_certificates,
+          { "Certificates", "ssl.handshake.certificates",
+            FT_NONE, BASE_NONE, NULL, 0x0,
+            "List of certificates", HFILL }
+        },
         { &hf_ssl_handshake_certificate,
           { "Certificate", "ssl.handshake.certificate",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+            FT_BYTES, BASE_NONE, NULL, 0x0,
             "Certificate", HFILL }
         },
         { &hf_ssl_handshake_certificate_len,
@@ -2312,9 +2381,14 @@ proto_register_ssl(void)
             FT_UINT24, BASE_DEC, NULL, 0x0,
             "Length of certificate", HFILL }
         },
+        { &hf_ssl_handshake_cert_types_count,
+          { "Certificate types count", "ssl.handshake.cert_types_count",
+            FT_UINT8, BASE_DEC, NULL, 0x0,
+            "Count of certificate types", HFILL }
+        },
         { &hf_ssl_handshake_cert_types,
           { "Certificate types", "ssl.handshake.cert_types",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+            FT_NONE, BASE_NONE, NULL, 0x0,
             "List of certificate types", HFILL }
         },
         { &hf_ssl_handshake_cert_type,
@@ -2342,9 +2416,14 @@ proto_register_ssl(void)
             FT_UINT8, BASE_DEC, NULL, 0x0,
             "Length of session ID field", HFILL }
         },
+        { &hf_ssl_handshake_dnames_len,
+          { "Distinguished Names Length", "ssl.handshake.dnames_len",
+            FT_UINT16, BASE_DEC, NULL, 0x0,
+            "Length of list of CAs that server trusts", HFILL }
+        },
         { &hf_ssl_handshake_dnames,
           { "Distinguished Names", "ssl.handshake.dnames",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+            FT_NONE, BASE_NONE, NULL, 0x0,
             "List of CAs that server trusts", HFILL }
         },
         { &hf_ssl_handshake_dname_len,
@@ -2353,8 +2432,8 @@ proto_register_ssl(void)
             "Length of distinguished name", HFILL }
         },
         { &hf_ssl_handshake_dname,
-          { "Distinguished Name", "ss.handshake.dname",
-            FT_STRING, BASE_NONE, NULL, 0x0,
+          { "Distinguished Name", "ssl.handshake.dname",
+            FT_BYTES, BASE_NONE, NULL, 0x0,
             "Distinguished name of a CA that server trusts", HFILL }
         },
         { &hf_ssl2_handshake_challenge,
@@ -2459,5 +2538,8 @@ proto_register_ssl(void)
 void
 proto_reg_handoff_ssl(void)
 {
-    dissector_add("tcp.port", TCP_PORT_SSL, dissect_ssl, proto_ssl);
+    dissector_handle_t ssl_handle;
+
+    ssl_handle = create_dissector_handle(dissect_ssl, proto_ssl);
+    dissector_add("tcp.port", TCP_PORT_SSL, ssl_handle);
 }
