@@ -2,10 +2,10 @@
  * Routines for telnet packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-telnet.c,v 1.24 2001/01/25 06:14:14 guy Exp $
+ * $Id: packet-telnet.c,v 1.27 2001/12/10 00:25:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-pop.c
@@ -152,17 +152,19 @@ telnet_sub_option(proto_tree *telnet_tree, tvbuff_t *tvb, int start_offset)
 
   subneg_len = offset - start_offset;
 
-  ti = proto_tree_add_text(telnet_tree, tvb, start_offset, subneg_len,
-			"Suboption Begin: %s", opt);
+  if (subneg_len > 0) {
+      ti = proto_tree_add_text(telnet_tree, tvb, start_offset, subneg_len,
+                "Suboption Begin: %s", opt);
 
-  option_tree = proto_item_add_subtree(ti, ett_telnet_subopt);
+      option_tree = proto_item_add_subtree(ti, ett_telnet_subopt);
 
-  proto_tree_add_text(option_tree, tvb, start_offset + 2, 2,
-			"%s %s", (req ? "Send your" : "Here's my"), opt);
+      proto_tree_add_text(option_tree, tvb, start_offset + 2, 2,
+                "%s %s", (req ? "Send your" : "Here's my"), opt);
 
-  if (req == 0) {  /* Add the value */
-    proto_tree_add_text(option_tree, tvb, start_offset + 4, subneg_len - 4,
-	"Value: %s", tvb_format_text(tvb, start_offset + 4, subneg_len - 4));
+      if (req == 0) {  /* Add the value */
+        proto_tree_add_text(option_tree, tvb, start_offset + 4, subneg_len - 4,
+        "Value: %s", tvb_format_text(tvb, start_offset + 4, subneg_len - 4));
+      }
   }
   return offset;
 }
@@ -363,11 +365,11 @@ dissect_telnet(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
         proto_tree      *telnet_tree, *ti;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "TELNET");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "TELNET");
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_add_fstr(pinfo->fd, COL_INFO, "Telnet Data ...");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_fstr(pinfo->cinfo, COL_INFO, "Telnet Data ...");
 
 	if (tree) {
 	  gint offset = 0;
@@ -432,6 +434,8 @@ proto_register_telnet(void)
 void
 proto_reg_handoff_telnet(void)
 {
-	dissector_add("tcp.port", TCP_PORT_TELNET, dissect_telnet,
-	    proto_telnet);
+	dissector_handle_t telnet_handle;
+
+	telnet_handle = create_dissector_handle(dissect_telnet, proto_telnet);
+	dissector_add("tcp.port", TCP_PORT_TELNET, telnet_handle);
 }
