@@ -5,7 +5,7 @@
  * Craig Newell <CraigN@cheque.uq.edu.au>
  *	RFC2347 TFTP Option Extension
  *
- * $Id: packet-tftp.c,v 1.26 2001/06/18 02:17:53 guy Exp $
+ * $Id: packet-tftp.c,v 1.34 2001/12/10 00:25:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -55,6 +55,8 @@ static int hf_tftp_error_string = -1;
 
 static gint ett_tftp = -1;
 
+static dissector_handle_t tftp_handle;
+
 #define UDP_PORT_TFTP    69
 
 #define	TFTP_RRQ	1
@@ -91,16 +93,13 @@ static void tftp_dissect_options(tvbuff_t *tvb, int offset, proto_tree *tree);
 static void
 dissect_tftp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	proto_tree	*tftp_tree;
+	proto_tree	*tftp_tree = NULL;
 	proto_item	*ti;
 	conversation_t  *conversation;
 	gint		offset = 0;
 	guint16		opcode;
 	u_int           i1;
-
-	CHECK_DISPLAY_AS_DATA(proto_tftp, tvb, pinfo, tree);
-
-	pinfo->current_proto = "TFTP";
+	guint16         error;
 
 	/*
 	 * The first TFTP packet goes to the TFTP port; the second one
@@ -125,20 +124,19 @@ dissect_tftp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					   pinfo->srcport, 0, NO_PORT_B);
 	  if (conversation == NULL) {
 	    conversation = conversation_new(&pinfo->src, &pinfo->dst, PT_UDP,
-					    pinfo->srcport, 0, NULL,
-					    NO_PORT2);
-	    conversation_set_dissector(conversation, dissect_tftp);
+					    pinfo->srcport, 0, NO_PORT2);
+	    conversation_set_dissector(conversation, tftp_handle);
 	  }
 	}
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "TFTP");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "TFTP");
 
 	opcode = tvb_get_ntohs(tvb, offset);
 
-	if (check_col(pinfo->fd, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO)) {
 
-	  col_add_fstr(pinfo->fd, COL_INFO, "TFTP %s",
+	  col_add_fstr(pinfo->cinfo, COL_INFO, "%s",
 	    val_to_str(opcode, tftp_opcode_vals, "Unknown (0x%04x)"));
 
 	}
@@ -151,64 +149,127 @@ dissect_tftp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	  proto_tree_add_uint(tftp_tree, hf_tftp_opcode, tvb,
 			    offset, 2, opcode);
-	  offset += 2;
+	}
+	offset += 2;
 	    
-	  switch (opcode) {
-	  case TFTP_RRQ:
-	    i1 = tvb_strsize(tvb, offset);
+	switch (opcode) {
+
+	case TFTP_RRQ:
+	  i1 = tvb_strsize(tvb, offset);
+	  if (tree) {
 	    proto_tree_add_item(tftp_tree, hf_tftp_source_file,
 			    tvb, offset, i1, FALSE);
-	    offset += i1;
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", File: %s",
+			    tvb_get_ptr(tvb, offset, i1));
+	  }
+	  offset += i1;
 
-	    i1 = tvb_strsize(tvb, offset);
+	  i1 = tvb_strsize(tvb, offset);
+	  if (tree) {
 	    ti = proto_tree_add_item(tftp_tree, hf_tftp_transfer_type,
 			    tvb, offset, i1, FALSE);
-	    offset += i1;
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", Transfer type: %s",
+			    tvb_get_ptr(tvb, offset, i1));
+	  }
+	  offset += i1;
 
+	  if (tree)
 	    tftp_dissect_options(tvb, offset, tftp_tree);
-	    break;
-	  case TFTP_WRQ:
-	    i1 = tvb_strsize(tvb, offset);
+	  break;
+
+	case TFTP_WRQ:
+	  i1 = tvb_strsize(tvb, offset);
+	  if (tree) {
 	    proto_tree_add_item(tftp_tree, hf_tftp_destination_file,
 			    tvb, offset, i1, FALSE);
-	    offset += i1;
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", File: %s",
+			    tvb_get_ptr(tvb, offset, i1));
+	  }
+	  offset += i1;
 
-	    i1 = tvb_strsize(tvb, offset);
+	  i1 = tvb_strsize(tvb, offset);
+	  if (tree) {
 	    ti = proto_tree_add_item(tftp_tree, hf_tftp_transfer_type,
 			    tvb, offset, i1, FALSE);
-	    offset += i1;
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", Transfer type: %s",
+			    tvb_get_ptr(tvb, offset, i1));
+	  }
+	  offset += i1;
 
+	  if (tree)
 	    tftp_dissect_options(tvb, offset, tftp_tree);
-	    break;
-	  case TFTP_DATA:
+	  break;
+
+	case TFTP_DATA:
+	  if (tree) {
 	    proto_tree_add_item(tftp_tree, hf_tftp_blocknum, tvb, offset, 2,
 	    		    FALSE);
-	    offset += 2;
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", Block: %i",
+			    tvb_get_ntohs(tvb, offset));
+	  }
+	  offset += 2;
 
+	  if (tree) {
 	    proto_tree_add_text(tftp_tree, tvb, offset, tvb_length_remaining(tvb, offset),
 		"Data (%d bytes)", tvb_length_remaining(tvb, offset));
-	    break;
-	  case TFTP_ACK:
+	  }
+	  break;
+
+	case TFTP_ACK:
+	  if (tree) {
 	    proto_tree_add_item(tftp_tree, hf_tftp_blocknum, tvb, offset, 2,
 	    		    FALSE);
-	    break;
-	  case TFTP_ERROR:
-	    proto_tree_add_item(tftp_tree, hf_tftp_error_code, tvb, offset, 2,
-			    FALSE);
-	    offset += 2;
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", Block: %i",
+			    tvb_get_ntohs(tvb, offset));
+	  }
+	  break;
 
-	    i1 = tvb_strsize(tvb, offset);
+	case TFTP_ERROR:
+	  error = tvb_get_ntohs(tvb, offset);
+	  if (tree) {
+	    proto_tree_add_uint(tftp_tree, hf_tftp_error_code, tvb, offset, 2,
+			    error);
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", Code: %s",
+			    val_to_str(error, tftp_error_code_vals, "Unknown (%u)"));
+	  }
+	  offset += 2;
+
+	  i1 = tvb_strsize(tvb, offset);
+	  if (tree) {
 	    proto_tree_add_item(tftp_tree, hf_tftp_error_string, tvb, offset,
 	        i1, FALSE);
-	    break;
-	  case TFTP_OACK:
+	  }
+	  if (check_col(pinfo->cinfo, COL_INFO)) {
+	    col_append_fstr(pinfo->cinfo, COL_INFO, ", Message: %s",
+			    tvb_get_ptr(tvb, offset, i1));
+	  }
+	  break;
+
+	case TFTP_OACK:
+	  if (tree)
 	    tftp_dissect_options(tvb, offset, tftp_tree);
-	    break;
-	  default:
+	  break;
+
+	default:
+	  if (tree) {
 	    proto_tree_add_text(tftp_tree, tvb, offset, tvb_length_remaining(tvb, offset),
 		"Data (%d bytes)", tvb_length_remaining(tvb, offset));
-	    break;
 	  }
+	  break;
 
 	}
 }
@@ -278,10 +339,12 @@ proto_register_tftp(void)
 				       "TFTP", "tftp");
   proto_register_field_array(proto_tftp, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
+
+  tftp_handle = create_dissector_handle(dissect_tftp, proto_tftp);
 }
 
 void
 proto_reg_handoff_tftp(void)
 {
-  dissector_add("udp.port", UDP_PORT_TFTP, dissect_tftp, proto_tftp);
+  dissector_add("udp.port", UDP_PORT_TFTP, tftp_handle);
 }
