@@ -3,10 +3,10 @@
  *
  * Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-time.c,v 1.14 2001/06/18 02:17:53 guy Exp $
+ * $Id: packet-time.c,v 1.16 2001/12/10 00:25:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * Copied from packet-tftp.c
@@ -45,11 +45,11 @@ dissect_time(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   proto_tree	*time_tree;
   proto_item	*ti;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "TIME");
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "TIME");
   
-  if (check_col(pinfo->fd, COL_INFO)) {
-    col_add_fstr(pinfo->fd, COL_INFO, "TIME %s",
+  if (check_col(pinfo->cinfo, COL_INFO)) {
+    col_add_fstr(pinfo->cinfo, COL_INFO, "TIME %s",
 		 pinfo->srcport == pinfo->match_port ? "Response":"Request");
   }
   
@@ -92,5 +92,8 @@ proto_register_time(void)
 void
 proto_reg_handoff_time(void)
 {
-  dissector_add("udp.port", UDP_PORT_TIME, dissect_time, proto_time);
+  dissector_handle_t time_handle;
+
+  time_handle = create_dissector_handle(dissect_time, proto_time);
+  dissector_add("udp.port", UDP_PORT_TIME, time_handle);
 }
