@@ -7,7 +7,7 @@
  * Routine to dissect RFC 1006 TPKT packet containing OSI TP PDU
  * Copyright 2001, Martin Thomas <Martin_A_Thomas@yahoo.com>
  *
- * $Id: packet-tpkt.c,v 1.7 2001/06/18 02:17:53 guy Exp $
+ * $Id: packet-tpkt.c,v 1.9 2001/12/10 00:25:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -108,14 +108,14 @@ dissect_tpkt_header( tvbuff_t *tvb, unsigned int* offset, packet_info *pinfo, pr
 
 	pinfo->current_proto = "TPKT";
 
-	if ( check_col( pinfo->fd, COL_PROTOCOL ) ) {
-		col_set_str( pinfo->fd, COL_PROTOCOL, "TPKT" );
+	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) ) {
+		col_set_str( pinfo->cinfo, COL_PROTOCOL, "TPKT" );
 	}
 	
 	data_len = tvb_get_ntohs( tvb, (*offset) + 2 );
 
-	if ( check_col( pinfo->fd, COL_INFO) ) {
-		col_add_fstr( pinfo->fd, COL_INFO, "TPKT Data length = %u",
+	if ( check_col( pinfo->cinfo, COL_INFO) ) {
+		col_add_fstr( pinfo->cinfo, COL_INFO, "TPKT Data length = %u",
 		    data_len );
 	}
 
@@ -231,6 +231,9 @@ proto_register_tpkt(void)
 void
 proto_reg_handoff_tpkt(void)
 {
+	dissector_handle_t tpkt_handle;
+
 	osi_tp_handle = find_dissector("ositp");
-	dissector_add("tcp.port", TCP_PORT_TPKT, dissect_tpkt, proto_tpkt);
+	tpkt_handle = create_dissector_handle(dissect_tpkt, proto_tpkt);
+	dissector_add("tcp.port", TCP_PORT_TPKT, tpkt_handle);
 }
