@@ -1,8 +1,8 @@
 /* packet-tr.c
  * Routines for Token-Ring packet disassembly
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-tr.c,v 1.63 2001/07/03 04:56:46 guy Exp $
+ * $Id: packet-tr.c,v 1.68 2001/12/10 00:25:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -118,6 +118,7 @@ static const value_string direction_vals[] = {
 
 static dissector_handle_t trmac_handle;
 static dissector_handle_t llc_handle;
+static dissector_handle_t data_handle;
 
 /*
  * DODGY LINUX HACK DODGY LINUX HACK
@@ -179,7 +180,7 @@ static void
 add_ring_bridge_pairs(int rcf_len, tvbuff_t*, proto_tree *tree);
 
 void
-capture_tr(const u_char *pd, int offset, packet_counts *ld) {
+capture_tr(const u_char *pd, int offset, int len, packet_counts *ld) {
 
 	int			source_routed = 0;
 	int			frame_type;
@@ -192,7 +193,7 @@ capture_tr(const u_char *pd, int offset, packet_counts *ld) {
 	guint8			trn_fc;		/* field control field */
 	const guint8		*trn_shost;	/* source host */
 
-	if (!BYTES_ARE_IN_FRAME(offset, TR_MIN_HEADER_LEN)) {
+	if (!BYTES_ARE_IN_FRAME(offset, len, TR_MIN_HEADER_LEN)) {
 		ld->other++;
 		return;
 	}
@@ -284,7 +285,7 @@ capture_tr(const u_char *pd, int offset, packet_counts *ld) {
 			ld->other++;
 			break;
 		case 1:
-			capture_llc(pd, offset, ld);
+			capture_llc(pd, offset, len, ld);
 			break;
 		default:
 			/* non-MAC, non-LLC, i.e., "Reserved" */
@@ -325,8 +326,8 @@ dissect_tr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* Token-Ring Strings */
 	char *fc[] = { "MAC", "LLC", "Reserved", "Unknown" };
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "TR");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "TR");
 
 	if ((x = check_for_old_linux_tvb((tvbuff_t*) tvb))) {
 		/* Actually packet starts x bytes into what we have got but with all
@@ -347,8 +348,8 @@ dissect_tr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	trn_shost_nonsr[0] &= 127;
 	frame_type = (trn_fc & 192) >> 6;
 
-	if (check_col(pinfo->fd, COL_INFO))
-		col_add_fstr(pinfo->fd, COL_INFO, "Token-Ring %s", fc[frame_type]);
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_add_fstr(pinfo->cinfo, COL_INFO, "Token-Ring %s", fc[frame_type]);
 
 	/* if the high bit on the first byte of src hwaddr is 1, then
 		this packet is source-routed */
@@ -527,7 +528,7 @@ dissect_tr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			break;
 		default:
 			/* non-MAC, non-LLC, i.e., "Reserved" */
-			dissect_data(next_tvb, 0, pinfo, tree);
+			call_dissector(data_handle,next_tvb, pinfo, tree);
 			break;
 	}
 }
@@ -673,12 +674,15 @@ proto_register_tr(void)
 void
 proto_reg_handoff_tr(void)
 {
+	dissector_handle_t tr_handle;
+
 	/*
 	 * Get handles for the TR MAC and LLC dissectors.
 	 */
 	trmac_handle = find_dissector("trmac");
 	llc_handle = find_dissector("llc");
+	data_handle = find_dissector("data");
 
-	dissector_add("wtap_encap", WTAP_ENCAP_TOKEN_RING, dissect_tr,
-	    proto_tr);
+	tr_handle = find_dissector("tr");
+	dissector_add("wtap_encap", WTAP_ENCAP_TOKEN_RING, tr_handle);
 }
