@@ -2,12 +2,11 @@
  * Routines for v120 frame disassembly
  * Bert Driehuis <driehuis@playbeing.org>
  *
- * $Id: packet-v120.c,v 1.19 2001/06/18 02:17:53 guy Exp $
+ * $Id: packet-v120.c,v 1.22 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,6 +49,8 @@ static gint ett_v120_address = -1;
 static gint ett_v120_control = -1;
 static gint ett_v120_header = -1;
 
+static dissector_handle_t data_handle;
+
 static int dissect_v120_header(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
 
 static void
@@ -65,22 +66,22 @@ dissect_v120(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     guint16	control;
     tvbuff_t	*next_tvb;
 
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-	col_set_str(pinfo->fd, COL_PROTOCOL, "V.120");
-    if (check_col(pinfo->fd, COL_INFO))
-	col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "V.120");
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_clear(pinfo->cinfo, COL_INFO);
 
     byte0 = tvb_get_guint8(tvb, 0);
 
-    if(check_col(pinfo->fd, COL_RES_DL_SRC))
-	col_add_fstr(pinfo->fd, COL_RES_DL_SRC, "0x%02X", byte0);
+    if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+	col_add_fstr(pinfo->cinfo, COL_RES_DL_SRC, "0x%02X", byte0);
 
     byte1 = tvb_get_guint8(tvb, 1);
 
     if ((byte0 & 0x01) != 0x00 && (byte1 && 0x01) != 0x01)
     {
-	if (check_col(pinfo->fd, COL_INFO))
-	    col_set_str(pinfo->fd, COL_INFO, "Invalid V.120 frame");
+	if (check_col(pinfo->cinfo, COL_INFO))
+	    col_set_str(pinfo->cinfo, COL_INFO, "Invalid V.120 frame");
 	if (tree)
 	    ti = proto_tree_add_protocol_format(tree, proto_v120, tvb, 0, tvb_length(tvb),
 			                    "Invalid V.120 frame");
@@ -88,16 +89,16 @@ dissect_v120(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
     if (pinfo->pseudo_header->x25.flags & FROM_DCE) {
-	if(check_col(pinfo->fd, COL_RES_DL_DST))
-	    col_set_str(pinfo->fd, COL_RES_DL_DST, "DTE");
-	if(check_col(pinfo->fd, COL_RES_DL_SRC))
-	    col_set_str(pinfo->fd, COL_RES_DL_SRC, "DCE");
+	if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+	    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "DTE");
+	if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+	    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "DCE");
     }
     else {
-	if(check_col(pinfo->fd, COL_RES_DL_DST))
-	    col_set_str(pinfo->fd, COL_RES_DL_DST, "DCE");
-	if(check_col(pinfo->fd, COL_RES_DL_SRC))
-	    col_set_str(pinfo->fd, COL_RES_DL_SRC, "DTE");
+	if(check_col(pinfo->cinfo, COL_RES_DL_DST))
+	    col_set_str(pinfo->cinfo, COL_RES_DL_DST, "DCE");
+	if(check_col(pinfo->cinfo, COL_RES_DL_SRC))
+	    col_set_str(pinfo->cinfo, COL_RES_DL_SRC, "DTE");
     }
 
     if (((pinfo->pseudo_header->x25.flags & FROM_DCE) && byte0 & 0x02) ||
@@ -142,7 +143,7 @@ dissect_v120(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		v120len += dissect_v120_header(tvb, v120len, pinfo, v120_tree);
 	proto_item_set_len(ti, v120len);
 	next_tvb = tvb_new_subset(tvb, v120len, -1, -1);
-	dissect_data(next_tvb, 0, pinfo, v120_tree);
+	call_dissector(data_handle,next_tvb, pinfo, v120_tree);
     }
 }
 
@@ -233,6 +234,9 @@ proto_register_v120(void)
 void
 proto_reg_handoff_v120(void)
 {
-	dissector_add("wtap_encap", WTAP_ENCAP_V120, dissect_v120,
-	    proto_v120);
+    dissector_handle_t v120_handle;
+
+    data_handle = find_dissector("data");
+    v120_handle = create_dissector_handle(dissect_v120, proto_v120);
+    dissector_add("wtap_encap", WTAP_ENCAP_V120, v120_handle);
 }
