@@ -1,7 +1,7 @@
 /* packet-vines.c
  * Routines for Banyan VINES protocol packet disassembly
  *
- * $Id: packet-vines.c,v 1.33 2001/06/18 02:17:53 guy Exp $
+ * $Id: packet-vines.c,v 1.38 2001/12/10 00:25:41 guy Exp $
  *
  * Don Lafontaine <lafont02@cn.ca>
  *
@@ -9,7 +9,6 @@
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  * Joerg Mayer <jmayer@loplof.de>
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -72,12 +71,13 @@ static void dissect_vines_spp(tvbuff_t *, packet_info *, proto_tree *);
 static void dissect_vines(tvbuff_t *, packet_info *, proto_tree *);
 
 void
-capture_vines(const u_char *pd, int offset, packet_counts *ld)
+capture_vines(const u_char *pd, int offset, int len, packet_counts *ld)
 {
 	ld->vines++;
 }
 
 static dissector_handle_t vines_handle;
+static dissector_handle_t data_handle;
 
 /* AFAIK Vines FRP (Fragmentation Protocol) is used on all media except
  * Ethernet and TR (and probably FDDI) - Fragmentation on these media types
@@ -93,10 +93,10 @@ dissect_vines_frp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gchar	frp_flags_str[32];
 	tvbuff_t *next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "Vines FRP");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines FRP");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_vines_frp, tvb, 0, 2,
@@ -162,18 +162,15 @@ proto_register_vines_frp(void)
 void
 proto_reg_handoff_vines_frp(void)
 {
-	/*
-	 * Get handle for the Vines dissector.
-	 */
-	vines_handle = find_dissector("vines");
+	dissector_handle_t vines_frp_handle;
 
-	dissector_add("ip.proto", IP_PROTO_VINES, dissect_vines_frp,
+	vines_frp_handle = create_dissector_handle(dissect_vines_frp,
 	    proto_vines_frp);
+	dissector_add("ip.proto", IP_PROTO_VINES, vines_frp_handle);
 
 	/* XXX: AFAIK, src and dst port must be the same; should
 	   the dissector check for that? */
-	dissector_add("udp.port", UDP_PORT_VINES, dissect_vines_frp,
-	    proto_vines_frp);
+	dissector_add("udp.port", UDP_PORT_VINES, vines_frp_handle);
 }
 
 static dissector_table_t vines_dissector_table;
@@ -191,10 +188,10 @@ dissect_vines(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int  hops = 0;
 	tvbuff_t *next_tvb;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "Vines");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	/* To do: check for runts, errs, etc. */
 
@@ -210,40 +207,40 @@ dissect_vines(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	switch (viph.vip_proto) {
  	case VIP_PROTO_IPC:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "Vines IPC");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "IPC (%02x)", viph.vip_proto);
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines IPC");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "IPC (%02x)", viph.vip_proto);
  		break;
  	case VIP_PROTO_SPP:      
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "Vines SPP");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "SPP (%02x)", viph.vip_proto);
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines SPP");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "SPP (%02x)", viph.vip_proto);
 		break;
 	case VIP_PROTO_ARP:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "Vines ARP");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "ARP (%02x)", viph.vip_proto);
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines ARP");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "ARP (%02x)", viph.vip_proto);
 		break;
 	case VIP_PROTO_RTP:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "Vines RTP");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "RTP (%02x)", viph.vip_proto);
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines RTP");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "RTP (%02x)", viph.vip_proto);
 		break;
 	case VIP_PROTO_ICP:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "Vines ICP");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "ICP (%02x)", viph.vip_proto);
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines ICP");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "ICP (%02x)", viph.vip_proto);
 		break;
 	default:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "Vines IP");
-		if (check_col(pinfo->fd, COL_INFO))
-			col_add_fstr(pinfo->fd, COL_INFO, "Unknown VIP protocol (%02x)", 
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "Vines IP");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_add_fstr(pinfo->cinfo, COL_INFO, "Unknown VIP protocol (%02x)", 
 				     viph.vip_proto);
 	}
 
@@ -317,7 +314,7 @@ dissect_vines(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	next_tvb = tvb_new_subset(tvb, offset, -1, -1);
 	if (!dissector_try_port(vines_dissector_table, viph.vip_proto,
 	    next_tvb, pinfo, tree))
-		dissect_data(next_tvb, 0, pinfo, tree);
+		call_dissector(data_handle,next_tvb, pinfo, tree);
 }
 
 void
@@ -339,16 +336,19 @@ proto_register_vines(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 	/* subdissector code */
-	vines_dissector_table = register_dissector_table("vines.proto");
+	vines_dissector_table = register_dissector_table("vines.proto",
+	    "Vines protocol", FT_UINT8, BASE_HEX);
 
 	register_dissector("vines", dissect_vines, proto_vines);
+	vines_handle = find_dissector("vines");
 }
 
 void
 proto_reg_handoff_vines(void)
 {
-	dissector_add("ethertype", ETHERTYPE_VINES, dissect_vines, proto_vines);
-	dissector_add("ppp.protocol", PPP_VINES, dissect_vines, proto_vines);
+	dissector_add("ethertype", ETHERTYPE_VINES, vines_handle);
+	dissector_add("ppp.protocol", PPP_VINES, vines_handle);
+	data_handle = find_dissector("data");
 }
 
 static void
@@ -359,10 +359,10 @@ dissect_vines_spp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	proto_tree *vspp_tree;
 	proto_item *ti;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "VSPP");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_clear(pinfo->fd, COL_INFO);
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "VSPP");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_clear(pinfo->cinfo, COL_INFO);
 
 	/* To do: check for runts, errs, etc. */
 
@@ -376,28 +376,28 @@ dissect_vines_spp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	switch (viph.vspp_pkttype) {
 	case VSPP_PKTTYPE_DATA:      
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "VSPP Data");
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "VSPP Data");
 		break;
 	case VSPP_PKTTYPE_DISC:      
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "VSPP Disconnect");
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "VSPP Disconnect");
 		break;
 	case VSPP_PKTTYPE_PROBE:      
-		if (check_col(pinfo->fd, COL_PROTOCOL))
-			col_set_str(pinfo->fd, COL_PROTOCOL, "VSPP Probe");
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "VSPP Probe");
 		break;
 	case VSPP_PKTTYPE_ACK:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
- 			col_set_str(pinfo->fd, COL_PROTOCOL, "VSPP Ack");
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+ 			col_set_str(pinfo->cinfo, COL_PROTOCOL, "VSPP Ack");
 		break;
 	default:
-		if (check_col(pinfo->fd, COL_PROTOCOL))
- 			col_set_str(pinfo->fd, COL_PROTOCOL, "VSPP Unknown");
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+ 			col_set_str(pinfo->cinfo, COL_PROTOCOL, "VSPP Unknown");
 	}
 
-	if (check_col(pinfo->fd, COL_INFO))
- 		col_add_fstr(pinfo->fd, COL_INFO, 
+	if (check_col(pinfo->cinfo, COL_INFO))
+ 		col_add_fstr(pinfo->cinfo, COL_INFO, 
 			     "NS=%04x NR=%04x Window=%04x RID=%04x LID=%04x D=%04x S=%04x", 
 			     viph.vspp_seqno, viph.vspp_ack, viph.vspp_win, 
 			     viph.vspp_rmtid, viph.vspp_lclid, viph.vspp_dport,
@@ -455,7 +455,7 @@ dissect_vines_spp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				    "Window: 0x%04x", viph.vspp_win);
 	}
 	offset += 16; /* sizeof SPP */
-	dissect_data(tvb, offset, pinfo, tree);
+	call_dissector(data_handle,tvb_new_subset(tvb, offset,-1,tvb_reported_length_remaining(tvb,offset)), pinfo, tree);
 }
 
 void
@@ -473,6 +473,9 @@ proto_register_vines_spp(void)
 void
 proto_reg_handoff_vines_spp(void)
 {
-	dissector_add("vines.proto", VIP_PROTO_SPP, dissect_vines_spp,
+	dissector_handle_t vines_spp_handle;
+
+	vines_spp_handle = create_dissector_handle(dissect_vines_spp,
 	    proto_vines_spp);
+	dissector_add("vines.proto", VIP_PROTO_SPP, vines_spp_handle);
 }
