@@ -1,12 +1,11 @@
 /* packet-vlan.c
  * Routines for VLAN 802.1Q ethernet header disassembly
  *
- * $Id: packet-vlan.c,v 1.34 2001/06/18 02:17:53 guy Exp $
+ * $Id: packet-vlan.c,v 1.37 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +39,7 @@
 #include "packet-ieee8023.h"
 #include "packet-ipx.h"
 #include "packet-llc.h"
+#include "packet-vlan.h"
 #include "etypes.h"
 
 static int proto_vlan = -1;
@@ -53,21 +53,21 @@ static int hf_vlan_trailer = -1;
 static gint ett_vlan = -1;
 
 void
-capture_vlan(const u_char *pd, int offset, packet_counts *ld ) {
+capture_vlan(const u_char *pd, int offset, int len, packet_counts *ld ) {
   guint16 encap_proto;
-  if ( !BYTES_ARE_IN_FRAME(offset,5) ) {
+  if ( !BYTES_ARE_IN_FRAME(offset,len,5) ) {
     ld->other++;
     return; 
   }
   encap_proto = pntohs( &pd[offset+2] );
   if ( encap_proto <= IEEE_802_3_MAX_LEN) {
     if ( pd[offset+4] == 0xff && pd[offset+5] == 0xff ) {
-      capture_ipx(pd,offset+4,ld);
+      capture_ipx(pd,offset+4,len,ld);
     } else {
-      capture_llc(pd,offset+4,ld);
+      capture_llc(pd,offset+4,len,ld);
     }
   } else {
-    capture_ethertype(encap_proto, offset+4, pd, ld);
+    capture_ethertype(encap_proto, pd, offset+4, len, ld);
   }
 }
 
@@ -79,15 +79,15 @@ dissect_vlan(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   volatile gboolean is_802_2;
   proto_tree *volatile vlan_tree;
 
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_set_str(pinfo->fd, COL_PROTOCOL, "VLAN");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "VLAN");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   tci = tvb_get_ntohs( tvb, 0 );
 
-  if (check_col(pinfo->fd, COL_INFO)) {
-    col_add_fstr(pinfo->fd, COL_INFO, "PRI: %d  CFI: %d  ID: %d",
+  if (check_col(pinfo->cinfo, COL_INFO)) {
+    col_add_fstr(pinfo->cinfo, COL_INFO, "PRI: %d  CFI: %d  ID: %d",
       (tci >> 13), ((tci >> 12) & 1), (tci & 0xFFF));
   }
 
@@ -166,5 +166,8 @@ proto_register_vlan(void)
 void
 proto_reg_handoff_vlan(void)
 {
-  dissector_add("ethertype", ETHERTYPE_VLAN, dissect_vlan, proto_vlan);
+  dissector_handle_t vlan_handle;
+
+  vlan_handle = create_dissector_handle(dissect_vlan, proto_vlan);
+  dissector_add("ethertype", ETHERTYPE_VLAN, vlan_handle);
 }
