@@ -1,13 +1,12 @@
 /* packet-vtp.c
  * Routines for the disassembly of Cisco's Virtual Trunking Protocol
  *
- * $Id: packet-vtp.c,v 1.14 2001/06/18 02:17:54 guy Exp $
+ * $Id: packet-vtp.c,v 1.17 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  * 
- * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -102,10 +101,10 @@ dissect_vtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	const guint8 *upd_timestamp;
 	int vlan_info_len;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "VTP");
-	if (check_col(pinfo->fd, COL_INFO))
-		col_set_str(pinfo->fd, COL_INFO, "Virtual Trunking Protocol"); 
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "VTP");
+	if (check_col(pinfo->cinfo, COL_INFO))
+		col_set_str(pinfo->cinfo, COL_INFO, "Virtual Trunking Protocol"); 
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_vtp, tvb, offset,
@@ -271,7 +270,6 @@ dissect_vlan_info(tvbuff_t *tvb, int offset, proto_tree *tree)
 	guint8 vlan_name_len;
 	guint8 type;
 	int length;
-	char *type_str;
 	proto_tree *tlv_tree;
 
 	vlan_info_len = tvb_get_guint8(tvb, offset);
@@ -345,11 +343,11 @@ dissect_vlan_info(tvbuff_t *tvb, int offset, proto_tree *tree)
 	while (vlan_info_left > 0) {
 		type = tvb_get_guint8(tvb, offset + 0);
 		length = tvb_get_guint8(tvb, offset + 1);
-		type_str = val_to_str(type, vlan_tlv_type_vals,
-		    "Unknown (0x%04x)");
 
-		ti = proto_tree_add_notext(vlan_info_tree, tvb, offset,
-		    2 + length*2);
+		ti = proto_tree_add_text(vlan_info_tree, tvb, offset,
+		    2 + length*2, "%s",
+		    val_to_str(type, vlan_tlv_type_vals,
+		      "Unknown TLV type: 0x%02x"));
 		tlv_tree = proto_item_add_subtree(ti, ett_vtp_tlv);
 		proto_tree_add_uint(tlv_tree, hf_vtp_vlan_tlvtype, tvb, offset,
 		    1, type);
@@ -588,7 +586,6 @@ dissect_vlan_info_tlv(tvbuff_t *tvb, int offset, int length,
 		break;
 
 	default:
-		proto_item_set_text(ti, "Unknown TLV type: 0x%02x", type);
 		proto_tree_add_text(tree, tvb, offset, length, "Data");
 		break;
 	}
@@ -698,5 +695,8 @@ proto_register_vtp(void)
 void
 proto_reg_handoff_vtp(void)
 {
-	dissector_add("llc.cisco_pid", 0x2003, dissect_vtp, proto_vtp);
+	dissector_handle_t vtp_handle;
+
+	vtp_handle = create_dissector_handle(dissect_vtp, proto_vtp);
+	dissector_add("llc.cisco_pid", 0x2003, vtp_handle);
 }
