@@ -2,10 +2,10 @@
  *
  * Declarations for WAP packet disassembly
  *
- * $Id: packet-wap.h,v 1.2 2001/02/13 00:17:54 guy Exp $
+ * $Id: packet-wap.h,v 1.6 2001/12/03 02:10:31 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
  * WAP dissector based on original work by Ben Fowler
@@ -38,7 +38,9 @@
 #define UDP_PORT_WSP			9200		/* wap-wsp			*/
 #define UDP_PORT_WTP_WSP		9201		/* wap-wsp-wtp		*/
 #define UDP_PORT_WTLS_WSP		9202		/* wap-wsp-s		*/
-#define UDP_PORT_WTLS_WTP_WSP	9203		/* wap-wsp-wtp-s	*/
+#define UDP_PORT_WTLS_WTP_WSP		9203		/* wap-wsp-wtp-s	*/
+#define UDP_PORT_WSP_PUSH		2948		/* wap-wsp		*/
+#define UDP_PORT_WTLS_WSP_PUSH		2949		/* wap-wsp-s		*/
 
 /*
  * Note:
@@ -61,6 +63,9 @@
  *   9202  WTLS  ->           WSP  ->  WMLC
  *   9203  WTLS  ->  WTP  ->  WSP  ->  WMLC
  *
+ *   2948                     WSP  ->  WMLC (Push)
+ *   2949  WTLS  ->           WSP  ->  WMLC (Push)
+ *
  *   At present, only the unencrypted parts of WTLS can be analysed. Therefore
  *   the WTP and WSP dissectors are not called.
  */
