@@ -2,11 +2,11 @@
  *
  * Routines to dissect WTLS component of WAP traffic.
  * 
- * $Id: packet-wtls.c,v 1.10 2001/07/03 09:53:22 guy Exp $
+ * $Id: packet-wtls.c,v 1.16 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
- * Copyright 1998 Didier Jorand
+ * By Gerald Combs <gerald@ethereal.com>
+ * Copyright 1998 Gerald Combs
  *
  * WAP dissector based on original work by Ben Fowler
  * Updated by Neil Hunter <neil.hunter@energis-squared.com>
@@ -331,7 +331,6 @@ static void dissect_wtls_handshake (proto_tree *, tvbuff_t *, guint, guint);
 static void
 dissect_wtls(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	frame_data *fdata = pinfo->fd;
 	int offset = 0;
 
 	char pdut;
@@ -345,22 +344,22 @@ dissect_wtls(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	proto_tree *wtls_rec_tree;
 	proto_tree *wtls_msg_type_tree;
 
-	if (check_col(fdata, COL_PROTOCOL)) 
+	if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
 	{
 		switch ( pinfo->match_port )
 		{
 			case UDP_PORT_WTLS_WSP:
-				col_set_str(fdata, COL_PROTOCOL, "WTLS+WSP" );
+				col_set_str(pinfo->cinfo, COL_PROTOCOL, "WTLS+WSP" );
 				break;
 			case UDP_PORT_WTLS_WTP_WSP:
-				col_set_str(fdata, COL_PROTOCOL, "WTLS+WTP+WSP" );
+				col_set_str(pinfo->cinfo, COL_PROTOCOL, "WTLS+WTP+WSP" );
 				break;
 		}
 	}
 
 	/* Develop the string to put in the Info column */
-	if (check_col(fdata, COL_INFO)) {
-		col_set_str(fdata, COL_INFO, "WTLS" );
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_set_str(pinfo->cinfo, COL_INFO, "WTLS" );
 	};
 
 	/* In the interest of speed, if "tree" is NULL, don't do any work not
@@ -449,7 +448,7 @@ static void
 dissect_wtls_handshake(proto_tree *tree, tvbuff_t *tvb, guint offset, guint count)
 {
 	char pdu_msg_type;
-	struct timeval timeValue;
+	nstime_t timeValue;
 	int client_size = 0;
 	guint value = 0;
 	int size = 0;
@@ -485,8 +484,8 @@ dissect_wtls_handshake(proto_tree *tree, tvbuff_t *tvb, guint offset, guint coun
 			ti = proto_tree_add_item (wtls_msg_type_item_tree, hf_wtls_hands_cli_hello_version,
 					tvb,offset,1,bo_big_endian);
 			offset++;
-			timeValue.tv_sec = tvb_get_ntohl (tvb, offset);
-			timeValue.tv_usec = 0;
+			timeValue.secs = tvb_get_ntohl (tvb, offset);
+			timeValue.nsecs = 0;
 			ti = proto_tree_add_time (wtls_msg_type_item_tree, hf_wtls_hands_cli_hello_gmt, tvb,
 					offset, 4, &timeValue);
 			offset+=4;
@@ -870,8 +869,8 @@ dissect_wtls_handshake(proto_tree *tree, tvbuff_t *tvb, guint offset, guint coun
 			ti = proto_tree_add_item (wtls_msg_type_item_tree, hf_wtls_hands_serv_hello_version,
 					tvb,offset,1,bo_big_endian);
 			offset++;
-			timeValue.tv_sec = tvb_get_ntohl (tvb, offset);
-			timeValue.tv_usec = 0;
+			timeValue.secs = tvb_get_ntohl (tvb, offset);
+			timeValue.nsecs = 0;
 			ti = proto_tree_add_time (wtls_msg_type_item_tree, hf_wtls_hands_serv_hello_gmt, tvb,
 					offset, 4, &timeValue);
 			offset+=4;
@@ -1008,15 +1007,15 @@ dissect_wtls_handshake(proto_tree *tree, tvbuff_t *tvb, guint offset, guint coun
 							case IDENTIFIER_X509 :
 								break;
 						}
-						timeValue.tv_sec = tvb_get_ntohl (tvb, offset);
-						timeValue.tv_usec = 0;
+						timeValue.secs = tvb_get_ntohl (tvb, offset);
+						timeValue.nsecs = 0;
 						ti = proto_tree_add_time (wtls_msg_type_item_sub_tree, 
 								hf_wtls_hands_certificate_wtls_valid_not_before,
 								tvb, offset, 4, &timeValue);
 						offset+=4;
 						client_size+=4;
-						timeValue.tv_sec = tvb_get_ntohl (tvb, offset);
-						timeValue.tv_usec = 0;
+						timeValue.secs = tvb_get_ntohl (tvb, offset);
+						timeValue.nsecs = 0;
 						ti = proto_tree_add_time (wtls_msg_type_item_sub_tree, 
 								hf_wtls_hands_certificate_wtls_valid_not_after,
 								tvb, offset, 4, &timeValue);
@@ -1636,12 +1635,16 @@ proto_register_wtls(void)
 void
 proto_reg_handoff_wtls(void)
 {
+	dissector_handle_t wtls_handle;
+
 	/*
-	 * Get handles for the IP WTP and WSP dissectors
+	 * Get handles for the WTP and connectionless WSP dissectors.
 	 */
 	wtp_handle = find_dissector("wtp");
-	wsp_handle = find_dissector("wsp");
+	wsp_handle = find_dissector("wsp-cl");
 
-	dissector_add("udp.port", UDP_PORT_WTLS_WSP,     dissect_wtls, proto_wtls); 
-	dissector_add("udp.port", UDP_PORT_WTLS_WTP_WSP, dissect_wtls, proto_wtls);
+	wtls_handle = create_dissector_handle(dissect_wtls, proto_wtls);
+	dissector_add("udp.port", UDP_PORT_WTLS_WSP,     wtls_handle); 
+	dissector_add("udp.port", UDP_PORT_WTLS_WTP_WSP, wtls_handle);
+	dissector_add("udp.port", UDP_PORT_WTLS_WSP_PUSH,wtls_handle);
 }
