@@ -2,11 +2,11 @@
  *
  * Routines to dissect WTP component of WAP traffic.
  * 
- * $Id: packet-wtp.c,v 1.15 2001/07/03 09:53:22 guy Exp $
+ * $Id: packet-wtp.c,v 1.23 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
- * Copyright 1998 Didier Jorand
+ * By Gerald Combs <gerald@ethereal.com>
+ * Copyright 1998 Gerald Combs
  *
  * WAP dissector based on original work by Ben Fowler
  * Updated by Neil Hunter <neil.hunter@energis-squared.com>
@@ -100,7 +100,7 @@ static const value_string vals_pdu_type[] = {
 	{ 0, NULL }
 };
 
-static const value_string vals_transmission_trailer[] = {
+static const value_string vals_transaction_trailer[] = {
 	{ 0, "Not last packet" },
 	{ 1, "Last packet of message" },
 	{ 2, "Last packet of group" },
@@ -136,11 +136,19 @@ static const value_string vals_abort_reason_provider[] = {
 	{ 0x00, NULL }
 };
 
+static const value_string vals_transaction_classes[] = {
+	{ 0x00, "Unreliable Invoke without Result" },
+	{ 0x01, "Reliable Invoke without Result" },
+	{ 0x02, "Reliable Invoke with Reliable Result" },
+	{ 0x00, NULL }
+};
+
 /* File scoped variables for the protocol and registered fields */
 static int proto_wtp 							= HF_EMPTY;
 
 /* These fields used by fixed part of header */
 static int hf_wtp_header_fixed_part 			= HF_EMPTY;
+static int hf_wtp_header_sub_pdu_size 			= HF_EMPTY;
 static int hf_wtp_header_flag_continue 			= HF_EMPTY;
 static int hf_wtp_header_pdu_type 				= HF_EMPTY;
 static int hf_wtp_header_flag_Trailer 			= HF_EMPTY;
@@ -182,8 +190,6 @@ static char retransmission_indicator(unsigned char octet);
 static void
 dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	frame_data *fdata = pinfo->fd;
-
 	char szInfo[ 50 ];
 	int offCur 			= 0; /* current offset from start of WTP data */
 
@@ -197,12 +203,13 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* continuation flag */
 	unsigned char  	fCon;
 	unsigned char  	fRID;
-	int 		cbHeader   	= 0;
+	guint 		cbHeader   	= 0;
+	guint 		vHeader   	= 0;
 	int 		abortType  	= 0;
 
 /* Set up structures we will need to add the protocol subtree and manage it */
 	proto_item *ti;
-	proto_tree *wtp_tree;
+	proto_tree *wtp_tree = NULL;
 	proto_tree *wtp_header_fixed;
 	
 	char pdut;
@@ -212,15 +219,38 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int i;
 	tvbuff_t *wsp_tvb = NULL;
 
-#ifdef DEBUG
-	fprintf( stderr, "dissect_wtp: (Entering) Frame data at %p\n", fdata ); 
-	fprintf( stderr, "dissect_wtp: tvb length is %d\n", tvb_reported_length( tvb ) ); 
-#endif
-	if (check_col(fdata, COL_INFO)) {
-		col_clear(fdata, COL_INFO);
-	};
-    
 	b0 = tvb_get_guint8 (tvb, offCur + 0);
+	/* Discover Concatenated PDUs */
+	if (b0 == 0) {
+		if (tree) {
+			wtp_tree = proto_tree_add_item(tree, proto_wtp, tvb, offCur, 1, bo_little_endian);
+		}
+		offCur = 1;
+		i = 1;
+		while (offCur < (int) tvb_reported_length (tvb)) {
+			b0 = tvb_get_guint8 (tvb, offCur + 0);
+			if (b0 & 0x80) {
+				vHeader = 2;
+				cbHeader = ((b0 & 0x7f) << 8) |
+							tvb_get_guint8 (tvb, offCur + 1);
+			} else {
+				vHeader = 1;
+				cbHeader = b0;
+			}
+		    if (tree) {
+				proto_tree_add_item(wtp_tree, hf_wtp_header_sub_pdu_size, tvb, offCur, vHeader, bo_big_endian);
+			}
+			if (i > 1 && check_col(pinfo->cinfo, COL_INFO)) {
+				col_append_str (pinfo->cinfo, COL_INFO, ", ");
+			}
+			wsp_tvb = tvb_new_subset(tvb,
+				offCur + vHeader, -1, cbHeader);
+			dissect_wtp_common (wsp_tvb, pinfo, wtp_tree);
+			offCur += vHeader + cbHeader;
+			i++;
+		}
+		return;
+	}
 	fCon = b0 & 0x80;
 	fRID = retransmission_indicator( b0 );
 	pdut = pdu_type( b0 );
@@ -258,6 +288,20 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if( fRID ) {
 		strcat( szInfo, " R" );
 	};
+	if( fCon ) {
+		unsigned char	tCon;
+		unsigned char	tByte;
+
+		do {
+			tByte = tvb_get_guint8(tvb, offCur + cbHeader + vHeader);
+			tCon = tByte & 0x80;
+			if (tByte & 0x04)
+				vHeader = vHeader + tvb_get_guint8(tvb,
+					offCur + cbHeader + vHeader + 1) + 2;
+			else
+				vHeader = vHeader + (tByte & 0x03) + 1;
+		} while (tCon);
+	}
 
 #ifdef DEBUG
 	fprintf( stderr, "dissect_wtp: cbHeader = %d\n", cbHeader );  
@@ -266,11 +310,12 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 /* This field shows up as the "Info" column in the display; you should make
    it, if possible, summarize what's in the packet, so that a user looking
    at the list of packets can tell what type of packet it is. */
-	if (check_col(fdata, COL_INFO)) {
+	if (check_col(pinfo->cinfo, COL_INFO) &&
+		tvb_reported_length (tvb) <= cbHeader + vHeader) {
 #ifdef DEBUG
 		fprintf( stderr, "dissect_wtp: (6) About to set info_col header to %s\n", szInfo );
 #endif
-		col_add_str(fdata, COL_INFO, szInfo );
+		col_append_str(pinfo->cinfo, COL_INFO, szInfo );
 	};
 /* In the interest of speed, if "tree" is NULL, don't do any work not
    necessary to generate protocol tree items. */
@@ -278,7 +323,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 #ifdef DEBUG
 		fprintf( stderr, "dissect_wtp: cbHeader = %d\n", cbHeader );  
 #endif
-		ti = proto_tree_add_item(tree, proto_wtp, tvb, offCur, cbHeader, bo_little_endian);
+		ti = proto_tree_add_item(tree, proto_wtp, tvb, offCur, cbHeader + vHeader, bo_little_endian);
 #ifdef DEBUG
 		fprintf( stderr, "dissect_wtp: (7) Returned from proto_tree_add_item\n" );  
 #endif
@@ -396,8 +441,8 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 						wtp_tree, 			/* tree */
 						hf_wtp_header_variable_part, 	/* id */
 						tvb, 
-						offCur + 4, 			/* start */
-						4,				/* length */
+						offCur + cbHeader,		/* start */
+						vHeader,			/* length */
 						"What should go here!",		/* value */
 						"Header (Variable part) %02X %02X %02X %02X"  ,			/* format */
 						0, 1, 2, 3
@@ -414,16 +459,12 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	/* Any remaining data ought to be WSP data,
 	 * so hand off to the WSP dissector */
-	if (tvb_reported_length (tvb) > cbHeader)
+	if (tvb_reported_length (tvb) > cbHeader + vHeader)
 	{
-		wsp_tvb = tvb_new_subset(tvb, cbHeader, -1,
-			tvb_reported_length (tvb)-cbHeader);
+		wsp_tvb = tvb_new_subset(tvb, cbHeader + vHeader, -1,
+			tvb_reported_length (tvb)-cbHeader-vHeader);
 		call_dissector(wsp_handle, wsp_tvb, pinfo, tree);
 	}
-
-#ifdef DEBUG
-	fprintf( stderr, "dissect_wtp: (leaving) fdata->cinfo is %p\n", fdata->cinfo ); 
-#endif
 }
 
 /*
@@ -433,8 +474,11 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 static void
 dissect_wtp_fromudp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "WTP+WSP" );
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "WTP+WSP" );
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_clear(pinfo->cinfo, COL_INFO);
+	}
 
 	dissect_wtp_common(tvb, pinfo, tree);
 }
@@ -451,8 +495,11 @@ dissect_wtp_fromudp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 static void
 dissect_wtp_fromwap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "WTLS+WTP+WSP" );
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "WTLS+WTP+WSP" );
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_clear(pinfo->cinfo, COL_INFO);
+	}
 
 	dissect_wtp_common(tvb, pinfo, tree);
 }
@@ -499,6 +546,13 @@ proto_register_wtp(void)
 				"Fixed part of the header", HFILL
 			}
 		},
+		{ &hf_wtp_header_sub_pdu_size,
+			{ 	"Sub PDU size",           
+				"wtp.sub_pdu_size",
+				FT_BYTES, BASE_HEX, NULL, 0x0,
+				"Size of Sub-PDU", HFILL
+			}
+		},
 		{ &hf_wtp_header_flag_continue,
 			{ 	"Continue Flag",           
 				"wtp.continue_flag",
@@ -516,8 +570,8 @@ proto_register_wtp(void)
 		{ &hf_wtp_header_flag_Trailer,
 			{ 	"Trailer Flags",           
 				"wtp.trailer_flags",
-				 FT_UINT8, BASE_HEX, VALS( vals_transmission_trailer ), 0x06,
-				"PDU Type", HFILL
+				 FT_UINT8, BASE_HEX, VALS( vals_transaction_trailer ), 0x06,
+				"Trailer Flags", HFILL
 			}
 		},
 		{ &hf_wtp_header_flag_RID,
@@ -535,10 +589,10 @@ proto_register_wtp(void)
 			}
 		},
 		{ &hf_wtp_header_flag_TID,
-			{ 	"Transmission ID",           
+			{ 	"Transaction ID",           
 				"wtp.TID",
 				 FT_UINT16, BASE_HEX, NULL, 0x7FFF,
-				"Transmission ID", HFILL
+				"Transaction ID", HFILL
 			}
 		},
 		{ &hf_wtp_header_Inv_version,
@@ -572,7 +626,7 @@ proto_register_wtp(void)
 		{ &hf_wtp_header_Inv_TransactionClass,
 			{ 	"Transaction Class",           
 				"wtp.inv.transaction_class",
-				 FT_UINT8, BASE_HEX, NULL, 0x03,
+				 FT_UINT8, BASE_HEX, VALS( vals_transaction_classes ), 0x03,
 				"Transaction Class", HFILL
 			}
 		},
@@ -655,16 +709,20 @@ proto_register_wtp(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 	register_dissector("wtp", dissect_wtp_fromwap, proto_wtp);
+	register_dissector("wtp-udp", dissect_wtp_fromudp, proto_wtp);
 };
 
 void
 proto_reg_handoff_wtp(void)
 {
-	/*
-	 * Get a handle for the IP WSP dissector - if WTP PDUs have data, it is WSP
-	 */
-	wsp_handle = find_dissector("wsp");
+	dissector_handle_t wtp_fromudp_handle;
 
-	dissector_add("udp.port", UDP_PORT_WTP_WSP, dissect_wtp_fromudp,
-	    proto_wtp);
+	/*
+	 * Get a handle for the connection-oriented WSP dissector - if WTP
+	 * PDUs have data, it is WSP.
+	 */
+	wsp_handle = find_dissector("wsp-co");
+
+	wtp_fromudp_handle = find_dissector("wtp-udp");
+	dissector_add("udp.port", UDP_PORT_WTP_WSP, wtp_fromudp_handle);
 }
