@@ -2,7 +2,7 @@
  * Routines for X11 dissection
  * Copyright 2000, Christophe Tronche <ch.tronche@computer.org>
  *
- * $Id: packet-x11.c,v 1.21 2001/06/18 02:17:54 guy Exp $
+ * $Id: packet-x11.c,v 1.27 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -72,6 +72,8 @@ static int proto_x11 = -1;
 static gint ett_x11 = -1;
 static gint ett_x11_request = -1;
 
+static dissector_handle_t data_handle;
+
 #define TCP_PORT_X11			6000
 #define TCP_PORT_X11_2			6001
 #define TCP_PORT_X11_3			6002
@@ -794,7 +796,7 @@ static void listOfArc(tvbuff_t *tvb, int hf, int length)
 	    gint16 angle2 = VALUE16(tvb, cur_offset + 10);
 
 	    proto_tree *ttt = proto_tree_add_none_format(tt, hf_x11_arc, tvb, cur_offset, 12, 
-							     "arc: %dx%d+%d+%d, angle %d -> %d (%f -> %f)",
+							     "arc: %dx%d+%d+%d, angle %d -> %d (%f degrees -> %f degrees)",
 							     width, height, x, y, angle1, angle2,
 							     angle1 / 64.0, angle2 / 64.0);
 	    proto_tree_add_int(ttt, hf_x11_arc_x, tvb, cur_offset, 2, x); cur_offset += 2;
@@ -825,7 +827,7 @@ static void listOfByte(tvbuff_t *tvb, int hf, int length)
 	    length = next_offset -  cur_offset;
       }
       if (length <= 0) length = 1;
-      proto_tree_add_bytes(t, hf, tvb, cur_offset, length, tvb_get_ptr(tvb, cur_offset, length));
+      proto_tree_add_item(t, hf, tvb, cur_offset, length, little_endian);
       cur_offset += length;
 }
 
@@ -1043,7 +1045,7 @@ static void stringCopy(char *dest, const char *source, int length)
 static void listOfString8(tvbuff_t *tvb, int hf, int hf_item, int length)
 {
       char *s = NULL;
-      int allocated = 0;
+      guint allocated = 0;
       proto_tree *tt;
       int i;
 
@@ -1060,7 +1062,7 @@ static void listOfString8(tvbuff_t *tvb, int hf, int hf_item, int length)
 
       while(length--) {
 	    unsigned l = VALUE8(tvb, cur_offset);
-	    if (allocated < l + 1) {
+	    if (allocated < (l + 1)) {
 		  /* g_realloc doesn't work ??? */
 		  g_free(s);
 		  s = g_malloc(l + 1);
@@ -1103,7 +1105,7 @@ static void string16_with_buffer_preallocated(tvbuff_t *tvb, proto_tree *t,
 		  truncated = TRUE;
 		  l = STRING16_MAX_DISPLAYED_LENGTH;
 	    }
-	    if (*sLength < l + 3) {
+	    if (*sLength < (int) l + 3) {
 		  g_free(*s);
 		  *s = g_malloc(l + 3);
 		  *sLength = l + 3;
@@ -1126,7 +1128,7 @@ static void string16_with_buffer_preallocated(tvbuff_t *tvb, proto_tree *t,
 	    proto_tree_add_string_format(t, hf, tvb, offset, length, tvb_get_ptr(tvb, offset, length), "%s: %s", 
 					proto_registrar_get_nth(hf) -> name, *s);
       } else
-	    proto_tree_add_bytes(t, hf_bytes, tvb, offset, length, tvb_get_ptr(tvb, offset, length));
+	    proto_tree_add_item(t, hf_bytes, tvb, offset, length, little_endian);
 
 }
 
@@ -1163,7 +1165,7 @@ static void listOfTextItem(tvbuff_t *tvb, int hf, int sizeIs16)
 		  proto_tree *ttt;
 		  gint8 delta = VALUE8(tvb, cur_offset + 1);
 		  if (sizeIs16) l += l;
-		  if (allocated < l + 1) {
+		  if ((unsigned) allocated < l + 1) {
 			/* g_realloc doesn't work ??? */
 			g_free(s);
 			s = g_malloc(l + 1);
@@ -2845,8 +2847,8 @@ dissect_x11_request(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 /* This field shows up as the "Info" column in the display; you should make
    it, if possible, summarize what's in the packet, so that a user looking
    at the list of packets can tell what type of packet it is. */
-      if (check_col(pinfo->fd, COL_INFO)) 
-	    col_set_str(pinfo->fd, COL_INFO, "X11 request");
+      if (check_col(pinfo->cinfo, COL_INFO)) 
+	    col_set_str(pinfo->cinfo, COL_INFO, "X11 request");
 
 /* In the interest of speed, if "tree" is NULL, don't do any work not
    necessary to generate protocol tree items. */
@@ -2859,7 +2861,7 @@ dissect_x11_request(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       little_endian = guess_byte_ordering(tvb, pinfo);
       left = dissect_x11_request_loop(tvb, x11_tree);
       if (left)
-	    dissect_data(tvb, cur_offset, pinfo, x11_tree);
+	    call_dissector(data_handle,tvb_new_subset(tvb, cur_offset,-1, tvb_reported_length_remaining(tvb,cur_offset)), pinfo, x11_tree);
 }
 
 static void
@@ -2872,8 +2874,8 @@ dissect_x11_event(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 /* This field shows up as the "Info" column in the display; you should make
    it, if possible, summarize what's in the packet, so that a user looking
    at the list of packets can tell what type of packet it is. */
-      if (check_col(pinfo->fd, COL_INFO)) 
-	    col_set_str(pinfo->fd, COL_INFO, "X11 event");
+      if (check_col(pinfo->cinfo, COL_INFO)) 
+	    col_set_str(pinfo->cinfo, COL_INFO, "X11 event");
 
 /* In the interest of speed, if "tree" is NULL, don't do any work not
    necessary to generate protocol tree items. */
@@ -2884,15 +2886,15 @@ dissect_x11_event(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 /* Code to process the packet goes here */
 
-	    dissect_data(tvb, 0, pinfo, x11_tree);
+	    call_dissector(data_handle,tvb, pinfo, x11_tree);
       }
 }
 
 static void
 dissect_x11(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-      if (check_col(pinfo->fd, COL_PROTOCOL)) 
-	    col_set_str(pinfo->fd, COL_PROTOCOL, "X11");
+      if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+	    col_set_str(pinfo->cinfo, COL_PROTOCOL, "X11");
     
       if (pinfo->match_port == pinfo->destport)
 	    dissect_x11_request(tvb, pinfo, tree);
@@ -2940,7 +2942,11 @@ void proto_register_x11(void)
 void
 proto_reg_handoff_x11(void)
 {
-  dissector_add("tcp.port", TCP_PORT_X11, dissect_x11, proto_x11);
-  dissector_add("tcp.port", TCP_PORT_X11_2, dissect_x11, proto_x11);
-  dissector_add("tcp.port", TCP_PORT_X11_3, dissect_x11, proto_x11);
+  dissector_handle_t x11_handle;
+
+  x11_handle = create_dissector_handle(dissect_x11, proto_x11);
+  dissector_add("tcp.port", TCP_PORT_X11, x11_handle);
+  dissector_add("tcp.port", TCP_PORT_X11_2, x11_handle);
+  dissector_add("tcp.port", TCP_PORT_X11_3, x11_handle);
+  data_handle = find_dissector("data");
 }
