@@ -3,12 +3,11 @@
  *
  * Copyright 2000, Paul Ionescu	<paul@acorp.ro>
  *
- * $Id: packet-xot.c,v 1.5 2001/02/12 09:06:17 guy Exp $
+ * $Id: packet-xot.c,v 1.8 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +39,9 @@
 #define TCP_PORT_XOT 1998
 
 static gint proto_xot = -1;
+static gint hf_xot_version = -1;
+static gint hf_xot_length = -1;
+
 static gint ett_xot = -1;
 
 static dissector_handle_t x25_handle;
@@ -51,53 +53,64 @@ static void dissect_xot(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint16 version,len;
   tvbuff_t   *next_tvb; 
     
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-      col_set_str(pinfo->fd, COL_PROTOCOL, "XOT");
-  if (check_col(pinfo->fd, COL_INFO))
-      col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+      col_set_str(pinfo->cinfo, COL_PROTOCOL, "XOT");
+  if (check_col(pinfo->cinfo, COL_INFO))
+      col_clear(pinfo->cinfo, COL_INFO);
 
   version = tvb_get_ntohs(tvb,0);
   len     = tvb_get_ntohs(tvb,2);
 
-  if (check_col(pinfo->fd, COL_INFO)) 
-     col_add_fstr(pinfo->fd, COL_INFO, "XOT Version = %u, size = %u",version,len );
+  if (check_col(pinfo->cinfo, COL_INFO)) 
+     col_add_fstr(pinfo->cinfo, COL_INFO, "XOT Version = %u, size = %u",version,len );
 
   if (tree) {
 	
       ti = proto_tree_add_protocol_format(tree, proto_xot, tvb, 0, 4, "X.25 over TCP");
       xot_tree = proto_item_add_subtree(ti, ett_xot);
-      
-      ti = proto_tree_add_text(xot_tree, tvb, 0,2,"XOT Version : %u %s",version,(version==0?"":" - Unknown version")) ;
-      ti = proto_tree_add_text(xot_tree, tvb, 2,2,"XOT length : %u",len) ;
+     
+      proto_tree_add_uint(xot_tree, hf_xot_version, tvb, 0, 2, version);
+      proto_tree_add_uint(xot_tree, hf_xot_length, tvb, 2, 2, len);
+
   }
   next_tvb =  tvb_new_subset(tvb,4, -1 , -1);
   call_dissector(x25_handle,next_tvb,pinfo,tree);
 }
  
 /* Register the protocol with Ethereal */
-void proto_register_xot(void)
-{                 
+void 
+proto_register_xot(void)
+{
+	static hf_register_info hf[] = {
+		{ &hf_xot_version,
+			{ "Version", "xot.version", FT_UINT16, BASE_DEC,
+			NULL, 0, "Version of X.25 over TCP protocol", HFILL }},
 
+		{ &hf_xot_length,
+			{ "Length", "xot.length", FT_UINT16, BASE_DEC,
+			NULL, 0, "Length of X.25 over TCP packet", HFILL }}
 
-  /* Setup protocol subtree array */
-  static gint *ett[] = {
-    &ett_xot,
-  };
+	};
 
-  /* Register the protocol name and description */
-  proto_xot = proto_register_protocol("X.25 over TCP", "XOT", "xot");
+	static gint *ett[] = {
+		&ett_xot,
+	};
 
-  /* Required function calls to register the header fields and subtrees used */
-  proto_register_subtree_array(ett, array_length(ett));
+	proto_xot = proto_register_protocol("X.25 over TCP", "XOT", "xot");
+	proto_register_field_array(proto_xot, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 };
 
 void
 proto_reg_handoff_xot(void)
 {
-  /*
-   * Get a handle for the X.25 dissector.
-   */
-  x25_handle = find_dissector("x.25");
+	dissector_handle_t xot_handle;
 
-  dissector_add("tcp.port", TCP_PORT_XOT, dissect_xot, proto_xot);
+	/*
+	 * Get a handle for the X.25 dissector.
+	 */
+	x25_handle = find_dissector("x.25");
+
+	xot_handle = create_dissector_handle(dissect_xot, proto_xot);
+	dissector_add("tcp.port", TCP_PORT_XOT, xot_handle);
 }
