@@ -2,7 +2,7 @@
  * Routines for yahoo messenger packet dissection
  * Copyright 1999, Nathan Neulinger <nneul@umr.edu>
  *
- * $Id: packet-yhoo.c,v 1.17 2001/07/03 04:56:46 guy Exp $
+ * $Id: packet-yhoo.c,v 1.20 2001/12/17 23:17:34 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -111,10 +111,6 @@ dissect_yhoo(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int offset = 0;
 	int length = 0;
 
-	if (!proto_is_protocol_enabled(proto_yhoo)) {
-		return FALSE;
-	}
-  
 	if (pinfo->srcport != TCP_PORT_YHOO && pinfo->destport != TCP_PORT_YHOO) {
 		/* Not the Yahoo port - not a Yahoo Messenger packet. */
 		return FALSE;
@@ -136,12 +132,12 @@ dissect_yhoo(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		return FALSE;
 	}
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "YHOO");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "YHOO");
 
 	offset = 0;
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_fstr(pinfo->fd, COL_INFO, 
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_fstr(pinfo->cinfo, COL_INFO, 
 			"%s: %s", 
 			( strncmp(tvb_get_ptr(tvb, offset + 0, 4), "YPNS", 4) == 0 ) ? "Request" : "Response",
 			val_to_str(tvb_get_letohl(tvb, offset + 12),
@@ -154,44 +150,44 @@ dissect_yhoo(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			tvb_length_remaining(tvb, offset), FALSE);
 		yhoo_tree = proto_item_add_subtree(ti, ett_yhoo);
 
-		proto_tree_add_string(yhoo_tree, hf_yhoo_version, tvb, 
-			offset, 8, tvb_get_ptr(tvb, offset, 8));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_version, tvb, 
+			offset, 8, TRUE);
 		offset += 8;
 
-		proto_tree_add_uint(yhoo_tree, hf_yhoo_len, tvb, 
-			offset, 4, tvb_get_letohl(tvb, offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_len, tvb, 
+			offset, 4, TRUE);
 		offset += 4;
 
-		proto_tree_add_uint(yhoo_tree, hf_yhoo_service, tvb, 
-			offset, 4, tvb_get_letohl(tvb, offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_service, tvb, 
+			offset, 4, TRUE);
 		offset += 4;
 
-		proto_tree_add_uint(yhoo_tree, hf_yhoo_connection_id, tvb, 
-			offset, 4, tvb_get_letohl(tvb, offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_connection_id, tvb, 
+			offset, 4, TRUE);
 		offset += 4;
 
-		proto_tree_add_uint(yhoo_tree, hf_yhoo_magic_id, tvb, 
-			offset, 4, tvb_get_letohl(tvb, offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_magic_id, tvb, 
+			offset, 4, TRUE);
 		offset += 4;
 
-		proto_tree_add_uint(yhoo_tree, hf_yhoo_unknown1, tvb, 
-			offset, 4, tvb_get_letohl(tvb, offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_unknown1, tvb, 
+			offset, 4, TRUE);
 		offset += 4;
 
-		proto_tree_add_uint(yhoo_tree, hf_yhoo_msgtype, tvb, 
-			offset, 4, tvb_get_letohl(tvb, offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_msgtype, tvb, 
+			offset, 4, TRUE);
 		offset += 4;
 
-		proto_tree_add_string(yhoo_tree, hf_yhoo_nick1, tvb, 
-			offset, 36, tvb_get_ptr(tvb, offset, 36));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_nick1, tvb, 
+			offset, 36, TRUE);
 		offset += 36;
 
-		proto_tree_add_string(yhoo_tree, hf_yhoo_nick2, tvb, 
-			offset, 36, tvb_get_ptr(tvb, offset, 36));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_nick2, tvb, 
+			offset, 36, TRUE);
 		offset += 36;
 
-		proto_tree_add_string(yhoo_tree, hf_yhoo_content, tvb, 
-			offset, length, tvb_get_ptr(tvb, offset, length-offset));
+		proto_tree_add_item(yhoo_tree, hf_yhoo_content, tvb, 
+			offset, length-offset, TRUE);
 	}
 
 	return TRUE;
