@@ -3,12 +3,11 @@
  *
  * Jochen Friedrich <jochen@scram.de>
  *
- * $Id: packet-zebra.c,v 1.13 2001/06/18 02:17:58 guy Exp $
+ * $Id: packet-zebra.c,v 1.17 2001/12/10 00:25:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,8 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -214,9 +211,8 @@ dissect_zebra_request(proto_tree *tree, gboolean request, tvbuff_t *tvb,
 			if (request) break;
 			/* Request just subscribes to messages */
 
-			proto_tree_add_string(tree, hf_zebra_interface, 
-				tvb, offset, INTERFACE_NAMSIZ, 
-				tvb_get_ptr(tvb, offset, INTERFACE_NAMSIZ));
+			proto_tree_add_item(tree, hf_zebra_interface, 
+				tvb, offset, INTERFACE_NAMSIZ, FALSE);
 			offset += INTERFACE_NAMSIZ;
 
 			proto_tree_add_item(tree, hf_zebra_index, tvb,
@@ -241,9 +237,8 @@ dissect_zebra_request(proto_tree *tree, gboolean request, tvbuff_t *tvb,
 
 			break;
 		case ZEBRA_INTERFACE_DELETE:
-			proto_tree_add_string(tree, hf_zebra_interface,
-				tvb, offset, INTERFACE_NAMSIZ,
-				tvb_get_ptr(tvb, offset, INTERFACE_NAMSIZ));
+			proto_tree_add_item(tree, hf_zebra_interface,
+				tvb, offset, INTERFACE_NAMSIZ, FALSE);
 			offset += INTERFACE_NAMSIZ;
 
 			proto_tree_add_item(tree, hf_zebra_index, tvb,
@@ -320,7 +315,7 @@ dissect_zebra_request(proto_tree *tree, gboolean request, tvbuff_t *tvb,
 			prefix = tvb_get_ptr(tvb, offset, PSIZE(prefixlen));
 			prefix4 = 0;
 			memcpy(&prefix4, prefix,
-			    MIN(PSIZE(prefixlen), sizeof prefix4));
+			    MIN((unsigned) PSIZE(prefixlen), sizeof prefix4));
 			proto_tree_add_ipv4(tree, hf_zebra_prefix4,
 				tvb, offset, PSIZE(prefixlen), prefix4);
 			offset += PSIZE(prefixlen);
@@ -398,7 +393,7 @@ dissect_zebra_request(proto_tree *tree, gboolean request, tvbuff_t *tvb,
 			prefix = tvb_get_ptr(tvb, offset, PSIZE(prefixlen));
 			memset(buffer6, '\0', sizeof buffer6);
 			memcpy(buffer6, prefix,
-			    MIN(PSIZE(prefixlen), sizeof buffer6));
+			    MIN((unsigned) PSIZE(prefixlen), sizeof buffer6));
 			proto_tree_add_ipv6(tree, hf_zebra_prefix6,
 				tvb, offset, PSIZE(prefixlen), buffer6);
 			offset += PSIZE(prefixlen);
@@ -476,15 +471,15 @@ dissect_zebra(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gboolean	request;
 	int		left, offset;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_set_str(pinfo->fd, COL_PROTOCOL, "ZEBRA");
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "ZEBRA");
 
 	request = (pinfo->destport == pinfo->match_port);
 	left = tvb_reported_length(tvb);
 	offset = 0;
 
-	if (check_col(pinfo->fd, COL_INFO)) {
-		col_add_str(pinfo->fd, COL_INFO, 
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_add_str(pinfo->cinfo, COL_INFO, 
 			request? "ZEBRA Request" : "ZEBRA Reply");
 	}
 	if (tree) {
@@ -650,5 +645,8 @@ proto_register_zebra(void)
 void
 proto_reg_handoff_zebra(void)
 {
-  dissector_add("tcp.port", TCP_PORT_ZEBRA, dissect_zebra, proto_zebra);
+  dissector_handle_t zebra_handle;
+
+  zebra_handle = create_dissector_handle(dissect_zebra, proto_zebra);
+  dissector_add("tcp.port", TCP_PORT_ZEBRA, zebra_handle);
 }
