@@ -1,12 +1,13 @@
 /* packet-gryphon.c
  * Routines for Gryphon protocol packet disassembly
- *
- * $Id: packet-gryphon.c,v 1.23 2001/07/08 21:51:32 hagbard Exp $
- *
- * Ethereal - Network traffic analyzer
  * By Steve Limkemann <stevelim@dgtech.com>
  * Copyright 1998 Steve Limkemann
  *
+ * $Id: packet-gryphon.c,v 1.26 2001/12/10 00:26:20 guy Exp $
+ *
+ * Ethereal - Network traffic analyzer
+ * By Gerald Combs <gerald@ethereal.com>
+ * Copyright 1998
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,8 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,6 +47,8 @@
 #include "packet.h"
 #include "packet-gryphon.h"
 
+#include "plugins/plugin_api_defs.h"
+
 #ifndef __ETHEREAL_STATIC__
 G_MODULE_EXPORT const gchar version[] = VERSION;
 #endif
@@ -156,24 +157,24 @@ dissect_gryphon(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     proto_tree *gryphon_tree;
     guint8 frmtyp;
 
-    if (check_col(pinfo->fd, COL_PROTOCOL))
-	col_set_str(pinfo->fd, COL_PROTOCOL, "Gryphon");
-    if (check_col(pinfo->fd, COL_INFO))
-	col_clear(pinfo->fd, COL_INFO);
+    if (check_col(pinfo->cinfo, COL_PROTOCOL))
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "Gryphon");
+    if (check_col(pinfo->cinfo, COL_INFO))
+	col_clear(pinfo->cinfo, COL_INFO);
 
     ti = proto_tree_add_item(tree, proto_gryphon, tvb, 0,
 	    	    tvb_length(tvb), FALSE);
     gryphon_tree = proto_item_add_subtree(ti, ett_gryphon);
 
-    if (check_col(pinfo->fd, COL_INFO)) {
+    if (check_col(pinfo->cinfo, COL_INFO)) {
 	/*
 	 * Indicate what kind of message the first message is.
 	 */
 	frmtyp = tvb_get_guint8(tvb, 6) & ~RESPONSE_FLAGS;
 	if (frmtyp >= SIZEOF (frame_type))
-    	    col_set_str(pinfo->fd, COL_INFO, "- Invalid -");
+    	    col_set_str(pinfo->cinfo, COL_INFO, "- Invalid -");
 	else
-    	    col_set_str(pinfo->fd, COL_INFO, frame_type[frmtyp]);
+    	    col_set_str(pinfo->cinfo, COL_INFO, frame_type[frmtyp]);
     }
 
     if (tree) {
@@ -1871,7 +1872,10 @@ proto_register_gryphon(void)
 void
 proto_reg_handoff_gryphon(void)
 {
-    dissector_add("tcp.port", 7000, &dissect_gryphon, proto_gryphon);
+    dissector_handle_t gryphon_handle;
+
+    gryphon_handle = create_dissector_handle(dissect_gryphon, proto_gryphon);
+    dissector_add("tcp.port", 7000, gryphon_handle);
 }
 
 /* Start the functions we need for the plugin stuff */
