@@ -2,13 +2,14 @@
  * Routines for mgcp packet disassembly
  * RFC 2705
  *
- * $Id: packet-mgcp.c,v 1.26 2001/07/08 21:51:34 hagbard Exp $
+ * $Id: packet-mgcp.c,v 1.29 2001/12/10 00:26:21 guy Exp $
  * 
  * Copyright (c) 2000 by Ed Warnicke <hagbard@physics.rutgers.edu>
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1999 Gerald Combs
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -51,6 +52,8 @@
 #include "prefs.h"
 #include "strutil.h"
 
+#include "plugins/plugin_api_defs.h"
+
 #ifndef __ETHEREAL_STATIC__
 G_MODULE_EXPORT const gchar version[] = VERSION;
 #endif
@@ -214,10 +217,10 @@ dissect_mgcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    * Set the columns now, so that they'll be set correctly if we throw
    * an exception.  We can set them later as well....
    */
-  if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_add_str(pinfo->fd, COL_PROTOCOL, "MGCP");
-  if (check_col(pinfo->fd, COL_INFO))
-    col_clear(pinfo->fd, COL_INFO);
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_add_str(pinfo->cinfo, COL_PROTOCOL, "MGCP");
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_clear(pinfo->cinfo, COL_INFO);
 
   /* 
    * Check to see whether we're really dealing with MGCP by looking 
@@ -263,24 +266,24 @@ dissect_mgcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      * in order to prevent the column info changing to reflect the SDP.
      */
     tvb_sectionbegin = 0;
-    if (check_col(pinfo->fd, COL_PROTOCOL)){
+    if (check_col(pinfo->cinfo, COL_PROTOCOL)){
       if( global_mgcp_message_count == TRUE ){
 	if(num_messages > 1){
-	  col_add_fstr(pinfo->fd, COL_PROTOCOL, "MGCP (%i messages)",num_messages);
+	  col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "MGCP (%i messages)",num_messages);
 	}
 	else {
-	  col_add_fstr(pinfo->fd, COL_PROTOCOL, "MGCP (%i message)",num_messages);
+	  col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "MGCP (%i message)",num_messages);
 	}
       }
       else {
-	  col_add_str(pinfo->fd, COL_PROTOCOL, "MGCP");
+	  col_add_str(pinfo->cinfo, COL_PROTOCOL, "MGCP");
       }
     }
 	
-    if (check_col(pinfo->fd, COL_INFO) ){
+    if (check_col(pinfo->cinfo, COL_INFO) ){
       sectionlen = tvb_find_line_end(tvb, tvb_sectionbegin,-1,
 				     &tvb_sectionend);
-      col_add_fstr(pinfo->fd,COL_INFO, "%s", 
+      col_add_fstr(pinfo->cinfo, COL_INFO, "%s", 
 		   tvb_format_text(tvb,tvb_sectionbegin,sectionlen));
     } 
   }  
@@ -563,20 +566,22 @@ void
 proto_reg_handoff_mgcp(void)
 {
   static int mgcp_prefs_initialized = FALSE;
+  static dissector_handle_t mgcp_handle;
 
   /*
    * Get a handle for the SDP dissector.
    */
   sdp_handle = find_dissector("sdp");
 
-  if (mgcp_prefs_initialized) {
-    dissector_delete("tcp.port", gateway_tcp_port, dissect_mgcp);
-    dissector_delete("udp.port", gateway_udp_port, dissect_mgcp);
-    dissector_delete("tcp.port", callagent_tcp_port, dissect_mgcp);
-    dissector_delete("udp.port", callagent_udp_port, dissect_mgcp);
+  if (!mgcp_prefs_initialized) {
+    mgcp_handle = create_dissector_handle(dissect_mgcp, proto_mgcp);
+    mgcp_prefs_initialized = TRUE;
   }
   else {
-    mgcp_prefs_initialized = TRUE;
+    dissector_delete("tcp.port", gateway_tcp_port, mgcp_handle);
+    dissector_delete("udp.port", gateway_udp_port, mgcp_handle);
+    dissector_delete("tcp.port", callagent_tcp_port, mgcp_handle);
+    dissector_delete("udp.port", callagent_udp_port, mgcp_handle);
   }
 
   /* Set our port number for future use */
@@ -587,14 +592,10 @@ proto_reg_handoff_mgcp(void)
   callagent_tcp_port = global_mgcp_callagent_tcp_port;
   callagent_udp_port = global_mgcp_callagent_udp_port;
 
-  dissector_add("tcp.port", global_mgcp_gateway_tcp_port, dissect_mgcp,
-		proto_mgcp);
-  dissector_add("udp.port", global_mgcp_gateway_udp_port, dissect_mgcp,
-		proto_mgcp);
-  dissector_add("tcp.port", global_mgcp_callagent_tcp_port, dissect_mgcp,
-		proto_mgcp);
-  dissector_add("udp.port", global_mgcp_callagent_udp_port, dissect_mgcp,
-		proto_mgcp);
+  dissector_add("tcp.port", global_mgcp_gateway_tcp_port, mgcp_handle);
+  dissector_add("udp.port", global_mgcp_gateway_udp_port, mgcp_handle);
+  dissector_add("tcp.port", global_mgcp_callagent_tcp_port, mgcp_handle);
+  dissector_add("udp.port", global_mgcp_callagent_udp_port, mgcp_handle);
 
 }
 
@@ -1271,5 +1272,3 @@ plugin_init(plugin_address_table_t *pat){
 #endif
 
 /* End the functions we need for plugin stuff */
-
-
