@@ -1,11 +1,10 @@
 /* plugin_api.c
  * Routines for Ethereal plugins.
  *
- * $Id: plugin_api.c,v 1.19 2001/04/25 08:41:22 guy Exp $
+ * $Id: plugin_api.c,v 1.32 2001/12/03 10:00:22 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * Copyright 2000 by Gilbert Ramirez <gram@xiexie.org>
- *
+ * Copyright 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +19,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,12 +31,9 @@
 
 #include "plugin_api.h"
 
-packet_info			*p_pi = NULL;
-
 void
 plugin_address_table_init(plugin_address_table_t *pat)
 {
-	p_pi					= pat->p_pi;
 	p_check_col				= pat->p_check_col;
 	p_col_clear				= pat->p_col_clear;
 	p_col_add_fstr				= pat->p_col_add_fstr;
@@ -46,20 +41,27 @@ plugin_address_table_init(plugin_address_table_t *pat)
 	p_col_add_str				= pat->p_col_add_str;
 	p_col_append_str			= pat->p_col_append_str;
 	p_col_set_str				= pat->p_col_set_str;
+	p_register_init_routine			= pat->p_register_init_routine;
+	p_conversation_new			= pat->p_conversation_new;
+	p_find_conversation			= pat->p_find_conversation;
+	p_match_strval				= pat->p_match_strval;
+	p_val_to_str		 		= pat->p_val_to_str;
 	p_proto_register_protocol		= pat->p_proto_register_protocol;
 	p_proto_register_field_array		= pat->p_proto_register_field_array;
 	p_proto_register_subtree_array		= pat->p_proto_register_subtree_array;
 	p_dissector_add				= pat->p_dissector_add;
 	p_dissector_delete			= pat->p_dissector_delete;
+	p_dissector_add_handle			= pat->p_dissector_add_handle;
 	p_heur_dissector_add			= pat->p_heur_dissector_add;
 	p_register_dissector			= pat->p_register_dissector;
 	p_find_dissector			= pat->p_find_dissector;
+	p_create_dissector_handle		= pat->p_create_dissector_handle;
 	p_call_dissector			= pat->p_call_dissector;
-	p_dissect_data				= pat->p_dissect_data;
 	p_proto_is_protocol_enabled		= pat->p_proto_is_protocol_enabled;
 	p_proto_item_get_len			= pat->p_proto_item_get_len;
 	p_proto_item_set_len			= pat->p_proto_item_set_len;
 	p_proto_item_set_text			= pat->p_proto_item_set_text;
+	p_proto_item_append_text		= pat->p_proto_item_append_text;
 	p_proto_item_add_subtree		= pat->p_proto_item_add_subtree;
 	p_proto_tree_add_item			= pat->p_proto_tree_add_item;
 	p_proto_tree_add_item_hidden		= pat->p_proto_tree_add_item_hidden;
@@ -98,7 +100,6 @@ plugin_address_table_init(plugin_address_table_t *pat)
 	p_proto_tree_add_int_hidden		= pat->p_proto_tree_add_int_hidden;
 	p_proto_tree_add_int_format		= pat->p_proto_tree_add_int_format;
 	p_proto_tree_add_text			= pat->p_proto_tree_add_text;
-	p_proto_tree_add_notext			= pat->p_proto_tree_add_notext;
 	p_tvb_new_subset			= pat->p_tvb_new_subset;
 	p_tvb_length				= pat->p_tvb_length;
 	p_tvb_length_remaining			= pat->p_tvb_length_remaining;
@@ -110,15 +111,9 @@ plugin_address_table_init(plugin_address_table_t *pat)
 	p_tvb_get_ntohs				= pat->p_tvb_get_ntohs;
 	p_tvb_get_ntoh24			= pat->p_tvb_get_ntoh24;
 	p_tvb_get_ntohl				= pat->p_tvb_get_ntohl;
-#ifdef G_HAVE_GINT64
-	p_tvb_get_ntohll			= pat->p_tvb_get_ntohll;
-#endif
 	p_tvb_get_letohs			= pat->p_tvb_get_letohs;
 	p_tvb_get_letoh24			= pat->p_tvb_get_letoh24;
 	p_tvb_get_letohl			= pat->p_tvb_get_letohl;
-#ifdef G_HAVE_GINT64
-	p_tvb_get_letohll			= pat->p_tvb_get_letohll;
-#endif
 	p_tvb_memcpy				= pat->p_tvb_memcpy;
 	p_tvb_memdup				= pat->p_tvb_memdup;
 	p_tvb_get_ptr				= pat->p_tvb_get_ptr;
@@ -137,4 +132,34 @@ plugin_address_table_init(plugin_address_table_t *pat)
 	p_prefs_register_uint_preference	= pat->p_prefs_register_uint_preference;
 	p_prefs_register_bool_preference	= pat->p_prefs_register_bool_preference;
 	p_prefs_register_enum_preference	= pat->p_prefs_register_enum_preference;
+	p_prefs_register_string_preference	= pat->p_prefs_register_string_preference;
+
+	/* GIOP Begin */
+
+	p_register_giop_user			= pat->p_register_giop_user;
+	p_is_big_endian				= pat->p_is_big_endian;
+	p_get_CDR_encap_info                    = pat->p_get_CDR_encap_info;
+ 
+	p_get_CDR_any				= pat->p_get_CDR_any;
+	p_get_CDR_boolean			= pat->p_get_CDR_boolean;
+	p_get_CDR_char				= pat->p_get_CDR_char;
+	p_get_CDR_double			= pat->p_get_CDR_double;
+	p_get_CDR_enum				= pat->p_get_CDR_enum;
+	p_get_CDR_fixed				= pat->p_get_CDR_fixed;
+	p_get_CDR_float				= pat->p_get_CDR_float;
+	p_get_CDR_interface			= pat->p_get_CDR_interface;
+	p_get_CDR_long				= pat->p_get_CDR_long;
+	p_get_CDR_object			= pat->p_get_CDR_object;
+	p_get_CDR_octet 			= pat->p_get_CDR_octet;
+	p_get_CDR_octet_seq 			= pat->p_get_CDR_octet_seq;
+	p_get_CDR_short 			= pat->p_get_CDR_short;
+	p_get_CDR_string			= pat->p_get_CDR_string;
+	p_get_CDR_typeCode			= pat->p_get_CDR_typeCode;
+	p_get_CDR_ulong				= pat->p_get_CDR_ulong;
+	p_get_CDR_ushort			= pat->p_get_CDR_ushort;
+	p_get_CDR_wchar 			= pat->p_get_CDR_wchar;
+	p_get_CDR_wstring 			= pat->p_get_CDR_wstring;
+
+	/* GIOP End */
+
 }
