@@ -1,11 +1,10 @@
 /* plugin_api.h
  * Routines for Ethereal plugins.
  *
- * $Id: plugin_api.h,v 1.19 2001/04/25 08:41:22 guy Exp $
+ * $Id: plugin_api.h,v 1.32 2001/12/03 10:00:23 guy Exp $
  *
  * Ethereal - Network traffic analyzer
- * Copyright 2000 by Gilbert Ramirez <gram@xiexie.org>
- *
+ * Copyright 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,20 +19,21 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
 
 #ifdef PLUGINS_NEED_ADDRESS_TABLE
 
 /* Some OSes (Win32) have DLLs that cannot reference symbols in the parent
    executable, so the executable needs to provide a collection of pointers
-   to those functions for the DLL plugin to use. */
+   to global variables and functions for the DLL plugin to use. */
 
-/* #defines for those functions that call through pointers.
-   #defined in this fashion so that the declaration of the functions,
-   from the system header files, turn into declarations of pointers
-   to functions, and the calls to it in plugins, in the plugins, turn
-   into calls through the pointers. */
+/* #defines for those functions that are called through pointers,
+   and global variables that are referred to through pointers.
+
+   #defined in this fashion so that the declaration of the functions
+   and variables, from the system header files, turn into declarations
+   of pointers to functions and variables, and the references to them in
+   plugins, in the plugins, turn into references through the pointers. */
 #define	check_col			(*p_check_col)
 #define	col_clear			(*p_col_clear)
 #define	col_add_fstr			(*p_col_add_fstr)
@@ -42,26 +42,33 @@
 #define	col_append_str			(*p_col_append_str)
 #define	col_set_str			(*p_col_set_str)
 
+#define register_init_routine		(*p_register_init_routine)
+#define conversation_new		(*p_conversation_new)
+#define find_conversation		(*p_find_conversation)
+#define match_strval			(*p_match_strval)
+#define val_to_str			(*p_val_to_str)
+
 #define	proto_register_protocol		(*p_proto_register_protocol)
 #define	proto_register_field_array	(*p_proto_register_field_array)
 #define	proto_register_subtree_array	(*p_proto_register_subtree_array)
 
 #define	dissector_add			(*p_dissector_add)
 #define dissector_delete		(*p_dissector_delete)
+#define	dissector_add_handle		(*p_dissector_add_handle)
 
 #define	heur_dissector_add		(*p_heur_dissector_add)
 
 #define register_dissector		(*p_register_dissector)
 #define find_dissector			(*p_find_dissector)
+#define create_dissector_handle		(*p_create_dissector_handle)
 #define call_dissector			(*p_call_dissector)
 
-#define dissect_data			(*p_dissect_data)
-
 #define proto_is_protocol_enabled	(*p_proto_is_protocol_enabled)
 
 #define proto_item_get_len		(*p_proto_item_get_len)
 #define proto_item_set_len		(*p_proto_item_set_len)
 #define proto_item_set_text		(*p_proto_item_set_text)
+#define proto_item_append_text		(*p_proto_item_append_text)
 #define	proto_item_add_subtree		(*p_proto_item_add_subtree)
 #define	proto_tree_add_item		(*p_proto_tree_add_item)
 #define	proto_tree_add_item_hidden	(*p_proto_tree_add_item_hidden)
@@ -100,7 +107,6 @@
 #define	proto_tree_add_int_hidden	(*p_proto_tree_add_int_hidden)
 #define	proto_tree_add_int_format	(*p_proto_tree_add_int_format)
 #define	proto_tree_add_text		(*p_proto_tree_add_text)
-#define	proto_tree_add_notext		(*p_proto_tree_add_notext)
 
 #define tvb_new_subset			(*p_tvb_new_subset)
 
@@ -116,16 +122,10 @@
 #define tvb_get_ntohs			(*p_tvb_get_ntohs)
 #define tvb_get_ntoh24			(*p_tvb_get_ntoh24)
 #define tvb_get_ntohl			(*p_tvb_get_ntohl)
-#ifdef G_HAVE_GINT64
-#define tvb_get_ntohll			(*p_tvb_get_ntohll)
-#endif
 
 #define tvb_get_letohs			(*p_tvb_get_letohs)
 #define tvb_get_letoh24			(*p_tvb_get_letoh24)
 #define tvb_get_letohl			(*p_tvb_get_letohl)
-#ifdef G_HAVE_GINT64
-#define tvb_get_letohll			(*p_tvb_get_letohll)
-#endif
 
 #define tvb_memcpy			(*p_tvb_memcpy)
 #define tvb_memdup			(*p_tvb_memdup)
@@ -154,13 +154,43 @@
 #define prefs_register_uint_preference	(*p_prefs_register_uint_preference)
 #define prefs_register_bool_preference	(*p_prefs_register_bool_preference)
 #define prefs_register_enum_preference	(*p_prefs_register_enum_preference)
+#define prefs_register_string_preference (*p_prefs_register_string_preference)
 
-#define pi	(*p_pi)
+
+/* GIOP entries Begin */
+
+#define register_giop_user		(*p_register_giop_user)
+#define is_big_endian			(*p_is_big_endian)
+#define get_CDR_encap_info		(*p_get_CDR_encap_info)
+
+#define get_CDR_any			(*p_get_CDR_any)
+#define get_CDR_boolean			(*p_get_CDR_boolean)
+#define get_CDR_char			(*p_get_CDR_char)
+#define get_CDR_double			(*p_get_CDR_double)
+#define get_CDR_enum			(*p_get_CDR_enum)
+#define get_CDR_fixed			(*p_get_CDR_fixed)
+#define get_CDR_float			(*p_get_CDR_float)
+#define get_CDR_interface		(*p_get_CDR_interface)
+#define get_CDR_long         	        (*p_get_CDR_long)
+#define get_CDR_object			(*p_get_CDR_object)
+#define get_CDR_octet         	        (*p_get_CDR_octet)
+#define get_CDR_octet_seq     	        (*p_get_CDR_octet_seq)
+#define get_CDR_short         	        (*p_get_CDR_short)
+#define get_CDR_string			(*p_get_CDR_string)
+#define get_CDR_typeCode		(*p_get_CDR_typeCode)
+#define get_CDR_ulong			(*p_get_CDR_ulong)
+#define get_CDR_ushort			(*p_get_CDR_ushort)
+#define get_CDR_wchar			(*p_get_CDR_wchar)
+#define get_CDR_wstring			(*p_get_CDR_wstring)
+
+/* GIOP entries End */
 
 #endif
 
 #include "packet.h"
+#include "conversation.h"
 #include "prefs.h"
+#include "packet-giop.h"
 
 #include "plugin_table.h"
 
