@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c,v 1.53 2001/06/18 01:49:17 guy Exp $
+ * $Id: prefs.c,v 1.74 2001/12/08 01:45:35 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -30,10 +30,6 @@
 #include <sys/types.h>
 #endif
 
-#ifdef HAVE_DIRECT_H
-#include <direct.h>
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -43,11 +39,8 @@
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
+#include <glib.h>
 
-#include <epan.h>
 #include <filesystem.h>
 #include "globals.h"
 #include "packet.h"
@@ -60,17 +53,19 @@
 #include "prefs-int.h"
 
 /* Internal functions */
+static module_t *find_module(const char *name);
+static struct preference *find_preference(module_t *, const char *);
 static int    set_pref(gchar*, gchar*);
 static GList *get_string_list(gchar *);
+static gchar *put_string_list(GList *);
 static void   clear_string_list(GList *);
 static void   free_col_info(e_prefs *);
 
-#define PF_NAME "preferences"
-
-#define GPF_PATH	DATAFILE_DIR "/ethereal.conf"
+#define GPF_NAME	"ethereal.conf"
+#define PF_NAME		"preferences"
 
 static gboolean init_prefs = TRUE;
-static gchar *pf_path = NULL;
+static gchar *gpf_path = NULL;
 
 /*
  * XXX - variables to allow us to attempt to interpret the first
@@ -97,6 +92,15 @@ gchar	*gui_hex_dump_highlight_style_text[] =
  */
 static GList *modules;
 
+static gint
+module_compare_name(gconstpointer p1_arg, gconstpointer p2_arg)
+{
+	const module_t *p1 = p1_arg;
+	const module_t *p2 = p2_arg;
+
+	return g_strcasecmp(p1->name, p2->name);
+}
+
 /*
  * Register a module that will have preferences.
  * Specify the name used for the module in the preferences file, the
@@ -108,6 +112,7 @@ prefs_register_module(const char *name, const char *title,
     void (*apply_cb)(void))
 {
 	module_t *module;
+	const guchar *p;
 
 	module = g_malloc(sizeof (module_t));
 	module->name = name;
@@ -117,7 +122,29 @@ prefs_register_module(const char *name, const char *title,
 	module->numprefs = 0;
 	module->prefs_changed = FALSE;
 
-	modules = g_list_append(modules, module);
+	/*
+	 * Make sure that only lower-case ASCII letters, numbers,
+	 * underscores, and dots appear in the module name.
+	 *
+	 * Crash if there is, as that's an error in the code;
+	 * you can make the title a nice string with capitalization,
+	 * white space, punctuation, etc., but the name can be used
+	 * on the command line, and shouldn't require quoting,
+	 * shifting, etc.
+	 */
+	for (p = name; *p != '\0'; p++)
+		g_assert(isascii(*p) &&
+		    (islower(*p) || isdigit(*p) || *p == '_' || *p == '.'));
+
+	/*
+	 * Make sure there's not already a module with that
+	 * name.  Crash if there is, as that's an error in the
+	 * code, and the code has to be fixed not to register
+	 * more than one module with the same name.
+	 */
+	g_assert(find_module(name) == NULL);
+
+	modules = g_list_insert_sorted(modules, module, module_compare_name);
 
 	return module;
 }
@@ -146,11 +173,11 @@ module_match(gconstpointer a, gconstpointer b)
 }
 
 static module_t *
-find_module(char *name)
+find_module(const char *name)
 {
 	GList *list_entry;
 
-	list_entry = g_list_find_custom(modules, name, module_match);
+	list_entry = g_list_find_custom(modules, (gpointer)name, module_match);
 	if (list_entry == NULL)
 		return NULL;	/* no such module */
 	return (module_t *) list_entry->data;
@@ -215,6 +242,7 @@ register_preference(module_t *module, const char *name, const char *title,
     const char *description)
 {
 	pref_t *preference;
+	const guchar *p;
 
 	preference = g_malloc(sizeof (pref_t));
 	preference->name = name;
@@ -222,6 +250,32 @@ register_preference(module_t *module, const char *name, const char *title,
 	preference->description = description;
 	preference->ordinal = module->numprefs;
 
+	/*
+	 * Make sure that only lower-case ASCII letters, numbers,
+	 * underscores, and dots appear in the preference name.
+	 *
+	 * Crash if there is, as that's an error in the code;
+	 * you can make the title and description nice strings
+	 * with capitalization, white space, punctuation, etc.,
+	 * but the name can be used on the command line,
+	 * and shouldn't require quoting, shifting, etc.
+	 */
+	for (p = name; *p != '\0'; p++)
+		g_assert(isascii(*p) &&
+		    (islower(*p) || isdigit(*p) || *p == '_' || *p == '.'));
+
+	/*
+	 * Make sure there's not already a preference with that
+	 * name.  Crash if there is, as that's an error in the
+	 * code, and the code has to be fixed not to register
+	 * more than one preference with the same name.
+	 */
+	g_assert(find_preference(module, name) == NULL);
+
+	/*
+	 * There isn't already one with that name, so add the
+	 * preference.
+	 */
 	module->prefs = g_list_append(module->prefs, preference);
 	module->numprefs++;
 
@@ -242,11 +296,12 @@ preference_match(gconstpointer a, gconstpointer b)
 }
 
 static struct preference *
-find_preference(module_t *module, char *name)
+find_preference(module_t *module, const char *name)
 {
 	GList *list_entry;
 
-	list_entry = g_list_find_custom(module->prefs, name, preference_match);
+	list_entry = g_list_find_custom(module->prefs, (gpointer)name,
+	    preference_match);
 	if (list_entry == NULL)
 		return NULL;	/* no such preference */
 	return (struct preference *) list_entry->data;
@@ -332,6 +387,18 @@ prefs_register_string_preference(module_t *module, const char *name,
 	preference->saved_val.string = NULL;
 }
 
+/*
+ * Register a preference that used to be supported but no longer is.
+ */
+void
+prefs_register_obsolete_preference(module_t *module, const char *name)
+{
+	pref_t *preference;
+
+	preference = register_preference(module, name, NULL, NULL);
+	preference->type = PREF_OBSOLETE;
+}
+
 typedef struct {
 	pref_cb callback;
 	gpointer user_data;
@@ -343,6 +410,17 @@ do_pref_callback(gpointer data, gpointer user_data)
 	pref_t *pref = data;
 	pref_cb_arg_t *arg = user_data;
 
+	if (pref->type == PREF_OBSOLETE) {
+		/*
+		 * This preference is no longer supported; it's not a
+		 * real preference, so we don't call the callback for
+		 * it (i.e., we treat it as if it weren't found in the
+		 * list of preferences, and we weren't called in the
+		 * first place).
+		 */
+		return;
+	}
+
 	(*arg->callback)(pref, arg->user_data);
 }
 
@@ -368,55 +446,161 @@ prefs_register_modules(void)
 {
 }
 
-/* Parse through a list of comma-separated, quoted strings.  Return a
-   list of the string data */
+/* Parse through a list of comma-separated, possibly quoted strings.
+   Return a list of the string data. */
 static GList *
-get_string_list(gchar *str) {
-  enum { PRE_QUOT, IN_QUOT, POST_QUOT };
+get_string_list(gchar *str)
+{
+  enum { PRE_STRING, IN_QUOT, NOT_IN_QUOT };
 
-  gint      state = PRE_QUOT, i = 0, j = 0;
+  gint      state = PRE_STRING, i = 0, j = 0;
   gboolean  backslash = FALSE;
-  gchar     cur_c, *slstr = NULL;
+  guchar    cur_c;
+  gchar    *slstr = NULL;
   GList    *sl = NULL;
-  
-  while ((cur_c = str[i]) != '\0') {
+
+  /* Allocate a buffer for the first string.   */
+  slstr = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+  j = 0;
+
+  for (;;) {
+    cur_c = str[i];
+    if (cur_c == '\0') {
+      /* It's the end of the input, so it's the end of the string we
+         were working on, and there's no more input. */
+      if (state == IN_QUOT || backslash) {
+        /* We were in the middle of a quoted string or backslash escape,
+           and ran out of characters; that's an error.  */
+        g_free(slstr);
+        clear_string_list(sl);
+        return NULL;
+      }
+      slstr[j] = '\0';
+      sl = g_list_append(sl, slstr);
+      break;
+    }
     if (cur_c == '"' && ! backslash) {
       switch (state) {
-        case PRE_QUOT:
+        case PRE_STRING:
+          /* We hadn't yet started processing a string; this starts the
+             string, and we're now quoting.  */
           state = IN_QUOT;
-          slstr = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
-          j = 0;
           break;
         case IN_QUOT:
-          state  = POST_QUOT;
-          slstr[j] = '\0';
-          sl = g_list_append(sl, slstr);
+          /* We're in the middle of a quoted string, and we saw a quotation
+             mark; we're no longer quoting.   */
+          state = NOT_IN_QUOT;
           break;
-        case POST_QUOT:
-          clear_string_list(sl);
-          return NULL;
+        case NOT_IN_QUOT:
+          /* We're working on a string, but haven't seen a quote; we're
+             now quoting.  */
+          state = IN_QUOT;
           break;
         default:
           break;
       }
     } else if (cur_c == '\\' && ! backslash) {
+      /* We saw a backslash, and the previous character wasn't a
+         backslash; escape the next character.
+
+         This also means we've started a new string. */
       backslash = TRUE;
-    } else if (cur_c == ',' && state == POST_QUOT) {
-      state = PRE_QUOT;
-    } else if (state == IN_QUOT && j < COL_MAX_LEN) {
-      slstr[j] = str[i];
-      j++;
+      if (state == PRE_STRING)
+        state = NOT_IN_QUOT;
+    } else if (cur_c == ',' && state != IN_QUOT && ! backslash) {
+      /* We saw a comma, and we're not in the middle of a quoted string
+         and it wasn't preceded by a backslash; it's the end of
+         the string we were working on...  */
+      slstr[j] = '\0';
+      sl = g_list_append(sl, slstr);
+
+      /* ...and the beginning of a new string.  */
+      state = PRE_STRING;
+      slstr = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+      j = 0;
+    } else if (!isspace(cur_c) || state != PRE_STRING) {
+      /* Either this isn't a white-space character, or we've started a
+         string (i.e., already seen a non-white-space character for that
+         string and put it into the string).
+
+         The character is to be put into the string; do so if there's
+         room.  */
+      if (j < COL_MAX_LEN) {
+        slstr[j] = cur_c;
+        j++;
+      }
+
+      /* If it was backslash-escaped, we're done with the backslash escape.  */
+      backslash = FALSE;
     }
     i++;
   }
-  if (state != POST_QUOT) {
-    clear_string_list(sl);
-  }
   return(sl);
 }
 
-void
-clear_string_list(GList *sl) {
+#define MAX_FMT_PREF_LEN      1024
+#define MAX_FMT_PREF_LINE_LEN   60
+static gchar *
+put_string_list(GList *sl)
+{
+  static gchar  pref_str[MAX_FMT_PREF_LEN] = "";
+  GList        *clp = g_list_first(sl);
+  gchar        *str;
+  int           cur_pos = 0, cur_len = 0;
+  gchar        *quoted_str;
+  int           str_len;
+  gchar        *strp, *quoted_strp, c;
+  int           fmt_len;
+
+  while (clp) {
+    str = clp->data;
+
+    /* Allocate a buffer big enough to hold the entire string, with each
+       character quoted (that's the worst case).  */
+    str_len = strlen(str);
+    quoted_str = g_malloc(str_len*2 + 1);
+
+    /* Now quote any " or \ characters in it. */
+    strp = str;
+    quoted_strp = quoted_str;
+    while ((c = *strp++) != '\0') {
+      if (c == '"' || c == '\\') {
+        /* It has to be backslash-quoted.  */
+        *quoted_strp++ = '\\';
+      }
+      *quoted_strp++ = c;
+    }
+    *quoted_strp = '\0';
+
+    fmt_len = strlen(quoted_str) + 4;
+    if ((fmt_len + cur_len) < (MAX_FMT_PREF_LEN - 1)) {
+      if ((fmt_len + cur_pos) > MAX_FMT_PREF_LINE_LEN) {
+        /* Wrap the line.  */
+        cur_len--;
+        cur_pos = 0;
+        pref_str[cur_len] = '\n'; cur_len++;
+        pref_str[cur_len] = '\t'; cur_len++;
+      }
+      sprintf(&pref_str[cur_len], "\"%s\", ", quoted_str);
+      cur_pos += fmt_len;
+      cur_len += fmt_len;
+    }
+    g_free(quoted_str);
+    clp = clp->next;
+  }
+
+  /* If the string is at least two characters long, the last two characters
+     are ", ", and should be discarded, as there are no more items in the
+     string.  */
+  if (cur_len >= 2)
+    pref_str[cur_len - 2] = '\0';
+
+  return(pref_str);
+}    
+
+static void
+clear_string_list(GList *sl)
+{
   GList *l = sl;
   
   while (l) {
@@ -502,16 +686,16 @@ static void read_prefs_file(const char *pf_path, FILE *pf);
    return NULL. */
 e_prefs *
 read_prefs(int *gpf_errno_return, char **gpf_path_return,
-	   int *pf_errno_return, char **pf_path_return)
+	   int *pf_errno_return, const char **pf_path_return)
 {
-  int       i;
-  FILE     *pf;
-  fmt_data *cfmt;
-  gchar    *col_fmt[] = {"No.",      "%m", "Time",        "%t",
-                         "Source",   "%s", "Destination", "%d",
-                         "Protocol", "%p", "Info",        "%i"};
+  int         i;
+  const char *pf_path;
+  FILE       *pf;
+  fmt_data   *cfmt;
+  gchar      *col_fmt[] = {"No.",      "%m", "Time",        "%t",
+                           "Source",   "%s", "Destination", "%d",
+                           "Protocol", "%p", "Info",        "%i"};
 
-  
   if (init_prefs) {
     /* Initialize preferences to wired-in default values.
        They may be overridded by the global preferences file or the
@@ -617,11 +801,19 @@ read_prefs(int *gpf_errno_return, char **gpf_path_return,
     prefs.name_resolve        = PREFS_RESOLV_ALL;
   }
 
+  /* Construct the pathname of the global preferences file. */
+  if (! gpf_path) {
+    gpf_path = (gchar *) g_malloc(strlen(get_datafile_dir()) +
+      strlen(GPF_NAME) + 2);
+    sprintf(gpf_path, "%s" G_DIR_SEPARATOR_S "%s",
+      get_datafile_dir(), GPF_NAME);
+  }
+
   /* Read the global preferences file, if it exists. */
   *gpf_path_return = NULL;
-  if ((pf = fopen(GPF_PATH, "r")) != NULL) {
+  if ((pf = fopen(gpf_path, "r")) != NULL) {
     /* We succeeded in opening it; read it. */
-    read_prefs_file(GPF_PATH, pf);
+    read_prefs_file(gpf_path, pf);
     fclose(pf);
   } else {
     /* We failed to open it.  If we failed for some reason other than
@@ -629,16 +821,12 @@ read_prefs(int *gpf_errno_return, char **gpf_path_return,
        caller can report the error. */
     if (errno != ENOENT) {
       *gpf_errno_return = errno;
-      *gpf_path_return = GPF_PATH;
+      *gpf_path_return = gpf_path;
     }
   }
 
   /* Construct the pathname of the user's preferences file. */
-  if (! pf_path) {
-    pf_path = (gchar *) g_malloc(strlen(get_home_dir()) + strlen(PF_DIR) +
-      strlen(PF_NAME) + 4);
-    sprintf(pf_path, "%s/%s/%s", get_home_dir(), PF_DIR, PF_NAME);
-  }
+  pf_path = get_persconffile_path(PF_NAME, FALSE);
     
   /* Read the user's preferences file, if it exists. */
   *pf_path_return = NULL;
@@ -711,6 +899,13 @@ read_prefs_file(const char *pf_path, FILE *pf)
                 g_warning ("%s line %d: No such preference \"%s\"", pf_path,
 				pline, cur_var);
                 break;
+
+	      case PREFS_SET_OBSOLETE:
+	        /* We silently ignore attempts to set these; it's
+	           probably not the user's fault that it's in there -
+	           they may have saved preferences with a release that
+	           supported them. */
+                break;
               }
             } else {
               g_warning ("%s line %d: Incomplete preference", pf_path, pline);
@@ -772,6 +967,12 @@ read_prefs_file(const char *pf_path, FILE *pf)
         g_warning ("%s line %d: No such preference \"%s\"", pf_path,
 			pline, cur_var);
         break;
+
+      case PREFS_SET_OBSOLETE:
+	/* We silently ignore attempts to set these; it's probably not
+	   the user's fault that it's in there - they may have saved
+	   preferences with a release that supported it. */
+        break;
       }
     } else {
       g_warning ("%s line %d: Incomplete preference", pf_path, pline);
@@ -934,7 +1135,7 @@ string_to_name_resolve(char *string, guint32 *name_resolve)
 static int
 set_pref(gchar *pref_name, gchar *value)
 {
-  GList    *col_l;
+  GList    *col_l, *col_l_elt;
   gint      llen;
   fmt_data *cfmt;
   unsigned long int cval;
@@ -942,9 +1143,10 @@ set_pref(gchar *pref_name, gchar *value)
   gboolean bval;
   gint     enum_val;
   char     *p;
-  gchar    *dotp;
+  gchar    *dotp, *last_dotp;
   module_t *module;
   pref_t   *pref;
+  gboolean had_a_dot;
 
   if (strcmp(pref_name, PRS_PRINT_FMT) == 0) {
     if (strcmp(value, pr_formats[PR_FMT_TEXT]) == 0) {
@@ -969,21 +1171,49 @@ set_pref(gchar *pref_name, gchar *value)
     if (prefs.pr_cmd) g_free(prefs.pr_cmd);
     prefs.pr_cmd = g_strdup(value);
   } else if (strcmp(pref_name, PRS_COL_FMT) == 0) {
-    if ((col_l = get_string_list(value)) && (g_list_length(col_l) % 2) == 0) {
-      free_col_info(&prefs);
-      prefs.col_list = NULL;
-      llen             = g_list_length(col_l);
-      prefs.num_cols   = llen / 2;
-      col_l = g_list_first(col_l);
-      while(col_l) {
-        cfmt = (fmt_data *) g_malloc(sizeof(fmt_data));
-        cfmt->title    = g_strdup(col_l->data);
-        col_l          = col_l->next;
-        cfmt->fmt      = g_strdup(col_l->data);
-        col_l          = col_l->next;
-        prefs.col_list = g_list_append(prefs.col_list, cfmt);
+    col_l = get_string_list(value);
+    if (col_l == NULL)
+      return PREFS_SET_SYNTAX_ERR;
+    if ((g_list_length(col_l) % 2) != 0) {
+      /* A title didn't have a matching format.  */
+      clear_string_list(col_l);
+      return PREFS_SET_SYNTAX_ERR;
+    }
+    /* Check to make sure all column formats are valid.  */
+    col_l_elt = g_list_first(col_l);
+    while(col_l_elt) {
+      /* Make sure the title isn't empty.  */
+      if (strcmp(col_l_elt->data, "") == 0) {
+      	/* It is.  */
+        clear_string_list(col_l);
+        return PREFS_SET_SYNTAX_ERR;
       }
-      /* To do: else print some sort of error? */
+
+      /* Go past the title.  */
+      col_l_elt = col_l_elt->next;
+
+      /* Check the format.  */
+      if (get_column_format_from_str(col_l_elt->data) == -1) {
+        /* It's not a valid column format.  */
+        clear_string_list(col_l);
+        return PREFS_SET_SYNTAX_ERR;
+      }
+
+      /* Go past the format.  */
+      col_l_elt = col_l_elt->next;
+    }
+    free_col_info(&prefs);
+    prefs.col_list = NULL;
+    llen             = g_list_length(col_l);
+    prefs.num_cols   = llen / 2;
+    col_l_elt = g_list_first(col_l);
+    while(col_l_elt) {
+      cfmt = (fmt_data *) g_malloc(sizeof(fmt_data));
+      cfmt->title    = g_strdup(col_l_elt->data);
+      col_l_elt      = col_l_elt->next;
+      cfmt->fmt      = g_strdup(col_l_elt->data);
+      col_l_elt      = col_l_elt->next;
+      prefs.col_list = g_list_append(prefs.col_list, cfmt);
     }
     clear_string_list(col_l);
   } else if (strcmp(pref_name, PRS_STREAM_CL_FG) == 0) {
@@ -1087,71 +1317,97 @@ set_pref(gchar *pref_name, gchar *value)
     }
   } else {
     /* To which module does this preference belong? */
-    dotp = strchr(pref_name, '.');
-    if (dotp == NULL)
-      return PREFS_SET_SYNTAX_ERR;	/* no ".", so no module/name separator */
-    *dotp = '\0';		/* separate module and preference name */
-    module = find_module(pref_name);
+    module = NULL;
+    last_dotp = pref_name;
+    had_a_dot = FALSE;
+    while (!module) {
+        dotp = strchr(last_dotp, '.');
+        if (dotp == NULL) {
+            if (had_a_dot) {
+              /* no such module */
+              return PREFS_SET_NO_SUCH_PREF;
+            }
+            else {
+              /* no ".", so no module/name separator */
+              return PREFS_SET_SYNTAX_ERR;
+            }
+        }
+        else {
+            had_a_dot = TRUE;
+        }
+        *dotp = '\0';		/* separate module and preference name */
+        module = find_module(pref_name);
+
+        /*
+         * XXX - "Diameter" rather than "diameter" was used in earlier
+         * versions of Ethereal; if we didn't find the module, and its name
+         * was "Diameter", look for "diameter" instead.
+         *
+         * In addition, the BEEP protocol used to be the BXXP protocol,
+         * so if we didn't find the module, and its name was "bxxp",
+         * look for "beep" instead.
+         */
+        if (module == NULL) {
+          if (strcmp(pref_name, "Diameter") == 0)
+            module = find_module("diameter");
+          else if (strcmp(pref_name, "bxxp") == 0)
+            module = find_module("beep");
+        }
+        *dotp = '.';		/* put the preference string back */
+        dotp++;			/* skip past separator to preference name */
+        last_dotp = dotp;
+    }
 
-    /*
-     * XXX - "Diameter" rather than "diameter" was used in earlier
-     * versions of Ethereal; if we didn't find the module, and its name
-     * was "Diameter", look for "diameter" instead.
-     */
-    if (module == NULL && strcmp(pref_name, "Diameter") == 0)
-      module = find_module("diameter");
-    *dotp = '.';		/* put the preference string back */
-    if (module == NULL)
-      return PREFS_SET_NO_SUCH_PREF;	/* no such module */
-    dotp++;			/* skip past separator to preference name */
     pref = find_preference(module, dotp);
 
-    /*
-     * XXX - "mgcp.display raw text toggle" and "mgcp.display dissect tree"
-     * rather than "mgcp.display_raw_text" and "mgcp.display_dissect_tree"
-     * were used in earlier versions of Ethereal; if we didn't find the
-     * preference, it was an MGCP preference, and its name was
-     * "display raw text toggle" or "display dissect tree", look for
-     * "display_raw_text" or "display_dissect_tree" instead.
-     *
-     * "mgcp.tcp.port" and "mgcp.udp.port" are harder to handle, as both
-     * the gateway and callagent ports were given those names; we interpret
-     * the first as "mgcp.{tcp,udp}.gateway_port" and the second as
-     * "mgcp.{tcp,udp}.callagent_port", as that's the order in which
-     * they were registered by the MCCP dissector and thus that's the
-     * order in which they were written to the preferences file.  (If
-     * we're not reading the preferences file, but are handling stuff
-     * from a "-o" command-line option, we have no clue which the user
-     * had in mind - they should have used "mgcp.{tcp,udp}.gateway_port"
-     * or "mgcp.{tcp,udp}.callagent_port" instead.)
-     */
-    if (pref == NULL && strncmp(pref_name, "mgcp.", 5) == 0) {
-      if (strcmp(dotp, "display raw text toggle") == 0)
-        pref = find_preference(module, "display_raw_text");
-      else if (strcmp(dotp, "display dissect tree") == 0)
-        pref = find_preference(module, "display_dissect_tree");
-      else if (strcmp(dotp, "tcp.port") == 0) {
-	mgcp_tcp_port_count++;
-	if (mgcp_tcp_port_count == 1) {
-	  /* It's the first one */
-	  pref = find_preference(module, "tcp.gateway_port");
-	} else if (mgcp_tcp_port_count == 2) {
-	  /* It's the second one */
-	  pref = find_preference(module, "tcp.callagent_port");
+    if (pref == NULL) {
+      if (strncmp(pref_name, "mgcp.", 5) == 0) {
+        /*
+         * XXX - "mgcp.display raw text toggle" and "mgcp.display dissect tree"
+         * rather than "mgcp.display_raw_text" and "mgcp.display_dissect_tree"
+         * were used in earlier versions of Ethereal; if we didn't find the
+         * preference, it was an MGCP preference, and its name was
+         * "display raw text toggle" or "display dissect tree", look for
+         * "display_raw_text" or "display_dissect_tree" instead.
+         *
+         * "mgcp.tcp.port" and "mgcp.udp.port" are harder to handle, as both
+         * the gateway and callagent ports were given those names; we interpret
+         * the first as "mgcp.{tcp,udp}.gateway_port" and the second as
+         * "mgcp.{tcp,udp}.callagent_port", as that's the order in which
+         * they were registered by the MCCP dissector and thus that's the
+         * order in which they were written to the preferences file.  (If
+         * we're not reading the preferences file, but are handling stuff
+         * from a "-o" command-line option, we have no clue which the user
+         * had in mind - they should have used "mgcp.{tcp,udp}.gateway_port"
+         * or "mgcp.{tcp,udp}.callagent_port" instead.)
+         */
+        if (strcmp(dotp, "display raw text toggle") == 0)
+          pref = find_preference(module, "display_raw_text");
+        else if (strcmp(dotp, "display dissect tree") == 0)
+          pref = find_preference(module, "display_dissect_tree");
+        else if (strcmp(dotp, "tcp.port") == 0) {
+          mgcp_tcp_port_count++;
+          if (mgcp_tcp_port_count == 1) {
+            /* It's the first one */
+            pref = find_preference(module, "tcp.gateway_port");
+ 	  } else if (mgcp_tcp_port_count == 2) {
+            /* It's the second one */
+            pref = find_preference(module, "tcp.callagent_port");
+	  }
+          /* Otherwise it's from the command line, and we don't bother
+             mapping it. */
+	} else if (strcmp(dotp, "udp.port") == 0) {
+          mgcp_udp_port_count++;
+          if (mgcp_udp_port_count == 1) {
+            /* It's the first one */
+            pref = find_preference(module, "udp.gateway_port");
+	  } else if (mgcp_udp_port_count == 2) {
+            /* It's the second one */
+            pref = find_preference(module, "udp.callagent_port");
+	  }
+          /* Otherwise it's from the command line, and we don't bother
+             mapping it. */
 	}
-	/* Otherwise it's from the command line, and we don't bother
-	   mapping it. */
-      } else if (strcmp(dotp, "udp.port") == 0) {
-	mgcp_udp_port_count++;
-	if (mgcp_udp_port_count == 1) {
-	  /* It's the first one */
-	  pref = find_preference(module, "udp.gateway_port");
-	} else if (mgcp_udp_port_count == 2) {
-	  /* It's the second one */
-	  pref = find_preference(module, "udp.callagent_port");
-	}
-	/* Otherwise it's from the command line, and we don't bother
-	   mapping it. */
       }
     }
     if (pref == NULL)
@@ -1199,6 +1455,9 @@ set_pref(gchar *pref_name, gchar *value)
         *pref->varp.string = g_strdup(value);
       }
       break;
+
+    case PREF_OBSOLETE:
+      return PREFS_SET_OBSOLETE;	/* no such preference any more */
     }
   }
   
@@ -1221,6 +1480,16 @@ write_pref(gpointer data, gpointer user_data)
 	const enum_val_t *enum_valp;
 	const char *val_string;
 
+	if (pref->type == PREF_OBSOLETE) {
+		/*
+		 * This preference is no longer supported; it's not a
+		 * real preference, so we don't write it out (i.e., we
+		 * treat it as if it weren't found in the list of
+		 * preferences, and we weren't called in the first place).
+		 */
+		return;
+	}
+
 	fprintf(arg->pf, "\n# %s\n", pref->description);
 
 	switch (pref->type) {
@@ -1278,6 +1547,10 @@ write_pref(gpointer data, gpointer user_data)
 		fprintf(arg->pf, "%s.%s: %s\n", arg->module->name, pref->name,
 		    *pref->varp.string);
 		break;
+
+	case PREF_OBSOLETE:
+		g_assert_not_reached();
+		break;
 	}
 }
 
@@ -1296,31 +1569,20 @@ write_module_prefs(gpointer data, gpointer user_data)
    If we got an error, stuff a pointer to the path of the preferences file
    into "*pf_path_return", and return the errno. */
 int
-write_prefs(char **pf_path_return)
+write_prefs(const char **pf_path_return)
 {
+  const char  *pf_path;
   FILE        *pf;
-  struct stat  s_buf;
-  
+  GList       *clp, *col_l;
+  fmt_data    *cfmt;
+
   /* To do:
    * - Split output lines longer than MAX_VAL_LEN
    * - Create a function for the preference directory check/creation
    *   so that duplication can be avoided with filter.c
    */
 
-  if (! pf_path) {
-    pf_path = (gchar *) g_malloc(strlen(get_home_dir()) + strlen(PF_DIR) +
-      strlen(PF_NAME) + 4);
-  }
-
-  sprintf(pf_path, "%s/%s", get_home_dir(), PF_DIR);
-  if (stat(pf_path, &s_buf) != 0)
-#ifdef WIN32
-    mkdir(pf_path);
-#else
-    mkdir(pf_path, 0755);
-#endif
-
-  sprintf(pf_path, "%s/%s/%s", get_home_dir(), PF_DIR, PF_NAME);
+  pf_path = get_persconffile_path(PF_NAME, TRUE);
   if ((pf = fopen(pf_path, "w")) == NULL) {
     *pf_path_return = pf_path;
     return errno;
@@ -1348,9 +1610,21 @@ write_prefs(char **pf_path_return)
     "is set to \"command\"\n"
     "%s: %s\n\n", PRS_PRINT_CMD, prefs.pr_cmd);
 
+  clp = prefs.col_list;
+  col_l = NULL;
+  while (clp) {
+    cfmt = (fmt_data *) clp->data;
+    col_l = g_list_append(col_l, cfmt->title);
+    col_l = g_list_append(col_l, cfmt->fmt);
+    clp = clp->next;
+  }
   fprintf (pf, "# Packet list column format.  Each pair of strings consists "
     "of a column title \n# and its format.\n"
-    "%s: %s\n\n", PRS_COL_FMT, col_format_to_pref_str());
+    "%s: %s\n\n", PRS_COL_FMT, put_string_list(col_l));
+  /* This frees the list of strings, but not the strings to which it
+     refers; that's what we want, as we haven't copied those strings,
+     we just referred to them.  */
+  g_list_free(col_l);
 
   fprintf (pf, "# TCP stream window color preferences.  Each value is a six "
     "digit hexadecimal value in the form rrggbb.\n");
