@@ -1,9 +1,9 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c,v 1.36 2001/06/18 01:49:17 guy Exp $
+ * $Id: print.c,v 1.38 2001/12/18 21:31:02 gram Exp $
  *
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -147,7 +147,7 @@ get_field_data(GSList *src_list, field_info *fi)
 static
 void proto_tree_print_node_text(GNode *node, gpointer data)
 {
-	field_info	*fi = (field_info*) (node->data);
+	field_info	*fi = PITEM_FINFO(node);
 	print_data	*pdata = (print_data*) data;
 	int		i;
 	int		num_spaces;
@@ -288,7 +288,7 @@ void print_hex_data_text(FILE *fh, register const u_char *cp,
 static
 void proto_tree_print_node_ps(GNode *node, gpointer data)
 {
-	field_info	*fi = (field_info*) (node->data);
+	field_info	*fi = PITEM_FINFO(node);
 	print_data	*pdata = (print_data*) data;
 	gchar		label_str[ITEM_LABEL_LENGTH];
 	gchar		*label_ptr;
