@@ -1,9 +1,9 @@
 /* print.h
  * Definitions for printing packet analysis trees.
  *
- * $Id: print.h,v 1.22 2001/06/08 08:50:49 guy Exp $
+ * $Id: print.h,v 1.24 2001/11/13 23:55:30 gram Exp $
  *
- * Gilbert Ramirez <gram@xiexie.org>
+ * Gilbert Ramirez <gram@alumni.rice.edu>
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -43,6 +43,11 @@ typedef struct {
 				   FALSE if we should print only if not dissected. */
   gboolean	expand_all;	/* TRUE if we should expand all levels;
 				   FALSE if we should expand as displayed. */
+  gboolean      suppress_unmarked; /* TRUE if we should suppress unmarked 
+				     frames FALSE if we should allow 
+				     both marked and unmarked frames to be 
+				     printed.
+				  */
 } print_args_t;
 
 /* Functions in print.h */
