@@ -1,12 +1,13 @@
 /* tethereal.c
  *
- * $Id: tethereal.c,v 1.87 2001/07/05 00:34:39 guy Exp $
+ * $Id: tethereal.c,v 1.112 2001/12/21 20:32:53 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
- * Text-mode variant, by Gilbert Ramirez <gram@xiexie.org>.
+ * Text-mode variant, by Gilbert Ramirez <gram@alumni.rice.edu>
+ * and Guy Harris <guy@alum.mit.edu>.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,7 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #include <locale.h>
 #include <limits.h>
 
@@ -96,10 +97,17 @@
 #include "print.h"
 #include "resolv.h"
 #include "util.h"
+#ifdef HAVE_LIBPCAP
+#include "pcap-util.h"
+#endif
 #include "conversation.h"
 #include "reassemble.h"
 #include "plugins.h"
 #include "register.h"
+#include "conditions.h"
+#include "capture_stop_conditions.h"
+#include "ringbuffer.h"
+#include "epan_dissect.h"
 
 #ifdef WIN32
 #include "capture-wpcap.h"
@@ -117,6 +125,7 @@ typedef struct _loop_data {
   gint           linktype;
   pcap_t        *pch;
   wtap_dumper   *pdh;
+  gboolean       go;
 } loop_data;
 
 static loop_data ld;
@@ -133,13 +142,12 @@ typedef struct {
 } cb_args_t;
 
 static int load_cap_file(capture_file *, int);
-static void wtap_dispatch_cb_write(u_char *, const struct wtap_pkthdr *, int,
+static void wtap_dispatch_cb_write(u_char *, const struct wtap_pkthdr *, long,
     union wtap_pseudo_header *, const u_char *);
 static void show_capture_file_io_error(const char *, int, gboolean);
-static void wtap_dispatch_cb_print(u_char *, const struct wtap_pkthdr *, int,
+static void wtap_dispatch_cb_print(u_char *, const struct wtap_pkthdr *, long,
     union wtap_pseudo_header *, const u_char *);
 
-packet_info  pi;
 capture_file cfile;
 FILE        *data_out_file = NULL;
 ts_type timestamp_type = RELATIVE;
@@ -155,8 +163,11 @@ print_usage(void)
   fprintf(stderr, "This is GNU t%s %s, compiled %s\n", PACKAGE, VERSION,
 	comp_info_str->str);
 #ifdef HAVE_LIBPCAP
-  fprintf(stderr, "t%s [ -DvVhlp ] [ -c <count> ] [ -f <capture filter> ]\n", PACKAGE);
-  fprintf(stderr, "\t[ -F <capture file type> ] [ -i <interface> ] [ -n ] [ -N <resolving> ]\n");
+  fprintf(stderr, "t%s [ -DvVhlp ] [ -a <capture autostop condition> ] ...\n",
+	  PACKAGE);
+  fprintf(stderr, "\t[ -b <number of ring buffer files> ] [ -c <count> ]\n");
+  fprintf(stderr, "\t[ -f <capture filter> ] [ -F <capture file type> ]\n");
+  fprintf(stderr, "\t[ -i <interface> ] [ -n ] [ -N <resolving> ]\n");
   fprintf(stderr, "\t[ -o <preference setting> ] ... [ -r <infile> ] [ -R <read filter> ]\n");
   fprintf(stderr, "\t[ -s <snaplen> ] [ -t <time stamp format> ] [ -w <savefile> ] [ -x ]\n");
 #else
@@ -173,7 +184,7 @@ print_usage(void)
   fprintf(stderr, "\tdefault is libpcap\n");
 }
 
-int
+static int
 get_positive_int(const char *string, const char *name)
 {
   long number;
@@ -198,6 +209,51 @@ get_positive_int(const char *string, const char *name)
   return number;
 }
 
+/*
+ * Given a string of the form "<autostop criterion>:<value>", as might appear
+ * as an argument to a "-a" option, parse it and set the criterion in
+ * question.  Return an indication of whether it succeeded or failed
+ * in some fashion.
+ */
+static gboolean
+set_autostop_criterion(const char *autostoparg)
+{
+  u_char *p, *colonp;
+
+  colonp = strchr(autostoparg, ':');
+  if (colonp == NULL)
+    return FALSE;
+
+  p = colonp;
+  *p++ = '\0';
+
+  /*
+   * Skip over any white space (there probably won't be any, but
+   * as we allow it in the preferences file, we might as well
+   * allow it here).
+   */
+  while (isspace(*p))
+    p++;
+  if (*p == '\0') {
+    /*
+     * Put the colon back, so if our caller uses, in an
+     * error message, the string they passed us, the message
+     * looks correct.
+     */
+    *colonp = ':';
+    return FALSE;
+  }
+  if (strcmp(autostoparg,"duration") == 0) {
+    cfile.autostop_duration = get_positive_int(p,"autostop duration");
+  } else if (strcmp(autostoparg,"filesize") == 0) {
+    cfile.autostop_filesize = get_positive_int(p,"autostop filesize");
+  } else {
+    return FALSE;
+  }
+  *colonp = ':';	/* put the colon back */
+  return TRUE;
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -205,23 +261,22 @@ main(int argc, char *argv[])
   extern char         *optarg;
   gboolean             arg_error = FALSE;
 #ifdef HAVE_LIBPCAP
-#ifdef WIN32
-  char pcap_version[] = WPCAP_STRING;
-#else
+#ifdef HAVE_PCAP_VERSION
   extern char          pcap_version[];
-#endif
-#endif
+#endif /* HAVE_PCAP_VERSION */
+#endif /* HAVE_LIBPCAP */
 
 #ifdef WIN32
   WSADATA		wsaData;
 #endif
 
-  char                *gpf_path, *pf_path;
+  char                *gpf_path;
+  const char          *pf_path;
   int                  gpf_open_errno, pf_open_errno;
   int                  err;
 #ifdef HAVE_LIBPCAP
   gboolean             capture_filter_specified = FALSE;
-  int                  packet_count = 0;
+  guint                packet_count = 0;
   GList               *if_list, *if_entry;
   gchar                err_str[PCAP_ERRBUF_SIZE];
 #else
@@ -289,6 +344,12 @@ main(int argc, char *argv[])
   cfile.save_file_fd	= -1;
   cfile.snap		= WTAP_MAX_PACKET_SIZE;
   cfile.count		= 0;
+#ifdef HAVE_LIBPCAP
+  cfile.autostop_duration = 0;
+  cfile.autostop_filesize = 0;
+  cfile.ringbuffer_on = FALSE;
+  cfile.ringbuffer_num_files = RINGBUFFER_MIN_NUM_FILES;
+#endif
   col_init(&cfile.cinfo, prefs->num_cols);
 
   /* Assemble the compile-time options */
@@ -305,10 +366,14 @@ main(int argc, char *argv[])
 
 #ifdef HAVE_LIBPCAP
   g_string_append(comp_info_str, ", with libpcap ");
+#ifdef HAVE_PCAP_VERSION
   g_string_append(comp_info_str, pcap_version);
-#else
+#else /* HAVE_PCAP_VERSION */
+  g_string_append(comp_info_str, "(version unknown)");
+#endif /* HAVE_PCAP_VERSION */
+#else /* HAVE_LIBPCAP */
   g_string_append(comp_info_str, ", without libpcap");
-#endif
+#endif /* HAVE_LIBPCAP */
 
 #ifdef HAVE_LIBZ
   g_string_append(comp_info_str, ", with libz ");
@@ -341,8 +406,28 @@ main(int argc, char *argv[])
 #endif
     
   /* Now get our args */
-  while ((opt = getopt(argc, argv, "c:Df:F:hi:lnN:o:pr:R:s:t:vw:Vx")) != EOF) {
+  while ((opt = getopt(argc, argv, "a:b:c:Df:F:hi:lnN:o:pr:R:s:t:vw:Vx")) != EOF) {
     switch (opt) {
+      case 'a':        /* autostop criteria */
+#ifdef HAVE_LIBPCAP
+        if (set_autostop_criterion(optarg) == FALSE) {
+          fprintf(stderr, "ethereal: Invalid or unknown -a flag \"%s\"\n", optarg);
+          exit(1);          
+        }
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+        break;
+      case 'b':        /* Ringbuffer option */
+#ifdef HAVE_LIBPCAP
+        cfile.ringbuffer_on = TRUE;
+        cfile.ringbuffer_num_files = get_positive_int(optarg, "number of ring buffer files");
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+        break;
       case 'c':        /* Capture xxx packets */
 #ifdef HAVE_LIBPCAP
         packet_count = get_positive_int(optarg, "packet count");
@@ -443,6 +528,7 @@ main(int argc, char *argv[])
           break;
 
         case PREFS_SET_NO_SUCH_PREF:
+        case PREFS_SET_OBSOLETE:
           fprintf(stderr, "tethereal: -o flag \"%s\" specifies unknown preference\n",
 			optarg);
           exit(1);
@@ -530,6 +616,29 @@ main(int argc, char *argv[])
     }
   }
 
+  /* If they didn't specify a "-w" flag, but specified a maximum capture
+     file size, tell them that this doesn't work, and exit. */
+  if (cfile.autostop_filesize != 0 && cfile.save_file == NULL) {
+    fprintf(stderr, "tethereal: Maximum capture file size specified, but capture isn't being saved to a file.\n");
+    exit(2);
+  }
+
+  if (cfile.ringbuffer_on) {
+    /* Ring buffer works only under certain conditions:
+       a) ring buffer does not work if you're not saving the capture to
+          a file;
+       b) it makes no sense to enable the ring buffer if the maximum
+          file size is set to "infinite". */
+    if (cfile.save_file == NULL) {
+      fprintf(stderr, "tethereal: Ring buffer requested, but capture isn't being saved to a file.\n");
+      exit(2);
+    }
+    if (cfile.autostop_filesize == 0) {
+      fprintf(stderr, "tethereal: Ring buffer requested, but no maximum capture file size was specified.\n");
+      exit(2);
+    }
+  }
+
 #ifdef WIN32
   /* Start windows sockets */
   WSAStartup( MAKEWORD( 1, 1 ), &wsaData );
@@ -566,6 +675,12 @@ main(int argc, char *argv[])
   else if (cfile.snap < MIN_PACKET_SIZE)
     cfile.snap = MIN_PACKET_SIZE;
   
+  /* Check the value range of the ringbuffer_num_files parameter */
+  if (cfile.ringbuffer_num_files < RINGBUFFER_MIN_NUM_FILES)
+    cfile.ringbuffer_num_files = RINGBUFFER_MIN_NUM_FILES;
+  else if (cfile.ringbuffer_num_files > RINGBUFFER_MAX_NUM_FILES)
+    cfile.ringbuffer_num_files = RINGBUFFER_MAX_NUM_FILES;
+  
   if (rfilter != NULL) {
     if (!dfilter_compile(rfilter, &rfcode)) {
       fprintf(stderr, "tethereal: %s\n", dfilter_error_msg);
@@ -620,6 +735,10 @@ main(int argc, char *argv[])
         free_interface_list(if_list);
     }
     capture(packet_count, out_file_type);
+
+    if (cfile.ringbuffer_on) {
+      ringbuf_free();
+    }
 #else
     /* No - complain. */
     fprintf(stderr, "This version of Tethereal was not built with support for capturing packets.\n");
@@ -638,17 +757,21 @@ main(int argc, char *argv[])
 static int
 capture(int packet_count, int out_file_type)
 {
-  gchar       err_str[PCAP_ERRBUF_SIZE];
+  gchar       open_err_str[PCAP_ERRBUF_SIZE];
+  gchar       lookup_net_err_str[PCAP_ERRBUF_SIZE];
   bpf_u_int32 netnum, netmask;
   struct bpf_program fcode;
   void        (*oldhandler)(int);
   int         err, inpkts;
   char        errmsg[1024+1];
+  condition *cnd_stop_capturesize;
+  condition *cnd_stop_timeout;
 #ifndef _WIN32
   static const char ppamsg[] = "can't find PPA for ";
   char       *libpcap_warn;
 #endif
   struct pcap_stat stats;
+  gboolean    dump_ok;
 
   /* Initialize the table of conversations. */
   epan_conversation_init();
@@ -665,8 +788,13 @@ capture(int packet_count, int out_file_type)
   ld.linktype       = WTAP_ENCAP_UNKNOWN;
   ld.pdh            = NULL;
 
-  /* Open the network interface to capture from it. */
-  ld.pch = pcap_open_live(cfile.iface, cfile.snap, promisc_mode, 1000, err_str);
+  /* Open the network interface to capture from it.
+     Some versions of libpcap may put warnings into the error buffer
+     if they succeed; to tell if that's happened, we have to clear
+     the error buffer, and check if it's still a null string.  */
+  open_err_str[0] = '\0';
+  ld.pch = pcap_open_live(cfile.iface, cfile.snap, promisc_mode, 1000,
+			  open_err_str);
 
   if (ld.pch == NULL) {
     /* Well, we couldn't start the capture. */
@@ -682,14 +810,14 @@ capture(int packet_count, int out_file_type)
 	"Note that the driver Tethereal uses for packet capture on Windows\n"
 	"doesn't support capturing on Token Ring interfaces, and doesn't\n"
 	"support capturing on PPP/WAN interfaces in Windows NT/2000.\n",
-	err_str);
+	open_err_str);
 #else
       /* If we got a "can't find PPA for XXX" message, warn the user (who
          is running Ethereal on HP-UX) that they don't have a version
 	 of libpcap that properly handles HP-UX (libpcap 0.6.x and later
 	 versions, which properly handle HP-UX, say "can't find /dev/dlpi
 	 PPA for XXX" rather than "can't find PPA for XXX"). */
-      if (strncmp(err_str, ppamsg, sizeof ppamsg - 1) == 0)
+      if (strncmp(open_err_str, ppamsg, sizeof ppamsg - 1) == 0)
 	libpcap_warn =
 	  "\n\n"
 	  "You are running Tethereal with a version of the libpcap library\n"
@@ -706,21 +834,21 @@ capture(int packet_count, int out_file_type)
     snprintf(errmsg, sizeof errmsg,
       "The capture session could not be initiated (%s).\n"
       "Please check to make sure you have sufficient permissions, and that\n"
-      "you have the proper interface specified.%s", err_str, libpcap_warn);
+      "you have the proper interface specified.%s", open_err_str, libpcap_warn);
 #endif
     goto error;
   }
 
   if (cfile.cfilter) {
     /* A capture filter was specified; set it up. */
-    if (pcap_lookupnet (cfile.iface, &netnum, &netmask, err_str) < 0) {
+    if (pcap_lookupnet(cfile.iface, &netnum, &netmask, lookup_net_err_str) < 0) {
       /*
        * Well, we can't get the netmask for this interface; it's used
        * only for filters that check for broadcast IP addresses, so
        * we just warn the user, and punt and use 0.
        */
       fprintf(stderr, 
-        "Warning:  Couldn't obtain netmask info (%s)\n.", err_str);
+        "Warning:  Couldn't obtain netmask info (%s)\n.", lookup_net_err_str);
       netmask = 0;
     }
     if (pcap_compile(ld.pch, &fcode, cfile.cfilter, 1, netmask) < 0) {
@@ -735,7 +863,8 @@ capture(int packet_count, int out_file_type)
     }
   }
 
-  ld.linktype = wtap_pcap_encap_to_wtap_encap(pcap_datalink(ld.pch));
+  ld.linktype = wtap_pcap_encap_to_wtap_encap(get_pcap_linktype(ld.pch,
+	cfile.iface));
   if (cfile.save_file != NULL) {
     /* Set up to write to the capture file. */
     if (ld.linktype == WTAP_ENCAP_UNKNOWN) {
@@ -743,8 +872,19 @@ capture(int packet_count, int out_file_type)
                " that Tethereal doesn't support.");
       goto error;
     }
-    ld.pdh = wtap_dump_open(cfile.save_file, out_file_type,
-		ld.linktype, pcap_snapshot(ld.pch), &err);
+    if (cfile.ringbuffer_on) {
+      cfile.save_file_fd = ringbuf_init(cfile.save_file,
+        cfile.ringbuffer_num_files);
+      if (cfile.save_file_fd != -1) {
+        ld.pdh = ringbuf_init_wtap_dump_fdopen(out_file_type, ld.linktype,
+          pcap_snapshot(ld.pch), &err);
+      } else {
+        ld.pdh = NULL;
+      }
+    } else {
+      ld.pdh = wtap_dump_open(cfile.save_file, out_file_type,
+		 ld.linktype, pcap_snapshot(ld.pch), &err);
+    }
 
     if (ld.pdh == NULL) {
       snprintf(errmsg, sizeof errmsg, file_open_error_message(errno, TRUE),
@@ -753,6 +893,11 @@ capture(int packet_count, int out_file_type)
     }
   }
 
+  /* Does "open_err_str" contain a non-empty string?  If so, "pcap_open_live()"
+     returned a warning; print it, but keep capturing. */
+  if (open_err_str[0] != '\0')
+    fprintf(stderr, "tethereal: WARNING: %s.\n", open_err_str);
+
   /* Catch SIGINT and SIGTERM and, if we get either of them, clean up
      and exit.
      XXX - deal with signal semantics on various platforms.  Or just
@@ -768,7 +913,50 @@ capture(int packet_count, int out_file_type)
   fprintf(stderr, "Capturing on %s\n", cfile.iface);
   fflush(stderr);
 
-  inpkts = pcap_loop(ld.pch, packet_count, capture_pcap_cb, (u_char *) &ld);
+  /* initialize capture stop conditions */ 
+  init_capture_stop_conditions();
+  /* create stop conditions */
+  cnd_stop_capturesize = cnd_new((char*)CND_CLASS_CAPTURESIZE,
+                                 (long)cfile.autostop_filesize * 1000);
+  cnd_stop_timeout = cnd_new((char*)CND_CLASS_TIMEOUT,
+                             (gint32)cfile.autostop_duration);
+
+  if (packet_count == 0)
+    packet_count = -1; /* infinite capturng */
+  ld.go = TRUE;
+  while (ld.go) {
+    if (packet_count > 0)
+      packet_count--;
+    inpkts = pcap_dispatch(ld.pch, 1, capture_pcap_cb, (u_char *) &ld);
+    if (packet_count == 0) {
+      ld.go = FALSE;
+    } else if (cnd_eval(cnd_stop_timeout) == TRUE) {
+      /* The specified capture time has elapsed; stop the capture. */
+      ld.go = FALSE;
+    } else if (ld.pdh != NULL && (cnd_eval(cnd_stop_capturesize, 
+                  (guint32)wtap_get_bytes_dumped(ld.pdh))) == TRUE){
+      /* We're saving the capture to a file, and the capture file reached
+         its maximum size. */
+      if (cfile.ringbuffer_on) {
+        /* Switch to the next ringbuffer file */
+        if (ringbuf_switch_file(&cfile, &ld.pdh, &err) == TRUE) {
+          /* File switch failed: reset the condition */
+          cnd_reset(cnd_stop_capturesize);
+        } else {
+          /* File switch failed: stop here */
+          ld.go = FALSE;
+          continue;
+        }
+      } else {
+        /* No ringbuffer - just stop. */
+        ld.go = FALSE;
+      }
+    }
+  }
+  
+  /* delete stop conditions */
+  cnd_delete(cnd_stop_capturesize);
+  cnd_delete(cnd_stop_timeout);
 
   if (cfile.save_file != NULL) {
     /* We're saving to a file, which means we're printing packet counts
@@ -793,11 +981,26 @@ capture(int packet_count, int out_file_type)
     fprintf(stderr, "tethereal: Can't get packet-drop statistics: %s\n",
 	pcap_geterr(ld.pch));
   }
+
   pcap_close(ld.pch);
 
+  if (cfile.save_file != NULL) {
+    /* We're saving to a file or files; close all files. */
+    if (cfile.ringbuffer_on) {
+      dump_ok = ringbuf_wtap_dump_close(&cfile, &err);
+    } else {
+      dump_ok = wtap_dump_close(ld.pdh, &err);
+    }
+    if (!dump_ok)
+      show_capture_file_io_error(cfile.save_file, err, TRUE);
+  }
+
   return TRUE;
 
 error:
+  if (cfile.ringbuffer_on) {
+    ringbuf_error_cleanup();
+  }
   g_free(cfile.save_file);
   cfile.save_file = NULL;
   fprintf(stderr, "tethereal: %s\n", errmsg);
@@ -835,17 +1038,9 @@ capture_pcap_cb(u_char *user, const struct pcap_pkthdr *phdr,
 static void
 capture_cleanup(int signum)
 {
-  int err;
-
-  fprintf(stderr, "\n");
-  pcap_close(ld.pch);
-  if (ld.pdh != NULL) {
-    if (!wtap_dump_close(ld.pdh, &err)) {
-      show_capture_file_io_error(cfile.save_file, err, TRUE);
-      exit(2);
-    }
-  }
-  exit(0);
+  /* Just set the loop flag to false. This will initiate 
+     a proper termination. */
+  ld.go = FALSE;
 }
 #endif /* HAVE_LIBPCAP */
 
@@ -909,6 +1104,10 @@ load_cap_file(capture_file *cf, int out_file_type)
     args.pdh = pdh;
     success = wtap_loop(cf->wth, 0, wtap_dispatch_cb_write, (u_char *) &args,
  			&err);
+
+    /* Now close the capture file. */
+    if (!wtap_dump_close(pdh, &err))
+      show_capture_file_io_error(cfile.save_file, err, TRUE);
   } else {
     args.cf = cf;
     args.pdh = NULL;
@@ -962,7 +1161,7 @@ out:
 static void
 fill_in_fdata(frame_data *fdata, capture_file *cf,
 	const struct wtap_pkthdr *phdr,
-	const union wtap_pseudo_header *pseudo_header, int offset)
+	const union wtap_pseudo_header *pseudo_header, long offset)
 {
   int i;
 
@@ -974,7 +1173,6 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
   fdata->pkt_len = phdr->len;
   fdata->cap_len = phdr->caplen;
   fdata->file_off = offset;
-  fdata->cinfo = NULL;
   fdata->lnk_t = phdr->pkt_encap;
   fdata->abs_secs  = phdr->ts.tv_sec;
   fdata->abs_usecs = phdr->ts.tv_usec;
@@ -1007,8 +1205,8 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
   /* If it's greater than the current elapsed time, set the elapsed time
      to it (we check for "greater than" so as not to be confused by
      time moving backwards). */
-  if (cf->esec < fdata->rel_secs
-	|| (cf->esec == fdata->rel_secs && cf->eusec < fdata->rel_usecs)) {
+  if ((gint32)cf->esec < fdata->rel_secs
+	|| ((gint32)cf->esec == fdata->rel_secs && (gint32)cf->eusec < fdata->rel_usecs)) {
     cf->esec = fdata->rel_secs;
     cf->eusec = fdata->rel_usecs;
   }
@@ -1019,12 +1217,6 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
 		fdata->abs_secs, fdata->abs_usecs, prevsec, prevusec);
   prevsec = fdata->abs_secs;
   prevusec = fdata->abs_usecs;
-
-  fdata->cinfo = &cf->cinfo;
-  for (i = 0; i < fdata->cinfo->num_cols; i++) {
-    fdata->cinfo->col_buf[i][0] = '\0';
-    fdata->cinfo->col_data[i] = fdata->cinfo->col_buf[i];
-  }
 }
 
 /* Free up all data attached to a "frame_data" structure. */
@@ -1038,14 +1230,13 @@ clear_fdata(frame_data *fdata)
 }
 
 static void
-wtap_dispatch_cb_write(u_char *user, const struct wtap_pkthdr *phdr, int offset,
-  union wtap_pseudo_header *pseudo_header, const u_char *buf)
+wtap_dispatch_cb_write(u_char *user, const struct wtap_pkthdr *phdr,
+  long offset, union wtap_pseudo_header *pseudo_header, const u_char *buf)
 {
   cb_args_t    *args = (cb_args_t *) user;
   capture_file *cf = args->cf;
   wtap_dumper  *pdh = args->pdh;
   frame_data    fdata;
-  proto_tree   *protocol_tree;
   int           err;
   gboolean      passed;
   epan_dissect_t *edt;
@@ -1053,11 +1244,11 @@ wtap_dispatch_cb_write(u_char *user, const struct wtap_pkthdr *phdr, int offset,
   cf->count++;
   if (cf->rfcode) {
     fill_in_fdata(&fdata, cf, phdr, pseudo_header, offset);
-    protocol_tree = proto_tree_create_root();
-    edt = epan_dissect_new(pseudo_header, buf, &fdata, protocol_tree);
+    edt = epan_dissect_new(TRUE, FALSE);
+    epan_dissect_prime_dfilter(edt, cf->rfcode);
+    epan_dissect_run(edt, pseudo_header, buf, &fdata, NULL);
     passed = dfilter_apply_edt(cf->rfcode, edt);
   } else {
-    protocol_tree = NULL;
     passed = TRUE;
     edt = NULL;
   }
@@ -1079,8 +1270,6 @@ wtap_dispatch_cb_write(u_char *user, const struct wtap_pkthdr *phdr, int offset,
       exit(2);
     }
   }
-  if (protocol_tree != NULL)
-    proto_tree_free(protocol_tree);
   if (edt != NULL)
     epan_dissect_free(edt);
   if (cf->rfcode)
@@ -1135,34 +1324,40 @@ show_capture_file_io_error(const char *fname, int err, gboolean is_close)
 }
 
 static void
-wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
-  union wtap_pseudo_header *pseudo_header, const u_char *buf)
+wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr,
+  long offset, union wtap_pseudo_header *pseudo_header, const u_char *buf)
 {
   cb_args_t    *args = (cb_args_t *) user;
   capture_file *cf = args->cf;
   frame_data    fdata;
-  proto_tree   *protocol_tree;
   gboolean      passed;
   print_args_t  print_args;
   epan_dissect_t *edt;
+  gboolean      create_proto_tree;
   int           i;
 
   cf->count++;
 
-  /* The protocol tree will be "visible", i.e., printed, only if we're
-     not printing a summary. */
-  proto_tree_is_visible = verbose;
-
   fill_in_fdata(&fdata, cf, phdr, pseudo_header, offset);
 
   passed = TRUE;
   if (cf->rfcode || verbose)
-    protocol_tree = proto_tree_create_root();
+    create_proto_tree = TRUE;
   else
-    protocol_tree = NULL;
-  edt = epan_dissect_new(pseudo_header, buf, &fdata, protocol_tree);
-  if (cf->rfcode)
+    create_proto_tree = FALSE;
+  /* The protocol tree will be "visible", i.e., printed, only if we're
+     not printing a summary.
+
+     We only need the columns if we're *not* verbose; in verbose mode,
+     we print the protocol tree, not the protocol summary. */
+  edt = epan_dissect_new(create_proto_tree, verbose);
+  if (cf->rfcode) {
+    epan_dissect_prime_dfilter(edt, cf->rfcode);
+  }
+  epan_dissect_run(edt, pseudo_header, buf, &fdata, verbose ? NULL : &cf->cinfo);
+  if (cf->rfcode) {
     passed = dfilter_apply_edt(cf->rfcode, edt);
+  }
   if (passed) {
     /* The packet passed the read filter. */
     if (verbose) {
@@ -1172,7 +1367,8 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
       print_args.print_summary = FALSE;
       print_args.print_hex = print_hex;
       print_args.expand_all = TRUE;
-      proto_tree_print(FALSE, &print_args, (GNode *)protocol_tree,
+      print_args.suppress_unmarked = FALSE;
+      proto_tree_print(FALSE, &print_args, (GNode *)edt->tree,
 			&fdata, stdout);
       if (!print_hex) {
         /* "print_hex_data()" will put out a leading blank line, as well
@@ -1182,7 +1378,7 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
       }
     } else {
       /* Just fill in the columns. */
-      fill_in_columns(&fdata);
+      epan_dissect_fill_in_columns(edt);
 
       /* Now print them. */
       for (i = 0; i < cf->cinfo.num_cols; i++) {
@@ -1367,7 +1563,6 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
       print_hex_data(stdout, print_args.format, &fdata);
       putchar('\n');
     }
-    fdata.cinfo = NULL;
   }
 
   /* The ANSI C standard does not appear to *require* that a line-buffered
@@ -1392,14 +1587,10 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
      having to wait until a standard I/O buffer fills up. */
   if (line_buffered)
     fflush(stdout);
-  if (protocol_tree != NULL)
-    proto_tree_free(protocol_tree);
 
   epan_dissect_free(edt);
 
   clear_fdata(&fdata);
-
-  proto_tree_is_visible = FALSE;
 }
 
 char *
@@ -1446,7 +1637,7 @@ file_open_error_message(int err, gboolean for_writing)
 
   case WTAP_ERR_SHORT_READ:
     errmsg = "The file \"%s\" appears to have been cut short"
-             " in the middle of a packet.";
+             " in the middle of a packet or other data.";
     break;
 
   case WTAP_ERR_SHORT_WRITE:
