@@ -6,14 +6,12 @@
  *
  * (c) Copyright 2001 Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: text2pcap.c,v 1.3 2001/07/13 00:55:52 guy Exp $
+ * $Id: text2pcap.c,v 1.8 2001/11/24 08:59:38 guy Exp $
  * 
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  * 
- * 
- * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -83,9 +81,11 @@
 # include "config.h"
 #endif
 
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
@@ -103,6 +103,10 @@
 # include "getopt.h"
 #endif
 
+#ifdef NEED_STRPTIME_H
+# include "strptime.h"
+#endif
+
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -137,13 +141,23 @@ unsigned long hdr_udp_src = 0;
 
 /* This is where we store the packet currently being built */
 #define MAX_PACKET 64000
-unsigned char   packet_buf[MAX_PACKET];
+unsigned char packet_buf[MAX_PACKET];
 unsigned long curr_offset = 0;
 
+/* This buffer contains strings present before the packet offset 0 */
+#define PACKET_PREAMBLE_MAX_LEN	2048
+static unsigned char packet_preamble[PACKET_PREAMBLE_MAX_LEN+1];
+static int packet_preamble_len = 0;
+
 /* Number of packets read and written */
 unsigned long num_packets_read = 0;
 unsigned long num_packets_written = 0;
 
+/* Time code of packet, derived from packet_preamble */
+static unsigned long ts_sec  = 0;
+static unsigned long ts_usec = 0;
+static char *ts_fmt = NULL;
+
 /* Input file */
 char *input_filename;
 FILE *input_file = NULL;
@@ -282,6 +296,15 @@ write_byte (char *str)
     curr_offset ++;
 }
 
+/*----------------------------------------------------------------------
+ * Remove bytes from the current packet
+ */
+static void
+unwrite_bytes (unsigned long nbytes)
+{
+    curr_offset -= nbytes;
+}
+
 /*----------------------------------------------------------------------
  * Compute one's complement checksum (from RFC1071)
  */
@@ -337,8 +360,8 @@ write_current_packet (void)
         }
 
         /* Write PCap header */
-        ph.ts_sec = num_packets_written;
-        ph.ts_usec = num_packets_written;
+        ph.ts_sec = ts_sec;
+        ph.ts_usec = ts_usec;
         ph.incl_len = length;
         ph.orig_len = length;
         fwrite(&ph, sizeof(ph), 1, output_file);
@@ -402,6 +425,116 @@ write_file_header (void)
     fwrite(&fh, sizeof(fh), 1, output_file);
 }
 
+/*----------------------------------------------------------------------
+ * Append a token to the packet preamble.
+ */
+static void
+append_to_preamble(char *str)
+{
+    size_t toklen;
+
+    if (packet_preamble_len != 0) {
+        if (packet_preamble_len == PACKET_PREAMBLE_MAX_LEN)
+            return;	/* no room to add more preamble */
+        /* Add a blank separator between the previous token and this token. */
+        packet_preamble[packet_preamble_len++] = ' ';
+    }
+    toklen = strlen(str);
+    if (toklen != 0) {
+        if (packet_preamble_len + toklen > PACKET_PREAMBLE_MAX_LEN)
+            return;	/* no room to add the token to the preamble */
+        strcpy(&packet_preamble[packet_preamble_len], str);
+        packet_preamble_len += toklen;
+    }
+}
+
+/*----------------------------------------------------------------------
+ * Parse the preamble to get the timecode.
+ */
+static void
+parse_preamble (void)
+{
+	struct tm timecode;
+	char *subsecs;
+	char *p;
+	int  subseclen;
+	int  i;
+
+	/*
+	 * If no "-t" flag was specified, don't attempt to parse a packet
+	 * preamble to extract a time stamp.
+	 */
+	if (ts_fmt == NULL)
+	    return;
+
+	ts_sec  = 0;
+	ts_usec = 0;
+
+	/*
+	 * Null-terminate the preamble.
+	 */
+	packet_preamble[packet_preamble_len] = '\0';
+
+	/* Ensure preamble has more than two chars before atempting to parse.
+	 * This should cover line breaks etc that get counted.
+	 */
+	if ( strlen(packet_preamble) > 2 ) {
+		memset(&timecode, '\0', sizeof timecode);
+
+		/* Get Time leaving subseconds */
+		subsecs = strptime( packet_preamble, ts_fmt, &timecode );
+		if (subsecs != NULL) {
+			/* Get the long time from the tm structure */
+			ts_sec  = (unsigned long)mktime( &timecode );
+		} else
+			ts_sec = -1;	/* we failed to parse it */
+
+		/* This will ensure incorrectly parsed dates get set to zero */
+		if ( -1L == (long)ts_sec ) 
+		{
+			ts_sec  = 0;
+			ts_usec = 0;
+		}
+		else
+		{
+			/* Parse subseconds */
+			ts_usec = strtol(subsecs, &p, 10);
+			if (subsecs == p) {
+				/* Error */
+				ts_usec = 0;
+			} else {
+				/*
+				 * Convert that number to a number
+				 * of microseconds; if it's N digits
+				 * long, it's in units of 10^(-N) seconds,
+				 * so, to convert it to units of
+				 * 10^-6 seconds, we multiply by
+				 * 10^(6-N).
+				 */
+				subseclen = p - subsecs;
+				if (subseclen > 6) {
+					/*
+					 * *More* than 6 digits; 6-N is
+					 * negative, so we divide by
+					 * 10^(N-6).
+					 */
+					for (i = subseclen - 6; i != 0; i--)
+						ts_usec /= 10;
+				} else if (subseclen < 6) {
+					for (i = 6 - subseclen; i != 0; i--)
+						ts_usec *= 10;
+				}
+			}
+		}
+	}
+
+
+	/*printf("Format(%s), time(%u), subsecs(%u)\n\n", ts_fmt, ts_sec, ts_usec);*/
+	
+	/* Clear Preamble */
+	packet_preamble_len = 0;
+}
+
 /*----------------------------------------------------------------------
  * Start a new packet 
  */
@@ -415,6 +548,9 @@ start_new_packet (void)
     write_current_packet();
     curr_offset = 0;
     num_packets_read ++;
+
+    /* Ensure we parse the packet preamble as it may contain the time */
+    parse_preamble();
 }
 
 /*----------------------------------------------------------------------
@@ -455,6 +591,9 @@ parse_token (token_t token, char *str)
     /* ----- Waiting for new packet -------------------------------------------*/
     case INIT:
         switch(token) {
+        case T_TEXT:
+            append_to_preamble(str);
+            break;
         case T_DIRECTIVE:
             process_directive(str);
             break;
@@ -474,6 +613,9 @@ parse_token (token_t token, char *str)
     /* ----- Processing packet, start of new line -----------------------------*/
     case START_OF_LINE:
         switch(token) {
+        case T_TEXT:
+            append_to_preamble(str);
+            break;
         case T_DIRECTIVE:
             process_directive(str);
             break;
@@ -484,12 +626,26 @@ parse_token (token_t token, char *str)
                 start_new_packet();
                 state = READ_OFFSET;
             } else if (num != curr_offset) {
-                /* Bad offset; switch to INIT state */
-                if (debug>=1)
-                    fprintf(stderr, "Inconsistent offset. Expecting %0lX, got %0lX. Ignoring rest of packet\n", 
-                            curr_offset, num);
-                write_current_packet();
-                state = INIT;
+                /*
+                 * The offset we read isn't the one we expected.
+                 * This may only mean that we mistakenly interpreted
+                 * some text as byte values (e.g., if the text dump
+                 * of packet data included a number with spaces around
+                 * it).  If the offset is less than what we expected,
+                 * assume that's the problem, and throw away the putative
+                 * extra byte values.
+                 */
+                if (num < curr_offset) {
+                    unwrite_bytes(curr_offset - num);
+                    state = READ_OFFSET;
+                } else {
+                    /* Bad offset; switch to INIT state */
+                    if (debug>=1)
+                        fprintf(stderr, "Inconsistent offset. Expecting %0lX, got %0lX. Ignoring rest of packet\n", 
+                                curr_offset, num);
+                    write_current_packet();
+                    state = INIT;
+                }
             } else 
                 state = READ_OFFSET;
             break;
@@ -569,7 +725,7 @@ help (char *progname)
     fprintf(stderr, 
             "\n"
             "Usage: %s [-d] [-q] [-o h|o] [-l typenum] [-e l3pid] [-i proto] \n"
-            "          [-u srcp destp] <input-filename> <output-filename>\n"
+            "          [-u srcp destp] [-t timefmt] <input-filename> <output-filename>\n"
             "\n"
             "where <input-filename> specifies input filename (use - for standard input)\n"
             "      <output-filename> specifies output filename (use - for standard output)\n"
@@ -585,12 +741,22 @@ help (char *progname)
             " -q           : Generate no output at all (automatically turns off -d)\n"
             " -e l3pid     : Prepend dummy Ethernet II header with specified L3PID (in HEX)\n"
             "                Example: -e 0x800\n"
-            " -i proto     : Prepend dummy IP header with specified IP protocol (in DECIMAL). \n"
-            "                Automatically prepends Ethernet header as well. Example: -i 46\n"
-            " -u srcp destp: Prepend dummy UDP header with specified dest and source ports (in DECIMAL).\n"
-            "                Automatically prepends Ethernet and IP headers as well\n"
-            "                Example: -u 30 40"
-            "\n", 
+            " -i proto     : Prepend dummy IP header with specified IP protocol (in DECIMAL).\n"
+            "                Automatically prepends Ethernet header as well.\n"
+            "                Example: -i 46\n"
+            " -u srcp destp: Prepend dummy UDP header with specified dest and source ports\n"
+            "                (in DECIMAL).\n"
+            "                Automatically prepends Ethernet and IP headers as well.\n"
+            "                Example: -u 30 40\n"
+            " -t timefmt   : Treats the text before the packet as a date/time code; the\n"
+            "                specified argument is a format string of the sort supported\n"
+            "                by strptime.\n"
+            "                Example: The time \"10:15:14.5476\" has the format code\n"
+            "                \"%%H:%%M:%%S.\"\n"
+            "                NOTE:    The subsecond component delimiter must be specified\n"
+            "                         (.) but no pattern is required; the remaining number\n"
+            "                         is assumed to be fractions of a second.\n"
+            "",
             progname);
 
     exit(-1);
@@ -603,9 +769,10 @@ static void
 parse_options (int argc, char *argv[])
 {
     int c;
+    char *p;
 
     /* Scan CLI parameters */
-    while ((c = getopt(argc, argv, "dqr:w:e:i:l:o:u:")) != -1) {
+    while ((c = getopt(argc, argv, "dqr:w:e:i:l:o:u:t:")) != -1) {
         switch(c) {
         case '?': help(argv[0]); break;
         case 'h': help(argv[0]); break;
@@ -613,40 +780,49 @@ parse_options (int argc, char *argv[])
         case 'q': quiet = TRUE; debug = FALSE; break;
         case 'l': pcap_link_type = atoi(optarg); break;
         case 'o': 
-            if (!optarg || (optarg[0]!='h' && optarg[0] != 'o')) {
-                fprintf(stderr, "Bad argument for '-e': %s\n",
-                        optarg ? optarg : "");
+            if (optarg[0]!='h' && optarg[0] != 'o') {
+                fprintf(stderr, "Bad argument for '-e': %s\n", optarg);
                 help(argv[0]);
             }
             offset_base = (optarg[0]=='o') ? 8 : 16;
             break;
         case 'e':
             hdr_ethernet = TRUE;
-            if (!optarg || sscanf(optarg, "%lx", &hdr_ethernet_proto) < 1) {
-                fprintf(stderr, "Bad argument for '-e': %s\n",
-                        optarg ? optarg : "");
+            if (sscanf(optarg, "%lx", &hdr_ethernet_proto) < 1) {
+                fprintf(stderr, "Bad argument for '-e': %s\n", optarg);
                 help(argv[0]);
             }
             break;
             
         case 'i':
             hdr_ip = TRUE;
-            if (!optarg || sscanf(optarg, "%ld", &hdr_ip_proto) < 1) {
-                fprintf(stderr, "Bad argument for '-i': %s\n",
-                        optarg ? optarg : "");
+            if (sscanf(optarg, "%ld", &hdr_ip_proto) < 1) {
+                fprintf(stderr, "Bad argument for '-i': %s\n", optarg);
                 help(argv[0]);
             }
             hdr_ethernet = TRUE;
             hdr_ethernet_proto = 0x800;
             break;
+
+        case 't':
+            ts_fmt = optarg;
+            break;
             
         case 'u':
             hdr_udp = TRUE;
-            if (!optarg || sscanf(optarg, "%ld", &hdr_udp_src) < 1) {
+            hdr_udp_src = strtol(optarg, &p, 10);
+            if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad src port for '-u'\n");
                 help(argv[0]);
             }
-            if (optind >= argc || sscanf(argv[optind], "%ld", &hdr_udp_dest) < 1) {
+            if (*p == '\0') {
+                fprintf(stderr, "No dest port specified for '-u'\n");
+                help(argv[0]);
+            }
+            p++;
+            optarg = p;
+            hdr_udp_dest = strtol(optarg, &p, 10);
+            if (p == optarg || *p != '\0') {
                 fprintf(stderr, "Bad dest port for '-u'\n");
                 help(argv[0]);
             }
