@@ -1,9 +1,9 @@
 /* file.c
  *
- * $Id: file.c,v 1.67 2001/07/13 00:55:58 guy Exp $
+ * $Id: file.c,v 1.77 2001/12/04 22:28:19 guy Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,8 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
  */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -36,6 +36,10 @@
 #include <sys/stat.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <errno.h>
 
 #ifdef HAVE_IO_H
@@ -60,6 +64,9 @@
 #include "csids.h"
 #include "pppdump.h"
 #include "etherpeek.h"
+#include "vms.h"
+#include "dbs-etherwatch.h"
+#include "visual.h"
 
 /* The open_file_* routines should return:
  *
@@ -96,22 +103,26 @@ static int (*open_routines[])(wtap *, int *) = {
 	netxray_open,
 	radcom_open,
 	nettl_open,
+	visual_open,
 	pppdump_open,
-	etherpeek_open,
 
-	/* Files whose magic headers are in text *somewhere* in the
-	 * file (usually because the trace is just a saved copy of
-	 * the telnet session). 
+	/* Files that don't have magic bytes at a fixed location,
+	 * but that instead require a heuristic of some sort to
+	 * identify them.  This includes the ASCII trace files that
+	 * would be saved copies of a Telnet session to some box.
 	 */
+	etherpeek_open,
 	ascend_open,
 	toshiba_open,
 	i4btrace_open,
 	csids_open,
+	vms_open,
+	dbs_etherwatch_open,
 };
 
 #define	N_FILE_TYPES	(sizeof open_routines / sizeof open_routines[0])
 
-int wtap_def_seek_read(wtap *wth, int seek_off,
+int wtap_def_seek_read(wtap *wth, long seek_off,
 	union wtap_pseudo_header *pseudo_header, guint8 *pd, int len)
 {
 	file_seek(wth->random_fh, seek_off, SEEK_SET);
@@ -286,6 +297,10 @@ static const struct file_type_info {
 	{ "Nokia libpcap (tcpdump)", "nokialibpcap",
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
+	/* WTAP_FILE_PCAP_AIX */
+	{ "AIX libpcap (tcpdump)", NULL,
+	  NULL, NULL },
+
 	/* WTAP_FILE_LANALYZER */
 	{ "Novell LANalyzer", NULL,
 	  NULL, NULL },
@@ -365,6 +380,18 @@ static const struct file_type_info {
 	/* WTAP_FILE_ETHERPEEK_MAC_V7 */
 	{ "Etherpeek trace (Macintosh V7)", NULL,
 	  NULL, NULL },
+
+	/* WTAP_FILE_VMS */
+	{ "TCPIPtrace (VMS)", NULL,
+	  NULL, NULL},
+
+	/* WTAP_FILE_DBS_ETHERWATCH */
+	{ "DBS Etherwatch (VMS)", NULL,
+	  NULL, NULL},
+
+	/* WTAP_FILE_VISUAL_NETWORKS */
+	{ "Visual Networks traffic capture", "visual",
+	  visual_dump_can_write_encap, visual_dump_open },
 };
 
 /* Name that should be somewhat descriptive. */
@@ -420,14 +447,28 @@ gboolean wtap_dump_can_write_encap(int filetype, int encap)
 	return TRUE;
 }
 
-static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype,
+static gboolean wtap_dump_open_check(int filetype, int encap, int *err);
+static wtap_dumper* wtap_dump_alloc_wdh(int filetype, int encap, int snaplen,
+    int *err);
+static gboolean wtap_dump_open_finish(wtap_dumper *wdh, int filetype,
     int encap, int snaplen, int *err);
 
 wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
 				int snaplen, int *err)
 {
+	wtap_dumper *wdh;
 	FILE *fh;
 
+	/* Check whether we can open a capture file with that file type
+	   and that encapsulation. */
+	if (!wtap_dump_open_check(filetype, encap, err))
+		return NULL;
+
+	/* Allocate a data structure for the output stream. */
+	wdh = wtap_dump_alloc_wdh(filetype, encap, snaplen, err);
+	if (wdh == NULL)
+		return NULL;	/* couldn't allocate it */
+
 	/* In case "fopen()" fails but doesn't set "errno", set "errno"
 	   to a generic "the open failed" error. */
 	errno = WTAP_ERR_CANT_OPEN;
@@ -436,14 +477,33 @@ wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
 		*err = errno;
 		return NULL;	/* can't create file */
 	}
-	return wtap_dump_open_common(fh, filetype, encap, snaplen, err);
+	wdh->fh = fh;
+
+	if (!wtap_dump_open_finish(wdh, filetype, encap, snaplen, err)) {
+		/* Get rid of the file we created; we couldn't finish
+		   opening it. */
+		unlink(filename);
+		return NULL;
+	}
+	return wdh;
 }
 
 wtap_dumper* wtap_dump_fdopen(int fd, int filetype, int encap, int snaplen,
 				int *err)
 {
+	wtap_dumper *wdh;
 	FILE *fh;
 
+	/* Check whether we can open a capture file with that file type
+	   and that encapsulation. */
+	if (!wtap_dump_open_check(filetype, encap, err))
+		return NULL;
+
+	/* Allocate a data structure for the output stream. */
+	wdh = wtap_dump_alloc_wdh(filetype, encap, snaplen, err);
+	if (wdh == NULL)
+		return NULL;	/* couldn't allocate it */
+
 	/* In case "fopen()" fails but doesn't set "errno", set "errno"
 	   to a generic "the open failed" error. */
 	errno = WTAP_ERR_CANT_OPEN;
@@ -452,63 +512,68 @@ wtap_dumper* wtap_dump_fdopen(int fd, int filetype, int encap, int snaplen,
 		*err = errno;
 		return NULL;	/* can't create standard I/O stream */
 	}
-	return wtap_dump_open_common(fh, filetype, encap, snaplen, err);
+	wdh->fh = fh;
+
+	if (!wtap_dump_open_finish(wdh, filetype, encap, snaplen, err))
+		return NULL;
+	return wdh;
 }
 
-static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype, int encap,
-					int snaplen, int *err)
+static gboolean wtap_dump_open_check(int filetype, int encap, int *err)
 {
-	wtap_dumper *wdh;
-
-	if (filetype < 0 || filetype >= WTAP_NUM_FILE_TYPES
-	    || dump_open_table[filetype].dump_open == NULL) {
+	if (!wtap_dump_can_open(filetype)) {
 		/* Invalid type, or type we don't know how to write. */
 		*err = WTAP_ERR_UNSUPPORTED_FILE_TYPE;
-		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
-		   will be closed if we can't write that file type. */
-		fclose(fh);
-		return NULL;
+		return FALSE;
 	}
 
 	/* OK, we know how to write that type; can we write the specified
 	   encapsulation type? */
 	*err = (*dump_open_table[filetype].can_write_encap)(filetype, encap);
-	if (*err != 0) {
-		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
-		   will be closed if we can't write that encapsulation type. */
-		fclose(fh);
-		return NULL;
-	}
+	if (*err != 0)
+		return FALSE;
+
+	/* All systems go! */
+	return TRUE;
+}
+
+static wtap_dumper* wtap_dump_alloc_wdh(int filetype, int encap, int snaplen,
+					int *err)
+{
+	wtap_dumper *wdh;
 
-	/* OK, we can write the specified encapsulation type.  Allocate
-	   a data structure for the output stream. */
 	wdh = g_malloc(sizeof (wtap_dumper));
 	if (wdh == NULL) {
 		*err = errno;
-		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
-		   will be closed if the malloc fails. */
-		fclose(fh);
 		return NULL;
 	}
-	wdh->fh = fh;
+	wdh->fh = NULL;
 	wdh->file_type = filetype;
 	wdh->snaplen = snaplen;
 	wdh->encap = encap;
+	wdh->bytes_dumped = 0;
 	wdh->dump.opaque = NULL;
 	wdh->subtype_write = NULL;
 	wdh->subtype_close = NULL;
+	return wdh;
+}
 
+static gboolean wtap_dump_open_finish(wtap_dumper *wdh, int filetype,
+				      int encap, int snaplen, int *err)
+{
 	/* Now try to open the file for writing. */
 	if (!(*dump_open_table[filetype].dump_open)(wdh, err)) {
-		/* The attempt failed. */
-		g_free(wdh);
-		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
+		/* The attempt failed.  Close the stream for the file.
+		   NOTE: this means the FD handed to "wtap_dump_fdopen()"
 		   will be closed if the open fails. */
-		fclose(fh);
-		return NULL;
+		fclose(wdh->fh);
+
+		/* Now free up the dumper handle. */
+		g_free(wdh);
+		return FALSE;
 	}
 
-	return wdh;	/* success! */
+	return TRUE;	/* success! */
 }
 
 FILE* wtap_dump_file(wtap_dumper *wdh)
@@ -547,3 +612,14 @@ gboolean wtap_dump_close(wtap_dumper *wdh, int *err)
 	g_free(wdh);
 	return ret;
 }
+
+long wtap_get_bytes_dumped(wtap_dumper *wdh)
+{
+	return wdh->bytes_dumped;
+}
+
+void wtap_set_bytes_dumped(wtap_dumper *wdh, long bytes_dumped)
+{
+	wdh->bytes_dumped = bytes_dumped;
+}
+
