@@ -1,9 +1,9 @@
 /* iptrace.c
  *
- * $Id: iptrace.c,v 1.33 2001/03/10 06:33:57 guy Exp $
+ * $Id: iptrace.c,v 1.36 2001/11/13 23:55:43 gram Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,12 +31,12 @@
 #include "buffer.h"
 #include "iptrace.h"
 
-static gboolean iptrace_read_1_0(wtap *wth, int *err, int *data_offset);
-static int iptrace_seek_read_1_0(wtap *wth, int seek_off,
+static gboolean iptrace_read_1_0(wtap *wth, int *err, long *data_offset);
+static int iptrace_seek_read_1_0(wtap *wth, long seek_off,
     union wtap_pseudo_header *pseudo_header, u_char *pd, int packet_size);
 
-static gboolean iptrace_read_2_0(wtap *wth, int *err, int *data_offset);
-static int iptrace_seek_read_2_0(wtap *wth, int seek_off,
+static gboolean iptrace_read_2_0(wtap *wth, int *err, long *data_offset);
+static int iptrace_seek_read_2_0(wtap *wth, long seek_off,
     union wtap_pseudo_header *pseudo_header, u_char *pd, int packet_size);
 
 static int iptrace_read_rec_header(FILE_T fh, guint8 *header, int header_len,
@@ -96,7 +96,7 @@ typedef struct {
 } iptrace_1_0_phdr;
 
 /* Read the next packet */
-static gboolean iptrace_read_1_0(wtap *wth, int *err, int *data_offset)
+static gboolean iptrace_read_1_0(wtap *wth, int *err, long *data_offset)
 {
 	int			ret;
 	guint32			packet_size;
@@ -161,7 +161,7 @@ static gboolean iptrace_read_1_0(wtap *wth, int *err, int *data_offset)
 	return TRUE;
 }
 
-static int iptrace_seek_read_1_0(wtap *wth, int seek_off,
+static int iptrace_seek_read_1_0(wtap *wth, long seek_off,
     union wtap_pseudo_header *pseudo_header, u_char *pd, int packet_size)
 {
 	int			ret;
@@ -204,7 +204,7 @@ typedef struct {
 } iptrace_2_0_phdr;
 
 /* Read the next packet */
-static gboolean iptrace_read_2_0(wtap *wth, int *err, int *data_offset)
+static gboolean iptrace_read_2_0(wtap *wth, int *err, long *data_offset)
 {
 	int			ret;
 	guint32			packet_size;
@@ -273,7 +273,7 @@ static gboolean iptrace_read_2_0(wtap *wth, int *err, int *data_offset)
 	return TRUE;
 }
 
-static int iptrace_seek_read_2_0(wtap *wth, int seek_off,
+static int iptrace_seek_read_2_0(wtap *wth, long seek_off,
     union wtap_pseudo_header *pseudo_header, u_char *pd, int packet_size)
 {
 	int			ret;
@@ -404,7 +404,7 @@ wtap_encap_ift(unsigned int  ift)
 /* 0x4 */	WTAP_ENCAP_RAW_IP,	/* IFT_X25DDN */
 /* 0x5 */	WTAP_ENCAP_UNKNOWN,	/* IFT_X25 */
 /* 0x6 */	WTAP_ENCAP_ETHERNET,	/* IFT_ETHER */
-/* 0x7 */	WTAP_ENCAP_UNKNOWN,	/* IFT_ISO88023 */
+/* 0x7 */	WTAP_ENCAP_ETHERNET,	/* IFT_ISO88023 */
 /* 0x8 */	WTAP_ENCAP_UNKNOWN,	/* IFT_ISO88024 */
 /* 0x9 */	WTAP_ENCAP_TOKEN_RING,	/* IFT_ISO88025 */
 /* 0xa */	WTAP_ENCAP_UNKNOWN,	/* IFT_ISO88026 */
