@@ -1,9 +1,9 @@
 /* libpcap.c
  *
- * $Id: libpcap.c,v 1.49 2001/07/13 00:55:58 guy Exp $
+ * $Id: libpcap.c,v 1.62 2001/12/04 07:32:05 guy Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -47,7 +47,7 @@ typedef enum {
 } libpcap_try_t;
 static libpcap_try_t libpcap_try(wtap *wth, int *err);
 
-static gboolean libpcap_read(wtap *wth, int *err, int *data_offset);
+static gboolean libpcap_read(wtap *wth, int *err, long *data_offset);
 static int libpcap_read_header(wtap *wth, int *err,
     struct pcaprec_ss990915_hdr *hdr, gboolean silent);
 static void adjust_header(wtap *wth, struct pcaprec_hdr *hdr);
@@ -363,6 +363,11 @@ static const struct {
 	 * of libpcap.
 	 */
 	{ 113,		WTAP_ENCAP_SLL },	/* Linux cooked capture */
+
+	{ 114,		WTAP_ENCAP_LOCALTALK },	/* Localtalk */
+
+	{ 118,		WTAP_ENCAP_CISCO_IOS },
+	{ 119,		WTAP_ENCAP_PRISM_HEADER }, /* Prism monitor mode hdr */
 };
 #define NUM_PCAP_ENCAPS (sizeof pcap_to_wtap_map / sizeof pcap_to_wtap_map[0])
 
@@ -373,6 +378,7 @@ int libpcap_open(wtap *wth, int *err)
 	struct pcap_hdr hdr;
 	gboolean byte_swapped;
 	gboolean modified;
+	gboolean aix;
 	int file_encap;
 
 	/* Read in the number that should be at the start of a "libpcap" file */
@@ -448,6 +454,54 @@ int libpcap_open(wtap *wth, int *err)
 		*err = WTAP_ERR_UNSUPPORTED;
 		return -1;
 	}
+
+	/*
+	 * AIX's non-standard tcpdump uses a minor version number of 2.
+	 * Unfortunately, older versions of libpcap might have used
+	 * that as well.
+	 *
+	 * The AIX libpcap uses RFC 1573 ifType values rather than
+	 * DLT_ values in the header; the ifType values for LAN devices
+	 * are:
+	 *
+	 *	Ethernet	6
+	 *	Token Ring	9
+	 *	FDDI		15
+	 *
+	 * which correspond to DLT_IEEE802 (used for Token Ring),
+	 * DLT_PPP, and DLT_SLIP_BSDOS, respectively.  We shall
+	 * assume that if the minor version number is 2, and
+	 * the network type is 6, 9, or 15, that it's AIX libpcap;
+	 *
+	 * I'm assuming those older versions of libpcap didn't
+	 * use DLT_IEEE802 for Token Ring, and didn't use DLT_SLIP_BSDOS
+	 * as that came later.  It may have used DLT_SLIP, however, in
+	 * which case we're out of luck; we assume it's Token Ring
+	 * in AIX libpcap rather than PPP in standard libpcap, as
+	 * you're probably more likely to be handing an AIX libpcap
+	 * token-ring capture than an old (pre-libpcap 0.4) PPP capture
+	 * to Ethereal.
+	 */
+	aix = FALSE;	/* assume it's not AIX */
+	if (hdr.version_major == 2 && hdr.version_minor == 2) {
+		switch (hdr.network) {
+
+		case 6:
+			hdr.network = 1;	/* DLT_EN10MB, Ethernet */
+			aix = TRUE;
+			break;
+
+		case 8:
+			hdr.network = 6;	/* DLT_IEEE802, Token Ring */
+			aix = TRUE;
+			break;
+
+		case 15:
+			hdr.network = 10;	/* DLT_FDDI, FDDI */
+			aix = TRUE;
+			break;
+		}
+	}
 	file_encap = wtap_pcap_encap_to_wtap_encap(hdr.network);
 	if (file_encap == WTAP_ENCAP_UNKNOWN) {
 		g_message("pcap: network type %u unknown or unsupported",
@@ -468,7 +522,18 @@ int libpcap_open(wtap *wth, int *err)
 	wth->snapshot_length = hdr.snaplen;
 
 	/*
-	 * Yes.  Let's look at the header for the first record,
+	 * Is this AIX format?
+	 */
+	if (aix) {
+		/*
+		 * Yes.  Skip all the tests for other mutant formats.
+		 */
+		wth->file_type = WTAP_FILE_PCAP_AIX;
+		return 1;
+	}
+
+	/*
+	 * No.  Let's look at the header for the first record,
 	 * and see if, interpreting it as a standard header (if the
 	 * magic number was standard) or a modified header (if the
 	 * magic number was modified), the position where it says the
@@ -705,7 +770,7 @@ static libpcap_try_t libpcap_try(wtap *wth, int *err)
 }
 
 /* Read the next packet */
-static gboolean libpcap_read(wtap *wth, int *err, int *data_offset)
+static gboolean libpcap_read(wtap *wth, int *err, long *data_offset)
 {
 	struct pcaprec_ss990915_hdr hdr;
 	guint packet_size;
@@ -728,7 +793,7 @@ static gboolean libpcap_read(wtap *wth, int *err, int *data_offset)
 	bytes_read = file_read(buffer_start_ptr(wth->frame_buffer), 1,
 			packet_size, wth->fh);
 
-	if (bytes_read != packet_size) {
+	if ((guint)bytes_read != packet_size) {
 		*err = file_error(wth->fh);
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
@@ -760,6 +825,7 @@ static int libpcap_read_header(wtap *wth, int *err,
 	switch (wth->file_type) {
 
 	case WTAP_FILE_PCAP:
+	case WTAP_FILE_PCAP_AIX:
 		bytes_to_read = sizeof (struct pcaprec_hdr);
 		break;
 
@@ -839,6 +905,11 @@ adjust_header(wtap *wth, struct pcaprec_hdr *hdr)
 		hdr->orig_len = BSWAP32(hdr->orig_len);
 	}
 
+	/* If this is AIX, convert the time stamp from seconds/nanoseconds
+	   to seconds/microseconds.  */
+	if (wth->file_type == WTAP_FILE_PCAP_AIX)
+		hdr->ts_usec = hdr->ts_usec/1000;
+
 	/* In file format version 2.3, the "incl_len" and "orig_len" fields
 	   were swapped, in order to match the BPF header layout.
 
@@ -916,7 +987,7 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, int *err)
 {
 	guint32 magic;
 	struct pcap_hdr file_hdr;
-	int nwritten;
+	size_t nwritten;
 
 	/* This is a libpcap file */
 	wdh->subtype_write = libpcap_dump;
@@ -945,12 +1016,13 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, int *err)
 
 	nwritten = fwrite(&magic, 1, sizeof magic, wdh->fh);
 	if (nwritten != sizeof magic) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
 	}
+	wdh->bytes_dumped += sizeof magic;
 
 	/* current "libpcap" format is 2.4 */
 	file_hdr.version_major = 2;
@@ -961,12 +1033,13 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, int *err)
 	file_hdr.network = wtap_wtap_encap_to_pcap_encap(wdh->encap);
 	nwritten = fwrite(&file_hdr, 1, sizeof file_hdr, wdh->fh);
 	if (nwritten != sizeof file_hdr) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
 	}
+	wdh->bytes_dumped += sizeof file_hdr;
 
 	return TRUE;
 }
@@ -977,8 +1050,8 @@ static gboolean libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     const union wtap_pseudo_header *pseudo_header, const u_char *pd, int *err)
 {
 	struct pcaprec_ss990915_hdr rec_hdr;
-	int hdr_size;
-	int nwritten;
+	size_t hdr_size;
+	size_t nwritten;
 
 	rec_hdr.hdr.ts_sec = phdr->ts.tv_sec;
 	rec_hdr.hdr.ts_usec = phdr->ts.tv_usec;
@@ -1045,19 +1118,21 @@ static gboolean libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 
 	nwritten = fwrite(&rec_hdr, 1, hdr_size, wdh->fh);
 	if (nwritten != hdr_size) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
 	}
+	wdh->bytes_dumped += hdr_size;
 	nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
 	if (nwritten != phdr->caplen) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
 	}
+        wdh->bytes_dumped += phdr->caplen;
 	return TRUE;
 }
