@@ -1,9 +1,9 @@
 /* netmon.c
  *
- * $Id: netmon.c,v 1.38 2001/07/13 00:55:58 guy Exp $
+ * $Id: netmon.c,v 1.44 2001/12/04 23:38:55 guy Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -93,7 +93,7 @@ struct netmonrec_2_x_hdr {
 	guint32	incl_len;	/* number of octets captured in file */
 };
 
-static gboolean netmon_read(wtap *wth, int *err, int *data_offset);
+static gboolean netmon_read(wtap *wth, int *err, long *data_offset);
 static void netmon_close(wtap *wth);
 static gboolean netmon_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     const union wtap_pseudo_header *pseudo_header, const u_char *pd, int *err);
@@ -123,7 +123,7 @@ int netmon_open(wtap *wth, int *err)
 	struct tm tm;
 	int frame_table_offset;
 	guint32 frame_table_length;
-	int frame_table_size;
+	guint32 frame_table_size;
 	guint32 *frame_table;
 #ifdef WORDS_BIGENDIAN
 	unsigned int i;
@@ -252,11 +252,11 @@ int netmon_open(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	file_seek(wth->fh, frame_table_offset, SEEK_SET);
 	bytes_read = file_read(frame_table, 1, frame_table_length, wth->fh);
-	if (bytes_read != frame_table_length) {
+	if ((guint32)bytes_read != frame_table_length) {
 		*err = file_error(wth->fh);
-		if (*err != 0)
-			return -1;
-		return 0;
+		if (*err == 0)
+			*err = WTAP_ERR_SHORT_READ;
+		return -1;
 	}
 	wth->capture.netmon->frame_table_size = frame_table_size;
 	wth->capture.netmon->frame_table = frame_table;
@@ -276,7 +276,7 @@ int netmon_open(wtap *wth, int *err)
 }
 
 /* Read the next packet */
-static gboolean netmon_read(wtap *wth, int *err, int *data_offset)
+static gboolean netmon_read(wtap *wth, int *err, long *data_offset)
 {
 	netmon_t *netmon = wth->capture.netmon;
 	guint32	packet_size = 0;
@@ -361,7 +361,7 @@ static gboolean netmon_read(wtap *wth, int *err, int *data_offset)
 	bytes_read = file_read(buffer_start_ptr(wth->frame_buffer), 1,
 			packet_size, wth->fh);
 
-	if (bytes_read != packet_size) {
+	if ((guint32)bytes_read != packet_size) {
 		*err = file_error(wth->fh);
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
@@ -435,7 +435,7 @@ int netmon_dump_can_write_encap(int filetype, int encap)
 	if (encap == WTAP_ENCAP_PER_PACKET)
 		return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
 
-	if (encap < 0 || encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
+	if (encap < 0 || (unsigned) encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
 		return WTAP_ERR_UNSUPPORTED_ENCAP;
 
 	return 0;
@@ -474,8 +474,8 @@ static gboolean netmon_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 	struct netmonrec_1_x_hdr rec_1_x_hdr;
 	struct netmonrec_2_x_hdr rec_2_x_hdr;
 	char *hdrp;
-	int hdr_size;
-	int nwritten;
+	size_t hdr_size;
+	size_t nwritten;
 	double t;
 	guint32 time_low, time_high;
 
@@ -528,7 +528,7 @@ static gboolean netmon_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 
 	nwritten = fwrite(hdrp, 1, hdr_size, wdh->fh);
 	if (nwritten != hdr_size) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
@@ -536,7 +536,7 @@ static gboolean netmon_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 	}
 	nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
 	if (nwritten != phdr->caplen) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
@@ -578,11 +578,11 @@ static gboolean netmon_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 static gboolean netmon_dump_close(wtap_dumper *wdh, int *err)
 {
 	netmon_dump_t *netmon = wdh->dump.netmon;
-	int n_to_write;
-	int nwritten;
+	size_t n_to_write;
+	size_t nwritten;
 	struct netmon_hdr file_hdr;
 	const char *magicp;
-	int magic_size;
+	size_t magic_size;
 	struct tm *tm;
 
 	/* Write out the frame table.  "netmon->frame_table_index" is
@@ -590,7 +590,7 @@ static gboolean netmon_dump_close(wtap_dumper *wdh, int *err)
 	n_to_write = netmon->frame_table_index * sizeof *netmon->frame_table;
 	nwritten = fwrite(netmon->frame_table, 1, n_to_write, wdh->fh);
 	if (nwritten != n_to_write) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
@@ -626,7 +626,7 @@ static gboolean netmon_dump_close(wtap_dumper *wdh, int *err)
 	}
 	nwritten = fwrite(magicp, 1, magic_size, wdh->fh);
 	if (nwritten != magic_size) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
@@ -635,21 +635,31 @@ static gboolean netmon_dump_close(wtap_dumper *wdh, int *err)
 
 	file_hdr.network = htoles(wtap_encap[wdh->encap]);
 	tm = localtime(&netmon->first_record_time.tv_sec);
-	file_hdr.ts_year = htoles(1900 + tm->tm_year);
-	file_hdr.ts_month = htoles(tm->tm_mon + 1);
-	file_hdr.ts_dow = htoles(tm->tm_wday);
-	file_hdr.ts_day = htoles(tm->tm_mday);
-	file_hdr.ts_hour = htoles(tm->tm_hour);
-	file_hdr.ts_min = htoles(tm->tm_min);
-	file_hdr.ts_sec = htoles(tm->tm_sec);
-	file_hdr.ts_msec = htoles(netmon->first_record_time.tv_usec/1000);
+    if (tm != NULL) {
+      file_hdr.ts_year  = htoles(1900 + tm->tm_year);
+      file_hdr.ts_month = htoles(tm->tm_mon + 1);
+      file_hdr.ts_dow   = htoles(tm->tm_wday);
+      file_hdr.ts_day   = htoles(tm->tm_mday);
+      file_hdr.ts_hour  = htoles(tm->tm_hour);
+      file_hdr.ts_min   = htoles(tm->tm_min);
+      file_hdr.ts_sec   = htoles(tm->tm_sec);
+    } else {
+      file_hdr.ts_year  = htoles(1900 + 0);
+      file_hdr.ts_month = htoles(0 + 1);
+      file_hdr.ts_dow   = htoles(0);
+      file_hdr.ts_day   = htoles(0);
+      file_hdr.ts_hour  = htoles(0);
+      file_hdr.ts_min   = htoles(0);
+      file_hdr.ts_sec   = htoles(0);
+    }      
+    file_hdr.ts_msec  = htoles(netmon->first_record_time.tv_usec/1000);
 		/* XXX - what about rounding? */
 	file_hdr.frametableoffset = htolel(netmon->frame_table_offset);
 	file_hdr.frametablelength =
 	    htolel(netmon->frame_table_index * sizeof *netmon->frame_table);
 	nwritten = fwrite(&file_hdr, 1, sizeof file_hdr, wdh->fh);
 	if (nwritten != sizeof file_hdr) {
-		if (nwritten < 0)
+		if (nwritten == 0 && ferror(wdh->fh))
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
