@@ -1,9 +1,9 @@
 /* nettl.c
  *
- * $Id: nettl.c,v 1.19 2001/03/10 06:33:57 guy Exp $
+ * $Id: nettl.c,v 1.21 2001/11/13 23:55:43 gram Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,8 +66,8 @@ struct nettlrec_ns_ls_ip_hdr {
 
 /* header is followed by data and once again the total length (2 bytes) ! */
 
-static gboolean nettl_read(wtap *wth, int *err, int *data_offset);
-static int nettl_seek_read(wtap *wth, int seek_off,
+static gboolean nettl_read(wtap *wth, int *err, long *data_offset);
+static int nettl_seek_read(wtap *wth, long seek_off,
 		union wtap_pseudo_header *pseudo_header, u_char *pd, int length);
 static int nettl_read_rec_header(wtap *wth, FILE_T fh,
 		struct wtap_pkthdr *phdr, union wtap_pseudo_header *pseudo_header,
@@ -124,7 +124,7 @@ int nettl_open(wtap *wth, int *err)
 }
 
 /* Read the next packet */
-static gboolean nettl_read(wtap *wth, int *err, int *data_offset)
+static gboolean nettl_read(wtap *wth, int *err, long *data_offset)
 {
     int ret;
 
@@ -150,7 +150,7 @@ static gboolean nettl_read(wtap *wth, int *err, int *data_offset)
 }
 
 static int
-nettl_seek_read(wtap *wth, int seek_off,
+nettl_seek_read(wtap *wth, long seek_off,
 		union wtap_pseudo_header *pseudo_header, u_char *pd, int length)
 {
     int ret;
