@@ -1,9 +1,9 @@
 /* netxray.c
  *
- * $Id: netxray.c,v 1.39 2001/05/09 04:42:27 guy Exp $
+ * $Id: netxray.c,v 1.43 2001/11/13 23:55:43 gram Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -101,7 +101,7 @@ struct netxrayrec_2_x_hdr {
 	guint32	xxx[7];		/* unknown */
 };
 
-static gboolean netxray_read(wtap *wth, int *err, int *data_offset);
+static gboolean netxray_read(wtap *wth, int *err, long *data_offset);
 static void netxray_close(wtap *wth);
 static gboolean netxray_dump_1_1(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 	const union wtap_pseudo_header *pseudo_header, const u_char *pd, int *err);
@@ -231,7 +231,7 @@ int netxray_open(wtap *wth, int *err)
 }
 
 /* Read the next packet */
-static gboolean netxray_read(wtap *wth, int *err, int *data_offset)
+static gboolean netxray_read(wtap *wth, int *err, long *data_offset)
 {
 	guint32	packet_size;
 	int	bytes_read;
@@ -292,7 +292,7 @@ reread:
 	bytes_read = file_read(buffer_start_ptr(wth->frame_buffer), 1,
 			packet_size, wth->fh);
 
-	if (bytes_read != packet_size) {
+	if ((guint32)bytes_read != packet_size) {
 		*err = file_error(wth->fh);
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
@@ -346,7 +346,7 @@ int netxray_dump_can_write_encap(int filetype, int encap)
     if (encap == WTAP_ENCAP_PER_PACKET)
 	return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
 
-    if (encap < 0 || encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
+    if (encap < 0 || (unsigned)encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
 	return WTAP_ERR_UNSUPPORTED_ENCAP;
 
     return 0;
@@ -383,7 +383,7 @@ static gboolean netxray_dump_1_1(wtap_dumper *wdh, const struct wtap_pkthdr *phd
     netxray_dump_t *netxray = wdh->dump.netxray;
     guint32 timestamp;
     struct netxrayrec_1_x_hdr rec_hdr;
-    int nwritten;
+    size_t nwritten;
 
     /* NetXRay/Windows Sniffer files have a capture start date/time
        in the header, in a UNIX-style format, with one-second resolution,
@@ -411,7 +411,7 @@ static gboolean netxray_dump_1_1(wtap_dumper *wdh, const struct wtap_pkthdr *phd
 	
     nwritten = fwrite(&rec_hdr, 1, sizeof(rec_hdr), wdh->fh);
     if (nwritten != sizeof(rec_hdr)) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -421,7 +421,7 @@ static gboolean netxray_dump_1_1(wtap_dumper *wdh, const struct wtap_pkthdr *phd
     /* write the packet data */	
     nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
     if (nwritten != phdr->caplen) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -441,7 +441,7 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     netxray_dump_t *netxray = wdh->dump.netxray;
     guint32 filelen;
     struct netxray_hdr file_hdr;
-    int nwritten;
+    size_t nwritten;
 
     filelen = ftell(wdh->fh);
 
@@ -451,7 +451,7 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     /* Rewrite the file header. */
     nwritten = fwrite(netxray_magic, 1, sizeof netxray_magic, wdh->fh);
     if (nwritten != sizeof netxray_magic) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -473,7 +473,7 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     memcpy(hdr_buf, &file_hdr, sizeof(file_hdr));
     nwritten = fwrite(hdr_buf, 1, sizeof hdr_buf, wdh->fh);
     if (nwritten != sizeof hdr_buf) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
