@@ -1,9 +1,9 @@
 /* ngsniffer.c
  *
- * $Id: ngsniffer.c,v 1.64 2001/07/06 00:17:36 guy Exp $
+ * $Id: ngsniffer.c,v 1.69 2001/11/13 23:55:43 gram Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -285,8 +285,8 @@ struct frame6_rec {
 static double Usec[] = { 15.0, 0.838096, 15.0, 0.5, 2.0, 1.0, 0.1 };
 
 static int skip_header_records(wtap *wth, int *err, gint16 version);
-static gboolean ngsniffer_read(wtap *wth, int *err, int *data_offset);
-static int ngsniffer_seek_read(wtap *wth, int seek_off,
+static gboolean ngsniffer_read(wtap *wth, int *err, long *data_offset);
+static int ngsniffer_seek_read(wtap *wth, long seek_off,
     union wtap_pseudo_header *pseudo_header, u_char *pd, int packet_size);
 static int ngsniffer_read_rec_header(wtap *wth, gboolean is_random,
     guint16 *typep, guint16 *lengthp, int *err);
@@ -645,7 +645,7 @@ skip_header_records(wtap *wth, int *err, gint16 version)
 }
 
 /* Read the next packet */
-static gboolean ngsniffer_read(wtap *wth, int *err, int *data_offset)
+static gboolean ngsniffer_read(wtap *wth, int *err, long *data_offset)
 {
 	int	ret;
 	guint16	type, length;
@@ -858,7 +858,7 @@ found:
 	return TRUE;
 }
 
-static int ngsniffer_seek_read(wtap *wth, int seek_off,
+static int ngsniffer_seek_read(wtap *wth, long seek_off,
     union wtap_pseudo_header *pseudo_header, u_char *pd, int packet_size)
 {
 	int	ret;
@@ -1139,7 +1139,7 @@ int ngsniffer_dump_can_write_encap(int filetype, int encap)
     if (encap == WTAP_ENCAP_PER_PACKET)
 	return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
 
-    if (encap < 0 || encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
+    if (encap < 0 || (unsigned)encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
 	return WTAP_ERR_UNSUPPORTED_ENCAP;
 
     return 0;
@@ -1149,7 +1149,7 @@ int ngsniffer_dump_can_write_encap(int filetype, int encap)
    failure */
 gboolean ngsniffer_dump_open(wtap_dumper *wdh, int *err)
 {
-    int nwritten;
+    size_t nwritten;
     char buf[6] = {REC_VERS, 0x00, 0x12, 0x00, 0x00, 0x00}; /* version record */
 
     /* This is a sniffer file */
@@ -1163,7 +1163,7 @@ gboolean ngsniffer_dump_open(wtap_dumper *wdh, int *err)
     /* Write the file header. */
     nwritten = fwrite(ngsniffer_magic, 1, sizeof ngsniffer_magic, wdh->fh);
     if (nwritten != sizeof ngsniffer_magic) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -1171,7 +1171,7 @@ gboolean ngsniffer_dump_open(wtap_dumper *wdh, int *err)
     }
     nwritten = fwrite(buf, 1, 6, wdh->fh);
     if (nwritten != 6) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -1188,7 +1188,7 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 {
     ngsniffer_dump_t *priv = wdh->dump.ngsniffer;
     struct frame2_rec rec_hdr;
-    int nwritten;
+    size_t nwritten;
     char buf[6];
     double t;
     guint16 t_low, t_med, t_high;
@@ -1204,11 +1204,16 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     if (priv->first_frame) {
 	priv->first_frame=FALSE;
 	tm = localtime(&phdr->ts.tv_sec);
-	start_date = (tm->tm_year - (1980 - 1900)) << 9;
-	start_date |= (tm->tm_mon + 1) << 5;
-	start_date |= tm->tm_mday;
-	/* record the start date, not the start time */
-	priv->start = phdr->ts.tv_sec - (3600*tm->tm_hour + 60*tm->tm_min + tm->tm_sec);
+	if (tm != NULL) {
+	  start_date = (tm->tm_year - (1980 - 1900)) << 9;
+	  start_date |= (tm->tm_mon + 1) << 5;
+	  start_date |= tm->tm_mday;
+	  /* record the start date, not the start time */
+	  priv->start = phdr->ts.tv_sec - (3600*tm->tm_hour + 60*tm->tm_min + tm->tm_sec);
+	} else {
+	  start_date = 0;
+	  priv->start = 0;
+	}
 
 	/* "sniffer" version ? */
 	maj_vers = 4;
@@ -1227,7 +1232,7 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 	version.rsvd[1] = 0;
 	nwritten = fwrite(&version, 1, sizeof version, wdh->fh);
 	if (nwritten != sizeof version) {
-	    if (nwritten < 0)
+	    if (nwritten == 0 && ferror(wdh->fh))
 		*err = errno;
 	    else
 		*err = WTAP_ERR_SHORT_WRITE;
@@ -1243,7 +1248,7 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     buf[5] = 0x00;
     nwritten = fwrite(buf, 1, 6, wdh->fh);
     if (nwritten != 6) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -1267,7 +1272,7 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     rec_hdr.rsvd = 0;
     nwritten = fwrite(&rec_hdr, 1, sizeof rec_hdr, wdh->fh);
     if (nwritten != sizeof rec_hdr) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -1275,7 +1280,7 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     }
     nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
     if (nwritten != phdr->caplen) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -1290,11 +1295,11 @@ static gboolean ngsniffer_dump_close(wtap_dumper *wdh, int *err)
 {
     /* EOF record */
     char buf[6] = {REC_EOF, 0x00, 0x00, 0x00, 0x00, 0x00};
-    int nwritten;
+    size_t nwritten;
 
     nwritten = fwrite(buf, 1, 6, wdh->fh);
     if (nwritten != 6) {
-	if (nwritten < 0)
+	if (nwritten == 0 && ferror(wdh->fh))
 	    *err = errno;
 	else
 	    *err = WTAP_ERR_SHORT_WRITE;
@@ -1707,7 +1712,7 @@ ng_file_seek_seq(wtap *wth, long offset, int whence)
     /* Ok, now read and discard "delta" bytes. */
     while (delta != 0) {
 	amount_to_read = delta;
-	if (amount_to_read > sizeof buf)
+	if ((unsigned long)amount_to_read > sizeof buf)
 	    amount_to_read = sizeof buf;
 	if (ng_file_read(buf, 1, amount_to_read, wth, FALSE, &err) < 0)
 	    return -1;	/* error */
@@ -1760,7 +1765,7 @@ ng_file_seek_rand(wtap *wth, long offset, int whence)
     if (delta > 0) {
 	/* We're going forwards.
 	   Is the place to which we're seeking within the current buffer? */
-	if (ngsniffer->rand.nextout + delta >= ngsniffer->rand.nbytes) {
+	if ((unsigned)ngsniffer->rand.nextout + delta >= ngsniffer->rand.nbytes) {
 	    /* No.  Search for a blob that contains the target offset in
 	       the uncompressed byte stream, starting with the blob
 	       following the current blob. */
@@ -1829,7 +1834,7 @@ ng_file_seek_rand(wtap *wth, long offset, int whence)
 	   the blob, as otherwise it'd mean we need to seek before
 	   the beginning or after the end of this blob. */
 	delta = offset - ngsniffer->rand.uncomp_offset;
-	g_assert(delta >= 0 && delta < ngsniffer->rand.nbytes);
+	g_assert(delta >= 0 && (unsigned long)delta < ngsniffer->rand.nbytes);
     }
 
     /* OK, the place to which we're seeking is in the buffer; adjust
