@@ -1,9 +1,9 @@
 /* wtap-int.h
  *
- * $Id: wtap-int.h,v 1.12 2001/02/13 07:07:17 guy Exp $
+ * $Id: wtap-int.h,v 1.18 2001/12/13 05:49:13 gram Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -93,8 +93,8 @@ typedef struct {
 	guint32	start_usecs;
 	guint8	version_major;
 	guint32 *frame_table;
-	int	frame_table_size;
-	int	current_frame;
+	guint32	frame_table_size;
+	guint	current_frame;
 } netmon_t;
 
 typedef struct {
@@ -109,15 +109,15 @@ typedef struct {
 typedef struct {
 	time_t inittime;
 	int adjusted;
-	int next_packet_seek_start;
+	long next_packet_seek_start;
 } ascend_t;
 
 typedef struct {
 	gboolean byteswapped;
 } csids_t;
 
-typedef int (*subtype_read_func)(struct wtap*, int*, int*);
-typedef int (*subtype_seek_read_func)(struct wtap*, int, union wtap_pseudo_header*,
+typedef int (*subtype_read_func)(struct wtap*, int*, long*);
+typedef int (*subtype_seek_read_func)(struct wtap*, long, union wtap_pseudo_header*,
 					guint8*, int);
 struct wtap {
 	FILE_T			fh;
@@ -177,8 +177,8 @@ typedef struct {
 	struct timeval first_record_time;
 	guint32	frame_table_offset;
 	guint32	*frame_table;
-	int	frame_table_index;
-	int	frame_table_size;
+	guint	frame_table_index;
+	guint	frame_table_size;
 } netmon_dump_t;
 
 struct wtap_dumper {
@@ -186,6 +186,7 @@ struct wtap_dumper {
 	int			file_type;
 	int			snaplen;
 	int			encap;
+	long			bytes_dumped;
 
 	union {
 		void			*opaque;
@@ -346,4 +347,9 @@ struct wtap_dumper {
 	} \
 	G_STMT_END
 
+/* glib doesn't have g_ptr_array_len of all things!*/
+#ifndef g_ptr_array_len
+#define g_ptr_array_len(a)      ((a)->len)
+#endif
+
 #endif /* __WTAP_INT_H__ */
