@@ -1,9 +1,9 @@
 /* wtap.c
  *
- * $Id: wtap.c,v 1.54 2001/07/13 00:55:58 guy Exp $
+ * $Id: wtap.c,v 1.58 2001/11/30 07:14:22 guy Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -124,6 +124,15 @@ static const struct encap_type_info {
 
 	/* WTAP_ENCAP_CHDLC */
 	{ "Cisco HDLC", "chdlc" },
+
+	/* WTAP_ENCAP_CISCO_IOS */
+	{ "Cisco IOS internal", "ios" },
+
+	/* WTAP_ENCAP_LOCALTALK */
+	{ "Localtalk", "ltalk" },
+
+	/* WTAP_ENCAP_PRISM_HEADER */
+	{ "IEEE 802.11 plus Prism II monitor mode header", "prism" },
 };
 
 /* Name that should be somewhat descriptive. */
@@ -247,7 +256,7 @@ wtap_close(wtap *wth)
 }
 
 gboolean
-wtap_read(wtap *wth, int *err, int *data_offset)
+wtap_read(wtap *wth, int *err, long *data_offset)
 {
 	return wth->subtype_read(wth, err, data_offset);
 }
@@ -273,7 +282,8 @@ wtap_buf_ptr(wtap *wth)
 gboolean
 wtap_loop(wtap *wth, int count, wtap_handler callback, u_char* user, int *err)
 {
-	int		data_offset, loop = 0;
+	long		data_offset;
+	int		loop = 0;
 
 	/* Start by clearing error flag */
 	*err = 0;
@@ -292,7 +302,7 @@ wtap_loop(wtap *wth, int count, wtap_handler callback, u_char* user, int *err)
 }
 
 int
-wtap_seek_read(wtap *wth, int seek_off,
+wtap_seek_read(wtap *wth, long seek_off,
 	union wtap_pseudo_header *pseudo_header, guint8 *pd, int len)
 {
 	return wth->subtype_seek_read(wth, seek_off, pseudo_header, pd, len);
