@@ -1,9 +1,9 @@
 /* wtap.h
  *
- * $Id: wtap.h,v 1.87 2001/07/05 00:34:42 guy Exp $
+ * $Id: wtap.h,v 1.99 2001/12/04 22:28:19 guy Exp $
  *
  * Wiretap Library
- * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -53,7 +53,7 @@
  * it on the machine on which the capture was taken.
  *
  * WTAP_ENCAP_LINUX_ATM_CLIP is the encapsulation you get with the
- * ATM on Linux code from <http://lrcwww.epfl.ch/linux-atm/>;
+ * ATM on Linux code from <http://linux-atm.sourceforge.net/>;
  * that code adds a DLT_ATM_CLIP DLT_ code of 19, and that
  * encapsulation isn't the same as the DLT_ATM_RFC1483 encapsulation
  * presumably used on some BSD systems, which we turn into
@@ -98,9 +98,12 @@
 #define WTAP_ENCAP_SLL				19
 #define WTAP_ENCAP_FRELAY			20
 #define WTAP_ENCAP_CHDLC			21
+#define WTAP_ENCAP_CISCO_IOS			22
+#define WTAP_ENCAP_LOCALTALK			23
+#define WTAP_ENCAP_PRISM_HEADER			24
 
 /* last WTAP_ENCAP_ value + 1 */
-#define WTAP_NUM_ENCAP_TYPES			22
+#define WTAP_NUM_ENCAP_TYPES			25
 
 /* File types that can be read by wiretap.
    We support writing some many of these file types, too, so we
@@ -112,29 +115,33 @@
 #define WTAP_FILE_PCAP_SS990915			4
 #define WTAP_FILE_PCAP_SS991029			5
 #define WTAP_FILE_PCAP_NOKIA			6
-#define WTAP_FILE_LANALYZER			7
-#define WTAP_FILE_NGSNIFFER_UNCOMPRESSED	8
-#define WTAP_FILE_NGSNIFFER_COMPRESSED		9
-#define WTAP_FILE_SNOOP				10
-#define WTAP_FILE_IPTRACE_1_0			11
-#define WTAP_FILE_IPTRACE_2_0			12
-#define WTAP_FILE_NETMON_1_x			13
-#define WTAP_FILE_NETMON_2_x			14
-#define WTAP_FILE_NETXRAY_1_0			15
-#define WTAP_FILE_NETXRAY_1_1			16
-#define WTAP_FILE_NETXRAY_2_00x			17
-#define WTAP_FILE_RADCOM			18
-#define WTAP_FILE_ASCEND			19
-#define WTAP_FILE_NETTL				20
-#define WTAP_FILE_TOSHIBA			21
-#define WTAP_FILE_I4BTRACE			22
-#define WTAP_FILE_CSIDS				23
-#define WTAP_FILE_PPPDUMP			24
-#define WTAP_FILE_ETHERPEEK_MAC_V56		25
-#define WTAP_FILE_ETHERPEEK_MAC_V7		26
+#define WTAP_FILE_PCAP_AIX			7
+#define WTAP_FILE_LANALYZER			8
+#define WTAP_FILE_NGSNIFFER_UNCOMPRESSED	9
+#define WTAP_FILE_NGSNIFFER_COMPRESSED		10
+#define WTAP_FILE_SNOOP				11
+#define WTAP_FILE_IPTRACE_1_0			12
+#define WTAP_FILE_IPTRACE_2_0			13
+#define WTAP_FILE_NETMON_1_x			14
+#define WTAP_FILE_NETMON_2_x			15
+#define WTAP_FILE_NETXRAY_1_0			16
+#define WTAP_FILE_NETXRAY_1_1			17
+#define WTAP_FILE_NETXRAY_2_00x			18
+#define WTAP_FILE_RADCOM			19
+#define WTAP_FILE_ASCEND			20
+#define WTAP_FILE_NETTL				21
+#define WTAP_FILE_TOSHIBA			22
+#define WTAP_FILE_I4BTRACE			23
+#define WTAP_FILE_CSIDS				24
+#define WTAP_FILE_PPPDUMP			25
+#define WTAP_FILE_ETHERPEEK_MAC_V56		26
+#define WTAP_FILE_ETHERPEEK_MAC_V7		27
+#define WTAP_FILE_VMS				28
+#define WTAP_FILE_DBS_ETHERWATCH		29
+#define WTAP_FILE_VISUAL_NETWORKS		30
 
 /* last WTAP_FILE_ value + 1 */
-#define WTAP_NUM_FILE_TYPES			27
+#define WTAP_NUM_FILE_TYPES			31
 
 /*
  * Maximum packet size we'll support.
@@ -270,7 +277,7 @@ struct wtap_pkthdr {
 };
 
 typedef void (*wtap_handler)(u_char*, const struct wtap_pkthdr*,
-		int, union wtap_pseudo_header *pseudo_header, const u_char *);
+		long, union wtap_pseudo_header *pseudo_header, const u_char *);
 
 struct wtap;
 struct Buffer;
@@ -296,7 +303,7 @@ gboolean wtap_loop(wtap *wth, int, wtap_handler, u_char*, int *err);
 /* Returns TRUE if read was successful. FALSE if failure. data_offset is
  * set the the offset in the file where the data for the read packet is
  * located. */
-gboolean wtap_read(wtap *wth, int *err, int *data_offset);
+gboolean wtap_read(wtap *wth, int *err, long *data_offset);
 
 struct wtap_pkthdr *wtap_phdr(wtap *wth);
 union wtap_pseudo_header *wtap_pseudoheader(wtap *wth);
@@ -318,9 +325,9 @@ int wtap_short_string_to_encap(const char *short_name);
 const char *wtap_strerror(int err);
 void wtap_sequential_close(wtap *wth);
 void wtap_close(wtap *wth);
-int wtap_seek_read (wtap *wth, int seek_off,
+int wtap_seek_read (wtap *wth, long seek_off,
 	union wtap_pseudo_header *pseudo_header, guint8 *pd, int len);
-int wtap_def_seek_read (wtap *wth, int seek_off,
+int wtap_def_seek_read (wtap *wth, long seek_off,
 	union wtap_pseudo_header *pseudo_header, guint8 *pd, int len);
 
 gboolean wtap_dump_can_open(int filetype);
@@ -333,6 +340,8 @@ gboolean wtap_dump(wtap_dumper *, const struct wtap_pkthdr *,
 	const union wtap_pseudo_header *pseudo_header, const u_char *, int *err);
 FILE* wtap_dump_file(wtap_dumper *);
 gboolean wtap_dump_close(wtap_dumper *, int *);
+long wtap_get_bytes_dumped(wtap_dumper *);
+void wtap_set_bytes_dumped(wtap_dumper *wdh, long bytes_dumped);
 
 /* XXX - needed until "wiretap" can do live packet captures */
 int wtap_pcap_encap_to_wtap_encap(int encap);
